@@ -3,7 +3,9 @@
  *  Summary:    Fixed size vector class that asserts if you do something bad.
  *  Written by: Jesse Jones
  *
- *  Change History (most recent first):    
+ *  Modified for Crawl Reference by $Author: dolorous $ on $Date: 2008-05-16T17:01:20.658839Z $
+ *
+ *  Change History (most recent first):
  *
  *         <2>     7/29/00    JDJ        Added a variable argument ctor.
  *         <1>     6/16/00    JDJ        Created
@@ -12,11 +14,15 @@
 #ifndef FIXVEC_H
 #define FIXVEC_H
 
+#if _MSC_VER
+// Benign: FixedVector has an array in member init list
+# pragma warning(disable : 4351)
+#endif
+
 #include <cstdarg>
 #include <cstring>
 
 #include "debug.h"
-
 // ==========================================================================
 //    class FixedVector
 // ==========================================================================
@@ -36,7 +42,7 @@ public:
 
     typedef unsigned long   size_type;
     typedef long            difference_type;
-    
+
     typedef TYPE*           iterator;
     typedef const TYPE*     const_iterator;
 
