@@ -3,7 +3,7 @@
  *  Summary:    Player kill tracking
  *  Written by: Darshan Shaligram
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-10-02T11:02:32.986158Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-06-12T09:03:37.281687Z $
  */
 #include "AppHdr.h"
 
@@ -14,6 +14,7 @@
 #include "mon-util.h"
 #include "monstuff.h"
 #include "files.h"
+#include "ghost.h"
 #include "itemname.h"
 #include "place.h"
 #include "travel.h"
@@ -30,9 +31,9 @@ static void kill_lua_filltable(std::vector<kill_exp> &v);
 
 ///////////////////////////////////////////////////////////////////////////
 // KillMaster
-// 
+//
 
-const char *kill_category_names[] = 
+const char *kill_category_names[] =
 {
     "you",
     "collateral kills",
@@ -67,27 +68,29 @@ bool KillMaster::empty() const
     return (true);
 }
 
-void KillMaster::save(FILE *file) const
+void KillMaster::save(writer& outf) const
 {
     // Write the version of the kills file
-    writeByte(file, KILLS_MAJOR_VERSION);
-    writeByte(file, KILLS_MINOR_VERSION);
+    marshallByte(outf, KILLS_MAJOR_VERSION);
+    marshallByte(outf, KILLS_MINOR_VERSION);
 
     for (int i = 0; i < KC_NCATEGORIES; ++i)
-        categorized_kills[i].save(file);
+        categorized_kills[i].save(outf);
 }
 
-void KillMaster::load(FILE *file)
+void KillMaster::load(reader& inf)
 {
-    unsigned char major = readByte(file),
-                  minor = readByte(file);
-    if (major != KILLS_MAJOR_VERSION || 
-            (minor != KILLS_MINOR_VERSION && minor > 0))
-        return ;
+    unsigned char major = unmarshallByte(inf),
+                  minor = unmarshallByte(inf);
+    if (major != KILLS_MAJOR_VERSION ||
+        (minor != KILLS_MINOR_VERSION && minor > 0))
+    {
+        return;
+    }
 
     for (int i = 0; i < KC_NCATEGORIES; ++i)
     {
-        categorized_kills[i].load(file);
+        categorized_kills[i].load(inf);
         if (!minor)
             break;
     }
@@ -95,7 +98,7 @@ void KillMaster::load(FILE *file)
 
 void KillMaster::record_kill(const monsters *mon, int killer, bool ispet)
 {
-    const kill_category kc = 
+    const kill_category kc =
         YOU_KILL(killer)? KC_YOU :
         ispet?            KC_FRIENDLY :
                           KC_OTHER;
@@ -108,7 +111,7 @@ std::string KillMaster::kill_info() const
         return ("");
 
     std::string killtext;
-    
+
     bool needseparator = false;
     int categories = 0;
     long grandtotal = 0L;
@@ -119,7 +122,7 @@ std::string KillMaster::kill_info() const
         int targ = Options.kill_map[i];
         catkills[targ].merge( categorized_kills[i] );
     }
-    
+
     for (int i = KC_YOU; i < KC_NCATEGORIES; ++i)
     {
         if (catkills[i].empty())
@@ -130,8 +133,8 @@ std::string KillMaster::kill_info() const
         long count = catkills[i].get_kills(kills);
         grandtotal += count;
 
-        add_kill_info( killtext, 
-                       kills, 
+        add_kill_info( killtext,
+                       kills,
                        count,
                        i == KC_YOU? NULL :
                                     category_name((kill_category) i),
@@ -189,7 +192,7 @@ void KillMaster::add_kill_info(std::string &killtext,
 
     lua_pushboolean(clua, separator);
 
-    unwind_var<int> lthrottle(clua.throttle_unit_lines, 500000);    
+    unwind_var<int> lthrottle(clua.throttle_unit_lines, 500000);
     if (!clua.callfn("c_kill_list", 3, 0))
 #endif
     {
@@ -216,7 +219,7 @@ void KillMaster::add_kill_info(std::string &killtext,
         }
         {
             char numbuf[100];
-            snprintf(numbuf, sizeof numbuf, 
+            snprintf(numbuf, sizeof numbuf,
                     "%ld creature%s vanquished." "\n", count,
                     count > 1? "s" : "");
             killtext += numbuf;
@@ -290,45 +293,45 @@ long Kills::get_kills(std::vector<kill_exp> &all_kills) const
     return (count);
 }
 
-void Kills::save(FILE *file) const
+void Kills::save(writer& outf) const
 {
     // How many kill records do we have?
-    writeLong(file, kills.size());
+    marshallLong(outf, kills.size());
 
     for ( kill_map::const_iterator iter = kills.begin();
           iter != kills.end(); ++iter)
     {
-        iter->first.save(file);
-        iter->second.save(file);
+        iter->first.save(outf);
+        iter->second.save(outf);
     }
 
     // How many ghosts do we have?
-    writeShort(file, ghosts.size());
-    for (ghost_vec::const_iterator iter = ghosts.begin(); 
+    marshallShort(outf, ghosts.size());
+    for (ghost_vec::const_iterator iter = ghosts.begin();
          iter != ghosts.end(); ++iter)
     {
-        iter->save(file);
+        iter->save(outf);
     }
 }
 
-void Kills::load(FILE *file)
+void Kills::load(reader& inf)
 {
     // How many kill records?
-    long kill_count = readLong(file);
+    long kill_count = unmarshallLong(inf);
     kills.clear();
     for (long i = 0; i < kill_count; ++i)
     {
         kill_monster_desc md;
-        md.load(file);
-        kills[md].load(file);
+        md.load(inf);
+        kills[md].load(inf);
     }
 
-    short ghost_count = readShort(file);
+    short ghost_count = unmarshallShort(inf);
     ghosts.clear();
     for (short i = 0; i < ghost_count; ++i)
     {
         kill_ghost kg;
-        kg.load(file);
+        kg.load(inf);
         ghosts.push_back(kg);
     }
 }
@@ -356,10 +359,10 @@ std::string apostrophise(const std::string &name)
 
 // For monster names ending with these suffixes, we pluralise directly without
 // attempting to use the "of" rule. For instance:
-// 
+//
 //      moth of wrath           => moths of wrath but
 //      moth of wrath zombie    => moth of wrath zombies.
-//      
+//
 // This is not necessary right now, since there are currently no monsters that
 // require this special treatment (no monster with 'of' in its name is eligible
 // for zombies or skeletons).
@@ -459,7 +462,7 @@ std::string kill_def::base_name(const kill_monster_desc &md) const
         name = "spectral " + name;
         break;
       default:
-        // Silence compiler warning about not handling M_NORMAL and 
+        // Silence compiler warning about not handling M_NORMAL and
         // M_SHAPESHIFTER
         break;
     }
@@ -506,14 +509,14 @@ std::string kill_def::info(const kill_monster_desc &md) const
     return append_places(md, name);
 }
 
-std::string kill_def::append_places(const kill_monster_desc &md, 
+std::string kill_def::append_places(const kill_monster_desc &md,
                                 const std::string &name) const
 {
     if (Options.dump_kill_places == KDO_NO_PLACES) return name;
 
     int nplaces = places.size();
     if ( nplaces == 1 || mons_is_unique(md.monnum)
-            || Options.dump_kill_places == KDO_ALL_PLACES ) 
+            || Options.dump_kill_places == KDO_ALL_PLACES )
     {
         std::string augmented = name;
         augmented += " (";
@@ -530,30 +533,28 @@ std::string kill_def::append_places(const kill_monster_desc &md,
     return name;
 }
 
-void kill_def::save(FILE *file) const
+void kill_def::save(writer& outf) const
 {
-    writeShort(file, kills);
-    writeShort(file, exp);
+    marshallShort(outf, kills);
+    marshallShort(outf, exp);
 
-    writeShort(file, places.size());
+    marshallShort(outf, places.size());
     for (std::vector<unsigned short>::const_iterator iter = places.begin();
             iter != places.end(); ++iter)
     {
-        writeShort(file, *iter);
+        marshallShort(outf, *iter);
     }
 }
 
-void kill_def::load(FILE *file)
+void kill_def::load(reader& inf)
 {
-    kills = (unsigned short) readShort(file);
-    exp   = readShort(file);
+    kills = (unsigned short) unmarshallShort(inf);
+    exp   = unmarshallShort(inf);
 
     places.clear();
-    short place_count = readShort(file);
+    short place_count = unmarshallShort(inf);
     for (short i = 0; i < place_count; ++i)
-    {
-        places.push_back((unsigned short) readShort(file));
-    }
+        places.push_back((unsigned short) unmarshallShort(inf));
 }
 
 kill_ghost::kill_ghost(const monsters *mon)
@@ -565,33 +566,32 @@ kill_ghost::kill_ghost(const monsters *mon)
     // Check whether this is really a ghost, since we also have to handle
     // the Pandemonic demons.
     if (mon->type == MONS_PLAYER_GHOST)
-        ghost_name = "The ghost of " + ghost_description(*mon, true);
+        ghost_name = "The ghost of " + get_ghost_description(*mon, true);
 }
 
 std::string kill_ghost::info() const
 {
-    return ghost_name + 
-        (Options.dump_kill_places != KDO_NO_PLACES?  
-            " (" + short_place_name(place) + ")" : std::string(""));
+    return ghost_name
+           + (Options.dump_kill_places != KDO_NO_PLACES?
+                " (" + short_place_name(place) + ")" : std::string(""));
 }
 
-void kill_ghost::save(FILE *file) const
+void kill_ghost::save(writer& outf) const
 {
-    writeString(file, ghost_name);
-    writeShort(file, (unsigned short) exp);
-    writeShort(file, place);
+    marshallString4(outf, ghost_name);
+    marshallShort(outf, (unsigned short) exp);
+    marshallShort(outf, place);
 }
 
-void kill_ghost::load(FILE *file)
+void kill_ghost::load(reader& inf)
 {
-    ghost_name = readString(file);
-    exp = readShort(file);
-    place = (unsigned short) readShort(file);
+    unmarshallString4(inf, ghost_name);
+    exp = unmarshallShort(inf);
+    place = (unsigned short) unmarshallShort(inf);
 }
 
 kill_monster_desc::kill_monster_desc(const monsters *mon)
 {
-
     monnum = mon->type;
     modifier = M_NORMAL;
     switch (mon->type)
@@ -608,10 +608,12 @@ kill_monster_desc::kill_monster_desc(const monsters *mon)
         case MONS_SPECTRAL_THING:
             modifier = M_SPECTRE;
             break;
+        default: break;
     }
-    if (modifier != M_NORMAL) monnum = mon->number;
+    if (modifier != M_NORMAL)
+        monnum = mon->base_monster;
 
-    if (mon->has_ench(ENCH_SHAPESHIFTER) || 
+    if (mon->has_ench(ENCH_SHAPESHIFTER) ||
             mon->has_ench(ENCH_GLOWING_SHAPESHIFTER))
         modifier = M_SHAPESHIFTER;
 
@@ -620,16 +622,16 @@ kill_monster_desc::kill_monster_desc(const monsters *mon)
         monnum = MONS_WEAPON_MIMIC;
 }
 
-void kill_monster_desc::save(FILE *file) const
+void kill_monster_desc::save(writer& outf) const
 {
-    writeShort(file, (short) monnum);
-    writeShort(file, (short) modifier);
+    marshallShort(outf, (short) monnum);
+    marshallShort(outf, (short) modifier);
 }
 
-void kill_monster_desc::load(FILE *file)
+void kill_monster_desc::load(reader& inf)
 {
-    monnum = (int) readShort(file);
-    modifier = (name_modifier) readShort(file);
+    monnum = (int) unmarshallShort(inf);
+    modifier = (name_modifier) unmarshallShort(inf);
 }
 
 ///////////////////////////////////////////////////////////////////////////
@@ -660,10 +662,10 @@ KILLEXP_ACCESS(base_name, string, base_name.c_str())
 KILLEXP_ACCESS(desc, string, desc.c_str())
 KILLEXP_ACCESS(monnum, number, monnum)
 KILLEXP_ACCESS(isghost, boolean,
-               monnum == -1 && 
+               monnum == -1 &&
                ke->desc.find("The ghost of") != std::string::npos)
 KILLEXP_ACCESS(ispandemon, boolean,
-               monnum == -1 && 
+               monnum == -1 &&
                ke->desc.find("The ghost of") == std::string::npos)
 KILLEXP_ACCESS(isunique, boolean,
                monnum != -1 && mons_is_unique(ke->monnum))
@@ -740,7 +742,7 @@ static int kill_lualc_place_name(lua_State *ls)
 
 static bool is_ghost(const kill_exp *ke)
 {
-    return ke->monnum == -1 
+    return ke->monnum == -1
         && ke->desc.find("The ghost of") != std::string::npos;
 }
 
@@ -792,7 +794,7 @@ static int kill_lualc_symbol(lua_State *ls)
     kill_exp *ke = static_cast<kill_exp*>( lua_touserdata(ls, 1) );
     if (ke)
     {
-        unsigned char ch = ke->monnum != -1? 
+        unsigned char ch = ke->monnum != -1?
                     mons_char(ke->monnum) :
               is_ghost(ke)? 'p' : '&';
 
@@ -864,7 +866,7 @@ static int kill_lualc_write(lua_State *ls)
             return 0;
         }
 
-        std::string *skill = static_cast<std::string *>( 
+        std::string *skill = static_cast<std::string *>(
                                     lua_touserdata(ls, -1) );
         // Pop the userdata off the stack.
         lua_settop(ls, -2);
