@@ -2,7 +2,10 @@
  *  File:       Kills.h
  *  Summary:    Tracks monsters the player has killed.
  *  Written by: Darshan Shaligram
+ *
+ *  Modified for Crawl Reference by $Author: pauldubois $ on $Date: 2008-03-23T09:42:58.924713Z $
  */
+
 #ifndef KILLS_H
 #define KILLS_H
 
@@ -14,16 +17,18 @@
 
 std::string apostrophise(const std::string &name);
 
-struct monsters;
+class monsters;
+class reader;
+class writer;
 
 // Not intended for external use!
 struct kill_monster_desc
 {
-    kill_monster_desc(const struct monsters *);
+    kill_monster_desc(const monsters *);
     kill_monster_desc() { }
 
-    void save(FILE*) const;
-    void load(FILE*);
+    void save(writer&) const;
+    void load(reader&);
 
     enum name_modifier
     {
@@ -49,16 +54,16 @@ struct kill_monster_desc
 class kill_def
 {
 public:
-    kill_def(const struct monsters *mon);
+    kill_def(const monsters *mon);
     kill_def() : kills(0), exp(0)
     {
         // This object just says to the world that it's uninitialized
     }
 
-    void save(FILE*) const;
-    void load(FILE*);
+    void save(writer&) const;
+    void load(reader&);
 
-    void add_kill(const struct monsters *mon, unsigned short place);
+    void add_kill(const monsters *mon, unsigned short place);
     void add_place(unsigned short place, bool force = false);
 
     void merge(const kill_def &k, bool unique_monster);
@@ -80,11 +85,11 @@ private:
 class kill_ghost
 {
 public:
-    kill_ghost(const struct monsters *mon);
+    kill_ghost(const monsters *mon);
     kill_ghost() { }
 
-    void save(FILE*) const;
-    void load(FILE*);
+    void save(writer&) const;
+    void load(reader&);
 
     std::string info() const;
 
@@ -135,8 +140,8 @@ public:
     void merge(const Kills &k);
 
     bool empty() const;
-    void save(FILE*) const;
-    void load(FILE*);
+    void save(writer&) const;
+    void load(reader&);
 
     long get_kills(std::vector<kill_exp> &v) const;
 private:
@@ -148,7 +153,7 @@ private:
     kill_map    kills;
     ghost_vec   ghosts;
 
-    void record_ghost_kill(const struct monsters *mon);
+    void record_ghost_kill(const monsters *mon);
 };
 
 class KillMaster
@@ -161,8 +166,8 @@ public:
     void record_kill(const monsters *mon, int killer, bool ispet);
 
     bool empty() const;
-    void save(FILE*) const;
-    void load(FILE*);
+    void save(writer&) const;
+    void load(reader&);
 
     std::string kill_info() const;
 private:
