@@ -3,7 +3,7 @@
  *  Summary:    Functions related to special abilities.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-10-10T19:16:57.923694Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-06-18T20:21:53.360974Z $
  *
  *  Change History (most recent first):
  *
@@ -39,7 +39,7 @@ struct generic_cost
     {
         return generic_cost(low, high - low + 1, rolls);
     }
-    
+
     int cost() const;
 
     operator bool () const { return base > 0 || add > 0; }
@@ -70,7 +70,8 @@ const struct ability_def & get_ability_def( ability_type abil );
 const char* ability_name(ability_type ability);
 const std::string make_cost_description( ability_type ability );
 std::vector<const char*> get_ability_names();
-int choose_ability_menu(const std::vector<talent>& talents);
+int choose_ability_menu(const std::vector<talent>& talents,
+                        bool describe = false);
 
 // last updated 12may2000 {dlb}
 /* ***********************************************************************
