@@ -1,9 +1,9 @@
 /*
  *  File:       abyss.cc
- *  Summary:    Misc functions (most of which don't appear to be related to priests).
+ *  Summary:    Misc abyss specific functions.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-09T21:43:14.392740Z $
  *
  *  Change History (most recent first):
  *
@@ -15,6 +15,7 @@
 #include "abyss.h"
 
 #include <stdlib.h>
+#include <algorithm>
 
 #include "externs.h"
 
@@ -22,6 +23,7 @@
 #include "makeitem.h"
 #include "mapmark.h"
 #include "message.h"
+#include "misc.h"
 #include "monplace.h"
 #include "mtransit.h"
 #include "player.h"
@@ -30,15 +32,20 @@
 #include "lev-pand.h"
 #include "randart.h"
 #include "stuff.h"
+#include "spells3.h"
 #include "terrain.h"
+#include "tiles.h"
 #include "traps.h"
 #include "view.h"
+#include "xom.h"
 
-static bool place_feature_near( const coord_def &centre,
-                                int radius,
-                                dungeon_feature_type candidate,
-                                dungeon_feature_type replacement,
-                                int tries )
+// If not_seen is true, don't place the feature where it can be seen from
+// the centre.
+static bool _place_feature_near( const coord_def &centre,
+                                 int radius,
+                                 dungeon_feature_type candidate,
+                                 dungeon_feature_type replacement,
+                                 int tries, bool not_seen = false )
 {
     const int radius2 = radius * radius + 1;
     for (int i = 0; i < tries; ++i)
@@ -46,9 +53,13 @@ static bool place_feature_near( const coord_def &centre,
         const coord_def &cp =
             centre + coord_def(random_range(-radius, radius),
                                random_range(-radius, radius));
+
         if (cp == centre || (cp - centre).abs() > radius2 || !in_bounds(cp))
             continue;
 
+        if (not_seen && grid_see_grid(cp.x, cp.y, centre.x, centre.y))
+            continue;
+
         if (grd(cp) == candidate)
         {
 #ifdef DEBUG_DIAGNOSTICS
@@ -63,14 +74,20 @@ static bool place_feature_near( const coord_def &centre,
     return (false);
 }
 
-// public for abyss generation
-void generate_abyss(void)
+//#define DEBUG_ABYSS
+
+// Public for abyss generation.
+void generate_abyss()
 {
     int i, j;                   // loop variables
     int temp_rand;              // probability determination {dlb}
 
+#ifdef DEBUG_ABYSS
+    mprf(MSGCH_DIAGNOSTICS,
+         "generate_abyss(); turn_on_level: %d", env.turns_on_level);
+#endif
+
     for (i = 5; i < (GXM - 5); i++)
-    {
         for (j = 5; j < (GYM - 5); j++)
         {
             temp_rand = random2(4000);
@@ -81,26 +98,84 @@ void generate_abyss(void)
                          (temp_rand >   0) ? DNGN_METAL_WALL    //  2.5%
                                            : DNGN_CLOSED_DOOR); // 1 in 4000
         }
-    }
 
-    grd[45][35] = DNGN_FLOOR;
-    if ( one_chance_in(5) )
-        place_feature_near( coord_def(45, 35), LOS_RADIUS,
-                            DNGN_FLOOR, DNGN_ALTAR_LUGONU, 50 );
+    // If we're starting out in the Abyss, make sure the starting grid is
+    // an altar to Lugonu and there's an exit near-by.
+    // Otherwise, we start out on floor and there's a chance there's an
+    // altar near-by.
+    if (you.char_direction == GDT_GAME_START)
+    {
+        grd[45][35] = DNGN_ALTAR_LUGONU;
+        _place_feature_near( coord_def(45, 35), LOS_RADIUS + 2,
+                             DNGN_FLOOR, DNGN_EXIT_ABYSS, 50, true );
+    }
+    else
+    {
+        grd[45][35] = DNGN_FLOOR;
+        if (one_chance_in(5))
+        {
+            _place_feature_near( coord_def(45, 35), LOS_RADIUS,
+                                 DNGN_FLOOR, DNGN_ALTAR_LUGONU, 50 );
+        }
+    }
 }
 
-static void generate_area(int gx1, int gy1, int gx2, int gy2)
+// Returns the roll to use to check if we want to create an abyssal rune.
+static int _abyssal_rune_roll()
 {
+    if (you.attribute[ATTR_ABYSSAL_RUNES])
+        return (-1);
+
+    // The longer the player's hung around in the Abyss, the more
+    // likely the rune. Never generate a new rune if the player
+    // already found one, but make the Abyssal rune eligible for
+    // generation again if the player loses it.
+
+    // If the player leaves the Abyss turns_on_level resets to 0. So
+    // hang in there if you want your Abyssal rune fix quick. :P
+
+    // Worshippers of Lugonu with decent piety will attract the rune
+    // to themselves.
+
+    const bool lugonu_favoured =
+        (you.religion == GOD_LUGONU && !player_under_penance()
+         && you.piety > 120);
+
+    const int cutoff = lugonu_favoured ? 50 : 500;
+    const int scale = lugonu_favoured ? 10 : 40;
+
+    const int odds =
+        std::max(200 - std::max((env.turns_on_level - cutoff) / scale, 0), 6);
+#ifdef DEBUG_ABYSS
+    mprf(MSGCH_DIAGNOSTICS, "Abyssal rune odds: 1 in %d", odds);
+#endif
+    return (odds);
+}
+
+static void _generate_area(int gx1, int gy1, int gx2, int gy2,
+                           bool spatter = false)
+{
+    // Any rune on the floor prevents the abyssal rune from being generated.
+    bool placed_abyssal_rune =
+        find_floor_item(OBJ_MISCELLANY, MISC_RUNE_OF_ZOT);
+
+#ifdef DEBUG_ABYSS
+    mprf(MSGCH_DIAGNOSTICS,
+         "_generate_area(). turns_on_level: %d, rune_on_floor: %s",
+         env.turns_on_level, placed_abyssal_rune? "yes" : "no");
+#endif
+
+    const int abyssal_rune_roll = _abyssal_rune_roll();
     int items_placed = 0;
     const int thickness = random2(70) + 30;
     int thing_created;
 
     FixedVector<dungeon_feature_type, 5> replaced;
 
-    // nuke map
+    // Nuke map.
     env.map.init(map_cell());
 
-    // generate level composition vector
+    // Generate level composition vector.
     for (int i = 0; i < 5; i++)
     {
         const int temp_rand = random2(10000);
@@ -118,7 +193,7 @@ static void generate_area(int gx1, int gy1, int gx2, int gy2)
     {
         int rooms_to_do = 1 + random2(10);
 
-        for ( int rooms_done = 0; rooms_done < rooms_to_do; ++rooms_done )
+        for (int rooms_done = 0; rooms_done < rooms_to_do; ++rooms_done)
         {
             const int x1 = 10 + random2(GXM - 20);
             const int y1 = 10 + random2(GYM - 20);
@@ -132,35 +207,53 @@ static void generate_area(int gx1, int gy1, int gx2, int gy2)
 
             for (int i = x1; room_ok && i < x2; i++)
                 for (int j = y1; room_ok && j < y2; j++)
+                {
                     if (grd[i][j] != DNGN_UNSEEN)
                         room_ok = false;
+                }
 
-            if ( room_ok )
+            if (room_ok)
+            {
                 for (int i = x1; i < x2; i++)
                     for (int j = y1; j < y2; j++)
                         grd[i][j] = DNGN_FLOOR;
+            }
         }
     }
 
-    for (int i = gx1; i <= gx2; i++)
+    // During game start, number and level of items mustn't be higher than
+    // that on level 1.
+    int num_items = 150, items_level = 51;
+    if (you.char_direction == GDT_GAME_START)
     {
+        num_items   = 3 + roll_dice( 3, 11 );
+        items_level = 0;
+    }
+
+    for (int i = gx1; i <= gx2; i++)
         for (int j = gy1; j <= gy2; j++)
         {
-            if (grd[i][j] == DNGN_UNSEEN && random2(100) <= thickness)
+            if (grd[i][j] == DNGN_UNSEEN && x_chance_in_y(thickness + 1, 100))
             {
                 grd[i][j] = DNGN_FLOOR;
 
-                if (items_placed < 150 && one_chance_in(200))
+                if (items_placed < num_items && one_chance_in(200))
                 {
-                    if (one_chance_in(200))
+                    if (!placed_abyssal_rune && abyssal_rune_roll != -1
+                        && you.char_direction != GDT_GAME_START
+                        && one_chance_in(abyssal_rune_roll))
                     {
-                        thing_created = items(1, OBJ_MISCELLANY, 
+                        thing_created = items(1, OBJ_MISCELLANY,
                                               MISC_RUNE_OF_ZOT, true, 51, 51);
+                        placed_abyssal_rune = true;
+#ifdef DEBUG_ABYSS
+                        mpr("Placing an Abyssal rune.", MSGCH_DIAGNOSTICS);
+#endif
                     }
                     else
                     {
-                        thing_created = items(1, OBJ_RANDOM,
-                                              OBJ_RANDOM, true, 51, 250);
+                        thing_created = items(1, OBJ_RANDOM, OBJ_RANDOM,
+                                              true, items_level, 250);
                     }
 
                     move_item_to_grid( &thing_created, i, j );
@@ -170,87 +263,232 @@ static void generate_area(int gx1, int gy1, int gx2, int gy2)
                 }
             }
         }
-    }
+
+    int exits_wanted  = 0;
+    int altars_wanted = 0;
 
     for (int i = gx1; i <= gx2; i++)
-    {
         for (int j = gy1; j <= gy2; j++)
         {
             if (grd[i][j] == DNGN_UNSEEN)
                 grd[i][j] = replaced[random2(5)];
 
             if (one_chance_in(7500)) // place an exit
-                grd[i][j] = DNGN_EXIT_ABYSS;
+                exits_wanted++;
 
-            if (one_chance_in(10000)) // place an altar
+            // Don't place exit under items.
+            if (exits_wanted > 0 && igrd[i][j] == NON_ITEM)
             {
-                do
-                {
-                    grd[i][j] =
-                        static_cast<dungeon_feature_type>(
-                            DNGN_ALTAR_ZIN + random2(NUM_GODS-1) );
-                }
-                while (grd[i][j] == DNGN_ALTAR_ZIN
-                       || grd[i][j] == DNGN_ALTAR_SHINING_ONE
-                       || grd[i][j] == DNGN_ALTAR_ELYVILON);
+                grd[i][j] = DNGN_EXIT_ABYSS;
+                exits_wanted--;
+#ifdef DEBUG_ABYSS
+                mpr("Placing Abyss exit.", MSGCH_DIAGNOSTICS);
+#endif
+            }
 
-                // Lugonu has a flat 50% chance of corrupting the altar
-                if ( coinflip() )
-                    grd[i][j] = DNGN_ALTAR_LUGONU;
+            // Except for the altar on the starting position, don't place
+            // any altars.
+            if (you.char_direction != GDT_GAME_START)
+            {
+                if (one_chance_in(10000)) // Place an altar.
+                    altars_wanted++;
+
+                // Don't place altars under items.
+                if (altars_wanted > 0 && igrd[i][j] == NON_ITEM)
+                {
+                    do
+                    {
+                        grd[i][j] = static_cast<dungeon_feature_type>(
+                                        DNGN_ALTAR_ZIN + random2(NUM_GODS-1) );
+                    }
+                    while (grd[i][j] == DNGN_ALTAR_ZIN
+                           || grd[i][j] == DNGN_ALTAR_SHINING_ONE
+                           || grd[i][j] == DNGN_ALTAR_ELYVILON);
+
+                    // Lugonu has a flat 50% chance of corrupting the altar.
+                    if (coinflip())
+                        grd[i][j] = DNGN_ALTAR_LUGONU;
+
+                    altars_wanted--;
+#ifdef DEBUG_ABYSS
+                    mpr("Placing altar.", MSGCH_DIAGNOSTICS);
+#endif
+                }
             }
         }
+
+    generate_random_blood_spatter_on_level();
+
+    setup_environment_effects();
+}
+
+static int _abyss_exit_nearness()
+{
+    int nearness = INFINITE_DISTANCE;
+
+    // is_terrain_known() doesn't work on unmappable levels because
+    // mapping flags are not set on such levels.
+    for (radius_iterator ri(you.pos(), LOS_RADIUS); ri; ++ri)
+        if (grd(*ri) == DNGN_EXIT_ABYSS && get_screen_glyph(*ri) != ' ')
+            nearness = std::min(nearness, grid_distance(you.pos(), *ri));
+
+    return (nearness);
+}
+
+static int _abyss_rune_nearness()
+{
+    int nearness = INFINITE_DISTANCE;
+
+    // See above comment about is_terrain_known().
+    for (radius_iterator ri(you.pos(), LOS_RADIUS); ri; ++ri)
+    {
+        if (get_screen_glyph(ri->x, ri->y) != ' ')
+        {
+            for (stack_iterator si(*ri); si; ++si)
+                if (is_rune(*si) && si->plus == RUNE_ABYSSAL)
+                    nearness = std::min(nearness, grid_distance(you.pos(),*ri));
+        }
+    }
+    return (nearness);
+}
+
+static int exit_was_near;
+static int rune_was_near;
+
+static void _xom_check_nearness_setup()
+{
+    exit_was_near = _abyss_exit_nearness();
+    rune_was_near = _abyss_rune_nearness();
+}
+
+// If the player was almost to the exit when it disappeared, Xom is
+// extremely amused.  He's also extremely amused if the player winds
+// up right next to an exit when there wasn't one there before.  The
+// same applies to Abyssal runes.
+static void _xom_check_nearness()
+{
+    // Update known terrain
+    viewwindow(true, false);
+
+    int exit_is_near = _abyss_exit_nearness();
+    int rune_is_near = _abyss_rune_nearness();
+
+    if (exit_was_near < INFINITE_DISTANCE
+            && exit_is_near == INFINITE_DISTANCE
+        || rune_was_near < INFINITE_DISTANCE
+            && rune_is_near == INFINITE_DISTANCE
+            && you.attribute[ATTR_ABYSSAL_RUNES] == 0)
+    {
+        xom_is_stimulated(255, "Xom snickers loudly.", true);
+    }
+
+    if (rune_was_near == INFINITE_DISTANCE
+            && rune_is_near < INFINITE_DISTANCE
+            && you.attribute[ATTR_ABYSSAL_RUNES] == 0
+        || exit_was_near == INFINITE_DISTANCE
+            && exit_is_near < INFINITE_DISTANCE)
+    {
+        xom_is_stimulated(255);
     }
 }
 
-static void abyss_lose_monster(monsters &mons)
+static void _abyss_lose_monster(monsters &mons)
 {
     if (mons.needs_transit())
         mons.set_transit( level_id(LEVEL_ABYSS) );
-    
+
     mons.reset();
 }
 
+#define LOS_DIAMETER (LOS_RADIUS * 2 + 1)
+
 void area_shift(void)
-/*******************/
 {
-    for (unsigned int i = 0; i < MAX_MONSTERS; i++)
-    {
-        monsters &m = menv[i];
-        
-        if (!m.alive())
-            continue;
+#ifdef DEBUG_ABYSS
+    mprf(MSGCH_DIAGNOSTICS, "area_shift() - player at pos (%d, %d)",
+         you.x_pos, you.y_pos);
+#endif
 
-        // remove non-nearby monsters
-        if (grid_distance(m.x, m.y, you.x_pos, you.y_pos) > 10)
-            abyss_lose_monster(m);
-    }
+    // Preserve floor props around the player, primarily so that
+    // blood-splatter doesn't appear out of nowhere when doing an
+    // area shift.
+    //
+    // Also shift sanctuary center if it's close.
+    bool      sanct_shifted = false;
+    coord_def sanct_pos(0, 0);
+    FixedArray<unsigned short, LOS_DIAMETER, LOS_DIAMETER> fprops;
+    const coord_def los_delta(LOS_RADIUS, LOS_RADIUS);
 
-    for (int i = 5; i < (GXM - 5); i++)
+    for (radius_iterator ri(you.pos(), LOS_RADIUS); ri; ++ri)
     {
-        for (int j = 5; j < (GYM - 5); j++)
+        fprops(you.pos() - *ri + los_delta) = env.map(*ri).property;
+        if (env.sanctuary_pos == *ri && env.sanctuary_time > 0)
         {
-            // don't modify terrain by player
-            if (grid_distance(i, j, you.x_pos, you.y_pos) <= 10)
-                continue;
-
-            // nuke terrain otherwise
-            grd[i][j] = DNGN_UNSEEN;
-
-            // nuke items
-            destroy_item_stack( i, j );
-
-            if (mgrd[i][j] != NON_MONSTER)
-                abyss_lose_monster( menv[ mgrd[i][j] ] );
+            sanct_pos     = *ri - you.pos();
+            sanct_shifted = true;
         }
     }
 
-    // shift all monsters & items to new area
-    for (int i = you.x_pos - 10; i < you.x_pos + 11; i++)
+    // If sanctuary centre is outside of preserved area then just get
+    // rid of it.
+    if (env.sanctuary_time > 0 && !sanct_shifted)
+    {
+        remove_sanctuary(false);
+
+        coord_def pos;
+        for (pos.x = 0; pos.x < LOS_DIAMETER; pos.x++)
+            for (pos.y = 0; pos.y < LOS_DIAMETER; pos.y++)
+            {
+                if (fprops(pos) == FPROP_SANCTUARY_1
+                    || fprops(pos) == FPROP_SANCTUARY_2)
+                {
+                    fprops(pos) = FPROP_NONE;
+                }
+            }
+    }
+
+    _xom_check_nearness_setup();
+
+    for (unsigned int i = 0; i < MAX_MONSTERS; i++)
+    {
+        monsters &m = menv[i];
+
+        if (!m.alive())
+            continue;
+
+        // Remove non-nearby monsters.
+        if (grid_distance(m.pos(), you.pos()) > 10)
+            _abyss_lose_monster(m);
+    }
+
+    for (int i = 5; i < (GXM - 5); i++)
+        for (int j = 5; j < (GYM - 5); j++)
+        {
+            // Don't modify terrain by player.
+            if (grid_distance(i, j, you.x_pos, you.y_pos) <= 10)
+                continue;
+
+            // Nuke terrain otherwise.
+            grd[i][j] = DNGN_UNSEEN;
+
+            // Nuke items.
+#ifdef DEBUG_ABYSS
+            if (igrd[i][j] != NON_ITEM)
+                mprf(MSGCH_DIAGNOSTICS, "Nuke item stack at (%d, %d)", i, j);
+#endif
+            lose_item_stack( i, j );
+
+            if (mgrd[i][j] != NON_MONSTER)
+                _abyss_lose_monster( menv[ mgrd[i][j] ] );
+        }
+
+    // Shift all monsters & items to new area.
+    for (int i = you.x_pos - 10; i <= you.x_pos + 10; i++)
     {
         if (i < 0 || i >= GXM)
             continue;
 
-        for (int j = you.y_pos - 10; j < you.y_pos + 11; j++)
+        for (int j = you.y_pos - 10; j <= you.y_pos + 10; j++)
         {
             if (j < 0 || j >= GYM)
                 continue;
@@ -258,13 +496,21 @@ void area_shift(void)
             const int ipos = 45 + i - you.x_pos;
             const int jpos = 35 + j - you.y_pos;
 
-            // move terrain
+            // Move terrain.
             grd[ipos][jpos] = grd[i][j];
 
-            // move item
+            // Move item.
+#ifdef DEBUG_ABYSS
+            if (igrd[i][j] != NON_ITEM)
+            {
+                 mprf(MSGCH_DIAGNOSTICS,
+                      "Move item stack from (%d, %d) to (%d, %d)",
+                      i, j, ipos, jpos);
+            }
+#endif
             move_item_stack_to_grid( i, j, ipos, jpos );
 
-            // move monster
+            // Move monster.
             mgrd[ipos][jpos] = mgrd[i][j];
             if (mgrd[i][j] != NON_MONSTER)
             {
@@ -273,13 +519,12 @@ void area_shift(void)
                 mgrd[i][j] = NON_MONSTER;
             }
 
-            // move cloud
+            // Move cloud,
             if (env.cgrid[i][j] != EMPTY_CLOUD)
                 move_cloud( env.cgrid[i][j], ipos, jpos );
         }
     }
 
-
     for (unsigned int i = 0; i < MAX_CLOUDS; i++)
     {
         if (env.cloud[i].type == CLOUD_NONE)
@@ -294,16 +539,27 @@ void area_shift(void)
 
     you.moveto(45, 35);
 
-    generate_area(5, 5, (GXM - 5), (GYM - 5));
+    _generate_area(MAPGEN_BORDER, MAPGEN_BORDER,
+                   GXM - MAPGEN_BORDER, GYM - MAPGEN_BORDER, true);
+
+    _xom_check_nearness();
+
+    for (radius_iterator ri(you.pos(), LOS_RADIUS); ri; ++ri)
+        env.map(*ri).property = fprops(you.pos() - *ri + los_delta);
+
+    if (sanct_shifted)
+        env.sanctuary_pos = sanct_pos + you.pos();
+
+    mgen_data mons;
+    mons.level_type = LEVEL_ABYSS;
+    mons.proximity  = PROX_AWAY_FROM_PLAYER;
 
     for (unsigned int mcount = 0; mcount < 15; mcount++)
-    {
-        mons_place( RANDOM_MONSTER, BEH_HOSTILE, MHITNOT, false, 1, 1, 
-                    LEVEL_ABYSS, PROX_AWAY_FROM_PLAYER ); // PROX_ANYWHERE?
-    }
+        mons_place(mons);
 
     // And allow monsters in transit another chance to return.
     place_transiting_monsters();
+    place_transiting_items();
 }
 
 void save_abyss_uniques()
@@ -319,17 +575,19 @@ void save_abyss_uniques()
 void abyss_teleport( bool new_area )
 /**********************************/
 {
+    _xom_check_nearness_setup();
+
     int x, y, i, j, k;
 
     if (!new_area)
     {
-        // try to find a good spot within the shift zone:
+        // Try to find a good spot within the shift zone.
         for (i = 0; i < 100; i++)
         {
             x = 16 + random2( GXM - 32 );
             y = 16 + random2( GYM - 32 );
 
-            if ((grd[x][y] == DNGN_FLOOR 
+            if ((grd[x][y] == DNGN_FLOOR
                     || grd[x][y] == DNGN_SHALLOW_WATER)
                 && mgrd[x][y] == NON_MONSTER
                 && env.cgrid[x][y] == EMPTY_CLOUD)
@@ -340,38 +598,39 @@ void abyss_teleport( bool new_area )
 
         if (i < 100)
         {
+#ifdef DEBUG_ABYSS
+            mpr("Non-new area Abyss teleport.", MSGCH_DIAGNOSTICS);
+#endif
             you.moveto(x, y);
+            _xom_check_nearness();
             return;
         }
     }
 
-    // teleport to a new area of the abyss:
+    remove_sanctuary(false);
 
-    init_pandemonium();          // get new monsters
-    dgn_set_colours_from_monsters(); // and new colours
+#ifdef DEBUG_ABYSS
+    mpr("New area Abyss teleport.", MSGCH_DIAGNOSTICS);
+#endif
 
-    for (i = 0; i < MAX_MONSTERS; i++)
-    {
+    // Teleport to a new area of the abyss.
+
+    // Get new monsters and colours.
+    init_pandemonium();
+#ifdef USE_TILE
+    tile_init_flavor();
+#endif
+
+    for (i = 0; i < MAX_MONSTERS; ++i)
         if (menv[i].alive())
-            abyss_lose_monster(menv[i]);
-    }
-    
-    // Orbs and fixed artefacts are marked as "lost in the abyss"
-    for (k = 0; k < MAX_ITEMS; k++)
+            _abyss_lose_monster(menv[i]);
+
+    // Orbs and fixed artefacts are marked as "lost in the abyss".
+    for (k = 0; k < MAX_ITEMS; ++k)
     {
         if (is_valid_item( mitm[k] ))
         {
-            if (mitm[k].base_type == OBJ_ORBS)
-            {
-                set_unique_item_status( OBJ_ORBS, mitm[k].sub_type, 
-                                        UNIQ_LOST_IN_ABYSS );
-            }
-            else if (is_fixed_artefact( mitm[k] ))
-            {
-                set_unique_item_status( OBJ_WEAPONS, mitm[k].special, 
-                                        UNIQ_LOST_IN_ABYSS );
-            }
-
+            item_was_lost( mitm[k] );
             destroy_item( k );
         }
     }
@@ -380,39 +639,48 @@ void abyss_teleport( bool new_area )
         delete_cloud( i );
 
     for (i = 10; i < (GXM - 9); i++)
-    {
         for (j = 10; j < (GYM - 9); j++)
         {
-            grd[i][j] = DNGN_UNSEEN;    // so generate_area will pick it up
-            igrd[i][j] = NON_ITEM;
-            mgrd[i][j] = NON_MONSTER;
+            grd[i][j]       = DNGN_UNSEEN;  // So generate_area will pick it up.
+            igrd[i][j]      = NON_ITEM;
+            mgrd[i][j]      = NON_MONSTER;
             env.cgrid[i][j] = EMPTY_CLOUD;
         }
-    }
 
-    ASSERT( env.cloud_no == 0 ); 
+    ASSERT( env.cloud_no == 0 );
 
     you.moveto(45, 35);
 
-    generate_area( 10, 10, (GXM - 10), (GYM - 10) );
+    _generate_area(MAPGEN_BORDER, MAPGEN_BORDER,
+                   GXM - MAPGEN_BORDER, GYM - MAPGEN_BORDER, true);
+
+#ifdef USE_TILE
+    // Update the wall colours.
+    TileLoadWall(true);
+#endif
+
+    _xom_check_nearness();
 
     grd[you.x_pos][you.y_pos] = DNGN_FLOOR;
-    if ( one_chance_in(5) )
-        place_feature_near( you.pos(), LOS_RADIUS,
-                            DNGN_FLOOR, DNGN_ALTAR_LUGONU, 50 );
+    if (one_chance_in(5))
+    {
+        _place_feature_near( you.pos(), LOS_RADIUS,
+                             DNGN_FLOOR, DNGN_ALTAR_LUGONU, 50 );
+    }
 
     place_transiting_monsters();
+    place_transiting_items();
 }
 
 //////////////////////////////////////////////////////////////////////////////
 // Abyss effects in other levels, courtesy Lugonu.
 
-static void place_corruption_seed(const coord_def &pos, int duration)
+static void _place_corruption_seed(const coord_def &pos, int duration)
 {
     env.markers.add(new map_corruption_marker(pos, duration));
 }
 
-static void initialise_level_corrupt_seeds(int power)
+static void _initialise_level_corrupt_seeds(int power)
 {
     const int low = power / 2, high = power * 3 / 2;
     int nseeds = random_range(1, std::min(2 + power / 110, 4));
@@ -422,21 +690,23 @@ static void initialise_level_corrupt_seeds(int power)
 #endif
 
     // The corruption centered on the player is free.
-    place_corruption_seed(you.pos(), high + 300);
+    _place_corruption_seed(you.pos(), high + 300);
 
     for (int i = 0; i < nseeds; ++i)
     {
         coord_def where;
         do
+        {
             where = coord_def(random2(GXM), random2(GYM));
+        }
         while (!in_bounds(where) || grd(where) != DNGN_FLOOR
                || env.markers.find(where, MAT_ANY));
 
-        place_corruption_seed(where, random_range(low, high, 2) + 300);
+        _place_corruption_seed(where, random_range(low, high, 2) + 300);
     }
 }
 
-static bool spawn_corrupted_servant_near(const coord_def &pos)
+static bool _spawn_corrupted_servant_near(const coord_def &pos)
 {
     // Thirty tries for a place
     for (int i = 0; i < 30; ++i)
@@ -445,28 +715,28 @@ static bool spawn_corrupted_servant_near(const coord_def &pos)
                            pos.y + random2avg(4, 3) + random2(3) );
         if (!in_bounds(p) || p == you.pos() || mgrd(p) != NON_MONSTER
             || !grid_compatible(DNGN_FLOOR, grd(p), true))
+        {
             continue;
+        }
 
         // Got a place, summon the beast.
         int level = 51;
-        monster_type mons =
-            pick_random_monster(level_id(LEVEL_ABYSS), level, level);
+        monster_type mons = pick_random_monster(level_id(LEVEL_ABYSS), level,
+                                                level);
         if (mons == MONS_PROGRAM_BUG)
             return (false);
 
         const beh_type beh =
-            one_chance_in(5 + you.skills[SK_INVOCATIONS] / 4)?
-            BEH_HOSTILE : BEH_NEUTRAL;
-        const int mid =
-            create_monster( mons, 5, beh, p.x, p.y, MHITNOT, 250 );
+            one_chance_in(5 + you.skills[SK_INVOCATIONS] / 4) ? BEH_HOSTILE
+                                                              : BEH_NEUTRAL;
+        const int mid = create_monster( mgen_data( mons, beh, 5, p ) );
 
         return (mid != -1);
     }
     return (false);
 }
-                                        
-static void apply_corruption_effect(
-    map_marker *marker, int duration)
+
+static void _apply_corruption_effect( map_marker *marker, int duration)
 {
     if (!duration)
         return;
@@ -477,10 +747,10 @@ static void apply_corruption_effect(
 
     for (int i = 0; i < neffects; ++i)
     {
-        if (random2(4000) < cmark->duration)
+        if (x_chance_in_y(cmark->duration, 4000)
+            && !_spawn_corrupted_servant_near(cmark->pos))
         {
-            if (!spawn_corrupted_servant_near(cmark->pos))
-                break;
+            break;
         }
     }
     cmark->duration -= duration;
@@ -499,17 +769,17 @@ void run_corruption_effects(int duration)
         if (mark->get_type() != MAT_CORRUPTION_NEXUS)
             continue;
 
-        apply_corruption_effect(mark, duration);
+        _apply_corruption_effect(mark, duration);
     }
 }
 
-static bool is_grid_corruptible(const coord_def &c)
+static bool _is_grid_corruptible(const coord_def &c)
 {
     if (c == you.pos())
         return (false);
-    
+
     const dungeon_feature_type feat = grd(c);
-    
+
     // Stairs and portals cannot be corrupted.
     if (grid_stair_direction(feat) != CMD_NO_CMD)
         return (false);
@@ -522,24 +792,23 @@ static bool is_grid_corruptible(const coord_def &c)
 
     case DNGN_METAL_WALL:
         return (one_chance_in(5));
-        
+
     case DNGN_STONE_WALL:
         return (one_chance_in(3));
 
     case DNGN_ROCK_WALL:
         return (!one_chance_in(3));
-        
+
     default:
         return (true);
     }
 }
 
-// Returns true if the square has <= 4 traversable neighbours. 
-static bool is_crowded_square(const coord_def &c)
+// Returns true if the square has <= 4 traversable neighbours.
+static bool _is_crowded_square(const coord_def &c)
 {
     int neighbours = 0;
     for (int xi = -1; xi <= 1; ++xi)
-    {
         for (int yi = -1; yi <= 1; ++yi)
         {
             if (!xi && !yi)
@@ -552,15 +821,14 @@ static bool is_crowded_square(const coord_def &c)
             if (++neighbours > 4)
                 return (false);
         }
-    }
+
     return (true);
 }
 
 // Returns true if the square has all opaque neighbours.
-static bool is_sealed_square(const coord_def &c)
+static bool _is_sealed_square(const coord_def &c)
 {
     for (int xi = -1; xi <= 1; ++xi)
-    {
         for (int yi = -1; yi <= 1; ++yi)
         {
             if (!xi && !yi)
@@ -573,43 +841,64 @@ static bool is_sealed_square(const coord_def &c)
             if (!grid_is_opaque(grd(n)))
                 return (false);
         }
-    }
+
     return (true);
 }
 
-static void corrupt_square(const crawl_environment &oenv, const coord_def &c)
+static void _corrupt_square(const crawl_environment &oenv, const coord_def &c)
 {
+    // To prevent the destruction of, say, branch entries.
+    bool preserve_feat = true;
     dungeon_feature_type feat = DNGN_UNSEEN;
     if (grid_altar_god(grd(c)) != GOD_NO_GOD)
     {
+        // altars may be safely overwritten, ha!
+        preserve_feat = false;
         if (!one_chance_in(3))
             feat = DNGN_ALTAR_LUGONU;
     }
     else
         feat = oenv.grid(c);
 
-    if (grid_is_trap(feat) || feat == DNGN_UNDISCOVERED_TRAP
-        || feat == DNGN_SECRET_DOOR || feat == DNGN_UNSEEN)
+    if (is_trap_square(feat) || feat == DNGN_SECRET_DOOR || feat == DNGN_UNSEEN)
         return;
 
     if (is_traversable(grd(c)) && !is_traversable(feat)
-        && is_crowded_square(c))
+        && _is_crowded_square(c))
+    {
         return;
+    }
 
-    if (!is_traversable(grd(c)) && is_traversable(feat) && is_sealed_square(c))
+    if (!is_traversable(grd(c)) && is_traversable(feat) && _is_sealed_square(c))
         return;
 
+    // What's this supposed to achieve? (jpeg)
+    // I mean, won't exits from the Abyss only turn up in the Abyss itself?
     if (feat == DNGN_EXIT_ABYSS)
         feat = DNGN_ENTER_ABYSS;
 
-    dungeon_terrain_changed(c, feat, true, true, true);
+    dungeon_terrain_changed(c, feat, preserve_feat, true, true);
     if (feat == DNGN_ROCK_WALL)
         env.grid_colours(c) = oenv.rock_colour;
     else if (feat == DNGN_FLOOR)
         env.grid_colours(c) = oenv.floor_colour;
+
+#ifdef USE_TILE
+    // Modify tile flavor to use corrupted tiles.
+    if (feat == DNGN_ROCK_WALL)
+    {
+        env.tile_flavor[c.x][c.y].wall =
+            TILE_DNGN_WALL_CORRUPT - get_wall_tile_idx() + random2(4);
+    }
+    else if (feat == DNGN_FLOOR)
+    {
+        env.tile_flavor[c.x][c.y].floor =
+            TILE_DNGN_FLOOR_CORRUPT - get_floor_tile_idx() + random2(4);
+    }
+#endif
 }
 
-static void corrupt_level_features(const crawl_environment &oenv)
+static void _corrupt_level_features(const crawl_environment &oenv)
 {
     std::vector<coord_def> corrupt_seeds;
     std::vector<map_marker*> corrupt_markers =
@@ -617,9 +906,8 @@ static void corrupt_level_features(const crawl_environment &oenv)
 
     for (int i = 0, size = corrupt_markers.size(); i < size; ++i)
         corrupt_seeds.push_back(corrupt_markers[i]->pos);
-    
+
     for (int y = MAPGEN_BORDER; y < GYM - MAPGEN_BORDER; ++y)
-    {
         for (int x = MAPGEN_BORDER; x < GXM - MAPGEN_BORDER; ++x)
         {
             const coord_def c(x, y);
@@ -632,28 +920,29 @@ static void corrupt_level_features(const crawl_environment &oenv)
             }
 
             if ((distance < 6 || one_chance_in(1 + distance - 6))
-                && is_grid_corruptible(c))
+                && _is_grid_corruptible(c))
             {
-                corrupt_square(oenv, c);
+                _corrupt_square(oenv, c);
             }
         }
-    }
 }
 
-static bool is_level_corrupted()
+static bool _is_level_corrupted()
 {
     if (you.level_type == LEVEL_ABYSS
         || you.level_type == LEVEL_PANDEMONIUM
         || player_in_hell()
         || player_in_branch(BRANCH_VESTIBULE_OF_HELL))
+    {
         return (true);
+    }
 
     return (!!env.markers.find(MAT_CORRUPTION_NEXUS));
 }
 
-static bool is_level_incorruptible()
+static bool _is_level_incorruptible()
 {
-    if (is_level_corrupted())
+    if (_is_level_corrupted())
     {
         mpr("This place is already infused with evil and corruption.");
         return (true);
@@ -662,15 +951,20 @@ static bool is_level_incorruptible()
     return (false);
 }
 
-static void corrupt_choose_colours()
+static void _corrupt_choose_colours()
 {
     int colour = BLACK;
     do
+    {
         colour = random_uncommon_colour();
+    }
     while (colour == env.rock_colour || colour == LIGHTGREY || colour == WHITE);
     env.rock_colour = colour;
+
     do
+    {
         colour = random_uncommon_colour();
+    }
     while (colour == env.floor_colour || colour == LIGHTGREY
            || colour == WHITE);
     env.floor_colour = colour;
@@ -678,30 +972,30 @@ static void corrupt_choose_colours()
 
 bool lugonu_corrupt_level(int power)
 {
-    if (is_level_incorruptible())
+    if (_is_level_incorruptible())
         return (false);
 
     mprf(MSGCH_GOD, "Lugonu's Hand of Corruption reaches out!");
 
     you.flash_colour = EC_MUTAGENIC;
     viewwindow(true, false);
-    
-    initialise_level_corrupt_seeds(power);
+
+    _initialise_level_corrupt_seeds(power);
 
     std::auto_ptr<crawl_environment> backup(new crawl_environment(env));
     generate_abyss();
-    generate_area(MAPGEN_BORDER, MAPGEN_BORDER,
-                  GXM - MAPGEN_BORDER, GYM - MAPGEN_BORDER);
+    _generate_area(MAPGEN_BORDER, MAPGEN_BORDER,
+                   GXM - MAPGEN_BORDER, GYM - MAPGEN_BORDER, false);
 
-    corrupt_choose_colours();
+    _corrupt_choose_colours();
 
     std::auto_ptr<crawl_environment> abyssal(new crawl_environment(env));
     env = *backup;
     backup.reset(NULL);
     dungeon_events.clear();
     env.markers.activate_all(false);
-    
-    corrupt_level_features(*abyssal);
+
+    _corrupt_level_features(*abyssal);
     run_corruption_effects(300);
 
     you.flash_colour = EC_MUTAGENIC;
