@@ -3,7 +3,7 @@
  *  Summary:    Dumps character info out to the morgue file.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-04T14:47:09.060049Z $
  *
  *  Change History (most recent first):
  *
@@ -24,7 +24,7 @@
 #include <string.h>
 #include <fcntl.h>
 #include <stdlib.h>
-#if !defined(__IBMCPP__)
+#if !defined(__IBMCPP__) && !defined(_MSC_VER)
 #include <unistd.h>
 #endif
 #include <ctype.h>
@@ -66,31 +66,32 @@
 
 struct dump_params;
 
-static void sdump_header(dump_params &);
-static void sdump_stats(dump_params &);
-static void sdump_location(dump_params &);
-static void sdump_religion(dump_params &);
-static void sdump_burden(dump_params &);
-static void sdump_hunger(dump_params &);
-static void sdump_transform(dump_params &);
-static void sdump_visits(dump_params &);
-static void sdump_misc(dump_params &);
-static void sdump_turns_by_place(dump_params &);
-static void sdump_notes(dump_params &);
-static void sdump_inventory(dump_params &);
-static void sdump_skills(dump_params &);
-static void sdump_spells(dump_params &);
-static void sdump_mutations(dump_params &);
-static void sdump_messages(dump_params &);
-static void sdump_screenshot(dump_params &);
-static void sdump_kills_by_place(dump_params &);
-static void sdump_kills(dump_params &);
-static void sdump_newline(dump_params &);
-static void sdump_overview(dump_params &);
-static void sdump_hiscore(dump_params &);
-static void sdump_separator(dump_params &);
+static void _sdump_header(dump_params &);
+static void _sdump_stats(dump_params &);
+static void _sdump_location(dump_params &);
+static void _sdump_religion(dump_params &);
+static void _sdump_burden(dump_params &);
+static void _sdump_hunger(dump_params &);
+static void _sdump_transform(dump_params &);
+static void _sdump_visits(dump_params &);
+static void _sdump_misc(dump_params &);
+static void _sdump_turns_by_place(dump_params &);
+static void _sdump_notes(dump_params &);
+static void _sdump_inventory(dump_params &);
+static void _sdump_skills(dump_params &);
+static void _sdump_spells(dump_params &);
+static void _sdump_mutations(dump_params &);
+static void _sdump_messages(dump_params &);
+static void _sdump_screenshot(dump_params &);
+static void _sdump_kills_by_place(dump_params &);
+static void _sdump_kills(dump_params &);
+static void _sdump_newline(dump_params &);
+static void _sdump_overview(dump_params &);
+static void _sdump_hiscore(dump_params &);
+static void _sdump_monster_list(dump_params &);
+static void _sdump_separator(dump_params &);
 #ifdef CLUA_BINDINGS
-static void sdump_lua(dump_params &);
+static void _sdump_lua(dump_params &);
 #endif
 static bool write_dump(const std::string &fname, dump_params &);
 
@@ -118,34 +119,35 @@ struct dump_params
 };
 
 static dump_section_handler dump_handlers[] = {
-    { "header",         sdump_header        },
-    { "stats",          sdump_stats         },
-    { "location",       sdump_location      },
-    { "religion",       sdump_religion      },
-    { "burden",         sdump_burden        },
-    { "hunger",         sdump_hunger        },
-    { "transform",      sdump_transform     },
-    { "visits",         sdump_visits        },
-    { "misc",           sdump_misc          },
-    { "turns_by_place", sdump_turns_by_place},
-    { "notes",          sdump_notes         },
-    { "inventory",      sdump_inventory     },
-    { "skills",         sdump_skills        },
-    { "spells",         sdump_spells        },
-    { "mutations",      sdump_mutations     },
-    { "messages",       sdump_messages      },
-    { "screenshot",     sdump_screenshot    },
-    { "kills_by_place", sdump_kills_by_place},
-    { "kills",          sdump_kills         },
-    { "overview",       sdump_overview      },
-    { "hiscore",        sdump_hiscore       },
+    { "header",         _sdump_header        },
+    { "stats",          _sdump_stats         },
+    { "location",       _sdump_location      },
+    { "religion",       _sdump_religion      },
+    { "burden",         _sdump_burden        },
+    { "hunger",         _sdump_hunger        },
+    { "transform",      _sdump_transform     },
+    { "visits",         _sdump_visits        },
+    { "misc",           _sdump_misc          },
+    { "turns_by_place", _sdump_turns_by_place},
+    { "notes",          _sdump_notes         },
+    { "inventory",      _sdump_inventory     },
+    { "skills",         _sdump_skills        },
+    { "spells",         _sdump_spells        },
+    { "mutations",      _sdump_mutations     },
+    { "messages",       _sdump_messages      },
+    { "screenshot",     _sdump_screenshot    },
+    { "kills_by_place", _sdump_kills_by_place},
+    { "kills",          _sdump_kills         },
+    { "overview",       _sdump_overview      },
+    { "hiscore",        _sdump_hiscore       },
+    { "monlist",        _sdump_monster_list  },
 
     // Conveniences for the .crawlrc artist.
-    { "",               sdump_newline       },
-    { "-",              sdump_separator     },
+    { "",               _sdump_newline       },
+    { "-",              _sdump_separator     },
 
 #ifdef CLUA_BINDINGS
-    { NULL,             sdump_lua           }
+    { NULL,             _sdump_lua           }
 #else
     { NULL,             NULL                }
 #endif
@@ -167,12 +169,12 @@ static void dump_section(dump_params &par)
 bool dump_char(const std::string &fname, bool show_prices, bool full_id,
                const scorefile_entry *se)
 {
-    // start with enough room for 100 80 character lines
+    // Start with enough room for 100 80 character lines.
     std::string text;
     text.reserve(100 * 80);
 
     dump_params par(text, "", show_prices, full_id, se);
-    
+
     for (int i = 0, size = Options.dump_order.size(); i < size; ++i)
     {
         par.section = Options.dump_order[i];
@@ -182,28 +184,21 @@ bool dump_char(const std::string &fname, bool show_prices, bool full_id,
     return write_dump(fname, par);
 }
 
-static void sdump_header(dump_params &par)
+static void _sdump_header(dump_params &par)
 {
     par.text += " " CRAWL " version " VERSION " character file.\n\n";
 }
 
-static void sdump_stats(dump_params &par)
+static void _sdump_stats(dump_params &par)
 {
-    std::vector<formatted_string> vfs =
-        get_full_detail(par.full_id, par.se? par.se->points : -1);
-
-    for (unsigned int i = 0; i < vfs.size(); i++)
-    {
-        par.text += vfs[i];
-        par.text += '\n';
-    }
+    par.text += dump_overview_screen(par.full_id);
     par.text += "\n\n";
 }
 
-static void sdump_burden(dump_params &par)
+static void _sdump_burden(dump_params &par)
 {
     std::string verb = par.se? "were" : "are";
-    
+
     switch (you.burden_state)
     {
     case BS_OVERLOADED:
@@ -217,18 +212,18 @@ static void sdump_burden(dump_params &par)
     }
 }
 
-static void sdump_hunger(dump_params &par)
+static void _sdump_hunger(dump_params &par)
 {
     if (par.se)
         par.text += "You were ";
     else
         par.text += "You are ";
-        
+
     par.text += hunger_level();
     par.text += ".\n\n";
 }
 
-static void sdump_transform(dump_params &par)
+static void _sdump_transform(dump_params &par)
 {
     std::string &text(par.text);
     if (you.attribute[ATTR_TRANSFORMATION])
@@ -273,7 +268,7 @@ static void sdump_transform(dump_params &par)
     }
 }
 
-static void sdump_visits(dump_params &par)
+static void _sdump_visits(dump_params &par)
 {
     std::string &text(par.text);
 
@@ -343,31 +338,31 @@ static void sdump_visits(dump_params &par)
     text += "\n";
 }
 
-static void sdump_misc(dump_params &par)
+static void _sdump_misc(dump_params &par)
 {
-    sdump_location(par);
-    sdump_religion(par);
-    sdump_burden(par);
-    sdump_hunger(par);
-    sdump_transform(par);
-    sdump_visits(par);
+    _sdump_location(par);
+    _sdump_religion(par);
+    _sdump_burden(par);
+    _sdump_hunger(par);
+    _sdump_transform(par);
+    _sdump_visits(par);
 }
 
-#define TO_PERCENT(x, y) (100.0 * ((float) (x)) / ((float) (y)))
+#define TO_PERCENT(x, y) (100.0f * ((float) (x)) / ((float) (y)))
 
-static std::string sdump_turns_place_info(PlaceInfo place_info,
-                                          std::string name = "")
+static std::string _sdump_turns_place_info(PlaceInfo place_info,
+                                           std::string name = "")
 {
     PlaceInfo   gi = you.global_info;
     std::string out;
 
-    if (name == "")
+    if (name.empty())
         name = place_info.short_name();
 
     float a, b, c, d, e, f;
     unsigned int non_interlevel =
         place_info.turns_total - place_info.turns_interlevel;
-    unsigned int global_non_interlevel = 
+    unsigned int global_non_interlevel =
         gi.turns_total - gi.turns_interlevel;
 
 
@@ -387,24 +382,24 @@ static std::string sdump_turns_place_info(PlaceInfo place_info,
     return out;
 }
 
-static void sdump_turns_by_place(dump_params &par)
+static void _sdump_turns_by_place(dump_params &par)
 {
     std::string &text(par.text);
 
     std::vector<PlaceInfo> all_visited =
         you.get_all_place_info(true);
 
-    text += 
+    text +=
 "Table legend:\n"
 " A = Turns spent in this place as a percentage of turns spent in the\n"
 "     entire game.\n"
-" B = Non-inter-level travel turns spent in this place as a perecentage of\n"
+" B = Non-inter-level travel turns spent in this place as a percentage of\n"
 "     non-inter-level travel turns spent in the entire game.\n"
-" C = Inter-level travel turns spent in this place as a perecentage  of\n"
+" C = Inter-level travel turns spent in this place as a percentage of\n"
 "     turns spent in this place.\n"
 " D = Turns resting spent in this place as a percentage of non-inter-level\n"
 "     travel turns spent in this place.\n"
-" E = Turns spent auto-exloring this place as a percentage of\n"
+" E = Turns spent auto-exploring this place as a percentage of\n"
 "     non-inter-level travel turns spent in this place.\n"
 " F = Non-inter-level travel turns spent in this place divided by the\n"
 "     number of levels of this place that you've seen.\n\n";
@@ -414,12 +409,12 @@ static void sdump_turns_by_place(dump_params &par)
     text += "               ";
     text += "+-------+-------+-------+-------+-------+----------------------\n";
 
-    text += sdump_turns_place_info(you.global_info, "Total");
+    text += _sdump_turns_place_info(you.global_info, "Total");
 
     for (unsigned int i = 0; i < all_visited.size(); i++)
     {
         PlaceInfo pi = all_visited[i];
-        text += sdump_turns_place_info(pi);
+        text += _sdump_turns_place_info(pi);
     }
 
     text += "               ";
@@ -428,12 +423,12 @@ static void sdump_turns_by_place(dump_params &par)
     text += "\n";
 }
 
-static void sdump_newline(dump_params &par)
+static void _sdump_newline(dump_params &par)
 {
     par.text += "\n";
 }
 
-static void sdump_separator(dump_params &par)
+static void _sdump_separator(dump_params &par)
 {
     par.text += std::string(79, '-') + "\n";
 }
@@ -441,7 +436,7 @@ static void sdump_separator(dump_params &par)
 #ifdef CLUA_BINDINGS
 // Assume this is an arbitrary Lua function name, call the function and
 // dump whatever it returns.
-static void sdump_lua(dump_params &par)
+static void _sdump_lua(dump_params &par)
 {
     std::string luatext;
     if (!clua.callfn(par.section.c_str(), ">s", &luatext)
@@ -537,7 +532,7 @@ std::string munge_description(const std::string & inStr)
     return (outStr);
 }                               // end munge_description()
 
-static void sdump_messages(dump_params &par)
+static void _sdump_messages(dump_params &par)
 {
     // A little message history:
     if (Options.dump_message_count > 0)
@@ -547,21 +542,22 @@ static void sdump_messages(dump_params &par)
     }
 }
 
-static void sdump_screenshot(dump_params &par)
+static void _sdump_screenshot(dump_params &par)
 {
     par.text += screenshot();
     par.text += "\n\n";
 }
 
-static void sdump_notes(dump_params &par)
+static void _sdump_notes(dump_params &par)
 {
     std::string &text(par.text);
-    if ( note_list.size() == 0 || Options.use_notes == false )
+    if ( note_list.empty() )
         return;
 
     text += "\nNotes\nTurn   | Place   | Note\n";
     text += "--------------------------------------------------------------\n";
-    for ( unsigned i = 0; i < note_list.size(); ++i ) {
+    for ( unsigned i = 0; i < note_list.size(); ++i )
+    {
         text += note_list[i].describe();
         text += "\n";
     }
@@ -573,12 +569,14 @@ static void sdump_notes(dump_params &par)
  // dump_location
  //
  //---------------------------------------------------------------
-static void sdump_location(dump_params &par)
+static void _sdump_location(dump_params &par)
 {
-    if (you.your_level == -1 
-            && you.where_are_you == BRANCH_MAIN_DUNGEON
-            && you.level_type == LEVEL_DUNGEON)
+    if (you.your_level == -1
+        && you.where_are_you == BRANCH_MAIN_DUNGEON
+        && you.level_type == LEVEL_DUNGEON)
+    {
         par.text += "You escaped";
+    }
     else if (par.se)
         par.text += "You were " + prep_branch_level_name();
     else
@@ -588,7 +586,7 @@ static void sdump_location(dump_params &par)
     par.text += "\n";
 }                               // end dump_location()
 
-static void sdump_religion(dump_params &par)
+static void _sdump_religion(dump_params &par)
 {
     std::string &text(par.text);
     if (you.religion != GOD_NO_GOD)
@@ -600,16 +598,20 @@ static void sdump_religion(dump_params &par)
         text += god_name(you.religion);
         text += ".\n";
 
-        std::string verb = par.se? "was" : "is";
-        if (!player_under_penance())
+        if (you.religion != GOD_XOM)
         {
-            text += god_prayer_reaction();
-            text += "\n";
-        }
-        else
-        {
-            text += god_name(you.religion);
-            text += verb + " demanding penance.\n";
+            if (!player_under_penance())
+            {
+                text += god_prayer_reaction();
+                text += "\n";
+            }
+            else
+            {
+                std::string verb = par.se ? "was" : "is";
+
+                text += god_name(you.religion);
+                text += " " + verb + " demanding penance.\n";
+            }
         }
     }
 }
@@ -671,7 +673,7 @@ static bool dump_item_origin(const item_def &item, int value)
  // dump_inventory
  //
  //---------------------------------------------------------------
-static void sdump_inventory(dump_params &par)
+static void _sdump_inventory(dump_params &par)
 {
     int i, j;
 
@@ -683,16 +685,14 @@ static void sdump_inventory(dump_params &par)
     char tmp_quant[20];
 
     for (i = 0; i < OBJ_GOLD; i++)
-    {
         inv_class2[i] = 0;
-    }
 
     for (i = 0; i < ENDOFPACK; i++)
     {
         if (is_valid_item( you.inv[i] ))
         {
             // adds up number of each class in invent.
-            inv_class2[you.inv[i].base_type]++;     
+            inv_class2[you.inv[i].base_type]++;
             inv_count++;
         }
     }
@@ -704,8 +704,7 @@ static void sdump_inventory(dump_params &par)
     }
     else
     {
-        text += "  Inventory:";
-        text += "\n";
+        text += "Inventory:\n\n";
 
         for (i = 0; i < OBJ_GOLD; i++)
         {
@@ -746,7 +745,7 @@ static void sdump_inventory(dump_params &par)
                         {
                             text += " (";
 
-                            itoa( ival = item_value( you.inv[j], true ), 
+                            itoa( ival = item_value( you.inv[j], true ),
                                   tmp_quant, 10 );
 
                             text += tmp_quant;
@@ -754,14 +753,14 @@ static void sdump_inventory(dump_params &par)
                         }
 
                         if (origin_describable(you.inv[j])
-                                && dump_item_origin(you.inv[j], ival))
+                            && dump_item_origin(you.inv[j], ival))
                         {
                             text += "\n" "   (" + origin_desc(you.inv[j]) + ")";
                         }
 
                         if (is_dumpable_artefact( you.inv[j], false ))
                         {
-                            text2 = get_item_description( you.inv[j], 
+                            text2 = get_item_description( you.inv[j],
                                                           false,
                                                           true );
 
@@ -784,7 +783,7 @@ static void sdump_inventory(dump_params &par)
 // dump_skills
 //
 //---------------------------------------------------------------
-static void sdump_skills(dump_params &par)
+static void _sdump_skills(dump_params &par)
 {
     std::string &text(par.text);
     char tmp_quant[20];
@@ -808,7 +807,7 @@ static void sdump_skills(dump_params &par)
         if (you.skills[i] > 0)
         {
             text += ( (you.skills[i] == 27)   ? " * " :
-                      (you.practise_skill[i]) ? " + " 
+                      (you.practise_skill[i]) ? " + "
                                               : " - " );
 
             text += "Level ";
@@ -846,7 +845,7 @@ static std::string spell_type_shortname(int spell_class, bool slash)
 // dump_spells
 //
 //---------------------------------------------------------------
-static void sdump_spells(dump_params &par)
+static void _sdump_spells(dump_params &par)
 {
     std::string &text(par.text);
     char tmp_quant[20];
@@ -874,13 +873,13 @@ static void sdump_spells(dump_params &par)
     int spell_levels = player_spell_levels();
 
     std::string verb = par.se? "had" : "have";
-    
+
     if (spell_levels == 1)
         text += "You " + verb + " one spell level left.";
     else if (spell_levels == 0)
     {
         verb = par.se? "couldn't" : "cannot";
-        
+
         text += "You " + verb + " memorise any spells.";
     }
     else
@@ -899,7 +898,7 @@ static void sdump_spells(dump_params &par)
     if (!you.spell_no)
     {
         verb = par.se? "didn't" : "don't";
-        
+
         text += "You " + verb + " know any spells.\n\n";
     }
     else
@@ -925,8 +924,8 @@ static void sdump_spells(dump_params &par)
 
                 if ( spell_line.length() > 24 )
                     spell_line = spell_line.substr(0, 24);
-                
-                for (int i = spell_line.length(); i < 26; i++) 
+
+                for (int i = spell_line.length(); i < 26; i++)
                     spell_line += ' ';
 
                 bool already = false;
@@ -966,18 +965,18 @@ static void sdump_spells(dump_params &par)
 }                               // end dump_spells()
 
 
-static void sdump_kills(dump_params &par)
+static void _sdump_kills(dump_params &par)
 {
     par.text += you.kills->kill_info();
 }
 
-static std::string sdump_kills_place_info(PlaceInfo place_info,
+static std::string _sdump_kills_place_info(PlaceInfo place_info,
                                           std::string name = "")
 {
     PlaceInfo   gi = you.global_info;
     std::string out;
 
-    if (name == "")
+    if (name.empty())
         name = place_info.short_name();
 
     unsigned int global_total_kills = 0;
@@ -1020,7 +1019,7 @@ static std::string sdump_kills_place_info(PlaceInfo place_info,
     return out;
 }
 
-static void sdump_kills_by_place(dump_params &par)
+static void _sdump_kills_by_place(dump_params &par)
 {
     std::string &text(par.text);
 
@@ -1030,20 +1029,20 @@ static void sdump_kills_by_place(dump_params &par)
     std::string result = "";
 
     std::string header =
-"Table legend:\n"
-" A = Kills in this place as a percentage of kills in entire the game.\n"
-" B = Kills by you in this place as a percentage of kills by you in\n"
-"     the entire game.\n"
-" C = Kills by friends in this place as a percentage of kills by\n"
-"     friends in the entire game.\n"
-" D = Other kills in this place as a percentage of other kills in the\n"
-"     entire game.\n"
-" E = Character level experience gained in this place as a percentage of\n"
-"     character level experience gained in the entire game.\n"
-" F = Skills experience gained in this place as a percentage of skills\n"
-"     experience gained in the entire game.\n"
-" G = Experience gained in this place divided by the number of levels of\n"
-"     this place that you have seen.\n\n";
+    "Table legend:\n"
+    " A = Kills in this place as a percentage of kills in entire the game.\n"
+    " B = Kills by you in this place as a percentage of kills by you in\n"
+    "     the entire game.\n"
+    " C = Kills by friends in this place as a percentage of kills by\n"
+    "     friends in the entire game.\n"
+    " D = Other kills in this place as a percentage of other kills in the\n"
+    "     entire game.\n"
+    " E = Character level experience gained in this place as a percentage of\n"
+    "     character level experience gained in the entire game.\n"
+    " F = Skills experience gained in this place as a percentage of skills\n"
+    "     experience gained in the entire game.\n"
+    " G = Experience gained in this place divided by the number of levels of\n"
+    "     this place that you have seen.\n\n";
 
     header += "               ";
     header += "    A       B       C       D       E       F          G\n";
@@ -1053,21 +1052,19 @@ static void sdump_kills_by_place(dump_params &par)
     std::string footer = "               ";
     footer += "+-------+-------+-------+-------+-------+-------+--------------\n";
 
-    result += sdump_kills_place_info(you.global_info, "Total");
+    result += _sdump_kills_place_info(you.global_info, "Total");
 
     for (unsigned int i = 0; i < all_visited.size(); i++)
     {
         PlaceInfo pi = all_visited[i];
-        result += sdump_kills_place_info(pi);
+        result += _sdump_kills_place_info(pi);
     }
 
     if (result.length() > 0)
-    {
         text += header + result + footer + "\n";
-    }
 }
 
-static void sdump_overview(dump_params &par)
+static void _sdump_overview(dump_params &par)
 {
     std::string overview =
         formatted_string::parse_string(overview_description_string());
@@ -1076,30 +1073,30 @@ static void sdump_overview(dump_params &par)
     par.text += "\n\n";
 }
 
-static void sdump_hiscore(dump_params &par)
+static void _sdump_hiscore(dump_params &par)
 {
     if (!par.se)
         return;
-        
+
     std::string hiscore = hiscores_format_single_long( *(par.se), true );
     trim_string(hiscore);
     par.text += hiscore;
     par.text += "\n\n";
 }
 
-static void sdump_mutations(dump_params &par)
+static void _sdump_monster_list(dump_params &par)
+{
+    std::string monlist = mpr_monster_list(par.se);
+    trim_string(monlist);
+    par.text += monlist;
+    par.text += "\n\n";
+}
+
+static void _sdump_mutations(dump_params &par)
 {
     std::string &text(par.text);
-    // Can't use how_mutated() here, as it doesn't count demonic powers
-    int xz = 0;
 
-    for (int xy = 0; xy < NUM_MUTATIONS; ++xy)
-    {
-        if (you.mutation[xy] > 0)
-            xz++;
-    }
-
-    if (xz > 0)
+    if (how_mutated(true, false))
     {
         text += "\n";
         text += describe_mutations();
@@ -1113,17 +1110,23 @@ static void sdump_mutations(dump_params &par)
 
 const char *hunger_level(void)
 {
-    return ((you.hunger <= 1000) ? "starving" :
-             (you.hunger <= 2600) ? "hungry" :
-             (you.hunger < 7000) ? "not hungry" :
-             (you.hunger < 11000) ? "full" : "completely stuffed");
+    const bool vamp = (you.species == SP_VAMPIRE);
+
+    return ((you.hunger <= 1000) ? (vamp ? "bloodless" : "starving") :
+            (you.hunger <= 1533) ? (vamp ? "near bloodless" : "near starving") :
+            (you.hunger <= 2066) ? (vamp ? "very thirsty" : "very hungry") :
+            (you.hunger <= 2600) ? (vamp ? "thirsty" : "hungry") :
+            (you.hunger <  7000) ? (vamp ? "not thirsty" : "not hungry") :
+            (you.hunger <  9000) ? "full" :
+            (you.hunger < 11000) ? "very full"
+                                 : (vamp ? "almost alive" : "completely stuffed"));
 }
 
 static std::string morgue_directory()
 {
-    std::string dir =
-        !Options.morgue_dir.empty()? Options.morgue_dir :
-        !SysEnv.crawl_dir.empty()  ? SysEnv.crawl_dir : "";
+    std::string dir = (!Options.morgue_dir.empty() ? Options.morgue_dir :
+                       !SysEnv.crawl_dir.empty()   ? SysEnv.crawl_dir
+                                                   : "");
 
     if (!dir.empty() && dir[dir.length() - 1] != FILE_SEPARATOR)
         dir += FILE_SEPARATOR;
@@ -1131,7 +1134,7 @@ static std::string morgue_directory()
     return (dir);
 }
 
-static void dump_map(const char* fname)
+void dump_map(FILE *fp)
 {
     // Duplicate the screenshot() trick.
     FixedVector<unsigned, NUM_DCHAR_TYPES> char_table_bk;
@@ -1140,42 +1143,43 @@ static void dump_map(const char* fname)
     init_char_table(CSET_ASCII);
     init_feature_table();
 
-    FILE* fp = fopen(fname, "w");
-    if ( !fp )
-        return;
-
     int min_x = GXM-1, max_x = 0, min_y = GYM-1, max_y = 0;
 
-    for (int i = 0; i < GXM; i++)
-        for (int j = 0; j < GYM; j++)
+    for (int i = X_BOUND_1; i <= X_BOUND_2; i++)
+        for (int j = Y_BOUND_1; j <= Y_BOUND_2; j++)
             if (env.map[i][j].known())
             {
-                if ( i > max_x )
-                    max_x = i;
-                if ( i < min_x )
-                    min_x = i;
-                if ( j > max_y )
-                    max_y = j;
-                if ( j < min_y )
-                    min_y = j;
+                if (i > max_x) max_x = i;
+                if (i < min_x) min_x = i;
+                if (j > max_y) max_y = j;
+                if (j < min_y) min_y = j;
             }
-    
-    for ( int y = min_y; y <= max_y; ++y )
+
+    for (int y = min_y; y <= max_y; ++y)
     {
-        for ( int x = min_x; x <= max_x; ++x )
+        for (int x = min_x; x <= max_x; ++x)
             fputc( env.map[x][y].glyph(), fp );
+
         fputc('\n', fp);
     }
-    fclose(fp);
 
     // Restore char and feature tables
     Options.char_table = char_table_bk;
     init_feature_table();
 }
 
-static bool write_dump(
-        const std::string &fname,
-        dump_params &par)
+void dump_map(const char* fname)
+{
+    FILE* fp = fopen(fname, "w");
+    if (!fp)
+        return;
+
+    dump_map(fp);
+
+    fclose(fp);
+}
+
+static bool write_dump( const std::string &fname, dump_params &par)
 {
     bool succeeded = false;
 
@@ -1183,10 +1187,12 @@ static bool write_dump(
 
     file_name += strip_filename_unsafe_chars(fname);
 
+    StashTrack.update_corpses();
+
     std::string stash_file_name;
     stash_file_name = file_name;
     stash_file_name += ".lst";
-    stashes.dump(stash_file_name.c_str(), par.full_id);
+    StashTrack.dump(stash_file_name.c_str(), par.full_id);
 
     std::string map_file_name = file_name + ".map";
     dump_map(map_file_name.c_str());
@@ -1205,53 +1211,45 @@ static bool write_dump(
         succeeded = true;
     }
     else
-        mprf("Error opening file '%s'", file_name.c_str());
-    
+        mprf(MSGCH_ERROR, "Error opening file '%s'", file_name.c_str());
+
     return (succeeded);
 }
 
 void display_notes()
 {
     Menu scr;
-    scr.set_title( new MenuEntry("Turn   | Place   | Note"));
-    for ( unsigned int i = 0; i < note_list.size(); ++i )
+    scr.set_tag("notes");
+    scr.set_title(new MenuEntry("Turn   | Place   | Note"));
+    for (unsigned int i = 0; i < note_list.size(); ++i)
     {
         std::string prefix = note_list[i].describe(true, true, false);
         std::string suffix = note_list[i].describe(false, false, true);
-        if ( suffix.empty() )
+        if (suffix.empty())
             continue;
+
         int spaceleft = get_number_of_cols() - prefix.length() - 1;
-        if ( spaceleft <= 0 )
+        if (spaceleft <= 0)
             return;
-        linebreak_string(suffix, spaceleft - 4, spaceleft);
+
+        // Use smarter linebreak function.
+        // was:  linebreak_string(suffix, spaceleft - 4, spaceleft);
+        linebreak_string2(suffix, spaceleft);
         std::vector<std::string> parts = split_string("\n", suffix);
-        if ( parts.empty() ) // disregard pure-whitespace notes
+        if (parts.empty()) // Disregard pure-whitespace notes.
             continue;
+
         scr.add_entry(new MenuEntry(prefix + parts[0]));
-        for ( unsigned int j = 1; j < parts.size(); ++j )
+        for (unsigned int j = 1; j < parts.size(); ++j)
+        {
             scr.add_entry(new MenuEntry(std::string(prefix.length()-2, ' ') +
                                         std::string("| ") + parts[j]));
-
+        }
     }
     scr.show();
     redraw_screen();
 }
 
-void resists_screen()
-{
-    std::vector<formatted_string> vfs = get_full_detail(false);
-    clrscr();
-    gotoxy(1,1);
-    textcolor(LIGHTGREY);
-    
-    formatted_scroller scr;
-    for ( unsigned i = 0; i < vfs.size(); ++i )
-        scr.add_item_formatted_string(vfs[i]);
-
-    scr.show();
-    redraw_screen();
-}
-
 #ifdef DGL_WHEREIS
 ///////////////////////////////////////////////////////////////////////////
 // whereis player
