@@ -1,9 +1,9 @@
 /*
- *  File:       chardump.cc
+ *  File:       chardump.h
  *  Summary:    Dumps character info out to the morgue file.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2007-11-20T01:01:18.878413Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-06-12T09:03:37.281687Z $
  *
  *  Change History (most recent first):
  *
@@ -16,6 +16,7 @@
 #define CHARDUMP_H
 
 #include <string>
+#include <cstdio>
 
 enum item_origin_dump_selector
 {
@@ -36,7 +37,8 @@ bool dump_char(const std::string &fname,
                bool show_prices,
                bool full_id = false,
                const scorefile_entry *se = NULL);
-void resists_screen();
+void dump_map(const char* fname);
+void dump_map(FILE *fp);
 void display_notes();
 std::string munge_description(const std::string &inStr);
 const char *hunger_level(void);
