@@ -1,9 +1,14 @@
 /*
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T16:15:56.136594Z $
+ *  File:       clua.cc
+ *  Created by: dshaligram on Wed Aug 2 12:54:15 2006 UTC
+ *
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-06T16:47:06.336575Z $
  */
 
 #include "AppHdr.h"
 
+#include <algorithm>
+
 #include "clua.h"
 
 #include "abl-show.h"
@@ -27,6 +32,7 @@
 #include "mapdef.h"
 #include "message.h"
 #include "mon-util.h"
+#include "newgame.h"
 #include "output.h"
 #include "player.h"
 #include "randart.h"
@@ -57,16 +63,12 @@
     } \
     while (false)
 
-static int clua_panic(lua_State *);
-static void clua_throttle_hook(lua_State *, lua_Debug *);
-static void *clua_allocator(void *ud, void *ptr, size_t osize, size_t nsize);
-static int clua_guarded_pcall(lua_State *);
-static int clua_dofile(lua_State *);
-static int clua_loadfile(lua_State *);
-
-CLua clua(true);
-
-const int CLua::MAX_THROTTLE_SLEEPS;
+static int  _clua_panic(lua_State *);
+static void _clua_throttle_hook(lua_State *, lua_Debug *);
+static void *_clua_allocator(void *ud, void *ptr, size_t osize, size_t nsize);
+static int  _clua_guarded_pcall(lua_State *);
+static int  _clua_dofile(lua_State *);
+static int  _clua_loadfile(lua_State *);
 
 CLua::CLua(bool managed)
     : error(), managed_vm(managed), shutting_down(false),
@@ -123,7 +125,7 @@ void CLua::setregistry(const char *name)
     lua_settable(state(), LUA_REGISTRYINDEX);
 }
 
-void CLua::getregistry(const char *name) 
+void CLua::getregistry(const char *name)
 {
     lua_pushstring(state(), name);
     lua_gettable(state(), LUA_REGISTRYINDEX);
@@ -154,7 +156,7 @@ int CLua::file_write(lua_State *ls)
     FILE *f = sf->get_file();
     if (!f)
         return (0);
-    
+
     const char *text = luaL_checkstring(ls, 2);
     if (text)
         fprintf(f, "%s", text);
@@ -169,7 +171,7 @@ FILE *CLua::CLuaSave::get_file()
     return (handle);
 }
 
-void CLua::set_error(int err, lua_State *ls) 
+void CLua::set_error(int err, lua_State *ls)
 {
     if (!err)
     {
@@ -190,7 +192,7 @@ void CLua::init_throttle()
 {
     if (!managed_vm)
         return;
-    
+
     if (throttle_unit_lines <= 0)
         throttle_unit_lines = 500;
 
@@ -199,10 +201,10 @@ void CLua::init_throttle()
 
     if (throttle_sleep_end < throttle_sleep_start)
         throttle_sleep_end = throttle_sleep_start;
-    
+
     if (!mixed_call_depth)
     {
-        lua_sethook(_state, clua_throttle_hook,
+        lua_sethook(_state, _clua_throttle_hook,
                     LUA_MASKCOUNT, throttle_unit_lines);
         throttle_sleep_ms = 0;
         n_throttle_sleeps = 0;
@@ -226,7 +228,7 @@ int CLua::execstring(const char *s, const char *context, int nresults)
     int err = 0;
     if ((err = loadstring(s, context)))
         return (err);
-    
+
     lua_State *ls = state();
     lua_call_throttle strangler(this);
     err = lua_pcall(ls, 0, nresults, 0);
@@ -254,7 +256,7 @@ int CLua::loadfile(lua_State *ls, const char *filename, bool trusted,
             make_stringf("invalid filename: %s", filename).c_str());
         return (-1);
     }
-    
+
     std::string file = datafile_path(filename, die_on_fail);
     if (file.empty())
     {
@@ -324,7 +326,7 @@ bool CLua::runhook(const char *hook, const char *params, ...)
 void CLua::fnreturns(const char *format, ...)
 {
     lua_State *ls = _state;
-    
+
     if (!format || !ls)
         return;
 
@@ -345,7 +347,7 @@ void CLua::vfnreturns(const char *format, va_list args)
         format = gs + 1;
     else if ((gs = strchr(format, ':')))
         format = gs + 1;
-    
+
     for (const char *run = format; *run; ++run)
     {
         char argtype = *run;
@@ -379,7 +381,6 @@ void CLua::vfnreturns(const char *format, va_list args)
     lua_pop(ls, nrets);
 }
 
-static void push_monster(lua_State *ls, monsters *mons);
 static int push_activity_interrupt(lua_State *ls, activity_interrupt_data *t);
 int CLua::push_args(lua_State *ls, const char *format, va_list args,
                     va_list *targ)
@@ -457,7 +458,7 @@ int CLua::return_count(lua_State *ls, const char *format)
     const char *gs = strchr(format, '>');
     if (gs)
         return (strlen(gs + 1));
-    
+
     const char *cs = strchr(format, ':');
     if (cs && isdigit(*format))
     {
@@ -474,7 +475,7 @@ int CLua::return_count(lua_State *ls, const char *format)
     return (0);
 }
 
-bool CLua::calltopfn(lua_State *ls, const char *params, va_list args, 
+bool CLua::calltopfn(lua_State *ls, const char *params, va_list args,
                      int retc, va_list *copyto)
 {
     // We guarantee to remove the function from the stack
@@ -508,7 +509,7 @@ bool CLua::callbooleanfn(bool def, const char *fn, const char *params, ...)
     bool ret = calltopfn(ls, params, args, 1);
     if (!ret)
         CL_RESETSTACK_RETURN(ls, stacktop, def);
-    
+
     def = lua_toboolean(ls, -1);
     CL_RESETSTACK_RETURN(ls, stacktop, def);
 }
@@ -563,7 +564,7 @@ bool CLua::callfn(const char *fn, int nargs, int nret)
             lua_settop(ls, -nargs - 2);
             return (false);
         }
-        
+
         // Slide the function in front of its args and call it.
         if (nargs)
             lua_insert(ls, -nargs - 1);
@@ -575,7 +576,7 @@ bool CLua::callfn(const char *fn, int nargs, int nret)
     return !err;
 }
 
-// Defined in Kills.cc because the kill bindings refer to Kills.cc local 
+// Defined in Kills.cc because the kill bindings refer to Kills.cc local
 // structs
 extern void luaopen_kills(lua_State *ls);
 
@@ -593,13 +594,13 @@ void CLua::init_lua()
     if (_state)
         return;
 
-    _state = managed_vm? lua_newstate(clua_allocator, this) : luaL_newstate();
+    _state = managed_vm? lua_newstate(_clua_allocator, this) : luaL_newstate();
     if (!_state)
         return;
 
     lua_stack_cleaner clean(_state);
 
-    lua_atpanic(_state, clua_panic);
+    lua_atpanic(_state, _clua_panic);
 
     luaopen_base(_state);
     luaopen_string(_state);
@@ -621,14 +622,14 @@ void CLua::init_lua()
     load_cmacro();
     load_chooks();
 
-    lua_register(_state, "loadfile", clua_loadfile);
-    lua_register(_state, "dofile", clua_dofile);
+    lua_register(_state, "loadfile", _clua_loadfile);
+    lua_register(_state, "dofile", _clua_dofile);
 
     if (managed_vm)
     {
-        lua_register(_state, "pcall", clua_guarded_pcall);
+        lua_register(_state, "pcall", _clua_guarded_pcall);
         execfile("clua/userbase.lua", true, true);
-    }        
+    }
 
     lua_pushboolean(_state, managed_vm);
     setregistry("lua_vm_is_managed");
@@ -658,12 +659,12 @@ void CLua::load_cmacro()
 
 /////////////////////////////////////////////////////////////////////
 
-static void clua_register_metatable(lua_State *ls, const char *tn, 
-                                   const luaL_reg *lr,
-                                   int (*gcfn)(lua_State *ls) = NULL)
+static void _clua_register_metatable(lua_State *ls, const char *tn,
+                                     const luaL_reg *lr,
+                                     int (*gcfn)(lua_State *ls) = NULL)
 {
     int top = lua_gettop(ls);
-    
+
     luaL_newmetatable(ls, tn);
     lua_pushstring(ls, "__index");
     lua_pushvalue(ls, -2);
@@ -675,7 +676,7 @@ static void clua_register_metatable(lua_State *ls, const char *tn,
         lua_pushcfunction(ls, gcfn);
         lua_settable(ls, -3);
     }
-    
+
     luaL_openlib(ls, NULL, lr, 0);
 
     lua_settop(ls, top);
@@ -717,15 +718,29 @@ LUARET1(you_name, string, you.your_name)
 LUARET1(you_race, string,
         species_name(you.species, you.experience_level).c_str())
 LUARET1(you_class, string, get_class_name(you.char_class))
-LUARET1(you_god, string, god_name(you.religion))
+LUARET1(you_god, string, god_name(you.religion).c_str())
+LUARET1(you_good_god, boolean,
+        lua_isstring(ls, 1) ? is_good_god(str_to_god(lua_tostring(ls, 1)))
+        : is_good_god(you.religion))
+LUARET1(you_evil_god, boolean,
+        lua_isstring(ls, 1) ? is_evil_god(str_to_god(lua_tostring(ls, 1)))
+        : is_evil_god(you.religion))
+LUARET1(you_god_likes_butchery, boolean,
+        lua_isstring(ls, 1) ?
+        god_likes_butchery(str_to_god(lua_tostring(ls, 1))) :
+        god_likes_butchery(you.religion))
 LUARET2(you_hp, number, you.hp, you.hp_max)
 LUARET2(you_mp, number, you.magic_points, you.max_magic_points)
+LUARET2(you_pos, number, you.x_pos, you.y_pos)
 LUARET1(you_hunger, string, hunger_level())
 LUARET2(you_strength, number, you.strength, you.max_strength)
 LUARET2(you_intelligence, number, you.intel, you.max_intel)
 LUARET2(you_dexterity, number, you.dex, you.max_dex)
 LUARET1(you_exp, number, you.experience_level)
 LUARET1(you_exp_points, number, you.experience)
+LUARET1(you_skill, number,
+        lua_isstring(ls, 1) ? you.skills[str_to_skill(lua_tostring(ls, 1))]
+                            : 0)
 LUARET1(you_res_poison, number, player_res_poison(false))
 LUARET1(you_res_fire, number, player_res_fire(false))
 LUARET1(you_res_cold, number, player_res_cold(false))
@@ -735,20 +750,26 @@ LUARET1(you_res_statdrain, number, player_sust_abil(false))
 LUARET1(you_res_mutation, number, wearing_amulet(AMU_RESIST_MUTATION, false))
 LUARET1(you_res_slowing, number, wearing_amulet(AMU_RESIST_SLOW, false))
 LUARET1(you_gourmand, boolean, wearing_amulet(AMU_THE_GOURMAND, false))
+LUARET1(you_saprovorous, number, player_mutation_level(MUT_SAPROVOROUS))
 LUARET1(you_levitating, boolean, you.flight_mode() == FL_LEVITATE)
 LUARET1(you_flying, boolean, you.flight_mode() == FL_FLY)
 LUARET1(you_transform, string, transform_name())
 LUARET1(you_where, string, level_id::current().describe().c_str())
 LUARET1(you_branch, string, level_id::current().describe(false, false).c_str())
 LUARET1(you_subdepth, number, level_id::current().depth)
+// Increase by 1 because check happens on old level.
 LUARET1(you_absdepth, number, you.your_level + 1)
 LUAWRAP(you_stop_activity, interrupt_activity(AI_FORCE_INTERRUPT))
+LUARET1(you_taking_stairs, boolean,
+        current_delay_action() == DELAY_ASCENDING_STAIRS
+        || current_delay_action() == DELAY_DESCENDING_STAIRS)
 LUARET1(you_turns, number, you.num_turns)
 LUARET1(you_see_grid, boolean,
         see_grid(luaL_checkint(ls, 1), luaL_checkint(ls, 2)))
-// increase by 1 because check happens on old level
-LUARET1(bzr_floor_colour, string,
-        colour_to_str(bazaar_floor_colour(you.your_level + 2)))
+LUARET1(you_see_grid_no_trans, boolean,
+        see_grid_no_trans(luaL_checkint(ls, 1), luaL_checkint(ls, 2)))
+LUARET1(you_can_smell, boolean, player_can_smell())
+LUARET1(you_has_claws, number, you.has_claws(false))
 
 void lua_push_floor_items(lua_State *ls);
 static int you_floor_items(lua_State *ls)
@@ -786,6 +807,15 @@ static int l_you_abils(lua_State *ls)
     return (1);
 }
 
+static int you_can_consume_corpses(lua_State *ls)
+{
+    lua_pushboolean(ls,
+                    can_ingest(OBJ_FOOD, FOOD_CHUNK, true, false, false)
+                    || can_ingest(OBJ_CORPSES, CORPSE_BODY, true, false, false)
+                    );
+    return (1);
+}
+
 static const struct luaL_reg you_lib[] =
 {
     { "turn_is_over", you_turn_is_over },
@@ -796,15 +826,18 @@ static const struct luaL_reg you_lib[] =
     { "race"        , you_race },
     { "class"       , you_class },
     { "god"         , you_god },
+    { "good_god"    , you_good_god },
+    { "evil_god"    , you_evil_god },
     { "hp"          , you_hp },
     { "mp"          , you_mp },
+    { "pos"         , you_pos },
     { "hunger"      , you_hunger },
     { "strength"    , you_strength },
     { "intelligence", you_intelligence },
     { "dexterity"   , you_dexterity },
+    { "skill"       , you_skill },
     { "xl"          , you_exp },
     { "exp"         , you_exp_points },
-
     { "res_poison"  , you_res_poison },
     { "res_fire"    , you_res_fire   },
     { "res_cold"    , you_res_cold   },
@@ -813,12 +846,17 @@ static const struct luaL_reg you_lib[] =
     { "res_statdrain", you_res_statdrain },
     { "res_mutation", you_res_mutation },
     { "res_slowing",  you_res_slowing },
+    { "saprovorous",  you_saprovorous },
     { "gourmand",     you_gourmand },
     { "levitating",   you_levitating },
     { "flying",       you_flying },
     { "transform",    you_transform },
 
+    { "god_likes_butchery",  you_god_likes_butchery  },
+    { "can_consume_corpses", you_can_consume_corpses },
+
     { "stop_activity", you_stop_activity },
+    { "taking_stairs", you_taking_stairs },
 
     { "floor_items",  you_floor_items },
 
@@ -827,9 +865,11 @@ static const struct luaL_reg you_lib[] =
     { "subdepth",     you_subdepth },
     { "absdepth",     you_absdepth },
 
-    { "see_grid",     you_see_grid },
-    { "bazaar_floor", bzr_floor_colour },
-    
+    { "see_grid",          you_see_grid },
+    { "see_grid_no_trans", you_see_grid_no_trans },
+    { "can_smell",         you_can_smell },
+    { "has_claws",         you_has_claws },
+
     { NULL, NULL },
 };
 
@@ -896,11 +936,13 @@ static int l_item_swap_slots(lua_State *ls)
     int slot1 = luaL_checkint(ls, 1),
         slot2 = luaL_checkint(ls, 2);
     bool verbose = lua_toboolean(ls, 3);
-    if (slot1 < 0 || slot1 >= ENDOFPACK ||
-            slot2 < 0 || slot2 >= ENDOFPACK ||
-            slot1 == slot2 || !is_valid_item(you.inv[slot1]))
+    if (slot1 < 0 || slot1 >= ENDOFPACK
+        || slot2 < 0 || slot2 >= ENDOFPACK
+        || slot1 == slot2 || !is_valid_item(you.inv[slot1]))
+    {
         return (0);
-    
+    }
+
     swap_inv_slots(slot1, slot2, verbose);
 
     return (0);
@@ -968,7 +1010,7 @@ static int l_item_remove(lua_State *ls)
         mpr("Item is not equipped");
         return (0);
     }
-    
+
     bool result = false;
     if (eq == EQ_WEAPON)
         result = wield_weapon(true, -1);
@@ -1098,6 +1140,20 @@ static int l_item_pickup(lua_State *ls)
 }
 
 static int l_item_equipped(lua_State *ls)
+{
+    LUA_ITEM(item, 1);
+    if (!item || !in_inventory(*item))
+        return (0);
+
+    int eq = get_equip_slot(item);
+    if (eq < 0 || eq >= NUM_EQUIP)
+        return (0);
+
+    return (1);
+}
+
+// Returns item equipped in a slot defined in an argument.
+static int l_item_equipped_at(lua_State *ls)
 {
     int eq = -1;
     if (lua_isnumber(ls, 1))
@@ -1164,7 +1220,7 @@ static const char *ring_types[] =
     "protection from magic",
     "fire",
     "ice",
-    "teleport control",
+    "teleport control"
 };
 
 static const char *amulet_types[] =
@@ -1193,6 +1249,8 @@ static int l_item_subtype(lua_State *ls)
             {
                 if (item->sub_type == POT_BLOOD)
                    s = "blood";
+                else if (item->sub_type == POT_BLOOD_COAGULATED)
+                   s = "coagulated blood";
                 else if (item->sub_type == POT_WATER)
                    s = "water";
                 else if (item->sub_type == POT_PORRIDGE)
@@ -1202,11 +1260,20 @@ static int l_item_subtype(lua_State *ls)
                 else if (item->sub_type == POT_GAIN_STRENGTH
                          || item->sub_type == POT_GAIN_DEXTERITY
                          || item->sub_type == POT_GAIN_INTELLIGENCE)
+                {
                    s = "gain ability";
+                }
                 else if (item->sub_type == POT_CURE_MUTATION)
                    s = "cure mutation";
             }
-            
+            else if (item->base_type == OBJ_BOOKS)
+            {
+                if (item->sub_type == BOOK_MANUAL)
+                    s = "manual";
+                else
+                    s = "spellbook";
+            }
+
             if (s)
                 lua_pushstring(ls, s);
             else
@@ -1222,77 +1289,15 @@ static int l_item_subtype(lua_State *ls)
     return (2);
 }
 
-static int l_item_potion_type(lua_State *ls)
-{
-    LUA_ITEM(item, 1);
-    int val = 99;
-
-    if (item && item->base_type == OBJ_POTIONS)
-    {
-       if (!item_type_known(*item))
-       {
-          val = 0;
-       }
-       else
-       {
-          switch(item->sub_type)
-          {
-            // good potions:
-            case POT_HEALING:
-            case POT_HEAL_WOUNDS:
-            case POT_SPEED:
-            case POT_MIGHT:
-            case POT_LEVITATION:
-            case POT_INVISIBILITY:
-            case POT_EXPERIENCE:
-            case POT_MAGIC:
-            case POT_RESTORE_ABILITIES:
-            case POT_RESISTANCE:
-                 val = 1;
-                 break;
-
-            // bad potions:
-            case POT_POISON:
-            case POT_STRONG_POISON:
-            case POT_SLOWING:
-            case POT_PARALYSIS:
-            case POT_CONFUSION:
-            case POT_DEGENERATION:
-            case POT_DECAY:
-            case POT_MUTATION:
-                 val = 2;
-                 break;
-
-            // need more refined handling:
-            // for eating habits
-            case POT_BLOOD:
-            case POT_WATER:
-            case POT_PORRIDGE:
-            // for undead
-            case POT_BERSERK_RAGE:
-            case POT_GAIN_STRENGTH:
-            case POT_GAIN_DEXTERITY:
-            case POT_GAIN_INTELLIGENCE:
-            case POT_CURE_MUTATION:
-            default:
-                 val = 3;
-          }
-       }
-    }
-    lua_pushnumber(ls, val);
-    return (1);
-}
-
 static int l_item_cursed(lua_State *ls)
 {
     LUA_ITEM(item, 1);
-    bool cursed = item && item_ident(*item, ISFLAG_KNOW_CURSE) 
-                    && item_cursed(*item);
+    bool cursed = item && item_ident(*item, ISFLAG_KNOW_CURSE)
+                       && item_cursed(*item);
     lua_pushboolean(ls, cursed);
     return (1);
 }
 
-
 static int l_item_worn(lua_State *ls)
 {
     LUA_ITEM(item, 1);
@@ -1348,8 +1353,8 @@ static int l_item_slot(lua_State *ls)
     LUA_ITEM(item, 1);
     if (item)
     {
-        int slot = in_inventory(*item)? item->link :
-                        letter_to_index(item->slot);
+        int slot = in_inventory(*item) ? item->link
+                                       : letter_to_index(item->slot);
         lua_pushnumber(ls, slot);
     }
     else
@@ -1409,6 +1414,28 @@ static int l_item_weap_skill(lua_State *ls)
     return (2);
 }
 
+static int l_item_dropped(lua_State *ls)
+{
+    LUA_ITEM(item, 1);
+    if (!item || !is_valid_item(*item))
+        return (0);
+
+    lua_pushboolean(ls, item->flags & ISFLAG_DROPPED);
+
+    return (1);
+}
+
+static int l_item_can_cut_meat(lua_State *ls)
+{
+    LUA_ITEM(item, 1);
+    if (!item || !is_valid_item(*item))
+        return (0);
+
+    lua_pushboolean(ls, can_cut_meat(*item));
+
+    return (1);
+}
+
 static int l_item_artefact(lua_State *ls)
 {
     LUA_ITEM(item, 1);
@@ -1451,7 +1478,6 @@ static const struct luaL_reg item_lib[] =
     { "branded",           l_item_branded },
     { "class",             l_item_class },
     { "subtype",           l_item_subtype },
-    { "potion_type",       l_item_potion_type },
     { "cursed",            l_item_cursed },
     { "worn",              l_item_worn },
     { "name",              l_item_name },
@@ -1469,10 +1495,13 @@ static const struct luaL_reg item_lib[] =
     { "remove",            l_item_remove },
     { "drop",              l_item_drop },
     { "pickup",            l_item_pickup },
-    { "equipped_at",       l_item_equipped },
+    { "equipped_at",       l_item_equipped_at },
+    { "equipped",          l_item_equipped },
     { "equip_type",        l_item_equip_type },
     { "weap_skill",        l_item_weap_skill },
-    
+    { "dropped",           l_item_dropped },
+    { "can_cut_meat",      l_item_can_cut_meat },
+
     { NULL, NULL },
 };
 
@@ -1591,11 +1620,11 @@ static int food_eat(lua_State *ls)
 static int food_rotting(lua_State *ls)
 {
     LUA_ITEM(item, 1);
+
     bool rotting = false;
     if (item && item->base_type == OBJ_FOOD && item->sub_type == FOOD_CHUNK)
-    {
-        rotting = item->special < 100;
-    }
+        rotting = food_is_rotten(*item);
+
     lua_pushboolean(ls, rotting);
     return (1);
 }
@@ -1604,20 +1633,20 @@ static int food_ischunk(lua_State *ls)
 {
     LUA_ITEM(item, 1);
     lua_pushboolean(ls,
-            item && item->base_type == OBJ_FOOD 
+            item && item->base_type == OBJ_FOOD
                  && item->sub_type == FOOD_CHUNK);
     return (1);
 }
 
 static const struct luaL_reg food_lib[] =
 {
-    { "do_eat",         food_do_eat },
-    { "prompt_floor",   food_prompt_floor },
+    { "do_eat",           food_do_eat },
+    { "prompt_floor",     food_prompt_floor },
     { "prompt_inventory", food_prompt_inventory },
-    { "can_eat",        food_can_eat },
-    { "eat",            food_eat },
-    { "rotting",        food_rotting },
-    { "ischunk",        food_ischunk },
+    { "can_eat",          food_can_eat },
+    { "eat",              food_eat },
+    { "rotting",          food_rotting },
+    { "ischunk",          food_ischunk },
     { NULL, NULL },
 };
 
@@ -1634,7 +1663,7 @@ static int crawl_mpr(lua_State *ls)
 {
     if (!crawl_state.io_inited)
         return (0);
-    
+
     const char *message = luaL_checkstring(ls, 1);
     if (!message)
         return (0);
@@ -1663,7 +1692,7 @@ static int crawl_input_line(lua_State *ls)
 {
     // This is arbitrary, but anybody entering so many characters is psychotic.
     char linebuf[500];
-    
+
     get_input_line(linebuf, sizeof linebuf);
     lua_pushstring(ls, linebuf);
     return (1);
@@ -1672,7 +1701,7 @@ static int crawl_input_line(lua_State *ls)
 static int crawl_c_input_line(lua_State *ls)
 {
     char linebuf[500];
-    
+
     bool valid = !cancelable_get_line(linebuf, sizeof linebuf);
     if (valid)
         lua_pushstring(ls, linebuf);
@@ -1748,7 +1777,7 @@ static int crawl_setopt(lua_State *ls)
 {
     if (!lua_isstring(ls, 1))
         return (0);
-    
+
     const char *s = lua_tostring(ls, 1);
     if (s)
     {
@@ -1759,6 +1788,16 @@ static int crawl_setopt(lua_State *ls)
     return (0);
 }
 
+static int crawl_read_options(lua_State *ls)
+{
+    if (!lua_isstring(ls, 1))
+        return (0);
+
+    const char* filename = lua_tostring(ls, 1);
+    Options.include(filename, true, true);
+    return (0);
+}
+
 static int crawl_bindkey(lua_State *ls)
 {
     const char *s = NULL;
@@ -1767,7 +1806,7 @@ static int crawl_bindkey(lua_State *ls)
         s = lua_tostring(ls, 1);
     }
 
-    if (!s || !lua_isfunction(ls, 2) || !lua_gettop(ls) == 2)
+    if (!s || !lua_isfunction(ls, 2) || lua_gettop(ls) != 2)
         return (0);
 
     lua_pushvalue(ls, 2);
@@ -1789,7 +1828,7 @@ static int crawl_msgch_num(lua_State *ls)
     int ch = str_to_channel(s);
     if (ch == -1)
         return (0);
-    
+
     lua_pushnumber(ls, ch);
     return (1);
 }
@@ -1811,8 +1850,8 @@ static int crawl_regex(lua_State *ls)
     if (!s)
         return (0);
 
-    
-    text_pattern **tpudata = 
+
+    text_pattern **tpudata =
             clua_new_userdata< text_pattern* >(ls, REGEX_METATABLE);
     if (tpudata)
     {
@@ -1824,11 +1863,11 @@ static int crawl_regex(lua_State *ls)
 
 static int crawl_regex_find(lua_State *ls)
 {
-    text_pattern **pattern = 
+    text_pattern **pattern =
             clua_get_userdata< text_pattern* >(ls, REGEX_METATABLE);
     if (!pattern)
         return (0);
-    
+
     const char *text = luaL_checkstring(ls, -1);
     if (!text)
         return (0);
@@ -1840,7 +1879,7 @@ static int crawl_regex_find(lua_State *ls)
 static int crawl_regex_gc(lua_State *ls)
 {
     text_pattern **pattern =
-        static_cast<text_pattern **>( lua_touserdata(ls, 1) );        
+        static_cast<text_pattern **>( lua_touserdata(ls, 1) );
     if (pattern)
         delete *pattern;
     return (0);
@@ -1859,7 +1898,7 @@ static int crawl_message_filter(lua_State *ls)
         return (0);
 
     int num = lua_isnumber(ls, 2)? luaL_checkint(ls, 2) : -1;
-    message_filter **mf = 
+    message_filter **mf =
             clua_new_userdata< message_filter* >( ls, MESSF_METATABLE );
     if (mf)
     {
@@ -1871,11 +1910,11 @@ static int crawl_message_filter(lua_State *ls)
 
 static int crawl_messf_matches(lua_State *ls)
 {
-    message_filter **mf = 
+    message_filter **mf =
             clua_get_userdata< message_filter* >(ls, MESSF_METATABLE);
     if (!mf)
         return (0);
-    
+
     const char *pattern = luaL_checkstring(ls, 2);
     int ch = luaL_checkint(ls, 3);
     if (pattern)
@@ -1919,7 +1958,7 @@ static int crawl_split(lua_State *ls)
                *token = luaL_checkstring(ls, 2);
     if (!s || !token)
         return (0);
-        
+
     std::vector<std::string> segs = split_string(token, s);
     lua_newtable(ls);
     for (int i = 0, count = segs.size(); i < count; ++i)
@@ -1940,6 +1979,58 @@ LUARET1(crawl_random_range, number,
                       lua_isnumber(ls, 3)? luaL_checkint(ls, 3) : 1 ))
 LUARET1(crawl_coinflip, boolean, coinflip())
 
+static int crawl_random_element(lua_State *ls)
+{
+    const int table_idx = 1;
+    const int value_idx = 2;
+
+    if (lua_gettop(ls) == 0)
+    {
+        lua_pushnil(ls);
+        return 1;
+    }
+
+    // Only the first arg does anything now.  Maybe this should
+    // select from a variable number of table args?
+    lua_pop(ls, lua_gettop(ls) - 1);
+
+    // Keep max value on the stack, as it could be any type of value.
+    lua_pushnil(ls);
+    int rollsize = 0;
+
+    lua_pushnil(ls);
+    while (lua_next(ls, table_idx) != 0)
+    {
+        const int weight_idx = -1;
+        const int key_idx = -2;
+
+        int this_weight = lua_isnil(ls, weight_idx) ?
+            1 : (int)lua_tonumber(ls, weight_idx);
+
+        if (rollsize > 0)
+        {
+            rollsize += this_weight;
+            if (x_chance_in_y(this_weight, rollsize))
+            {
+                lua_pushvalue(ls, key_idx);
+                lua_replace(ls, value_idx);
+            }
+        }
+        else
+        {
+            lua_pushvalue(ls, key_idx);
+            lua_replace(ls, value_idx);
+            rollsize = this_weight;
+        }
+
+        lua_pop(ls, 1);
+    }
+
+    lua_pushvalue(ls, value_idx);
+
+    return 1;
+}
+
 static int crawl_err_trace(lua_State *ls)
 {
     const int nargs = lua_gettop(ls);
@@ -1982,6 +2073,7 @@ static const struct luaL_reg crawl_lib[] =
     { "random2avg"   ,  crawl_random2avg },
     { "coinflip",       crawl_coinflip },
     { "random_range",   crawl_random_range },
+    { "random_element", crawl_random_element },
     { "redraw_screen",  crawl_redraw_screen },
     { "input_line",     crawl_input_line },
     { "c_input_line",   crawl_c_input_line},
@@ -1993,6 +2085,7 @@ static const struct luaL_reg crawl_lib[] =
     { "runmacro",       crawl_runmacro },
     { "bindkey",        crawl_bindkey },
     { "setopt",         crawl_setopt },
+    { "read_options",   crawl_read_options },
     { "msgch_num",      crawl_msgch_num },
     { "msgch_name",     crawl_msgch_name },
 
@@ -2008,10 +2101,10 @@ static const struct luaL_reg crawl_lib[] =
 
 void luaopen_crawl(lua_State *ls)
 {
-    clua_register_metatable(ls, REGEX_METATABLE, crawl_regex_ops,
-                            crawl_regex_gc);
-    clua_register_metatable(ls, MESSF_METATABLE, crawl_messf_ops,
-                            crawl_messf_gc);
+    _clua_register_metatable(ls, REGEX_METATABLE, crawl_regex_ops,
+                             crawl_regex_gc);
+    _clua_register_metatable(ls, MESSF_METATABLE, crawl_messf_ops,
+                             crawl_messf_gc);
 
     luaL_openlib(ls, "crawl", crawl_lib, 0);
 }
@@ -2042,7 +2135,7 @@ struct option_handler
     ohandler    handler;
 };
 
-static int option_hboolean(lua_State *ls, const char *name, void *data, 
+static int option_hboolean(lua_State *ls, const char *name, void *data,
                            bool get)
 {
     if (get)
@@ -2061,38 +2154,32 @@ static int option_hboolean(lua_State *ls, const char *name, void *data,
 static option_handler handlers[] =
 {
     // Boolean options come first
-    { "easy_open", &Options.easy_open, option_hboolean },
-    { "colour_map", &Options.colour_map, option_hboolean },
-    { "clean_map", &Options.clean_map, option_hboolean },
+    { "easy_open",     &Options.easy_open, option_hboolean },
+    { "colour_map",    &Options.colour_map, option_hboolean },
+    { "clean_map",     &Options.clean_map, option_hboolean },
     { "show_uncursed", &Options.show_uncursed, option_hboolean },
-    { "always_greet", &Options.always_greet, option_hboolean },
-    { "easy_open", &Options.easy_open, option_hboolean },
-    { "easy_armour", &Options.easy_unequip, option_hboolean },
-    { "easy_unequip", &Options.easy_unequip, option_hboolean },
-    { "easy_butcher", &Options.easy_butcher, option_hboolean },
-    { "terse_hand", &Options.terse_hand, option_hboolean },
-    { "increasing_skill_progress", &Options.increasing_skill_progress, option_hboolean },
-    { "confirm_self_target", &Options.confirm_self_target, option_hboolean },
-    { "default_target", &Options.default_target, option_hboolean },
-    { "safe_autopickup", &Options.safe_autopickup, option_hboolean },
+    { "easy_open",     &Options.easy_open, option_hboolean },
+    { "easy_armour",   &Options.easy_unequip, option_hboolean },
+    { "easy_unequip",  &Options.easy_unequip, option_hboolean },
+    { "easy_butcher",  &Options.easy_butcher, option_hboolean },
+    { "always_confirm_butcher", &Options.always_confirm_butcher, option_hboolean },
+    { "default_target",       &Options.default_target, option_hboolean },
     { "autopickup_no_burden", &Options.autopickup_no_burden, option_hboolean },
-    { "note_skill_max", &Options.note_skill_max, option_hboolean },
-    { "use_notes", &Options.use_notes, option_hboolean },
-    { "delay_message_clear", &Options.delay_message_clear, option_hboolean },
-    { "no_dark_brand", &Options.no_dark_brand, option_hboolean },
-    { "auto_list", &Options.auto_list, option_hboolean },
-    { "lowercase_invocations", &Options.lowercase_invocations, 
-                    option_hboolean },
-    { "pickup_thrown", &Options.pickup_thrown, option_hboolean },
-    { "pickup_dropped", &Options.pickup_dropped, option_hboolean },
-    { "show_waypoints", &Options.show_waypoints, option_hboolean },
-    { "item_colour", &Options.item_colour, option_hboolean },
+    { "note_skill_max",       &Options.note_skill_max, option_hboolean },
+    { "delay_message_clear",  &Options.delay_message_clear, option_hboolean },
+    { "no_dark_brand",   &Options.no_dark_brand, option_hboolean },
+    { "auto_list",       &Options.auto_list, option_hboolean },
+    { "pickup_thrown",   &Options.pickup_thrown, option_hboolean },
+    { "pickup_dropped",  &Options.pickup_dropped, option_hboolean },
+    { "show_waypoints",  &Options.show_waypoints, option_hboolean },
+    { "item_colour",     &Options.item_colour, option_hboolean },
     { "target_zero_exp", &Options.target_zero_exp, option_hboolean },
-    { "safe_zero_exp", &Options.safe_zero_exp, option_hboolean },
-    { "target_wrap", &Options.target_wrap, option_hboolean },
-    { "easy_exit_menu", &Options.easy_exit_menu, option_hboolean },
-    { "dos_use_background_intensity", &Options.dos_use_background_intensity, 
-                    option_hboolean },
+    { "target_wrap",     &Options.target_wrap, option_hboolean },
+    { "easy_exit_menu",  &Options.easy_exit_menu, option_hboolean },
+    { "dos_use_background_intensity", &Options.dos_use_background_intensity,
+                                      option_hboolean },
+    { "menu_colour_prefix_class", &Options.menu_colour_prefix_class,
+                                  option_hboolean },
 };
 
 static const option_handler *get_handler(const char *optname)
@@ -2123,7 +2210,7 @@ static int option_get(lua_State *ls)
         lua_pushstring(ls, ov.c_str());
         return (1);
     }
-    
+
     const option_handler *oh = get_handler(opt);
     if (oh)
         return (oh->handler(ls, opt, oh->data, true));
@@ -2237,14 +2324,14 @@ static int monster_set(lua_State *ls)
     return (0);
 }
 
-static int push_activity_interrupt(lua_State *ls, activity_interrupt_data *t) 
+static int push_activity_interrupt(lua_State *ls, activity_interrupt_data *t)
 {
     if (!t->data)
     {
         lua_pushnil(ls);
         return 0;
     }
-    
+
     switch (t->apt)
     {
     case AIP_HP_LOSS:
@@ -2271,7 +2358,7 @@ static int push_activity_interrupt(lua_State *ls, activity_interrupt_data *t)
     return 0;
 }
 
-static void push_monster(lua_State *ls, monsters *mons)
+void push_monster(lua_State *ls, monsters *mons)
 {
     MonsterWrap *mw = clua_new_userdata< MonsterWrap >(ls, MONS_METATABLE);
     mw->turn = you.num_turns;
@@ -2362,7 +2449,7 @@ struct lua_pat_op
 {
     const char *token;
     const char *luatok;
-    
+
     bool pretext;       // Does this follow a pattern?
     bool posttext;      // Is this followed by a pattern?
 };
@@ -2459,14 +2546,14 @@ std::string lua_text_pattern::new_fn_name()
 bool lua_text_pattern::translate() const
 {
     if (translated || !isvalid)
-        return false;
+        return (false);
 
     if (pattern.find("]]") != std::string::npos
         || pattern.find("[[") != std::string::npos)
     {
         return (false);
     }
-    
+
     std::string textp;
     std::string luafn;
     const lua_pat_op *currop = NULL;
@@ -2485,7 +2572,7 @@ bool lua_text_pattern::translate() const
                         textp.erase(0, textp.find_first_not_of(" \r\n\t"));
                     pre_pattern(textp, luafn);
                 }
-                
+
                 currop = &lop;
                 luafn += lop.luatok;
 
@@ -2536,7 +2623,7 @@ lua_call_throttle::lua_clua_map lua_call_throttle::lua_map;
 // cases the Lua interpreter will throw an exception instead of
 // panicking.
 //
-static int clua_panic(lua_State *ls)
+static int _clua_panic(lua_State *ls)
 {
     if (crawl_state.need_save && !crawl_state.saving_game
         && !crawl_state.updating_scores)
@@ -2546,15 +2633,17 @@ static int clua_panic(lua_State *ls)
     return (0);
 }
 
-static void *clua_allocator(void *ud, void *ptr, size_t osize, size_t nsize)
+static void *_clua_allocator(void *ud, void *ptr, size_t osize, size_t nsize)
 {
     CLua *cl = static_cast<CLua *>( ud );
     cl->memory_used += nsize - osize;
 
     if (nsize > osize && cl->memory_used >= CLUA_MAX_MEMORY_USE * 1024
-           && cl->mixed_call_depth)
+        && cl->mixed_call_depth)
+    {
         return (NULL);
-    
+    }
+
     if (!nsize)
     {
         free(ptr);
@@ -2564,7 +2653,7 @@ static void *clua_allocator(void *ud, void *ptr, size_t osize, size_t nsize)
         return (realloc(ptr, nsize));
 }
 
-static void clua_throttle_hook(lua_State *ls, lua_Debug *dbg)
+static void _clua_throttle_hook(lua_State *ls, lua_Debug *dbg)
 {
     CLua *lua = lua_call_throttle::find_clua(ls);
 
@@ -2572,7 +2661,7 @@ static void clua_throttle_hook(lua_State *ls, lua_Debug *dbg)
     // fudge it.
     if (!lua)
         lua = &clua;
-    
+
     if (lua)
     {
         if (!lua->throttle_sleep_ms)
@@ -2621,7 +2710,7 @@ CLua *lua_call_throttle::find_clua(lua_State *ls)
 //
 // If we did not intercept pcall, the script could do the equivalent
 // of this:
-// 
+//
 //    while true do
 //      pcall(function () while true do end end)
 //    end
@@ -2631,7 +2720,7 @@ CLua *lua_call_throttle::find_clua(lua_State *ls)
 // levels of nesting would just increase the chance of the script
 // beating our throttling).
 //
-static int clua_guarded_pcall(lua_State *ls)
+static int _clua_guarded_pcall(lua_State *ls)
 {
     const int nargs = lua_gettop(ls);
     const int err = lua_pcall(ls, nargs - 1, LUA_MULTRET, 0);
@@ -2642,14 +2731,14 @@ static int clua_guarded_pcall(lua_State *ls)
         if (!errs || strstr(errs, BUGGY_SCRIPT_ERROR))
             luaL_error(ls, errs? errs : BUGGY_PCALL_ERROR);
     }
-    
+
     lua_pushboolean(ls, !err);
     lua_insert(ls, 1);
 
     return (lua_gettop(ls));
 }
 
-static int clua_loadfile(lua_State *ls)
+static int _clua_loadfile(lua_State *ls)
 {
     const char *file = luaL_checkstring(ls, 1);
     if (!file)
@@ -2666,7 +2755,7 @@ static int clua_loadfile(lua_State *ls)
     return (1);
 }
 
-static int clua_dofile(lua_State *ls)
+static int _clua_dofile(lua_State *ls)
 {
     const char *file = luaL_checkstring(ls, 1);
     if (!file)
