@@ -1,3 +1,11 @@
+/*
+ *  File:       clua.h
+ *  Created by: dshaligram on Wed Aug 2 12:54:15 2006 UTC
+ *
+ *  Modified for Crawl Reference by $Author: dolorous $ on $Date: 2008-06-10T02:10:45.405621Z $
+ */
+
+
 #ifndef __CLUA_H__
 #define __CLUA_H__
 
@@ -41,7 +49,7 @@ public:
     ~lua_call_throttle();
 
     static CLua *find_clua(lua_State *ls);
-    
+
 private:
     CLua *lua;
 
@@ -131,7 +139,7 @@ private:
 
     bool proc_returns(const char *par) const;
 
-    bool calltopfn(lua_State *ls, const char *format, va_list args, 
+    bool calltopfn(lua_State *ls, const char *format, va_list args,
                    int retc = -1, va_list *fnr = NULL);
 
     int push_args(lua_State *ls, const char *format, va_list args,
@@ -175,6 +183,7 @@ private:
     static std::string new_fn_name();
 };
 
+// Defined in acr.cc
 extern CLua clua;
 
 void lua_set_exclusive_item(const item_def *item = NULL);
@@ -235,6 +244,8 @@ template <class T> T *clua_new_userdata(
     return static_cast<T*>( udata );
 }
 
+void push_monster(lua_State *ls, monsters *mons);
+
 #define MAP_METATABLE "dgn.mtmap"
 #define DEVENT_METATABLE "dgn.devent"
 #define MAPMARK_METATABLE "dgn.mapmark"
