@@ -14,12 +14,10 @@ function PortalDescriptor:new(properties)
 end
 
 function PortalDescriptor:write(marker, th)
-  file.marshall(th, self.desc or '')
   lmark.marshall_table(th, self.props)
 end
 
 function PortalDescriptor:read(marker, th)
-  self.desc  = file.unmarshall_string(th)
   self.props = lmark.unmarshall_table(th)
   setmetatable(self, PortalDescriptor)
   return self
@@ -33,6 +31,6 @@ function PortalDescriptor:property(marker, pname)
   return self.props and self.props[pname] or ''
 end
 
-function portal_desc(desc, props)
-  return PortalDescriptor:new(desc, props)
+function portal_desc(props)
+  return PortalDescriptor:new(props)
 end
