@@ -7,9 +7,10 @@
 ------------------------------------------------------------------------------
 
 local des_files = {
-   "bazaar.des", "entry.des", "elf.des", "float.des", "hells.des", "hive.des",
-   "lab.des", "lair.des", "large.des", "mini.des", "orc.des", "pan.des",
-   "portal.des", "temple.des", "vaults.des", "crypt.des", "zot.des"
+   "altar.des", "bazaar.des", "entry.des", "elf.des", "float.des", "hells.des",
+   "hive.des", "lab.des", "lair.des", "large.des", "layout.des", "mini.des", 
+   "orc.des", "pan.des", "portal.des", "temple.des", "vaults.des", "crypt.des",
+   "zot.des"
 }
 
 for _, file in ipairs(des_files) do
