@@ -4,7 +4,7 @@
  *  Summary:    Functions used to print information about various game objects.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-17T11:11:48.745321Z $
  *
  *  Change History (most recent first):
  *
@@ -19,6 +19,8 @@
 #include "externs.h"
 #include "enum.h"
 
+// If you add any more description types, remember to also
+// change item_description in externs.h
 enum item_description_type
 {
     IDESC_WANDS = 0,
@@ -26,6 +28,7 @@ enum item_description_type
     IDESC_SCROLLS,                      // special field (like the others)
     IDESC_RINGS,
     IDESC_SCROLLS_II,
+    IDESC_STAVES,
     NUM_IDESC
 };
 
@@ -40,37 +43,52 @@ bool is_dumpable_artefact( const item_def &item, bool verbose );
  * called from: chardump - describe
  * *********************************************************************** */
 std::string get_item_description( const item_def &item, bool verbose,
-                                  bool dump = false );
+                                  bool dump = false, bool noquote = false );
 
-// last updated 12may2000 {dlb}
+// last updated 12 Jun 2008 {jpeg}
 /* ***********************************************************************
  * called from: acr - religion
  * *********************************************************************** */
 void describe_god( god_type which_god, bool give_title );
 
+// last updated 12 Jun 2008 {jpeg}
+/* ***********************************************************************
+ * called from: directn
+ * *********************************************************************** */
 void describe_feature_wide(int x, int y);
 
 /* ***********************************************************************
  * called from: item_use - shopping
  * *********************************************************************** */
 void describe_item( item_def &item, bool allow_inscribe = false );
+void inscribe_item( item_def &item, bool proper_prompt );
 
+// last updated 12 Jun 2008 {jpeg}
 /* ***********************************************************************
- * called from: direct
+ * called from: command - direct
  * *********************************************************************** */
 void describe_monsters(monsters &mons);
 
-// last updated 12may2000 {dlb}
+// last updated 12 Jun 2008 {jpeg}
 /* ***********************************************************************
- * called from: item_use
+ * called from: item_use - spl-cast
  * *********************************************************************** */
 void describe_spell(spell_type spelled);
 
 // last updated 13oct2003 {darshan}
 /* ***********************************************************************
- * called from: describe_monsters - describe, kill_ghost - Kills
+ * called from: Kills - monstuff
  * *********************************************************************** */
-std::string ghost_description(const monsters &mons, bool concise = false);
+std::string get_ghost_description(const monsters &mons, bool concise = false);
+
+std::string get_skill_description(int skill, bool need_title = false);
+
+void describe_skill(int skill);
+
+void print_description(const std::string &d, const std::string title = "",
+                       const std::string suffix = "",
+                       const std::string prefix = "",
+                       const std::string footer = "");
 
 const char *trap_name(trap_type trap);
 int str_to_trap(const std::string &s);
