@@ -1,9 +1,9 @@
 /*
- *  File:       dungeon.cc
+ *  File:       dungeon.h
  *  Summary:    Functions used when building new levels.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-06-12T09:03:37.281687Z $
  *
  *  Change History (most recent first):
  *
@@ -21,6 +21,7 @@
 #include "travel.h"
 #include "stuff.h"
 #include <vector>
+#include <algorithm>
 
 enum portal_type
 {
@@ -54,7 +55,8 @@ const int MAKE_GOOD_ITEM = 351;
 typedef char map_type[MAP_SIDE + 1][MAP_SIDE + 1];
 typedef FixedArray<unsigned short, GXM, GYM> map_mask;
 
-extern map_mask dgn_map_mask;
+extern map_mask dgn_Map_Mask;
+extern std::string dgn_Layout_Type;
 
 // Map mask constants.
 
@@ -72,8 +74,9 @@ enum map_mask_type
 class dgn_region;
 typedef std::vector<dgn_region> dgn_region_list;
 
-struct dgn_region
+class dgn_region
 {
+ public:
     // pos is top-left corner.
     coord_def pos, size;
 
@@ -124,13 +127,13 @@ struct dgn_region
         return (xp >= pos.x && xp < pos.x + size.x
                 && yp >= pos.y && yp < pos.y + size.y);
     }
-    
+
     bool fully_contains(const coord_def &p) const
     {
         return (p.x > pos.x && p.x < pos.x + size.x - 1
                 && p.y > pos.y && p.y < pos.y + size.y - 1);
     }
-    
+
     bool overlaps(const dgn_region &other) const;
     bool overlaps_any(const dgn_region_list &others) const;
     bool overlaps(const dgn_region_list &others,
@@ -153,7 +156,7 @@ public:
     bool has_exit_from(const coord_def &start);
 
     bool did_leave_vault() const { return left_vault; }
-    
+
 protected:
     bool path_flood(const coord_def &c, const coord_def &dc);
 protected:
@@ -272,7 +275,7 @@ bool flood_find<fgrd, bound_check>::path_flood(
         }
         return (false);
     }
-    
+
     if (needed_features[ grid ])
     {
         unexplored_place = dc;
@@ -291,7 +294,7 @@ bool flood_find<fgrd, bound_check>::path_flood(
         left_vault = true;
 
     good_square(dc);
-    
+
     return (false);
 }
 //////////////////////////////////////////////////////////////////////////
@@ -299,11 +302,9 @@ bool flood_find<fgrd, bound_check>::path_flood(
 
 bool builder(int level_number, int level_type);
 
-int bazaar_floor_colour(int curr_level);
 // Set floor/wall colour based on the mons_alloc array. Used for
 // Abyss and Pan.
 void dgn_set_colours_from_monsters();
-void dgn_set_floor_colours();
 void dgn_set_grid_colour_at(const coord_def &c, int colour);
 
 bool dgn_place_map(int map, bool generating_level, bool clobber,
@@ -311,18 +312,62 @@ bool dgn_place_map(int map, bool generating_level, bool clobber,
                    const coord_def &pos = coord_def(-1, -1));
 void level_clear_vault_memory();
 void level_welcome_messages();
-void define_zombie(int mid, int ztype, int cs, int power);
-bool is_wall(int feature);
+
 bool place_specific_trap(int spec_x, int spec_y,  trap_type spec_type);
 void place_spec_shop(int level_number, int shop_x, int shop_y,
                      int force_s_type, bool representative = false);
+void replace_area_wrapper(dungeon_feature_type old_feat,
+                          dungeon_feature_type new_feat);
 bool unforbidden(const coord_def &c, unsigned mask);
 coord_def dgn_find_nearby_stair(dungeon_feature_type stair_to_find,
-                                bool find_closest);
+                                coord_def base_pos, bool find_closest);
 
 class mons_spec;
-bool dgn_place_monster(const mons_spec &mspec,
+bool dgn_place_monster(mons_spec &mspec,
                        int monster_level, int vx, int vy,
-                       bool generate_awake);
+                       bool force_pos = false, bool generate_awake = false,
+                       bool patrolling = false);
+
+bool set_level_flags(unsigned long flags, bool silent = false);
+bool unset_level_flags(unsigned long flags, bool silent = false);
+
+void dgn_set_lt_callback(std::string level_type_name,
+                         std::string callback_name);
+
+// Returns true if the given square is okay for use by any character,
+// but always false for squares in non-transparent vaults. This
+// function returns sane results only immediately after dungeon generation
+// (specifically, saving and restoring a game discards information on the
+// vaults used in the current level).
+bool dgn_square_is_passable(const coord_def &c);
+
+struct spec_room
+{
+    bool created;
+    bool hooked_up;
+    int x1;
+    int y1;
+    int x2;
+    int y2;
+
+    spec_room() : created(false), hooked_up(false), x1(0), y1(0), x2(0), y2(0)
+    {
+    }
+};
+
+bool join_the_dots(const coord_def &from, const coord_def &to,
+                   unsigned mmask, bool early_exit = false);
+void spotty_level(bool seeded, int iterations, bool boxy);
+void smear_feature(int iterations, bool boxy, dungeon_feature_type feature,
+                   int x1, int y1, int x2, int y2);
+bool octa_room(spec_room &sr, int oblique_max,
+               dungeon_feature_type type_floor);
+
+int count_feature_in_box(int x0, int y0, int x1, int y1,
+                         dungeon_feature_type feat);
+int count_antifeature_in_box(int x0, int y0, int x1, int y1,
+                             dungeon_feature_type feat);
+int count_neighbours(int x, int y, dungeon_feature_type feat);
+
 
 #endif
