@@ -1,9 +1,9 @@
 /*
- *  File:       fight.cc
+ *  File:       fight.h
  *  Summary:    Functions used during combat.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-06-06T14:43:57.222011Z $
  *
  *  Change History (most recent first):
  *
@@ -28,6 +28,18 @@ enum unarmed_attack_type
     UNAT_BITE
 };
 
+enum unchivalric_attack_type
+{
+    UCAT_NO_ATTACK,                    //    0
+    UCAT_DISTRACTED,
+    UCAT_CONFUSED,
+    UCAT_FLEEING,
+    UCAT_INVISIBLE,
+    UCAT_HELD_IN_NET,
+    UCAT_PARALYSED,
+    UCAT_SLEEPING
+};
+
 struct mon_attack_def;
 
 // added Sept 18, 2000 -- bwr
@@ -36,6 +48,9 @@ struct mon_attack_def;
  * *********************************************************************** */
 int effective_stat_bonus( int wepType = -1 );
 
+int resist_adjust_damage(actor *defender, beam_type flavour,
+                         int res, int rawdamage, bool ranged = false);
+
 // added Sept 18, 2000 -- bwr
 /* ***********************************************************************
  * called from: describe.cc
@@ -54,20 +69,25 @@ bool you_attack(int monster_attacked, bool unarmed_attacks);
 /* ***********************************************************************
  * called from: monstuff
  * *********************************************************************** */
-void monster_attack(int monster_attacking);
+bool monster_attack(int monster_attacking, bool allow_unarmed = true);
 
 
 // last updated: 08jun2000 {dlb}
 /* ***********************************************************************
  * called from: monstuff
  * *********************************************************************** */
-bool monsters_fight(int monster_attacking, int monster_attacked);
+bool monsters_fight(int monster_attacking, int monster_attacked,
+                    bool allow_unarmed = true);
 
 int calc_your_to_hit( bool random_factor );
 
 int calc_heavy_armour_penalty( bool random_factor );
 
-struct melee_attack
+unchivalric_attack_type is_unchivalric_attack(const actor *attacker,
+                                              const actor *defender,
+                                              const monsters *def);
+
+class melee_attack
 {
 public:
     // At the moment this only covers players fighting monsters
@@ -83,10 +103,10 @@ public:
     bool      needs_message;
     bool      attacker_visible, defender_visible;
     bool      attacker_invisible, defender_invisible;
-    
+
     bool      unarmed_ok;
     int       attack_number;
-    
+
     int       to_hit;
     int       base_damage;
     int       potential_damage;
@@ -101,7 +121,7 @@ public:
     int       final_attack_delay;
 
     // Attacker's damage output potential:
-    
+
     item_def  *weapon;
     int       damage_brand;  // Can be special even if unarmed (transforms)
     int       wpn_skill, hands;
@@ -123,7 +143,7 @@ public:
     // Armour penalties?
     int       heavy_armour_penalty;
     bool      can_do_unarmed;
-    
+
     // Attacker uses watery terrain to advantage vs defender. Implies that
     // both attacker and defender are in water.
     bool      water_attack;
@@ -163,8 +183,9 @@ private:
 
     bool attack_shield_blocked(bool verbose);
     bool apply_damage_brand();
-    void calc_elemental_brand_damage(int res, const char *verb);
-    int resist_adjust_damage(int res, int rawdamage);
+    void calc_elemental_brand_damage(beam_type flavour,
+                                     int res,
+                                     const char *verb);
     int fire_res_apply_cerebov_downgrade(int res);
     void drain_defender();
     void drain_player();
@@ -223,7 +244,7 @@ private:
     int  player_stab_weapon_bonus(int damage);
     int  player_stab(int damage);
     int  player_weapon_type_modify(int damage);
-    
+
     bool player_hits_monster();
     int  player_calc_base_weapon_damage();
     int  player_calc_base_unarmed_damage();
@@ -242,6 +263,7 @@ private:
     void player_announce_hit();
     std::string player_why_missed();
     void player_warn_miss();
+    void player_check_weapon_effects();
 };
 
 #endif
