@@ -1,9 +1,9 @@
 /*
- *  File:       files.cc
+ *  File:       files.h
  *  Summary:    Functions used to save and load levels/games.
  *  Written by: Linley Henzell and Alexey Guzeev
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-09-30T18:49:52.760797Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-17T07:19:04.587696Z $
  *
  *  Change History (most recent first):
  *
@@ -22,9 +22,10 @@
 
 enum load_mode_type
 {
-    LOAD_START_GAME,
-    LOAD_RESTART_GAME,
-    LOAD_ENTER_LEVEL
+    LOAD_START_GAME,            // game has just begun
+    LOAD_RESTART_GAME,          // loaded savefile
+    LOAD_ENTER_LEVEL,           // entered a level for the first time
+    LOAD_VISITOR                // Visitor pattern to see all levels
 };
 
 // referenced in files - newgame - ouch - overmap:
@@ -33,11 +34,24 @@ enum load_mode_type
 // referenced in files - newgame - ouch:
 extern FixedArray<bool, MAX_LEVELS, NUM_BRANCHES> tmp_file_pairs;
 
+bool file_exists(const std::string &name);
+bool dir_exists(const std::string &dir);
+bool is_absolute_path(const std::string &path);
+bool is_read_safe_path(const std::string &path);
+void assert_read_safe_path(const std::string &path) throw (std::string);
+
 std::string datafile_path(std::string basename,
                           bool croak_on_fail = true,
                           bool test_base_path = false);
+
+bool get_dos_compatible_file_name(std::string *fname);
 std::string get_parent_directory(const std::string &filename);
 std::string get_base_filename(const std::string &filename);
+std::string get_path_relative_to(const std::string &referencefile,
+                                 const std::string &relativepath);
+std::string catpath(const std::string &first, const std::string &second);
+std::string canonicalise_file_separator(const std::string &path);
+
 bool check_dir(const std::string &what, std::string &dir, bool silent = false);
 
 bool travel_load_map( branch_type branch, int absdepth );
@@ -61,7 +75,7 @@ void check_newer(const std::string &target,
                  void (*action)());
 
 
-bool load( dungeon_feature_type stair_taken, int load_mode,
+bool load( dungeon_feature_type stair_taken, load_mode_type load_mode,
            level_area_type old_level_type, char old_level,
            branch_type where_were_you2 );
 
@@ -80,6 +94,15 @@ void save_game_state();
  * *********************************************************************** */
 void restore_game(void);
 
+bool apply_to_all_dungeons(bool (*applicator)());
+
+class level_id;
+
+bool apply_to_level(const level_id &level, bool preserve_current,
+                    bool (*applicator)());
+
+bool is_existing_level(const level_id &level);
+
 
 // last updated 12may2000 {dlb}
 /* ***********************************************************************
@@ -94,20 +117,6 @@ void save_ghost( bool force = false );
 std::string make_filename( const char *prefix, int level, branch_type branch,
                            level_area_type lt, bool isGhost );
 
-// Default cap on strings marshalled.
-#define STR_CAP 1000
-
-void writeShort(FILE *file, short s);
-short readShort(FILE *file);
-void writeByte(FILE *file, unsigned char byte);
-unsigned char readByte(FILE *file);
-void writeString(FILE* file, const std::string &s, int cap = STR_CAP);
-std::string readString(FILE *file, int cap = STR_CAP);
-void writeLong(FILE* file, long num);
-long readLong(FILE *file);
-void writeCoord(FILE *file, const coord_def &pos);
-void readCoord(FILE *file, coord_def &pos);
-
 FILE *lk_open(const char *mode, const std::string &file);
 void lk_close(FILE *handle, const char *mode, const std::string &file);
 
