@@ -3,7 +3,7 @@
  *  Summary:    Functions for inventory related commands.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-15T22:46:43.579671Z $
  *
  *  Change History (most recent first):
  *
@@ -30,6 +30,7 @@
 #include "externs.h"
 
 #include "clua.h"
+#include "decks.h"
 #include "describe.h"
 #include "food.h"
 #include "initfile.h"
@@ -42,16 +43,20 @@
 #include "stuff.h"
 #include "view.h"
 #include "menu.h"
+#include "mon-util.h"
 #include "randart.h"
+#include "state.h"
+
+#include "tiles.h"
 
 ///////////////////////////////////////////////////////////////////////////////
 // Inventory menu shenanigans
 
-static void get_inv_items_to_show(
-        std::vector<const item_def*> &v, int selector);
+static void _get_inv_items_to_show( std::vector<const item_def*> &v,
+                                    int selector, int excluded_slot = -1);
 
 InvTitle::InvTitle( Menu *mn, const std::string &title,
-          invtitle_annotator tfn ) 
+          invtitle_annotator tfn )
     : MenuEntry( title, MEL_TITLE )
 {
     m       = mn;
@@ -156,13 +161,9 @@ const int InvEntry::item_freshness() const
         return 0;
 
     int freshness = item->special;
-    
-    if (freshness >= 100 || you.species == SP_TROLL || you.species == SP_KOBOLD 
-                         || you.species == SP_GHOUL || you.species == SP_OGRE
-                         || you.species == SP_HILL_ORC)
-    {
+
+    if (freshness >= 100 || player_mutation_level(MUT_SAPROVOROUS))
         freshness -= 300;
-    }
 
     // Ensure that chunk freshness is never zero, since zero means
     // that the item isn't a chunk.
@@ -177,9 +178,9 @@ std::string InvEntry::get_text() const
     std::ostringstream tstr;
 
     tstr << ' ' << static_cast<char>(hotkeys[0]) << ' ';
-    if ( !selected_qty )
+    if (!selected_qty)
         tstr << '-';
-    else if ( selected_qty < quantity )
+    else if (selected_qty < quantity)
         tstr << '#';
     else
         tstr << '+';
@@ -191,12 +192,12 @@ std::string InvEntry::get_text() const
             tstr << " (" << value << " gold)";
     }
 
-    if ( Options.show_inventory_weights )
+    if (Options.show_inventory_weights)
     {
         const int mass = item_mass(*item) * item->quantity;
         tstr << std::setw(get_number_of_cols() - tstr.str().length() - 1)
              << std::right
-             << make_stringf("(%d.%d aum)", mass / 10, mass % 10);
+             << make_stringf("(%.1f aum)", BURDEN_TO_AUM * mass);
     }
     return tstr.str();
 }
@@ -231,7 +232,7 @@ void InvEntry::add_class_hotkeys(const item_def &i)
         add_hotkey('?');
         break;
     case OBJ_JEWELLERY:
-        add_hotkey(i.sub_type >= AMU_RAGE? '"' : '='); 
+        add_hotkey(i.sub_type >= AMU_RAGE ? '"' : '=');
         break;
     case OBJ_POTIONS:
         add_hotkey('!');
@@ -305,53 +306,129 @@ void InvMenu::set_title(const std::string &s)
     std::string stitle = s;
     if (stitle.empty())
     {
+        std::ostringstream default_title;
         const int cap = carrying_capacity(BS_UNENCUMBERED);
 
-        char title_buf[200];
-        snprintf( title_buf, sizeof title_buf,
-                  "Inventory: %d.%d/%d.%d aum (%d%%, %d/52 slots)",
-                    you.burden / 10, you.burden % 10, 
-                    cap / 10, cap % 10,
-                    (you.burden * 100) / cap, 
-                    inv_count() );
-        stitle = title_buf;
+        default_title << make_stringf(
+            "Inventory: %.0f/%.0f aum (%d%%, %d/52 slots)",
+            BURDEN_TO_AUM * you.burden,
+            BURDEN_TO_AUM * cap,
+            (you.burden * 100) / cap,
+            inv_count() );
+
+        default_title << std::setw(get_number_of_cols() - default_title.str().length() - 1)
+                      << std::right
+                      << "Press item letter to examine.";
+
+        stitle = default_title.str();
     }
-    
+
     set_title(new InvTitle(this, stitle, title_annotate));
 }
 
-void InvMenu::load_inv_items(int item_selector,
+static std::string _no_selectables_message(int item_selector)
+{
+    switch (item_selector)
+    {
+    case OSEL_ANY:
+        return("You aren't carrying anything.");
+    case OSEL_WIELD:
+    case OBJ_WEAPONS:
+        return("You aren't carrying any weapons.");
+    case OBJ_ARMOUR:
+        return("You aren't carrying any armour.");
+    case OSEL_UNIDENT:
+        return("You don't have any unidentified items.");
+    case OSEL_MEMORISE:
+        return("You aren't carrying any spellbooks.");
+    case OSEL_RECHARGE:
+        return("You aren't carrying any rechargable items.");
+    case OSEL_ENCH_ARM:
+        return("You aren't carrying any armour which can be enchanted "
+               "further.");
+    case OBJ_CORPSES:
+    case OSEL_VAMP_EAT:
+        return("You aren't carrying any corpses which you can drain.");
+    case OSEL_DRAW_DECK:
+        return("You aren't carrying any decks from which to draw.");
+    case OBJ_FOOD:
+        return("You aren't carrying any food.");
+    case OBJ_POTIONS:
+        return("You aren't carrying any potions.");
+    case OBJ_SCROLLS:
+    case OBJ_BOOKS:
+        return("You aren't carrying any books or scrolls.");
+    case OBJ_WANDS:
+        return("You aren't carrying any wands.");
+    case OSEL_THROWABLE:
+        return("You aren't carrying any items that might be thrown or fired.");
+    }
+
+    return("You aren't carrying any such object.");
+}
+
+void InvMenu::load_inv_items(int item_selector, int excluded_slot,
                              MenuEntry *(*procfn)(MenuEntry *me))
 {
     std::vector<const item_def *> tobeshown;
-    get_inv_items_to_show(tobeshown, item_selector);
+    _get_inv_items_to_show(tobeshown, item_selector, excluded_slot);
 
     load_items(tobeshown, procfn);
+
     if (!item_count())
-    {
-        std::string s;
-        switch (item_selector)
-        {
-        case OSEL_ANY:
-            s = "You aren't carrying anything.";
-            break;
-        case OSEL_WIELD:
-        case OBJ_WEAPONS:
-            s = "You aren't carrying any weapons.";
-            break;
-        case OSEL_UNIDENT:
-            s = "You don't have any unidentified items.";
-            break;
-        default:
-            s = "You aren't carrying any such object.";
-            break;
-        }
-        set_title(s);
-    }
+        set_title(_no_selectables_message(item_selector));
     else
-    {
         set_title("");
+}
+
+void InvMenu::draw_stock_item(int index, const MenuEntry *me) const
+{
+#ifdef USE_TILE
+    const InvEntry *ie = dynamic_cast<const InvEntry*>(me);
+    if (ie && me->quantity > 0)
+    {
+        int draw_quantity = ie->quantity > 1 ? ie->quantity : -1;
+        bool is_item_on_floor = true;
+        bool is_item_tried = false;
+        char c = 0;
+        int idx = -1;
+        if (ie->item)
+        {
+            is_item_on_floor = ie->item->x != -1;
+            is_item_tried = item_type_tried(*ie->item);
+            c = ie->hotkeys.size() > 0 ? ie->hotkeys[0] : 0;
+            idx = (is_item_on_floor ? ie->item->index() :
+                letter_to_index(c));
+        }
+
+        TileDrawOneItem(REGION_INV2, get_entry_index(ie), c,
+            idx, tileidx_item(*ie->item), draw_quantity,
+            is_item_on_floor, ie->selected(), ie->is_item_equipped(),
+            is_item_tried, ie->is_item_cursed());
     }
+#endif
+
+    Menu::draw_stock_item(index, me);
+}
+
+bool InvMenu::is_selectable(int index) const
+{
+    if (type == MT_DROP)
+    {
+        InvEntry *item = dynamic_cast<InvEntry*>(items[index]);
+        if (item->is_item_cursed() && item->is_item_equipped())
+            return (false);
+
+        std::string text = item->get_text();
+
+        if (text.find("!*") != std::string::npos
+            || text.find("!d") != std::string::npos)
+        {
+            return (false);
+        }
+    }
+
+    return Menu::is_selectable(index);
 }
 
 template <std::string (*proc)(const InvEntry *a)>
@@ -418,7 +495,12 @@ bool sort_item_equipped(const InvEntry *a)
     return !a->is_item_equipped();
 }
 
-static bool compare_invmenu_items(const InvEntry *a, const InvEntry *b,
+bool sort_item_identified(const InvEntry *a)
+{
+    return !item_type_known(*(a->item));
+}
+
+static bool _compare_invmenu_items(const InvEntry *a, const InvEntry *b,
                                   const item_sort_comparators *cmps)
 {
     for (item_sort_comparators::const_iterator i = cmps->begin();
@@ -435,7 +517,7 @@ static bool compare_invmenu_items(const InvEntry *a, const InvEntry *b,
 struct menu_entry_comparator
 {
     const menu_sort_condition *cond;
-    
+
     menu_entry_comparator(const menu_sort_condition *c)
         : cond(c)
     {
@@ -445,7 +527,7 @@ struct menu_entry_comparator
     {
         const InvEntry *ia = dynamic_cast<const InvEntry *>(a);
         const InvEntry *ib = dynamic_cast<const InvEntry *>(b);
-        return compare_invmenu_items(ia, ib, &cond->cmp);
+        return _compare_invmenu_items(ia, ib, &cond->cmp);
     }
 };
 
@@ -466,6 +548,7 @@ void init_item_sort_comparators(item_sort_comparators &list,
           { "ego",       compare_item<bool, sort_item_ego> },
           { "art",       compare_item<bool, sort_item_art> },
           { "equipped",  compare_item<bool, sort_item_equipped> },
+          { "identified",compare_item<bool, sort_item_identified> },
           { "qty",       compare_item<int, sort_item_qty> },
           { "slot",      compare_item<int, sort_item_slot> },
           { "freshness", compare_item<int, sort_item_freshness> }
@@ -478,12 +561,12 @@ void init_item_sort_comparators(item_sort_comparators &list,
         std::string s = cmps[i];
         if (s.empty())
             continue;
-        
+
         const bool negated = s[0] == '>';
         if (s[0] == '<' || s[0] == '>')
             s = s.substr(1);
 
-        for (unsigned ci = 0; ci < ARRAYSIZE(cmp_map); ++ci)
+        for (unsigned ci = 0; ci < ARRAYSZ(cmp_map); ++ci)
             if (cmp_map[ci].cname == s)
             {
                 list.push_back( item_comparator( cmp_map[ci].cmp, negated ) );
@@ -492,15 +575,18 @@ void init_item_sort_comparators(item_sort_comparators &list,
     }
 
     if (list.empty())
+    {
         list.push_back(
             item_comparator(compare_item_str<sort_item_fullname>));
+    }
 }
 
 const menu_sort_condition *InvMenu::find_menu_sort_condition() const
 {
-    for (int i = 0, size = Options.sort_menus.size(); i < size; ++i)
+    for (int i = Options.sort_menus.size() - 1; i >= 0; --i)
         if (Options.sort_menus[i].matches(type))
             return &Options.sort_menus[i];
+
     return (NULL);
 }
 
@@ -509,7 +595,7 @@ void InvMenu::sort_menu(std::vector<InvEntry*> &invitems,
 {
     if (!cond || cond->sort == -1 || (int) invitems.size() < cond->sort)
         return;
-    
+
     std::sort( invitems.begin(), invitems.end(), menu_entry_comparator(cond) );
 }
 
@@ -522,19 +608,20 @@ void InvMenu::load_items(const std::vector<const item_def*> &mitems,
 
     menu_letter ckey;
     std::vector<InvEntry*> items_in_class;
-
     const menu_sort_condition *cond = find_menu_sort_condition();
 
     for (int i = 0; i < NUM_OBJECT_CLASSES; ++i)
     {
-        if (!inv_class[i]) continue;
+        if (!inv_class[i])
+            continue;
 
         add_entry( new MenuEntry( item_class_name(i), MEL_SUBTITLE ) );
         items_in_class.clear();
 
         for (int j = 0, count = mitems.size(); j < count; ++j)
         {
-            if (mitems[j]->base_type != i) continue;
+            if (mitems[j]->base_type != i)
+                continue;
             items_in_class.push_back( new InvEntry(*mitems[j]) );
         }
 
@@ -543,6 +630,8 @@ void InvMenu::load_items(const std::vector<const item_def*> &mitems,
         for (unsigned int j = 0; j < items_in_class.size(); ++j)
         {
             InvEntry *ie = items_in_class[j];
+            if (this->tag == "pickup")
+                ie->tag = "pickup";
             // If there's no hotkey, provide one.
             if (ie->hotkeys[0] == ' ')
                 ie->hotkeys[0] = ckey++;
@@ -578,7 +667,7 @@ std::vector<SelItem> InvMenu::get_selitems() const
         InvEntry *inv = dynamic_cast<InvEntry*>(sel[i]);
         selected_items.push_back(
                 SelItem(
-                    inv->hotkeys[0], 
+                    inv->hotkeys[0],
                     inv->selected_qty,
                     inv->item ) );
     }
@@ -587,13 +676,13 @@ std::vector<SelItem> InvMenu::get_selitems() const
 
 bool InvMenu::process_key( int key )
 {
-    if (items.size() 
-            && type == MT_DROP
-            && (key == CONTROL('D') || key == '@'))
+    if (items.size()
+        && type == MT_DROP
+        && (key == CONTROL('D') || key == '@'))
     {
         int newflag =
-            is_set(MF_MULTISELECT)? 
-                MF_SINGLESELECT | MF_ANYPRINTABLE 
+            is_set(MF_MULTISELECT)?
+                MF_SINGLESELECT | MF_ANYPRINTABLE
               : MF_MULTISELECT;
 
         flags &= ~(MF_SINGLESELECT | MF_MULTISELECT | MF_ANYPRINTABLE);
@@ -602,7 +691,7 @@ bool InvMenu::process_key( int key )
         deselect_all();
         sel.clear();
         draw_select_count(0, true);
-        return true;
+        return (true);
     }
     return Menu::process_key( key );
 }
@@ -610,9 +699,11 @@ bool InvMenu::process_key( int key )
 unsigned char InvMenu::getkey() const
 {
     unsigned char mkey = lastch;
-    if (!isalnum(mkey) && mkey != '$' && mkey != '-' && mkey != '?' 
-            && mkey != '*' && mkey != ESCAPE)
+    if (!isalnum(mkey) && mkey != '$' && mkey != '-' && mkey != '?'
+        && mkey != '*' && mkey != ESCAPE)
+    {
         mkey = ' ';
+    }
     return (mkey);
 }
 
@@ -626,17 +717,20 @@ bool in_inventory( const item_def &i )
 unsigned char get_invent( int invent_type )
 {
     unsigned char select;
-    
-    while (true) {
-       select = invent_select(NULL, MT_INVLIST, invent_type,
-                                         MF_SINGLESELECT);
-       if ( isalpha(select) )
-       {
-           const int invidx = letter_to_index(select);
-           if ( is_valid_item(you.inv[invidx]) )
+
+    while (true)
+    {
+        select = invent_select(NULL, MT_INVLIST, invent_type, -1,
+                               MF_SINGLESELECT);
+
+        if (isalpha(select))
+        {
+            const int invidx = letter_to_index(select);
+            if ( is_valid_item(you.inv[invidx]) )
                describe_item( you.inv[invidx], true );
-       }
-       else break;
+        }
+        else
+            break;
     }
     redraw_screen();
     return select;
@@ -701,16 +795,18 @@ std::vector<SelItem> select_items( const std::vector<const item_def*> &items,
         InvMenu menu;
         menu.set_type(mtype);
         menu.set_title(title);
+        if (mtype == MT_PICKUP)
+            menu.set_tag("pickup");
         menu.load_items(items);
-        menu.set_flags(noselect ? MF_NOSELECT :
-                       MF_MULTISELECT | MF_ALLOW_FILTER);
+        menu.set_flags(noselect ? MF_NOSELECT | MF_SHOW_PAGENUMBERS :
+                       MF_MULTISELECT | MF_ALLOW_FILTER | MF_SHOW_PAGENUMBERS);
         menu.show();
         selected = menu.get_selitems();
     }
     return (selected);
 }
 
-static bool item_class_selected(const item_def &i, int selector)
+static bool _item_class_selected(const item_def &i, int selector)
 {
     const int itype = i.base_type;
     if (selector == OSEL_ANY || selector == itype)
@@ -720,71 +816,123 @@ static bool item_class_selected(const item_def &i, int selector)
     {
     case OSEL_UNIDENT:
         return !fully_identified(i);
+
     case OBJ_MISSILES:
         return (itype == OBJ_MISSILES || itype == OBJ_WEAPONS);
+
+    case OSEL_THROWABLE:
+    {
+        if (i.base_type != OBJ_WEAPONS && i.base_type != OBJ_MISSILES)
+            return (false);
+
+        const launch_retval projected = is_launched(&you, you.weapon(), i);
+
+        if (projected == LRET_FUMBLED)
+            return (false);
+
+        return (true);
+    }
     case OBJ_WEAPONS:
     case OSEL_WIELD:
-        return (itype == OBJ_WEAPONS || itype == OBJ_STAVES 
+        return (itype == OBJ_WEAPONS || itype == OBJ_STAVES
                 || itype == OBJ_MISCELLANY);
+
+    case OSEL_MEMORISE:
+        return (itype == OBJ_BOOKS && i.sub_type != BOOK_MANUAL
+                && (i.sub_type != BOOK_DESTRUCTION || !item_type_known(i)));
+
     case OBJ_SCROLLS:
         return (itype == OBJ_SCROLLS || itype == OBJ_BOOKS);
+
+    case OSEL_RECHARGE:
+        return (item_is_rechargable(i, true));
+
+    case OSEL_ENCH_ARM:
+        return (is_enchantable_armour(i, true));
+
+    case OSEL_VAMP_EAT:
+        return (itype == OBJ_CORPSES && i.sub_type == CORPSE_BODY
+                && !food_is_rotten(i) && mons_has_blood(i.plus));
+
+    case OSEL_DRAW_DECK:
+        return (is_deck(i));
+
     case OSEL_EQUIP:
         for (int eq = 0; eq < NUM_EQUIP; eq++)
         {
              if (you.equip[eq] == i.link)
                  return (true);
         }
-        // fall through
+        return (false);
+
     default:
         return (false);
     }
 }
 
-static bool userdef_item_selected(const item_def &i, int selector)
+static bool _userdef_item_selected(const item_def &i, int selector)
 {
 #if defined(CLUA_BINDINGS)
-    const char *luafn = selector == OSEL_WIELD? "ch_item_wieldable" :
-                                                NULL;
+    const char *luafn = selector == OSEL_WIELD ? "ch_item_wieldable"
+                                               : NULL;
     return (luafn && clua.callbooleanfn(false, luafn, "u", &i));
 #else
     return (false);
 #endif
 }
 
-static bool is_item_selected(const item_def &i, int selector)
+static bool _is_item_selected(const item_def &i, int selector)
 {
-    return item_class_selected(i, selector)
-        || userdef_item_selected(i, selector);
+    return (_item_class_selected(i, selector)
+            || _userdef_item_selected(i, selector));
 }
 
-static void get_inv_items_to_show(std::vector<const item_def*> &v, int selector)
+static void _get_inv_items_to_show(std::vector<const item_def*> &v,
+                                   int selector, int excluded_slot)
 {
     for (int i = 0; i < ENDOFPACK; i++)
     {
-        if (is_valid_item(you.inv[i]) && is_item_selected(you.inv[i], selector))
+        if (is_valid_item(you.inv[i])
+            && you.inv[i].slot != excluded_slot
+            && _is_item_selected(you.inv[i], selector))
+        {
             v.push_back( &you.inv[i] );
+        }
     }
 }
 
-unsigned char invent_select( 
-                      const char *title,
-                      menu_type type,
-                      int item_selector,
-                      int flags,
-                      invtitle_annotator titlefn,
-                      std::vector<SelItem> *items,
-                      std::vector<text_pattern> *filter,
-                      Menu::selitem_tfn selitemfn,
-                      const std::vector<SelItem> *pre_select )
+static bool _any_items_to_select(int selector)
+{
+    for (int i = 0; i < ENDOFPACK; i++)
+    {
+        if (is_valid_item(you.inv[i])
+            && _is_item_selected(you.inv[i], selector))
+        {
+            return (true);
+        }
+    }
+    return (false);
+}
+
+unsigned char invent_select( const char *title,
+                             menu_type type,
+                             int item_selector,
+                             int excluded_slot,
+                             int flags,
+                             invtitle_annotator titlefn,
+                             std::vector<SelItem> *items,
+                             std::vector<text_pattern> *filter,
+                             Menu::selitem_tfn selitemfn,
+                             const std::vector<SelItem> *pre_select )
 {
     InvMenu menu(flags);
-    
+
     menu.set_preselect(pre_select);
     menu.set_title_annotator(titlefn);
     menu.f_selitem = selitemfn;
     if (filter)
         menu.set_select_filter( *filter );
-    menu.load_inv_items(item_selector);
+    menu.load_inv_items(item_selector, excluded_slot);
     menu.set_type(type);
 
     // Don't override title if there are no items.
@@ -803,15 +951,15 @@ unsigned char invent( int item_class_inv, bool show_price )
 {
     InvShowPrices show_item_prices(show_price);
     return (invent_select(NULL, MT_INVLIST, item_class_inv));
-}                               // end invent()
+}
 
 // Reads in digits for a count and apprends then to val, the
 // return value is the character that stopped the reading.
-static unsigned char get_invent_quant( unsigned char keyin, int &quant )
+static unsigned char _get_invent_quant( unsigned char keyin, int &quant )
 {
     quant = keyin - '0';
 
-    for(;;)
+    while (true)
     {
         keyin = get_ch();
 
@@ -869,7 +1017,7 @@ std::vector<SelItem> prompt_invent_items(
 
     std::vector<SelItem> items;
     int count = -1;
-    for (;;)
+    while (true)
     {
         if (need_redraw)
         {
@@ -897,18 +1045,19 @@ std::vector<SelItem> prompt_invent_items(
         else if (keyin == '?' || keyin == '*' || keyin == ',')
         {
             int selmode =
-                Options.drop_mode == DM_SINGLE 
+                Options.drop_mode == DM_SINGLE
                     && (!pre_select || pre_select->empty())?
                         MF_SINGLESELECT | MF_EASY_EXIT | MF_ANYPRINTABLE :
                         MF_MULTISELECT | MF_ALLOW_FILTER;
+
             // The "view inventory listing" mode.
-            int ch = invent_select(
-                        prompt,
-                        mtype,
-                        keyin == '*'? OSEL_ANY : type_expect,
-                        selmode,
-                        titlefn, &items, select_filter, fn,
-                        pre_select );
+            int ch = invent_select( prompt,
+                                    mtype,
+                                    keyin == '*' ? OSEL_ANY : type_expect,
+                                    -1,
+                                    selmode,
+                                    titlefn, &items, select_filter, fn,
+                                    pre_select );
 
             if ((selmode & MF_SINGLESELECT) || ch == ESCAPE)
             {
@@ -931,14 +1080,14 @@ std::vector<SelItem> prompt_invent_items(
                 return (items);
             }
 
-            need_redraw = !(keyin == '?' || keyin == '*' 
+            need_redraw = !(keyin == '?' || keyin == '*'
                             || keyin == ',' || keyin == '+');
             need_prompt = need_redraw;
         }
         else if (isdigit( keyin ))
         {
             // The "read in quantity" mode
-            keyin = get_invent_quant( keyin, count );
+            keyin = _get_invent_quant( keyin, count );
 
             need_prompt = false;
             need_getch  = false;
@@ -951,7 +1100,7 @@ std::vector<SelItem> prompt_invent_items(
             ret = PROMPT_ABORT;
             break;
         }
-        else if (isalpha( keyin )) 
+        else if (isalpha( keyin ))
         {
             ret = letter_to_index( keyin );
 
@@ -975,22 +1124,27 @@ std::vector<SelItem> prompt_invent_items(
     if (ret != PROMPT_ABORT)
         items.push_back(
             SelItem( ret, count,
-                     ret != PROMPT_GOT_SPECIAL? &you.inv[ret] : NULL ) );
+                     ret != PROMPT_GOT_SPECIAL ? &you.inv[ret] : NULL ) );
     return items;
 }
 
-static int digit_to_index( char digit, operation_types oper ) {
-    
+static int _digit_to_index( char digit, operation_types oper )
+{
+
     const char iletter = static_cast<char>(oper);
 
-    for ( int i = 0; i < ENDOFPACK; ++i ) {
-        if (is_valid_item(you.inv[i])) {
+    for ( int i = 0; i < ENDOFPACK; ++i )
+    {
+        if (is_valid_item(you.inv[i]))
+        {
             const std::string& r(you.inv[i].inscription);
-            /* note that r.size() is unsigned */
-            for ( unsigned int j = 0; j + 2 < r.size(); ++j ) {
-                if ( r[j] == '@' &&
-                     (r[j+1] == iletter || r[j+1] == '*') &&
-                     r[j+2] == digit ) {
+            // Note that r.size() is unsigned.
+            for ( unsigned int j = 0; j + 2 < r.size(); ++j )
+            {
+                if ( r[j] == '@'
+                     && (r[j+1] == iletter || r[j+1] == '*')
+                     && r[j+2] == digit )
+                {
                     return i;
                 }
             }
@@ -1003,17 +1157,25 @@ bool has_warning_inscription(const item_def& item,
                              operation_types oper)
 {
     const char iletter = static_cast<char>(oper);
-    
+
     const std::string& r(item.inscription);
     for ( unsigned int i = 0; i + 1 < r.size(); ++i )
-        if (r[i] == '!' && (r[i+1] == iletter || r[i+1] == '*'))
-            return true;
-    return false;
+    {
+        if (r[i] == '!')
+        {
+            if (r[i+1] == iletter || r[i+1] == '*')
+                return (true);
+            else if (oper == OPER_ZAP && r[i+1] == 'z') // for the 0.3.4. keys
+                return (true);
+        }
+    }
+
+    return (false);
 }
 
 // checks if current item (to be removed) has a warning inscription
 // and prompts the user for confirmation
-static bool check_old_item_warning( const item_def& item,
+static bool _check_old_item_warning( const item_def& item,
                                     operation_types oper )
 {
     item_def old_item;
@@ -1022,11 +1184,11 @@ static bool check_old_item_warning( const item_def& item,
     {
         if (you.equip[EQ_WEAPON] == -1)
             return (true);
-            
+
         old_item = you.inv[you.equip[EQ_WEAPON]];
         if (!has_warning_inscription(old_item, OPER_WIELD))
             return (true);
-            
+
         prompt += "Really unwield ";
     }
     else if (oper == OPER_WEAR) // can we safely take off old item?
@@ -1042,14 +1204,14 @@ static bool check_old_item_warning( const item_def& item,
 
         if (!has_warning_inscription(old_item, OPER_TAKEOFF))
             return (true);
-            
+
         prompt += "Really take off ";
     }
     else if (oper == OPER_PUTON) // can we safely remove old item?
     {
         if (item.base_type != OBJ_JEWELLERY)
             return (true);
-            
+
         if (jewellery_is_amulet(item))
         {
             if (you.equip[EQ_AMULET] == -1)
@@ -1058,7 +1220,7 @@ static bool check_old_item_warning( const item_def& item,
             old_item = you.inv[you.equip[EQ_AMULET]];
             if (!has_warning_inscription(old_item, OPER_TAKEOFF))
                 return (true);
-                
+
             prompt += "Really remove ";
         }
         else // rings handled in prompt_ring_to_remove
@@ -1073,14 +1235,15 @@ static bool check_old_item_warning( const item_def& item,
     return yesno(prompt.c_str(), false, 'n');
 }
 
-static std::string operation_verb(operation_types oper)
+static std::string _operation_verb(operation_types oper)
 {
     switch (oper)
     {
     case OPER_WIELD:          return "wield";
     case OPER_QUAFF:          return "quaff";
     case OPER_DROP:           return "drop";
-    case OPER_EAT:            return "eat";
+    case OPER_EAT:            return (you.species == SP_VAMPIRE ?
+                                      "drain" : "eat");
     case OPER_TAKEOFF:        return "take off";
     case OPER_WEAR:           return "wear";
     case OPER_PUTON:          return "put on";
@@ -1088,31 +1251,70 @@ static std::string operation_verb(operation_types oper)
     case OPER_READ:           return "read";
     case OPER_MEMORISE:       return "memorise from";
     case OPER_ZAP:            return "zap";
-    case OPER_THROW:          return "throw";
     case OPER_EXAMINE:        return "examine";
     case OPER_FIRE:           return "fire";
     case OPER_PRAY:           return "sacrifice";
     case OPER_EVOKE:          return "evoke";
+    case OPER_DESTROY:        return "destroy";
+    case OPER_QUIVER:         return "quiver";
     case OPER_ANY:
     default:
         return "choose";
     }
 }
 
-/* return true if user OK'd it (or no warning), false otherwise */ 
+// Returns true if user OK'd it (or no warning), false otherwise.
 bool check_warning_inscriptions( const item_def& item,
                                  operation_types oper )
 {
     if (is_valid_item( item ) && has_warning_inscription(item, oper) )
     {
-        std::string prompt = "Really " + operation_verb(oper) + " ";
+        // When it's about destroying an item, don't even ask.
+        // If the player really wants to do that, they'll have
+        // to remove the inscription.
+        if (oper == OPER_DESTROY)
+            return (false);
+
+        if (oper == OPER_WEAR)
+        {
+            if (item.base_type != OBJ_ARMOUR)
+                return (true);
+
+            // Don't ask if item already worn.
+            int equip = you.equip[get_armour_slot(item)];
+            if (equip != -1 && item.link == equip)
+                return (_check_old_item_warning(item, oper));
+        }
+        else if (oper == OPER_PUTON)
+        {
+            if (item.base_type != OBJ_JEWELLERY)
+                return (true);
+
+            // Don't ask if item already worn.
+            int equip = -1;
+            if (jewellery_is_amulet(item))
+                equip = you.equip[EQ_AMULET];
+            else
+            {
+                equip = you.equip[EQ_LEFT_RING];
+                if (equip != -1 && item.link == equip)
+                    return (_check_old_item_warning(item, oper));
+                // or maybe the other ring?
+                equip = you.equip[EQ_RIGHT_RING];
+            }
+
+            if (equip != -1 && item.link == equip)
+                return (_check_old_item_warning(item, oper));
+        }
+
+        std::string prompt = "Really " + _operation_verb(oper) + " ";
         prompt += item.name(DESC_INVENTORY);
-        prompt += "? ";
+        prompt += "?";
         return (yesno(prompt.c_str(), false, 'n')
-                && check_old_item_warning(item, oper));
+                && _check_old_item_warning(item, oper));
     }
     else
-        return (check_old_item_warning(item, oper));
+        return (_check_old_item_warning(item, oper));
 }
 
 // This function prompts the user for an item, handles the '?' and '*'
@@ -1122,16 +1324,26 @@ bool check_warning_inscriptions( const item_def& item,
 //
 // It returns PROMPT_ABORT       if the player hits escape.
 // It returns PROMPT_GOT_SPECIAL if the player hits the "other_valid_char".
+// It returns PROMPT_NOTHING     if there are no matching items.
 //
 // Note: This function never checks if the item is appropriate.
-int prompt_invent_item( const char *prompt, 
-                        menu_type mtype, int type_expect, 
+int prompt_invent_item( const char *prompt,
+                        menu_type mtype, int type_expect,
                         bool must_exist, bool allow_auto_list,
                         bool allow_easy_quit,
                         const char other_valid_char,
+                        int excluded_slot,
                         int *const count,
                         operation_types oper )
 {
+    if (!_any_items_to_select(type_expect) && type_expect != OSEL_WIELD
+        && mtype == MT_INVLIST)
+    {
+        mprf(MSGCH_PROMPT, "%s",
+             _no_selectables_message(type_expect).c_str());
+        return (PROMPT_NOTHING);
+    }
+
     unsigned char  keyin = 0;
     int            ret = PROMPT_ABORT;
 
@@ -1143,10 +1355,14 @@ int prompt_invent_item( const char *prompt,
     if (auto_list)
     {
         need_prompt = need_getch = false;
-        keyin       = '?';
+
+        if (_any_items_to_select(type_expect))
+            keyin = '?';
+        else
+            keyin = '*';
     }
 
-    for (;;)
+    while (true)
     {
         if (need_redraw)
         {
@@ -1178,10 +1394,11 @@ int prompt_invent_item( const char *prompt,
             keyin = invent_select(
                         prompt,
                         mtype,
-                        keyin == '*'? OSEL_ANY : type_expect, 
+                        keyin == '*' ? OSEL_ANY : type_expect,
+                        excluded_slot,
                         MF_SINGLESELECT | MF_ANYPRINTABLE | MF_NO_SELECT_QTY
-                            | MF_EASY_EXIT, 
-                        NULL, 
+                            | MF_EASY_EXIT,
+                        NULL,
                         &items );
 
 
@@ -1195,6 +1412,7 @@ int prompt_invent_item( const char *prompt,
             {
                 if (count)
                     *count = items[0].quantity;
+
                 redraw_screen();
                 mesclr( true );
             }
@@ -1202,7 +1420,7 @@ int prompt_invent_item( const char *prompt,
         else if (count != NULL && isdigit( keyin ))
         {
             // The "read in quantity" mode
-            keyin = get_invent_quant( keyin, *count );
+            keyin = _get_invent_quant( keyin, *count );
 
             need_prompt = false;
             need_getch  = false;
@@ -1210,8 +1428,9 @@ int prompt_invent_item( const char *prompt,
         else if ( count == NULL && isdigit( keyin ) )
         {
             /* scan for our item */
-            int res = digit_to_index( keyin, oper );
-            if ( res != -1 ) {
+            int res = _digit_to_index( keyin, oper );
+            if ( res != -1 )
+            {
                 ret = res;
                 if ( check_warning_inscriptions( you.inv[ret], oper ) )
                     break;
@@ -1225,17 +1444,14 @@ int prompt_invent_item( const char *prompt,
             ret = PROMPT_ABORT;
             break;
         }
-        else if (isalpha( keyin )) 
+        else if (isalpha( keyin ))
         {
             ret = letter_to_index( keyin );
 
             if (must_exist && !is_valid_item( you.inv[ret] ))
                 mpr( "You do not have any such object." );
-            else {
-                if ( check_warning_inscriptions( you.inv[ret], oper ) ) {
-                    break;
-                }
-            }
+            else if ( check_warning_inscriptions( you.inv[ret], oper ) )
+                break;
         }
         else if (!isspace( keyin ))
         {
@@ -1251,3 +1467,21 @@ int prompt_invent_item( const char *prompt,
 
     return (ret);
 }
+
+bool prompt_failed(int retval, std::string msg)
+{
+    if (retval != PROMPT_ABORT && retval != PROMPT_NOTHING)
+        return (false);
+
+    if (msg.empty())
+    {
+        if (retval == PROMPT_ABORT)
+            canned_msg(MSG_OK);
+    }
+    else
+        mprf(MSGCH_PROMPT, msg.c_str());
+
+    crawl_state.cancel_cmd_repeat();
+
+    return (true);
+}
