@@ -1,9 +1,9 @@
 /*
- *  File:       invent.cc
+ *  File:       invent.h
  *  Summary:    Functions for inventory related commands.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-06-30T15:44:43.648058Z $
  *
  *  Change History (most recent first):
  *
@@ -22,14 +22,21 @@
 
 enum object_selector
 {
-    OSEL_ANY     = -1,
-    OSEL_WIELD   = -2,
-    OSEL_UNIDENT = -3,
-    OSEL_EQUIP   = -4
+    OSEL_ANY       =  -1,
+    OSEL_WIELD     =  -2,
+    OSEL_UNIDENT   =  -3,
+    OSEL_EQUIP     =  -4,
+    OSEL_MEMORISE  =  -5,
+    OSEL_RECHARGE  =  -6,
+    OSEL_ENCH_ARM  =  -7,
+    OSEL_VAMP_EAT  =  -8,
+    OSEL_DRAW_DECK =  -9,
+    OSEL_THROWABLE = -10
 };
 
 #define PROMPT_ABORT        -1
 #define PROMPT_GOT_SPECIAL  -2
+#define PROMPT_NOTHING      -3
 
 struct SelItem
 {
@@ -39,7 +46,7 @@ struct SelItem
 
     SelItem() : slot(0), quantity(0), item(NULL) { }
     SelItem( int s, int q, const item_def *it = NULL )
-        : slot(s), quantity(q), item(it) 
+        : slot(s), quantity(q), item(it)
     {
     }
 };
@@ -51,7 +58,7 @@ struct InvTitle : public MenuEntry
 {
     Menu *m;
     invtitle_annotator titlefn;
-    
+
     InvTitle( Menu *mn, const std::string &title,
               invtitle_annotator tfn );
 
@@ -67,7 +74,7 @@ private:
 
     mutable std::string basename;
     mutable std::string qualname;
-    
+
     friend class InvShowPrices;
 
 public:
@@ -89,7 +96,8 @@ public:
     virtual int highlight_colour() const
     {
         return menu_colour(get_text(),
-                           menu_colour_item_prefix( *( (item_def*) item) ) );
+                           menu_colour_item_prefix( *( (item_def*) item) ),
+                           tag );
     }
 
 
@@ -107,8 +115,8 @@ public:
 class InvMenu : public Menu
 {
 public:
-    InvMenu(int mflags = MF_MULTISELECT) 
-        : Menu(mflags), type(MT_INVLIST), pre_select(NULL),
+    InvMenu(int mflags = MF_MULTISELECT)
+        : Menu(mflags, "inventory"), type(MT_INVLIST), pre_select(NULL),
           title_annotate(NULL)
     {
     }
@@ -135,7 +143,7 @@ public:
     // Loads items from the player's inventory into the menu, and sets the
     // title to the stock title. If "procfn" is provided, it'll be called for
     // each MenuEntry added, *excluding the title*.
-    void load_inv_items(int item_selector = OSEL_ANY,
+    void load_inv_items(int item_selector = OSEL_ANY, int excluded_slot = -1,
                         MenuEntry *(*procfn)(MenuEntry *me) = NULL);
 
     std::vector<SelItem> get_selitems() const;
@@ -151,6 +159,8 @@ protected:
     void sort_menu(std::vector<InvEntry*> &items,
                    const menu_sort_condition *cond);
     const menu_sort_condition *find_menu_sort_condition() const;
+    virtual void draw_stock_item(int index, const MenuEntry *me) const;
+    virtual bool is_selectable(int index) const;
 
 protected:
     menu_type type;
@@ -163,15 +173,16 @@ protected:
 int prompt_invent_item( const char *prompt,
                         menu_type type,
                         int type_expect,
-                        bool must_exist = true, 
-                        bool allow_auto_list = true, 
+                        bool must_exist = true,
+                        bool allow_auto_list = true,
                         bool allow_easy_quit = true,
                         const char other_valid_char = '\0',
+                        int excluded_slot = -1,
                         int *const count = NULL,
                         operation_types oper = OPER_ANY );
 
 std::vector<SelItem> select_items(
-                        const std::vector<const item_def*> &items, 
+                        const std::vector<const item_def*> &items,
                         const char *title, bool noselect = false,
                         menu_type mtype = MT_PICKUP );
 
@@ -200,6 +211,7 @@ unsigned char invent_select(
                    // MT_DROP allows the multidrop toggle
                    menu_type type = MT_INVLIST,
                    int item_selector = OSEL_ANY,
+                   int excluded_slot = -1,
                    int menu_select_flags = MF_NOSELECT,
                    invtitle_annotator titlefn = NULL,
                    std::vector<SelItem> *sels = NULL,
@@ -223,4 +235,6 @@ bool has_warning_inscription(const item_def& item, operation_types oper);
 void init_item_sort_comparators(item_sort_comparators &list,
                                 const std::string &set);
 
+bool prompt_failed(int retval, std::string msg = "");
+
 #endif
