@@ -3,7 +3,7 @@
  *  Summary:    Functions for using wands, potions, and weapon/armour removal.4\3
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-06T17:12:09.347938Z $
  *
  *  Change History (most recent first):
  *
@@ -27,11 +27,13 @@
 #include "beam.h"
 #include "effects.h"
 #include "food.h"
+#include "item_use.h"
 #include "itemname.h"
 #include "itemprop.h"
 #include "misc.h"
 #include "mutation.h"
 #include "player.h"
+#include "quiver.h"
 #include "randart.h"
 #include "religion.h"
 #include "skills2.h"
@@ -39,19 +41,20 @@
 #include "spl-cast.h"
 #include "spl-util.h"
 #include "stuff.h"
+#include "tutorial.h"
 #include "view.h"
+#include "xom.h"
 
 // From an actual potion, pow == 40 -- bwr
-bool potion_effect( potion_type pot_eff, int pow )
+bool potion_effect( potion_type pot_eff, int pow, bool was_known )
 {
     bool effect = true;  // current behaviour is all potions id on quaffing
 
-    int new_value = 0;
-
     if (pow > 150)
         pow = 150;
 
-    const int factor = (you.species == SP_VAMPIRE ? 2 : 1);
+    const int factor =
+        (you.species == SP_VAMPIRE && you.hunger_state < HS_SATIATED ? 2 : 1);
 
     switch (pot_eff)
     {
@@ -60,41 +63,80 @@ bool potion_effect( potion_type pot_eff, int pow )
         inc_hp( (5 + random2(7)) / factor, false);
         mpr("You feel better.");
 
-        if (you.species == SP_VAMPIRE)
+        // only fix rot when healed to full
+        if (you.hp == you.hp_max)
         {
-           if (!one_chance_in(3))
-               you.rotting = 0;
-           if (!one_chance_in(3))
-               you.duration[DUR_CONF] = 0;
-        }
-        else
-        {
-            // only fix rot when healed to full 
-            if (you.hp == you.hp_max)
-            {
-                unrot_hp(1);
-                set_hp(you.hp_max, false);
-            }
-            
-            you.duration[DUR_POISONING] = 0;
-            you.rotting = 0;
-            you.disease = 0;
-            you.duration[DUR_CONF] = 0;
+            unrot_hp(1);
+            set_hp(you.hp_max, false);
         }
+
+        you.duration[DUR_POISONING] = 0;
+        you.rotting = 0;
+        you.disease = 0;
+        you.duration[DUR_CONF] = 0;
         break;
 
     case POT_HEAL_WOUNDS:
         inc_hp((10 + random2avg(28, 3)) / factor, false);
         mpr("You feel much better.");
 
-        // only fix rot when healed to full 
-        if ( you.species != SP_VAMPIRE && you.hp == you.hp_max )
+        // only fix rot when healed to full
+        if (you.hp == you.hp_max)
         {
-            unrot_hp( 2 + random2avg(5, 2) );
+            unrot_hp( (2 + random2avg(5, 2)) / factor );
             set_hp(you.hp_max, false);
         }
         break;
 
+  case POT_BLOOD:
+  case POT_BLOOD_COAGULATED:
+        if (you.species == SP_VAMPIRE)
+        {
+            if (pot_eff == POT_BLOOD)
+                mpr("Yummy - fresh blood!");
+            else // coagulated
+                mpr("This tastes delicious!");
+
+            lessen_hunger(1000, true);
+
+            // healing depends on thirst
+            if (you.hunger_state < HS_SATIATED) // !heal wounds
+            {
+                inc_hp((10 + random2avg(30,2)) / factor, false);
+                mpr("You feel much better.");
+            }
+            else
+            {
+                if (you.hunger_state <= HS_FULL)
+                    inc_hp(5 + random2(10), false); // !healing
+                else
+                    inc_hp(2 + random2(5), false);
+                mpr("You feel better.");
+            }
+        }
+        else
+        {
+            if (you.omnivorous() || player_mutation_level(MUT_CARNIVOROUS))
+            {
+                // Likes it
+                mpr("This tastes like blood.");
+                lessen_hunger(200, true);
+            }
+            else
+            {
+                mpr("Blech - this tastes like blood!");
+                if (!player_mutation_level(MUT_HERBIVOROUS) && one_chance_in(3))
+                    lessen_hunger(100, true);
+                else
+                {
+                    disease_player( 50 + random2(100) );
+                    xom_is_stimulated(32);
+                }
+            }
+        }
+        did_god_conduct(DID_DRINK_BLOOD, 1 + random2(3), was_known);
+        break;
+
     case POT_SPEED:
         haste_player( (40 + random2(pow)) / factor );
         break;
@@ -102,41 +144,44 @@ bool potion_effect( potion_type pot_eff, int pow )
     case POT_MIGHT:
     {
         const bool were_mighty = (you.duration[DUR_MIGHT] > 0);
-        
+
         mprf(MSGCH_DURATION, "You feel %s all of a sudden.",
              were_mighty ? "mightier" : "very mighty");
 
-        if ( were_mighty )
-            contaminate_player(1);
+        if (were_mighty)
+            contaminate_player(1, was_known);
         else
-            modify_stat(STAT_STRENGTH, 5, true);
+            modify_stat(STAT_STRENGTH, 5, true, "");
 
         // conceivable max gain of +184 {dlb}
-        you.duration[DUR_MIGHT] += 35 + random2(pow);
-        
+        you.duration[DUR_MIGHT] += (35 + random2(pow)) / factor;
+
         // files.cc permits values up to 215, but ... {dlb}
         if (you.duration[DUR_MIGHT] > 80)
             you.duration[DUR_MIGHT] = 80;
 
-        did_god_conduct( DID_STIMULANTS, 4 + random2(4) );
+        did_god_conduct( DID_STIMULANTS, 4 + random2(4), was_known );
         break;
     }
 
     case POT_GAIN_STRENGTH:
-        mutate(MUT_STRONG);
+        if (mutate(MUT_STRONG))
+            learned_something_new(TUT_YOU_MUTATED);
         break;
 
     case POT_GAIN_DEXTERITY:
-        mutate(MUT_AGILE);
+        if (mutate(MUT_AGILE))
+            learned_something_new(TUT_YOU_MUTATED);
         break;
 
     case POT_GAIN_INTELLIGENCE:
-        mutate(MUT_CLEVER);
+        if (mutate(MUT_CLEVER))
+            learned_something_new(TUT_YOU_MUTATED);
         break;
 
     case POT_LEVITATION:
-        mprf("You feel %s buoyant.",
-             (!player_is_airborne()) ? "very" : "more");
+        mprf("You feel %s buoyant.", !player_is_airborne() ? "very"
+                                                           : "more");
 
         if (!player_is_airborne())
             mpr("You gently float upwards from the floor.");
@@ -161,7 +206,7 @@ bool potion_effect( potion_type pot_eff, int pow )
             mprf("That liquid tasted %s nasty...",
                  (pot_eff == POT_POISON) ? "very" : "extremely" );
 
-            poison_player( ((pot_eff == POT_POISON) ? 1 + random2avg(5, 2) 
+            poison_player( ((pot_eff == POT_POISON) ? 1 + random2avg(5, 2)
                                                     : 3 + random2avg(13, 2)) );
             xom_is_stimulated(128);
         }
@@ -173,8 +218,8 @@ bool potion_effect( potion_type pot_eff, int pow )
         break;
 
     case POT_PARALYSIS:
-        you.paralyse(2 + random2( 6 + you.duration[DUR_PARALYSIS] ));
-        xom_is_stimulated(64);
+        you.paralyse((2 + random2( 6 + you.duration[DUR_PARALYSIS] )) / factor);
+        xom_is_stimulated(64 / factor);
         break;
 
     case POT_CONFUSION:
@@ -183,13 +228,13 @@ bool potion_effect( potion_type pot_eff, int pow )
         break;
 
     case POT_INVISIBILITY:
-        mpr( (!you.duration[DUR_INVIS]) ? "You fade into invisibility!"
-             : "You fade further into invisibility." );
+        mpr(!you.duration[DUR_INVIS] ? "You fade into invisibility!"
+                                     : "You fade further into invisibility.");
 
         // Invisibility cancels backlight.
         you.duration[DUR_BACKLIGHT] = 0;
 
-        // now multiple invisiblity casts aren't as good -- bwr
+        // Now multiple invisiblity casts aren't as good. -- bwr
         if (!you.duration[DUR_INVIS])
             you.duration[DUR_INVIS] = 15 + random2(pow);
         else
@@ -202,7 +247,8 @@ bool potion_effect( potion_type pot_eff, int pow )
         break;
 
     case POT_PORRIDGE:          // oatmeal - always gluggy white/grey?
-        if (you.species == SP_VAMPIRE || you.mutation[MUT_CARNIVOROUS] == 3)
+        if (you.species == SP_VAMPIRE
+            || player_mutation_level(MUT_CARNIVOROUS) == 3)
         {
             mpr("Blech - that potion was really gluggy!");
         }
@@ -214,23 +260,25 @@ bool potion_effect( potion_type pot_eff, int pow )
         break;
 
     case POT_DEGENERATION:
-        if ( pow == 40 )
+        if (pow == 40)
             mpr("There was something very wrong with that liquid!");
-        if (lose_stat(STAT_RANDOM, 1 + random2avg(4, 2)))
-            xom_is_stimulated(64);
+
+        if (lose_stat(STAT_RANDOM, (1 + random2avg(4, 2)) / factor, false,
+                      "drinking a potion of degeneration"))
+        {
+            xom_is_stimulated(64 / factor);
+        }
         break;
 
-    // Don't generate randomly - should be rare and interesting
+    // Don't generate randomly - should be rare and interesting.
     case POT_DECAY:
-        if (rot_player(10 + random2(10)))
-            xom_is_stimulated(64);
+        if (rot_player((10 + random2(10)) / factor))
+            xom_is_stimulated(64 / factor);
         break;
 
     case POT_WATER:
         if (you.species == SP_VAMPIRE)
-        {
             mpr("Blech - this tastes like water.");
-        }
         else
         {
             mpr("This tastes like water.");
@@ -251,8 +299,9 @@ bool potion_effect( potion_type pot_eff, int pow )
         break;                  // I'll let this slip past robe of archmagi
 
     case POT_MAGIC:
+    {
         mpr( "You feel magical!" );
-        new_value = 5 + random2avg(19, 2);
+        int new_value = 5 + random2avg(19, 2);
 
         // increase intrinsic MP points
         if (you.magic_points + new_value > you.max_magic_points)
@@ -263,20 +312,29 @@ bool potion_effect( potion_type pot_eff, int pow )
 
         inc_mp( new_value, true );
         break;
-
+    }
     case POT_RESTORE_ABILITIES:
-        // give a message if restore_stat doesn't
-        if (!restore_stat(STAT_ALL, false))
+    {
+        bool nothing_happens = true;
+        if (you.duration[DUR_BREATH_WEAPON])
+        {
+            mpr("You have got your breath back.", MSGCH_RECOVERY);
+            you.duration[DUR_BREATH_WEAPON] = 0;
+            nothing_happens = false;
+        }
+
+        // Give a message if no message otherwise.
+        if (!restore_stat(STAT_ALL, 0, false) && nothing_happens)
             mpr( "You feel refreshed." );
         break;
-
+    }
     case POT_BERSERK_RAGE:
-        if (you.species == SP_VAMPIRE)
+        if (you.species == SP_VAMPIRE && you.hunger_state <= HS_SATIATED)
         {
             mpr("You feel slightly irritated.");
             make_hungry(100, false);
         }
-        else if ( you.duration[DUR_BUILDING_RAGE] )
+        else if (you.duration[DUR_BUILDING_RAGE])
         {
             you.duration[DUR_BUILDING_RAGE] = 0;
             mpr("Your blood cools.");
@@ -292,68 +350,29 @@ bool potion_effect( potion_type pot_eff, int pow )
         mpr("It has a very clean taste.");
         for (int i = 0; i < 7; i++)
             if (random2(10) > i)
-                delete_mutation(RANDOM_MUTATION);
+                delete_mutation(RANDOM_MUTATION, false);
         break;
 
     case POT_MUTATION:
         mpr("You feel extremely strange.");
         for (int i = 0; i < 3; i++)
             mutate(RANDOM_MUTATION, false);
-        did_god_conduct(DID_STIMULANTS, 4 + random2(4));
-        break;
-        
-  case POT_BLOOD:
-        if (you.species == SP_VAMPIRE)
-        {
-            int temp_rand = random2(9);
-            strcpy(info, (temp_rand == 0) ? "human" :
-                         (temp_rand == 1) ? "rat" :
-                         (temp_rand == 2) ? "goblin" :
-                         (temp_rand == 3) ? "elf" :
-                         (temp_rand == 4) ? "goat" :
-                         (temp_rand == 5) ? "sheep" :
-                         (temp_rand == 6) ? "gnoll" :
-                         (temp_rand == 7) ? "sheep"
-                                          : "yak");
-                                          
-           mprf("Yummy - fresh %s blood!", info);
 
-           lessen_hunger(1000, true);
-           mpr("You feel better.");
-           inc_hp(1 + random2(10), false);
-        }
-        else
-        {
-            bool likes_blood = (you.species == SP_OGRE
-                                || you.species == SP_TROLL
-                                || you.mutation[MUT_CARNIVOROUS]);
-
-            if (likes_blood)
-            {
-                mpr("This tastes like blood.");
-                lessen_hunger(200, true);
-            }
-            else
-            {
-                mpr("Blech - this tastes like blood!");
-                if (!you.mutation[MUT_HERBIVOROUS] && one_chance_in(3))
-                    lessen_hunger(100, true);
-                else
-                    disease_player( 50 + random2(100) );
-                xom_is_stimulated(32);
-            }
-        }
-        did_god_conduct(DID_DRINK_BLOOD, 1 + random2(3));
+        learned_something_new(TUT_YOU_MUTATED);
+        did_god_conduct(DID_DELIBERATE_MUTATING, 10, was_known);
+        did_god_conduct(DID_STIMULANTS, 4 + random2(4), was_known);
         break;
 
     case POT_RESISTANCE:
         mpr("You feel protected.");
-        you.duration[DUR_RESIST_FIRE]   += random2(pow) + 10;
-        you.duration[DUR_RESIST_COLD]   += random2(pow) + 10;
-        you.duration[DUR_RESIST_POISON] += random2(pow) + 10;
-        you.duration[DUR_INSULATION]    += random2(pow) + 10;
-        // one contamination point for each resist
-        contaminate_player(4);
+        you.duration[DUR_RESIST_FIRE]   += (random2(pow) + 10) / factor;
+        you.duration[DUR_RESIST_COLD]   += (random2(pow) + 10) / factor;
+        you.duration[DUR_RESIST_POISON] += (random2(pow) + 10) / factor;
+        you.duration[DUR_INSULATION]    += (random2(pow) + 10) / factor;
+
+        // Just one point of contamination. These potions are really rare,
+        // and contamination is nastier.
+        contaminate_player(1, was_known);
         break;
 
     case NUM_POTIONS:
@@ -364,26 +383,35 @@ bool potion_effect( potion_type pot_eff, int pow )
     return (effect);
 }                               // end potion_effect()
 
-void unwield_item(bool showMsgs)
+bool unwield_item(bool showMsgs)
 {
     const int unw = you.equip[EQ_WEAPON];
-    if ( unw == -1 )
-        return;
-    
+    if (unw == -1)
+        return (false);
+
+    if (you.duration[DUR_BERSERKER])
+    {
+        canned_msg(MSG_TOO_BERSERK);
+        return (false);
+    }
+
+    if (!safe_to_remove_or_wear(you.inv[unw], true))
+        return (false);
+
     you.equip[EQ_WEAPON] = -1;
-    you.special_wield = SPWLD_NONE;
-    you.wield_change = true;
+    you.special_wield    = SPWLD_NONE;
+    you.wield_change     = true;
+    you.m_quiver->on_weapon_changed();
 
     item_def &item(you.inv[unw]);
 
-    if ( item.base_type == OBJ_MISCELLANY &&
-         item.sub_type == MISC_LANTERN_OF_SHADOWS )
+    if (item.base_type == OBJ_MISCELLANY
+        && item.sub_type == MISC_LANTERN_OF_SHADOWS )
     {
         you.current_vision += 2;
         setLOSRadius(you.current_vision);
     }
-
-    if (item.base_type == OBJ_WEAPONS)
+    else if (item.base_type == OBJ_WEAPONS)
     {
         if (is_fixed_artefact( item ))
         {
@@ -399,11 +427,11 @@ void unwield_item(bool showMsgs)
                 break;
             case SPWPN_SCYTHE_OF_CURSES:
             case SPWPN_STAFF_OF_OLGREB:
-                item.plus = 0;
+                item.plus  = 0;
                 item.plus2 = 0;
                 break;
             case SPWPN_STAFF_OF_WUCAD_MU:
-                item.plus = 0;
+                item.plus  = 0;
                 item.plus2 = 0;
                 miscast_effect( SPTYP_DIVINATION, 9, 90, 100,
                                 "the Staff of Wucad Mu" );
@@ -412,7 +440,7 @@ void unwield_item(bool showMsgs)
                 break;
             }
 
-            return;
+            return (true);
         }
 
         const int brand = get_weapon_brand( item );
@@ -423,7 +451,7 @@ void unwield_item(bool showMsgs)
         if (brand != SPWPN_NORMAL)
         {
             const std::string msg = item.name(DESC_CAP_YOUR);
-            
+
             switch (brand)
             {
             case SPWPN_SWORD_OF_CEREBOV:
@@ -451,7 +479,7 @@ void unwield_item(bool showMsgs)
             case SPWPN_PROTECTION:
                 if (showMsgs)
                     mpr("You feel less protected.");
-                you.redraw_armour_class = 1;
+                you.redraw_armour_class = true;
                 break;
 
             case SPWPN_VAMPIRICISM:
@@ -459,9 +487,6 @@ void unwield_item(bool showMsgs)
                     mpr("You feel the strange hunger wane.");
                 break;
 
-                /* case 8: draining
-                   case 9: speed, 10 slicing etc */
-
             case SPWPN_DISTORTION:
                 // Removing the translocations skill reduction of effect,
                 // it might seem sensible, but this brand is supposed
@@ -474,36 +499,44 @@ void unwield_item(bool showMsgs)
 
                 // int effect = 9 - random2avg( you.skills[SK_TRANSLOCATIONS] * 2, 2 );
                 miscast_effect( SPTYP_TRANSLOCATION, 9, 90, 100,
-                                "a distortion effect" );
+                                "distortion unwield" );
                 break;
 
-                // when more are added here, *must* duplicate unwielding
-                // effect in vorpalise weapon scroll effect in read_scoll
-            }                   // end switch
+                // NOTE: When more are added here, *must* duplicate unwielding
+                // effect in vorpalise weapon scroll effect in read_scoll.
+            }
 
             if (you.duration[DUR_WEAPON_BRAND])
             {
                 you.duration[DUR_WEAPON_BRAND] = 0;
                 set_item_ego_type( item, OBJ_WEAPONS, SPWPN_NORMAL );
-                // we're letting this through even if hiding messages
+
+                // We're letting this through even if hiding messages.
                 mpr("Your branding evaporates.");
             }
-        }                       // end if
+        }
     }
 
     if (item.base_type == OBJ_STAVES && item.sub_type == STAFF_POWER)
+    {
         calc_mp();
+        mpr("You feel your mana capacity decrease.");
+    }
 
-    return;
+    you.attribute[ATTR_WEAPON_SWAP_INTERRUPTED] = 0;
+
+    return (true);
 }                               // end unwield_item()
 
 // This does *not* call ev_mod!
 void unwear_armour(char unw)
 {
-    you.redraw_armour_class = 1;
-    you.redraw_evasion = 1;
+    you.redraw_armour_class = true;
+    you.redraw_evasion = true;
 
-    switch (get_armour_ego_type( you.inv[unw] ))
+    item_def &item(you.inv[unw]);
+
+    switch (get_armour_ego_type( item ))
     {
     case SPARM_RUNNING:
         mpr("You feel rather sluggish.");
@@ -533,15 +566,15 @@ void unwear_armour(char unw)
         break;
 
     case SPARM_STRENGTH:
-        modify_stat(STAT_STRENGTH, -3, false);
+        modify_stat(STAT_STRENGTH, -3, false, item, true);
         break;
 
     case SPARM_DEXTERITY:
-        modify_stat(STAT_DEXTERITY, -3, false);
+        modify_stat(STAT_DEXTERITY, -3, false, item, true);
         break;
 
     case SPARM_INTELLIGENCE:
-        modify_stat(STAT_INTELLIGENCE, -3, false);
+        modify_stat(STAT_INTELLIGENCE, -3, false, item, true);
         break;
 
     case SPARM_PONDEROUSNESS:
@@ -550,7 +583,6 @@ void unwear_armour(char unw)
         break;
 
     case SPARM_LEVITATION:
-        //you.duration[DUR_LEVITATION]++;
         if (you.duration[DUR_LEVITATION])
             you.duration[DUR_LEVITATION] = 1;
         break;
@@ -585,9 +617,7 @@ void unwear_armour(char unw)
 
     if (is_random_artefact( you.inv[unw] ))
         unuse_randart(unw);
-
-    return;
-}                               // end unwear_armour()
+}
 
 void unuse_randart(unsigned char unw)
 {
@@ -598,36 +628,48 @@ void unuse_randart(const item_def &item)
 {
     ASSERT( is_random_artefact( item ) );
 
-    const bool ident = fully_identified(item);
-
     randart_properties_t proprt;
-    randart_wpn_properties( item, proprt );
+    randart_known_props_t known;
+    randart_wpn_properties( item, proprt, known );
 
     if (proprt[RAP_AC])
     {
-        you.redraw_armour_class = 1;
-        if (!ident)
+        you.redraw_armour_class = true;
+        if (!known[RAP_AC])
         {
             mprf("You feel less %s.",
-            proprt[RAP_AC] > 0? "well-protected" : "vulnerable");
+                 proprt[RAP_AC] > 0? "well-protected" : "vulnerable");
         }
     }
 
     if (proprt[RAP_EVASION])
     {
-        you.redraw_evasion = 1;
-        if (!ident)
+        you.redraw_evasion = true;
+        if (!known[RAP_EVASION])
         {
             mprf("You feel less %s.",
                  proprt[RAP_EVASION] > 0? "nimble" : "awkward");
         }
     }
 
-    // modify ability scores, always output messages
-    modify_stat( STAT_STRENGTH,     -proprt[RAP_STRENGTH],     false );
-    modify_stat( STAT_INTELLIGENCE, -proprt[RAP_INTELLIGENCE], false );
-    modify_stat( STAT_DEXTERITY,    -proprt[RAP_DEXTERITY],    false );
+    if (proprt[RAP_MAGICAL_POWER])
+    {
+        you.redraw_magic_points = true;
+        if (!known[RAP_MAGICAL_POWER])
+        {
+            mprf("You feel your mana capacity %s.",
+                 proprt[RAP_MAGICAL_POWER] > 0 ? "decrease" : "increase");
+        }
+    }
+
+    // Modify ability scores; always output messages.
+    modify_stat( STAT_STRENGTH,     -proprt[RAP_STRENGTH],     false, item,
+                 true);
+    modify_stat( STAT_INTELLIGENCE, -proprt[RAP_INTELLIGENCE], false, item,
+                 true);
+    modify_stat( STAT_DEXTERITY,    -proprt[RAP_DEXTERITY],    false, item,
+                 true);
 
     if (proprt[RAP_NOISES] != 0)
         you.special_wield = SPWLD_NONE;
-}                               // end unuse_randart()
+}
