@@ -3,7 +3,7 @@
  *  Summary:    Functions for using some of the wackier inventory items.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T13:10:01.706311Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-07T14:23:40.277492Z $
  *
  *  Change History (most recent first):
  *
@@ -25,7 +25,7 @@
 #include "cloud.h"
 #include "database.h"
 #include "decks.h"
-#include "direct.h"
+#include "directn.h"
 #include "effects.h"
 #include "fight.h"
 #include "food.h"
@@ -47,11 +47,14 @@
 #include "skills2.h"
 #include "spells1.h"
 #include "spells2.h"
+#include "spells3.h"
 #include "spl-book.h"
 #include "spl-cast.h"
 #include "spl-util.h"
+#include "state.h"
 #include "stuff.h"
 #include "view.h"
+#include "xom.h"
 
 static bool ball_of_energy(void);
 static bool ball_of_fixation(void);
@@ -74,7 +77,7 @@ void special_wielded()
     case SPWLD_NOISE:
     {
         makes_noise = (one_chance_in(20) && !silenced(you.x_pos, you.y_pos));
-          
+
         if (makes_noise)
         {
             std::string msg;
@@ -82,7 +85,7 @@ void special_wielded()
             if (you.special_wield == SPWLD_SING)
             {
                 msg = getSpeakString("Singing Sword");
-                if (msg != "")
+                if (!msg.empty())
                 {
                     // "Your Singing Sword" sounds disrespectful
                     // (as if there could be more than one!)
@@ -93,21 +96,21 @@ void special_wielded()
             else // SPWLD_NOISE
             {
                 msg = getSpeakString("noisy weapon");
-                if (msg != "")
+                if (!msg.empty())
                 {
                     msg = replace_all(msg, "@Your_weapon@", "Your @weapon@");
                     msg = replace_all(msg, "@your_weapon@", "your @weapon@");
                 }
             }
-            
-            // set appropriate channel (will usually be TALK)
+
+            // Set appropriate channel (will usually be TALK).
             msg_channel_type channel = MSGCH_TALK;
 
-            // disallow anything with VISUAL in it;
-            if (msg != "" && msg.find("VISUAL") != std::string::npos)
+            // Disallow anything with VISUAL in it.
+            if (!msg.empty() && msg.find("VISUAL") != std::string::npos)
                 msg = "";
-                
-            if (msg != "")
+
+            if (!msg.empty())
             {
                 std::string param = "";
                 std::string::size_type pos = msg.find(":");
@@ -132,12 +135,12 @@ void special_wielded()
                     else if (param != "TALK")
                         match = false;
 
-                    if (match && msg != "")
+                    if (match && !msg.empty())
                         msg = msg.substr(pos + 1);
                 }
             }
-            
-            if (msg == "") // give default noises
+
+            if (msg.empty()) // give default noises
             {
                 if (you.special_wield == SPWLD_SING)
                     msg = "@The_weapon@ sings.";
@@ -147,7 +150,7 @@ void special_wielded()
                     msg = "You hear a strange noise.";
                 }
             }
-            
+
             // replace weapon references
             msg = replace_all(msg, "@The_weapon@", "The @weapon@");
             msg = replace_all(msg, "@the_weapon@", "the @weapon@");
@@ -157,8 +160,8 @@ void special_wielded()
 
         } // makes_noise
         break;
-    } 
-    
+    }
+
     case SPWLD_CURSE:
         if (one_chance_in(30))
             curse_an_item(false);
@@ -167,10 +170,10 @@ void special_wielded()
     case SPWLD_VARIABLE:
         do_uncurse_item( you.inv[wpn] );
 
-        if (random2(5) < 2)     // 40% chance {dlb}
+        if (x_chance_in_y(2, 5))     // 40% chance {dlb}
             you.inv[wpn].plus  += (coinflip() ? +1 : -1);
 
-        if (random2(5) < 2)     // 40% chance {dlb}
+        if (x_chance_in_y(2, 5))     // 40% chance {dlb}
             you.inv[wpn].plus2 += (coinflip() ? +1 : -1);
 
         if (you.inv[wpn].plus < -4)
@@ -189,28 +192,28 @@ void special_wielded()
     case SPWLD_TORMENT:
         if (one_chance_in(200))
         {
-            torment( TORMENT_SPWLD, you.x_pos, you.y_pos );
-            did_god_conduct( DID_UNHOLY, 1 );
+            torment(TORMENT_SPWLD, you.x_pos, you.y_pos);
+            did_god_conduct(DID_UNHOLY, 1);
         }
         break;
 
     case SPWLD_ZONGULDROK:
         if (one_chance_in(5))
         {
-            animate_dead( &you, 1 + random2(3), BEH_HOSTILE, MHITYOU, 1 );
-            did_god_conduct( DID_NECROMANCY, 1 );
+            animate_dead(&you, 1 + random2(3), BEH_HOSTILE, MHITYOU);
+            did_god_conduct(DID_NECROMANCY, 1);
         }
         break;
 
     case SPWLD_POWER:
-        you.inv[wpn].plus = stepdown_value( -4 + (you.hp / 5), 4, 4, 4, 20 );
+        you.inv[wpn].plus  = stepdown_value( -4 + (you.hp / 5), 4, 4, 4, 20 );
         you.inv[wpn].plus2 = you.inv[wpn].plus;
         break;
 
     case SPWLD_OLGREB:
         // Giving Olgreb's staff a little lift since staves of poison have
         // been made better. -- bwr
-        you.inv[wpn].plus = you.skills[SK_POISON_MAGIC] / 3;
+        you.inv[wpn].plus  = you.skills[SK_POISON_MAGIC] / 3;
         you.inv[wpn].plus2 = you.inv[wpn].plus;
         break;
 
@@ -220,11 +223,12 @@ void special_wielded()
         break;
 
     case SPWLD_SHADOW:
-        if (random2(8) <= player_spec_death())
+        if (x_chance_in_y(player_spec_death() + 1, 8))
         {
-            did_god_conduct( DID_NECROMANCY, 1 );
-            create_monster( MONS_SHADOW, 2, BEH_FRIENDLY,
-                            you.x_pos, you.y_pos, you.pet_target, 250 );
+            create_monster(
+                mgen_data(MONS_SHADOW, BEH_FRIENDLY,
+                          2, you.pos(), you.pet_target));
+            did_god_conduct(DID_NECROMANCY, 1);
         }
 
         show_green = DARKGREY;
@@ -236,7 +240,7 @@ void special_wielded()
     }
 
     if (makes_noise)
-        noisy( 25, you.x_pos, you.y_pos );
+        noisy(25, you.x_pos, you.y_pos);
 
     if (old_plus != you.inv[wpn].plus
         || old_plus2 != you.inv[wpn].plus2
@@ -244,36 +248,37 @@ void special_wielded()
     {
         you.wield_change = true;
     }
-
-    return;
 }                               // end special_wielded()
 
-static bool reaching_weapon_attack(void)
+static bool _reaching_weapon_attack(const item_def& wpn)
 {
     dist beam;
-    int x_distance, y_distance;
-    int x_middle, y_middle;
-    int skill;
 
     mpr("Attack whom?", MSGCH_PROMPT);
 
-    direction(beam, DIR_TARGET, TARG_ENEMY);
+    direction(beam, DIR_TARGET, TARG_ENEMY, 2);
+
     if (!beam.isValid)
-        return false;
+        return (false);
 
     if (beam.isMe)
     {
         canned_msg(MSG_UNTHINKING_ACT);
-        return false;
+        return (false);
     }
 
-    x_distance = abs(beam.tx - you.x_pos);
-    y_distance = abs(beam.ty - you.y_pos);
+    const int x_distance = abs(beam.tx - you.x_pos);
+    const int y_distance = abs(beam.ty - you.y_pos);
 
     if (x_distance > 2 || y_distance > 2)
     {
         mpr("Your weapon cannot reach that far!");
-        return false;
+        return (false);
+    }
+    else if (!see_grid_no_trans(beam.tx, beam.ty))
+    {
+        mpr("There's a wall in the way.");
+        return (false);
     }
     else if (mgrd[beam.tx][beam.ty] == NON_MONSTER)
     {
@@ -282,66 +287,152 @@ static bool reaching_weapon_attack(void)
         // here to prevent scumming...but that would just encourage
         // finding popcorn monsters.
         mpr("You attack empty space.");
-        return true;
+        return (true);
     }
 
-    /* BCR - Added a check for monsters in the way.  Only checks cardinal
-     *       directions.  Knight moves are ignored.  Assume the weapon
-     *       slips between the squares.
-     */
+    // BCR - Added a check for monsters in the way.  Only checks cardinal
+    //       directions.  Knight moves are ignored.  Assume the weapon
+    //       slips between the squares.
 
-    // if we're attacking more than a space away
-    if ((x_distance > 1) || (y_distance > 1))
+    // If we're attacking more than a space away...
+    if (x_distance > 1 || y_distance > 1)
     {
-        x_middle = MAX(beam.tx, you.x_pos) - (x_distance / 2);
-        y_middle = MAX(beam.ty, you.y_pos) - (y_distance / 2);
+        const int x_middle = MAX(beam.tx, you.x_pos) - (x_distance / 2);
+        const int y_middle = MAX(beam.ty, you.y_pos) - (y_distance / 2);
 
-        // if either the x or the y is the same, we should check for
+        bool success = false;
+        // If either the x or the y is the same, we should check for
         // a monster:
-        if (((beam.tx == you.x_pos) || (beam.ty == you.y_pos))
-                && (mgrd[x_middle][y_middle] != NON_MONSTER))
+        if ((beam.tx == you.x_pos || beam.ty == you.y_pos)
+            && mgrd[x_middle][y_middle] != NON_MONSTER)
         {
-            skill = weapon_skill( you.inv[you.equip[EQ_WEAPON]].base_type,
-                                  you.inv[you.equip[EQ_WEAPON]].sub_type );
+            const int skill = weapon_skill( wpn.base_type, wpn.sub_type );
 
-            if ((5 + (3 * skill)) > random2(40))
+            if (x_chance_in_y(5 + (3 * skill), 40))
             {
                 mpr("You reach to attack!");
-                you_attack(mgrd[beam.tx][beam.ty], false);
+                success = you_attack(mgrd[beam.tx][beam.ty], false);
             }
             else
             {
                 mpr("You could not reach far enough!");
-                return true;
+                return (true);
             }
         }
         else
         {
             mpr("You reach to attack!");
-            you_attack(mgrd[beam.tx][beam.ty], false);
+            success = you_attack(mgrd[beam.tx][beam.ty], false);
+        }
+
+        if (success)
+        {
+            int mid = mgrd[beam.tx][beam.ty];
+            if (mid != NON_MONSTER)
+            {
+                monsters *mon = &menv[mgrd[beam.tx][beam.ty]];
+                if (mons_is_mimic( mon->type ))
+                    mimic_alert(mon);
+            }
         }
     }
     else
-    {
         you_attack(mgrd[beam.tx][beam.ty], false);
-    }
 
-    return true;
-}                               // end reaching_weapon_attack()
+    return (true);
+}
 
-// returns true if item successfully evoked.
-bool evoke_wielded( void )
+static bool evoke_horn_of_geryon()
+{
+    // Note: This assumes that the Vestibule has not been changed.
+    bool rc = false;
+
+    if (player_in_branch( BRANCH_VESTIBULE_OF_HELL ))
+    {
+        mpr("You produce a weird and mournful sound.");
+
+        for (int count_x = 0; count_x < GXM; count_x++)
+            for (int count_y = 0; count_y < GYM; count_y++)
+            {
+                if (grd[count_x][count_y] == DNGN_STONE_ARCH)
+                {
+                    rc = true;
+
+                    map_marker *marker =
+                        env.markers.find(coord_def(count_x, count_y),
+                                         MAT_FEATURE);
+
+                    if (marker)
+                    {
+                        map_feature_marker *featm =
+                            dynamic_cast<map_feature_marker*>(marker);
+                        grd[count_x][count_y] = featm->feat;
+                        env.markers.remove(marker);
+                    }
+                }
+            }
+
+        if (rc)
+            mpr("Your way has been unbarred.");
+    }
+    else
+    {
+        mpr("You produce a hideous howling noise!", MSGCH_SOUND);
+        create_monster(
+            mgen_data(MONS_BEAST, BEH_HOSTILE,
+                      4, you.pos(), MHITYOU));
+    }
+    return (rc);
+}
+
+static bool evoke_sceptre_of_asmodeus()
+{
+    bool rc = true;
+    if (one_chance_in(21))
+        rc = false;
+    else if (coinflip())
+    {
+        // Summon devils, maybe a Fiend.
+        const monster_type mtype = (one_chance_in(4) ? MONS_FIEND :
+                                    summon_any_demon(DEMON_COMMON));
+        const bool good_summon =
+            create_monster(
+                mgen_data(mtype, BEH_HOSTILE,
+                          6, you.pos(), MHITYOU)) != -1;
+
+        if (good_summon)
+        {
+            if (mtype == MONS_FIEND)
+                mpr("\"Your arrogance condemns you, mortal!\"");
+            else
+                mpr("The Sceptre summons one of its servants.");
+        }
+        else
+            mpr("The air shimmers briefly.");
+    }
+    else
+    {
+        // Cast a destructive spell
+        const spell_type spl = static_cast<spell_type>(
+            random_choose_weighted( 114, SPELL_BOLT_OF_FIRE,
+                                    57,  SPELL_LIGHTNING_BOLT,
+                                    57,  SPELL_BOLT_OF_DRAINING,
+                                    12,  SPELL_HELLFIRE,
+                                    0 ));
+        your_spells( spl, you.skills[SK_EVOCATIONS] * 8, false );
+    }
+    return (true);
+}
+
+// Returns true if item successfully evoked.
+bool evoke_wielded()
 {
-    char opened_gates = 0;
-    unsigned char spell_casted = random2(21);
-    int count_x, count_y;
-    int temp_rand = 0;      // for probability determination {dlb}
     int power = 0;
 
-    int pract = 0;
-    bool did_work = false;  // used for default "nothing happens" message
+    int pract = 0; // By how much Evocations is practised.
+    bool did_work = false;  // Used for default "nothing happens" message.
 
-    int wield = you.equip[EQ_WEAPON];
+    const int wield = you.equip[EQ_WEAPON];
 
     if (you.duration[DUR_BERSERKER])
     {
@@ -351,6 +442,7 @@ bool evoke_wielded( void )
     else if (wield == -1)
     {
         mpr("You aren't wielding anything!");
+        crawl_state.zero_turns_taken();
         return (false);
     }
 
@@ -358,28 +450,28 @@ bool evoke_wielded( void )
     bool unevokable = false;
 
     // Check inscriptions.
-    if ( !check_warning_inscriptions(wpn, OPER_EVOKE) )
-        return false;
+    if (!check_warning_inscriptions(wpn, OPER_EVOKE))
+        return (false);
 
     switch (wpn.base_type)
     {
     case OBJ_WEAPONS:
         if (get_weapon_brand(wpn) == SPWPN_REACHING)
         {
-            if (reaching_weapon_attack())
+            if (_reaching_weapon_attack(wpn))
             {
                 pract = 0;
                 did_work = true;
             }
             else
-                return false;
+                return (false);
         }
         else if (is_fixed_artefact( wpn ))
         {
             switch (wpn.special)
             {
             case SPWPN_STAFF_OF_DISPATER:
-                if (you.duration[DUR_DEATHS_DOOR] || !enough_hp(11, true) 
+                if (you.duration[DUR_DEATHS_DOOR] || !enough_hp(11, true)
                     || !enough_mp(5, true))
                 {
                     break;
@@ -389,7 +481,7 @@ bool evoke_wielded( void )
 
                 dec_hp( 5 + random2avg(19, 2), false, "Staff of Dispater" );
                 dec_mp( 2 + random2avg(5, 2) );
-                make_hungry( 100, false );
+                make_hungry(100, false, true);
 
                 power = you.skills[SK_EVOCATIONS] * 8;
                 your_spells( SPELL_HELLFIRE, power, false );
@@ -397,67 +489,24 @@ bool evoke_wielded( void )
                 did_work = true;
                 break;
 
-            // let me count the number of ways spell_casted is
-            // used here ... one .. two .. three ... >CRUNCH<
-            // three licks to get to the center of a ... {dlb}
             case SPWPN_SCEPTRE_OF_ASMODEUS:
-                spell_casted = random2(21);
-
-                if (spell_casted == 0)
-                    break;
-
-                make_hungry( 200, false );
-                pract = 1;
-
-                if (spell_casted < 2)   // summon devils, maybe a Fiend
+                if (evoke_sceptre_of_asmodeus())
                 {
-
-                    spell_casted = (one_chance_in(4) ? MONS_FIEND
-                                                 : MONS_HELLION + random2(10));
-
-                    bool good_summon = (create_monster( spell_casted, 
-                                            6, BEH_HOSTILE,
-                                            you.x_pos, you.y_pos, 
-                                            MHITYOU, 250) != -1);
-
-                    if (good_summon)
-                    {
-                        if (spell_casted == MONS_FIEND)
-                            mpr("\"Your arrogance condemns you, mortal!\"");
-                        else
-                            mpr("The Sceptre summons one of its servants.");
-                    }
-
+                    make_hungry(200, false, true);
                     did_work = true;
-                    break;
+                    pract = 1;
                 }
-
-                temp_rand = random2(240);
-
-                if (temp_rand > 125)
-                    spell_casted = SPELL_BOLT_OF_FIRE;      // 114 in 240
-                else if (temp_rand > 68)
-                    spell_casted = SPELL_LIGHTNING_BOLT;    //  57 in 240
-                else if (temp_rand > 11)
-                    spell_casted = SPELL_BOLT_OF_DRAINING;  //  57 in 240
-                else
-                    spell_casted = SPELL_HELLFIRE;          //  12 in 240
-
-                power = you.skills[SK_EVOCATIONS] * 8;
-                your_spells( static_cast<spell_type>(spell_casted),
-                             power, false );
-                did_work = true;
                 break;
 
             case SPWPN_STAFF_OF_OLGREB:
-                if (!enough_mp( 4, true ) 
+                if (!enough_mp( 4, true )
                     || you.skills[SK_EVOCATIONS] < random2(6))
                 {
                     break;
                 }
 
                 dec_mp(4);
-                make_hungry( 50, false );
+                make_hungry(50, false, true);
                 pract = 1;
                 did_work = true;
 
@@ -465,7 +514,7 @@ bool evoke_wielded( void )
 
                 your_spells( SPELL_OLGREBS_TOXIC_RADIANCE, power, false );
 
-                if (you.skills[SK_EVOCATIONS] >= random2(10))
+                if (x_chance_in_y(you.skills[SK_EVOCATIONS] + 1, 10))
                     your_spells( SPELL_VENOM_BOLT, power, false );
                 break;
 
@@ -479,13 +528,13 @@ bool evoke_wielded( void )
                 mpr("Magical energy flows into your mind!");
 
                 inc_mp( 3 + random2(5) + you.skills[SK_EVOCATIONS] / 3, false );
-                make_hungry( 50, false );
+                make_hungry(50, false, true);
                 pract = 1;
                 did_work = true;
 
                 if (one_chance_in(3))
                 {
-                    miscast_effect( SPTYP_DIVINATION, random2(9), 
+                    miscast_effect( SPTYP_DIVINATION, random2(9),
                                     random2(70), 100, "the Staff of Wucad Mu" );
                 }
                 break;
@@ -513,22 +562,23 @@ bool evoke_wielded( void )
         }
         else if (wpn.sub_type == STAFF_CHANNELING)
         {
-            if (you.magic_points < you.max_magic_points 
-                && you.skills[SK_EVOCATIONS] >= random2(30)) 
+            if (you.magic_points < you.max_magic_points
+                && x_chance_in_y(you.skills[SK_EVOCATIONS] + 11, 40))
             {
                 mpr("You channel some magical energy.");
                 inc_mp( 1 + random2(3), false );
-                make_hungry( 50, false );
+                make_hungry(50, false, true);
                 pract = (one_chance_in(5) ? 1 : 0);
                 did_work = true;
 
                 if (!item_type_known(wpn))
                 {
+                    set_ident_type( OBJ_STAVES, wpn.sub_type, ID_KNOWN_TYPE );
                     set_ident_flags( wpn, ISFLAG_KNOW_TYPE );
 
                     mprf("You are wielding %s.",
                          wpn.name(DESC_NOCAP_A).c_str());
-                         
+
                     more();
 
                     you.wield_change = true;
@@ -543,8 +593,8 @@ bool evoke_wielded( void )
 
     case OBJ_MISCELLANY:
         did_work = true; // easier to do it this way for misc items
-        
-        if ( is_deck(wpn) )
+
+        if (is_deck(wpn))
         {
             evoke_deck(wpn);
             pract = 1;
@@ -568,7 +618,7 @@ bool evoke_wielded( void )
                 canned_msg(MSG_NOTHING_HAPPENS);
             else
             {
-                summon_elemental(100, MONS_AIR_ELEMENTAL, 4);
+                cast_summon_elemental(100, GOD_NO_GOD, MONS_AIR_ELEMENTAL, 4);
                 pract = (one_chance_in(5) ? 1 : 0);
             }
             break;
@@ -578,7 +628,7 @@ bool evoke_wielded( void )
                 canned_msg(MSG_NOTHING_HAPPENS);
             else
             {
-                summon_elemental(100, MONS_FIRE_ELEMENTAL, 4);
+                cast_summon_elemental(100, GOD_NO_GOD, MONS_FIRE_ELEMENTAL, 4);
                 pract = (one_chance_in(5) ? 1 : 0);
             }
             break;
@@ -588,57 +638,14 @@ bool evoke_wielded( void )
                 canned_msg(MSG_NOTHING_HAPPENS);
             else
             {
-                summon_elemental(100, MONS_EARTH_ELEMENTAL, 4);
+                cast_summon_elemental(100, GOD_NO_GOD, MONS_EARTH_ELEMENTAL, 4);
                 pract = (one_chance_in(5) ? 1 : 0);
             }
             break;
 
         case MISC_HORN_OF_GERYON:
-            // Note: This assumes that the Vestibule has not been changed.
-            if (player_in_branch( BRANCH_VESTIBULE_OF_HELL ))
-            {
-                mpr("You produce a weird and mournful sound.");
-
-                for (count_x = 0; count_x < GXM; count_x++)
-                {
-                    for (count_y = 0; count_y < GYM; count_y++)
-                    {
-                        if (grd[count_x][count_y] == DNGN_STONE_ARCH)
-                        {
-                            opened_gates++;
-
-                            map_marker *marker =
-                                env.markers.find(coord_def(count_x, count_y),
-                                                 MAT_FEATURE);
-
-                            if (marker)
-                            {
-                                map_feature_marker *featm =
-                                    dynamic_cast<map_feature_marker*>(marker);
-                                grd[count_x][count_y] = featm->feat;
-                                env.markers.remove(marker);
-                            }
-                        }
-                    }
-                }
-
-                if (opened_gates)
-                {
-                    mpr("Your way has been unbarred.");
-                    pract = 1;
-                }
-            }
-            else
-            {
-                mpr("You produce a hideous howling noise!");
-                int midx = create_monster( MONS_BEAST, 4,
-                                           BEH_HOSTILE, you.x_pos, you.y_pos,
-                                           MHITYOU, 250 );
-                // avoid scumming; also prevents it from showing up on notes
-                if ( midx != -1 )
-                    menv[midx].flags |= MF_CREATED_FRIENDLY;
-                // no practice
-            }
+            if (evoke_horn_of_geryon())
+                pract = 1;
             break;
 
         case MISC_BOX_OF_BEASTS:
@@ -680,38 +687,43 @@ bool evoke_wielded( void )
 
     if (!unevokable)
         you.turn_is_over = true;
+    else
+        crawl_state.zero_turns_taken();
 
     return (did_work);
 }                               // end evoke_wielded()
 
 static bool efreet_flask(void)
 {
-    const beh_type behaviour =
-        ((you.skills[SK_EVOCATIONS] / 3 + 10 > random2(20)) 
-         ? BEH_FRIENDLY : BEH_HOSTILE);
+    bool friendly = x_chance_in_y(10 + you.skills[SK_EVOCATIONS] / 3, 20);
 
     mpr("You open the flask...");
 
-    const int efreet = create_monster( MONS_EFREET, 0, behaviour, 
-                                       you.x_pos, you.y_pos, MHITYOU, 250,
-                                       false, false, true );
-    if (efreet != -1)
+    const int monster =
+        create_monster(
+            mgen_data(MONS_EFREET,
+                      friendly ? BEH_FRIENDLY : BEH_HOSTILE,
+                      0, you.pos(),
+                      friendly ? you.pet_target : MHITYOU,
+                      MG_FORCE_BEH));
+
+    if (monster != -1)
     {
-        monsters *mon = &menv[efreet];
-        
-        mpr( "...and a huge efreet comes out." );
-        player_angers_monster(mon);
-        mpr( (mon->attitude == ATT_FRIENDLY)?
-             "\"Thank you for releasing me!\""
-             : "It howls insanely!" );
+        mpr("...and a huge efreet comes out.");
+
+        if (player_angers_monster(&menv[monster]))
+            friendly = false;
+
+        mpr(friendly ? "\"Thank you for releasing me!\""
+                     : "It howls insanely!");
     }
     else
         canned_msg(MSG_NOTHING_HAPPENS);
 
-    dec_inv_item_quantity( you.equip[EQ_WEAPON], 1 );    
+    dec_inv_item_quantity(you.equip[EQ_WEAPON], 1);
 
     return (true);
-}                               // end efreet_flask()
+}
 
 static bool ball_of_seeing(void)
 {
@@ -720,12 +732,12 @@ static bool ball_of_seeing(void)
 
     mpr("You gaze into the crystal ball.");
 
-    use = ((!you.duration[DUR_CONF]) ?
-           random2(you.skills[SK_EVOCATIONS] * 6) : random2(5));
+    use = (!you.duration[DUR_CONF] ? random2(you.skills[SK_EVOCATIONS] * 6)
+                                   : random2(5));
 
     if (use < 2)
     {
-        lose_stat( STAT_INTELLIGENCE, 1 );
+        lose_stat( STAT_INTELLIGENCE, 1, false, "using a ball of seeing");
     }
     else if (use < 5 && enough_mp(1, true))
     {
@@ -763,11 +775,11 @@ static bool disc_of_storms(void)
     const int fail_rate = (30 - you.skills[SK_EVOCATIONS]);
     bool ret = false;
 
-    if (player_res_electricity() || (random2(100) < fail_rate))
+    if (player_res_electricity() || x_chance_in_y(fail_rate, 100))
         canned_msg(MSG_NOTHING_HAPPENS);
-    else if (random2(100) < fail_rate)
+    else if (x_chance_in_y(fail_rate, 100))
         mpr("The disc glows for a moment, then fades.");
-    else if (random2(100) < fail_rate)
+    else if (x_chance_in_y(fail_rate, 100))
         mpr("Little bolts of electricity crackle over the disc.");
     else
     {
@@ -790,6 +802,7 @@ static bool disc_of_storms(void)
             beam.target_x = you.x_pos + random2(13) - 6;
             beam.target_y = you.y_pos + random2(13) - 6;
 
+            // Non-controlleable, so no player tracer.
             zapping( which_zap, 30 + you.skills[SK_EVOCATIONS] * 2, beam );
 
             disc_count--;
@@ -799,17 +812,12 @@ static bool disc_of_storms(void)
     }
 
     return (ret);
-}                               // end disc_of_storms()
+}
 
-void tome_of_power(char sc_read_2)
+void tome_of_power(int slot)
 {
-    int temp_rand = 0;          // probability determination {dlb}
-
-    int powc = 5 + you.skills[SK_EVOCATIONS] 
-                 + roll_dice( 5, you.skills[SK_EVOCATIONS] ); 
-
-    spell_type spell_casted = SPELL_NO_SPELL;
-    struct bolt beam;
+    int powc = 5 + you.skills[SK_EVOCATIONS]
+                 + roll_dice( 5, you.skills[SK_EVOCATIONS] );
 
     msg::stream << "The book opens to a page covered in "
                 << weird_writing() << '.' << std::endl;
@@ -819,59 +827,48 @@ void tome_of_power(char sc_read_2)
     if (!yesno("Read it?"))
         return;
 
-    set_ident_flags( you.inv[sc_read_2], ISFLAG_IDENT_MASK );
+    set_ident_flags( you.inv[slot], ISFLAG_KNOW_TYPE );
 
-    if (you.mutation[MUT_BLURRY_VISION] > 0
-        && random2(4) < you.mutation[MUT_BLURRY_VISION])
+    if (player_mutation_level(MUT_BLURRY_VISION) > 0
+        && x_chance_in_y(player_mutation_level(MUT_BLURRY_VISION), 4))
     {
         mpr("The page is too blurry for you to read.");
         return;
     }
 
     mpr("You find yourself reciting the magical words!");
-    exercise( SK_EVOCATIONS, 1 ); 
+    exercise( SK_EVOCATIONS, 1 );
 
-    temp_rand = random2(50) + random2( you.skills[SK_EVOCATIONS] / 3 ); 
-
-    switch (random2(50))
+    if (x_chance_in_y(7, 50))
     {
-    case 0:
-    case 3:
-    case 4:
-    case 6:
-    case 7:
-    case 8:
-    case 9:
         mpr("A cloud of weird smoke pours from the book's pages!");
         big_cloud( random_smoke_type(), KC_YOU,
                    you.x_pos, you.y_pos, 20, 10 + random2(8) );
         xom_is_stimulated(16);
-        return;
-    case 1:
-    case 14:
+    }
+    else if (x_chance_in_y(2, 43))
+    {
         mpr("A cloud of choking fumes pours from the book's pages!");
         big_cloud(CLOUD_POISON, KC_YOU,
                   you.x_pos, you.y_pos, 20, 7 + random2(5));
         xom_is_stimulated(64);
-        return;
-
-    case 2:
-    case 13:
+    }
+    else if (x_chance_in_y(2, 41))
+    {
         mpr("A cloud of freezing gas pours from the book's pages!");
         big_cloud(CLOUD_COLD, KC_YOU, you.x_pos, you.y_pos, 20, 8 + random2(5));
         xom_is_stimulated(64);
-        return;
-
-    case 5:
-    case 11:
-    case 12:
+    }
+    else if (x_chance_in_y(3, 39))
+    {
         if (one_chance_in(5))
         {
             mpr("The book disappears in a mighty explosion!");
-            dec_inv_item_quantity( sc_read_2, 1 );
+            dec_inv_item_quantity( slot, 1 );
         }
 
-        beam.type = SYM_BURST;
+        bolt beam;
+        beam.type = dchar_glyph(DCHAR_FIRED_BURST);
         beam.damage = dice_def( 3, 15 );
         // unsure about this    // BEAM_EXPLOSION instead? [dlb]
         beam.flavour = BEAM_FIRE;
@@ -882,117 +879,127 @@ void tome_of_power(char sc_read_2)
         // your explosion, (not someone else's explosion)
         beam.beam_source = NON_MONSTER;
         beam.thrower = KILL_YOU;
-        beam.aux_source = "an exploding Tome of Power";
+        beam.aux_source = "an exploding tome of Destruction";
         beam.ex_size = 2;
         beam.is_tracer = false;
         beam.is_explosion = true;
 
         explosion(beam);
         xom_is_stimulated(255);
-        return;
-
-    case 10:
-        if (create_monster( MONS_ABOMINATION_SMALL, 6, BEH_HOSTILE,
-                            you.x_pos, you.y_pos, MHITYOU, 250 ) != -1)
+    }
+    else if (one_chance_in(36))
+    {
+        if (create_monster(
+                mgen_data(MONS_ABOMINATION_SMALL, BEH_HOSTILE,
+                          6, you.pos(), MHITYOU)) != -1)
         {
             mpr("A horrible Thing appears!");
             mpr("It doesn't look too friendly.");
         }
         xom_is_stimulated(255);
-        return;
-    }
-
-    viewwindow(1, false);
-
-    temp_rand = random2(23) + random2( you.skills[SK_EVOCATIONS] / 3 );
-
-    if (temp_rand > 25)
-        temp_rand = 25;
-
-    spell_casted = ((temp_rand > 19) ? SPELL_FIREBALL :
-                    (temp_rand > 16) ? SPELL_BOLT_OF_FIRE :
-                    (temp_rand > 13) ? SPELL_BOLT_OF_COLD :
-                    (temp_rand > 11) ? SPELL_LIGHTNING_BOLT :
-                    (temp_rand > 10) ? SPELL_LEHUDIBS_CRYSTAL_SPEAR :
-                    (temp_rand >  9) ? SPELL_VENOM_BOLT :
-                    (temp_rand >  8) ? SPELL_BOLT_OF_DRAINING :
-                    (temp_rand >  7) ? SPELL_BOLT_OF_INACCURACY :
-                    (temp_rand >  6) ? SPELL_STICKY_FLAME :
-                    (temp_rand >  5) ? SPELL_TELEPORT_SELF :
-                    (temp_rand >  4) ? SPELL_CIGOTUVIS_DEGENERATION :
-                    (temp_rand >  3) ? SPELL_POLYMORPH_OTHER :
-                    (temp_rand >  2) ? SPELL_MEPHITIC_CLOUD :
-                    (temp_rand >  1) ? SPELL_THROW_FLAME :
-                    (temp_rand >  0) ? SPELL_THROW_FROST
-                                     : SPELL_MAGIC_DART);
-
-    your_spells( spell_casted, powc, false );
-}                               // end tome_of_power()
-
-void skill_manual(char sc_read_2)
-{
-    set_ident_flags( you.inv[sc_read_2], ISFLAG_IDENT_MASK );
-
-    const int skill = you.inv[sc_read_2].plus;
-
-    const char* skname = skill_name(skill);
-    mprf("This is a manual of %s!", skname);
-
-    you.turn_is_over = true;
-
-    if (!yesno("Read it?"))
-        return;
-
-    mprf("You read about %s.", skname);
-
-    exercise(skill, 500);
-
-    if (one_chance_in(10))
-    {
-        mpr("The book crumbles into dust.");
-        dec_inv_item_quantity( sc_read_2, 1 );
     }
     else
     {
-        mpr("The book looks somewhat more worn.");
+        viewwindow(true, false);
+
+        int temp_rand = random2(23) + random2(you.skills[SK_EVOCATIONS] / 3);
+
+        if (temp_rand > 25)
+            temp_rand = 25;
+
+        const spell_type spell_casted =
+            ((temp_rand > 24) ? SPELL_LEHUDIBS_CRYSTAL_SPEAR :
+             (temp_rand > 21) ? SPELL_BOLT_OF_FIRE :
+             (temp_rand > 18) ? SPELL_BOLT_OF_COLD :
+             (temp_rand > 16) ? SPELL_LIGHTNING_BOLT :
+             (temp_rand > 10) ? SPELL_FIREBALL :
+             (temp_rand >  9) ? SPELL_VENOM_BOLT :
+             (temp_rand >  8) ? SPELL_BOLT_OF_DRAINING :
+             (temp_rand >  7) ? SPELL_BOLT_OF_INACCURACY :
+             (temp_rand >  6) ? SPELL_STICKY_FLAME :
+             (temp_rand >  5) ? SPELL_TELEPORT_SELF :
+             (temp_rand >  4) ? SPELL_CIGOTUVIS_DEGENERATION :
+             (temp_rand >  3) ? SPELL_POLYMORPH_OTHER :
+             (temp_rand >  2) ? SPELL_MEPHITIC_CLOUD :
+             (temp_rand >  1) ? SPELL_THROW_FLAME :
+             (temp_rand >  0) ? SPELL_THROW_FROST
+                              : SPELL_MAGIC_DART);
+
+        your_spells( spell_casted, powc, false );
     }
+}
+
+void skill_manual(int slot)
+{
+    // Removed confirmation request because you know it's
+    // a manual in advance.
+    you.turn_is_over = true;
+    item_def& manual(you.inv[slot]);
+    set_ident_flags( manual, ISFLAG_KNOW_TYPE );
+    const int skill = manual.plus;
+
+    mprf("You read about %s.", skill_name(skill));
+
+    exercise(skill, 500);
+
+    if (--manual.plus2 <= 0)
+    {
+        mpr("The manual crumbles into dust.");
+        dec_inv_item_quantity( slot, 1 );
+    }
+    else
+        mpr("The manual looks somewhat more worn.");
+
     xom_is_stimulated(14);
-}                               // end skill_manual()
+}
 
 static bool box_of_beasts()
 {
-    int beasty = MONS_PROGRAM_BUG;      // error trapping {dlb}
-    int temp_rand = 0;          // probability determination {dlb}
-
-    int ret = false;
+    bool success = false;
 
     mpr("You open the lid...");
 
-    if (random2(100) < 60 + you.skills[SK_EVOCATIONS])
+    if (x_chance_in_y(60 + you.skills[SK_EVOCATIONS], 100))
     {
-        temp_rand = random2(11);
+        monster_type beasty = MONS_PROGRAM_BUG;
 
-        beasty = ((temp_rand == 0) ? MONS_GIANT_BAT :
-                  (temp_rand == 1) ? MONS_HOUND :
-                  (temp_rand == 2) ? MONS_JACKAL :
-                  (temp_rand == 3) ? MONS_RAT :
-                  (temp_rand == 4) ? MONS_ICE_BEAST :
-                  (temp_rand == 5) ? MONS_SNAKE :
-                  (temp_rand == 6) ? MONS_YAK :
-                  (temp_rand == 7) ? MONS_BUTTERFLY :
-                  (temp_rand == 8) ? MONS_HELL_HOUND :
-                  (temp_rand == 9) ? MONS_BROWN_SNAKE 
-                                   : MONS_GIANT_LIZARD);
-
-        beh_type beh = (one_chance_in(you.skills[SK_EVOCATIONS] + 5)
-                        ? BEH_HOSTILE : BEH_FRIENDLY);
-
-        if (create_monster( beasty, 2 + random2(4), beh,
-                            you.x_pos, you.y_pos, MHITYOU, 250 ) != -1)
+        // If you worship a good god, don't summon an evil beast (in
+        // this case, the hell hound).
+        do
         {
+            int temp_rand = random2(11);
+
+            beasty = ((temp_rand == 0) ? MONS_GIANT_BAT :
+                      (temp_rand == 1) ? MONS_HOUND :
+                      (temp_rand == 2) ? MONS_JACKAL :
+                      (temp_rand == 3) ? MONS_RAT :
+                      (temp_rand == 4) ? MONS_ICE_BEAST :
+                      (temp_rand == 5) ? MONS_SNAKE :
+                      (temp_rand == 6) ? MONS_YAK :
+                      (temp_rand == 7) ? MONS_BUTTERFLY :
+                      (temp_rand == 8) ? MONS_BROWN_SNAKE :
+                      (temp_rand == 9) ? MONS_GIANT_LIZARD
+                                       : MONS_HELL_HOUND);
+        }
+        while (player_will_anger_monster(beasty));
+
+        beh_type beha = BEH_FRIENDLY;
+        unsigned short hitting = you.pet_target;
+
+        if (one_chance_in(you.skills[SK_EVOCATIONS] + 5))
+        {
+            beha = BEH_HOSTILE;
+            hitting = MHITYOU;
+        }
+
+        if (create_monster(
+                mgen_data(beasty, beha, 2 + random2(4),
+                          you.pos(), hitting)) != -1)
+        {
+            success = true;
+
             mpr("...and something leaps out!");
             xom_is_stimulated(14);
-            ret = true;
         }
     }
     else
@@ -1006,8 +1013,8 @@ static bool box_of_beasts()
         }
     }
 
-    return (ret);
-}                               // end box_of_beasts()
+    return (success);
+}
 
 static bool ball_of_energy(void)
 {
@@ -1022,9 +1029,9 @@ static bool ball_of_energy(void)
 
     if (use < 2 || you.max_magic_points == 0)
     {
-        lose_stat(STAT_INTELLIGENCE, 1);
+        lose_stat(STAT_INTELLIGENCE, 1, false, "using a ball of energy");
     }
-    else if ((use < 4 && enough_mp(1, true))
+    else if (use < 4 && enough_mp(1, true)
              || you.magic_points == you.max_magic_points)
     {
         mpr( "You feel your power drain away!" );
@@ -1039,7 +1046,7 @@ static bool ball_of_energy(void)
         proportional = you.magic_points * 100;
         proportional /= you.max_magic_points;
 
-        if (random2avg(77 - you.skills[SK_EVOCATIONS] * 2, 4) > proportional 
+        if (random2avg(77 - you.skills[SK_EVOCATIONS] * 2, 4) > proportional
             || one_chance_in(25))
         {
             mpr( "You feel your power drain away!" );
@@ -1055,15 +1062,16 @@ static bool ball_of_energy(void)
     }
 
     return (ret);
-}                               // end ball_of_energy()
+}
 
 static bool ball_of_fixation(void)
 {
     mpr("You gaze into the crystal ball.");
     mpr("You are mesmerised by a rainbow of scintillating colours!");
 
-    you.duration[DUR_PARALYSIS] = 100;
-    you.duration[DUR_SLOW] = 100;
+    const int duration = random_range(15, 40);
+    you.duration[DUR_PARALYSIS] = duration;
+    you.duration[DUR_SLOW] = duration;
 
     return (true);
-}                               // end ball_of_fixation()
+}
