@@ -1,8 +1,10 @@
 /*
- *  File:       it_use3.cc
+ *  File:       it_use3.h
  *  Summary:    Functions for using some of the wackier inventory items.
  *  Written by: Linley Henzell
  *
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-03-08T10:44:09.947052Z $
+ *
  *  Change History (most recent first):
  *
  *               <1>     -/--/--        LRH             Created
@@ -17,28 +19,28 @@
 /* ***********************************************************************
  * called from: item_use
  * *********************************************************************** */
-void skill_manual(char sc_read_2);
+void skill_manual(int slot);
 
 
 // last updated 12may2000 {dlb}
 /* ***********************************************************************
  * called from: item_use - spl-book
  * *********************************************************************** */
-void tome_of_power(char sc_read_2);
+void tome_of_power(int slot);
 
 
 // last updated 12may2000 {dlb}
 /* ***********************************************************************
  * called from: acr
  * *********************************************************************** */
-bool evoke_wielded(void);
+bool evoke_wielded();
 
 
 // last updated 12may2000 {dlb}
 /* ***********************************************************************
  * called from: acr
  * *********************************************************************** */
-void special_wielded(void);
+void special_wielded();
 
 
 #endif
