@@ -1,9 +1,9 @@
 /*
- *  File:       item_use.cc
+ *  File:       item_use.h
  *  Summary:    Functions for making use of inventory items.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-15T20:43:17.007642Z $
  *
  *  Change History (most recent first):
  *
@@ -20,6 +20,12 @@
 #include "externs.h"
 #include "enum.h"
 
+enum enchant_stat_type
+{
+    ENCHANT_TO_HIT,
+    ENCHANT_TO_DAM
+};
+
 enum fire_type
 {
     FIRE_NONE      = 0x0000,
@@ -33,7 +39,8 @@ enum fire_type
     FIRE_CLUB      = 0x0080,
     FIRE_ROCK      = 0x0100,
     FIRE_NET       = 0x0200,
-    FIRE_RETURNING = 0x0400
+    FIRE_RETURNING = 0x0400,
+    FIRE_INSCRIBED = 0x0800    // Only used for _get_fire_order
 };
 
 struct bolt;
@@ -58,10 +65,13 @@ bool takeoff_armour(int index);
 /* ***********************************************************************
  * called from: acr
  * *********************************************************************** */
-void drink(void);
+void drink(int slot = -1);
 
 bool elemental_missile_beam(int launcher_brand, int ammo_brand);
 
+bool safe_to_remove_or_wear(const item_def &item, bool remove,
+                            bool quiet = false);
+
 // last updated 12may2000 {dlb}
 /* ***********************************************************************
  * called from: acr
@@ -74,13 +84,13 @@ void examine_object(void);
  * called from: acr
  * *********************************************************************** */
 bool puton_ring(int slot = -1, bool prompt_finger = true);
-
+void jewellery_remove_effects(item_def &item, bool mesg = true);
 
 // last updated 12may2000 {dlb}
 /* ***********************************************************************
  * called from: acr
  * *********************************************************************** */
-void read_scroll(void);
+void read_scroll(int slot = -1);
 
 
 // last updated 12may2000 {dlb}
@@ -94,23 +104,16 @@ bool remove_ring(int slot = -1, bool announce = false);
 /* ***********************************************************************
  * called from: acr
  * *********************************************************************** */
-int get_fire_item_index(int start_from = 0, bool forward = true,
-                        bool check_quiver = true);
-void shoot_thing(void);
-
+int get_next_fire_item(int current, int offset);
+int get_ammo_to_shoot(int item, dist &target, bool teleport = false);
+void fire_thing(int item = -1);
+void throw_item_no_quiver(void);
 
 // last updated 12may2000 {dlb}
 /* ***********************************************************************
  * called from: acr
  * *********************************************************************** */
-void throw_anything(void);
-
-
-// last updated 12may2000 {dlb}
-/* ***********************************************************************
- * called from: acr
- * *********************************************************************** */
-void wear_armour( void );
+void wear_armour( int slot = -1 );
 
 bool can_wear_armour(const item_def &item, bool verbose, bool ignore_temporary);
 
@@ -139,7 +142,7 @@ bool wield_weapon(bool auto_wield, int slot = -1, bool show_we_messages = true);
 /* ***********************************************************************
  * called from: acr
  * *********************************************************************** */
-void zap_wand(void);
+void zap_wand(int slot = -1);
 
 
 // last updated 15jan2001 {gdl}
@@ -153,19 +156,22 @@ void wield_effects(int item_wield_2, bool showMsgs);
  * called from: delay.cc item_use.cc it_use2.cc
  * *********************************************************************** */
 void use_randart( unsigned char item_wield_2 );
-void use_randart(const item_def &item);
+void use_randart(item_def &item);
 
 bool puton_item(int slot, bool prompt_finger = true);
 
-bool enchant_weapon( int which_stat, bool quiet = false );
+bool enchant_weapon( enchant_stat_type which_stat, bool quiet, item_def &wpn );
+bool enchant_armour( int &ac_change, bool quiet, item_def &arm );
 
-bool throw_it(bolt &pbolt, int throw_2, bool teleport=false, int acc_bonus=0,
-              dist *target = NULL);
+bool throw_it(bolt &pbolt, int throw_2, bool teleport = false,
+              int acc_bonus = 0, dist *target = NULL);
 
-void inscribe_item();
-int launcher_shield_slowdown(const item_def &launcher, 
+bool thrown_object_destroyed( item_def *item, int x, int y, bool returning );
+
+void prompt_inscribe_item();
+int launcher_shield_slowdown(const item_def &launcher,
                              const item_def *shield);
-int launcher_final_speed(const item_def &launcher, 
+int launcher_final_speed(const item_def &launcher,
                          const item_def *shield);
 
 void warn_shield_penalties();
@@ -174,4 +180,11 @@ int item_special_wield_effect(const item_def &item);
 
 bool wearing_slot(int inv_slot);
 
+#ifdef USE_TILE
+/* ***********************************************************************
+ * called from: acr
+ * *********************************************************************** */
+void tile_use_item(int idx, InvAction act);
+#endif
+
 #endif
