@@ -3,6 +3,8 @@
  *  Summary:    Misc functions.
  *  Written by: Brent Ross
  *
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-03T14:41:49.650207Z $
+ *
  *  Change History (most recent first):
  *
  *               <1>     -/--/--        BWR             Created
@@ -16,45 +18,45 @@
 
 enum armour_type
 {
-    ARM_ROBE,                          //    0
+    ARM_ROBE,                    //    0
     ARM_LEATHER_ARMOUR,
     ARM_RING_MAIL,
     ARM_SCALE_MAIL,
     ARM_CHAIN_MAIL,
-    ARM_SPLINT_MAIL,                   //    5
+    ARM_SPLINT_MAIL,             //    5
     ARM_BANDED_MAIL,
     ARM_PLATE_MAIL,
     ARM_SHIELD,
     ARM_CLOAK,
-    ARM_HELMET,                        //   10
+    ARM_HELMET,                  //   10
+    ARM_CAP,
+    ARM_WIZARD_HAT,
     ARM_GLOVES,
     ARM_BOOTS,
-    ARM_BUCKLER,
+    ARM_BUCKLER,                 //   15
     ARM_LARGE_SHIELD,
-    ARM_DRAGON_HIDE,                   //   15
+    ARM_DRAGON_HIDE,
     ARM_TROLL_HIDE,
     ARM_CRYSTAL_PLATE_MAIL,
-    ARM_DRAGON_ARMOUR,
+    ARM_DRAGON_ARMOUR,           //   20
     ARM_TROLL_LEATHER_ARMOUR,
-    ARM_ICE_DRAGON_HIDE,               //   20
+    ARM_ICE_DRAGON_HIDE,
     ARM_ICE_DRAGON_ARMOUR,
     ARM_STEAM_DRAGON_HIDE,
-    ARM_STEAM_DRAGON_ARMOUR,
+    ARM_STEAM_DRAGON_ARMOUR,     //   25
     ARM_MOTTLED_DRAGON_HIDE,
-    ARM_MOTTLED_DRAGON_ARMOUR,         //   25
+    ARM_MOTTLED_DRAGON_ARMOUR,
     ARM_STORM_DRAGON_HIDE,
     ARM_STORM_DRAGON_ARMOUR,
-    ARM_GOLD_DRAGON_HIDE,
+    ARM_GOLD_DRAGON_HIDE,        //   30
     ARM_GOLD_DRAGON_ARMOUR,
-    ARM_ANIMAL_SKIN,                   //   30
+    ARM_ANIMAL_SKIN,
     ARM_SWAMP_DRAGON_HIDE,
     ARM_SWAMP_DRAGON_ARMOUR,
-    ARM_STUDDED_LEATHER_ARMOUR,
-    ARM_CAP,
-    ARM_CENTAUR_BARDING,               //   35
+    ARM_CENTAUR_BARDING,         //   35
     ARM_NAGA_BARDING,
-    
-    NUM_ARMOURS
+
+    NUM_ARMOURS                  //   37
 };
 
 enum armour_property_type
@@ -71,27 +73,34 @@ enum boot_type          // used in pluses2
     NUM_BOOT_TYPES
 };
 
+const int SP_FORBID_EGO   = -1;
+const int SP_FORBID_BRAND = -1;
+
 enum brand_type // equivalent to (you.inv[].special or mitm[].special) % 30
 {
+    SPWPN_FORBID_BRAND = -1,           //   -1
     SPWPN_NORMAL,                      //    0
     SPWPN_FLAMING,
     SPWPN_FREEZING,
     SPWPN_HOLY_WRATH,
     SPWPN_ELECTROCUTION,
     SPWPN_ORC_SLAYING,                 //    5
+    SPWPN_DRAGON_SLAYING,
     SPWPN_VENOM,
     SPWPN_PROTECTION,
     SPWPN_DRAINING,
-    SPWPN_SPEED,
-    SPWPN_VORPAL,                      //   10
-    SPWPN_FLAME,
-    SPWPN_FROST,
+    SPWPN_SPEED,                       //   10
+    SPWPN_VORPAL,
+    SPWPN_FLAME,   // ranged, only
+    SPWPN_FROST,   // ranged, only
     SPWPN_VAMPIRICISM,
-    SPWPN_DISRUPTION,
     SPWPN_PAIN,                        //   15
     SPWPN_DISTORTION,
     SPWPN_REACHING,                    //   17
-    SPWPN_RETURNING,
+
+    MAX_PAN_LORD_BRANDS,               //   18
+
+    SPWPN_RETURNING = MAX_PAN_LORD_BRANDS,
     SPWPN_CONFUSE,
     SPWPN_RANDART_I = 25,              //   25
     SPWPN_RANDART_II,
@@ -102,24 +111,29 @@ enum brand_type // equivalent to (you.inv[].special or mitm[].special) % 30
     SPWPN_DUMMY_CRUSHING,        // ONLY TEMPORARY USAGE -- converts to VORPAL
 
     // everything above this point is a special artefact wield:
-    SPWPN_SINGING_SWORD = 181,          //  181
+    SPWPN_START_FIXEDARTS = 181,
+
+    SPWPN_SINGING_SWORD = SPWPN_START_FIXEDARTS,
     SPWPN_WRATH_OF_TROG,
     SPWPN_SCYTHE_OF_CURSES,
     SPWPN_MACE_OF_VARIABILITY,
-    SPWPN_GLAIVE_OF_PRUNE,              //  185
+    SPWPN_GLAIVE_OF_PRUNE,
     SPWPN_SCEPTRE_OF_TORMENT,
     SPWPN_SWORD_OF_ZONGULDROK,
-
-    // these three are not generated randomly {dlb}
-    SPWPN_SWORD_OF_CEREBOV,
-    SPWPN_STAFF_OF_DISPATER,
-    SPWPN_SCEPTRE_OF_ASMODEUS,          //  190
-
     SPWPN_SWORD_OF_POWER,
     SPWPN_KNIFE_OF_ACCURACY,
     SPWPN_STAFF_OF_OLGREB,
     SPWPN_VAMPIRES_TOOTH,
-    SPWPN_STAFF_OF_WUCAD_MU             //  195
+    SPWPN_STAFF_OF_WUCAD_MU,
+
+    // these three are not generated randomly {dlb}
+    SPWPN_START_NOGEN_FIXEDARTS,
+
+    SPWPN_SWORD_OF_CEREBOV = SPWPN_START_NOGEN_FIXEDARTS,
+    SPWPN_STAFF_OF_DISPATER,
+    SPWPN_SCEPTRE_OF_ASMODEUS,
+
+    SPWPN_END_FIXEDARTS = SPWPN_SCEPTRE_OF_ASMODEUS
 };
 
 enum corpse_type
@@ -131,39 +145,30 @@ enum corpse_type
 enum hands_reqd_type
 {
     HANDS_ONE,
-    HANDS_HALF, 
+    HANDS_HALF,
     HANDS_TWO,
 
     HANDS_DOUBLE        // not a level, marks double ended weapons (== half)
 };
 
-enum helmet_type
+enum helmet_desc_type
 {
-    THELM_HELMET        = 0x0000,
-    THELM_HELM          = 0x0001,
-    THELM_CAP           = 0x0002,
-    THELM_WIZARD_HAT    = 0x0003,
-    THELM_NUM_TYPES     = 4,
-
-    THELM_SPECIAL       = 0x0004,  // type used only for artefacts (mask, hat)
-    THELM_TYPE_MASK     = 0x00ff,
-
-    THELM_DESC_PLAIN    = 0x0000,
-    THELM_DESC_WINGED   = 0x0100,
-    THELM_DESC_HORNED   = 0x0200,
-    THELM_DESC_CRESTED  = 0x0300,
-    THELM_DESC_PLUMED   = 0x0400,
-    THELM_DESC_SPIKED   = 0x0500,
-    THELM_DESC_VISORED  = 0x0600,
-    THELM_DESC_JEWELLED = 0x0700,
-
-    THELM_DESC_MASK     = 0xff00
+    THELM_DESC_PLAIN    = 0,
+    THELM_DESC_WINGED,
+    THELM_DESC_HORNED,
+    THELM_DESC_CRESTED,
+    THELM_DESC_PLUMED,
+    THELM_DESC_MAX_SOFT = THELM_DESC_PLUMED,
+    THELM_DESC_SPIKED,
+    THELM_DESC_VISORED,
+    THELM_DESC_JEWELLED,
+    THELM_NUM_DESCS
 };
 
 enum jewellery_type
 {
     RING_FIRST_RING = 0,
-    
+
     RING_REGENERATION = RING_FIRST_RING, //    0
     RING_PROTECTION,
     RING_PROTECTION_FROM_FIRE,
@@ -203,7 +208,7 @@ enum jewellery_type
     AMU_CONTROLLED_FLIGHT,
     AMU_INACCURACY,
     AMU_RESIST_MUTATION,
-    
+
     NUM_JEWELLERY
 };
 
@@ -240,7 +245,7 @@ enum misc_item_type
     // mixed decks
     MISC_DECK_OF_WAR,
     MISC_DECK_OF_CHANGES,
-    MISC_DECK_OF_DEFENSE,
+    MISC_DECK_OF_DEFENCE,
 
     MISC_RUNE_OF_ZOT,
 
@@ -254,11 +259,11 @@ enum missile_type
     MI_BOLT,
     MI_DART,
     MI_NEEDLE,
-    MI_LARGE_ROCK,
+    MI_LARGE_ROCK,                     //    5
     MI_SLING_BULLET,
     MI_JAVELIN,
     MI_THROWING_NET,
-    NUM_MISSILES,
+    NUM_MISSILES,                      //    9
     MI_NONE             // was MI_EGGPLANT... used for launch type detection
 };
 
@@ -281,7 +286,7 @@ enum rune_type
     RUNE_DEMONIC                = 50,
     RUNE_ABYSSAL,
 
-    // Runes 60-63 correspond to the Pandemonium demonlords, 
+    // Runes 60-63 correspond to the Pandemonium demonlords,
     // and are equal to the corresponding vault.
     RUNE_MNOLEG                 = 60,
     RUNE_LOM_LOBON,
@@ -310,17 +315,19 @@ enum scroll_type
     SCR_BLINKING,
     SCR_PAPER,                         //   15
     SCR_MAGIC_MAPPING,
-    SCR_FORGETFULNESS,
+    SCR_FOG,
     SCR_ACQUIREMENT,
     SCR_ENCHANT_WEAPON_II,
     SCR_VORPALISE_WEAPON,              //   20
     SCR_RECHARGING,
     SCR_ENCHANT_WEAPON_III,
+    SCR_HOLY_WORD,
     NUM_SCROLLS
 };
 
 enum special_armour_type
 {
+    SPARM_FORBID_EGO = -1,             //   -1
     SPARM_NORMAL,                      //    0
     SPARM_RUNNING,
     SPARM_FIRE_RESISTANCE,
@@ -339,25 +346,22 @@ enum special_armour_type
     SPARM_RESISTANCE,                  //   15
     SPARM_POSITIVE_ENERGY,
     SPARM_ARCHMAGI,
-    SPARM_PRESERVATION,                //   18
-    SPARM_RANDART_I = 25, // must remain at 25 for now - how high do they go? {dlb}
-    SPARM_RANDART_II = 26,             //   26
-    SPARM_RANDART_III = 27,            //   27
-    SPARM_RANDART_IV = 28,             //   28
-    SPARM_RANDART_V = 29 //   29 - highest value found thus far {dlb}
+    SPARM_PRESERVATION                //   18
 };
 
 enum special_missile_type // to separate from weapons in general {dlb}
 {
+    SPMSL_FORBID_BRAND = -1,           //   -1
     SPMSL_NORMAL,                      //    0
     SPMSL_FLAME,                       //    1
     SPMSL_ICE,                         //    2
     SPMSL_POISONED,                    //    3
     SPMSL_POISONED_II,                 //    4 - unused
-    SPMSL_CURARE                       //    5
+    SPMSL_CURARE,                      //    5
+    SPMSL_RETURNING                    //    6
 };
 
-enum special_ring_type // jewellery mitm[].special values 
+enum special_ring_type // jewellery mitm[].special values
 {
     SPRING_RANDART = 200,
     SPRING_UNRANDART = 201
@@ -395,7 +399,7 @@ enum stave_type
     STAFF_SUMMONING,
     STAFF_AIR,
     STAFF_EARTH,
-    STAFF_CHANNELING,
+    STAFF_CHANNELING,           // 12
     // rods
     STAFF_SMITING,              // must be first rod!
     STAFF_SPELL_SUMMONING,
@@ -407,8 +411,8 @@ enum stave_type
     STAFF_DISCOVERY,
     STAFF_DEMONOLOGY,
     STAFF_STRIKING,
-    STAFF_VENOM,
-    NUM_STAVES                         // must remain last member {dlb}
+    STAFF_VENOM,                // 23
+    NUM_STAVES                  // must remain last member {dlb}
 };
 
 enum weapon_type
@@ -445,7 +449,7 @@ enum weapon_type
     WPN_DOUBLE_SWORD,
     WPN_TRIPLE_SWORD,
     WPN_HAMMER,
-    WPN_ANCUS,
+    WPN_ANKUS,
     WPN_WHIP,                          //   30
     WPN_SABRE,
     WPN_DEMON_BLADE,
@@ -461,12 +465,19 @@ enum weapon_type
     WPN_KNIFE,
     WPN_BLOWGUN,
     WPN_FALCHION,
-    WPN_BLESSED_BLADE,                 //   44
-    WPN_LONGBOW,
+    WPN_BLESSED_EUDEMON_BLADE,
+    WPN_LONGBOW,                       //   45
     WPN_LAJATANG,
-    WPN_LOCHABER_AXE,
+    WPN_BARDICHE,
+    WPN_BLESSED_FALCHION,
+    WPN_BLESSED_LONG_SWORD,
+    WPN_BLESSED_SCIMITAR,              //   50
+    WPN_BLESSED_KATANA,
+    WPN_BLESSED_DOUBLE_SWORD,
+    WPN_BLESSED_GREAT_SWORD,
+    WPN_BLESSED_TRIPLE_SWORD,
 
-    NUM_WEAPONS,                       //   48 - must be last regular member {dlb}
+    NUM_WEAPONS,                       //   55 - must be last regular member {dlb}
 
 // special cases
     WPN_UNARMED = 500,                 //  500
@@ -494,8 +505,8 @@ enum weapon_property_type
 
 enum vorpal_damage_type
 {
-    // Types of damage a weapon can do... currently assuming that anything 
-    // with BLUDGEON always does "AND" with any other specified types, 
+    // Types of damage a weapon can do... currently assuming that anything
+    // with BLUDGEON always does "AND" with any other specified types,
     // and and sets not including BLUDGEON are "OR".
     DAM_BASH            = 0x0000,       // non-melee weapon blugeoning
     DAM_BLUDGEON        = 0x0001,       // crushing
@@ -511,7 +522,7 @@ enum vorpal_damage_type
     DVORP_CHOPPING      = 0x4000,       // used for axes
     DVORP_SLASHING      = 0x5000,       // used for whips
     DVORP_STABBING      = 0x6000,       // used for knives/daggers
-    
+
     DVORP_CLAWING       = 0x7000,       // claw damage
 
     // These are shortcuts to tie vorpal/damage types for easy setting...
@@ -528,8 +539,9 @@ enum vorpal_damage_type
     DAMV_MASK           = 0xf000        // strips non-vorpal specification
 };
 
-// NOTE:  This order is very special!  Its basically the same as ZAP_*, 
-// and there are bits of the code that still use that fact.. see zap_wand().
+// NOTE:  This order is very special!  Its basically the same as ZAP_*,
+// and there are bits of the code that still use that fact.
+// See item_def::zap().
 enum wand_type                         // mitm[].subtype
 {
     WAND_FLAME,                        //    0
@@ -570,8 +582,8 @@ bool item_is_critical(const item_def &item);
 bool item_cursed( const item_def &item );
 bool item_known_cursed( const item_def &item );
 bool item_known_uncursed( const item_def &item );
-void do_curse_item(  item_def &item );
-void do_uncurse_item(  item_def &item );
+void do_curse_item( item_def &item, bool quiet = true );
+void do_uncurse_item( item_def &item );
 
 // stationary:
 void set_item_stationary( item_def &item );
@@ -592,17 +604,16 @@ unsigned long get_equip_race( const item_def &item );
 unsigned long get_equip_desc( const item_def &item );
 
 // helmet functions:
-void  set_helmet_type( item_def &item, short flags );
 void  set_helmet_desc( item_def &item, short flags );
 void  set_helmet_random_desc( item_def &item );
 
-short get_helmet_type( const item_def &item );
 short get_helmet_desc( const item_def &item );
 
-bool  is_helmet_type( const item_def &item, short val );
+bool  is_helmet( const item_def& item );
+bool  is_hard_helmet( const item_def& item );
 
 // ego items:
-bool set_item_ego_type( item_def &item, int item_type, int ego_type ); 
+bool set_item_ego_type( item_def &item, int item_type, int ego_type );
 int  get_weapon_brand( const item_def &item );
 special_armour_type get_armour_ego_type( const item_def &item );
 int  get_ammo_brand( const item_def &item );
@@ -626,6 +637,10 @@ int   fit_armour_size( const item_def &item, size_type size );
 bool  check_armour_size( const item_def &item, size_type size );
 bool  check_armour_shape( const item_def &item, bool quiet );
 
+bool item_is_rechargable(const item_def &it, bool known = false);
+bool is_enchantable_weapon(const item_def &wpn, bool uncurse);
+bool is_enchantable_armour(const item_def &arm, bool uncurse);
+
 // weapon functions:
 int weapon_rarity( int w_type );
 
@@ -652,11 +667,14 @@ bool is_double_ended( const item_def &item );
 
 int double_wpn_awkward_speed( const item_def &item );
 
-bool  is_demonic( const item_def &item );
+bool is_demonic(const item_def &item);
+bool is_blessed_blade(const item_def &item);
+bool is_convertible(const item_def &item);
+bool convert2good(item_def &item, bool allow_blessed = true);
 
-int   get_vorpal_type( const item_def &item );
-int   get_damage_type( const item_def &item );
-bool  does_damage_type( const item_def &item, int dam_type );
+int get_vorpal_type( const item_def &item );
+int get_damage_type( const item_def &item );
+bool does_damage_type( const item_def &item, int dam_type );
 
 int weapon_str_weight( const item_def &wpn );
 int weapon_dex_weight( const item_def &wpn );
@@ -676,8 +694,9 @@ missile_type  fires_ammo_type( const item_def &item );
 missile_type fires_ammo_type( weapon_type wtype );
 const char *  ammo_name( const item_def &bow );
 const char *  ammo_name( missile_type mtyp );
+bool          has_launcher( const item_def &ammo );
 bool          is_throwable( const item_def &wpn,
-                            size_type bodysize = SIZE_MEDIUM ); 
+                            size_type bodysize = SIZE_MEDIUM );
 launch_retval is_launched(actor *actor, const item_def *launcher,
                          const item_def &missile);
 
@@ -691,9 +710,11 @@ int ring_has_pluses( const item_def &item );
 // food functions:
 bool food_is_meat( const item_def &item );
 bool food_is_veg( const item_def &item );
+bool is_blood_potion( const item_def &item );
 int  food_value( const item_def &item );
 int  food_turns( const item_def &item );
 bool can_cut_meat( const item_def &item );
+bool food_is_rotten( const item_def &item );
 
 // generic item property functions:
 bool is_tool( const item_def &item );
@@ -706,7 +727,7 @@ size_type item_size( const item_def &item );
 bool is_colourful_item( const item_def &item );
 
 bool is_shield(const item_def &item);
-bool is_shield_incompatible(const item_def &weapon, 
+bool is_shield_incompatible(const item_def &weapon,
                             const item_def *shield = NULL);
 
 // Only works for armour/weapons/missiles
