@@ -1,9 +1,9 @@
 /*
- *  File:       items.cc
+ *  File:       items.h
  *  Summary:    Misc (mostly) inventory related functions.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-06-29T14:14:04.027747Z $
  *
  *  Change History (most recent first):
  *
@@ -21,9 +21,9 @@
 enum acquirement_agent_type
 {
     AQ_SCROLL   = 0,
-    
+
     // Empty space for the gods
-    
+
     AQ_CARD_GENIE = 100,
 
     AQ_WIZMODE          = 200
@@ -42,10 +42,12 @@ void move_item_stack_to_grid( int x, int y, int targ_x, int targ_y );
 int  move_item_to_player( int obj, int quant_got, bool quiet = false );
 void mark_items_non_pickup_at(const coord_def &pos);
 bool is_stackable_item( const item_def &item );
+bool items_similar( const item_def &item1, const item_def &item2 );
 bool items_stack( const item_def &item1, const item_def &item2,
-                  bool force = false );
+                  bool force_merge = false );
 
 item_def find_item_type(object_class_type base_type, std::string name);
+item_def *find_floor_item(object_class_type cls, int sub_type);
 
 void init_item( int item );
 
@@ -61,12 +63,6 @@ void link_items(void);
  * *********************************************************************** */
 void fix_item_coordinates(void);
 
-// last updated: 19apr2001 {gdl}
-/* ***********************************************************************
- * called from: dungeon
- * *********************************************************************** */
-int cull_items(void);
-
 // last updated: 16oct2001 -- bwr
 int get_item_slot( int reserve = 50 );
 
@@ -76,8 +72,10 @@ int get_item_slot( int reserve = 50 );
  *              religion - spells2 - spells3 - spells4
  * *********************************************************************** */
 void unlink_item(int dest);
+void destroy_item( item_def &item, bool never_created = false );
 void destroy_item(int dest, bool never_created = false);
-void destroy_item_stack( int x, int y );
+void destroy_item_stack( int x, int y, int cause = -1 );
+void lose_item_stack( int x, int y );
 
 // last updated: 08jun2000 {dlb}
 /* ***********************************************************************
@@ -97,7 +95,7 @@ void pickup(void);
 /* ***********************************************************************
  * called from: beam - items - transfor
  * *********************************************************************** */
-bool copy_item_to_grid( const item_def &item, int x_plos, int y_plos, 
+bool copy_item_to_grid( const item_def &item, int x_plos, int y_plos,
                         int quant_drop = -1,    // item.quantity by default
                         bool mark_dropped = false);
 
@@ -105,7 +103,7 @@ bool copy_item_to_grid( const item_def &item, int x_plos, int y_plos,
 /* ***********************************************************************
  * called from: spells4.cc
  * *********************************************************************** */
-bool move_top_item( int src_x, int src_y, int dest_x, int dest_y );
+bool move_top_item( const coord_def &src, const coord_def &dest );
 
 // last updated: 08jun2000 {dlb}
 /* ***********************************************************************
@@ -113,31 +111,12 @@ bool move_top_item( int src_x, int src_y, int dest_x, int dest_y );
  * *********************************************************************** */
 void drop(void);
 
-
-// last updated: 08jun2000 {dlb}
-/* ***********************************************************************
- * called from: files - items
- * *********************************************************************** */
-void update_corpses(double elapsedTime);
-void update_level(double elapsedTime);
-
-
-// last updated: 08jun2000 {dlb}
-/* ***********************************************************************
- * called from: acr
- * *********************************************************************** */
-void handle_time( long time_delta );
-
 // last updated: 08jun2000 {dlb}
 /* ***********************************************************************
  * called from: command food item_use shopping spl-book transfor
  * *********************************************************************** */
 int inv_count(void);
 
-void show_items();
-
-void cmd_destroy_item( void );
-
 bool pickup_single_item(int link, int qty);
 
 bool drop_item( int item_dropped, int quant_drop, bool try_offer = false );
@@ -146,7 +125,6 @@ int get_equip_slot(const item_def *item);
 
 void origin_set(int x, int y);
 void origin_set_monster(item_def &item, const monsters *monster);
-void origin_freeze(item_def &item, int x, int y);
 bool origin_known(const item_def &item);
 bool origin_describable(const item_def &item);
 std::string origin_desc(const item_def &item);
@@ -155,6 +133,7 @@ void origin_acquired(item_def &item, int agent);
 void origin_set_startequip(item_def &item);
 void origin_set_unknown(item_def &item);
 void origin_set_inventory( void (*oset)(item_def &item) );
+std::string origin_monster_name(const item_def &item);
 
 bool item_needs_autopickup(const item_def &);
 bool can_autopickup();
@@ -164,9 +143,15 @@ void autopickup();
 
 int find_free_slot(const item_def &i);
 bool is_rune(const item_def &item);
+bool is_unique_rune(const item_def &item);
 
 bool need_to_autoinscribe();
 void request_autoinscribe(bool do_inscribe = true);
 void autoinscribe();
 
+bool item_is_equipped(const item_def &item);
+
+void item_was_lost(const item_def &item);
+void item_was_destroyed(const item_def &item, int cause = -1);
+
 #endif
