@@ -3,7 +3,7 @@
  *  Summary:    Functions used in Pandemonium.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dolorous $ on $Date: 2008-07-04T01:27:02.490149Z $
  *
  *  Change History (most recent first):
  *
@@ -11,10 +11,13 @@
  */
 
 #include "AppHdr.h"
+
+#include <algorithm>
+
 #include "lev-pand.h"
 
 #include "externs.h"
-
+#include "dungeon.h"
 #include "monplace.h"
 #include "mon-pick.h"
 #include "stuff.h"
@@ -22,11 +25,10 @@
 void init_pandemonium(void)
 {
     int pc = 0;
-    struct monsters *monster = 0;       // NULL
 
-    for (pc = 0; pc < MAX_MONSTERS; pc++)
+    for (pc = 0; pc < MAX_MONSTERS; ++pc)
     {
-        monster = &menv[pc];
+        monsters *monster = &menv[pc];
 
         // Looks for unique demons and sets appropriate lists of demons.
         // NB - also sets the level colours.
@@ -91,10 +93,10 @@ void init_pandemonium(void)
         }
     }
 
-// colour of monster 9 is colour of floor, 8 is colour of rock
-// IIRC, BLACK is set to LIGHTGRAY
+    // colour of monster 9 is colour of floor, 8 is colour of rock
+    // IIRC, BLACK is set to LIGHTGREY
 
-    for (pc = 0; pc < 10; pc++)
+    for (pc = 0; pc < 10; ++pc)
     {
         switch (random2(17))
         {
@@ -113,7 +115,10 @@ void init_pandemonium(void)
         }
 
         if (one_chance_in(10))
-            env.mons_alloc[pc] = MONS_HELLION + random2(10);
+        {
+            env.mons_alloc[pc] =
+                static_cast<monster_type>(MONS_HELLION + random2(10));
+        }
 
         if (one_chance_in(30))
             env.mons_alloc[pc] = MONS_RED_DEVIL;
@@ -122,17 +127,29 @@ void init_pandemonium(void)
             env.mons_alloc[pc] = MONS_IMP;
 
         if (one_chance_in(20))
-            env.mons_alloc[pc] = MONS_DEMONIC_CRAWLER + random2(5);
+        {
+            env.mons_alloc[pc] =
+                static_cast<monster_type>(MONS_DEMONIC_CRAWLER + random2(5));
+        }
     }
 
     if (one_chance_in(8))
-        env.mons_alloc[7] = MONS_EXECUTIONER + random2(5);
+    {
+        env.mons_alloc[7] =
+            static_cast<monster_type>(MONS_EXECUTIONER + random2(5));
+    }
 
     if (one_chance_in(5))
-        env.mons_alloc[8] = MONS_EXECUTIONER + random2(5);
+    {
+        env.mons_alloc[8] =
+            static_cast<monster_type>(MONS_EXECUTIONER + random2(5));
+    }
 
     if (one_chance_in(3))
-        env.mons_alloc[9] = MONS_EXECUTIONER + random2(5);
+    {
+        env.mons_alloc[9] =
+            static_cast<monster_type>(MONS_EXECUTIONER + random2(5));
+    }
 
     if (one_chance_in(10))
         env.mons_alloc[7 + random2(3)] = MONS_FIEND;
@@ -146,23 +163,27 @@ void init_pandemonium(void)
     if (one_chance_in(10))
         env.mons_alloc[7 + random2(3)] = MONS_PIT_FIEND;
 
-    // set at least some specific monsters for the special levels - this
-    // can also be used to set some colours
-}                               // end init_pandemonium()
+    // Set at least some specific monsters for the special levels - this
+    // can also be used to set some colours.
+
+    env.floor_colour = BLACK;
+    env.rock_colour  = BLACK;
+    dgn_set_colours_from_monsters();
+}
 
 void pandemonium_mons(void)
 {
-    // must leave allowance for monsters rare on pandemonium (eg wizards etc)
+    // must leave allowance for monsters rare on pandemonium (wizards, etc.)
     int pan_mons = env.mons_alloc[random2(10)];
 
     if (one_chance_in(40))
     {
         do
-        {
             pan_mons = random2(NUM_MONSTERS);   // was random2(400) {dlb}
-        }
         while (!mons_pan(pan_mons));
     }
-    mons_place(pan_mons, BEH_HOSTILE, MHITNOT, false, 50,50,
-               LEVEL_PANDEMONIUM);
-}                               // end pandemonium_mons()
+    mgen_data mg(static_cast<monster_type>(pan_mons));
+    mg.level_type = LEVEL_PANDEMONIUM;
+
+    mons_place(mg);
+}
