@@ -3,7 +3,9 @@
  *  Summary:    Functions for DOS support.
  *  Written by: Darshan Shaligram
  *
- *  Added for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-22T16:20:45.803473Z $
+ *  Added for Crawl Reference by dshaligram on Wed Nov 22 08:41:20 2006 UTC
+ *
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-03-08T10:44:09.947052Z $
  */
 
 // Every .cc must include AppHdr or bad things happen.
@@ -43,12 +45,12 @@ static void scroll_message_window()
     movetext(crawl_view.msgp.x, crawl_view.msgp.y + 1,
              get_number_of_cols(), get_number_of_lines(),
              crawl_view.msgp.x, crawl_view.msgp.y);
-    gotoxy(1, get_number_of_lines());
+    cgotoxy(1, get_number_of_lines());
     clreol();
 
     // Cursor also scrolls up so prompts don't look brain-damaged.
     if (y == get_number_of_lines())
-        gotoxy(x, y - 1);
+        cgotoxy(x, y - 1);
 }
 
 void message_out(int which_line, int colour, const char *s, int firstcol,
@@ -57,7 +59,7 @@ void message_out(int which_line, int colour, const char *s, int firstcol,
     if (!firstcol)
         firstcol = Options.delay_message_clear? 2 : 1;
 
-    gotoxy(firstcol + crawl_view.msgp.x - 1,
+    cgotoxy(firstcol + crawl_view.msgp.x - 1,
            which_line + crawl_view.msgp.y);
     textcolor(colour);
 
