@@ -1,3 +1,13 @@
+/*
+ *  File:       libdos.h
+ *  Summary:    Functions for DOS support.
+ *  Written by: Darshan Shaligram
+ *
+ *  Added for Crawl Reference by dshaligram on Wed Nov 22 08:41:20 2006 UTC
+ *
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-03-08T10:44:09.947052Z $
+ */
+
 #ifndef __LIBDOS_H__
 #define __LIBDOS_H__
 
@@ -10,6 +20,7 @@ void init_libdos();
 int get_number_of_lines();
 int get_number_of_cols();
 
+inline void gotoxy_sys(int x, int y) { gotoxy(x, y); }
 inline void enable_smart_cursor(bool ) { }
 inline bool is_smart_cursor_enabled()  { return (false); }
 void set_cursor_enabled(bool enabled);
