@@ -3,7 +3,7 @@
  *  Summary:    Functions for unix and curses support
  *  Written by: ?
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-04T13:13:40.045599Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-06T20:14:35.747799Z $
  *
  *  Change History (most recent first):
  *
@@ -28,7 +28,7 @@
    The colour exchange system is perhaps a little overkill, but I wanted
    it to be general to support future changes.. The only thing not
    supported properly is black on black (used by curses for "normal" mode)
-   and white on white (used by me for "bright black" (darkgray) on black
+   and white on white (used by me for "bright black" (darkgrey) on black
 
    Jan 1998 Svante Gerhard <svante@algonet.se>                          */
 
@@ -54,6 +54,7 @@
 #ifdef UNICODE_GLYPHS
 #include <wchar.h>
 #include <locale.h>
+#include <langinfo.h>
 #endif
 
 #include <termios.h>
@@ -75,9 +76,6 @@ static struct termios game_term;
     #include CURSES_INCLUDE_FILE
 #endif
 
-// Character set variable
-static int Character_Set = CHARACTER_SET;
-
 // Globals holding current text/backg. colors
 static short FG_COL = WHITE;
 static short BG_COL = BLACK;
@@ -85,6 +83,7 @@ static int   Current_Colour = COLOR_PAIR(BG_COL * 8 + FG_COL);
 
 static int curs_fg_attr(int col);
 static int curs_bg_attr(int col);
+static int waddstr_with_altcharset(WINDOW* w, const char* s);
 
 static bool cursor_is_enabled = true;
 
@@ -149,16 +148,6 @@ static short translate_colour( short col )
     }
 }
 
-void set_altcharset( bool alt_on )
-{
-    Character_Set = ((alt_on) ? A_ALTCHARSET : 0);
-}
-
-bool get_altcharset( void )
-{
-    return (Character_Set != 0);
-}
-
 static void setup_colour_pairs( void )
 {
     short i, j;
@@ -206,7 +195,7 @@ static void termio_init()
     def_term.c_cc[VINTR] = (char) 3;        // ctrl-C
     game_term.c_cc[VINTR] = (char) 3;       // ctrl-C
 
-    // Lets recover some control sequences
+    // Let's recover some control sequences
     game_term.c_cc[VSTART] = (char) -1;     // ctrl-Q
     game_term.c_cc[VSTOP] = (char) -1;      // ctrl-S
     game_term.c_cc[VSUSP] = (char) -1;      // ctrl-Y
@@ -265,7 +254,7 @@ static int proc_mouse_event(int c, const MEVENT *me)
         cme.bstate |= c_mouse_event::BUTTON4;
     else if (me->bstate & BUTTON4_DOUBLE_CLICKED)
         cme.bstate |= c_mouse_event::BUTTON4_DBL;
-    
+
     if (cme)
     {
         new_mouse_event(cme);
@@ -290,7 +279,10 @@ static int raw_m_getch()
     }
 #endif
     default:
-        return (c);
+        // getch() returns -1 on EOF, convert that into an Escape. Evil hack,
+        // but the alternative is to explicitly check for -1 everywhere where
+        // we might otherwise spin in a tight keyboard input loop.
+        return (c == -1? ESCAPE : c);
     }
 }
 
@@ -345,7 +337,7 @@ static void handle_hangup(int)
 
     if (crawl_state.saving_game || crawl_state.updating_scores)
         return;
-    
+
     crawl_state.saving_game = true;
     if (crawl_state.need_save)
     {
@@ -376,7 +368,7 @@ static void setup_message_window()
         // Never reaches here unless the game didn't need saving.
         end(1, false, "Unable to create message window!");
     }
-    
+
     scrollok(Message_Window, true);
     idlok(Message_Window, true);
 }
@@ -404,7 +396,8 @@ void message_out(int which_line, int color, const char *s, int firstcol,
     else
         firstcol--;
 
-    mvwaddstr(Message_Window, which_line, firstcol, s);
+    wmove(Message_Window, which_line, firstcol);
+    waddstr_with_altcharset(Message_Window, s);
 
     if (newline && which_line == crawl_view.msgsz.y - 1)
     {
@@ -443,7 +436,7 @@ void unixcurses_startup( void )
 #endif
 
     signal(SIGWINCH, handle_sigwinch);
-    
+
 #endif
 
     initscr();
@@ -479,7 +472,7 @@ void unixcurses_shutdown()
         delwin(Message_Window);
         Message_Window = NULL;
     }
-        
+
     // resetty();
     endwin();
 
@@ -552,11 +545,51 @@ int cprintf(const char *format,...)
     va_start(argp, format);
     vsprintf(buffer, format, argp);
     va_end(argp);
-    i = addstr(buffer);
+    i = waddstr_with_altcharset(stdscr, buffer);
     refresh();
     return (i);
 }
 
+// Wrapper around curses waddstr(); handles switching to the alt charset
+// when necessary, and performing the funny DEC translation.
+// Returns a curses success value.
+static int waddstr_with_altcharset(WINDOW* w, const char* str)
+{
+    int ret = OK;
+    if (Options.char_set == CSET_ASCII || Options.char_set == CSET_UNICODE)
+    {
+        // In ascii, we don't expect any high-bit chars.
+        // In Unicode, str is UTF-8 and we shouldn't touch anything.
+        ret = waddstr(w, str);
+    }
+    else
+    {
+        // Otherwise, high bit indicates alternate charset.
+        // DEC line-drawing chars don't have the high bit, so
+        // we map them into 0xE0 and above.
+        const bool bDEC = (Options.char_set == CSET_DEC);
+        const char* begin = str;
+
+        while (true)
+        {
+            const char* end = begin;
+            // Output a range of normal characters (the common case)
+            while (*end && (unsigned char)*end <= 127) ++end;
+            if (end-begin) ret = waddnstr(w, begin, end-begin);
+
+            // Then a single high-bit character
+            chtype c = (unsigned char)*end;
+            if (c == 0)
+                break;
+            else if (bDEC && c >= 0xE0)
+                ret = waddch(w, (c & 0x7F) | A_ALTCHARSET);
+            else // if (*end > 127)
+                ret = waddch(w, c | A_ALTCHARSET);
+            begin = end+1;
+        }
+    }
+    return ret;
+}
 
 int putch(unsigned char chr)
 {
@@ -596,58 +629,59 @@ int window(int x1, int y1, int x2, int y2)
     return (refresh());
 }
 
-// NOTE: This affects libunix.cc draw state; use this just before setting
-// textcolour and drawing a character and call set_altcharset(false)
-// after you're done drawing.
-//
-unsigned cset_adjust(unsigned raw)
-{
-    if (Options.char_set != CSET_ASCII && Options.char_set != CSET_UNICODE)
-    {
-        // switch to alternate char set for 8-bit characters:
-        set_altcharset( raw > 127 );
-
-        // shift the DEC line drawing set:
-        if (Options.char_set == CSET_DEC 
-            && raw >= 0xE0)
-        {
-            raw &= 0x7F;
-        }
-    }
-    return (raw);
-}
-
 void put_colour_ch(int colour, unsigned ch)
 {
-    ch = cset_adjust(ch);
-    textattr(colour);
-    putwch(ch);
+    if (Options.char_set == CSET_ASCII || Options.char_set == CSET_UNICODE)
+    {
+        // Unicode can't or-in curses attributes; but it also doesn't have
+        // to fool around with altcharset
+        textattr(colour);
+        putwch(ch);
+    }
+    else
+    {
+        const unsigned oldch = ch;
+        if (oldch & 0x80) ch |= A_ALTCHARSET;
+        // Shift the DEC line drawing set
+        if (oldch >= 0xE0 && Options.char_set == CSET_DEC)
+            ch ^= 0x80;
+        textattr(colour);
+
+        // putch truncates ch to a byte, so don't use it.
+        if ((ch & 0x7f) == 0) ch |= ' ';
+        addch(ch);
+    }
 }
 
 void puttext(int x1, int y1, int x2, int y2, const screen_buffer_t *buf)
 {
+    const bool will_scroll = (x2 == get_number_of_cols());
+
+    if (will_scroll)
+        scrollok(stdscr, FALSE);
+
     for (int y = y1; y <= y2; ++y)
     {
-        gotoxy(x1, y);
+        cgotoxy(x1, y);
         for (int x = x1; x <= x2; ++x)
         {
-            const screen_buffer_t ch = cset_adjust(*buf);
-            textattr( buf[1] );
-            putwch( ch );
+            put_colour_ch( buf[1], buf[0] );
             buf += 2;
         }
     }
     update_screen();
+
+    if (will_scroll)
+        scrollok(stdscr, TRUE);
 }
 
-// These next four are front functions so that we can reduce 
+// These next four are front functions so that we can reduce
 // the amount of curses special code that occurs outside this
-// this file.  This is good, since there are some issues with 
-// name space collisions between curses macros and the standard 
+// this file.  This is good, since there are some issues with
+// name space collisions between curses macros and the standard
 // C++ string class.  -- bwr
 void update_screen(void)
 {
-    set_altcharset(false);
     refresh();
 }
 
@@ -710,14 +744,15 @@ bool is_cursor_enabled()
 
 inline unsigned get_brand(int col)
 {
-    return (col & COLFLAG_FRIENDLY_MONSTER)? Options.friend_brand :
-        (col & COLFLAG_ITEM_HEAP)?           Options.heap_brand :
-        (col & COLFLAG_WILLSTAB)?            Options.stab_brand :
-        (col & COLFLAG_MAYSTAB)?             Options.may_stab_brand :
-        (col & COLFLAG_STAIR_ITEM)?          Options.stair_item_brand :
-        (col & COLFLAG_TRAP_ITEM)?           Options.trap_item_brand :
-        (col & COLFLAG_REVERSE)?             CHATTR_REVERSE :
-                                             CHATTR_NORMAL;
+    return (col & COLFLAG_FRIENDLY_MONSTER) ? Options.friend_brand :
+           (col & COLFLAG_NEUTRAL_MONSTER)  ? Options.neutral_brand :
+           (col & COLFLAG_ITEM_HEAP)        ? Options.heap_brand :
+           (col & COLFLAG_WILLSTAB)         ? Options.stab_brand :
+           (col & COLFLAG_MAYSTAB)          ? Options.may_stab_brand :
+           (col & COLFLAG_FEATURE_ITEM)     ? Options.feature_item_brand :
+           (col & COLFLAG_TRAP_ITEM)        ? Options.trap_item_brand :
+           (col & COLFLAG_REVERSE)          ? CHATTR_REVERSE
+                                            : CHATTR_NORMAL;
 }
 
 void textattr(int col)
@@ -746,12 +781,12 @@ static int curs_fg_attr(int col)
         {
             bg = translate_colour(
                     macro_colour( (brand & CHATTR_COLMASK) >> 8 ));
-            
+
             if (fg == bg)
                 fg = COLOR_BLACK;
         }
 
-        // If we can't do a dark grey friend brand, then we'll 
+        // If we can't do a dark grey friend brand, then we'll
         // switch the colour to light grey.
         if (Options.no_dark_brand
                 && fg == (COLOR_BLACK | COLFLAG_CURSES_BRIGHTEN)
@@ -779,7 +814,7 @@ static int curs_fg_attr(int col)
     // figure out which colour pair we want
     const int pair = (fg == 0 && bg == 0) ? 63 : (bg * 8 + fg);
 
-    return ( COLOR_PAIR(pair) | flags | Character_Set );
+    return ( COLOR_PAIR(pair) | flags );
 }
 
 void textcolor(int col)
@@ -809,8 +844,8 @@ static int curs_bg_attr(int col)
             if (fg == bg)
                 fg = COLOR_BLACK;
         }
-        
-        // If we can't do a dark grey friend brand, then we'll 
+
+        // If we can't do a dark grey friend brand, then we'll
         // switch the colour to light grey.
         if (Options.no_dark_brand
                 && fg == (COLOR_BLACK | COLFLAG_CURSES_BRIGHTEN)
@@ -838,7 +873,7 @@ static int curs_bg_attr(int col)
     // figure out which colour pair we want
     const int pair = (fg == 0 && bg == 0) ? 63 : (bg * 8 + fg);
 
-    return ( COLOR_PAIR(pair) | flags | Character_Set );
+    return ( COLOR_PAIR(pair) | flags );
 }
 
 void textbackground(int col)
@@ -847,7 +882,7 @@ void textbackground(int col)
 }
 
 
-int gotoxy(int x, int y)
+int gotoxy_sys(int x, int y)
 {
     return (move(y - 1, x - 1));
 }
@@ -888,18 +923,19 @@ static void flip_colour(cchar_t &ch)
     }
 
     const int newpair = (fg * 8 + bg);
-    ch.attr = COLOR_PAIR(newpair);
+    ch.attr = COLOR_PAIR(newpair) | (ch.attr & A_ALTCHARSET);
 }
 #else // ! UNICODE_GLYPHS
-typedef unsigned char_info;
+typedef unsigned long char_info;
 #define character_at(y,x)    mvinch(y,x)
 #define valid_char(x) (x)
 #define write_char_at(y,x,c) mvaddch(y, x, c)
 
 #define char_info_character(c) ((c) & A_CHARTEXT)
 #define char_info_colour(c)    ((c) & A_COLOR)
+#define char_info_attributes(c) ((c) & A_ATTRIBUTES)
 
-static void flip_colour(unsigned &ch)
+static void flip_colour(char_info &ch)
 {
     const unsigned colour = char_info_colour(ch);
     const int pair        = PAIR_NUMBER(colour);
@@ -914,7 +950,8 @@ static void flip_colour(unsigned &ch)
     }
 
     const int newpair = (fg * 8 + bg);
-    ch = ((ch & 127) | COLOR_PAIR(newpair));
+    ch = (char_info_character(ch) | COLOR_PAIR(newpair) |
+          (char_info_attributes(ch) & A_ALTCHARSET));
 }
 #endif
 
@@ -931,7 +968,7 @@ void fakecursorxy(int x, int y)
         else
             return;
     }
-    
+
     char_info c = character_at(y - 1, x - 1);
     oldch   = c;
     faked_x = x - 1;
