@@ -3,7 +3,7 @@
  *  Summary:    Functions that may be missing from some systems
  *  Written by: ?
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-02T12:45:55.886430Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2008-07-12T01:51:42.276671Z $
  *
  *  Change History (most recent first):
  *
@@ -13,6 +13,7 @@
 
 #include "AppHdr.h"
 #include "defines.h"
+#include "directn.h"
 #include "initfile.h"
 #include "libutil.h"
 #include "externs.h"
@@ -24,8 +25,10 @@
 #include <stdarg.h>
 #include <string.h>
 
-#ifdef WIN32CONSOLE
+#if defined(WIN32CONSOLE) || defined(WIN32TILES)
+    #undef ARRAYSZ
     #include <windows.h>
+    #undef max
 
     #ifdef WINMM_PLAY_SOUNDS
         #include <mmsystem.h>
@@ -34,7 +37,7 @@
 
 #ifdef REGEX_PCRE
     // Statically link pcre on Windows
-    #ifdef WIN32CONSOLE
+    #if defined(WIN32CONSOLE) || defined(WIN32TILES)
         #define PCRE_STATIC
     #endif
 
@@ -51,7 +54,7 @@ description_level_type description_type_by_name(const char *desc)
 {
     if (!desc)
         return DESC_PLAIN;
-    
+
     if (!strcmp("The", desc))
         return DESC_CAP_THE;
     else if (!strcmp("the", desc))
@@ -92,13 +95,11 @@ std::string apply_description(description_level_type desc,
     case DESC_NOCAP_THE:
         return ("the " + name);
     case DESC_CAP_A:
-        return (quantity > 1?
-                number_to_string(quantity, in_words) + name
-                : article_a(name, false));
+        return (quantity > 1 ? number_to_string(quantity, in_words) + name
+                             : article_a(name, false));
     case DESC_NOCAP_A:
-        return (quantity > 1?
-                number_to_string(quantity, in_words) + name
-                : article_a(name, true));
+        return (quantity > 1 ? number_to_string(quantity, in_words) + name
+                             : article_a(name, true));
     case DESC_CAP_YOUR:
         return ("Your " + name);
     case DESC_NOCAP_YOUR:
@@ -113,8 +114,8 @@ std::string apply_description(description_level_type desc,
 // the shell can do damage with.
 bool shell_safe(const char *file)
 {
-    int match = strcspn(file, "\\`$*?|><&\n!");
-    return !(match >= 0 && file[match]);
+    int match = strcspn(file, "\\`$*?|><&\n!;");
+    return (match < 0 || !file[match]);
 }
 
 void play_sound( const char *file )
@@ -123,11 +124,12 @@ void play_sound( const char *file )
     // Check whether file exists, is readable, etc.?
     if (file && *file)
         sndPlaySound(file, SND_ASYNC | SND_NODEFAULT);
+
 #elif defined(SOUND_PLAY_COMMAND)
     char command[255];
     command[0] = 0;
     if (file && *file && (strlen(file) + strlen(SOUND_PLAY_COMMAND) < 255)
-            && shell_safe(file))
+        && shell_safe(file))
     {
         snprintf(command, sizeof command, SOUND_PLAY_COMMAND, file);
         system(command);
@@ -157,6 +159,7 @@ std::string &uppercase(std::string &s)
 {
     for (unsigned i = 0, sz = s.size(); i < sz; ++i)
         s[i] = toupper(s[i]);
+
     return (s);
 }
 
@@ -164,6 +167,7 @@ std::string &lowercase(std::string &s)
 {
     for (unsigned i = 0, sz = s.size(); i < sz; ++i)
         s[i] = tolower(s[i]);
+
     return (s);
 }
 
@@ -173,22 +177,15 @@ std::string lowercase_string(std::string s)
     return (s);
 }
 
-bool ends_with(const std::string &s, const std::string &suffix)
-{
-    if (s.length() < suffix.length())
-        return false;
-    return (s.substr(s.length() - suffix.length()) == suffix);
-}
-
 int ends_with(const std::string &s, const char *suffixes[])
 {
     if (!suffixes)
         return (0);
+
     for (int i = 0; suffixes[i]; ++i)
-    {
         if (ends_with(s, suffixes[i]))
             return (1 + i);
-    }
+
     return (0);
 }
 
@@ -213,7 +210,7 @@ bool strip_tag(std::string &s, const std::string &tag, bool skip_padding)
         }
         return (false);
     }
-    
+
     if ((pos = s.find(" " + tag + " ")) != std::string::npos)
     {
         // Leave one space intact.
@@ -235,14 +232,20 @@ bool strip_tag(std::string &s, const std::string &tag, bool skip_padding)
 
 std::string strip_tag_prefix(std::string &s, const std::string &tagprefix)
 {
-    const std::string::size_type pos = s.find(tagprefix);
+    std::string::size_type pos = s.find(tagprefix);
+
+    while (pos && pos != std::string::npos && !isspace(s[pos - 1]))
+    {
+        pos = s.find(tagprefix, pos + 1);
+    }
+
     if (pos == std::string::npos)
         return ("");
 
     std::string::size_type ns = s.find(" ", pos);
     if (ns == std::string::npos)
         ns = s.length();
-    
+
     const std::string argument =
         s.substr(pos + tagprefix.length(), ns - pos - tagprefix.length());
 
@@ -261,13 +264,18 @@ int strip_number_tag(std::string &s, const std::string &tagprefix)
 // Naively prefix A/an to a noun.
 std::string article_a(const std::string &name, bool lowercase)
 {
-    if (!name.length()) return name;
-    const char *a  = lowercase? "a " : "A ";
+    if (!name.length())
+        return name;
+
+    const char *a  = lowercase? "a "  : "A ";
     const char *an = lowercase? "an " : "An ";
     switch (name[0])
     {
         case 'a': case 'e': case 'i': case 'o': case 'u':
         case 'A': case 'E': case 'I': case 'O': case 'U':
+            // XXX: Hack
+            if (starts_with(name, "one-way"))
+                return a + name;
             return an + name;
         default:
             return a + name;
@@ -279,7 +287,7 @@ const char *standard_plural_qualifiers[] =
     " of ", " labeled ", NULL
 };
 
-// Pluralises a monster or item name. This'll need to be updated for
+// Pluralises a monster or item name.  This'll need to be updated for
 // correctness whenever new monsters/items are added.
 std::string pluralise(const std::string &name,
                       const char *qualifiers[],
@@ -290,49 +298,85 @@ std::string pluralise(const std::string &name,
     if (qualifiers)
     {
         for (int i = 0; qualifiers[i]; ++i)
-            if ((pos = name.find(qualifiers[i])) != std::string::npos 
+            if ((pos = name.find(qualifiers[i])) != std::string::npos
                 && !ends_with(name, no_qualifier))
             {
                 return pluralise(name.substr(0, pos)) + name.substr(pos);
             }
     }
-    
+
+    if (!name.empty() && name[name.length() - 1] == ')'
+        && (pos = name.rfind(" (")) != std::string::npos)
+    {
+        return (pluralise(name.substr(0, pos)) + name.substr(pos));
+    }
+
     if (ends_with(name, "us"))
+    {
         // Fungus, ufetubus, for instance.
         return name.substr(0, name.length() - 2) + "i";
+    }
     else if (ends_with(name, "larva") || ends_with(name, "amoeba"))
+    {
         // Giant amoebae sounds a little weird, to tell the truth.
         return name + "e";
+    }
     else if (ends_with(name, "ex"))
+    {
         // Vortex; vortexes is legal, but the classic plural is cooler.
         return name.substr(0, name.length() - 2) + "ices";
+    }
     else if (ends_with(name, "cyclops"))
+    {
         return name.substr(0, name.length() - 1) + "es";
+    }
     else if (ends_with(name, "y"))
+    {
         return name.substr(0, name.length() - 1) + "ies";
-    else if (ends_with(name, "elf") || ends_with(name, "olf"))
-        // Elf, wolf. Dwarfs can stay dwarfs, if there were dwarfs.
+    }
+    else if (ends_with(name, "fe"))
+    {
+        // knife -> knives
+        return name.substr(0, name.length() - 2) + "ves";
+    }
+    else if (ends_with(name, "elf") || ends_with(name, "olf")
+              || ends_with(name, "arf"))
+    {
+        // Elf, wolf, dwarf.
         return name.substr(0, name.length() - 1) + "ves";
+    }
     else if (ends_with(name, "mage"))
+    {
         // mage -> magi
         return name.substr(0, name.length() - 1) + "i";
-    else if ( ends_with(name, "sheep") || ends_with(name, "manes")
-            || ends_with(name, "fish") )
+    }
+    else if (ends_with(name, "sheep") || ends_with(name, "manes")
+              || ends_with(name, "fish") || ends_with(name, "folk"))
+    {
         // Maybe we should generalise 'manes' to ends_with("es")?
         return name;
-    else if (ends_with(name, "ch") || ends_with(name, "sh") 
-            || ends_with(name, "x"))
-        // To handle cockroaches, fish and sphinxes. Fish will be netted by
-        // the previous check anyway.
+    }
+    else if (ends_with(name, "ch") || ends_with(name, "sh")
+             || ends_with(name, "x"))
+    {
+        // To handle cockroaches and sphinxes, and in case there's some monster
+        // ending with sh (except fish, which are caught in the previous check).
         return name + "es";
+    }
     else if (ends_with(name, "um"))
+    {
         // simulacrum -> simulacra
         return name.substr(0, name.length() - 2) + "a";
+    }
     else if (ends_with(name, "efreet"))
+    {
         // efreet -> efreeti. Not sure this is correct.
         return name + "i";
+    }
     else if (ends_with(name, "staff"))
+    {
         return name.substr(0, name.length() - 2) + "ves";
+    }
 
     return name + "s";
 }
@@ -371,14 +415,16 @@ static std::string tens_in_words(unsigned num)
         return numbers[num];
 
     int ten = num / 10, digit = num % 10;
-    return std::string(tens[ten]) 
-        + (digit? std::string("-") + numbers[digit] : "");
+    return std::string(tens[ten])
+             + (digit ? std::string("-") + numbers[digit] : "");
 }
 
 static std::string join_strings(const std::string &a, const std::string &b)
 {
-    return !a.empty() && !b.empty()? a + " " + b :
-        a.empty()? b : a;
+    if (!a.empty() && !b.empty())
+        return (a + " " + b);
+
+    return (a.empty() ? b : a);
 }
 
 static std::string hundreds_in_words(unsigned num)
@@ -395,12 +441,13 @@ std::string number_in_words(unsigned num, int pow)
         return number_in_words(num, 0) + pow_in_words(pow);
 
     unsigned thousands = num % 1000, rest = num / 1000;
-    if (!rest and !thousands)
+    if (!rest && !thousands)
         return ("zero");
 
     return join_strings((rest? number_in_words(rest, pow + 3) : ""),
-                (thousands? hundreds_in_words(thousands) + pow_in_words(pow)
-                : ""));
+                        (thousands? hundreds_in_words(thousands)
+                                    + pow_in_words(pow)
+                                  : ""));
 }
 
 std::string replace_all(std::string s,
@@ -409,7 +456,7 @@ std::string replace_all(std::string s,
 {
     std::string::size_type start = 0;
     std::string::size_type found;
-   
+
     while ((found = s.find(find, start)) != std::string::npos)
     {
         s.replace( found, find.length(), repl );
@@ -427,7 +474,7 @@ std::string replace_all_of(std::string s,
 {
     std::string::size_type start = 0;
     std::string::size_type found;
-   
+
     while ((found = s.find_first_of(tofind, start)) != std::string::npos)
     {
         s.replace( found, 1, replacement );
@@ -466,10 +513,16 @@ std::string &trim_string( std::string &str )
     return (str);
 }
 
-static void add_segment(std::vector<std::string> &segs,
-                        std::string s,
-                        bool trim,
-                        bool accept_empty)
+std::string &trim_string_right( std::string &str )
+{
+    str.erase( str.find_last_not_of( " \t\n\r" ) + 1 );
+    return (str);
+}
+
+static void add_segment( std::vector<std::string> &segs,
+                         std::string s,
+                         bool trim,
+                         bool accept_empty)
 {
     if (trim && !s.empty())
         trim_string(s);
@@ -478,12 +531,11 @@ static void add_segment(std::vector<std::string> &segs,
         segs.push_back(s);
 }
 
-std::vector<std::string> split_string(
-    const std::string &sep, 
-    std::string s,
-    bool trim_segments,
-    bool accept_empty_segments,
-    int nsplits)
+std::vector<std::string> split_string( const std::string &sep,
+                                       std::string s,
+                                       bool trim_segments,
+                                       bool accept_empty_segments,
+                                       int nsplits)
 {
     std::vector<std::string> segments;
     int separator_length = sep.length();
@@ -493,9 +545,9 @@ std::vector<std::string> split_string(
     {
         add_segment(segments, s.substr(0, pos),
                     trim_segments, accept_empty_segments);
-        
+
         s.erase(0, pos + separator_length);
-        
+
         if (nsplits > 0)
             --nsplits;
     }
@@ -506,7 +558,7 @@ std::vector<std::string> split_string(
     return segments;
 }
 
-// The old school way of doing short delays via low level I/O sync.  
+// The old school way of doing short delays via low level I/O sync.
 // Good for systems like old versions of Solaris that don't have usleep.
 #ifdef NEED_USLEEP
 
@@ -518,17 +570,17 @@ void usleep(unsigned long time)
 {
     struct timeval timer;
 
-    timer.tv_sec = (time / 1000000L);
+    timer.tv_sec  = (time / 1000000L);
     timer.tv_usec = (time % 1000000L);
 
     select(0, NULL, NULL, NULL, &timer);
 }
 #endif
 
-// Not the greatest version of snprintf, but a functional one that's 
+// Not the greatest version of snprintf, but a functional one that's
 // a bit safer than raw sprintf().  Note that this doesn't do the
-// special behaviour for size == 0, largely because the return value 
-// in that case varies depending on which standard is being used (SUSv2 
+// special behaviour for size == 0, largely because the return value
+// in that case varies depending on which standard is being used (SUSv2
 // returns an unspecified value < 1, whereas C99 allows str == NULL
 // and returns the number of characters that would have been written). -- bwr
 #ifdef NEED_SNPRINTF
@@ -540,7 +592,7 @@ int snprintf( char *str, size_t size, const char *format, ... )
     va_list argp;
     va_start( argp, format );
 
-    char *buff = new char [ 10 * size ];  // hopefully enough 
+    char *buff = new char [ 10 * size ];  // hopefully enough
     if (!buff)
         end(1, false, "Out of memory\n");
 
@@ -548,7 +600,7 @@ int snprintf( char *str, size_t size, const char *format, ... )
     strncpy( str, buff, size );
     str[ size - 1 ] = 0;
 
-    int ret = strlen( str );  
+    int ret = strlen( str );
     if ((unsigned int) ret == size - 1 && strlen( buff ) >= size)
         ret = -1;
 
@@ -561,6 +613,35 @@ int snprintf( char *str, size_t size, const char *format, ... )
 
 #endif
 
+#ifndef USE_TILE
+void cgotoxy(int x, int y, int region)
+{
+    ASSERT(x >= 1);
+    ASSERT(y >= 1);
+    switch(region)
+    {
+    case GOTO_MLIST:
+        ASSERT(x <= crawl_view.mlistsz.x);
+        ASSERT(y <= crawl_view.mlistsz.y);
+        gotoxy_sys(x + crawl_view.mlistp.x - 1, y + crawl_view.mlistp.y - 1);
+        break;
+    case GOTO_STAT:
+        ASSERT(x <= crawl_view.hudsz.x);
+        ASSERT(y <= crawl_view.hudsz.y);
+        gotoxy_sys(x + crawl_view.hudp.x - 1, y + crawl_view.hudp.y - 1);
+        break;
+    case GOTO_MSG:
+        ASSERT(x <= crawl_view.msgsz.x);
+        ASSERT(y <= crawl_view.msgsz.y);
+        gotoxy_sys(x + crawl_view.msgp.x - 1, y + crawl_view.msgp.y - 1);
+        break;
+    case GOTO_CRT:
+    default:
+        gotoxy_sys(x, y);
+        break;
+    }
+}
+#endif
 ///////////////////////////////////////////////////////////////////////
 // Pattern matching
 
@@ -576,13 +657,15 @@ static bool glob_match( const char *pattern, const char *text, bool icase )
     char p, t;
     bool special;
 
-    for (;;)
+    while (true)
     {
         p = pm_lower(*pattern++, icase);
         t = pm_lower(*text++, icase);
         special = true;
 
-        if (!p) return t == 0;
+        if (!p)
+            return (t == 0);
+
         if (p == '\\' && *pattern)
         {
             p       = pm_lower(*pattern++, icase);
@@ -590,14 +673,17 @@ static bool glob_match( const char *pattern, const char *text, bool icase )
         }
 
         if (p == '*' && special)
+        {
             // Try to match exactly at the current text position...
-            return !*pattern || glob_match(pattern, text - 1, icase)? true :
-                // Or skip one character in the text and try the wildcard
-                // match again. If this is the end of the text, the match has
-                // failed.
-                t? glob_match(pattern - 1, text, icase) : false;
-        else if (!t || (p != t && (p != '?' || !special)))
-            return false;
+            if (!*pattern || glob_match(pattern, text - 1, icase))
+                return (true);
+
+            // Or skip one character in the text and try the wildcard match
+            // again. If this is the end of the text, the match has failed.
+            return (t ? glob_match(pattern - 1, text, icase) : false);
+        }
+        else if (!t || p != t && (p != '?' || !special))
+            return (false);
     }
 }
 
@@ -610,7 +696,7 @@ struct glob_info
     bool ignore_case;
 };
 
-void *compile_glob_pattern(const char *pattern, bool icase) 
+void *compile_glob_pattern(const char *pattern, bool icase)
 {
     // If we're using simple globs, we need to box the pattern with '*'
     std::string s = std::string("*") + pattern + "*";
@@ -644,7 +730,7 @@ void *compile_pattern(const char *pattern, bool icase)
     const char *error;
     int erroffset;
     int flags = icase? PCRE_CASELESS : 0;
-    return pcre_compile(pattern, 
+    return pcre_compile(pattern,
                         flags,
                         &error,
                         &erroffset,
@@ -661,13 +747,9 @@ bool pattern_match(void *compiled_pattern, const char *text, int length)
 {
     int ovector[42];
     int pcre_rc = pcre_exec(static_cast<pcre *>(compiled_pattern),
-              NULL,
-              text,
-              length,
-              0,
-              0,
-              ovector,
-              sizeof(ovector) / sizeof(*ovector));
+                            NULL,
+                            text, length, 0, 0,
+                            ovector, sizeof(ovector) / sizeof(*ovector));
     return (pcre_rc >= 0);
 }
 
