@@ -2,7 +2,7 @@
  *  File:       libutil.h
  *  Summary:    System independent functions 
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-02T12:45:55.886430Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-25T18:11:54.006877Z $
  *
  *  Change History (most recent first):
  *
@@ -91,9 +91,22 @@ typedef void *(*p_compile)(const char *pattern, bool ignore_case);
 typedef void (*p_free)(void *cp);
 typedef bool (*p_match)(void *compiled_pattern, const char *text, int length);
 
-std::string & trim_string( std::string &str );
+std::string &trim_string( std::string &str );
+std::string &trim_string_right( std::string &str );
 std::string trimmed_string( std::string s );
 
+inline bool starts_with(const std::string &s, const std::string &prefix)
+{
+    return (s.rfind(prefix, 0) != std::string::npos);
+}
+
+inline bool ends_with(const std::string &s, const std::string &suffix)
+{
+    if (s.length() < suffix.length())
+        return false;
+    return (s.find(suffix, s.length() - suffix.length()) != std::string::npos);
+}
+
 // Splits string 's' on the separator 'sep'. If trim == true, trims each
 // segment. If accept_empties == true, accepts empty segments. If nsplits >= 0,
 // splits on the first nsplits occurrences of the separator, and stores the
@@ -113,6 +126,13 @@ inline std::string lowercase_first(std::string s)
     return (s);
 }
 
+inline std::string uppercase_first(std::string s)
+{
+    if (s.length())
+        s[0] = toupper(s[0]);
+    return (s);
+}
+
 template <typename Z>
 std::string comma_separated_line(Z start, Z end,
                                  const std::string &andc = " and ",
@@ -143,6 +163,10 @@ void usleep( unsigned long time );
 int snprintf( char *str, size_t size, const char *format, ... );
 #endif
 
+#ifndef USE_TILE
+void cgotoxy(int x, int y, int region = GOTO_CRT);
+#endif
+
 template <typename T>
 class unwind_var
 {
@@ -160,6 +184,17 @@ public:
     {
         val = oldval;
     }
+
+    T value() const
+    {
+        return val;
+    }
+
+    T original_value() const
+    {
+        return oldval;
+    }
+    
 private:
     T &val;
     T oldval;
