@@ -1,3 +1,6 @@
+#include "AppHdr.h"
+#include "externs.h"
+
 #if defined(WIN32CONSOLE)
 
 /*
@@ -5,7 +8,7 @@
  *  Summary:    Functions for windows32 console mode support
  *  Written by: Gordon Lipford
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2008-07-12T01:10:58.493034Z $
  *
  *  Change History (most recent first):
  *
@@ -42,7 +45,6 @@
 #define NOMDI             /* MDI support */
 #define NOCTLMGR          /* Control management and controls */
 #define NOHELP            /* Help support */
-
 /*
  * Exclude parts of WINDOWS.H that are not needed (Win32)
  */
@@ -57,7 +59,9 @@
 
 #include <excpt.h>
 #include <stdarg.h>
+#undef ARRAYSZ
 #include <windows.h>
+#undef max
 
 // END -- WINDOWS INCLUDES
 
@@ -67,7 +71,6 @@
 
 #include <string.h>
 #include <stdio.h>
-#include "AppHdr.h"
 #include "cio.h"
 #include "defines.h"
 #include "libutil.h"
@@ -83,9 +86,9 @@ static HANDLE outbuf = NULL;
 static int current_color = -1;
 static bool cursor_is_enabled = false;
 // dirty line (sx,ex,y)
-static int chsx=0, chex=0, chy=-1;
+static int chsx = 0, chex = 0, chy = -1;
 // cursor position (start at 0,0 --> 1,1)
-static int cx=0, cy=0;
+static int cx = 0, cy = 0;
 
 // and now, for the screen buffer
 static CHAR_INFO *screen = NULL;
@@ -136,6 +139,7 @@ void writeChar(char c)
             writeChar(' ');
         return;
     }
+
     bool noop = true;
     PCHAR_INFO pci;
 
@@ -150,7 +154,7 @@ void writeChar(char c)
         bFlush();
 
         // reposition
-        gotoxy(1, cy+2);
+        cgotoxy(1, cy+2);
 
         return;
     }
@@ -236,7 +240,7 @@ void set_mouse_enabled(bool enabled)
             inmode |= ENABLE_MOUSE_INPUT;
         else
             inmode &= ~ENABLE_MOUSE_INPUT;
-        
+
         ::SetConsoleMode(inbuf, inmode);
     }
 }
@@ -314,7 +318,7 @@ static void set_w32_screen_size()
         delete [] screen;
         screen = NULL;
     }
-   
+
     screen = new CHAR_INFO[screensize.X * screensize.Y];
 
     COORD topleft;
@@ -375,7 +379,7 @@ void init_libw32c(void)
 
     // set up screen size
     set_w32_screen_size();
-   
+
     // initialise text color
     textcolor(DARKGREY);
 
@@ -387,7 +391,7 @@ void init_libw32c(void)
 
     crawl_state.terminal_resize_handler = w32_term_resizer;
     crawl_state.terminal_resize_check   = w32_check_screen_resize;
-   
+
     // JWM, 06/12/2004: Code page setting, as XP does not use ANSI 437 by
     // default.
     InputCP = GetConsoleCP();
@@ -409,7 +413,7 @@ void deinit_libw32c(void)
     if (inbuf == NULL || outbuf == NULL)
         return;
 
-    // JWM, 06/12/2004: Code page stuff.  If it was the preferred code page, it 
+    // JWM, 06/12/2004: Code page stuff.  If it was the preferred code page, it
     // doesn't need restoring.  Shouldn't be an error and too bad if there is.
     if (InputCP && InputCP != PREFERRED_CODEPAGE)
         SetConsoleCP(InputCP);
@@ -426,7 +430,7 @@ void deinit_libw32c(void)
 
     delete [] screen;
     screen = NULL;
-   
+
     // finally, restore title
     SetConsoleTitle( oldTitle );
 }
@@ -457,7 +461,7 @@ void _setcursortype_internal(bool curstype)
     // now, if we just changed from NOCURSOR to CURSOR,
     // actually move screen cursor
     if (cursor_is_enabled)
-        gotoxy(cx+1, cy+1);
+        cgotoxy(cx+1, cy+1);
 }
 
 // This will force the cursor down to the next line.
@@ -477,15 +481,13 @@ void clrscr(void)
 
     PCHAR_INFO pci = screen;
 
-    for(x = 0; x < screensize.X; x++)
-    {
-        for(y = 0; y < screensize.Y; y++)
+    for (x = 0; x < screensize.X; x++)
+        for (y = 0; y < screensize.Y; y++)
         {
             pci->Char.AsciiChar = ' ';
             pci->Attributes = 0;
             pci++;
         }
-    }
 
     source.X = 0;
     source.Y = 0;
@@ -497,10 +499,10 @@ void clrscr(void)
     WriteConsoleOutput(outbuf, screen, screensize, source, &target);
 
     // reset cursor to 1,1 for convenience
-    gotoxy(1,1);
+    cgotoxy(1,1);
 }
 
-void gotoxy(int x, int y)
+void gotoxy_sys(int x, int y)
 {
     // always flush on goto
     bFlush();
@@ -538,7 +540,12 @@ void textattr(int c)
 void textcolor(int c)
 {
     // change current color used to stamp chars
-    current_color = c;
+    short fg = c & 0xF;
+    short bg = (c >> 4) & 0xF;
+    short macro_fg = Options.colour[fg];
+    short macro_bg = Options.colour[bg];
+
+    current_color = macro_fg | (macro_bg << 4);
 }
 
 void clear_message_window()
@@ -602,7 +609,7 @@ static void scroll_message_window()
 
     // Cursor also scrolls up so prompts don't look brain-damaged.
     if (wherey() == screensize.Y)
-        gotoxy(wherex(), wherey() - 1);
+        cgotoxy(wherex(), wherey() - 1);
 }
 
 void message_out(int which_line, int colour, const char *s, int firstcol,
@@ -611,7 +618,7 @@ void message_out(int which_line, int colour, const char *s, int firstcol,
     if (!firstcol)
         firstcol = Options.delay_message_clear? 2 : 1;
 
-    gotoxy(firstcol - 1 + crawl_view.msgp.x,
+    cgotoxy(firstcol - 1 + crawl_view.msgp.x,
            which_line + crawl_view.msgp.y);
     textcolor(colour);
 
@@ -636,7 +643,7 @@ static void cprintf_aux(const char *s)
 
     // loop through string
     char *p = (char *)s;
-    while(*p)
+    while (*p)
     {
         writeChar(*p++);
     }
@@ -683,7 +690,7 @@ int wherey(void)
 void putch(char c)
 {
     // special case: check for '0' char: map to space
-    if (c==0)
+    if (c == 0)
         c = ' ';
 
     writeChar(c);
@@ -719,7 +726,7 @@ int key_to_command( int keyin )
 {
     if (keyin >= CK_UP && keyin <= CK_CTRL_PGDN)
         return ck_tr[ keyin - CK_UP ];
-    
+
     if (keyin == CK_DELETE)
         return '.';
 
@@ -757,15 +764,16 @@ int vk_translate( WORD VirtCode, CHAR c, DWORD cKeys)
 
     // see if we're a vkey
     int mkey;
-    for(mkey = 0; mkey<VKEY_MAPPINGS; mkey++)
-        if (VirtCode == vk_tr[0][mkey]) break;
+    for (mkey = 0; mkey<VKEY_MAPPINGS; mkey++)
+        if (VirtCode == vk_tr[0][mkey])
+            break;
 
     // step 4 - just return the damn key.
     if (mkey == VKEY_MAPPINGS)
     {
         if (c)
             return c;
-      
+
         // ds -- Icky hacks to allow keymaps with funky keys.
         if (ctrlDown)
             VirtCode |= 512;
@@ -785,7 +793,7 @@ int vk_translate( WORD VirtCode, CHAR c, DWORD cKeys)
     // control key?
     if (ctrlDown)
         return vk_tr[3][mkey];
-   
+
     // shifted?
     if (shftDown)
         return vk_tr[2][mkey];
@@ -804,7 +812,7 @@ static int w32_proc_mouse_event(const MOUSE_EVENT_RECORD &mer)
 
     if (!crawl_state.mouse_enabled)
         return (0);
-    
+
     c_mouse_event cme(pos);
     if (mer.dwEventFlags & MOUSE_MOVED)
         return (CK_MOUSE_MOVE);
@@ -865,7 +873,7 @@ int getch_ck(void)
             case KEY_EVENT:
                 kr = &ir.Event.KeyEvent;
                 // ignore if it is a 'key up' - we only want 'key down'
-                if (kr->bKeyDown == true)
+                if (kr->bKeyDown)
                 {
                     key = vk_translate( kr->wVirtualKeyCode,
                                         kr->uChar.AsciiChar,
@@ -879,7 +887,7 @@ int getch_ck(void)
                     }
                 }
                 break;
-                
+
             case WINDOW_BUFFER_SIZE_EVENT:
                 w32_handle_resize_event();
                 break;
@@ -997,7 +1005,7 @@ void puttext(int x1, int y1, int x2, int y2, const screen_buffer_t *buf)
 {
     for (int y = y1; y <= y2; ++y)
     {
-        gotoxy(x1, y);
+        cgotoxy(x1, y);
         for (int x = x1; x <= x2; x++)
         {
             textattr( buf[1] );
@@ -1037,4 +1045,112 @@ int get_number_of_cols()
     return (screensize.X);
 }
 
+#if _MSC_VER
+struct DIR
+{
+ public:
+    DIR()
+        : hFind(INVALID_HANDLE_VALUE),
+          wfd_valid(false)
+    {
+        memset(&wfd, 0, sizeof(wfd));
+        memset(&entry, 0, sizeof(entry));
+    }
+
+    ~DIR()
+    {
+        if (hFind != INVALID_HANDLE_VALUE)
+        {
+            FindClose(hFind);
+        }
+    }
+
+    bool init(const char* szFind)
+    {
+        // Check that it's a directory, first.
+        {
+            const DWORD dwAttr = GetFileAttributes(szFind);
+            if (dwAttr == INVALID_FILE_ATTRIBUTES)
+                return (false);
+            if ((dwAttr & FILE_ATTRIBUTE_DIRECTORY) == 0)
+                return (false);
+        }
+
+        find = szFind;
+        find += "\\*";
+
+        hFind = FindFirstFileA(find.c_str(), &wfd);
+        wfd_valid = (hFind != INVALID_HANDLE_VALUE);
+        return (true);
+    }
+
+    dirent* readdir()
+    {
+        if (!wfd_valid)
+            return 0;
+
+        _convert_wfd_to_dirent();
+        wfd_valid = (bool) FindNextFileA(hFind, &wfd);
+
+        return (&entry);
+    }
+
+ private:
+    void _convert_wfd_to_dirent()
+    {
+        entry.d_reclen = sizeof(dirent);
+        entry.d_namlen = strlen(entry.d_name);
+        entry.d_type = (wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
+            ? DT_DIR : DT_REG;
+        strncpy(entry.d_name, wfd.cFileName, sizeof(entry.d_name));
+        entry.d_name[sizeof(entry.d_name)-1] = 0;
+    }
+
+ private:
+    HANDLE hFind;
+    bool wfd_valid;
+    WIN32_FIND_DATA wfd;
+    std::string find;
+    dirent entry;
+    // since opendir calls FindFirstFile, we need a means of telling the
+    // first call to readdir that we already have a file.
+    // that's the case iff this is == 0; we use a counter rather than a
+    // flag because that allows keeping statistics.
+    int num_entries_scanned;
+};
+
+
+DIR* opendir(char* path)
+{
+    DIR* d = new DIR();
+    if (d->init(path))
+    {
+        return d;
+    }
+    else
+    {
+        delete d;
+        return 0;
+    }
+}
+
+dirent* readdir(DIR* d)
+{
+    return d->readdir();
+}
+
+int closedir(DIR* d)
+{
+    delete d;
+    return 0;
+}
+
+int ftruncate(int fp, int size)
+{
+    ASSERT(false);              // unimplemented
+    return 0;
+}
+
+#endif /* #if _MSC_VER */
+
 #endif /* #if defined(WIN32CONSOLE) */
