@@ -1,6 +1,6 @@
 /*
-*  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-03T21:06:37.928309Z $
-*/
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-03-08T10:44:09.947052Z $
+ */
 
 #ifndef LIBW32C_H
 #define LIBW32C_H
@@ -25,7 +25,7 @@ bool is_cursor_enabled();
 
 void clrscr(void);
 void clear_to_end_of_line();
-void gotoxy(int x, int y);
+void gotoxy_sys(int x, int y);
 void textcolor(int c);
 void textattr(int c);
 void cprintf( const char *format, ... );
