@@ -3,7 +3,7 @@
  *  Summary:    Crude macro-capability
  *  Written by: Juho Snellman <jsnell@lyseo.edu.ouka.fi>
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T13:58:56.443565Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-08T00:51:16.081572Z $
  *
  *  Change History (most recent first):
  *
@@ -17,14 +17,14 @@
  *   - For generic game code, #define getch() getchm().
  *   - getchm() works by reading characters from an internal
  *     buffer. If none are available, new characters are read into
- *     the buffer with getch_mul(). 
+ *     the buffer with getch_mul().
  *   - getch_mul() reads at least one character, but will read more
  *     if available (determined using kbhit(), which should be defined
- *     in the platform specific libraries). 
+ *     in the platform specific libraries).
  *   - Before adding the characters read into the buffer, any macros
  *     in the sequence are replaced (see macro_add_buf_long for the
  *     details).
- * 
+ *
  * (When the above text mentions characters, it actually means int).
  */
 
@@ -46,14 +46,15 @@
 #include <cstdlib>
 
 #include "cio.h"
+#include "delay.h"
 #include "externs.h"
 #include "message.h"
+#include "state.h"
 #include "stuff.h"
 
 // for trim_string:
 #include "initfile.h"
 
-typedef std::deque<int> keyseq;
 typedef std::deque<int> keybuf;
 typedef std::map<keyseq,keyseq> macromap;
 
@@ -74,6 +75,8 @@ static keybuf Buffer;
 #define USERFUNCBASE -10000
 static std::vector<std::string> userfunctions;
 
+static std::vector<key_recorder*> recorders;
+
 inline int userfunc_index(int key)
 {
     int index = (key <= USERFUNCBASE? USERFUNCBASE - key : -1);
@@ -98,16 +101,16 @@ static bool is_userfunction(const keyseq &seq)
     return (userfunc_index(seq) != -1);
 }
 
-const char *get_userfunction(int key)
+std::string get_userfunction(int key)
 {
     int index = userfunc_index(key);
-    return (index == -1? NULL : userfunctions[index].c_str());
+    return (index == -1 ? NULL : userfunctions[index]);
 }
 
-static const char *get_userfunction(const keyseq &seq)
+static std::string get_userfunction(const keyseq &seq)
 {
     int index = userfunc_index(seq);
-    return (index == -1? NULL : userfunctions[index].c_str());
+    return (index == -1 ? NULL : userfunctions[index]);
 }
 
 static bool userfunc_referenced(int index, const macromap &mm)
@@ -155,7 +158,7 @@ static int userfunc_getindex(const std::string &fname)
             return (i);
     }
 
-    // Pass 2 to hunt for gaps
+    // Pass 2 to hunt for gaps.
     for (int i = 0, count = userfunctions.size(); i < count; ++i)
     {
         if (userfunctions[i].empty())
@@ -169,15 +172,12 @@ static int userfunc_getindex(const std::string &fname)
     return (userfunctions.size() - 1);
 }
 
-/* 
- * Returns the name of the file that contains macros.
- */
-static std::string get_macro_file() 
+// Returns the name of the file that contains macros.
+static std::string get_macro_file()
 {
-    std::string dir =
-        !Options.macro_dir.empty()? Options.macro_dir :
-        !SysEnv.crawl_dir.empty()?  SysEnv.crawl_dir : "";
-    
+    std::string dir = !Options.macro_dir.empty() ? Options.macro_dir :
+                      !SysEnv.crawl_dir.empty()  ? SysEnv.crawl_dir : "";
+
     if (!dir.empty())
     {
 #ifndef DGL_MACRO_ABSOLUTE_PATH
@@ -201,7 +201,7 @@ static void buf2keyseq(const char *buff, keyseq &k)
     if (!buff)
         return;
 
-    // convert c_str to keyseq 
+    // convert c_str to keyseq
 
     // Check for user functions
     if (*buff == '=' && buff[1] == '=' && buff[2] == '=' && buff[3])
@@ -211,7 +211,7 @@ static void buf2keyseq(const char *buff, keyseq &k)
             k.push_back( USERFUNCBASE - index );
     }
     else
-    {    
+    {
         const int len = strlen( buff );
         for (int i = 0; i < len; i++)
             k.push_back( buff[i] );
@@ -239,22 +239,22 @@ static int read_key_code(std::string s)
     return strtol(s.c_str(), &tail, base);
 }
 
-/* 
+/*
  * Takes as argument a string, and returns a sequence of keys described
  * by the string. Most characters produce their own ASCII code. These
- * are the cases: 
+ * are the cases:
  *   \\ produces the ASCII code of a single \
  *   \{123} produces 123 (decimal)
  *   \{^A}  produces 1 (Ctrl-A)
  *   \{x40} produces 64 (hexadecimal code)
  *   \{!more} or \{!m} disables -more- prompt until the end of the macro.
  */
-static keyseq parse_keyseq( std::string s ) 
+static keyseq parse_keyseq( std::string s )
 {
     int state = 0;
     keyseq v;
     int num;
-    
+
     if (s.find("===") == 0)
     {
         buf2keyseq(s.c_str(), v);
@@ -262,20 +262,20 @@ static keyseq parse_keyseq( std::string s )
     }
 
     bool more_reset = false;
-    for (int i = 0, size = s.length(); i < size; ++i) 
+    for (int i = 0, size = s.length(); i < size; ++i)
     {
         char c = s[i];
-        
-        switch (state) 
-        {           
+
+        switch (state)
+        {
         case 0: // Normal state
             if (c == '\\')
                 state = 1;
             else
                 v.push_back(c);
-            break;          
+            break;
 
-        case 1: // Last char is a '\' 
+        case 1: // Last char is a '\'
             if (c == '\\')
             {
                 state = 0;
@@ -285,7 +285,7 @@ static keyseq parse_keyseq( std::string s )
             {
                 state = 2;
                 num = 0;
-            } 
+            }
             // XXX Error handling
             break;
 
@@ -313,15 +313,15 @@ static keyseq parse_keyseq( std::string s )
         }
         }
     }
-    
+
     return (v);
 }
 
 /*
  * Serializes a key sequence into a string of the format described
- * above. 
+ * above.
  */
-static std::string vtostr( const keyseq &seq ) 
+static std::string vtostr( const keyseq &seq )
 {
     std::ostringstream s;
 
@@ -336,10 +336,11 @@ static std::string vtostr( const keyseq &seq )
 
         v = &dummy;
     }
-    
-    for (keyseq::const_iterator i = v->begin(); i != v->end(); i++) 
+
+    for (keyseq::const_iterator i = v->begin(); i != v->end(); i++)
     {
-        if (*i <= 32 || *i > 127) {
+        if (*i <= 32 || *i > 127)
+        {
             if (*i == KEY_MACRO_MORE_PROTECT)
                 s << "\\{!more}";
             else
@@ -348,20 +349,20 @@ static std::string vtostr( const keyseq &seq )
                 snprintf( buff, sizeof(buff), "\\{%d}", *i );
                 s << buff;
             }
-        } else if (*i == '\\') {
-            s << "\\\\";
-        } else {
-            s << static_cast<char>(*i);
         }
+        else if (*i == '\\')
+            s << "\\\\";
+        else
+            s << static_cast<char>(*i);
     }
-    
+
     return (s.str());
 }
 
 /*
  * Add a macro (suprise, suprise).
  */
-static void macro_add( macromap &mapref, keyseq key, keyseq action ) 
+static void macro_add( macromap &mapref, keyseq key, keyseq action )
 {
     mapref[key] = action;
 }
@@ -379,14 +380,14 @@ static void macro_add( macromap &mapref, keyseq key, const char *buff )
  */
 static void macro_del( macromap &mapref, keyseq key )
 {
-    mapref.erase( key );    
+    mapref.erase( key );
 }
 
 /*
  * Adds keypresses from a sequence into the internal keybuffer. Ignores
- * macros. 
+ * macros.
  */
-static void macro_buf_add( const keyseq &actions, bool reverse = false ) 
+void macro_buf_add( const keyseq &actions, bool reverse)
 {
     keyseq act;
     bool need_more_reset = false;
@@ -411,68 +412,80 @@ static void macro_buf_add( const keyseq &actions, bool reverse = false )
 /*
  * Adds a single keypress into the internal keybuffer.
  */
-void macro_buf_add( int key ) 
+void macro_buf_add( int key, bool reverse )
 {
-    Buffer.push_back( key );    
+    if (reverse)
+        Buffer.push_front( key );
+    else
+        Buffer.push_back( key );
 }
 
 
 /*
  * Adds keypresses from a sequence into the internal keybuffer. Does some
- * O(N^2) analysis to the sequence to replace macros. 
+ * O(N^2) analysis to the sequence to replace macros.
  */
 static void macro_buf_add_long( keyseq actions,
-                                macromap &keymap = Keymaps[KC_DEFAULT] ) 
+                                macromap &keymap = Keymaps[KC_DEFAULT] )
 {
     keyseq tmp;
-    
+
     // debug << "Adding: " << vtostr(actions) << endl;
     // debug.flush();
-    
-    // Check whether any subsequences of the sequence are macros. 
+
+    // Check whether any subsequences of the sequence are macros.
     // The matching starts from as early as possible, and is
     // as long as possible given the first constraint. I.e from
-    // the sequence "abcdef" and macros "ab", "bcde" and "de" 
+    // the sequence "abcdef" and macros "ab", "bcde" and "de"
     // "ab" and "de" are recognized as macros.
-    
-    while (actions.size() > 0) 
+
+    while (actions.size() > 0)
     {
         tmp = actions;
-                
-        while (tmp.size() > 0) 
+
+        while (tmp.size() > 0)
         {
             keyseq result = keymap[tmp];
-            
-            // Found a macro. Add the expansion (action) of the 
-            // macro into the buffer. 
+
+            // Found a macro. Add the expansion (action) of the
+            // macro into the buffer.
             if (result.size() > 0)
             {
                 macro_buf_add( result );
                 break;
             }
-            
+
             // Didn't find a macro. Remove a key from the end
-            // of the sequence, and try again. 
+            // of the sequence, and try again.
             tmp.pop_back();
-        }       
-        
-        if (tmp.size() == 0) {
+        }
+
+        if (tmp.size() == 0)
+        {
             // Didn't find a macro. Add the first keypress of the sequence
-            // into the buffer, remove it from the sequence, and try again. 
+            // into the buffer, remove it from the sequence, and try again.
             macro_buf_add( actions.front() );
             actions.pop_front();
-
-        } else {
+        }
+        else
+        {
             // Found a macro, which has already been added above. Now just
-            // remove the macroed keys from the sequence. 
+            // remove the macroed keys from the sequence.
             for (unsigned int i = 0; i < tmp.size(); i++)
                 actions.pop_front();
         }
     }
 }
 
+static int macro_keys_left = -1;
+
+bool is_processing_macro()
+{
+    return (macro_keys_left >= 0);
+}
+
 /*
- * Command macros are only applied from the immediate front of the 
+ * Command macros are only applied from the immediate front of the
  * buffer, and only when the game is expecting a command.
  */
 static void macro_buf_apply_command_macro( void )
@@ -482,16 +495,25 @@ static void macro_buf_apply_command_macro( void )
     // find the longest match from the start of the buffer and replace it
     while (tmp.size() > 0)
     {
-        const keyseq &result = Macros[tmp]; 
+        const keyseq &result = Macros[tmp];
 
         if (result.size() > 0)
         {
             // Found macro, remove match from front:
             for (unsigned int i = 0; i < tmp.size(); i++)
+            {
                 Buffer.pop_front();
+                if (macro_keys_left >= 0)
+                    macro_keys_left--;
+            }
+
+            if (macro_keys_left == -1)
+                macro_keys_left = 0;
+            macro_keys_left += result.size();
 
             macro_buf_add(result, true);
-            break;  
+
+            break;
         }
 
         tmp.pop_back();
@@ -499,29 +521,42 @@ static void macro_buf_apply_command_macro( void )
 }
 
 /*
- * Removes the earlies keypress from the keybuffer, and returns its
+ * Removes the earliest keypress from the keybuffer, and returns its
  * value. If buffer was empty, returns -1;
  */
-static int macro_buf_get( void ) 
+static int macro_buf_get( void )
 {
     if (Buffer.size() == 0)
+    {
+        // If we're trying to fetch a new keystroke, then the processing
+        // of the previous keystroke is complete.
+        if (macro_keys_left == 0)
+            macro_keys_left = -1;
+
         return (-1);
+    }
 
     int key = Buffer.front();
     Buffer.pop_front();
-    
+
+    if (macro_keys_left >= 0)
+        macro_keys_left--;
+
+    for (int i = 0, size_i = recorders.size(); i < size_i; i++)
+        recorders[i]->add_key(key);
+
     return (key);
 }
 
 static void write_map(std::ofstream &f, const macromap &mp, const char *key)
 {
-    for (macromap::const_iterator i = mp.begin(); i != mp.end(); i++) 
+    for (macromap::const_iterator i = mp.begin(); i != mp.end(); i++)
     {
-        // Need this check, since empty values are added into the 
+        // Need this check, since empty values are added into the
         // macro struct for all used keyboard commands.
         if (i->second.size())
         {
-            f << key  << vtostr((*i).first) << std::endl 
+            f << key  << vtostr((*i).first) << std::endl
               << "A:" << vtostr((*i).second) << std::endl << std::endl;
         }
     }
@@ -530,14 +565,15 @@ static void write_map(std::ofstream &f, const macromap &mp, const char *key)
 /*
  * Saves macros into the macrofile, overwriting the old one.
  */
-void macro_save( void ) 
+void macro_save( void )
 {
     std::ofstream f;
     f.open( get_macro_file().c_str() );
 
     f << "# WARNING: This file is entirely auto-generated." << std::endl
       << std::endl << "# Key Mappings:" << std::endl;
-    for (int mc = KC_DEFAULT; mc < KC_CONTEXT_COUNT; ++mc) {
+    for (int mc = KC_DEFAULT; mc < KC_CONTEXT_COUNT; ++mc)
+    {
         char keybuf[30] = "K:";
         if (mc)
             snprintf(keybuf, sizeof keybuf, "K%d:", mc);
@@ -546,63 +582,75 @@ void macro_save( void )
 
     f << "# Command Macros:" << std::endl;
     write_map(f, Macros, "M:");
-    
+
     f.close();
 }
 
 /*
- * Reads as many keypresses as are available (waiting for at least one), 
- * and returns them as a single keyseq. 
+ * Reads as many keypresses as are available (waiting for at least one),
+ * and returns them as a single keyseq.
  */
-static keyseq getch_mul( int (*rgetch)() = NULL ) 
+static keyseq getch_mul( int (*rgetch)() = NULL )
 {
-    keyseq keys; 
-    int a;
-    
+    keyseq keys;
+    int    a;
+
+    // Something's gone wrong with replaying keys if crawl needs to
+    // get new keys from the user.
+    if (crawl_state.is_replaying_keys())
+    {
+        mpr("(Key replay ran out of keys)");
+        crawl_state.cancel_cmd_repeat();
+        crawl_state.cancel_cmd_again();
+    }
+
     if (!rgetch)
         rgetch = m_getch;
 
     keys.push_back( a = rgetch() );
-    
-    // The a == 0 test is legacy code that I don't dare to remove. I 
-    // have a vague recollection of it being a kludge for conio support. 
-    while ((kbhit() || a == 0)) {
+
+    // The a == 0 test is legacy code that I don't dare to remove. I
+    // have a vague recollection of it being a kludge for conio support.
+    while (kbhit() || a == 0)
+    {
         keys.push_back( a = rgetch() );
     }
-    
+
     return (keys);
 }
 
-/* 
- * Replacement for getch(). Returns keys from the key buffer if available. 
- * If not, adds some content to the buffer, and returns some of it. 
+/*
+ * Replacement for getch(). Returns keys from the key buffer if available.
+ * If not, adds some content to the buffer, and returns some of it.
  */
-int getchm( int (*rgetch)() ) 
+int getchm( int (*rgetch)() )
 {
     return getchm( KC_DEFAULT, rgetch );
 }
 
-int getchm(KeymapContext mc, int (*rgetch)()) 
+int getchm(KeymapContext mc, int (*rgetch)())
 {
     int a;
-        
-    // Got data from buffer. 
+
+    // Got data from buffer.
     if ((a = macro_buf_get()) != -1)
         return (a);
 
     // Read some keys...
     keyseq keys = getch_mul(rgetch);
-    // ... and add them into the buffer
-    macro_buf_add_long( keys, Keymaps[mc] );
+    if (mc == KC_NONE)
+        macro_buf_add(keys);
+    else
+        macro_buf_add_long(keys, Keymaps[mc]);
 
     return (macro_buf_get());
 }
 
-/* 
- * Replacement for getch(). Returns keys from the key buffer if available. 
- * If not, adds some content to the buffer, and returns some of it. 
+/*
+ * Replacement for getch(). Returns keys from the key buffer if available.
+ * If not, adds some content to the buffer, and returns some of it.
  */
-int getch_with_command_macros( void ) 
+int getch_with_command_macros( void )
 {
     if (Buffer.size() == 0)
     {
@@ -620,11 +668,30 @@ int getch_with_command_macros( void )
 
 /*
  * Flush the buffer.  Later we'll probably want to give the player options
- * as to when this happens (ex. always before command input, casting failed).  
+ * as to when this happens (ex. always before command input, casting failed).
  */
 void flush_input_buffer( int reason )
 {
-    if (Options.flush_input[ reason ])
+    ASSERT(reason != FLUSH_KEY_REPLAY_CANCEL ||
+           crawl_state.is_replaying_keys() || crawl_state.cmd_repeat_start);
+
+    ASSERT(reason != FLUSH_ABORT_MACRO || is_processing_macro());
+
+    // Any attempt to flush means that the processing of the previously
+    // fetched keystroke is complete.
+    if (macro_keys_left == 0)
+        macro_keys_left = -1;
+
+    if (crawl_state.is_replaying_keys() && reason != FLUSH_ABORT_MACRO
+        && reason != FLUSH_KEY_REPLAY_CANCEL &&
+        reason != FLUSH_REPLAY_SETUP_FAILURE)
+    {
+        return;
+    }
+
+    if (Options.flush_input[ reason ] || reason == FLUSH_ABORT_MACRO
+        || reason == FLUSH_KEY_REPLAY_CANCEL
+        || reason == FLUSH_REPLAY_SETUP_FAILURE)
     {
         while (!Buffer.empty())
         {
@@ -633,6 +700,7 @@ void flush_input_buffer( int reason )
             if (key == KEY_MACRO_ENABLE_MORE)
                 Options.show_more_prompt = true;
         }
+        macro_keys_left = -1;
     }
 }
 
@@ -694,11 +762,15 @@ void macro_add_query( void )
                    "buggy") : "",
          (keymap ? "keymap" : "macro") );
 
-    keyseq key = getch_mul();
+    keyseq key;
+#ifdef USE_TILE
+    mouse_control mc(MOUSE_MODE_MACRO);
+#endif
+    key = getch_mul();
 
     cprintf( "%s" EOL, (vtostr( key )).c_str() ); // echo key to screen
 
-    if (mapref[key].size() > 0) 
+    if (mapref[key].size() > 0)
     {
         mprf(MSGCH_WARN, "Current Action: %s", vtostr(mapref[key]).c_str());
         mpr( "Do you wish to (r)edefine, (c)lear, or (a)bort?", MSGCH_PROMPT );
@@ -721,12 +793,12 @@ void macro_add_query( void )
 
     mpr( "Input Macro Action: ", MSGCH_PROMPT );
 
-    // Using getch_mul() here isn't very useful...  We'd like the 
+    // Using getch_mul() here isn't very useful...  We'd like the
     // flexibility to define multicharacter macros without having
     // to resort to editing files and restarting the game.  -- bwr
     // keyseq act = getch_mul();
 
-    char    buff[4096]; 
+    char    buff[4096];
     get_input_line(buff, sizeof buff);
 
     if (Options.macro_meta_entry)
@@ -741,52 +813,70 @@ void macro_add_query( void )
     redraw_screen();
 }
 
-
 /*
  * Initializes the macros.
  */
-int macro_init( void ) 
+static void _read_macros_from(const char* filename)
 {
     std::string s;
     std::ifstream f;
     keyseq key, action;
     bool keymap = false;
     KeymapContext keymc = KC_DEFAULT;
-      
-    f.open( get_macro_file().c_str() );
-    
-    while (f >> s) 
+
+    f.open( filename );
+
+    while (f >> s)
     {
         trim_string(s);  // remove white space from ends
 
-        if (s[0] == '#') { 
-            continue;                   // skip comments
-
-        } else if (s.substr(0, 2) == "K:") {
+        if (s[0] == '#')
+            continue;    // skip comments
+        else if (s.substr(0, 2) == "K:")
+        {
             key = parse_keyseq(s.substr(2));
             keymap = true;
             keymc  = KC_DEFAULT;
-                
-        } else if (s.length() >= 3 && s[0] == 'K' && s[2] == ':') {
+        }
+        else if (s.length() >= 3 && s[0] == 'K' && s[2] == ':')
+        {
             keymc  = KeymapContext( KC_DEFAULT + s[1] - '0' );
-            if (keymc >= KC_DEFAULT && keymc < KC_CONTEXT_COUNT) {
+            if (keymc >= KC_DEFAULT && keymc < KC_CONTEXT_COUNT)
+            {
                 key    = parse_keyseq(s.substr(3));
                 keymap = true;
             }
-        
-        } else if (s.substr(0, 2) == "M:") {
+        }
+        else if (s.substr(0, 2) == "M:")
+        {
             key = parse_keyseq(s.substr(2));
             keymap = false;
-
-        } else if (s.substr(0, 2) == "A:") {
+        }
+        else if (s.substr(0, 2) == "A:")
+        {
             action = parse_keyseq(s.substr(2));
             macro_add( (keymap ? Keymaps[keymc] : Macros), key, action );
         }
     }
-         
-    return (0);
 }
 
+int macro_init( void )
+{
+    _read_macros_from(get_macro_file().c_str());
+
+    const std::vector<std::string>& files = Options.additional_macro_files;
+    for (std::vector<std::string>::const_iterator it = files.begin();
+         it != files.end();
+         ++it)
+    {
+        _read_macros_from(it->c_str());
+    }
+
+
+    return 0;
+}
+
+
 void macro_userfn(const char *keys, const char *regname)
 {
     // TODO: Implement.
@@ -802,8 +892,103 @@ bool is_synthetic_key(int key)
     case KEY_MACRO_ENABLE_MORE:
     case KEY_MACRO_DISABLE_MORE:
     case KEY_MACRO_MORE_PROTECT:
+    case KEY_REPEAT_KEYS:
         return (true);
     default:
         return (false);
     }
 }
+
+key_recorder::key_recorder(key_recorder_callback cb, void* cb_data)
+    : paused(false), call_back(cb), call_back_data(cb_data)
+{
+    keys.clear();
+    macro_trigger_keys.clear();
+}
+
+void key_recorder::add_key(int key, bool reverse)
+{
+    if (paused)
+        return;
+
+    if (call_back)
+    {
+        // Don't record key if true
+        if ((*call_back)(this, key, reverse))
+            return;
+    }
+
+    if (reverse)
+        keys.push_front(key);
+    else
+        keys.push_back(key);
+}
+
+void key_recorder::remove_trigger_keys(int num_keys)
+{
+    ASSERT(num_keys >= 1);
+
+    if (paused)
+        return;
+
+    for (int i = 0; i < num_keys; i++)
+    {
+        ASSERT(keys.size() >= 1);
+
+        int key = keys[keys.size() - 1];
+
+        if (call_back)
+        {
+            // Key wasn't recorded in the first place, so no need to remove
+            // it
+            if ((*call_back)(this, key, true))
+                continue;
+        }
+
+        macro_trigger_keys.push_front(key);
+        keys.pop_back();
+    }
+}
+
+void key_recorder::clear()
+{
+    keys.clear();
+    macro_trigger_keys.clear();
+}
+
+void add_key_recorder(key_recorder* recorder)
+{
+    for (int i = 0, size = recorders.size(); i < size; i++)
+        ASSERT(recorders[i] != recorder);
+
+    recorders.push_back(recorder);
+}
+
+void remove_key_recorder(key_recorder* recorder)
+{
+    std::vector<key_recorder*>::iterator i;
+
+    for (i = recorders.begin(); i != recorders.end(); i++)
+        if (*i == recorder)
+        {
+            recorders.erase(i);
+            return;
+        }
+
+    end(1, true, "remove_key_recorder(): recorder not found\n");
+}
+
+// Add macro trigger keys to beginning of the buffer, then expand
+// them.
+void insert_macro_into_buff(const keyseq& keys)
+{
+    for (int i = (int) keys.size() - 1; i >= 0; i--)
+        macro_buf_add(keys[i], true);
+
+    macro_buf_apply_command_macro();
+}
+
+int get_macro_buf_size()
+{
+    return (Buffer.size());
+}
