@@ -1,8 +1,9 @@
 /*
  * File:       makeitem.h
  * Summary:    Item creation routines.
+ * Created by: haranp on Sat Apr 21 11:31:42 2007 UTC
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-22T16:20:45.803473Z $
+ *  Modified for Crawl Reference by $Author: dolorous $ on $Date: 2008-03-28T22:29:26.536127Z $
  */
 
 #ifndef MAKEITEM_H
@@ -21,9 +22,9 @@ enum item_make_species_type
     MAKE_ITEM_RANDOM_RACE = 250
 };
 
-int items( int allow_uniques, object_class_type force_class, int force_type, 
+int items( int allow_uniques, object_class_type force_class, int force_type,
            bool dont_place, int item_level, int item_race,
-           unsigned mapmask = 0 );
+           unsigned mapmask = 0, int force_ego = 0 );
 
 void item_colour( item_def &item );
 void init_rod_mp(item_def &item);
@@ -33,5 +34,6 @@ jewellery_type get_random_ring_type();
 jewellery_type get_random_amulet_type();
 armour_type get_random_body_armour_type(int level);
 armour_type get_random_armour_type(int item_level);
+void item_set_appearance(item_def &item);
 
 #endif
