@@ -1,8 +1,9 @@
 /*
  *  File:       mapdef.cc
  *  Summary:    Support code for Crawl des files.
+ *  Created by: dshaligram on Wed Nov 22 08:41:20 2006 UTC
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-07T11:58:54.306684Z $
  */
 
 #include <iostream>
@@ -14,7 +15,8 @@
 #include "AppHdr.h"
 #include "branch.h"
 #include "describe.h"
-#include "direct.h"
+#include "directn.h"
+#include "dungeon.h"
 #include "files.h"
 #include "initfile.h"
 #include "invent.h"
@@ -29,7 +31,7 @@
 #include "mon-util.h"
 #include "place.h"
 #include "stuff.h"
-#include "dungeon.h"
+#include "tags.h"
 
 static const char *map_section_names[] = {
     "",
@@ -89,7 +91,7 @@ static std::string split_key_item(const std::string &s,
 
     *arg = substitute;
     *separator = s[sep];
-    
+
     return ("");
 }
 
@@ -109,20 +111,20 @@ level_range::level_range(const raw_range &r)
 {
 }
 
-void level_range::write(FILE *outf) const
+void level_range::write(writer& outf) const
 {
-    writeShort(outf, branch);
-    writeShort(outf, shallowest);
-    writeShort(outf, deepest);
-    writeByte(outf, deny);
+    marshallShort(outf, branch);
+    marshallShort(outf, shallowest);
+    marshallShort(outf, deepest);
+    marshallByte(outf, deny);
 }
 
-void level_range::read(FILE *inf)
+void level_range::read(reader& inf)
 {
-    branch     = static_cast<branch_type>( readShort(inf) );
-    shallowest = readShort(inf);
-    deepest    = readShort(inf);
-    deny       = readByte(inf);
+    branch     = static_cast<branch_type>( unmarshallShort(inf) );
+    shallowest = unmarshallShort(inf);
+    deepest    = unmarshallShort(inf);
+    deny       = unmarshallByte(inf);
 }
 
 std::string level_range::str_depth_range() const
@@ -135,7 +137,7 @@ std::string level_range::str_depth_range() const
 
     if (shallowest == deepest)
         return make_stringf(":%d", shallowest);
-    
+
     return make_stringf(":%d-%d", shallowest, deepest);
 }
 
@@ -224,7 +226,7 @@ void level_range::parse_depth_range(const std::string &s, int *l, int *h)
         *h = 100;
         return;
     }
-    
+
     std::string::size_type hy = s.find('-');
     if (hy == std::string::npos)
     {
@@ -447,13 +449,13 @@ std::string map_lines::check_block_shuffle(const std::string &s)
 {
     const std::vector<std::string> segs = split_string("/", s);
     const unsigned seglen = segs[0].length();
-    
+
     for (int i = 1, size = segs.size(); i < size; ++i)
     {
         if (seglen != segs[i].length())
             return ("block shuffle segment length mismatch");
     }
-    
+
     return ("");
 }
 
@@ -463,7 +465,7 @@ std::string map_lines::check_shuffle(std::string &s)
         return ("use / for block shuffle, or multiple SHUFFLE: lines");
 
     s = clean_shuffle(s);
-    
+
     if (s.find('/') != std::string::npos)
         return check_block_shuffle(s);
 
@@ -506,7 +508,7 @@ std::string map_lines::parse_weighted_colours(const std::string &cspec,
     {
         std::string col = cspeclist[i];
         lowercase(col);
-        
+
         int weight = find_weight(col);
         if (weight == TAG_UNFOUND)
         {
@@ -522,7 +524,7 @@ std::string map_lines::parse_weighted_colours(const std::string &cspec,
                 trim_string(col);
             }
         }
-        
+
         const int colour = col == "none"? BLACK : str_to_colour(col, -1);
         if (colour != -1)
             colours.push_back(map_weighted_colour(colour, weight));
@@ -539,7 +541,7 @@ std::string map_lines::add_colour(const std::string &sub)
 
     if (s.empty())
         return ("");
-    
+
     int sep = 0;
     std::string key;
     std::string substitute;
@@ -563,7 +565,7 @@ std::string map_lines::add_subst(const std::string &sub)
 
     if (s.empty())
         return ("");
-    
+
     int sep = 0;
     std::string key;
     std::string substitute;
@@ -593,7 +595,7 @@ std::string map_lines::parse_nsubst_spec(const std::string &s,
     const int keyval = key == "*"? -1 : atoi(key.c_str());
     if (!keyval)
         return make_stringf("Illegal spec: %s", s.c_str());
-    
+
     glyph_replacements_t repl;
     err = parse_glyph_replacements(arg, repl);
     if (!err.empty())
@@ -613,7 +615,7 @@ std::string map_lines::add_nsubst(const std::string &s)
     std::string err = split_key_item(s, &key, &sep, &arg);
     if (!err.empty())
         return (err);
-    
+
     std::vector<std::string> segs = split_string("/", arg);
     for (int i = 0, size = segs.size(); i < size; ++i)
     {
@@ -642,7 +644,7 @@ std::string map_lines::add_shuffle(const std::string &raws)
 {
     std::string s = raws;
     const std::string err = check_shuffle(s);
-    
+
     if (err.empty())
         transforms.push_back( new shuffle_spec(s) );
 
@@ -729,7 +731,7 @@ void map_lines::clear_nsubsts()
 
 void map_lines::clear_substs()
 {
-    clear_transforms(map_transformer::TT_SUBST);    
+    clear_transforms(map_transformer::TT_SUBST);
 }
 
 int map_lines::width() const
@@ -824,7 +826,7 @@ void map_lines::overlay_colours(colour_spec &spec)
 {
     if (!colour_overlay.get())
         colour_overlay.reset( new Matrix<int>(width(), height(), BLACK) );
-    
+
     for (int y = 0, ysize = lines.size(); y < ysize; ++y)
     {
         std::string::size_type pos = 0;
@@ -906,7 +908,7 @@ void map_lines::resolve_shuffle(const std::string &shufflage)
 
     if (toshuffle.empty() || shuffled.empty())
         return;
-    
+
     for (int i = 0, size = lines.size(); i < size; ++i)
     {
         std::string &s = lines[i];
@@ -1050,7 +1052,7 @@ void map_lines::vmirror()
                 std::swap( (*colour_overlay)(j, i),
                            (*colour_overlay)(j, size - 1 - i) );
     }
-    
+
     vmirror_markers();
     solid_checked = false;
 }
@@ -1076,7 +1078,7 @@ void map_lines::hmirror()
                 std::swap( (*colour_overlay)(j, i),
                            (*colour_overlay)(map_width - 1 - j, i) );
     }
-    
+
     hmirror_markers();
     solid_checked = false;
 }
@@ -1142,7 +1144,7 @@ coord_def map_lines::find_first_glyph(const std::string &glyphs) const
 
 dlua_set_map::dlua_set_map(map_def *map)
 {
-    dlua.callfn("dgn_set_map", "m", map);    
+    dlua.callfn("dgn_set_map", "m", map);
 }
 
 dlua_set_map::~dlua_set_map()
@@ -1158,7 +1160,8 @@ map_def::map_def()
     : name(), tags(), place(), depths(), orient(), chance(),
       welcome_messages(), map(), mons(), items(), keyspecs(),
       prelude("dlprelude"), main("dlmain"), validate("dlvalidate"),
-      veto("dlveto"), index_only(false), cache_offset(0L)
+      veto("dlveto"), rock_colour(BLACK), floor_colour(BLACK),
+      index_only(false), cache_offset(0L)
 {
     init();
 }
@@ -1182,8 +1185,13 @@ void map_def::reinit()
 {
     items.clear();
     keyspecs.clear();
+    level_flags.clear();
+    branch_flags.clear();
+
     welcome_messages.clear();
 
+    rock_colour = floor_colour = BLACK;
+
     // Base chance; this is not a percentage.
     chance = 10;
 
@@ -1202,18 +1210,18 @@ int map_def::glyph_at(const coord_def &c) const
     return map(c);
 }
 
-void map_def::write_full(FILE *outf)
+void map_def::write_full(writer& outf)
 {
-    cache_offset = ftell(outf);
-    writeShort(outf, MAP_CACHE_VERSION);   // Level indicator.
-    writeString(outf, name);
+    cache_offset = outf.tell();
+    marshallShort(outf, MAP_CACHE_VERSION);   // Level indicator.
+    marshallString4(outf, name);
     prelude.write(outf);
     main.write(outf);
     validate.write(outf);
     veto.write(outf);
 }
 
-void map_def::read_full(FILE *inf)
+void map_def::read_full(reader& inf)
 {
     // There's a potential race-condition here:
     // - If someone modifies a .des file while there are games in progress,
@@ -1223,11 +1231,16 @@ void map_def::read_full(FILE *inf)
     // reloading the index), but it's easier to save the game at this
     // point and let the player reload.
 
-    if (readShort(inf) != MAP_CACHE_VERSION || readString(inf) != name)
+    const short fp_version = unmarshallShort(inf);
+    std::string fp_name;
+    unmarshallString4(inf, fp_name);
+    if (fp_version != MAP_CACHE_VERSION || fp_name != name)
+    {
         save_game(true,
                   make_stringf("Level file cache for %s is out-of-sync! "
                                "Please reload your game.",
                                file.c_str()).c_str());
+    }
 
     prelude.read(inf);
     main.read(inf);
@@ -1243,10 +1256,11 @@ void map_def::load()
     const std::string descache_base = get_descache_path(file, "");
     file_lock deslock(descache_base + ".lk", "rb", false);
     const std::string loadfile = descache_base + ".dsc";
-    FILE *inf = fopen(loadfile.c_str(), "rb");
-    fseek(inf, cache_offset, SEEK_SET);
+    FILE *fp = fopen(loadfile.c_str(), "rb");
+    fseek(fp, cache_offset, SEEK_SET);
+    reader inf(fp);
     read_full(inf);
-    fclose(inf);
+    fclose(fp);
 
     index_only = false;
 }
@@ -1266,49 +1280,49 @@ coord_def map_def::find_first_glyph(const std::string &s) const
     return map.find_first_glyph(s);
 }
 
-void map_def::write_index(FILE *outf) const
+void map_def::write_index(writer& outf) const
 {
     if (!cache_offset)
         end(1, false, "Map %s: can't write index - cache offset not set!",
             name.c_str());
-    writeString(outf, name);
-    writeString(outf, place_loaded_from.filename);
-    writeLong(outf, place_loaded_from.lineno);
-    writeShort(outf, orient);
-    writeLong(outf, chance);
-    writeLong(outf, cache_offset);
-    writeString(outf, tags);
+    marshallString4(outf, name);
+    marshallString4(outf, place_loaded_from.filename);
+    marshallLong(outf, place_loaded_from.lineno);
+    marshallShort(outf, orient);
+    marshallLong(outf, chance);
+    marshallLong(outf, cache_offset);
+    marshallString4(outf, tags);
     place.save(outf);
     write_depth_ranges(outf);
     prelude.write(outf);
 }
 
-void map_def::read_index(FILE *inf)
+void map_def::read_index(reader& inf)
 {
-    name         = readString(inf);
-    place_loaded_from.filename = readString(inf);
-    place_loaded_from.lineno   = readLong(inf);
-    orient       = static_cast<map_section_type>( readShort(inf) );
-    chance       = readLong(inf);
-    cache_offset = readLong(inf);
-    tags         = readString(inf);
+    unmarshallString4(inf, name);
+    unmarshallString4(inf, place_loaded_from.filename);
+    place_loaded_from.lineno   = unmarshallLong(inf);
+    orient       = static_cast<map_section_type>( unmarshallShort(inf) );
+    chance       = unmarshallLong(inf);
+    cache_offset = unmarshallLong(inf);
+    unmarshallString4(inf, tags);
     place.load(inf);
     read_depth_ranges(inf);
     prelude.read(inf);
     index_only   = true;
 }
 
-void map_def::write_depth_ranges(FILE *outf) const
+void map_def::write_depth_ranges(writer& outf) const
 {
-    writeShort(outf, depths.size());
+    marshallShort(outf, depths.size());
     for (int i = 0, sz = depths.size(); i < sz; ++i)
         depths[i].write(outf);
 }
 
-void map_def::read_depth_ranges(FILE *inf)
+void map_def::read_depth_ranges(reader& inf)
 {
     depths.clear();
-    const int nranges = readShort(inf);
+    const int nranges = unmarshallShort(inf);
     for (int i = 0; i < nranges; ++i)
     {
         level_range lr;
@@ -1329,7 +1343,7 @@ void map_def::set_file(const std::string &s)
 std::string map_def::run_lua(bool run_main)
 {
     dlua_set_map mset(this);
-    
+
     int err = prelude.load(dlua);
     if (err == -1000)
         lua_pushnil(dlua);
@@ -1366,7 +1380,7 @@ bool map_def::test_lua_boolchunk(dlua_chunk &chunk, bool defval, bool croak)
         if (croak)
             end(1, false, "Lua error: %s", validate.orig_error().c_str());
         else
-            mprf(MSGCH_WARN, "Lua error: %s", validate.orig_error().c_str());
+            mprf(MSGCH_ERROR, "Lua error: %s", validate.orig_error().c_str());
         return (result);
     }
     if (dlua.callfn("dgn_run_map", 1, 1))
@@ -1377,7 +1391,7 @@ bool map_def::test_lua_boolchunk(dlua_chunk &chunk, bool defval, bool croak)
             end(1, false, "Lua error: %s",
                 rewrite_chunk_errors(dlua.error).c_str());
         else
-            mprf(MSGCH_WARN, "Lua error: %s",
+            mprf(MSGCH_ERROR, "Lua error: %s",
                  rewrite_chunk_errors(dlua.error).c_str());
     }
     return (result);
@@ -1412,9 +1426,10 @@ std::string map_def::validate_map_def()
     if (!err.empty())
         return (err);
 
+    fixup();
     resolve();
     test_lua_validate(true);
-    
+
     if (orient == MAP_FLOAT || is_minivault())
     {
         if (map.width() > GXM - MAPGEN_BORDER * 2
@@ -1440,25 +1455,22 @@ std::string map_def::validate_map_def()
         }
     }
 
-    if (map.height() == 0)
-        return ("Must define map.");
-
     switch (orient)
     {
     case MAP_NORTH: case MAP_SOUTH:
-        if (map.height() >= GYM * 2 / 3)
+        if (map.height() > GYM * 2 / 3)
             return make_stringf("Map too large - height %d (max %d)",
                                 map.height(), GYM * 2 / 3);
         break;
     case MAP_EAST: case MAP_WEST:
-        if (map.width() >= GXM * 2 / 3)
+        if (map.width() > GXM * 2 / 3)
             return make_stringf("Map too large - width %d (max %d)",
                                 map.width(), GXM * 2 / 3);
         break;
     case MAP_NORTHEAST: case MAP_SOUTHEAST:
     case MAP_NORTHWEST: case MAP_SOUTHWEST:
     case MAP_FLOAT:
-        if (map.width() >= GXM * 2 / 3 || map.height() > GYM * 2 / 3)
+        if (map.width() > GXM * 2 / 3 || map.height() > GYM * 2 / 3)
             return make_stringf("Map too large - %dx%d (max %dx%d)",
                                 map.width(), map.height(),
                                 GXM * 2 / 3, GYM * 2 / 3);
@@ -1521,7 +1533,7 @@ coord_def map_def::float_dock()
           MAP_NORTHEAST, MAP_SOUTHEAST, MAP_NORTHWEST, MAP_SOUTHWEST };
     map_section_type which_orient = MAP_NONE;
     int norients = 0;
-    
+
     for (unsigned i = 0; i < sizeof(orients) / sizeof(*orients); ++i)
     {
         if (map.solid_borders(orients[i]) && can_dock(orients[i])
@@ -1597,7 +1609,7 @@ coord_def map_def::float_random_place() const
 
     if (GYM - 2 * minvborder < map.height())
         minvborder = (GYM - map.height()) / 2 - 1;
-    
+
     return coord_def(
         random_range(minhborder, GXM - minhborder - map.width()),
         random_range(minvborder, GYM - minvborder - map.height()));
@@ -1690,7 +1702,7 @@ void map_def::rotate(bool clock)
             { MAP_WEST,         MAP_NORTH       },
             { MAP_NORTHWEST,    MAP_NORTHEAST   },
         };
-        const int nrots = sizeof(clockrotate_orients) 
+        const int nrots = sizeof(clockrotate_orients)
                             / sizeof(*clockrotate_orients);
 
         const int refindex = !clock;
@@ -1705,8 +1717,7 @@ void map_def::rotate(bool clock)
 
 void map_def::normalise()
 {
-    // Minivaults are padded out with floor tiles, normal maps are
-    // padded out with rock walls.
+    // Pad out lines that are shorter than max.
     map.normalise(' ');
 }
 
@@ -1739,6 +1750,12 @@ bool map_def::has_tag_suffix(const std::string &suffix) const
         && tags.find(suffix + " ") != std::string::npos;
 }
 
+const keyed_mapspec *map_def::mapspec_for_key(int key) const
+{
+    keyed_specs::const_iterator i = keyspecs.find(key);
+    return i != keyspecs.end()? &i->second : NULL;
+}
+
 keyed_mapspec *map_def::mapspec_for_key(int key)
 {
     keyed_specs::iterator i = keyspecs.find(key);
@@ -1776,6 +1793,11 @@ std::string map_def::add_key_mons(const std::string &s)
     return add_key_field(s, &keyed_mapspec::set_mons);
 }
 
+std::string map_def::add_key_mask(const std::string &s)
+{
+    return add_key_field(s, &keyed_mapspec::set_mask);
+}
+
 std::vector<std::string> map_def::get_shuffle_strings() const
 {
     return map.get_shuffle_strings();
@@ -1798,10 +1820,9 @@ int mons_list::fix_demon(int demon) const
 {
     if (demon >= -1)
         return (demon);
-    
+
     demon = -100 - demon;
-    if (demon == DEMON_RANDOM)
-        demon = random2(DEMON_RANDOM);
+
     return (summon_any_demon( static_cast<demon_class_type>(demon) ));
 }
 
@@ -1809,12 +1830,12 @@ mons_spec mons_list::pick_monster(mons_spec_slot &slot)
 {
     int totweight = 0;
     mons_spec pick;
-    
+
     for (mons_spec_list::iterator i = slot.mlist.begin();
          i != slot.mlist.end(); ++i)
     {
         const int weight = i->genweight;
-        if (random2(totweight += weight) < weight)
+        if (x_chance_in_y(weight, totweight += weight))
         {
             pick = *i;
 
@@ -1832,7 +1853,7 @@ mons_spec mons_list::pick_monster(mons_spec_slot &slot)
         slot.mlist.push_back( pick );
         slot.fix_slot = false;
     }
-    
+
     if (pick.mid == MONS_WEAPON_MIMIC && !pick.fix_mons)
         pick.mid = random_range(MONS_GOLD_MIMIC, MONS_POTION_MIMIC);
 
@@ -1843,7 +1864,7 @@ mons_spec mons_list::get_monster(int index)
 {
     if (index < 0 || index >= (int) mons.size())
         return mons_spec(RANDOM_MONSTER);
-    
+
     return (pick_monster( mons[index] ));
 }
 
@@ -1888,16 +1909,51 @@ mons_list::mons_spec_slot mons_list::parse_mons_spec(std::string spec)
     for (int i = 0, ssize = specs.size(); i < ssize; ++i)
     {
         std::string s = specs[i];
+        std::vector<std::string> parts = split_string(";", s);
+        std::string mon_str = parts[0];
+
         mons_spec mspec;
 
-        mspec.genweight = find_weight(s);
+        if (parts.size() > 2)
+        {
+            error = make_stringf("Too many semi-colons for '%s' spec.",
+                                 mon_str.c_str());
+            return (slot);
+        }
+        else if (parts.size() == 2)
+        {
+            // TODO: Allow for a "fix_slot" type tag which will cause
+            // all monsters generated from this spec to have the
+            // exact same equipment.
+            std::string items_str = parts[1];
+            items_str = replace_all(items_str, "|", "/");
+
+            std::vector<std::string> segs = split_string(".", items_str);
+
+            if (segs.size() > NUM_MONSTER_SLOTS)
+            {
+                error = make_stringf("More items than monster item slots "
+                                     "for '%s'.", mon_str.c_str());
+                return (slot);
+            }
+
+            for (int j = 0, isize = segs.size(); j < isize; ++j)
+            {
+                error = mspec.items.add_item(segs[j], false);
+                if (!error.empty())
+                    return (slot);
+            }
+        }
+
+        mspec.genweight = find_weight(mon_str);
         if (mspec.genweight == TAG_UNFOUND || mspec.genweight <= 0)
             mspec.genweight = 10;
 
-        mspec.fix_mons = strip_tag(s, "fix_mons");
-        mspec.generate_awake = strip_tag(s, "generate_awake");
+        mspec.fix_mons       = strip_tag(mon_str, "fix_mons");
+        mspec.generate_awake = strip_tag(mon_str, "generate_awake");
+        mspec.patrolling     = strip_tag(mon_str, "patrolling");
 
-        std::string colour = strip_tag_prefix(s, "col:");
+        std::string colour = strip_tag_prefix(mon_str, "col:");
         if (!colour.empty())
         {
             mspec.colour = str_to_colour(colour, BLACK);
@@ -1909,27 +1965,45 @@ mons_list::mons_spec_slot mons_list::parse_mons_spec(std::string spec)
             }
         }
 
-        trim_string(s);
+        trim_string(mon_str);
 
-        if (s == "8")
+        if (mon_str == "8")
             mspec.mlevel = -8;
-        else if (s == "9")
+        else if (mon_str == "9")
             mspec.mlevel = -9;
-        else if (check_mimic(s, &mspec.mid, &mspec.fix_mons))
+        else if (check_mimic(mon_str, &mspec.mid, &mspec.fix_mons))
             ;
-        else if (s != "0")
+        else if (mon_str != "0")
         {
-            const mons_spec nspec = mons_by_name(s);
+            const mons_spec nspec = mons_by_name(mon_str);
 
             if (nspec.mid == MONS_PROGRAM_BUG)
             {
-                error = make_stringf("unrecognised monster \"%s\"", s.c_str());
+                error = make_stringf("unknown monster: \"%s\"",
+                                     mon_str.c_str());
                 return (slot);
             }
 
-            mspec.mid = nspec.mid;
-            mspec.monnum = nspec.monnum;
+            mspec.mid     = nspec.mid;
+            mspec.monbase = nspec.monbase;
+            mspec.number  = nspec.number;
         }
+
+        if (mspec.items.size() > 0)
+        {
+            if (mspec.mid == RANDOM_MONSTER)
+            {
+                error = "Can't give spec items to a random monster.";
+                return (slot);
+            };
+
+            if (mons_itemuse(mspec.mid) < MONUSE_STARTING_EQUIPMENT)
+            {
+                error = make_stringf("Monster '%s' can't use items.",
+                                     mon_str.c_str());
+            }
+        }
+
         slot.mlist.push_back(mspec);
     }
 
@@ -1946,7 +2020,7 @@ std::string mons_list::add_mons(const std::string &s, bool fix)
 
     if (fix)
         slotmons.fix_slot = true;
-    
+
     mons.push_back( slotmons );
 
     return (error);
@@ -1958,7 +2032,7 @@ std::string mons_list::set_mons(int index, const std::string &s)
 
     if (index < 0)
         return (error = make_stringf("Index out of range: %d", index));
-    
+
     mons_spec_slot slotmons = parse_mons_spec(s);
     if (!error.empty())
         return (error);
@@ -1979,12 +2053,12 @@ void mons_list::get_zombie_type(std::string s, mons_spec &spec) const
         " zombie", " skeleton", " simulacrum", NULL
     };
 
-    // This order must match zombie_types, indexed from one. 
+    // This order must match zombie_types, indexed from one.
     static const monster_type zombie_montypes[][2] =
-    {
-        { MONS_PROGRAM_BUG, MONS_PROGRAM_BUG },
-        { MONS_ZOMBIE_SMALL, MONS_ZOMBIE_LARGE },
-        { MONS_SKELETON_SMALL, MONS_SKELETON_LARGE },
+    {     // small               // large
+        { MONS_PROGRAM_BUG,      MONS_PROGRAM_BUG },
+        { MONS_ZOMBIE_SMALL,     MONS_ZOMBIE_LARGE },
+        { MONS_SKELETON_SMALL,   MONS_SKELETON_LARGE },
         { MONS_SIMULACRUM_SMALL, MONS_SIMULACRUM_LARGE }
     };
 
@@ -1996,8 +2070,8 @@ void mons_list::get_zombie_type(std::string s, mons_spec &spec) const
         {
             s = s.substr(spectre.length());
             spec.mid = MONS_SPECTRAL_THING;
-            spec.monnum = get_monster_by_name(s, true);
-            if (!mons_zombie_size(spec.monnum))
+            spec.monbase = get_monster_by_name(s, true);
+            if (!mons_zombie_size(spec.monbase))
                 spec.mid = MONS_PROGRAM_BUG;
             return;
         }
@@ -2007,10 +2081,10 @@ void mons_list::get_zombie_type(std::string s, mons_spec &spec) const
 
     s = s.substr(0, s.length() - strlen(zombie_types[mod - 1]));
     trim_string(s);
-    
-    spec.monnum = get_monster_by_name(s, true);
 
-    const int zombie_size = mons_zombie_size(spec.monnum);
+    spec.monbase = get_monster_by_name(s, true);
+
+    const int zombie_size = mons_zombie_size(spec.monbase);
     if (!zombie_size)
     {
         spec.mid = MONS_PROGRAM_BUG;
@@ -2024,11 +2098,89 @@ mons_spec mons_list::get_hydra_spec(const std::string &name) const
 {
     int nheads = atoi(name.c_str());
     if (nheads < 1)
-        nheads = 250; // Choose randomly, doesn't mean a 250-headed hydra.
+        nheads = MONS_PROGRAM_BUG;  // What can I say? :P
     else if (nheads > 19)
         nheads = 19;
 
-    return mons_spec(MONS_HYDRA, nheads);
+    return mons_spec(MONS_HYDRA, MONS_PROGRAM_BUG, nheads);
+}
+
+// Handle draconians specified as:
+// Exactly as in mon-data.h:
+//    yellow draconian or draconian knight - the monster specified.
+//
+// Others:
+//    any draconian => any random draconain
+//    any base draconian => any unspecialised coloured draconian.
+//    any nonbase draconian => any specialised coloured draconian.
+//    any <colour> draconian => any draconian of the colour.
+//    any nonbase <colour> draconian => any specialised drac of the colour.
+//
+mons_spec mons_list::drac_monspec(std::string name) const
+{
+    mons_spec spec;
+
+    spec.mid = get_monster_by_name(name, true);
+
+    // Check if it's a simple drac name, we're done.
+    if (spec.mid != MONS_PROGRAM_BUG)
+        return (spec);
+
+    spec.mid = RANDOM_DRACONIAN;
+
+    // Request for any draconian?
+    if (starts_with(name, "any "))
+        // Strip "any "
+        name = name.substr(4);
+
+    if (starts_with(name, "base "))
+    {
+        // Base dracs need no further work.
+        return (RANDOM_BASE_DRACONIAN);
+    }
+    else if (starts_with(name, "nonbase "))
+    {
+        spec.mid = RANDOM_NONBASE_DRACONIAN;
+        name = name.substr(8);
+    }
+
+    trim_string(name);
+
+    // Match "any draconian"
+    if (name == "draconian")
+        return (spec);
+
+    // Check for recognition again to match any (nonbase) <colour> draconian.
+    const monster_type colour = get_monster_by_name(name, true);
+    if (colour != MONS_PROGRAM_BUG)
+    {
+        spec.monbase = colour;
+        return (spec);
+    }
+
+    // Only legal possibility left is <colour> boss drac.
+    std::string::size_type wordend = name.find(' ');
+    if (wordend == std::string::npos)
+        return (MONS_PROGRAM_BUG);
+
+    std::string scolour = name.substr(0, wordend);
+    if ((spec.monbase = draconian_colour_by_name(scolour)) == MONS_PROGRAM_BUG)
+        return (MONS_PROGRAM_BUG);
+
+    name = trimmed_string(name.substr(wordend + 1));
+    spec.mid = get_monster_by_name(name, true);
+
+    // We should have a non-base draconian here.
+    if (spec.mid == MONS_PROGRAM_BUG
+        || mons_genus(spec.mid) != MONS_DRACONIAN
+        || spec.mid == MONS_DRACONIAN
+        || (spec.mid >= MONS_BLACK_DRACONIAN
+            && spec.mid <= MONS_PALE_DRACONIAN))
+    {
+        return (MONS_PROGRAM_BUG);
+    }
+
+    return (spec);
 }
 
 mons_spec mons_list::mons_by_name(std::string name) const
@@ -2036,30 +2188,28 @@ mons_spec mons_list::mons_by_name(std::string name) const
     lowercase(name);
 
     name = replace_all_of( name, "_", " " );
+    name = replace_all( name, "random", "any" );
 
     if (name == "nothing")
         return (-1);
-    
+
     // Special casery:
     if (name == "pandemonium demon")
         return (MONS_PANDEMONIUM_DEMON);
 
-    if (name == "random" || name == "random monster")
+    if (name == "any" || name == "any monster")
         return (RANDOM_MONSTER);
 
-    if (name == "any demon" || name == "demon" || name == "random demon")
+    if (name == "any demon")
         return (-100 - DEMON_RANDOM);
 
-    if (name == "any lesser demon" 
-            || name == "lesser demon" || name == "random lesser demon")
+    if (name == "any lesser demon" || name == "lesser demon")
         return (-100 - DEMON_LESSER);
 
-    if (name == "any common demon" 
-            || name == "common demon" || name == "random common demon")
+    if (name == "any common demon" || name == "common demon")
         return (-100 - DEMON_COMMON);
 
-    if (name == "any greater demon" 
-            || name == "greater demon" || name == "random greater demon")
+    if (name == "any greater demon" || name == "greater demon")
         return (-100 - DEMON_GREATER);
 
     if (name == "small zombie")
@@ -2094,6 +2244,9 @@ mons_spec mons_list::mons_by_name(std::string name) const
     if (spec.mid != MONS_PROGRAM_BUG)
         return (spec);
 
+    if (name.find("draconian") != std::string::npos)
+        return drac_monspec(name);
+
     return (get_monster_by_name(name, true));
 }
 
@@ -2113,7 +2266,7 @@ item_spec item_list::pick_item(item_spec_slot &slot)
          i != slot.ilist.end(); ++i)
     {
         const int weight = i->genweight;
-        if (random2(cumulative += weight) < weight)
+        if (x_chance_in_y(weight, cumulative += weight))
             spec = *i;
     }
 
@@ -2123,10 +2276,10 @@ item_spec item_list::pick_item(item_spec_slot &slot)
         slot.ilist.push_back(spec);
         slot.fix_slot = false;
     }
-    
+
     return (spec);
 }
-    
+
 item_spec item_list::get_item(int index)
 {
     if (index < 0 || index >= (int) items.size())
@@ -2134,7 +2287,7 @@ item_spec item_list::get_item(int index)
         const item_spec none;
         return (none);
     }
-    
+
     return (pick_item(items[index]));
 }
 
@@ -2149,7 +2302,7 @@ std::string item_list::add_item(const std::string &spec, bool fix)
             sp.fix_slot = true;
         items.push_back(sp);
     }
-    
+
     return (error);
 }
 
@@ -2168,8 +2321,117 @@ std::string item_list::set_item(int index, const std::string &spec)
         }
         items.push_back(sp);
     }
-    
-    return (error);    
+
+    return (error);
+}
+
+// TODO: More checking for innapropriate combinations, like the holy
+// wrath brand on a demonic weapon or the running ego on a helmet.
+static int str_to_ego(item_spec &spec, std::string ego_str)
+{
+    const char* armour_egos[] = {
+        "running",
+        "fire_resistance",
+        "cold_resistance",
+        "poison_resistance",
+        "see_invisible",
+        "darkness",
+        "strength",
+        "dexterity",
+        "intelligence",
+        "ponderousness",
+        "levitation",
+        "magic_resistance",
+        "protection",
+        "stealth",
+        "resistance",
+        "positive_energy",
+        "archmagi",
+        "preservation",
+        NULL
+    };
+
+    const char* weapon_brands[] = {
+        "flaming",
+        "freezing",
+        "holy_wrath",
+        "electrocution",
+        "orc_slaying",
+        "venom",
+        "protection",
+        "draining",
+        "speed",
+        "vorpal",
+        "flame",
+        "frost",
+        "vampiricism",
+        "disruption",
+        "pain",
+        "distortion",
+        "reaching",
+        "returning",
+        "confuse",
+        NULL
+    };
+
+    const char* missile_brands[] = {
+        "flame",
+        "ice",
+        "poisoned",
+        "poisoned_ii",
+        "curare",
+        "returning",
+        NULL
+    };
+
+    const char** name_lists[3] = {armour_egos, weapon_brands, missile_brands};
+
+    int armour_order[3]  = {0, 1, 2};
+    int weapon_order[3]  = {1, 0, 2};
+    int missile_order[3] = {2, 0, 1};
+
+    int *order;
+
+    switch(spec.base_type)
+    {
+    case OBJ_ARMOUR:
+        order = armour_order;
+        break;
+
+    case OBJ_WEAPONS:
+        order = weapon_order;
+        break;
+
+    case OBJ_MISSILES:
+        order = missile_order;
+        break;
+
+    default:
+        DEBUGSTR("Bad base_type for ego'd item.");
+        return 0;
+    }
+
+    const char** allowed = name_lists[order[0]];
+
+    for (int i = 0; allowed[i] != NULL; i++)
+    {
+        if (ego_str == allowed[i])
+            return (i + 1);
+    }
+
+    // Incompatible or non-existant ego type
+    for (int i = 1; i <= 2; i++)
+    {
+        const char** list = name_lists[order[i]];
+
+        for (int j = 0; list[j] != NULL; j++)
+            if (ego_str == list[j])
+                // Ego incompatible with base type.
+                return (-1);
+    }
+
+    // Non-existant ego
+    return 0;
 }
 
 item_spec item_list::parse_single_spec(std::string s)
@@ -2193,6 +2455,25 @@ item_spec item_list::parse_single_spec(std::string s)
     if (qty != TAG_UNFOUND)
         result.qty = qty;
 
+    std::string ego_str  = strip_tag_prefix(s, "ego:");
+    std::string race_str = strip_tag_prefix(s, "race:");
+    lowercase(ego_str);
+    lowercase(race_str);
+
+    if (race_str == "elven")
+        result.race = MAKE_ITEM_ELVEN;
+    else if (race_str == "dwarven")
+        result.race = MAKE_ITEM_DWARVEN;
+    else if (race_str == "orcish")
+        result.race = MAKE_ITEM_ORCISH;
+    else if (race_str == "none" || race_str == "no_race")
+        result.race = MAKE_ITEM_NO_RACE;
+    else if (!race_str.empty())
+    {
+        error = make_stringf("Bad race: %s", race_str.c_str());
+        return (result);
+    }
+
     if (strip_tag(s, "good_item"))
         result.level = MAKE_GOOD_ITEM;
     else
@@ -2201,7 +2482,10 @@ item_spec item_list::parse_single_spec(std::string s)
         if (number != TAG_UNFOUND)
         {
             if (number <= 0)
+            {
                 error = make_stringf("Bad item level: %d", number);
+                return (result);
+            }
 
             result.level = number;
         }
@@ -2217,14 +2501,20 @@ item_spec item_list::parse_single_spec(std::string s)
         if (uniq != TAG_UNFOUND)
         {
             if (uniq <= 0)
+            {
                 error = make_stringf("Bad uniq level: %d", uniq);
+                return (result);
+            }
             result.allow_uniques = uniq;
         }
     }
 
     // Clean up after any tag brain damage.
     trim_string(s);
-    
+
+    if (!ego_str.empty())
+        error = "Can't set an ego for random items.";
+
     // Completely random?
     if (s == "random" || s == "any" || s == "%")
         return (result);
@@ -2241,6 +2531,9 @@ item_spec item_list::parse_single_spec(std::string s)
     }
     else if (s == "$" || s == "gold")
     {
+        if (!ego_str.empty())
+            error = "Can't set an ego for gold.";
+
         result.base_type = OBJ_GOLD;
         result.sub_type = OBJ_RANDOM;
         return (result);
@@ -2248,6 +2541,7 @@ item_spec item_list::parse_single_spec(std::string s)
 
     if (s == "nothing")
     {
+        error.clear();
         result.base_type = OBJ_UNASSIGNED;
         return (result);
     }
@@ -2266,8 +2560,42 @@ item_spec item_list::parse_single_spec(std::string s)
     }
 
     // Check for actual item names.
+    error.clear();
     parse_raw_name(s, result);
 
+    if (!error.empty() || ego_str.empty())
+        return (result);
+
+    if (result.base_type != OBJ_WEAPONS
+        && result.base_type != OBJ_MISSILES
+        && result.base_type != OBJ_ARMOUR)
+    {
+        error = "An ego can only be applied to a weapon, missile or "
+            "armour.";
+        return (result);
+    }
+
+    if (ego_str == "none")
+    {
+        result.ego = -1;
+        return (result);
+    }
+
+    const int ego = str_to_ego(result, ego_str);
+
+    if (ego == 0)
+    {
+        error = make_stringf("No such ego as: %s", ego_str.c_str());
+        return (result);
+    }
+    else if (ego == -1)
+    {
+        error = make_stringf("Ego '%s' is incompatible with item '%s'.",
+                             ego_str.c_str(), s.c_str());
+        return (result);
+    }
+    result.ego = ego;
+
     return (result);
 }
 
@@ -2279,7 +2607,7 @@ void item_list::parse_random_by_class(std::string c, item_spec &spec)
         error = make_stringf("Bad item class: '%s'", c.c_str());
         return;
     }
-    
+
     for (int type = OBJ_WEAPONS; type < NUM_OBJECT_CLASSES; ++type)
     {
         if (c == item_class_name(type, true))
@@ -2345,10 +2673,8 @@ int subst_spec::value()
     int cumulative = 0;
     int chosen = 0;
     for (int i = 0, size = repl.size(); i < size; ++i)
-    {
-        if (random2(cumulative += repl[i].second) < repl[i].second)
+        if (x_chance_in_y(repl[i].second, cumulative += repl[i].second))
             chosen = repl[i].first;
-    }
 
     if (fix)
         frozen_value = chosen;
@@ -2439,7 +2765,7 @@ int colour_spec::get_colour()
     int chosen = BLACK;
     int cweight = 0;
     for (int i = 0, size = colours.size(); i < size; ++i)
-        if (random2(cweight += colours[i].second) < colours[i].second)
+        if (x_chance_in_y(colours[i].second, cweight += colours[i].second))
             chosen = colours[i].first;
     if (fix)
         fixed_colour = chosen;
@@ -2504,6 +2830,55 @@ std::string map_marker_spec::describe() const
     return ("unimplemented");
 }
 
+//////////////////////////////////////////////////////////////////////////
+// map_flags
+map_flags::map_flags()
+    : flags_set(0), flags_unset(0)
+{
+}
+
+void map_flags::clear()
+{
+    flags_set = flags_unset = 0;
+}
+
+typedef std::map<std::string, unsigned long> flag_map;
+
+map_flags map_flags::parse(const std::string flag_list[],
+                           const std::string &s) throw(std::string)
+{
+    map_flags mf;
+
+    const std::vector<std::string> segs = split_string("/", s);
+
+    flag_map flag_vals;
+    for (int i = 0; !flag_list[i].empty(); i++)
+        flag_vals[flag_list[i]] = 1 << i;
+
+    for (int i = 0, size = segs.size(); i < size; i++)
+    {
+        std::string flag   = segs[i];
+        bool        negate = false;
+
+        if (flag[0] == '!')
+        {
+            flag   = flag.substr(1);
+            negate = true;
+        }
+
+        flag_map::const_iterator val = flag_vals.find(flag);
+        if (val == flag_vals.end())
+            throw make_stringf("Unknown flag: '%s'", flag.c_str());
+
+        if (negate)
+            mf.flags_unset |= val->second;
+        else
+            mf.flags_set |= val->second;
+    }
+
+    return mf;
+}
+
 //////////////////////////////////////////////////////////////////////////
 // keyed_mapspec
 
@@ -2603,7 +2978,7 @@ feature_spec_list keyed_mapspec::parse_feature(const std::string &str)
                            str.c_str());
     else
         list.push_back( feature_spec( ftype, weight ) );
-    
+
     return (list);
 }
 
@@ -2631,9 +3006,39 @@ std::string keyed_mapspec::set_item(const std::string &s, bool fix)
     return (err);
 }
 
+std::string keyed_mapspec::set_mask(const std::string &s, bool garbage)
+{
+    UNUSED(garbage);
+
+    err.clear();
+
+    try
+    {
+        static std::string flag_list[] =
+            {"vault", "no_item_gen", "no_monster_gen", "no_pool_fixup",
+             "no_secret_doors", "opaque", ""};
+        map_mask = map_flags::parse(flag_list, s);
+    }
+    catch (const std::string &error)
+    {
+        err = error;
+        return (err);
+    }
+
+    // If not also a KFEAT...
+    if (feat.feats.size() == 0)
+    {
+        feature_spec fsp(-1, 10);
+        fsp.glyph = key_glyph;
+        feat.feats.push_back(fsp);
+    }
+
+    return (err);
+}
+
 feature_spec keyed_mapspec::get_feat()
 {
-    return feat.get_feat(key_glyph);
+    return feat.get_feat('.');
 }
 
 mons_list &keyed_mapspec::get_monsters()
@@ -2646,6 +3051,11 @@ item_list &keyed_mapspec::get_items()
     return (item);
 }
 
+map_flags &keyed_mapspec::get_mask()
+{
+    return (map_mask);
+}
+
 //////////////////////////////////////////////////////////////////////////
 // feature_slot
 
@@ -2663,11 +3073,11 @@ feature_spec feature_slot::get_feat(int def_glyph)
         chosen_feat.feat = -1;
         chosen_feat.glyph = def_glyph;
     }
-    
+
     for (int i = 0, size = feats.size(); i < size; ++i)
     {
         const feature_spec &feat = feats[i];
-        if (random2(tweight += feat.genweight) < feat.genweight)
+        if (x_chance_in_y(feat.genweight, tweight += feat.genweight))
             chosen_feat = feat;
     }
 
