@@ -1,6 +1,9 @@
 /*
  * mapdef.h:
  * Header for map structures used by the level compiler.
+ * Created by: dshaligram on Wed Nov 22 08:41:20 2006 UTC
+ *
+ * Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-05-26T12:35:56.942009Z $
  *
  * NOTE: When we refer to map, this could be a full map, filling an entire
  * level or a minivault that occupies just a portion of the level.
@@ -62,19 +65,19 @@ public:
 
     void set(int s, int d = -1);
     void set(const std::string &branch, int s, int d) throw (std::string);
-    
+
     void reset();
     bool matches(const level_id &) const;
     bool matches(int depth) const;
 
-    void write(FILE *) const;
-    void read(FILE *);
+    void write(writer&) const;
+    void read(reader&);
 
     bool valid() const;
     int span() const;
 
     static level_range parse(std::string lr) throw (std::string);
-    
+
     std::string describe() const;
     std::string str_depth_range() const;
 
@@ -108,7 +111,7 @@ public:
         TT_MARKER,
         TT_COLOUR
     };
-    
+
 public:
     virtual ~map_transformer() = 0;
     virtual std::string apply_transform(map_lines &map) = 0;
@@ -126,9 +129,9 @@ public:
     {
         return (foo);
     }
-    
+
     int value();
-    
+
     std::string apply_transform(map_lines &map);
     transform_type type() const;
     std::string describe() const;
@@ -139,7 +142,7 @@ private:
     int foo;        // The thing to replace.
     bool fix;       // If true, the first replacement fixes the value.
     int frozen_value;
-    
+
     glyph_replacements_t repl;
 };
 
@@ -150,7 +153,7 @@ public:
     std::string apply_transform(map_lines &map);
     transform_type type() const { return TT_NSUBST; }
     std::string describe() const;
-    
+
 public:
     int key;
     std::vector<subst_spec> specs;
@@ -158,7 +161,7 @@ public:
 
 typedef std::pair<int, int> map_weighted_colour;
 typedef std::vector<map_weighted_colour> map_colour_list;
-struct colour_spec : public map_transformer
+class colour_spec : public map_transformer
 {
 public:
     colour_spec(int _key, bool _fix, const map_colour_list &clist)
@@ -178,15 +181,16 @@ public:
     map_colour_list colours;
 };
 
-struct shuffle_spec : public map_transformer
+class shuffle_spec : public map_transformer
 {
+ public:
     std::string shuffle;
 
     shuffle_spec(const std::string &spec)
         : shuffle(spec)
     {
     }
-    
+
     std::string apply_transform(map_lines &map);
     transform_type type() const;
     std::string describe() const;
@@ -196,8 +200,9 @@ struct shuffle_spec : public map_transformer
     }
 };
 
-struct map_marker_spec : public map_transformer
+class map_marker_spec : public map_transformer
 {
+ public:
     int key;
     std::string marker;
 
@@ -219,7 +224,7 @@ public:
     map_lines &operator = (const map_lines &);
 
     bool in_map(const coord_def &pos) const;
-    
+
     void add_line(const std::string &s);
     std::string add_nsubst(const std::string &st);
     std::string add_subst(const std::string &st);
@@ -236,7 +241,7 @@ public:
     std::vector<coord_def> find_glyph(int glyph) const;
     coord_def find_first_glyph(int glyph) const;
     coord_def find_first_glyph(const std::string &glyphs) const;
-    
+
     void set_orientation(const std::string &s);
 
     int width() const;
@@ -244,7 +249,7 @@ public:
 
     int glyph(int x, int y) const;
     bool is_solid(int gly) const;
-    
+
     bool solid_borders(map_section_type border);
 
     std::string apply_transforms();
@@ -261,18 +266,18 @@ public:
 
     void add_marker(map_marker *marker);
     std::string add_feature_marker(const std::string &desc);
-    
+
     void apply_markers(const coord_def &pos);
     void apply_colours(const coord_def &pos);
     void apply_overlays(const coord_def &pos);
-    
+
     const std::vector<std::string> &get_lines() const;
     std::vector<std::string> &get_lines();
     std::vector<std::string> get_shuffle_strings() const;
     std::vector<std::string> get_subst_strings() const;
 
     int operator () (const coord_def &c) const;
-    
+
 private:
     void init_from(const map_lines &map);
     void clear_transforms();
@@ -285,7 +290,7 @@ private:
     void rotate_marker(map_marker *, int par);
     void translate_marker(void (map_lines::*xform)(map_marker *, int par),
                           int par = 0);
-    
+
     void resolve_shuffle(const std::string &shuffle);
     void subst(std::string &s, subst_spec &spec);
     void subst(subst_spec &);
@@ -313,33 +318,103 @@ private:
     friend class shuffle_spec;
     friend class map_marker_spec;
     friend class colour_spec;
-    
+
 private:
     std::vector<map_transformer *> transforms;
     std::vector<map_marker *> markers;
     std::vector<std::string> lines;
     std::auto_ptr< Matrix<int> > colour_overlay;
-    
+
     int map_width;
     bool solid_north, solid_east, solid_south, solid_west;
     bool solid_checked;
 };
 
-struct mons_spec
+enum item_spec_type
 {
+    ISPEC_GOOD   = -2,
+    ISPEC_SUPERB = -3
+};
+
+struct item_spec
+{
+    int genweight;
+
+    object_class_type base_type;
+    int sub_type;
+    int ego;
+    int allow_uniques;
+    int level;
+    int race;
+    int qty;
+
+    item_spec() : genweight(10), base_type(OBJ_RANDOM), sub_type(OBJ_RANDOM),
+        ego(0), allow_uniques(1), level(-1), race(MAKE_ITEM_RANDOM_RACE),
+        qty(0)
+    {
+    }
+};
+typedef std::vector<item_spec> item_spec_list;
+
+class item_list
+{
+public:
+    item_list() : items() { }
+
+    void clear();
+
+    item_spec get_item(int index);
+    size_t size() const { return items.size(); }
+
+    std::string add_item(const std::string &spec, bool fix = false);
+    std::string set_item(int index, const std::string &spec);
+
+private:
+    struct item_spec_slot
+    {
+        item_spec_list ilist;
+        bool fix_slot;
+
+        item_spec_slot() : ilist(), fix_slot(false)
+        {
+        }
+    };
+
+private:
+    item_spec item_by_specifier(const std::string &spec);
+    item_spec_slot parse_item_spec(std::string spec);
+    item_spec parse_single_spec(std::string s);
+    void parse_raw_name(std::string name, item_spec &spec);
+    void parse_random_by_class(std::string c, item_spec &spec);
+    item_spec pick_item(item_spec_slot &slot);
+
+private:
+    std::vector<item_spec_slot> items;
+    std::string error;
+};
+
+class mons_spec
+{
+ public:
     int  mid;
-    int  monnum;              // The zombified monster for zombies, or head
-                              // count for hydras.
+    monster_type monbase;     // Base monster for zombies and dracs.
+    int  number;              // Head count for hydras
     int  genweight, mlevel;
     bool fix_mons;
     bool generate_awake;
-    int colour;
+    bool patrolling;
+    int  colour;
 
-    mons_spec(int id = RANDOM_MONSTER, int num = 250,
+    item_list items;
+
+    mons_spec(int id = RANDOM_MONSTER,
+              monster_type base = MONS_PROGRAM_BUG,
+              int num = 0,
               int gw = 10, int ml = 0,
-              bool _fixmons = false, bool awaken = false)
-        : mid(id), monnum(num), genweight(gw), mlevel(ml), fix_mons(_fixmons),
-        generate_awake(awaken), colour(BLACK)
+              bool _fixmons = false, bool awaken = false, bool patrol = false)
+        : mid(id), monbase(base), number(num), genweight(gw), mlevel(ml),
+          fix_mons(_fixmons), generate_awake(awaken), patrolling(false),
+          colour(BLACK), items()
     {
     }
 };
@@ -380,6 +455,7 @@ private:
 
 private:
     mons_spec mons_by_name(std::string name) const;
+    mons_spec drac_monspec(std::string name) const;
     void get_zombie_type(std::string s, mons_spec &spec) const;
     mons_spec get_hydra_spec(const std::string &name) const;
     mons_spec get_zombified_monster(const std::string &name,
@@ -394,67 +470,6 @@ private:
     std::string error;
 };
 
-enum item_spec_type
-{
-    ISPEC_GOOD   = -2,
-    ISPEC_SUPERB = -3
-};
-
-struct item_spec
-{
-    int genweight;
-    
-    object_class_type base_type;
-    int sub_type;
-    int allow_uniques;
-    int level;
-    int race;
-    int qty;
-
-    item_spec() : genweight(10), base_type(OBJ_RANDOM), sub_type(OBJ_RANDOM),
-        allow_uniques(1), level(-1), race(MAKE_ITEM_RANDOM_RACE), qty(0)
-    {
-    }
-};
-typedef std::vector<item_spec> item_spec_list;
-
-class item_list
-{
-public:
-    item_list() : items() { }
-
-    void clear();
-
-    item_spec get_item(int index);
-    size_t size() const { return items.size(); }
-
-    std::string add_item(const std::string &spec, bool fix = false);
-    std::string set_item(int index, const std::string &spec);
-
-private:
-    struct item_spec_slot
-    {
-        item_spec_list ilist;
-        bool fix_slot;
-
-        item_spec_slot() : ilist(), fix_slot(false)
-        {
-        }
-    };
-    
-private:
-    item_spec item_by_specifier(const std::string &spec);
-    item_spec_slot parse_item_spec(std::string spec);
-    item_spec parse_single_spec(std::string s);
-    void parse_raw_name(std::string name, item_spec &spec);
-    void parse_random_by_class(std::string c, item_spec &spec);
-    item_spec pick_item(item_spec_slot &slot);
-
-private:
-    std::vector<item_spec_slot> items;
-    std::string error;
-};
-
 struct feature_spec
 {
     int genweight;
@@ -480,14 +495,27 @@ struct feature_slot
     feature_spec get_feat(int default_glyph);
 };
 
+struct map_flags
+{
+    unsigned long flags_set, flags_unset;
+
+    map_flags();
+    void clear();
+
+    static map_flags parse(const std::string flag_list[],
+                           const std::string &s) throw(std::string);
+};
+
+
 struct keyed_mapspec
 {
 public:
     int          key_glyph;
-    
+
     feature_slot feat;
     item_list    item;
     mons_list    mons;
+    map_flags    map_mask;
 
 public:
     keyed_mapspec();
@@ -495,10 +523,12 @@ public:
     std::string set_feat(const std::string &s, bool fix);
     std::string set_mons(const std::string &s, bool fix);
     std::string set_item(const std::string &s, bool fix);
+    std::string set_mask(const std::string &s, bool garbage);
 
     feature_spec get_feat();
-    mons_list &get_monsters();
-    item_list &get_items();
+    mons_list   &get_monsters();
+    item_list   &get_items();
+    map_flags   &get_mask();
 
 private:
     std::string err;
@@ -541,6 +571,31 @@ struct map_file_place
     }
 };
 
+/////////////////////////////////////////////////////////////////////////////
+// map_def: map definitions for maps loaded from .des files.
+//
+// Please read this before changing map_def.
+//
+// When adding Lua-visible fields to map_def, note that there are two
+// kinds of fields:
+//
+// * Fields that determine placement of the map, or are unchanging,
+//   such as "place", "depths" (determine placement), or "name" (does
+//   not change between different evaluations of the map). Such fields
+//   must be reset to their default values in map_def::init() if they
+//   determine placement, or just initialised in the constructor if
+//   they will not change.
+//
+// * Fields that do not determine placement and may change between
+//   different uses of the map (such as "mons", "items",
+//   "level_flags", etc.). Such fields must be reset to their default
+//   values in map_def::reinit(), which is called before the map is
+//   used.
+//
+// If you do not do this, maps will not work correctly, and will break
+// in obscure, hard-to-find ways. The level-compiler will not (cannot)
+// warn you.
+//
 class map_def
 {
 public:
@@ -558,6 +613,8 @@ public:
     mons_list       mons;
     item_list       items;
 
+    map_flags       level_flags, branch_flags;
+
     keyed_specs     keyspecs;
 
     dlua_chunk      prelude, main, validate, veto;
@@ -566,6 +623,8 @@ public:
 
     map_def         *original;
 
+    unsigned char   rock_colour, floor_colour;
+
 private:
     // This map has been loaded from an index, and not fully realised.
     bool            index_only;
@@ -582,16 +641,16 @@ public:
     bool in_map(const coord_def &p) const;
 
     coord_def size() const { return coord_def(map.width(), map.height()); }
-    
+
     std::vector<coord_def> find_glyph(int glyph) const;
     coord_def find_first_glyph(int glyph) const;
     coord_def find_first_glyph(const std::string &glyphs) const;
-    
-    void write_index(FILE *) const;
-    void write_full(FILE *);
 
-    void read_index(FILE *);
-    void read_full(FILE *);
+    void write_index(writer&) const;
+    void write_full(writer&);
+
+    void read_index(reader&);
+    void read_full(reader&);
 
     void set_file(const std::string &s);
     std::string run_lua(bool skip_main);
@@ -615,18 +674,20 @@ public:
     void fixup();
 
     bool is_usable_in(const level_id &lid) const;
-    
+
     keyed_mapspec *mapspec_for_key(int key);
+    const keyed_mapspec *mapspec_for_key(int key) const;
 
     bool has_depth() const;
     void add_depth(const level_range &depth);
     void add_depths(depth_ranges::const_iterator s,
                     depth_ranges::const_iterator e);
-    
+
     std::string add_key_item(const std::string &s);
     std::string add_key_mons(const std::string &s);
     std::string add_key_feat(const std::string &s);
-    
+    std::string add_key_mask(const std::string &s);
+
     bool can_dock(map_section_type) const;
     coord_def dock_pos(map_section_type) const;
     coord_def float_dock();
@@ -666,13 +727,13 @@ public:
                     && c.y >= 0 && c.y < map.map.height());
         }
     };
-    
+
 private:
-    void write_depth_ranges(FILE *) const;
-    void read_depth_ranges(FILE *);
+    void write_depth_ranges(writer&) const;
+    void read_depth_ranges(reader&);
     bool test_lua_boolchunk(dlua_chunk &, bool def = false, bool croak = false);
     std::string rewrite_chunk_errors(const std::string &s) const;
-    
+
     std::string add_key_field(
         const std::string &s,
         std::string (keyed_mapspec::*set_field)(
