@@ -1,16 +1,20 @@
 /*
  *  File:       mapmark.cc
  *  Summary:    Level markers (annotations).
+ *  Created by: dshaligram on Sat Jul 21 12:17:29 2007 UTC
+ *
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-17T11:11:48.745321Z $
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
- *  
  */
 
 #include "AppHdr.h"
+
+#include <algorithm>
+
 #include "mapmark.h"
 
 #include "clua.h"
-#include "direct.h"
+#include "directn.h"
 #include "libutil.h"
 #include "luadgn.h"
 #include "stuff.h"
@@ -49,13 +53,13 @@ void map_marker::activate(bool)
 {
 }
 
-void map_marker::write(tagHeader &outf) const
+void map_marker::write(writer &outf) const
 {
     marshallShort(outf, type);
     marshallCoord(outf, pos);
 }
 
-void map_marker::read(tagHeader &inf)
+void map_marker::read(reader &inf)
 {
     // Don't read type! The type has to be read by someone who knows how
     // to look up the unmarshall function.
@@ -72,7 +76,7 @@ std::string map_marker::property(const std::string &pname) const
     return ("");
 }
 
-map_marker *map_marker::read_marker(tagHeader &inf)
+map_marker *map_marker::read_marker(reader &inf)
 {
     const map_marker_type type =
         static_cast<map_marker_type>(unmarshallShort(inf));
@@ -109,13 +113,13 @@ map_feature_marker::map_feature_marker(
 {
 }
 
-void map_feature_marker::write(tagHeader &outf) const
+void map_feature_marker::write(writer &outf) const
 {
     this->map_marker::write(outf);
     marshallShort(outf, feat);
 }
 
-void map_feature_marker::read(tagHeader &inf)
+void map_feature_marker::read(reader &inf)
 {
     map_marker::read(inf);
     feat = static_cast<dungeon_feature_type>(unmarshallShort(inf));
@@ -126,7 +130,7 @@ map_marker *map_feature_marker::clone() const
     return new map_feature_marker(pos, feat);
 }
 
-map_marker *map_feature_marker::read(tagHeader &inf, map_marker_type)
+map_marker *map_feature_marker::read(reader &inf, map_marker_type)
 {
     map_marker *mapf = new map_feature_marker();
     mapf->read(inf);
@@ -161,14 +165,24 @@ map_lua_marker::map_lua_marker()
 {
 }
 
-map_lua_marker::map_lua_marker(const std::string &s, const std::string &)
+map_lua_marker::map_lua_marker(const std::string &s, const std::string &,
+                               bool mapdef_marker)
     : map_marker(MAT_LUA_MARKER, coord_def()), initialised(false)
 {
     lua_stack_cleaner clean(dlua);
-    if (dlua.loadstring(("return " + s).c_str(), "lua_marker"))
-        mprf(MSGCH_WARN, "lua_marker load error: %s", dlua.error.c_str());
-    if (!dlua.callfn("dgn_run_map", 1, 1))
-        mprf(MSGCH_WARN, "lua_marker exec error: %s", dlua.error.c_str());
+    if (mapdef_marker)
+    {
+        if (dlua.loadstring(("return " + s).c_str(), "lua_marker"))
+            mprf(MSGCH_ERROR, "lua_marker load error: %s", dlua.error.c_str());
+        if (!dlua.callfn("dgn_run_map", 1, 1))
+            mprf(MSGCH_ERROR, "lua_marker exec error: %s", dlua.error.c_str());
+    }
+    else
+    {
+        if (dlua.execstring(("return " + s).c_str(), "lua_marker_mapless", 1))
+            mprf(MSGCH_ERROR, "lua_marker_mapless exec error: %s",
+                 dlua.error.c_str());
+    }
     check_register_table();
 }
 
@@ -196,7 +210,7 @@ void map_lua_marker::check_register_table()
 {
     if (!lua_istable(dlua, -1))
     {
-        mprf(MSGCH_WARN, "lua_marker: Expected table, didn't get it.");
+        mprf(MSGCH_ERROR, "lua_marker: Expected table, didn't get it.");
         initialised = false;
         return;
     }
@@ -220,19 +234,19 @@ bool map_lua_marker::get_table() const
     return (lua_istable(dlua, -1));
 }
 
-void map_lua_marker::write(tagHeader &th) const
+void map_lua_marker::write(writer &outf) const
 {
-    map_marker::write(th);
-    
+    map_marker::write(outf);
+
     lua_stack_cleaner clean(dlua);
     bool init = initialised;
     if (!get_table())
     {
-        mprf(MSGCH_WARN, "Couldn't find table.");
+        mprf(MSGCH_ERROR, "Couldn't find table.");
         init = false;
     }
-    
-    marshallByte(th, init);
+
+    marshallByte(outf, init);
     if (!init)
         return;
 
@@ -247,7 +261,7 @@ void map_lua_marker::write(tagHeader &th) const
         end(1, false, "lua_marker: couldn't save read function: %s",
             reader.error.c_str());
 
-    marshallString(th, reader.compiled_chunk());
+    marshallString(outf, reader.compiled_chunk());
 
     // Okay, saved the reader. Now ask the writer to do its thing.
 
@@ -255,28 +269,28 @@ void map_lua_marker::write(tagHeader &th) const
     get_table();
     lua_pushstring(dlua, "write");
     lua_pushlightuserdata(dlua, const_cast<map_lua_marker*>(this));
-    lua_pushlightuserdata(dlua, &th);
+    lua_pushlightuserdata(dlua, &outf);
 
     if (!dlua.callfn("dlua_marker_method", 4))
         end(1, false, "lua_marker::write error: %s", dlua.error.c_str());
 }
 
-void map_lua_marker::read(tagHeader &th)
+void map_lua_marker::read(reader &inf)
 {
-    map_marker::read(th);
-    
-    if (!(initialised = unmarshallByte(th)))
+    map_marker::read(inf);
+
+    if (!(initialised = unmarshallByte(inf)))
         return;
 
     lua_stack_cleaner cln(dlua);
     // Read the Lua chunk we saved.
-    const std::string compiled = unmarshallString(th, LUA_CHUNK_MAX_SIZE);
+    const std::string compiled = unmarshallString(inf, LUA_CHUNK_MAX_SIZE);
 
     dlua_chunk chunk = dlua_chunk::precompiled(compiled);
     if (chunk.load(dlua))
         end(1, false, "lua_marker::read error: %s", chunk.error.c_str());
     dlua_push_userdata(dlua, this, MAPMARK_METATABLE);
-    lua_pushlightuserdata(dlua, &th);
+    lua_pushlightuserdata(dlua, &inf);
     if (!dlua.callfn("dlua_marker_read", 3, 1))
         end(1, false, "lua_marker::read error: %s", dlua.error.c_str());
 
@@ -284,10 +298,10 @@ void map_lua_marker::read(tagHeader &th)
     check_register_table();
 }
 
-map_marker *map_lua_marker::read(tagHeader &th, map_marker_type)
+map_marker *map_lua_marker::read(reader &inf, map_marker_type)
 {
     map_marker *marker = new map_lua_marker;
-    marker->read(th);
+    marker->read(inf);
     return (marker);
 }
 
@@ -308,7 +322,7 @@ bool map_lua_marker::callfn(const char *fn, bool warn_err, int args) const
     }
     const bool res = dlua.callfn("dlua_marker_method", args, 1);
     if (!res && warn_err)
-        mprf(MSGCH_WARN, "mlua error: %s", dlua.error.c_str());
+        mprf(MSGCH_ERROR, "mlua error: %s", dlua.error.c_str());
     return (res);
 }
 
@@ -326,8 +340,10 @@ void map_lua_marker::notify_dgn_event(const dgn_event &e)
     push_fn_args("event");
     clua_push_dgn_event(dlua, &e);
     if (!dlua.callfn("dlua_marker_method", 4, 0))
-        mprf(MSGCH_WARN, "notify_dgn_event: Lua error: %s",
+    {
+        mprf(MSGCH_ERROR, "notify_dgn_event: Lua error: %s",
              dlua.error.c_str());
+    }
 }
 
 std::string map_lua_marker::call_str_fn(const char *fn) const
@@ -363,17 +379,26 @@ std::string map_lua_marker::property(const std::string &pname) const
     std::string result;
     if (lua_isstring(dlua, -1))
         result = lua_tostring(dlua, -1);
-    return (result);    
+    return (result);
 }
 
 map_marker *map_lua_marker::parse(
     const std::string &s, const std::string &ctx) throw (std::string)
 {
-    if (s.find("lua:") != 0)
+    std::string raw           = s;
+    bool        mapdef_marker = true;
+
+    if (s.find("lua:") == 0)
+        strip_tag(raw, "lua:", true);
+    else if (s.find("lua_mapless:") == 0)
+    {
+        strip_tag(raw, "lua_mapless:", true);
+        mapdef_marker = false;
+    }
+    else
         return (NULL);
-    std::string raw = s;
-    strip_tag(raw, "lua:", true);
-    map_lua_marker *mark = new map_lua_marker(raw, ctx);
+
+    map_lua_marker *mark = new map_lua_marker(raw, ctx, mapdef_marker);
     if (!mark->initialised)
     {
         delete mark;
@@ -392,24 +417,24 @@ map_corruption_marker::map_corruption_marker(const coord_def &p,
 {
 }
 
-void map_corruption_marker::write(tagHeader &out) const
+void map_corruption_marker::write(writer &out) const
 {
     map_marker::write(out);
     marshallShort(out, duration);
     marshallShort(out, radius);
 }
 
-void map_corruption_marker::read(tagHeader &in)
+void map_corruption_marker::read(reader &in)
 {
     map_marker::read(in);
     duration = unmarshallShort(in);
     radius   = unmarshallShort(in);
 }
 
-map_marker *map_corruption_marker::read(tagHeader &th, map_marker_type)
+map_marker *map_corruption_marker::read(reader &in, map_marker_type)
 {
     map_corruption_marker *mc = new map_corruption_marker();
-    mc->read(th);
+    mc->read(in);
     return (mc);
 }
 
@@ -440,7 +465,7 @@ map_wiz_props_marker::map_wiz_props_marker(
 {
 }
 
-void map_wiz_props_marker::write(tagHeader &outf) const
+void map_wiz_props_marker::write(writer &outf) const
 {
     this->map_marker::write(outf);
     marshallShort(outf, properties.size());
@@ -452,7 +477,7 @@ void map_wiz_props_marker::write(tagHeader &outf) const
     }
 }
 
-void map_wiz_props_marker::read(tagHeader &inf)
+void map_wiz_props_marker::read(reader &inf)
 {
     map_marker::read(inf);
 
@@ -495,7 +520,7 @@ map_marker *map_wiz_props_marker::clone() const
     return new map_wiz_props_marker(*this);
 }
 
-map_marker *map_wiz_props_marker::read(tagHeader &inf, map_marker_type)
+map_marker *map_wiz_props_marker::read(reader &inf, map_marker_type)
 {
     map_marker *mapf = new map_wiz_props_marker();
     mapf->read(inf);
@@ -563,7 +588,7 @@ void map_markers::add(map_marker *marker)
     markers.insert(dgn_pos_marker(marker->pos, marker));
 }
 
-void map_markers::remove(map_marker *marker)
+void map_markers::unlink_marker(const map_marker *marker)
 {
     std::pair<dgn_marker_map::iterator, dgn_marker_map::iterator>
         els = markers.equal_range(marker->pos);
@@ -575,6 +600,11 @@ void map_markers::remove(map_marker *marker)
             break;
         }
     }
+}
+
+void map_markers::remove(map_marker *marker)
+{
+    unlink_marker(marker);
     delete marker;
 }
 
@@ -636,6 +666,13 @@ void map_markers::move(const coord_def &from, const coord_def &to)
     }
 }
 
+void map_markers::move_marker(map_marker *marker, const coord_def &to)
+{
+    unlink_marker(marker);
+    marker->pos = to;
+    add(marker);
+}
+
 std::vector<map_marker*> map_markers::get_all(map_marker_type mat)
 {
     std::vector<map_marker*> rmarkers;
@@ -645,7 +682,25 @@ std::vector<map_marker*> map_markers::get_all(map_marker_type mat)
         if (mat == MAT_ANY || i->second->get_type() == mat)
             rmarkers.push_back(i->second);
     }
-    return (rmarkers);    
+    return (rmarkers);
+}
+
+std::vector<map_marker*> map_markers::get_all(const std::string &key,
+                                              const std::string &val)
+{
+    std::vector<map_marker*> rmarkers;
+
+    for (dgn_marker_map::const_iterator i = markers.begin();
+         i != markers.end(); ++i)
+    {
+        map_marker*       marker = i->second;
+        const std::string prop   = marker->property(key);
+
+        if (val.empty() && !prop.empty() || !val.empty() && val == prop)
+            rmarkers.push_back(marker);
+    }
+
+    return (rmarkers);
 }
 
 std::vector<map_marker*> map_markers::get_markers_at(const coord_def &c)
@@ -680,22 +735,51 @@ void map_markers::clear()
     markers.clear();
 }
 
-void map_markers::write(tagHeader &th) const
+static const long MARKERS_COOKY = 0x17742C32;
+void map_markers::write(writer &outf) const
 {
-    // how many markers
-    marshallShort(th, markers.size());
+    marshallLong(outf, MARKERS_COOKY);
+
+    std::vector<unsigned char> buf;
+
+    marshallShort(outf, markers.size());
     for (dgn_marker_map::const_iterator i = markers.begin();
          i != markers.end(); ++i)
     {
-        i->second->write(th);
+        buf.clear();
+        writer tmp_outf(&buf);
+        i->second->write(tmp_outf);
+
+        // Write the marker data, prefixed by a size
+        marshallLong(outf, buf.size());
+        for ( std::vector<unsigned char>::const_iterator bi = buf.begin();
+              bi != buf.end(); ++bi )
+        {
+            outf.writeByte(*bi);
+        }
     }
 }
 
-void map_markers::read(tagHeader &th)
+void map_markers::read(reader &inf, int minorVersion)
 {
     clear();
-    const int nmarkers = unmarshallShort(th);
+
+    if (minorVersion >= TAG_MINOR_MAPMARK)
+    {
+        const long cooky = unmarshallLong(inf);
+        ASSERT(cooky == MARKERS_COOKY);
+        UNUSED(cooky);
+    }
+
+    const int nmarkers = unmarshallShort(inf);
     for (int i = 0; i < nmarkers; ++i)
-        if (map_marker *mark = map_marker::read_marker(th))
+    {
+        // used by tools
+        if (minorVersion >= TAG_MINOR_MAPMARK)
+            unmarshallLong(inf);
+        if (map_marker *mark = map_marker::read_marker(inf))
+        {
             add(mark);
+        }
+    }
 }
