@@ -3,7 +3,7 @@
  *  Summary:    Functions used to create vaults.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-10-29T15:20:44.220642Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-06T16:47:06.336575Z $
  *
  *  Change History (most recent first):
  *
@@ -14,11 +14,16 @@
 
 #include "AppHdr.h"
 #include "maps.h"
+#include "tags.h"
 
 #include <cstring>
 #include <cstdlib>
 #include <errno.h>
+#include <algorithm>
+
+#if !_MSC_VER
 #include <unistd.h>
+#endif
 
 #include "dungeon.h"
 #include "enum.h"
@@ -30,7 +35,7 @@
 #include "stuff.h"
 #include "terrain.h"
 
-static int write_vault(map_def &mdef, map_type mt, 
+static int write_vault(map_def &mdef, map_type mt,
                        vault_placement &,
                        bool check_place, bool clobber);
 static int apply_vault_definition(
@@ -49,29 +54,29 @@ static std::vector<map_def> vdefs;
 
 /* ******************** BEGIN PUBLIC FUNCTIONS ******************* */
 
-// remember (!!!) - if a member of the monster array isn't specified
+// Remember (!!!) - if a member of the monster array isn't specified
 // within a vault subroutine, assume that it holds a random monster
 // -- only in cases of explicit monsters assignment have I listed
 // out random monster insertion {dlb}
 
-// make sure that vault_n, where n is a number, is a vault which can be put
+// Make sure that vault_n, where n is a number, is a vault which can be put
 // anywhere, while other vault names are for specific level ranges, etc.
-int vault_main( 
-        map_type vgrid, 
-        vault_placement &place,
-        int which_vault,
-        bool check_place,
-        bool clobber)
+int vault_main( map_type vgrid,
+                vault_placement &place,
+                int which_vault,
+                bool check_place,
+                bool clobber)
 {
 #ifdef DEBUG_DIAGNOSTICS
-    mprf(MSGCH_DIAGNOSTICS, "Generating level: %s (%d,%d)", 
+    mprf(MSGCH_DIAGNOSTICS, "Generating level: %s (%d,%d)",
          vdefs[which_vault].name.c_str(),
          place.pos.x, place.pos.y);
+
     if (crawl_state.map_stat_gen)
         mapgen_report_map_try(vdefs[which_vault]);
 #endif
 
-    // first, fill in entirely with walls and null-terminate {dlb}:
+    // First, fill in entirely with walls and null-terminate {dlb}:
     for (int vx = 0; vx < MAP_SIDE; vx++)
     {
         for (int vy = 0; vy < MAP_SIDE; vy++)
@@ -88,16 +93,16 @@ int vault_main(
                          check_place, clobber ));
 }
 
-static int write_vault(map_def &mdef, map_type map, 
+static int write_vault(map_def &mdef, map_type map,
                        vault_placement &place,
                        bool check_place, bool clobber)
 {
     mdef.load();
-    
+
     // Copy the map so we can monkey with it.
     place.map = mdef;
     place.map.original = &mdef;
-    
+
     // Try so many times to place the map. This will always succeed
     // unless there are conflicting map placements in 'avoid', or there
     // is a map validate Lua hook that keeps rejecting the map.
@@ -128,7 +133,7 @@ static bool resolve_map(map_def &map, const map_def &original)
         if (crawl_state.map_stat_gen)
             mapgen_report_error(map, err);
 #endif
-        mprf(MSGCH_WARN, "Lua error: %s", err.c_str());
+        mprf(MSGCH_ERROR, "Lua error: %s", err.c_str());
         return (false);
     }
 
@@ -136,13 +141,13 @@ static bool resolve_map(map_def &map, const map_def &original)
     err = map.resolve();
     if (!err.empty())
     {
-        mprf(MSGCH_WARN, "Error: %s", err.c_str());
+        mprf(MSGCH_ERROR, "Error: %s", err.c_str());
         return (false);
     }
 
     if (!map.test_lua_validate(false))
         return (false);
-    
+
     // Mirroring is possible for any map that does not explicitly forbid it.
     // Note that mirroring also flips the orientation.
     if (coinflip())
@@ -166,7 +171,7 @@ static bool bad_map_place(const map_def &map,
 {
     if (!check_place || clobber)
         return (false);
-    
+
     const std::vector<std::string> &lines = map.map.get_lines();
     for (int y = sy; y < sy + height; ++y)
     {
@@ -175,18 +180,17 @@ static bool bad_map_place(const map_def &map,
             if (lines[y - sy][x - sx] == ' ')
                 continue;
 
-            if (dgn_map_mask[x][y])
+            if (dgn_Map_Mask[x][y])
                 return (true);
 
             if (igrd[x][y] != NON_ITEM || mgrd[x][y] != NON_MONSTER)
                 return (true);
-            
+
             const dungeon_feature_type grid = grd[x][y];
 
             if (!grid_is_opaque(grid)
                 && grid != DNGN_FLOOR
-                && !grid_is_water(grid)
-                && grid != DNGN_LAVA
+                && grid != DNGN_SHALLOW_WATER
                 && grid != DNGN_CLOSED_DOOR
                 && grid != DNGN_OPEN_DOOR
                 && grid != DNGN_SECRET_DOOR)
@@ -217,7 +221,7 @@ void fit_region_into_map_bounds(coord_def &pos, const coord_def &size)
         pos.y = Y_BOUND_2 - size.y + 1;
 }
 
-static bool apply_vault_grid(map_def &def, map_type map, 
+static bool apply_vault_grid(map_def &def, map_type map,
                              vault_placement &place,
                              bool check_place, bool clobber)
 {
@@ -232,7 +236,7 @@ static bool apply_vault_grid(map_def &def, map_type map,
             || orient == MAP_SOUTHWEST)
         start.y = GYM - height;
 
-    if (orient == MAP_EAST || orient == MAP_NORTHEAST 
+    if (orient == MAP_EAST || orient == MAP_NORTHEAST
             || orient == MAP_SOUTHEAST)
         start.x = GXM - width;
 
@@ -301,7 +305,7 @@ static int apply_vault_definition(
     int orient = def.orient;
     if (orient == MAP_NONE)
         orient = MAP_NORTH;
-    
+
     return (orient);
 }
 
@@ -314,6 +318,7 @@ static bool map_has_no_tags(const map_def &map, I begin, I end)
     for ( ; begin != end; ++begin)
         if (map.has_tag(*begin))
             return (false);
+
     return (true);
 }
 
@@ -324,18 +329,27 @@ static bool vault_unforbidden(const map_def &map)
                                you.uniq_map_tags.end()));
 }
 
+static bool map_matches_layout_type(const map_def &map)
+{
+    if (dgn_Layout_Type.empty() || !map.has_tag_prefix("layout_"))
+        return (true);
+
+    return map.has_tag("layout_" + dgn_Layout_Type);
+}
+
 int find_map_by_name(const std::string &name)
 {
     for (unsigned i = 0, size = vdefs.size(); i < size; ++i)
         if (vdefs[i].name == name)
             return (i);
+
     return (-1);
 }
 
 std::vector<std::string> find_map_matches(const std::string &name)
 {
     std::vector<std::string> matches;
-    
+
     for (unsigned i = 0, size = vdefs.size(); i < size; ++i)
         if (vdefs[i].name.find(name) != std::string::npos)
             matches.push_back(vdefs[i].name);
@@ -356,21 +370,23 @@ int random_map_for_place(const level_id &place, bool want_minivault)
         // We also accept tagged levels here.
         if (vdefs[i].place == place
             && vdefs[i].is_minivault() == want_minivault
+            && !vdefs[i].has_tag("layout")
+            && map_matches_layout_type(vdefs[i])
             && vault_unforbidden(vdefs[i]))
         {
             rollsize += vdefs[i].chance;
 
-            if (rollsize && random2(rollsize) < vdefs[i].chance)
+            if (rollsize && x_chance_in_y(vdefs[i].chance, rollsize))
                 mapindex = i;
         }
     }
 
     if (mapindex != -1 && vdefs[mapindex].has_tag("dummy"))
         mapindex = -1;
-    
+
 #ifdef DEBUG_DIAGNOSTICS
     if (mapindex != -1)
-        mprf(MSGCH_DIAGNOSTICS, "Found map %s for %s", 
+        mprf(MSGCH_DIAGNOSTICS, "Found map %s for %s",
              vdefs[mapindex].name.c_str(), place.describe().c_str());
 #endif
 
@@ -382,6 +398,8 @@ int random_map_in_depth(const level_id &place, bool want_minivault)
     int mapindex = -1;
     int rollsize = 0;
 
+    const bool check_layout = (place == level_id::current());
+
     for (unsigned i = 0, size = vdefs.size(); i < size; ++i)
         if (vdefs[i].is_minivault() == want_minivault
             && !vdefs[i].place.is_valid()
@@ -393,17 +411,19 @@ int random_map_in_depth(const level_id &place, bool want_minivault)
             && !vdefs[i].has_tag("pan")
             && !vdefs[i].has_tag("unrand")
             && !vdefs[i].has_tag("bazaar")
+            && !vdefs[i].has_tag("layout")
+            && (!check_layout || map_matches_layout_type(vdefs[i]))
             && vault_unforbidden(vdefs[i]))
         {
             rollsize += vdefs[i].chance;
 
-            if (rollsize && random2(rollsize) < vdefs[i].chance)
+            if (rollsize && x_chance_in_y(vdefs[i].chance, rollsize))
                 mapindex = i;
         }
 
     if (mapindex != -1 && vdefs[mapindex].has_tag("dummy"))
         mapindex = -1;
-    
+
     return (mapindex);
 }
 
@@ -419,18 +439,19 @@ int random_map_for_tag(const std::string &tag,
         if (vdefs[i].has_tag(tag) && vdefs[i].is_minivault() == want_minivault
             && (!check_depth || !vdefs[i].has_depth()
                 || vdefs[i].is_usable_in(level_id::current()))
+            && map_matches_layout_type(vdefs[i])
             && vault_unforbidden(vdefs[i]))
         {
             rollsize += vdefs[i].chance;
 
-            if (rollsize && random2(rollsize) < vdefs[i].chance)
+            if (rollsize && x_chance_in_y(vdefs[i].chance, rollsize))
                 mapindex = i;
         }
     }
 
     if (mapindex != -1 && vdefs[mapindex].has_tag("dummy"))
         mapindex = -1;
-    
+
 #ifdef DEBUG_DIAGNOSTICS
     if (mapindex != -1)
         mprf(MSGCH_DIAGNOSTICS, "Found map %s tagged '%s'",
@@ -444,7 +465,7 @@ const map_def *map_by_index(int index)
 {
     if (index < 0 || index >= (int) vdefs.size())
         return (NULL);
-    
+
     return &vdefs[index];
 }
 
@@ -491,7 +512,7 @@ static void check_des_index_dir()
 {
     if (checked_des_index_dir)
         return;
-    
+
     std::string desdir = get_savedir_path("des");
     if (!check_dir("Data file cache", desdir, true))
         end(1, true, "Can't create data file cache: %s", desdir.c_str());
@@ -509,12 +530,12 @@ std::string get_descache_path(const std::string &file,
 
 static bool verify_file_version(const std::string &file)
 {
-    FILE *inf = fopen(file.c_str(), "rb");
-    if (!inf)
+    FILE *fp = fopen(file.c_str(), "rb");
+    if (!fp)
         return (false);
-
-    const long ver = readLong(inf);
-    fclose(inf);
+    reader inf(fp);
+    const long ver = unmarshallLong(inf);
+    fclose(fp);
 
     return (ver == MAP_CACHE_VERSION);
 }
@@ -532,21 +553,26 @@ static bool verify_map_full(const std::string &base)
 static bool load_map_index(const std::string &base)
 {
     // If there's a global prelude, load that first.
-    FILE *inf = fopen((base + ".lux").c_str(), "rb");
-    if (inf)
     {
-        lc_global_prelude.read(inf);
-        fclose(inf);
+        FILE *fp = fopen((base + ".lux").c_str(), "rb");
+        if (fp)
+        {
+            reader inf(fp);
+            lc_global_prelude.read(inf);
+            fclose(fp);
 
-        global_preludes.push_back( lc_global_prelude );
+            global_preludes.push_back( lc_global_prelude );
+        }
     }
-    
-    inf = fopen((base + ".idx").c_str(), "rb");
-    if (!inf)
+
+    FILE* fp = fopen((base + ".idx").c_str(), "rb");
+    if (!fp)
         end(1, true, "Unable to read %s", (base + ".idx").c_str());
+    reader inf(fp);
+
     // Discard version (it's been checked by verify_map_index).
-    readLong(inf);
-    const int nmaps = readShort(inf);
+    (void) unmarshallLong(inf);
+    const int nmaps = unmarshallShort(inf);
     const int nexist = vdefs.size();
     vdefs.resize( nexist + nmaps, map_def() );
     for (int i = 0; i < nmaps; ++i)
@@ -557,7 +583,7 @@ static bool load_map_index(const std::string &base)
         lc_loaded_maps[vdef.name] = vdef.place_loaded_from;
         vdef.place_loaded_from.clear();
     }
-    fclose(inf);
+    fclose(fp);
 
     return (true);
 }
@@ -566,13 +592,13 @@ static bool load_map_cache(const std::string &filename)
 {
     check_des_index_dir();
     const std::string descache_base = get_descache_path(filename, "");
-    
+
     file_lock deslock(descache_base + ".lk", "rb", false);
 
     if (is_newer(filename, descache_base + ".idx")
         || is_newer(filename, descache_base + ".dsc"))
         return (false);
-    
+
     if (!verify_map_index(descache_base) || !verify_map_full(descache_base))
         return (false);
 
@@ -588,36 +614,39 @@ static void write_map_prelude(const std::string &filebase)
         return;
     }
 
-    FILE *outf = fopen(luafile.c_str(), "wb");
+    FILE *fp = fopen(luafile.c_str(), "wb");
+    writer outf(fp);
     lc_global_prelude.write(outf);
-    fclose(outf);
+    fclose(fp);
 }
 
 static void write_map_full(const std::string &filebase, size_t vs, size_t ve)
 {
     const std::string cfile = filebase + ".dsc";
-    FILE *outf = fopen(cfile.c_str(), "wb");
-    if (!outf)
+    FILE *fp = fopen(cfile.c_str(), "wb");
+    if (!fp)
         end(1, true, "Unable to open %s for writing", cfile.c_str());
 
-    writeLong(outf, MAP_CACHE_VERSION);
+    writer outf(fp);
+    marshallLong(outf, MAP_CACHE_VERSION);
     for (size_t i = vs; i < ve; ++i)
         vdefs[i].write_full(outf);
-    fclose(outf);
+    fclose(fp);
 }
 
 static void write_map_index(const std::string &filebase, size_t vs, size_t ve)
 {
     const std::string cfile = filebase + ".idx";
-    FILE *outf = fopen(cfile.c_str(), "wb");
-    if (!outf)
+    FILE *fp = fopen(cfile.c_str(), "wb");
+    if (!fp)
         end(1, true, "Unable to open %s for writing", cfile.c_str());
 
-    writeLong(outf, MAP_CACHE_VERSION);
-    writeShort(outf, ve > vs? ve - vs : 0);
+    writer outf(fp);
+    marshallLong(outf, MAP_CACHE_VERSION);
+    marshallShort(outf, ve > vs? ve - vs : 0);
     for (size_t i = vs; i < ve; ++i)
         vdefs[i].write_index(outf);
-    fclose(outf);
+    fclose(fp);
 }
 
 static void write_map_cache(const std::string &filename, size_t vs, size_t ve)
@@ -625,9 +654,9 @@ static void write_map_cache(const std::string &filename, size_t vs, size_t ve)
     check_des_index_dir();
 
     const std::string descache_base = get_descache_path(filename, "");
-    
+
     file_lock deslock(descache_base + ".lk", "wb");
-    
+
     write_map_prelude(descache_base);
     write_map_full(descache_base, vs, ve);
     write_map_index(descache_base, vs, ve);
@@ -640,7 +669,7 @@ static void parse_maps(const std::string &s)
         end(1, false, "Map file %s has already been read.", base.c_str());
 
     map_files_read.insert(base);
-    
+
     if (load_map_cache(s))
         return;
 
@@ -693,7 +722,7 @@ void run_map_preludes()
         if (!chunk.empty())
         {
             if (chunk.load_call(dlua, NULL))
-                mprf(MSGCH_WARN, "Lua error: %s", chunk.orig_error().c_str());
+                mprf(MSGCH_ERROR, "Lua error: %s", chunk.orig_error().c_str());
         }
     }
     for (int i = 0, size = vdefs.size(); i < size; ++i)
@@ -702,7 +731,7 @@ void run_map_preludes()
         {
             std::string err = vdefs[i].run_lua(true);
             if (!err.empty())
-                mprf(MSGCH_WARN, "Lua error (map %s): %s",
+                mprf(MSGCH_ERROR, "Lua error (map %s): %s",
                      vdefs[i].name.c_str(), err.c_str());
         }
     }
@@ -720,7 +749,7 @@ static weighted_map_names mg_find_random_vaults(
     const level_id &place, bool wantmini)
 {
     weighted_map_names wms;
-    
+
     if (!place.is_valid())
         return (wms);
 
@@ -735,7 +764,7 @@ static weighted_map_names mg_find_random_vaults(
             && !vdefs[i].has_tag_suffix("entry")
             && !vdefs[i].has_tag("pan")
             && !vdefs[i].has_tag("unrand")
-            && !vdefs[i].has_tag("bazaar"))        
+            && !vdefs[i].has_tag("bazaar"))
         {
             wms.push_back(
                 weighted_map_name( vdefs[i].name, vdefs[i].chance ) );
