@@ -3,8 +3,9 @@
  *  Summary:    Menus and associated malarkey.
  *  Written by: Darshan Shaligram
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T14:44:32.335576Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-06-26T10:39:39.656724Z $
  */
+
 #include <cctype>
 #include "AppHdr.h"
 #include "cio.h"
@@ -12,33 +13,109 @@
 #include "macro.h"
 #include "message.h"
 #include "player.h"
+#include "tiles.h"
 #include "tutorial.h"
 #include "view.h"
 #include "initfile.h"
 
-Menu::Menu( int _flags )
- :  f_selitem(NULL),
-    f_drawitem(NULL),
-    f_keyfilter(NULL),
-    title(NULL),
-    flags(_flags),
-    first_entry(0),
-    y_offset(0),
-    pagesize(0),
-    max_pagesize(0),
-    more("-more-", true),
-    items(),
-    sel(),
-    select_filter(),
-    highlighter(new MenuHighlighter),
-    num(-1),
-    lastch(0),
-    alive(false),
-    last_selected(-1)
+Menu::Menu( int _flags, const std::string& tagname )
+ : f_selitem(NULL), f_drawitem(NULL), f_keyfilter(NULL), title(NULL),
+   flags(_flags), tag(tagname), first_entry(0), y_offset(0),
+   pagesize(0), max_pagesize(0), more("-more-", true), items(),
+   sel(), select_filter(), highlighter(new MenuHighlighter), num(-1),
+   lastch(0), alive(false), last_selected(-1)
 {
     set_flags(flags);
 }
 
+Menu::Menu( const formatted_string &fs )
+ : f_selitem(NULL), f_drawitem(NULL), f_keyfilter(NULL), title(NULL),
+
+   // This is a text-viewer menu, init flags to be easy on the user.
+   flags(MF_NOSELECT | MF_EASY_EXIT),
+
+   tag(), first_entry(0), y_offset(0), pagesize(0),
+   max_pagesize(0), more("-more-", true), items(), sel(),
+   select_filter(), highlighter(new MenuHighlighter), num(-1),
+   lastch(0), alive(false), last_selected(-1)
+{
+    int colour = LIGHTGREY;
+    int last_text_colour = LIGHTGREY;
+    std::string line;
+    for (formatted_string::oplist::const_iterator i = fs.ops.begin();
+         i != fs.ops.end(); ++i)
+    {
+        const formatted_string::fs_op &op(*i);
+        switch (op.type)
+        {
+        case FSOP_COLOUR:
+            colour = op.x;
+            break;
+        case FSOP_TEXT:
+        {
+            line += op.text;
+
+            const std::string::size_type nonblankp =
+                op.text.find_first_not_of(" \t\r\n");
+            const bool nonblank = nonblankp != std::string::npos;
+            const std::string::size_type eolp = op.text.find(EOL);
+            const bool starts_with_eol =
+                nonblank && eolp != std::string::npos
+                && eolp < nonblankp;
+
+            if (nonblank && !starts_with_eol)
+                last_text_colour = colour;
+
+            check_add_formatted_line(last_text_colour, colour, line, true);
+
+            if (nonblank && starts_with_eol)
+                last_text_colour = colour;
+            break;
+        }
+        default:
+            break;
+        }
+    }
+    check_add_formatted_line(last_text_colour, colour, line, false);
+}
+
+void Menu::check_add_formatted_line(int firstcol, int nextcol,
+                                    std::string &line, bool check_eol)
+{
+    if (line.empty())
+        return;
+
+    if (check_eol && line.find(EOL) == std::string::npos)
+        return;
+
+    std::vector<std::string> lines = split_string(EOL, line, false, true);
+    int size = lines.size();
+
+    // If we have stuff after EOL, leave that in the line variable and
+    // don't add an entry for it, unless the caller told us not to
+    // check EOL sanity.
+    if (check_eol && !ends_with(line, EOL))
+        line = lines[--size];
+    else
+        line.clear();
+
+    for (int i = 0, col = firstcol; i < size; ++i, col = nextcol)
+    {
+        std::string &s(lines[i]);
+
+        trim_string_right(s);
+
+        MenuEntry *me = new MenuEntry(s);
+        me->colour = col;
+        if (!title)
+            set_title(me);
+        else
+            add_entry(me);
+    }
+
+    line.clear();
+}
+
 Menu::~Menu()
 {
     for (int i = 0, count = items.size(); i < count; ++i)
@@ -92,6 +169,7 @@ void Menu::set_title( MenuEntry *e )
 
 void Menu::add_entry( MenuEntry *entry )
 {
+    entry->tag = tag;
     items.push_back( entry );
 }
 
@@ -103,11 +181,17 @@ void Menu::reset()
 std::vector<MenuEntry *> Menu::show(bool reuse_selections)
 {
     cursor_control cs(false);
-    
+
     if (reuse_selections)
+    {
         get_selected(&sel);
+    }
     else
+    {
         deselect_all(false);
+        sel.clear();
+    }
+
 
     // Lose lines for the title + room for -more- line.
     pagesize = get_number_of_lines() - !!title - 1;
@@ -126,8 +210,12 @@ void Menu::do_menu()
     alive = true;
     while (alive)
     {
-        int keyin = getchm(c_getch);
-        
+#ifdef USE_TILE
+        TileRedrawInv(REGION_INV2);
+        mouse_control mc(MOUSE_MODE_COMMAND);
+#endif
+        int keyin = getchm(KC_NONE, c_getch);
+
         if (!process_key( keyin ))
             return;
     }
@@ -153,9 +241,9 @@ bool Menu::process_key( int keyin )
     if (items.size() == 0)
     {
         lastch = keyin;
-        return false;
+        return (false);
     }
-    
+
     bool nav = false, repaint = false;
 
     if (f_keyfilter)
@@ -166,14 +254,16 @@ bool Menu::process_key( int keyin )
     switch (keyin)
     {
     case 0:
-        return true;
+        return (true);
     case CK_ENTER:
-        return false;
+        return (false);
     case CK_ESCAPE:
+    case CK_MOUSE_B2:
         sel.clear();
         lastch = keyin;
-        return false;
+        return (false);
     case ' ': case CK_PGDN: case '>': case '\'':
+    case CK_MOUSE_B1:
         nav = true;
         repaint = page_down();
         if (!repaint && !is_set(MF_EASY_EXIT) && !is_set(MF_NOWRAP))
@@ -203,7 +293,7 @@ bool Menu::process_key( int keyin )
     {
         nav = true;
         const int breakpoint = (items.size() + 1) - pagesize;
-        if ( first_entry < breakpoint )
+        if (first_entry < breakpoint)
         {
             first_entry = breakpoint;
             repaint = true;
@@ -212,29 +302,29 @@ bool Menu::process_key( int keyin )
     }
     case CONTROL('F'):
     {
-        if ( !( flags & MF_ALLOW_FILTER ) )
+        if (!(flags & MF_ALLOW_FILTER))
             break;
         char linebuf[80];
-        gotoxy(1,1);
+        cgotoxy(1,1);
         clear_to_end_of_line();
         textcolor(WHITE);
         cprintf("Select what? (regex) ");
         textcolor(LIGHTGREY);
         bool validline = !cancelable_get_line(linebuf, sizeof linebuf, 80);
-        if ( validline && linebuf[0] )
+        if (validline && linebuf[0])
         {
             text_pattern tpat(linebuf, true);
-            for ( unsigned int i = 0; i < items.size(); ++i )
+            for (unsigned int i = 0; i < items.size(); ++i)
             {
-                if ( items[i]->level == MEL_ITEM &&
-                     tpat.matches(items[i]->get_text()) )
+                if (items[i]->level == MEL_ITEM
+                    && tpat.matches(items[i]->get_text()))
                 {
                     select_index(i);
-                    if ( flags & MF_SINGLESELECT )
+                    if (flags & MF_SINGLESELECT)
                     {
                         // Return the first item found.
                         get_selected(&sel);
-                        return false;
+                        return (false);
                     }
                 }
             }
@@ -266,7 +356,7 @@ bool Menu::process_key( int keyin )
 
         // If no selection at all is allowed, exit now.
         if (!(flags & (MF_SINGLESELECT | MF_MULTISELECT)))
-            return false;
+            return (false);
 
         if (!is_set(MF_NO_SELECT_QTY) && isdigit( keyin ))
         {
@@ -275,26 +365,27 @@ bool Menu::process_key( int keyin )
             num = (num == -1)? keyin - '0' :
                                num * 10 + keyin - '0';
         }
-        
+
         select_items( keyin, num );
         get_selected( &sel );
         if (sel.size() == 1 && (flags & MF_SINGLESELECT))
-            return false;
+            return (false);
+
         draw_select_count( sel.size() );
 
         if (flags & MF_ANYPRINTABLE
             && (!isdigit(keyin) || is_set(MF_NO_SELECT_QTY)))
         {
-            return false;
+            return (false);
         }
 
         break;
     }
 
-    if (last_selected != -1 &&
-        (items.size() == ((unsigned int) last_selected + 1)
-         || items[last_selected + 1] == NULL
-         || items[last_selected + 1]->level != MEL_ITEM))
+    if (last_selected != -1
+        && (items.size() == ((unsigned int) last_selected + 1)
+            || items[last_selected + 1] == NULL
+            || items[last_selected + 1]->level != MEL_ITEM))
     {
         last_selected = -1;
     }
@@ -310,10 +401,10 @@ bool Menu::process_key( int keyin )
         else if (sel.empty() && is_set(MF_EASY_EXIT))
         {
             sel.clear();
-            return false;
+            return (false);
         }
     }
-    return true;
+    return (true);
 }
 
 bool Menu::draw_title_suffix( const std::string &s, bool titlefirst )
@@ -326,8 +417,8 @@ bool Menu::draw_title_suffix( const std::string &s, bool titlefirst )
     int x = wherex();
     if (x > get_number_of_cols() || x < 1)
     {
-        gotoxy(oldx, oldy);
-        return false;
+        cgotoxy(oldx, oldy);
+        return (false);
     }
 
     // Note: 1 <= x <= get_number_of_cols(); we have no fear of overflow.
@@ -337,11 +428,48 @@ bool Menu::draw_title_suffix( const std::string &s, bool titlefirst )
                                 s + std::string(avail_width - s.length(), ' ');
 
     cprintf("%s", towrite.c_str());
-    
-    gotoxy( oldx, oldy );
-    return true;
+
+    cgotoxy( oldx, oldy );
+    return (true);
 }
-    
+
+bool Menu::draw_title_suffix( const formatted_string &fs, bool titlefirst )
+{
+    int oldx = wherex(), oldy = wherey();
+
+    if (titlefirst)
+        draw_title();
+
+    int x = wherex();
+    if (x > get_number_of_cols() || x < 1)
+    {
+        cgotoxy(oldx, oldy);
+        return (false);
+    }
+
+    // Note: 1 <= x <= get_number_of_cols(); we have no fear of overflow.
+    const unsigned int avail_width = get_number_of_cols() - x + 1;
+    const unsigned int fs_length = fs.length();
+    if (fs_length > avail_width)
+    {
+        formatted_string fs_trunc = fs.substr(0, avail_width);
+        fs_trunc.display();
+    }
+    else
+    {
+        fs.display();
+        if (fs_length < avail_width)
+        {
+            char fmt[20];
+            sprintf(fmt, "%%%ds", avail_width-fs_length);
+            cprintf(fmt, " ");
+        }
+    }
+
+    cgotoxy( oldx, oldy );
+    return (true);
+}
+
 void Menu::draw_select_count( int count, bool force )
 {
     if (!force && !is_set(MF_MULTISELECT))
@@ -355,7 +483,7 @@ void Menu::draw_select_count( int count, bool force )
     {
         char buf[100] = "";
         if (count)
-            snprintf(buf, sizeof buf, "  (%d item%s)  ", count, 
+            snprintf(buf, sizeof buf, "  (%d item%s)  ", count,
                     (count > 1? "s" : ""));
 
         draw_title_suffix( buf );
@@ -372,7 +500,7 @@ std::vector<MenuEntry*> Menu::selected_entries() const
 void Menu::get_selected( std::vector<MenuEntry*> *selected ) const
 {
     selected->clear();
-    
+
     for (int i = 0, count = items.size(); i < count; ++i)
         if (items[i]->selected())
             selected->push_back( items[i] );
@@ -416,7 +544,7 @@ void Menu::select_items( int key, int qty )
     {
         int final = items.size();
         bool selected = false;
-        
+
         // Process all items, in case user hits hotkey for an
         // item not on the current page.
 
@@ -458,20 +586,21 @@ void Menu::select_items( int key, int qty )
             }
         }
     }
-    gotoxy( x, y );
+    cgotoxy( x, y );
 }
 
 bool Menu::is_selectable(int item) const
 {
-    if (select_filter.empty()) return true;
-    
+    if (select_filter.empty())
+        return (true);
+
     std::string text = items[item]->get_text();
     for (int i = 0, count = select_filter.size(); i < count; ++i)
     {
         if (select_filter[i].matches(text))
-            return true;
+            return (true);
     }
-    return false;
+    return (false);
 }
 
 void Menu::select_index( int index, int qty )
@@ -485,8 +614,10 @@ void Menu::select_index( int index, int qty )
             for (int i = 0, count = items.size(); i < count; ++i)
             {
                 if (items[i]->level != MEL_ITEM
-                        || items[i]->hotkeys.empty())
+                    || items[i]->hotkeys.empty())
+                {
                     continue;
+                }
                 if (is_hotkey(i, items[i]->hotkeys[0]) && is_selectable(i))
                 {
                     last_selected = i;
@@ -501,8 +632,10 @@ void Menu::select_index( int index, int qty )
         for (int i = si + 1, count = items.size(); i < count; ++i)
         {
             if (items[i]->level != MEL_ITEM
-                    || items[i]->hotkeys.empty())
+                || items[i]->hotkeys.empty())
+            {
                 continue;
+            }
             if (is_hotkey(i, items[i]->hotkeys[0]))
             {
                 last_selected = i;
@@ -511,8 +644,8 @@ void Menu::select_index( int index, int qty )
             }
         }
     }
-    else if (items[si]->level == MEL_ITEM && 
-            (flags & (MF_SINGLESELECT | MF_MULTISELECT))) 
+    else if (items[si]->level == MEL_ITEM
+             && (flags & (MF_SINGLESELECT | MF_MULTISELECT)))
     {
         last_selected = si;
         items[si]->select( qty );
@@ -520,6 +653,21 @@ void Menu::select_index( int index, int qty )
     }
 }
 
+int Menu::get_entry_index( const MenuEntry *e ) const
+{
+    int index = 0;
+    for (unsigned int i = first_entry; i < items.size(); i++)
+    {
+        if (items[i] == e)
+            return (index);
+
+        if (items[i]->quantity != 0)
+            index++;
+    }
+
+    return -1;
+}
+
 void Menu::draw_menu()
 {
     clrscr();
@@ -530,48 +678,26 @@ void Menu::draw_menu()
 
     int end = first_entry + pagesize;
     if (end > (int) items.size()) end = items.size();
-    
+
     for (int i = first_entry; i < end; ++i)
-    {
         draw_item( i );
-    }
+
     if (end < (int) items.size() || is_set(MF_ALWAYS_SHOW_MORE))
     {
-        gotoxy( 1, y_offset + pagesize - count_linebreaks(more) );
+        cgotoxy( 1, y_offset + pagesize - count_linebreaks(more) );
         more.display();
     }
+
+#ifdef USE_TILE
+    TileRedrawInv(REGION_INV2);
+#endif
 }
 
 void Menu::update_title()
 {
     int x = wherex(), y = wherey();
     draw_title();
-    gotoxy(x, y);
-}
-
-// to highlight species in aptitudes list ('?%')
-static std::string get_species_key()
-{
-    if (player_genus(GENPC_DRACONIAN) && you.experience_level < 7)
-        return "";
-
-    std::string result = "";
-    switch (you.species)
-    {
-        case SP_RED_DRACONIAN:     result = "Red"; break;
-        case SP_WHITE_DRACONIAN:   result = "White"; break;
-        case SP_GREEN_DRACONIAN:   result = "Green"; break;
-        case SP_GOLDEN_DRACONIAN:  result = "Yellow"; break;
-        case SP_GREY_DRACONIAN:    result = "Grey"; break;
-        case SP_BLACK_DRACONIAN:   result = "Black"; break;
-        case SP_PURPLE_DRACONIAN:  result = "Purple"; break;
-        case SP_MOTTLED_DRACONIAN: result = "Mottled"; break;
-        case SP_PALE_DRACONIAN:    result = "Pale"; break;
-        default:
-             result = species_name(you.species, 1);
-    }
-    result += "  ";
-    return (result);
+    cgotoxy(x, y);
 }
 
 int Menu::item_colour(int, const MenuEntry *entry) const
@@ -579,12 +705,6 @@ int Menu::item_colour(int, const MenuEntry *entry) const
     int icol = -1;
     if (highlighter)
         icol = highlighter->entry_colour(entry);
-    else
-    {
-        text_pattern tp(get_species_key());
-        if (!tp.empty() && tp.matches(entry->text))
-            icol = WHITE;
-    }
 
     return (icol == -1? entry->colour : icol);
 }
@@ -593,7 +713,7 @@ void Menu::draw_title()
 {
     if (title)
     {
-        gotoxy(1, 1);
+        cgotoxy(1, 1);
         write_title();
     }
 }
@@ -602,10 +722,22 @@ void Menu::write_title()
 {
     textattr( item_colour(-1, title) );
     cprintf("%s", title->get_text().c_str());
+    if ( flags & MF_SHOW_PAGENUMBERS )
+    {
+        // The total number of pages is well defined, but the current
+        // page a bit less so. To make sense, we hack it so that your
+        // current page is based on the first line you're seeing, *unless*
+        // you're seeing the last item.
+        int numpages = items.empty() ? 1 : ((items.size()-1) / pagesize + 1);
+        int curpage = first_entry / pagesize + 1;
+        if ( in_page(items.size() - 1) )
+            curpage = numpages;
+        cprintf(" (page %d of %d)", curpage, numpages);
+    }
 
     const int x = wherex(), y = wherey();
     cprintf("%-*s", get_number_of_cols() - x, "");
-    gotoxy(x, y);
+    cgotoxy(x, y);
 }
 
 bool Menu::in_page(int index) const
@@ -617,7 +749,7 @@ void Menu::draw_item( int index ) const
 {
     if (!in_page(index))
         return;
-    gotoxy( 1, y_offset + index - first_entry );
+    cgotoxy( 1, y_offset + index - first_entry );
 
     draw_index_item(index, items[index]);
 }
@@ -655,9 +787,9 @@ bool Menu::page_down()
         //    first_entry = items.size() - pagesize;
 
         if (old_first != first_entry)
-            return true;
+            return (true);
     }
-    return false;
+    return (false);
 }
 
 bool Menu::page_up()
@@ -668,10 +800,11 @@ bool Menu::page_up()
     {
         if ((first_entry -= pagesize) < 0)
             first_entry = 0;
+
         if (old_first != first_entry)
-            return true;
+            return (true);
     }
-    return false;
+    return (false);
 }
 
 bool Menu::line_down()
@@ -679,9 +812,9 @@ bool Menu::line_down()
     if (first_entry + pagesize < (int) items.size())
     {
         ++first_entry;
-        return true;
+        return (true);
     }
-    return false;
+    return (false);
 }
 
 bool Menu::line_up()
@@ -689,9 +822,9 @@ bool Menu::line_up()
     if (first_entry > 0)
     {
         --first_entry;
-        return true;
+        return (true);
     }
-    return false;
+    return (false);
 }
 
 /////////////////////////////////////////////////////////////////
@@ -731,7 +864,7 @@ void slider_menu::select_search(const std::string &s)
         lowercase(text);
 
         std::string::size_type found = text.find(srch);
-        if (found != std::string::npos 
+        if (found != std::string::npos
                 && found == text.find_first_not_of(" "))
         {
             move_selection(i);
@@ -783,7 +916,7 @@ void slider_menu::adjust_pagesizes(int recurse_depth)
 {
     if (first_entry == 1 && selected == 1)
         first_entry = 0;
-    
+
     need_less = !!first_entry;
     pagesize = endy - starty + 1 - !!title - need_less;
     const int nitems = items.size();
@@ -795,7 +928,7 @@ void slider_menu::adjust_pagesizes(int recurse_depth)
             && (selected < first_entry || selected >= first_entry + pagesize)
             && recurse_depth > 0)
         fix_entry(recurse_depth - 1);
-    
+
     calc_y_offset();
 }
 
@@ -822,7 +955,7 @@ std::vector<MenuEntry *> slider_menu::show()
 
     if (!search.empty())
         select_search(search);
-    
+
     fix_entry();
     do_menu();
 
@@ -857,7 +990,7 @@ int slider_menu::item_colour(int index, const MenuEntry *me) const
     int colour = Menu::item_colour(index, me);
     if (index == selected && selected != -1)
     {
-#if defined(WIN32CONSOLE) || defined(DOS)
+#if defined(WIN32CONSOLE) || defined(DOS) || defined (USE_TILE)
         colour = dos_brand(colour, CHATTR_REVERSE);
 #else
         colour |= COLFLAG_REVERSE;
@@ -870,7 +1003,7 @@ void slider_menu::show_less()
 {
     if (!need_less)
         return ;
-    
+
     if (first_entry > 0)
         less.display();
     else
@@ -883,7 +1016,7 @@ void slider_menu::show_more()
     if (!need_more)
         return ;
     const int end = entry_end();
-    gotoxy( 1, y_offset + pagesize );
+    cgotoxy( 1, y_offset + pagesize );
     if (end < (int) items.size() || is_set(MF_ALWAYS_SHOW_MORE))
         more.display();
     else
@@ -893,7 +1026,7 @@ void slider_menu::show_more()
 
 void slider_menu::calc_y_offset()
 {
-    y_offset = starty + !!title + need_less;     
+    y_offset = starty + !!title + need_less;
 }
 
 int slider_menu::entry_end() const
@@ -905,7 +1038,7 @@ int slider_menu::entry_end() const
 
 void slider_menu::draw_menu()
 {
-    gotoxy(1, starty);
+    cgotoxy(1, starty);
     write_title();
 
     calc_y_offset();
@@ -914,17 +1047,17 @@ void slider_menu::draw_menu()
 
     // We're using get_number_of_cols() - 1 because we want to avoid line wrap
     // on DOS (the conio.h functions go batshit if that happens).
-    gotoxy(1, y_offset - 1);
+    cgotoxy(1, y_offset - 1);
 
     show_less();
-    
+
     for (int i = first_entry; i < end; ++i)
         draw_item( i );
 
     textattr(LIGHTGREY);
     for (int i = end; i < first_entry + pagesize; ++i)
     {
-        gotoxy(1, y_offset + i - first_entry);
+        cgotoxy(1, y_offset + i - first_entry);
         cprintf("%-*s", get_number_of_cols() - 2, "");
     }
 
@@ -969,18 +1102,26 @@ void slider_menu::new_selection(int nsel)
     if (nsel < 0)
     {
         if ( !is_set(MF_NOWRAP) )
-            do {
+        {
+            do
+            {
                 nsel += items.size();
-            } while ( nsel < 0 );
+            }
+            while ( nsel < 0 );
+        }
         else
             nsel = 0;
     }
     if (nsel >= static_cast<int>(items.size()))
     {
         if ( !is_set(MF_NOWRAP) )
-            do {
+        {
+            do
+            {
                 nsel -= items.size();
-            } while ( nsel >= static_cast<int>(items.size()) );
+            }
+            while ( nsel >= static_cast<int>(items.size()) );
+        }
         else
             nsel = items.size() - 1;
     }
@@ -1028,15 +1169,20 @@ bool slider_menu::line_up()
 // Menu colouring
 //
 
-int menu_colour(const std::string &text, const std::string &prefix)
+int menu_colour(const std::string &text, const std::string &prefix,
+                const std::string &tag)
 {
-    std::string tmp_text = prefix + text;
+    const std::string tmp_text = prefix + text;
 
-    for (int i = 0, size = Options.menu_colour_mappings.size(); i < size; ++i)
+    for (unsigned int i = 0; i < Options.menu_colour_mappings.size(); ++i)
     {
-        colour_mapping &cm = Options.menu_colour_mappings[i];
-        if (cm.pattern.matches(tmp_text))
+        const colour_mapping &cm = Options.menu_colour_mappings[i];
+        if ((cm.tag.empty() || cm.tag == "any" || cm.tag == tag
+             || cm.tag == "inventory" && tag == "pickup")
+            && cm.pattern.matches(tmp_text) )
+        {
             return (cm.colour);
+        }
     }
     return (-1);
 }
@@ -1113,8 +1259,8 @@ void column_composer::add_formatted(
     strip_blank_lines(newlines);
 
     compose_formatted_column(
-            newlines, 
-            col.lines, 
+            newlines,
+            col.lines,
             margin == -1? col.margin : margin);
 
     col.lines += newlines.size();
@@ -1182,7 +1328,7 @@ void formatted_scroller::add_text(const std::string& s)
             break;
         else
             eolpos = newpos + 1;
-    }    
+    }
 }
 
 void formatted_scroller::add_item_formatted_string(const formatted_string& fs,
@@ -1191,7 +1337,10 @@ void formatted_scroller::add_item_formatted_string(const formatted_string& fs,
     MenuEntry* me = new MenuEntry;
     me->data = new formatted_string(fs);
     if ( hotkey )
+    {
         me->add_hotkey(hotkey);
+        me->quantity = 1;
+    }
     add_entry(me);
 }
 
@@ -1274,7 +1423,7 @@ int linebreak_string( std::string& s, int wrapcol, int maxcol )
 int linebreak_string2( std::string& s, int maxcol )
 {
     size_t loc = 0;
-    int xpos = 0, spacepos = 0;
+    int xpos = 0, spaceloc = 0;
     int breakcount = 0;
     while ( loc < s.size() )
     {
@@ -1295,32 +1444,39 @@ int linebreak_string2( std::string& s, int maxcol )
         }
         else
         {
-            // user-forced newline
+            // user-forced newline, or one we just stuffed in
             if ( s[loc] == '\n' )
-                xpos = 0;
-            // hard linebreak
-            else if ( xpos >= maxcol )
             {
-                if (spacepos >= xpos-maxcol)
+                xpos = 0;
+                spaceloc = 0;
+                ++loc;
+                continue;
+            }
+
+            // force a wrap?
+            if ( xpos >= maxcol )
+            {
+                if (spaceloc)
                 {
-                    loc = spacepos;
+                    loc = spaceloc;
                     s.replace(loc, 1, "\n");
                 }
                 else
+                {
                     s.insert(loc, "\n");
-                xpos = 0;
+                }
                 ++breakcount;
+                // reset pointers when we come around and see the \n
+                continue;
             }
-            // soft linebreak
-            else if ( s[loc] == ' ' && xpos > 0)
-            {
-                spacepos = loc;
-                ++xpos;
-            }
-            // bog-standard
-            else
-                ++xpos;
 
+            // save possible linebreak location
+            if ( s[loc] == ' ' && xpos > 0)
+            {
+                spaceloc = loc;
+            }
+
+            ++xpos;
             ++loc;
         }
     }
@@ -1334,10 +1490,10 @@ std::string get_linebreak_string(const std::string& s, int maxcol)
     return r;
 }
 
-// takes a (possibly tagged) string, breaks it into lines and 
+// takes a (possibly tagged) string, breaks it into lines and
 // prints it into the given message channel
 void print_formatted_paragraph(std::string &s, int maxcol,
-                               msg_channel_type channel) 
+                               msg_channel_type channel)
 {
     linebreak_string2(s,maxcol);
     std::string text;
@@ -1345,25 +1501,28 @@ void print_formatted_paragraph(std::string &s, int maxcol,
     size_t loc = 0, oldloc = 0;
     while ( loc < s.size() )
     {
-        if (s[loc] == '\n') {
+        if (s[loc] == '\n')
+        {
             text = s.substr(oldloc, loc-oldloc);
             formatted_message_history( text, channel );
             oldloc = ++loc;
         }
         loc++;
-    }    
+    }
     formatted_message_history( s.substr(oldloc, loc-oldloc), channel );
 }
 
 bool formatted_scroller::jump_to( int i )
 {
-    if ( i == first_entry + 1 )
-        return false;
-    if ( i == 0 )
+    if (i == first_entry + 1)
+        return (false);
+
+    if (i == 0)
         first_entry = 0;
     else
         first_entry = i - 1;
-    return true;
+
+    return (true);
 }
 
 // Don't scroll past MEL_TITLE entries
@@ -1371,8 +1530,8 @@ bool formatted_scroller::page_down()
 {
     const int old_first = first_entry;
 
-    if ( (int)items.size() <= first_entry + pagesize )
-        return false;
+    if ((int) items.size() <= first_entry + pagesize)
+        return (false);
 
     // If, when scrolling forward, we encounter a MEL_TITLE
     // somewhere in the newly displayed page, stop scrolling
@@ -1396,13 +1555,14 @@ bool formatted_scroller::page_up()
     // somewhere in the newly displayed page, stop scrolling
     // just before it becomes visible
 
-    if ( items[first_entry]->level == MEL_TITLE )
-        return false;
+    if (items[first_entry]->level == MEL_TITLE)
+        return (false);
 
     for ( int i = 0; i < pagesize; ++i )
     {
         if (first_entry == 0 || items[first_entry-1]->level == MEL_TITLE)
             break;
+
         --first_entry;
     }
 
@@ -1411,13 +1571,13 @@ bool formatted_scroller::page_up()
 
 bool formatted_scroller::line_down()
 {
-    if (first_entry + pagesize < (int) items.size() &&
+    if (first_entry + pagesize < static_cast<int>(items.size()) &&
         items[first_entry + pagesize]->level != MEL_TITLE )
     {
         ++first_entry;
-        return true;
+        return (true);
     }
-    return false;
+    return (false);
 }
 
 bool formatted_scroller::line_up()
@@ -1426,17 +1586,18 @@ bool formatted_scroller::line_up()
         items[first_entry]->level != MEL_TITLE)
     {
         --first_entry;
-        return true;
+        return (true);
     }
-    return false;
+    return (false);
 }
 
 bool formatted_scroller::jump_to_hotkey( int keyin )
-{    
-    for ( unsigned int i = 0; i < items.size(); ++i )
-        if ( items[i]->is_hotkey(keyin) )
+{
+    for (unsigned int i = 0; i < items.size(); ++i)
+        if (items[i]->is_hotkey(keyin))
             return jump_to(i);
-    return false;
+
+    return (false);
 }
 
 
@@ -1449,17 +1610,23 @@ bool formatted_scroller::process_key( int keyin )
     bool repaint = false;
     // Any key is assumed to be a movement key for now...
     bool moved = true;
-    switch ( keyin )
+    switch (keyin)
     {
     case 0:
-        return true;
+        return (true);
     case -1:
     case CK_ESCAPE:
-        return false;
+        return (false);
     case ' ': case '+': case '=': case CK_PGDN: case '>': case '\'':
+#ifdef USE_TILE
+    case CK_MOUSE_B5:
+#endif
         repaint = page_down();
         break;
     case '-': case CK_PGUP: case '<': case ';':
+#ifdef USE_TILE
+    case CK_MOUSE_B4:
+#endif
         repaint = page_up();
         break;
     case CK_UP:
@@ -1480,7 +1647,16 @@ bool formatted_scroller::process_key( int keyin )
         break;
     }
     default:
-        repaint = jump_to_hotkey(keyin);
+        if (is_set(MF_SINGLESELECT))
+        {
+            select_items( keyin, -1 );
+            get_selected( &sel );
+            if (sel.size() >= 1)
+                return (false);
+        }
+        else
+            repaint = jump_to_hotkey(keyin);
+
         break;
     }
 
@@ -1488,8 +1664,8 @@ bool formatted_scroller::process_key( int keyin )
         draw_menu();
     else if (moved && is_set(MF_EASY_EXIT))
         return (false);
-            
-    return true;
+
+    return (true);
 }
 
 int ToggleableMenu::pre_process(int key)
