@@ -1,3 +1,11 @@
+/*
+ *  File:       menu.h
+ *  Summary:    Menus and associated malarkey.
+ *  Written by: Darshan Shaligram
+ *
+ *  Modified for Crawl Reference by $Author: zelgadis $ on $Date: 2008-06-03T02:51:49.921266Z $
+ */
+
 #ifndef __MENU_H__
 #define __MENU_H__
 
@@ -11,6 +19,8 @@
 #include "defines.h"
 #include "libutil.h"
 
+class formatted_string;
+
 enum MenuEntryLevel
 {
     MEL_NONE = -1,
@@ -27,7 +37,7 @@ struct menu_letter
     menu_letter(char c) : letter(c) { }
 
     operator char () const { return letter; }
-    
+
     const menu_letter &operator ++ ()
     {
         letter = letter == 'z'? 'A' :
@@ -46,12 +56,15 @@ struct menu_letter
 };
 
 struct item_def;
+class Menu;
 
 int menu_colour(const std::string &itemtext,
-                const std::string &prefix = "");
+                const std::string &prefix = "",
+                const std::string &tag = "");
 
 struct MenuEntry
 {
+    std::string tag;
     std::string text;
     int quantity, selected_qty;
     int colour;
@@ -61,20 +74,21 @@ struct MenuEntry
 
     MenuEntry( const std::string &txt = std::string(),
                MenuEntryLevel lev = MEL_ITEM,
-               int qty = 0,
+               int qty  = 0,
                int hotk = 0 ) :
         text(txt), quantity(qty), selected_qty(0), colour(-1),
         hotkeys(), level(lev), data(NULL)
     {
-        colour = lev == MEL_ITEM?       LIGHTGREY :
-                 lev == MEL_SUBTITLE?   BLUE  :
-                                        WHITE;
+        colour = (lev == MEL_ITEM     ?  LIGHTGREY :
+                  lev == MEL_SUBTITLE ?  BLUE  :
+                                         WHITE);
         if (hotk)
             hotkeys.push_back( hotk );
     }
     virtual ~MenuEntry() { }
 
-    bool operator<( const MenuEntry& rhs ) const {
+    bool operator<( const MenuEntry& rhs ) const
+    {
         return text < rhs.text;
     }
 
@@ -83,7 +97,7 @@ struct MenuEntry
         if (key && !is_hotkey(key))
             hotkeys.push_back( key );
     }
-    
+
     bool is_hotkey( int key ) const
     {
         return find( hotkeys.begin(), hotkeys.end(), key ) != hotkeys.end();
@@ -91,9 +105,9 @@ struct MenuEntry
 
     bool is_primary_hotkey( int key ) const
     {
-        return hotkeys.size()? hotkeys[0] == key : false;
+        return (hotkeys.size()? hotkeys[0] == key : false);
     }
-    
+
     virtual std::string get_text() const
     {
         if (level == MEL_ITEM && hotkeys.size())
@@ -108,12 +122,12 @@ struct MenuEntry
 
     virtual int highlight_colour() const
     {
-        return (menu_colour(get_text()));
+        return (menu_colour(get_text(), "", tag));
     }
 
     virtual bool selected() const
     {
-        return selected_qty > 0 && quantity;
+        return (selected_qty > 0 && quantity);
     }
 
     virtual void select( int qty = -1 )
@@ -121,7 +135,7 @@ struct MenuEntry
         if (selected())
             selected_qty = 0;
         else if (quantity)
-            selected_qty = qty == -1? quantity : qty;
+            selected_qty = (qty == -1? quantity : qty);
     }
 };
 
@@ -152,7 +166,7 @@ enum MenuFlag
     MF_SINGLESELECT     = 0x0001,   // Select just one item
     MF_MULTISELECT      = 0x0002,   // Select multiple items
     MF_NO_SELECT_QTY    = 0x0004,   // Disallow partial selections
-    MF_ANYPRINTABLE     = 0x0008,   // Any printable character is valid, and 
+    MF_ANYPRINTABLE     = 0x0008,   // Any printable character is valid, and
                                     // closes the menu.
     MF_SELECT_BY_PAGE   = 0x0010,   // Allow selections to occur only on
                                     // currently visible page.
@@ -163,6 +177,7 @@ enum MenuFlag
     MF_ALLOW_FILTER     = 0x0080,   // Control-F will ask for regex and
                                     // select the appropriate items.
     MF_ALLOW_FORMATTING = 0x0100,   // Parse index for formatted-string
+    MF_SHOW_PAGENUMBERS = 0x0200,   // Show "(page X of Y)" when appropriate
     MF_EASY_EXIT        = 0x1000
 };
 
@@ -173,10 +188,26 @@ enum MenuFlag
 // you pass in MUST be allocated with new, or Crawl will crash.
 
 #define NUMBUFSIZ 10
+
+// FIXME: MenuEntry is a large object, and shouldn't be used for
+// showing text files.
+
 class Menu
 {
 public:
-    Menu( int flags = MF_MULTISELECT );
+    Menu( int flags = MF_MULTISELECT, const std::string& tagname = "" );
+
+    // Initializes a Menu from a formatted_string as follows:
+    //
+    // 1) Splits the formatted_string on EOL (this is not necessarily \n).
+    // 2) Picks the most recently used non-whitespace colour as the colour
+    //    for the next line (so it can't do multiple colours on one line).
+    // 3) Ignores all cursor movement ops in the formatted_string.
+    //
+    // These are limitations that should be fixed eventually.
+    //
+    Menu( const formatted_string &fs );
+
     virtual ~Menu();
 
     // Remove all items from the Menu, leave title intact.
@@ -187,13 +218,15 @@ public:
     void set_flags(int new_flags, bool use_options = true);
     int  get_flags() const        { return flags; }
     virtual bool is_set( int flag ) const;
-    
+    void set_tag(const std::string& t) { tag = t; }
+
     bool draw_title_suffix( const std::string &s, bool titlefirst = true );
+    bool draw_title_suffix( const formatted_string &fs, bool titlefirst = true );
     void update_title();
 
     // Sets a replacement for the --more-- string.
     void set_more(const formatted_string &more);
-    
+
     void set_highlighter( MenuHighlighter *h );
     void set_title( MenuEntry *e );
     void add_entry( MenuEntry *entry );
@@ -214,6 +247,9 @@ public:
 
     size_t item_count() const    { return items.size(); }
 
+    // Get entry index, skipping quantity 0 entries. Returns -1 if not found.
+    int get_entry_index( const MenuEntry *e ) const;
+
 public:
     typedef std::string (*selitem_tfn)( const std::vector<MenuEntry*> *sel );
     typedef void (*drawitem_tfn)(int index, const MenuEntry *me);
@@ -226,7 +262,8 @@ public:
 protected:
     MenuEntry *title;
     int flags;
-    
+    std::string tag;
+
     int first_entry, y_offset;
     int pagesize, max_pagesize;
 
@@ -248,6 +285,8 @@ protected:
     int last_selected;
 
 protected:
+    void check_add_formatted_line(int firstcol, int nextcol,
+                                  std::string &line, bool check_eol);
     void do_menu();
     virtual void draw_select_count(int count, bool force = false);
     virtual void draw_item( int index ) const;
@@ -272,10 +311,10 @@ protected:
     void select_index( int index, int qty = -1 );
 
     bool is_hotkey(int index, int key );
-    bool is_selectable(int index) const;
+    virtual bool is_selectable(int index) const;
 
     virtual int item_colour(int index, const MenuEntry *me) const;
-    
+
     virtual bool process_key( int keyin );
 };
 
@@ -352,7 +391,7 @@ public:
     column_composer(int ncols, ...);
 
     void clear();
-    void add_formatted(int ncol, 
+    void add_formatted(int ncol,
             const std::string &tagged_text,
             bool  add_separator = true,
             bool  eol_ends_format = true,
@@ -386,9 +425,11 @@ private:
 };
 
 // This class is for when (some of) your items are formatted, and
-// selection is impossible...in other words, you just want a browser.
-// Hotkeys can be set on menu items, in which case you can jump
-// to them by pressing the appropriate key.
+// you want mostly a browser.
+//
+// If MF_NOSELECT, hotkeys jump to menu items.
+// If MF_SINGLESELECT, hotkeys end the menu immediately.
+// MF_MULTISELECT is not supported.
 class formatted_scroller : public Menu
 {
 public:
