@@ -3,7 +3,7 @@
  *  Summary:    Functions used to print messages.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-17T08:11:52.537934Z $
  *
  *  Change History (most recent first):
  *
@@ -18,6 +18,7 @@
 
 #include <cstdarg>
 #include <cstring>
+#include <sstream>
 
 #ifdef DOS
 #include <conio.h>
@@ -40,8 +41,9 @@
 #include "travel.h"
 #include "tutorial.h"
 #include "view.h"
+#include "menu.h"
 
-// circular buffer for keeping past messages
+// Circular buffer for keeping past messages.
 message_item Store_Message[ NUM_STORED_MESSAGES ];    // buffer of old messages
 int Next_Message = 0;                                 // end of messages
 
@@ -78,7 +80,7 @@ namespace msg
         stream << std::nounitbuf;
     }
 
-    void deinitalise_mpr_streams()
+    void deinitialise_mpr_streams()
     {
         for (unsigned int i = 0; i < stream_ptrs.size(); ++i)
             delete stream_ptrs[i];
@@ -107,7 +109,7 @@ namespace msg
     {
         this->muted = m;
     }
-    
+
     // again, can be improved
     int mpr_stream_buf::overflow(int c)
     {
@@ -119,7 +121,7 @@ namespace msg
             // null-terminate and print the string
             internal_buf[internal_count] = 0;
             mpr(internal_buf, channel, param);
-            
+
             internal_count = 0;
 
             // reset to defaults (param changing isn't sticky)
@@ -127,10 +129,10 @@ namespace msg
         }
         else
             internal_buf[internal_count++] = c;
-        
+
         if ( internal_count + 3 > INTERNAL_LENGTH )
         {
-            mpr("oops, hit overflow", MSGCH_DANGER);
+            mpr("oops, hit overflow", MSGCH_ERROR);
             internal_count = 0;
             return std::streambuf::traits_type::eof();
         }
@@ -168,7 +170,7 @@ static char god_message_altar_colour( god_type god )
         return (WHITE);
 
     case GOD_ELYVILON:
-        return (LIGHTBLUE);     // really, LIGHTGREY but that's plain text
+        return (LIGHTBLUE);     // Really, LIGHTGREY but that's plain text.
 
     case GOD_OKAWARU:
         return (CYAN);
@@ -219,15 +221,18 @@ static char god_message_altar_colour( god_type god )
 
 #ifdef USE_COLOUR_MESSAGES
 
-// returns a colour or MSGCOL_MUTED
+// Returns a colour or MSGCOL_MUTED.
 int channel_to_colour( msg_channel_type channel, int param )
 {
+    if (you.asleep())
+        return (DARKGREY);
+
     char ret;
 
     switch (Options.channels[ channel ])
     {
     case MSGCOL_PLAIN:
-        // note that if the plain channel is muted, then we're protecting
+        // Note that if the plain channel is muted, then we're protecting
         // the player from having that spread to other channels here.
         // The intent of plain is to give non-coloured messages, not to
         // suppress them.
@@ -244,8 +249,8 @@ int channel_to_colour( msg_channel_type channel, int param )
         case MSGCH_GOD:
         case MSGCH_PRAY:
             ret = (Options.channels[ channel ] == MSGCOL_DEFAULT)
-                ? god_colour( static_cast<god_type>(param) )
-                : god_message_altar_colour( static_cast<god_type>(param) );
+                   ? god_colour( static_cast<god_type>(param) )
+                   : god_message_altar_colour( static_cast<god_type>(param) );
             break;
 
         case MSGCH_DURATION:
@@ -257,11 +262,15 @@ int channel_to_colour( msg_channel_type channel, int param )
             break;
 
         case MSGCH_WARN:
+        case MSGCH_ERROR:
             ret = LIGHTRED;
             break;
 
         case MSGCH_FOOD:
-            ret = YELLOW;
+            if (param) // positive change
+                ret = GREEN;
+            else
+                ret = YELLOW;
             break;
 
         case MSGCH_INTRINSIC_GAIN:
@@ -287,12 +296,14 @@ int channel_to_colour( msg_channel_type channel, int param )
 
         case MSGCH_MONSTER_SPELL:
         case MSGCH_MONSTER_ENCHANT:
+        case MSGCH_FRIEND_SPELL:
+        case MSGCH_FRIEND_ENCHANT:
             ret = LIGHTMAGENTA;
             break;
 
         case MSGCH_MONSTER_DAMAGE:
             ret =  ((param == MDAM_DEAD)               ? RED :
-                    (param >= MDAM_HORRIBLY_DAMAGED)   ? LIGHTRED :
+                    (param >= MDAM_SEVERELY_DAMAGED)   ? LIGHTRED :
                     (param >= MDAM_MODERATELY_DAMAGED) ? YELLOW
                                                        : LIGHTGREY);
             break;
@@ -307,6 +318,7 @@ int channel_to_colour( msg_channel_type channel, int param )
             break;
 
         case MSGCH_PLAIN:
+        case MSGCH_FRIEND_ACTION:
         case MSGCH_ROTTEN_MEAT:
         case MSGCH_EQUIPMENT:
         case MSGCH_EXAMINE:
@@ -326,9 +338,9 @@ int channel_to_colour( msg_channel_type channel, int param )
         // cases should be handled above.
         if (channel == MSGCH_MONSTER_DAMAGE)
         {
-            // a special case right now for monster damage (at least until
+            // A special case right now for monster damage (at least until
             // the init system is improved)... selecting a specific
-            // colour here will result in only the death messages coloured
+            // colour here will result in only the death messages coloured.
             if (param == MDAM_DEAD)
                 ret = Options.channels[ channel ];
             else if (Options.channels[ MSGCH_PLAIN ] >= MSGCOL_DEFAULT)
@@ -353,14 +365,17 @@ int channel_to_colour( msg_channel_type channel, int param )
 
 #endif
 
-static void do_message_print( msg_channel_type channel, int param, 
+static void do_message_print( msg_channel_type channel, int param,
                               const char *format, va_list argp )
 {
     char buff[200];
-    vsnprintf( buff, sizeof( buff ), format, argp ); 
+    vsnprintf( buff, sizeof( buff ), format, argp );
     buff[199] = 0;
 
     mpr(buff, channel, param);
+
+    if (channel == MSGCH_ERROR)
+        interrupt_activity( AI_FORCE_INTERRUPT );
 }
 
 void mprf( msg_channel_type channel, int param, const char *format, ... )
@@ -369,13 +384,13 @@ void mprf( msg_channel_type channel, int param, const char *format, ... )
     va_start( argp, format );
     do_message_print( channel, param, format, argp );
     va_end( argp );
-}    
+}
 
 void mprf( msg_channel_type channel, const char *format, ... )
 {
     va_list  argp;
     va_start( argp, format );
-    do_message_print( channel, channel == MSGCH_GOD? you.religion : 0,
+    do_message_print( channel, channel == MSGCH_GOD ? you.religion : 0,
                       format, argp );
     va_end( argp );
 }
@@ -392,47 +407,67 @@ void mpr(const char *inf, msg_channel_type channel, int param)
 {
     if (!crawl_state.io_inited)
     {
-        if (channel == MSGCH_WARN)
+        if (channel == MSGCH_ERROR)
             fprintf(stderr, "%s\n", inf);
         return;
     }
+
+    if (channel == MSGCH_GOD && param == 0)
+        param = you.religion;
+
+    std::string help = inf;
+    if (help.find("</") != std::string::npos)
+    {
+        std::string col = colour_to_str(channel_to_colour(channel));
+        if (!col.empty())
+            help = "<" + col + ">" + help + "</" + col + ">";
+
+        // Handing over to the experts...
+        formatted_mpr(formatted_string::parse_string(help), channel);
+        return;
+    }
+
     char mbuf[400];
     size_t i = 0;
     const int stepsize = get_number_of_cols() - 1;
     const size_t msglen = strlen(inf);
     const int lookback_size = (stepsize < 12 ? 0 : 12);
-    // if a message is exactly STEPSIZE characters long,
+
+    // If a message is exactly STEPSIZE characters long,
     // it should precisely fit in one line. The printing is thus
     // from I to I + STEPSIZE - 1. Stop when I reaches MSGLEN.
-    while ( i < msglen || i == 0 )
+    while (i < msglen || i == 0)
     {
         strncpy( mbuf, inf + i, stepsize );
         mbuf[stepsize] = 0;
-        // did the message break?
-        if ( i + stepsize < msglen )
+        // Did the message break?
+        if (i + stepsize < msglen)
         {
-            // yes, find a nicer place to break it.
+            // Yes, find a nicer place to break it.
             int lookback, where = 0;
-            for ( lookback = 0; lookback < lookback_size; ++lookback )
+            for (lookback = 0; lookback < lookback_size; ++lookback)
             {
                 where = stepsize - 1 - lookback;
-                if ( where >= 0 && isspace(mbuf[where]) )
+                if (where >= 0 && isspace(mbuf[where]))
+                {
                     // aha!
                     break;
+                }
             }
 
-            if ( lookback != lookback_size )
+            if (lookback != lookback_size)
             {
-                // found a good spot to break
+                // Found a good spot to break.
                 mbuf[where] = 0;
-                i += where + 1; // skip past the space!
+                i += where + 1; // Skip past the space!
             }
             else
                 i += stepsize;
         }
         else
             i += stepsize;
-        base_mpr( mbuf, channel, param );
+
+        base_mpr(mbuf, channel, param);
     }
 }
 
@@ -448,7 +483,7 @@ void mpr_comma_separated_list(const std::string prefix,
     std::string out = prefix;
     unsigned width  = get_number_of_cols() - 1;
 
-    for(int i = 0, size = list.size(); i < size; i++)
+    for (int i = 0, size = list.size(); i < size; i++)
     {
         std::string new_str = list[i];
 
@@ -456,6 +491,8 @@ void mpr_comma_separated_list(const std::string prefix,
             new_str += comma;
         else if (i == (size - 2))
             new_str += andc;
+        else if (i == (size - 1))
+            new_str += ".";
 
         if (out.length() + new_str.length() >= width)
         {
@@ -469,8 +506,8 @@ void mpr_comma_separated_list(const std::string prefix,
 }
 
 
-// checks whether a given message contains patterns relevant for
-// notes, stop_running or sounds and handles these cases
+// Checks whether a given message contains patterns relevant for
+// notes, stop_running or sounds and handles these cases.
 static void mpr_check_patterns(const std::string& message,
                                msg_channel_type channel,
                                int param)
@@ -484,15 +521,19 @@ static void mpr_check_patterns(const std::string& message,
         }
     }
 
-    // reusing travel_stop_message here
     if (channel != MSGCH_DIAGNOSTICS && channel != MSGCH_EQUIPMENT
         && channel != MSGCH_TALK && channel != MSGCH_TALK_VISUAL
-        && channel != MSGCH_SOUND)
+        && channel != MSGCH_FRIEND_SPELL && channel != MSGCH_FRIEND_ENCHANT
+        && channel != MSGCH_FRIEND_ACTION && channel != MSGCH_SOUND)
     {
         interrupt_activity( AI_MESSAGE,
                             channel_to_str(channel) + ":" + message );
     }
 
+    // Any sound has a chance of waking the PC if the PC is asleep.
+    if (channel == MSGCH_SOUND)
+        you.check_awaken(5);
+
     // Check messages for all forms of running now.
     if (you.running)
     {
@@ -519,15 +560,46 @@ static void mpr_check_patterns(const std::string& message,
             }
         }
     }
-
 }
 
-// adds a given message to the message history
+static bool channel_message_history(msg_channel_type channel)
+{
+    switch (channel)
+    {
+    case MSGCH_PROMPT:
+    case MSGCH_EQUIPMENT:
+    case MSGCH_EXAMINE_FILTER:
+       return (false);
+    default:
+       return (true);
+    }
+}
+
+// Adds a given message to the message history.
 static void mpr_store_messages(const std::string& message,
                                msg_channel_type channel, int param)
 {
     const int num_lines = crawl_view.msgsz.y;
 
+    bool was_repeat = false;
+
+    if (Options.msg_condense_repeats)
+    {
+        int prev_message_num = Next_Message - 1;
+
+        if (prev_message_num < 0)
+            prev_message_num = NUM_STORED_MESSAGES - 1;
+
+        message_item &prev_message = Store_Message[prev_message_num];
+
+        if (prev_message.repeats > 0 && prev_message.channel == channel
+            && prev_message.param == param && prev_message.text == message)
+        {
+            prev_message.repeats++;
+            was_repeat = true;
+        }
+    }
+
     // Prompt lines are presumably shown to / seen by the player accompanied
     // by a request for input, which should do the equivalent of a more(); to
     // save annoyance, don't bump New_Message_Count for prompts.
@@ -537,16 +609,17 @@ static void mpr_store_messages(const std::string& message,
     if (Message_Line < num_lines - 1)
         Message_Line++;
 
-    // reset colour
+    // Reset colour.
     textcolor(LIGHTGREY);
 
-    // equipment lists just waste space in the message recall
-    if (channel != MSGCH_EQUIPMENT && channel != MSGCH_EXAMINE_FILTER)
+    // Equipment lists just waste space in the message recall.
+    if (!was_repeat && channel_message_history(channel))
     {
         // Put the message into Store_Message, and move the '---' line forward
-        Store_Message[ Next_Message ].text = message;
+        Store_Message[ Next_Message ].text    = message;
         Store_Message[ Next_Message ].channel = channel;
-        Store_Message[ Next_Message ].param = param;
+        Store_Message[ Next_Message ].param   = param;
+        Store_Message[ Next_Message ].repeats = 1;
         Next_Message++;
 
         if (Next_Message >= NUM_STORED_MESSAGES)
@@ -563,15 +636,15 @@ static int prepare_message(const std::string& imsg, msg_channel_type channel,
                            int param)
 {
     if (suppress_messages)
-        return MSGCOL_MUTED;    
+        return MSGCOL_MUTED;
 
     int colour = channel_to_colour( channel, param );
 
-    const std::vector<message_colour_mapping>& mcm =
-        Options.message_colour_mappings;
+    const std::vector<message_colour_mapping>& mcm
+               = Options.message_colour_mappings;
     typedef std::vector<message_colour_mapping>::const_iterator mcmci;
 
-    for ( mcmci ci = mcm.begin(); ci != mcm.end(); ++ci )
+    for (mcmci ci = mcm.begin(); ci != mcm.end(); ++ci)
     {
         if (ci->message.is_filtered(channel, imsg))
         {
@@ -580,15 +653,16 @@ static int prepare_message(const std::string& imsg, msg_channel_type channel,
         }
     }
 
-    if ( colour != MSGCOL_MUTED )
+    if (colour != MSGCOL_MUTED)
     {
         mpr_check_patterns(imsg, channel, param);
         flush_input_buffer( FLUSH_ON_MESSAGE );
         const int num_lines = crawl_view.msgsz.y;
-    
+
         if (New_Message_Count == num_lines - 1)
             more();
     }
+
     return colour;
 }
 
@@ -596,16 +670,16 @@ static void base_mpr(const char *inf, msg_channel_type channel, int param)
 {
     const std::string imsg = inf;
     const int colour = prepare_message( imsg, channel, param );
-    
-    if ( colour == MSGCOL_MUTED )
+
+    if (colour == MSGCOL_MUTED)
         return;
 
-    if (silenced(you.x_pos, you.y_pos) &&
-        (channel == MSGCH_SOUND || channel == MSGCH_TALK))
+    if (silenced(you.x_pos, you.y_pos)
+        && (channel == MSGCH_SOUND || channel == MSGCH_TALK))
     {
         return;
     }
-   
+
     if (need_prefix)
     {
         message_out( Message_Line, colour, "-", 1, false );
@@ -615,38 +689,55 @@ static void base_mpr(const char *inf, msg_channel_type channel, int param)
     message_out( Message_Line, colour, inf,
                  Options.delay_message_clear? 2 : 1 );
 
+    for (unsigned i = 0; i < Options.force_more_message.size(); ++i)
+    {
+        if (Options.force_more_message[i].is_filtered( channel, imsg ))
+        {
+            more();
+            New_Message_Count = 0;
+            // One more() is quite enough, thank you!
+            break;
+        }
+    }
+
     mpr_store_messages(imsg, channel, param);
-}                               // end mpr()
+}
 
 
 static void mpr_formatted_output(formatted_string fs, int colour)
 {
-    int curcol = 1;
+    int curcol = Options.delay_message_clear ? 2 : 1;
 
     if (need_prefix)
     {
         message_out( Message_Line, colour, "-", 1, false );
-        ++curcol;
         need_prefix = false;
     }
 
-    for ( unsigned i = 0; i < fs.ops.size(); ++i )
+    // Find last text op so that we can scroll the output.
+    unsigned last_text = fs.ops.size();
+    for (unsigned i = 0; i < fs.ops.size(); ++i)
     {
-        switch ( fs.ops[i].type )
+        if (fs.ops[i].type == FSOP_TEXT)
+            last_text = i;
+    }
+
+    for (unsigned i = 0; i < fs.ops.size(); ++i)
+    {
+        switch (fs.ops[i].type)
         {
         case FSOP_COLOUR:
             colour = fs.ops[i].x;
             break;
         case FSOP_TEXT:
             message_out(Message_Line, colour, fs.ops[i].text.c_str(), curcol,
-                        false);
+                        (i == last_text));
             curcol += multibyte_strlen(fs.ops[i].text);
             break;
         case FSOP_CURSOR:
             break;
         }
     }
-    message_out( Message_Line, colour, "", Options.delay_message_clear? 2 : 1);
 }
 
 // Line wrapping is not available here!
@@ -664,11 +755,16 @@ void formatted_mpr(const formatted_string& fs, msg_channel_type channel,
     mpr_store_messages(imsg, channel, param);
 }
 
-// output given string as formatted message, but check patterns
-// for string stripped of tags and store original tagged string
-// for message history
-void formatted_message_history(const std::string &st, msg_channel_type channel,
-                               int param)
+// Output given string as formatted message(s), but check patterns
+// for string stripped of tags and store the original tagged string
+// for message history.  Newlines break the string into multiple
+// messages.
+//
+// If wrap_col > 0, text is wrapped at that column.
+//
+void formatted_message_history(const std::string &st_nocolor,
+                               msg_channel_type channel,
+                               int param, int wrap_col)
 {
     if (suppress_messages)
         return;
@@ -677,20 +773,51 @@ void formatted_message_history(const std::string &st, msg_channel_type channel,
     if (colour == MSGCOL_MUTED)
         return;
 
-    formatted_string fs = formatted_string::parse_string(st);
+    // Apply channel color explicitly, so "foo <w>bar</w> baz"
+    // renders "baz" correctly
+    std::string st;
+    {
+        std::ostringstream text;
+        const std::string colour_str = colour_to_str(colour);
+        text << "<" << colour_str << ">"
+             << st_nocolor
+             << "</" << colour_str << ">";
+        text.str().swap(st);
+    }
 
-    mpr_check_patterns(fs.tostring(), channel, param);
+    if (wrap_col)
+        linebreak_string2(st, wrap_col);
 
-    flush_input_buffer( FLUSH_ON_MESSAGE );
+    std::vector<formatted_string> fss;
+    formatted_string::parse_string_to_multiple(st, fss);
 
-    const int num_lines = crawl_view.msgsz.y;
+    for (unsigned int i = 0; i < fss.size(); i++)
+    {
+        const formatted_string& fs = fss[i];
+        const std::string unformatted = fs.tostring();
+        mpr_check_patterns(unformatted, channel, param);
 
-    if (New_Message_Count == num_lines - 1)
-        more();
+        flush_input_buffer( FLUSH_ON_MESSAGE );
 
-    mpr_formatted_output(fs, colour);
+        const int num_lines = crawl_view.msgsz.y;
 
-    mpr_store_messages(st, channel, param);
+        if (New_Message_Count == num_lines - 1)
+            more();
+
+        mpr_formatted_output(fs, colour);
+
+        for (unsigned f = 0; f < Options.force_more_message.size(); ++f)
+        {
+            if (Options.force_more_message[f].is_filtered(channel, st_nocolor))
+            {
+                more();
+                New_Message_Count = 0;
+                // One more() is quite enough, thank you!
+                break;
+            }
+        }
+        mpr_store_messages(fs.to_colour_string(), channel, param);
+    }
 }
 
 bool any_messages(void)
@@ -702,8 +829,8 @@ void mesclr( bool force )
 {
     New_Message_Count = 0;
 
-    // if no messages, return.
-    if (!any_messages())
+    // If no messages, return.
+    if (!any_messages() && !force)
         return;
 
     if (!force && Options.delay_message_clear)
@@ -712,13 +839,13 @@ void mesclr( bool force )
         return;
     }
 
-    // turn cursor off -- avoid 'cursor dance'
+    // Turn cursor off -- avoid 'cursor dance'.
 
     cursor_control cs(false);
     clear_message_window();
     need_prefix = false;
     Message_Line = 0;
-}                               // end mseclr()
+}
 
 void more(void)
 {
@@ -729,23 +856,41 @@ void more(void)
         return;
     }
 #endif
+
+    if (crawl_state.is_replaying_keys())
+    {
+        mesclr();
+        return;
+    }
+
     if (Options.show_more_prompt && !suppress_messages)
     {
-        char keypress = 0;
+        int keypress = 0;
 
         if (Options.tutorial_left)
+        {
             message_out(crawl_view.msgsz.y - 1,
                         LIGHTGREY,
                         "--more--                        "
-                        "Press Ctrl-P to reread old messages",
+                        "Press Ctrl-P to reread old messages.",
                         2, false);
+        }
         else
+        {
             message_out(crawl_view.msgsz.y - 1,
                         LIGHTGREY, "--more--", 2, false);
+        }
+
+#ifdef USE_TILE
+        mouse_control mc(MOUSE_MODE_MORE);
+#endif
 
         do
+        {
             keypress = getch();
-        while (keypress != ' ' && keypress != '\r' && keypress != '\n');
+        }
+        while (keypress != ' ' && keypress != '\r' && keypress != '\n'
+               && keypress != -1);
     }
     mesclr(true);
 }                               // end more()
@@ -759,20 +904,24 @@ static bool is_channel_dumpworthy(msg_channel_type channel)
 
 std::string get_last_messages(int mcount)
 {
-    if (mcount <= 0) return std::string();
-    if (mcount > NUM_STORED_MESSAGES) mcount = NUM_STORED_MESSAGES;
+    if (mcount <= 0)
+        return std::string();
 
-    bool full_buffer = Store_Message[ NUM_STORED_MESSAGES - 1 ].text.length() == 0;
+    if (mcount > NUM_STORED_MESSAGES)
+        mcount = NUM_STORED_MESSAGES;
+
+    bool full_buffer
+       = (Store_Message[ NUM_STORED_MESSAGES - 1 ].text.length() == 0);
     int initial = Next_Message - mcount;
     if (initial < 0 || initial > NUM_STORED_MESSAGES)
-        initial = full_buffer? initial + NUM_STORED_MESSAGES : 0;
+        initial = full_buffer ? initial + NUM_STORED_MESSAGES : 0;
 
     std::string text;
     int count = 0;
     for (int i = initial; i != Next_Message; )
     {
         const message_item &msg = Store_Message[i];
-        
+
         if (msg.text.length() && is_channel_dumpworthy(msg.channel))
         {
             text += msg.text;
@@ -785,25 +934,26 @@ std::string get_last_messages(int mcount)
     }
 
     // An extra line of clearance.
-    if (count) text += EOL;
+    if (count)
+        text += EOL;
 
     return text;
 }
 
 void replay_messages(void)
 {
-    int            win_start_line = 0;
-    unsigned char  keyin;
+    int        win_start_line = 0;
+    int        keyin;
 
-    bool           full_buffer = true;
-    int            num_msgs = NUM_STORED_MESSAGES;
-    int            first_message = Next_Message;
+    bool       full_buffer = true;
+    int        num_msgs = NUM_STORED_MESSAGES;
+    int        first_message = Next_Message;
 
-    const int      num_lines = get_number_of_lines();
+    const int  num_lines = get_number_of_lines();
 
     if (Store_Message[ NUM_STORED_MESSAGES - 1 ].text.length() == 0)
     {
-        full_buffer = false;
+        full_buffer   = false;
         first_message = 0;
         num_msgs = Next_Message;
     }
@@ -825,64 +975,57 @@ void replay_messages(void)
     // Turn off the cursor
     cursor_control cursoff(false);
 
-    for(;;)
+    while (true)
     {
         clrscr();
 
-        gotoxy(1, 1);
+        cgotoxy(1, 1, GOTO_CRT);
 
         for (int i = 0; i < num_lines - 2; i++)
         {
-            // calculate line in circular buffer
+            // Calculate line in circular buffer.
             int line = win_start_line + i;
             if (line >= NUM_STORED_MESSAGES)
                 line -= NUM_STORED_MESSAGES;
 
-            // avoid wrap-around
+            // Avoid wrap-around.
             if (line == first_message && i != 0)
                 break;
 
-            int colour = channel_to_colour( Store_Message[ line ].channel,
-                                            Store_Message[ line ].param );
+            int colour = prepare_message( Store_Message[ line ].text,
+                                          Store_Message[ line ].channel,
+                                          Store_Message[ line ].param );
             if (colour == MSGCOL_MUTED)
                 continue;
 
             textcolor( colour );
 
             std::string text = Store_Message[ line ].text;
-            // for tutorial texts (for now, used for debugging)
-            // allow formatted output of tagged messages
-            if (Store_Message[ line ].channel == MSGCH_TUTORIAL)
+
+            // Allow formatted output of tagged messages.
+            formatted_string fs = formatted_string::parse_string(text, true);
+            int curcol = 1;
+            for ( unsigned int j = 0; j < fs.ops.size(); ++j )
             {
-                formatted_string fs = formatted_string::parse_string(text);
-                int curcol = 1;
-                for ( unsigned int j = 0; j < fs.ops.size(); ++j )
+                switch ( fs.ops[j].type )
                 {
-                    switch ( fs.ops[j].type )
-                    {
-                    case FSOP_COLOUR:
-                        colour = fs.ops[j].x;
-                        break;
-                    case FSOP_TEXT:
-                        textcolor( colour );
-                        gotoxy(curcol, wherey());
-                        cprintf(fs.ops[j].text.c_str());
-                        curcol += multibyte_strlen(fs.ops[j].text);
-                        break;
-                    case FSOP_CURSOR:
-                        break;
-                    }
+                case FSOP_COLOUR:
+                    colour = fs.ops[j].x;
+                    break;
+                case FSOP_TEXT:
+                    textcolor( colour );
+                    cgotoxy(curcol, wherey(), GOTO_CRT);
+                    cprintf(fs.ops[j].text.c_str());
+                    curcol += multibyte_strlen(fs.ops[j].text);
+                    break;
+                case FSOP_CURSOR:
+                    break;
                 }
             }
-            else
-#if DEBUG_DIAGNOSTICS
-            cprintf( "%d: %s", line, text.c_str() );
-#else
-            cprintf( "%s", text.c_str() );
-#endif
-
-            cprintf(EOL);
             textcolor(LIGHTGREY);
+            if (Store_Message[ line ].repeats > 1)
+                cprintf(" (x%d)", Store_Message[ line ].repeats);
+            cprintf(EOL);
         }
 
         // print a footer -- note: relative co-ordinates start at 1
@@ -896,7 +1039,7 @@ void replay_messages(void)
         }
         else if (win_start_line >= first_message)
             rel_start = win_start_line - first_message + 1;
-        else 
+        else
             rel_start = (win_start_line + NUM_STORED_MESSAGES) - first_message + 1;
 
         int rel_end = rel_start + (num_lines - 2) - 1;
@@ -907,8 +1050,8 @@ void replay_messages(void)
         cprintf(EOL);
         cprintf( "<< Lines %d-%d of %d (Up: k<-8, Down: j>+2) >>",
                  rel_start, rel_end, num_msgs );
-                 
-        keyin = get_ch();
+
+        keyin = getch();
 
         if ((full_buffer && NUM_STORED_MESSAGES > num_lines - 2)
             || (!full_buffer && Next_Message > num_lines - 2))
@@ -916,10 +1059,15 @@ void replay_messages(void)
             int new_line;
             int end_mark;
 
+#ifdef USE_TILE
+            if (keyin == CK_MOUSE_B4) keyin = '8';
+            if (keyin == CK_MOUSE_B5) keyin = '2';
+#endif
+
             if (keyin == 'k' || keyin == '8' || keyin == '-' || keyin == '<')
             {
                 new_line = win_start_line - (num_lines - 2);
-                
+
                 // end_mark is equivalent to Next_Message, but
                 // is always less than win_start_line.
                 end_mark = first_message;
@@ -955,7 +1103,7 @@ void replay_messages(void)
 
                 // hit bottom
                 if (new_line >= end_mark - (num_lines - 2))
-                    new_line = end_mark - (num_lines - 2) + 1; 
+                    new_line = end_mark - (num_lines - 2) + 1;
 
                 if (new_line >= NUM_STORED_MESSAGES)
                     new_line -= NUM_STORED_MESSAGES;
