@@ -3,7 +3,7 @@
  *  Summary:    Functions used to print messages.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: zelgadis $ on $Date: 2007-09-15T23:33:50.620328Z $
+ *  Modified for Crawl Reference by $Author: dolorous $ on $Date: 2008-06-25T17:20:38.534660Z $
  *
  *  Change History (most recent first):
  *
@@ -21,10 +21,17 @@
 #include "enum.h"
 #include "mpr.h"
 
-struct message_item {
+class message_item {
+public:
     msg_channel_type    channel;        // message channel
     int                 param;          // param for channel (god, enchantment)
     std::string         text;           // text of message
+    int                 repeats;
+
+    message_item() : channel(NUM_MESSAGE_CHANNELS), param(0), text(""),
+                     repeats(0)
+        {
+        }
 };
 
 
@@ -50,10 +57,11 @@ class formatted_string;
 
 void formatted_mpr(const formatted_string& fs,
                    msg_channel_type channel = MSGCH_PLAIN, int param = 0);
-                   
+
 void formatted_message_history(const std::string &st,
                                msg_channel_type channel = MSGCH_PLAIN,
-                               int param = 0);
+                               int param = 0,
+                               int wrap_col = 0);
 
 // mpr() an arbitrarily long list of strings
 void mpr_comma_separated_list(const std::string prefix,
@@ -62,7 +70,7 @@ void mpr_comma_separated_list(const std::string prefix,
                               const std::string &comma = ", ",
                               const msg_channel_type channel = MSGCH_PLAIN,
                               const int param = 0);
-                               
+
 class no_messages
 {
 public:
@@ -121,7 +129,7 @@ namespace msg
     };
 
     void initialise_mpr_streams();
-    void deinitalise_mpr_streams();
+    void deinitialise_mpr_streams();
 }
 
 std::ostream& operator<<(std::ostream& os, const msg::setparam& sp);
