@@ -3,7 +3,7 @@
  *  Summary:    Misc monster related functions.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-10-01T10:47:43.653687Z $
+ *  Modified for Crawl Reference by $Author: dolorous $ on $Date: 2008-07-02T23:48:38.320257Z $
  *
  *  Change History (most recent first):
  *
@@ -22,7 +22,7 @@ enum mon_dam_level_type
     MDAM_LIGHTLY_DAMAGED,
     MDAM_MODERATELY_DAMAGED,
     MDAM_HEAVILY_DAMAGED,
-    MDAM_HORRIBLY_DAMAGED,
+    MDAM_SEVERELY_DAMAGED,
     MDAM_ALMOST_DEAD,
     MDAM_DEAD
 };
@@ -34,13 +34,29 @@ enum mon_desc_type   // things that cross categorical lines {dlb}
     MDSC_NOMSG_WOUNDS
 };
 
+struct level_exit
+{
+    coord_def target;
+    bool unreachable;
+
+public:
+    level_exit(coord_def t = coord_def(-1, -1),
+               bool u = true)
+
+        : target(t), unreachable(u)
+    {
+    }
+};
+
 #define YOU_KILL(x) ((x) == KILL_YOU || (x) == KILL_YOU_MISSILE \
                      || (x) == KILL_YOU_CONF)
 #define MON_KILL(x) ((x) == KILL_MON || (x) == KILL_MON_MISSILE)
 
 // useful macro
-#define SAME_ATTITUDE(x) (mons_friendly(x)? BEH_FRIENDLY:    \
-                          mons_neutral(x)?  BEH_NEUTRAL : BEH_HOSTILE)
+#define SAME_ATTITUDE(x) (mons_friendly(x)     ? BEH_FRIENDLY : \
+                          mons_good_neutral(x) ? BEH_GOOD_NEUTRAL : \
+                          mons_neutral(x)      ? BEH_NEUTRAL \
+                                               : BEH_HOSTILE)
 
 #define MONST_INTERESTING(x) (x->flags & MF_INTERESTING)
 
@@ -62,8 +78,10 @@ enum poly_power_type {
     PPT_MORE,
     PPT_SAME
 };
+
 bool monster_polymorph(monsters *monster, monster_type targetc,
-                       poly_power_type p = PPT_SAME);
+                       poly_power_type power = PPT_SAME,
+                       bool force_beh = false);
 
 // last updated: 08jun2000 {dlb}
 /* ***********************************************************************
@@ -80,25 +98,25 @@ void mons_check_pool(monsters *monster, killer_type killer = KILL_NONE,
 /* ***********************************************************************
    * called from: monstuff - fight
    * *********************************************************************** */
-void monster_cleanup(struct monsters *monster);
+void monster_cleanup(monsters *monster);
 
 
 /* ***********************************************************************
  * called from: monstuff beam effects fight view
  * *********************************************************************** */
-void behaviour_event( struct monsters *mon, int event_type, 
-                      int src = MHITNOT, int src_x = 0, int src_y = 0 ); 
+void behaviour_event(monsters *mon, int event_type, int src = MHITNOT,
+                     int src_x = 0, int src_y = 0);
 
 /* ***********************************************************************
  * called from: fight - it_use3 - spells
  * *********************************************************************** */
-bool curse_an_item(bool decay_potions);
+bool curse_an_item(bool decay_potions, bool quiet = false);
 
 
 /* ***********************************************************************
  * called from: fight
  * *********************************************************************** */
-bool monster_blink(struct monsters *monster);
+bool monster_blink(monsters *monster);
 
 
 /* ***********************************************************************
@@ -107,7 +125,8 @@ bool monster_blink(struct monsters *monster);
  * false, false
  * *********************************************************************** */
 bool random_near_space( int ox, int oy, int &tx, int &ty,
-    bool allow_adjacent = false, bool restrict_LOS = true);
+    bool allow_adjacent = false, bool restrict_LOS = true,
+    bool forbid_sanctuary = false);
 
 
 /* ***********************************************************************
@@ -119,22 +138,33 @@ bool simple_monster_message(const monsters *monster, const char *event,
                             int param = 0,
                             description_level_type descrip = DESC_CAP_THE);
 
+void make_mons_leave_level(monsters *mon);
+
+bool choose_any_monster(const monsters* mon);
+monsters *choose_random_nearby_monster(
+    int weight,
+    bool (*suitable)(const monsters* mon) =
+    choose_any_monster,
+    bool in_sight = true,
+    bool prefer_named = false);
+
+monsters *choose_random_monster_on_level(
+    int weight,
+    bool (*suitable)(const monsters* mon) =
+    choose_any_monster,
+    bool in_sight = true, bool near_by = false,
+    bool prefer_named = false);
+
 /* ***********************************************************************
  * called from: acr
  * *********************************************************************** */
-bool swap_places(struct monsters *monster);
+bool swap_places(monsters *monster);
 
 
 /* ***********************************************************************
  * called from: bang - beam - direct - fight - spells1 - spells2 - spells3
  * *********************************************************************** */
-void print_wounds(const struct monsters *monster);
-
-
-/* ***********************************************************************
- * called from: fight
- * *********************************************************************** */
-bool wounded_damaged(int wound_class);
+void print_wounds(const monsters *monster);
 
 
 /* ***********************************************************************
@@ -143,13 +173,6 @@ bool wounded_damaged(int wound_class);
 void handle_monsters(void);
 
 
-/* ***********************************************************************
- * called from: acr - bang - beam - direct - dungeon - fight - files -
- *              monplace - mstuff2 - spells3 - view
- * *********************************************************************** */
-dungeon_feature_type monster_habitat(int which_class);
-
-
 /* ***********************************************************************
  * called from: misc
  * *********************************************************************** */
@@ -161,32 +184,17 @@ bool monster_descriptor(int which_class, unsigned char which_descriptor);
  * *********************************************************************** */
 bool message_current_target(void);
 
-
-/* ***********************************************************************
- * called from: xxx
- * *********************************************************************** */
 unsigned int monster_index(const monsters *monster);
 
+void mons_get_damage_level(const monsters*, std::string& desc, mon_dam_level_type&);
 
-// last updated 08jun2000 {dlb}
-/* ***********************************************************************
- * called from: bang - beam - effects - fight - monstuff - mstuff2 -
- *              spells2 - spells3 - spells4
- * *********************************************************************** */
 int hurt_monster(monsters *victim, int damage_dealt);
 
+bool heal_monster(monsters *patient, int health_boost, bool permit_growth);
 
-/* ***********************************************************************
- * called from: beam - fight - files - monstuff - spells1
- * *********************************************************************** */
-bool heal_monster(struct monsters *patient, int health_boost, bool permit_growth);
+void seen_monster(monsters *monster);
 
-/* ***********************************************************************
- * called from: monplace - spells2 - view
- * *********************************************************************** */
-void seen_monster(struct monsters *monster);
-
-bool shift_monster( struct monsters *mon, int x = 0, int y = 0 );
+bool shift_monster(monsters *mon, int x = 0, int y = 0 );
 
 int mons_weapon_damage_rating(const item_def &launcher);
 int mons_pick_best_missile(monsters *mons, item_def **launcher,
@@ -195,4 +203,6 @@ int mons_missile_damage(const item_def *launch,
                         const item_def *missile);
 int mons_thrown_weapon_damage(const item_def *weap);
 
+int mons_natural_regen_rate(monsters *monster);
+
 #endif
