@@ -3,7 +3,7 @@
  *  Summary:    Functions for handling player mutations.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dolorous $ on $Date: 2008-05-03T19:43:15.071733Z $
  *
  *  Change History (most recent first):
  *
@@ -25,6 +25,8 @@ struct mutation_def
     bool          bad;       // A mutation that's more bad than good. Xom uses
                              // this to decide which mutations to hand out as
                              // rewards.
+    bool          physical;  // A mutation affecting a character's outward
+                             // appearance.
 };
 
 void fixup_mutations();
@@ -35,7 +37,8 @@ void fixup_mutations();
  *              mutation - religion - spell - spells
  * *********************************************************************** */
 bool mutate(mutation_type which_mutation, bool failMsg = true,
-            bool force_mutation = false, bool demonspawn = false);
+            bool force_mutation = false, bool god_gift = false,
+            bool demonspawn = false);
 
 // last updated 12may2000 {dlb}
 /* ***********************************************************************
@@ -43,6 +46,8 @@ bool mutate(mutation_type which_mutation, bool failMsg = true,
  * *********************************************************************** */
 void display_mutations();
 
+bool mutation_is_fully_active(mutation_type mut);
+
 formatted_string describe_mutations();
 
 
@@ -50,36 +55,30 @@ formatted_string describe_mutations();
 /* ***********************************************************************
  * called from: decks - it_use2 - mutation - spells
  * *********************************************************************** */
-bool delete_mutation(mutation_type which_mutation, bool force = false);
-
+bool delete_mutation(mutation_type which_mutation, bool failMsg = true,
+                     bool force_mutation = false);
 
 // last updated 12may2000 {dlb}
 /* ***********************************************************************
  * called from: chardump
  * *********************************************************************** */
 // default of level == -1, means to use the player's current level
-const char *mutation_name( mutation_type which_mutat, int level = -1 );
-
-
-// last updated 12may2000 {dlb}
-/* ***********************************************************************
- * called from: items - religion
- * *********************************************************************** */
-bool give_cosmetic_mutation( void );
+const char *mutation_name(mutation_type which_mutat, int level = -1);
 
 // last updated 12may2000 {dlb}
 /* ***********************************************************************
  * called from: items - spells
  * *********************************************************************** */
-bool give_bad_mutation( bool forceMutation = false, bool failMsg = true );
+bool give_bad_mutation(bool failMsg = true, bool force_mutation = false);
 
 // last updated 12may2000 {dlb}
 /* ***********************************************************************
  * called from: player
  * *********************************************************************** */
-void demonspawn(void);
+void demonspawn();
 
 bool perma_mutate(mutation_type which_mut, int how_much);
+int how_mutated(bool all = false, bool levels = false);
 
 #ifdef DEBUG_DIAGNOSTICS
 void sanity_check_mutation_defs();
