@@ -1,8 +1,10 @@
 /*
- *  File:       notes.cc
+ *  File:       notes.h
  *  Summary:    Notetaking stuff
  *  Written by: Haran Pilpel
  *
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-05-20T11:47:37.390524Z $
+ *
  *  Change History (most recent first):
  *
  *     <1>     -/--/--     PH     Created
@@ -15,6 +17,9 @@
 #include <vector>
 #include <stdio.h>
 
+class reader;
+class writer;
+
 enum NOTE_TYPES
 {
     NOTE_HP_CHANGE = 0,         /* needs: new hp, max hp */
@@ -30,7 +35,7 @@ enum NOTE_TYPES
     NOTE_GET_MUTATION,          /* needs: mutation idx */
     NOTE_LOSE_MUTATION,         /* needs: mutation idx */
     NOTE_ID_ITEM,               /* needs: item name (string) */
-    NOTE_GET_ITEM,              /* needs: item name (string) NOT HOOKED */
+    NOTE_GET_ITEM,              /* needs: item name (string) */
     NOTE_GAIN_SKILL,            /* needs: skill id, level */
     NOTE_SEEN_MONSTER,          /* needs: monster name (string) */
     NOTE_KILL_MONSTER,          /* needs: monster name (string) */
@@ -49,25 +54,26 @@ struct Note
     Note();
     Note( NOTE_TYPES t, int f = 0, int s = 0, const char* n = 0,
           const char* d = 0);
+    void save(writer& outf) const;
+    void load(reader& inf);
+    std::string describe( bool when = true, bool where = true,
+                          bool what = true ) const;
+    void check_milestone() const;
+
     NOTE_TYPES type;
     int first, second;
     long turn;
     unsigned short packed_place;
     std::string name;
     std::string desc;
-    void load( FILE* fp );
-    void save( FILE* fp ) const;
-    std::string describe( bool when = true, bool where = true,
-                          bool what = true ) const;
-    void check_milestone() const;
 };
 
 extern std::vector<Note> note_list;
 void activate_notes( bool active );
 bool notes_are_active();
 void take_note( const Note& note, bool force = false );
-void save_notes( FILE* fp );
-void load_notes( FILE* fp );
+void save_notes(writer&);
+void load_notes(reader&);
 void make_user_note();
 
 #endif
