@@ -3,7 +3,7 @@
  *  Summary:    Functions used when Bad Things happen to the player.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T14:58:46.756027Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-17T07:47:50.838270Z $
  *
  *  Change History (most recent first):
  *
@@ -56,6 +56,7 @@
 #include "chardump.h"
 #include "delay.h"
 #include "files.h"
+#include "fight.h"
 #include "hiscores.h"
 #include "invent.h"
 #include "itemname.h"
@@ -67,116 +68,98 @@
 #include "mon-util.h"
 #include "monstuff.h"
 #include "notes.h"
+#include "output.h"
 #include "player.h"
 #include "randart.h"
 #include "religion.h"
 #include "shopping.h"
 #include "skills2.h"
-#include "spells4.h"
 #include "state.h"
 #include "stuff.h"
 #include "tutorial.h"
 #include "view.h"
+#include "xom.h"
 
 
 static void end_game( scorefile_entry &se );
 static void item_corrode( int itco );
 
 
-/* NOTE: DOES NOT check for hellfire!!! */
-int check_your_resists(int hurted, int flavour)
+// NOTE: DOES NOT check for hellfire!!!
+int check_your_resists(int hurted, beam_type flavour)
 {
     int resist;
+    int original = hurted;
 
 #if DEBUG_DIAGNOSTICS
     mprf(MSGCH_DIAGNOSTICS, "checking resistance: flavour=%d", flavour );
 #endif
 
-    if (flavour == BEAM_FIRE || flavour == BEAM_LAVA 
+    if (flavour == BEAM_FIRE || flavour == BEAM_LAVA
         || flavour == BEAM_HELLFIRE || flavour == BEAM_FRAG)
     {
         if (you.duration[DUR_CONDENSATION_SHIELD] > 0)
         {
             mpr( "Your icy shield dissipates!", MSGCH_DURATION );
             you.duration[DUR_CONDENSATION_SHIELD] = 0;
-            you.redraw_armour_class = 1;
+            you.redraw_armour_class = true;
         }
     }
 
     switch (flavour)
     {
     case BEAM_STEAM:
-        resist = player_res_steam();
-        if (resist > 0)
-        {
+        hurted = resist_adjust_damage(&you, flavour,
+                                      player_res_steam(), hurted, true);
+        if (hurted < original)
             canned_msg(MSG_YOU_RESIST);
-            hurted /= (1 + (resist * resist));
-        }
-        else if (resist < 0)
-        {
-            // We could use a superior message.
-            mpr("It burns terribly!");
-            hurted = hurted * 15 / 10;
-        }
+        else if (hurted > original)
+            mpr("It scalds you terribly!");
         break;
 
     case BEAM_FIRE:
-        resist = player_res_fire();
-        if (resist > 0)
-        {
+        hurted = resist_adjust_damage(&you, flavour,
+                                      player_res_fire(), hurted, true);
+        if (hurted < original)
             canned_msg(MSG_YOU_RESIST);
-            hurted /= (1 + (resist * resist));
-        }
-        else if (resist < 0)
+        else if (hurted > original)
         {
             mpr("It burns terribly!");
             xom_is_stimulated(200);
-            hurted *= 15;
-            hurted /= 10;
         }
         break;
 
     case BEAM_COLD:
-        resist = player_res_cold();
-        if (resist > 0)
-        {
+        hurted = resist_adjust_damage(&you, flavour,
+                                      player_res_cold(), hurted, true);
+        if (hurted < original)
             canned_msg(MSG_YOU_RESIST);
-            hurted /= (1 + (resist * resist));
-        }
-        else if (resist < 0)
+        else if (hurted > original)
         {
             mpr("You feel a terrible chill!");
             xom_is_stimulated(200);
-            hurted *= 15;
-            hurted /= 10;
         }
         break;
 
     case BEAM_ELECTRICITY:
-        resist = player_res_electricity();
-        if (resist >= 3)
-        {
-            canned_msg(MSG_YOU_RESIST);
-            hurted = 0;
-        }
-        else if (resist > 0)
-        {
-            canned_msg(MSG_YOU_RESIST);
-            hurted /= 3;
-        }
-        break;
+        hurted = resist_adjust_damage(&you, flavour,
+                                      player_res_electricity(),
+                                      hurted, true);
 
+        if (hurted < original)
+            canned_msg(MSG_YOU_RESIST);
+        break;
 
     case BEAM_POISON:
         resist = player_res_poison();
 
-        if (!resist)
+        if (resist <= 0)
             poison_player( coinflip() ? 2 : 1 );
-        else
-        {
+
+        hurted = resist_adjust_damage(&you, flavour, resist,
+                                      hurted, true);
+        if (resist > 0)
             canned_msg(MSG_YOU_RESIST);
-            hurted /= 3;
-        }
         break;
 
     case BEAM_POISON_ARROW:
@@ -186,86 +169,93 @@ int check_your_resists(int hurted, int flavour)
         resist = player_res_poison();
 
         if (!resist)
-            poison_player( 6 + random2(3), true );
-        else
-        {
-            mpr("You partially resist.");
-            hurted /= 2;
+            poison_player( 4 + random2(3), true );
+        else if (!you.is_undead)
+            poison_player( 2 + random2(3), true );
 
-            if (!you.is_undead)
-                poison_player( 2 + random2(3), true ); 
-        }
+        hurted = resist_adjust_damage(&you, flavour, resist, hurted);
+        if (hurted < original)
+            canned_msg(MSG_YOU_PARTIALLY_RESIST);
         break;
 
     case BEAM_NEG:
         resist = player_prot_life();
 
-        if (resist > 0)
+        // TSO's protection.
+        if (you.religion == GOD_SHINING_ONE && you.piety > resist * 50)
         {
-            hurted -= (resist * hurted) / 3;
+            int unhurted = (you.piety * hurted) / 150;
+
+            if (unhurted > hurted)
+                unhurted = hurted;
+
+            if (unhurted > 0)
+                hurted -= unhurted;
         }
+        else if (resist > 0)
+            hurted -= (resist * hurted) / 3;
 
         drain_exp();
         break;
 
     case BEAM_ICE:
-        resist = player_res_cold();
+        hurted = resist_adjust_damage(&you, flavour, player_res_cold(),
+                                      hurted, true);
 
-        if (resist > 0)
-        {
-            mpr("You partially resist.");
-            hurted /= 2;
-        }
-        else if (resist < 0)
+        if (hurted < original)
+            canned_msg(MSG_YOU_PARTIALLY_RESIST);
+        else if (hurted > original)
         {
             mpr("You feel a painful chill!");
             xom_is_stimulated(200);
-            hurted *= 13;
-            hurted /= 10;
         }
         break;
 
     case BEAM_LAVA:
-        resist = player_res_fire();
+        hurted = resist_adjust_damage(&you, flavour, player_res_fire(),
+                                      hurted, true);
 
-        if (resist > 1)
-        {
-            mpr("You partially resist.");
-            hurted /= (1 + resist);
-        }
-        else if (resist < 0)
+        if (hurted < original)
+            canned_msg(MSG_YOU_PARTIALLY_RESIST);
+        else if (hurted > original)
         {
             mpr("It burns terribly!");
             xom_is_stimulated(200);
-            hurted *= 15;
-            hurted /= 10;
         }
         break;
 
     case BEAM_ACID:
         if (player_res_acid())
         {
-            canned_msg( MSG_YOU_RESIST );
+            canned_msg(MSG_YOU_RESIST);
             hurted = hurted * player_acid_resist_factor() / 100;
         }
         break;
 
     case BEAM_MIASMA:
-        if (player_prot_life() > random2(3))
+        if (x_chance_in_y(player_prot_life(), 3))
         {
-            canned_msg( MSG_YOU_RESIST );
+            canned_msg(MSG_YOU_RESIST);
             hurted = 0;
         }
         break;
 
     case BEAM_HOLY:
-        if (!you.is_undead && you.species != SP_DEMONSPAWN)
-        {
-            canned_msg( MSG_YOU_RESIST );
+        if (is_good_god(you.religion))
             hurted = 0;
-        }
+        else if (you.is_undead || you.species != SP_DEMONSPAWN)
+            hurted = (hurted * 3) / 2;
+        else if (!is_evil_god(you.religion))
+            hurted /= 2;
+
+        if (hurted == 0)
+            canned_msg(MSG_YOU_RESIST);
+
         break;
-    }                           /* end switch */
+
+    default:
+        break;
+    }                           // end switch
 
     return (hurted);
 }                               // end check_your_resists()
@@ -275,7 +265,7 @@ void splash_with_acid( char acid_strength )
     char splc = 0;
     int  dam = 0;
 
-    const bool wearing_cloak = (you.equip[EQ_CLOAK] != -1);     
+    const bool wearing_cloak = (you.equip[EQ_CLOAK] != -1);
 
     for (splc = EQ_CLOAK; splc <= EQ_BODY_ARMOUR; splc++)
     {
@@ -287,7 +277,7 @@ void splash_with_acid( char acid_strength )
             continue;
         }
 
-        if (random2(20) <= acid_strength)
+        if (x_chance_in_y(acid_strength + 1, 20))
             item_corrode( you.equip[splc] );
     }
 
@@ -317,23 +307,21 @@ void weapon_acid( char acid_strength )
     if (hand_thing == -1)
     {
         msg::stream << "Your " << your_hand(true) << " burn!" << std::endl;
-        ouch( roll_dice( 1, acid_strength ), 0, KILLED_BY_ACID );  
+        ouch( roll_dice( 1, acid_strength ), 0, KILLED_BY_ACID );
     }
-    else if (random2(20) <= acid_strength)
-    {
+    else if (x_chance_in_y(acid_strength + 1, 20))
         item_corrode( hand_thing );
-    }
-}                               // end weapon_acid()
+}
 
 void item_corrode( int itco )
 {
     int chance_corr = 0;        // no idea what its full range is {dlb}
     bool it_resists = false;    // code simplifier {dlb}
     bool suppress_msg = false;  // code simplifier {dlb}
-    int how_rusty = ((you.inv[itco].base_type == OBJ_WEAPONS) 
-                                ? you.inv[itco].plus2 : you.inv[itco].plus);
+    int how_rusty = ((you.inv[itco].base_type == OBJ_WEAPONS)
+                            ? you.inv[itco].plus2 : you.inv[itco].plus);
 
-    // early return for "oRC and cloak/preservation {dlb}:
+    // Early return for "oRC and cloak/preservation {dlb}.
     if (wearing_amulet(AMU_RESIST_CORROSION) && !one_chance_in(10))
     {
 #if DEBUG_DIAGNOSTICS
@@ -366,7 +354,7 @@ void item_corrode( int itco )
         break;
 
     case OBJ_WEAPONS:
-        if (is_fixed_artefact(item) 
+        if (is_fixed_artefact(item)
             || is_random_artefact(item))
         {
             it_resists = true;
@@ -381,7 +369,7 @@ void item_corrode( int itco )
         break;
 
     case OBJ_MISSILES:
-        if (get_equip_race(item) == ISFLAG_DWARVEN 
+        if (get_equip_race(item) == ISFLAG_DWARVEN
                 && !one_chance_in(5))
         {
             it_resists = true;
@@ -389,8 +377,8 @@ void item_corrode( int itco )
         }
         break;
     default:
-        /* items which aren't missiles, etc...Could happen
-           if we're wielding a deck, say */
+        // items which aren't missiles, etc... could happen if we're
+        // e.g. wielding a deck.
         return;
     }
 
@@ -404,25 +392,27 @@ void item_corrode( int itco )
 
         // the embedded equation may look funny, but it actually works well
         // to generate a pretty probability ramp {6%, 18%, 34%, 58%, 98%}
-        // for values [0,4] which closely matches the original, ugly switch
+        // for values [0,4] which closely matches the original, ugly switch.
         // {dlb}
         if (chance_corr >= 0 && chance_corr <= 4)
         {
-            it_resists = (random2(100) <
-                            2 + (4 << chance_corr) + (chance_corr * 8));
+            it_resists
+               = x_chance_in_y(2 + (4 << chance_corr) + chance_corr * 8, 100);
         }
         else
             it_resists = true;  // no idea how often this occurs {dlb}
 
-        // if the checks get this far, you should hear about it {dlb}
+        // If the checks get this far, you should hear about it. {dlb}
         suppress_msg = false;
     }
 
     // handle message output and item damage {dlb}:
     if (!suppress_msg)
     {
-        mprf("%s %s", item.name(DESC_CAP_YOUR).c_str(),
-             (it_resists) ? "resists." : "is eaten away!");
+        if (it_resists)
+            mprf("%s resists.", item.name(DESC_CAP_YOUR).c_str());
+        else
+            mprf("The acid corrodes %s!", item.name(DESC_NOCAP_YOUR).c_str());
     }
 
     if (!it_resists)
@@ -433,9 +423,9 @@ void item_corrode( int itco )
         if (item.base_type == OBJ_WEAPONS)
             item.plus2 = how_rusty;
         else
-            item.plus = how_rusty;
+            item.plus  = how_rusty;
 
-        you.redraw_armour_class = 1;     // for armour, rings, etc. {dlb}
+        you.redraw_armour_class = true;     // for armour, rings, etc. {dlb}
 
         if (you.equip[EQ_WEAPON] == itco)
             you.wield_change = true;
@@ -446,7 +436,7 @@ void item_corrode( int itco )
 
 // Helper function for the expose functions below.
 // This currently works because elements only target a single type each.
-static int get_target_class( beam_type flavour )
+static int _get_target_class(beam_type flavour)
 {
     int target_class = OBJ_UNASSIGNED;
 
@@ -478,42 +468,41 @@ static int get_target_class( beam_type flavour )
 // XXX: These expose functions could use being reworked into a real system...
 // the usage and implementation is currently very hacky.
 // Handles the destruction of inventory items from the elements.
-static void expose_invent_to_element( beam_type flavour, int strength )
+static void _expose_invent_to_element(beam_type flavour, int strength)
 {
-    int i, j;
     int num_dest = 0;
 
-    const int target_class = get_target_class( flavour );
+    const int target_class = _get_target_class( flavour );
     if (target_class == OBJ_UNASSIGNED)
         return;
 
     // Currently we test against each stack (and item in the stack)
-    // independantly at strength%... perhaps we don't want that either
+    // independently at strength%... perhaps we don't want that either
     // because it makes the system very fair and removes the protection
     // factor of junk (which might be more desirable for game play).
-    for (i = 0; i < ENDOFPACK; i++)
+    for (int i = 0; i < ENDOFPACK; ++i)
     {
-        if (!is_valid_item( you.inv[i] ))
+        if (!is_valid_item(you.inv[i]))
             continue;
 
-        if (is_valid_item( you.inv[i] )
+        if (is_valid_item(you.inv[i])
             && (you.inv[i].base_type == target_class
-                || (target_class == OBJ_FOOD 
-                    && you.inv[i].base_type == OBJ_CORPSES)))
+                || target_class == OBJ_FOOD
+                   && you.inv[i].base_type == OBJ_CORPSES))
         {
             if (player_item_conserve() && !one_chance_in(10))
                 continue;
 
-            for (j = 0; j < you.inv[i].quantity; j++)
+            for (int j = 0; j < you.inv[i].quantity; ++j)
             {
-                if (random2(100) < strength)
+                if (x_chance_in_y(strength, 100))
                 {
                     num_dest++;
 
                     if (i == you.equip[EQ_WEAPON])
                         you.wield_change = true;
 
-                    if (dec_inv_item_quantity( i, 1 ))
+                    if (dec_inv_item_quantity(i, 1))
                         break;
                 }
             }
@@ -538,7 +527,7 @@ static void expose_invent_to_element( beam_type flavour, int strength )
             break;
 
         case OBJ_FOOD:
-            mprf("Some of your food is covered with spores!");
+            mpr("Some of your food is covered with spores!");
             break;
 
         default:
@@ -552,23 +541,80 @@ static void expose_invent_to_element( beam_type flavour, int strength )
     }
 }
 
+void expose_items_to_element(beam_type flavour, int x, int y, int strength)
+{
+    int num_dest = 0;
 
-// Handle side-effects for exposure to element other than damage.
-// This function exists because some code calculates its own damage 
-// instead of using check_resists and we want to isolate all the special
-// code they keep having to do... namely condensation shield checks, 
-// you really can't expect this function to even be called for much else.
+    const int target_class = _get_target_class(flavour);
+    if (target_class == OBJ_UNASSIGNED)
+        return;
+
+    for (stack_iterator si(coord_def(x,y)); si; ++si)
+    {
+        if (!is_valid_item(*si))
+            continue;
+
+        if (si->base_type == target_class
+            || target_class == OBJ_FOOD && si->base_type == OBJ_CORPSES)
+        {
+            if (x_chance_in_y(strength, 100))
+            {
+                num_dest++;
+                dec_mitm_item_quantity(si->index(), 1);
+            }
+        }
+    }
+
+    if (num_dest)
+    {
+        if (see_grid(x, y))
+        {
+            switch (target_class)
+            {
+            case OBJ_SCROLLS:
+                mprf("You see %s of smoke.",
+                     (num_dest > 1) ? "some puffs" : "a puff");
+                break;
+
+            case OBJ_POTIONS:
+                mprf("You see %s shatter.",
+                     (num_dest > 1) ? "some glass" : "glass");
+                break;
+
+            case OBJ_FOOD:
+                mprf("You see %s of spores.",
+                     (num_dest > 1) ? "some clouds" : "a cloud");
+                break;
+
+            default:
+                mprf("%s on the floor %s destroyed!",
+                     (num_dest > 1) ? "Some items" : "An item",
+                     (num_dest > 1) ? "were" : "was" );
+                break;
+            }
+        }
+
+        xom_is_stimulated((num_dest > 1) ? 32 : 16);
+    }
+}
+
+// Handle side-effects for exposure to element other than damage.  This
+// function exists because some code calculates its own damage instead
+// of using check_your_resists() and we want to isolate all the special
+// code they keep having to do... namely condensation shield checks,
+// you really can't expect this function to even be called for much
+// else.
 //
-// This function now calls expose_invent_to_element if strength > 0.
+// This function now calls _expose_invent_to_element() if strength > 0.
 //
-// XXX: this function is far from perfect and a work in progress.
-void expose_player_to_element( beam_type flavour, int strength )
-{ 
-    // Note that BEAM_TELEPORT is sent here when the player 
+// XXX: This function is far from perfect and a work in progress.
+void expose_player_to_element(beam_type flavour, int strength)
+{
+    // Note that BEAM_TELEPORT is sent here when the player
     // blinks or teleports.
-    if (flavour == BEAM_FIRE || flavour == BEAM_LAVA 
-        || flavour == BEAM_HELLFIRE || flavour == BEAM_FRAG 
-        || flavour == BEAM_TELEPORT || flavour == BEAM_NAPALM 
+    if (flavour == BEAM_FIRE || flavour == BEAM_LAVA
+        || flavour == BEAM_HELLFIRE || flavour == BEAM_FRAG
+        || flavour == BEAM_TELEPORT || flavour == BEAM_NAPALM
         || flavour == BEAM_STEAM)
     {
         if (you.duration[DUR_CONDENSATION_SHIELD] > 0)
@@ -580,21 +626,25 @@ void expose_player_to_element( beam_type flavour, int strength )
     }
 
     if (strength)
-        expose_invent_to_element( flavour, strength );
+        _expose_invent_to_element( flavour, strength );
 }
 
 void lose_level()
 {
-    // because you.experience is unsigned long, if it's going to be -ve
+    // Because you.experience is unsigned long, if it's going to be -ve
     // must die straightaway.
     if (you.experience_level == 1)
-        ouch( -9999, 0, KILLED_BY_DRAINING );
+    {
+        ouch(INSTANT_DEATH, 0, KILLED_BY_DRAINING);
+        // Return in case death was canceled via wizard mode
+        return;
+    }
 
     you.experience = exp_needed( you.experience_level + 1 ) - 1;
     you.experience_level--;
 
     mprf(MSGCH_WARN,
-         "You are now a level %d %s!", you.experience_level, you.class_name);
+         "You are now level %d!", you.experience_level);
 
     // Constant value to avoid grape jelly trick... see level_change() for
     // where these HPs and MPs are given back.  -- bwr
@@ -612,31 +662,29 @@ void lose_level()
             you.hp, you.hp_max, you.magic_points, you.max_magic_points);
     take_note(Note(NOTE_XP_LEVEL_CHANGE, you.experience_level, 0, buf));
 
-    you.redraw_experience = 1;
+    redraw_skill(you.your_name, player_title());
+    you.redraw_experience = true;
+
     xom_is_stimulated(255);
-}                               // end lose_level()
+}
 
 void drain_exp(bool announce_full)
 {
-    int protection = player_prot_life();
+    const int protection = player_prot_life();
 
-    if (you.duration[DUR_PRAYER]
-        && (you.religion == GOD_ZIN || you.religion == GOD_SHINING_ONE)
-        && random2(150) < you.piety)
-    {
-        simple_god_message(" protects your life force!");
-        return;
-    }
-
-    if (protection >= 3 || you.is_undead)
+    if (protection == 3)
     {
         if ( announce_full )
-            mpr("You fully resist.");
+            canned_msg(MSG_YOU_RESIST);
         return;
     }
 
     if (you.experience == 0)
-        ouch(-9999, 0, KILLED_BY_DRAINING);
+    {
+        ouch(INSTANT_DEATH, 0, KILLED_BY_DRAINING);
+        // Return in case death was escaped via wizard mode.
+        return;
+    }
 
     if (you.experience_level == 1)
     {
@@ -646,13 +694,25 @@ void drain_exp(bool announce_full)
 
     unsigned long total_exp = exp_needed( you.experience_level + 2 )
                                     - exp_needed( you.experience_level + 1 );
-    unsigned long exp_drained = total_exp * (10 + random2(11));
+    unsigned long exp_drained = (total_exp * (10 + random2(11))) / 100;
 
-    exp_drained /= 100;
-
-    if (protection > 0)
+    // TSO's protection.
+    if (you.religion == GOD_SHINING_ONE && you.piety > protection * 50)
     {
-        mpr("You partially resist.");
+        unsigned long undrained = (you.piety * exp_drained) / 150;
+
+        if (undrained > exp_drained)
+            undrained = exp_drained;
+
+        if (undrained > 0)
+        {
+            simple_god_message(" protects your life force!");
+            exp_drained -= undrained;
+        }
+    }
+    else if (protection > 0)
+    {
+        canned_msg(MSG_YOU_PARTIALLY_RESIST);
         exp_drained -= (protection * exp_drained) / 3;
     }
 
@@ -663,32 +723,40 @@ void drain_exp(bool announce_full)
         you.experience -= exp_drained;
         you.exp_available -= exp_drained;
 
-        if (you.exp_available < 0)
-            you.exp_available = 0;
+        you.exp_available = std::max(0, you.exp_available);
 
 #if DEBUG_DIAGNOSTICS
         mprf(MSGCH_DIAGNOSTICS, "You lose %ld experience points.",exp_drained);
 #endif
 
-        you.redraw_experience = 1;
+        you.redraw_experience = true;
 
         if (you.experience < exp_needed(you.experience_level + 1))
             lose_level();
     }
-}                               // end drain_exp()
+}
 
-static void xom_checks_damage(kill_method_type death_type,
-                              int dam, int death_source)
+static void _xom_checks_damage(kill_method_type death_type,
+                               int dam, int death_source)
 {
-    //if (you.hp <= dam)
-    //    xom_is_stimulated(32);
-
-    if ((death_type != KILLED_BY_MONSTER && death_type != KILLED_BY_BEAM)
-        || death_source < 0 || death_source >= MAX_MONSTERS)
+    if (death_type == KILLED_BY_TARGETTING)
     {
-        return ;
+        // Xom thinks the player hurting him/herself is funny.
+        xom_is_stimulated(255 * dam / (dam + you.hp));
+        return;
     }
-    
+    else if (death_type == KILLED_BY_FALLING_DOWN_STAIRS)
+    {
+        // Xom thinks falling down the stairs is hilarious.
+        xom_is_stimulated(255);
+        return;
+    }
+    else if (death_type != KILLED_BY_MONSTER && death_type != KILLED_BY_BEAM
+             || death_source < 0 || death_source >= MAX_MONSTERS)
+    {
+        return;
+    }
+
     int amusementvalue = 1;
 
     const monsters *monster = &menv[death_source];
@@ -696,22 +764,29 @@ static void xom_checks_damage(kill_method_type death_type,
     if (!monster->alive())
         return;
 
+    if (mons_wont_attack(monster))
+    {
+        // Xom thinks collateral damage is funny.
+        xom_is_stimulated(255 * dam / (dam + you.hp));
+        return;
+    }
+
     int leveldif = monster->hit_dice - you.experience_level;
 
     if (leveldif == 0)
         leveldif = 1;
 
-    /* Note that Xom is amused when you are significantly hurt
-     * by a creature of higher level than yourself as well as
-     * by a creatured of lower level than yourself. */
+    // Note that Xom is amused when you are significantly hurt by a
+    // creature of higher level than yourself, as well as by a creature
+    // of lower level than yourself.
     amusementvalue += leveldif * leveldif * dam;
-    
+
     if (!player_monster_visible(monster))
         amusementvalue += 10;
-    
+
     if (monster->speed < (int) player_movement_speed())
         amusementvalue += 8;
-    
+
     if (death_type != KILLED_BY_BEAM)
     {
         if (you.skills[SK_THROWING] <= (you.experience_level / 4))
@@ -725,19 +800,22 @@ static void xom_checks_damage(kill_method_type death_type,
 
     if (player_in_a_dangerous_place())
         amusementvalue += 2;
-            
+
     amusementvalue /= (you.hp > 0) ? you.hp : 1;
-  
+
     xom_is_stimulated(amusementvalue);
 }
 
-// death_source should be set to zero for non-monsters {dlb}
+// death_source should be set to zero for non-monsters. {dlb}
 void ouch( int dam, int death_source, kill_method_type death_type,
            const char *aux, bool see_source )
 {
     ait_hp_loss hpl(dam, death_type);
     interrupt_activity( AI_HP_LOSS, &hpl );
 
+    if (dam > 0)
+        you.check_awaken(500);
+
     if (you.duration[DUR_DEATHS_DOOR] && death_type != KILLED_BY_LAVA
         && death_type != KILLED_BY_WATER)
     {
@@ -746,24 +824,12 @@ void ouch( int dam, int death_source, kill_method_type death_type,
 
     if (dam > -9000)            // that is, a "death" caused by hp loss {dlb}
     {
-        switch (you.religion)
+        if (dam >= you.hp && god_protects_from_harm(you.religion))
         {
-        case GOD_ZIN:
-        case GOD_SHINING_ONE:
-        case GOD_ELYVILON:
-        case GOD_YREDELEMNUL:
-            if (dam >= you.hp && you.duration[DUR_PRAYER]
-                && random2(you.piety) >= 30)
-            {
-                simple_god_message( " protects you from harm!" );
-                return;
-            }
-            break;
-        default:
-            break;
+            simple_god_message( " protects you from harm!" );
+            return;
         }
 
-
         dec_hp( dam, true );
 
         // Even if we have low HP messages off, we'll still give a
@@ -779,7 +845,7 @@ void ouch( int dam, int death_source, kill_method_type death_type,
                 mpr( "* * * LOW HITPOINT WARNING * * *", MSGCH_DANGER );
             }
 
-            xom_checks_damage(death_type, dam, death_source);
+            _xom_checks_damage(death_type, dam, death_source);
 
             // for note taking
             std::string damage_desc = "";
@@ -797,18 +863,23 @@ void ouch( int dam, int death_source, kill_method_type death_type,
 
             take_note(
                 Note(NOTE_HP_CHANGE, you.hp, you.hp_max, damage_desc.c_str()) );
-                
+
             return;
         } // else hp <= 0
     }
 
+    // Construct scorefile entry.
+    scorefile_entry se(dam, death_source, death_type, aux);
+
 #ifdef WIZARD
-    if (death_type != KILLED_BY_QUITTING 
+    if (death_type != KILLED_BY_QUITTING
         && death_type != KILLED_BY_WINNING
         && death_type != KILLED_BY_LEAVING)
     {
         if (you.wizard)
         {
+            const std::string death_desc
+                = se.death_description(scorefile_entry::DDV_VERBOSE);
 #ifdef USE_OPTIONAL_WIZARD_DEATH
 
 #if DEBUG_DIAGNOSTICS
@@ -817,6 +888,8 @@ void ouch( int dam, int death_source, kill_method_type death_type,
 
             if (!yesno("Die?", false, 'n'))
             {
+                take_note(Note( NOTE_DEATH, you.hp, you.hp_max,
+                                death_desc.c_str()), true);
                 set_hp(you.hp_max, false);
                 return;
             }
@@ -824,6 +897,8 @@ void ouch( int dam, int death_source, kill_method_type death_type,
             mpr("Since you're a debugger, I'll let you live.");
             mpr("Be more careful next time, okay?");
 
+            take_note(Note( NOTE_DEATH, you.hp, you.hp_max,
+                            death_desc.c_str()), true);
             set_hp(you.hp_max, false);
             return;
 #endif  // USE_OPTIONAL_WIZARD_DEATH
@@ -831,30 +906,25 @@ void ouch( int dam, int death_source, kill_method_type death_type,
     }
 #endif  // WIZARD
 
-    //okay, so you're dead:
-
+    // Okay, so you're dead.
     crawl_state.need_save       = false;
     crawl_state.updating_scores = true;
 
-    take_note(
-         Note( NOTE_DEATH, you.hp, you.hp_max,
-           scorefile_entry(dam, death_source, death_type, aux)
-           .death_description(scorefile_entry::DDV_NORMAL).c_str()), true );
+    take_note(Note( NOTE_DEATH, you.hp, you.hp_max,
+                    se.death_description(scorefile_entry::DDV_NORMAL).c_str()),
+              true);
 
-    // prevent bogus notes
+    // Prevent bogus notes.
     activate_notes(false);
 
-    // CONSTRUCT SCOREFILE ENTRY
-    scorefile_entry se(dam, death_source, death_type, aux);
-
 #ifdef SCORE_WIZARD_CHARACTERS
-    // add this highscore to the score file.
+    // Add this highscore to the score file.
     hiscores_new_entry(se);
     logfile_new_entry(se);
 #else
 
-    // only add non-wizards to the score file.
-    // never generate bones files of wizard characters -- bwr
+    // Only add non-wizards to the score file.
+    // Never generate bones files of wizard characters -- bwr
     if (!you.wizard)
     {
         hiscores_new_entry(se);
@@ -867,7 +937,6 @@ void ouch( int dam, int death_source, kill_method_type death_type,
             save_ghost();
         }
     }
-
 #endif
 
     end_game(se);
@@ -896,11 +965,29 @@ static std::string morgue_name(time_t when_crawl_got_even)
 #endif // SHORT_FILE_NAMES
 }
 
-void end_game( struct scorefile_entry &se )
+void end_game( scorefile_entry &se )
 {
-    int i;
     bool dead = true;
 
+    for (int i = 0; i < ENDOFPACK; i++)
+        set_ident_flags( you.inv[i], ISFLAG_IDENT_MASK );
+
+    for (int i = 0; i < ENDOFPACK; i++)
+    {
+        if (you.inv[i].base_type != 0)
+        {
+            set_ident_type( you.inv[i], ID_KNOWN_TYPE );
+        }
+    }
+
+    if (!dump_char( morgue_name(se.death_time), !dead, true, &se ))
+    {
+        mpr("Char dump unsuccessful! Sorry about that.");
+        if (!crawl_state.seen_hups)
+            more();
+        clrscr();
+    }
+
     if (se.death_type == KILLED_BY_LEAVING  ||
         se.death_type == KILLED_BY_QUITTING ||
         se.death_type == KILLED_BY_WINNING)
@@ -948,7 +1035,8 @@ void end_game( struct scorefile_entry &se )
 
     const int num_suffixes = sizeof(suffixes) / sizeof(const char*);
 
-    for ( i = 0; i < num_suffixes; ++i ) {
+    for (int i = 0; i < num_suffixes; ++i)
+    {
         std::string tmpname = basename + suffixes[i];
         unlink( tmpname.c_str() );
     }
@@ -973,29 +1061,10 @@ void end_game( struct scorefile_entry &se )
     if (!crawl_state.seen_hups)
         more();
 
-    for (i = 0; i < ENDOFPACK; i++)
-        set_ident_flags( you.inv[i], ISFLAG_IDENT_MASK );
-
-    for (i = 0; i < ENDOFPACK; i++)
-    {
-        if (you.inv[i].base_type != 0)
-        {
-            set_ident_type( you.inv[i].base_type, 
-                            you.inv[i].sub_type, ID_KNOWN_TYPE );
-        }
-    }
-
     invent( -1, true );
+    textcolor( LIGHTGREY );
     clrscr();
 
-    if (!dump_char( morgue_name(se.death_time), !dead, true, &se ))
-    {
-        mpr("Char dump unsuccessful! Sorry about that.");
-        if (!crawl_state.seen_hups)
-            more();
-        clrscr();
-    }
-
     clrscr();
     cprintf( "Goodbye, %s.", you.your_name );
     cprintf( EOL EOL "    " ); // Space padding where # would go in list format
