@@ -1,8 +1,10 @@
 /*
- *  File:       ouch.cc
+ *  File:       ouch.h
  *  Summary:    Functions used when Bad Things happen to the player.
  *  Written by: Linley Henzell
  *
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-17T07:47:50.838270Z $
+ *
  *  Change History (most recent first):
  *
  *     <2>     9/29/99     BCR     Added the DEATH_NAME_LENGTH define
@@ -49,13 +51,14 @@ enum kill_method_type
     KILLED_BY_SPORE,
     KILLED_BY_TSO_SMITING,
     KILLED_BY_PETRIFICATION,           // 25
+    // 26
     KILLED_BY_SOMETHING = 27,
     KILLED_BY_FALLING_DOWN_STAIRS,
     KILLED_BY_ACID,
-    KILLED_BY_CURARE,                  
+    KILLED_BY_CURARE,                  // 30
     KILLED_BY_MELTING,
     KILLED_BY_BLEEDING,
-    KILLED_BY_BEOGH_SMITING,
+    KILLED_BY_BEOGH_SMITING,           // 33
 
     NUM_KILLBY
 };
@@ -65,7 +68,7 @@ enum kill_method_type
 /* ***********************************************************************
  * called from: bang - beam - effects - spells
  * *********************************************************************** */
-int check_your_resists(int hurted, int flavour);
+int check_your_resists(int hurted, beam_type flavour);
 
 
 // last updated 12may2000 {dlb}
@@ -82,14 +85,6 @@ void splash_with_acid(char acid_strength);
 void weapon_acid(char acid_strength);
 
 
-// last updated 12may2000 {dlb}
-/* ***********************************************************************
- * called from: acr - bang - beam - effects - fight - misc - spells -
- *              spells2
- * *********************************************************************** */
-void scrolls_burn(char burn_strength, char target_class);
-
-
 // last updated 12may2000 {dlb}
 /* ***********************************************************************
  * called from: acr - bang - beam - command - effects - fight - misc -
@@ -112,6 +107,7 @@ void lose_level(void);
  * *********************************************************************** */
 void drain_exp(bool announce_full = true);
 
-void expose_player_to_element( beam_type flavour, int strength = 0 );
+void expose_items_to_element(beam_type flavour, int x, int y, int strength = 0);
+void expose_player_to_element(beam_type flavour, int strength = 0);
 
 #endif
