@@ -6,7 +6,7 @@
  *  Summary:    Records location of stairs etc
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-13T20:06:21.893954Z $
  *
  *  Change History (most recent first):
  *
@@ -22,12 +22,14 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <algorithm>
 
 #include "externs.h"
 
 #include "branch.h"
+#include "cio.h"
 #include "dgnevent.h"
-#include "direct.h"
+#include "directn.h"
 #include "dungeon.h"
 #include "files.h"
 #include "menu.h"
@@ -43,11 +45,13 @@ typedef std::map<branch_type, level_id> stair_map_type;
 typedef std::map<level_pos, shop_type> shop_map_type;
 typedef std::map<level_pos, god_type> altar_map_type;
 typedef std::map<level_pos, portal_type> portal_map_type;
+typedef std::map<level_id, std::string> annotation_map_type;
 
 stair_map_type stair_level;
 shop_map_type shops_present;
 altar_map_type altars_present;
 portal_map_type portals_present;
+annotation_map_type level_annotations;
 
 static void seen_altar( god_type god, const coord_def& pos );
 static void seen_staircase(dungeon_feature_type which_staircase,
@@ -59,11 +63,11 @@ void seen_notable_thing( dungeon_feature_type which_thing, int x, int y )
 {
     // Tell the world first.
     dungeon_events.fire_position_event(DET_PLAYER_IN_LOS, coord_def(x, y));
-    
+
     // Don't record in temporary terrain
     if (you.level_type != LEVEL_DUNGEON)
         return;
-    
+
     const coord_def pos(x, y);
     const god_type god = grid_altar_god(which_thing);
     if (god != GOD_NO_GOD)
@@ -147,7 +151,7 @@ static altar_map_type get_notable_altars(const altar_map_type &altars)
     for ( altar_map_type::const_iterator na_iter = altars.begin();
           na_iter != altars.end(); ++na_iter )
     {
-        if (na_iter->first.id.branch != BRANCH_ECUMENICAL_TEMPLE)
+        if (na_iter->first.id != BRANCH_ECUMENICAL_TEMPLE)
             notable_altars[na_iter->first] = na_iter->second;
     }
     return (notable_altars);
@@ -198,11 +202,11 @@ std::string overview_description_string()
             {
                 disp += "\n<green>Branches:</green>";
                 if (crawl_state.need_save || !crawl_state.updating_scores)
-                    disp += " (use <white>Ctrl-G</white> to reach them)";
+                    disp += " (use <white>G</white> to reach them)";
                 disp += EOL;
                 seen_anything = true;
             }
-            
+
             ++branchcount;
 
             snprintf(buffer, sizeof buffer, "<yellow>%-6s</yellow>: %-7s",
@@ -221,7 +225,7 @@ std::string overview_description_string()
     // remove unworthy altars from the list we show the user. Yeah,
     // one more round of map iteration.
     const altar_map_type notable_altars = get_notable_altars(altars_present);
-    
+
     // print altars
     // we loop through everything a dozen times, oh well
     if ( !notable_altars.empty() )
@@ -234,7 +238,7 @@ std::string overview_description_string()
     }
 
     level_id last_id;
-    std::map<level_pos, god_type>::const_iterator ci_altar;    
+    std::map<level_pos, god_type>::const_iterator ci_altar;
     for ( int cur_god = GOD_NO_GOD; cur_god < NUM_GODS; ++cur_god )
     {
         last_id.depth = 10000;  // fake depth to be sure we don't match
@@ -322,7 +326,8 @@ std::string overview_description_string()
         disp += shoptype_to_string(ci_shops->second);
         ++column_count;
     }
-    disp += "\n";
+    if (!shops_present.empty())
+        disp += "\n";
 
     // print portals
     if ( !portals_present.empty() )
@@ -368,7 +373,62 @@ std::string overview_description_string()
             disp += "You didn't discover anything interesting.";
     }
 
-    return disp;
+    bool notes_exist = false;
+    bool has_notes[NUM_BRANCHES];
+
+    for (int i = 0; i < NUM_BRANCHES; ++i)
+    {
+        Branch branch = branches[i];
+
+        has_notes[i] = false;
+        for (int depth = 1; depth <= branch.depth; depth++)
+        {
+            const level_id li(branch.id, depth);
+
+            if (get_level_annotation(li).length() > 0)
+            {
+                notes_exist  = true;
+                has_notes[i] = true;
+                break;
+            }
+        }
+    }
+
+    if (notes_exist)
+    {
+        disp += "\n\n                    <white>Level Annotations</white>\n" ;
+
+        for (int i = 0; i < NUM_BRANCHES; ++i)
+        {
+            if (!has_notes[i])
+                continue;
+
+            Branch branch = branches[i];
+
+            disp += "\n<yellow>";
+            disp += branch.shortname;
+            disp += "</yellow>\n";
+
+            for (int depth = 1; depth <= branch.depth; depth++)
+            {
+                const level_id li(branch.id, depth);
+
+                if (get_level_annotation(li).length() > 0)
+                {
+                    char depth_str[3];
+                    sprintf(depth_str, "%2d", depth);
+
+                    disp += "<white>";
+                    disp += depth_str;
+                    disp += ":</white> ";
+                    disp += get_level_annotation(li);
+                    disp += "\n";
+                }
+            }
+        }
+    }
+
+    return disp.substr(0, disp.find_last_not_of('\n')+1);
 }
 
 template <typename Z, typename Key>
@@ -458,7 +518,7 @@ void seen_altar( god_type god, const coord_def& pos )
     // can't record in abyss or pan.
     if ( you.level_type != LEVEL_DUNGEON )
         return;
-    
+
     level_pos where(level_id::current(), pos);
     altars_present[where] = god;
 }
@@ -510,3 +570,96 @@ void seen_other_thing( dungeon_feature_type which_thing, const coord_def& pos )
         break;
     }
 }          // end seen_other_thing()
+
+////////////////////////////////////////////////////////////////////////
+
+void set_level_annotation(std::string str,
+                          level_id li)
+{
+    if (str.empty())
+    {
+        clear_level_annotation(li);
+        return;
+    }
+
+    level_annotations[li] = str;
+}
+
+void clear_level_annotation(level_id li)
+{
+    level_annotations.erase(li);
+}
+
+std::string get_level_annotation(level_id li)
+{
+    annotation_map_type::const_iterator i = level_annotations.find(li);
+
+    if (i == level_annotations.end())
+        return "";
+
+    return (i->second);
+}
+
+bool level_annotation_has(std::string find,
+                          level_id li)
+{
+    std::string str = get_level_annotation(li);
+
+    return (str.find(find) != std::string::npos);
+}
+
+void annotate_level()
+{
+    level_id li  = level_id::current();
+    level_id li2 = level_id::current();
+
+    if (is_stair(grd[you.x_pos][you.y_pos]))
+    {
+        li2 = level_id::get_next_level_id(you.pos());
+
+        if (li2.level_type != LEVEL_DUNGEON || li2.depth <= 0)
+            li2 = level_id::current();
+    }
+
+    if (you.level_type != LEVEL_DUNGEON && li2.level_type != LEVEL_DUNGEON)
+    {
+        mpr("You can't annotate this level.");
+        return;
+    }
+
+    if (you.level_type != LEVEL_DUNGEON)
+        li = li2;
+    else if (li2 != level_id::current())
+    {
+        if (yesno("Annotate level on other end of current stairs?", true, 'n'))
+            li = li2;
+    }
+
+    if (get_level_annotation(li).length() > 0)
+    {
+        mpr("Current level annotation is:", MSGCH_PROMPT);
+        mpr(get_level_annotation(li).c_str() );
+    }
+
+    mpr( "Set level annotation to what? ", MSGCH_PROMPT );
+
+    char buf[77];
+    if (cancelable_get_line( buf, sizeof(buf) ))
+        return;
+
+    if (strlen(buf) == 0)
+    {
+        if (get_level_annotation(li).length() > 0)
+        {
+            if (!yesno("Really clear the annotation?"))
+                return;
+        }
+        else
+        {
+            canned_msg(MSG_OK);
+            return;
+        }
+    }
+
+    set_level_annotation(buf, li);
+}
