@@ -3,7 +3,7 @@
  *  Summary:    Player related functions.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-06T20:14:35.747799Z $
  *
  *  Change History (most recent first):
  *
@@ -16,6 +16,9 @@
 
 #include "externs.h"
 
+class monsters;
+struct item_def;
+
 enum genus_type
 {
     GENPC_DRACONIAN,                   //    0
@@ -31,7 +34,8 @@ bool player_in_mappable_area(void);
 bool player_in_branch( int branch );
 bool player_in_hell( void );
 
-int get_player_wielded_weapon();
+int get_player_wielded_weapon(void);
+bool berserk_check_wielded_weapon(void);
 int player_equip( equipment_type slot, int sub_type, bool calc_unid = true );
 int player_equip_ego_type( int slot, int sub_type );
 int player_damage_type( void );
@@ -76,8 +80,8 @@ bool wearing_amulet(char which_am, bool calc_unid = true);
 /* ***********************************************************************
  * called from: acr - chardump - describe - newgame - view
  * *********************************************************************** */
-std::string species_name( species_type speci, int level,
-                          bool genus = false, bool adj = false);
+std::string species_name( species_type speci, int level, bool genus = false,
+                          bool adj = false, bool ogre = false );
 int str_to_species(const std::string &species);
 
 /* ***********************************************************************
@@ -135,6 +139,7 @@ int player_movement_speed(void);
  * *********************************************************************** */
 int player_hunger_rate(void);
 
+int calc_hunger(int food_cost);
 
 /* ***********************************************************************
  * called from: debug - it_use3 - spells0
@@ -145,7 +150,7 @@ int player_magical_power( void );
 /* ***********************************************************************
  * called from: fight - misc - ouch - spells
  * *********************************************************************** */
-int player_prot_life(bool calc_unid = true);
+int player_prot_life(bool calc_unid = true, bool temp = true);
 
 
 /* ***********************************************************************
@@ -157,40 +162,48 @@ int player_regen(void);
 /* ***********************************************************************
  * called from: fight - files - it_use2 - misc - ouch - spells - spells2
  * *********************************************************************** */
-int player_res_cold(bool calc_unid = true);
-int player_res_acid(bool consider_unidentified_gear = true);
+int player_res_cold(bool calc_unid = true, bool temp = true,
+                    bool items = true);
+int player_res_acid(bool calc_unid = true, bool items = true);
 int player_acid_resist_factor();
 
 int player_res_torment(bool calc_unid = true);
 
 bool player_res_corrosion(bool calc_unid = true);
 bool player_item_conserve(bool calc_unid = true);
-int player_mental_clarity(bool calc_unid = true);
+int player_mental_clarity(bool calc_unid = true, bool items = true);
 
 bool player_can_smell();
 bool player_can_swim();
-bool player_likes_water();
+bool player_likes_water(bool permanently = false);
+
+int player_mutation_level(mutation_type mut);
 
 /* ***********************************************************************
  * called from: fight - files - ouch
  * *********************************************************************** */
-int player_res_electricity(bool calc_unid = true);
+int player_res_electricity(bool calc_unid = true, bool temp = true,
+                           bool items = true);
 
 
 /* ***********************************************************************
  * called from: acr - fight - misc - ouch - spells
  * *********************************************************************** */
-int player_res_fire(bool calc_unid = true);
-int player_res_steam(bool calc_unid = true);
+int player_res_fire(bool calc_unid = true, bool temp = true,
+                    bool items = true);
+int player_res_steam(bool calc_unid = true, bool temp = true,
+                     bool items = true);
 
 
 /* ***********************************************************************
  * called from: beam - decks - fight - fod - it_use2 - misc - ouch -
  *              spells - spells2
  * *********************************************************************** */
-int player_res_poison(bool calc_unid = true);
+int player_res_poison(bool calc_unid = true, bool temp = true,
+                      bool items = true);
 
-bool player_control_teleport(bool calc_unid = true);
+bool player_control_teleport(bool calc_unid = true, bool temp = true,
+                             bool items = true);
 
 int player_res_magic(void);
 
@@ -225,6 +238,10 @@ int player_speed(void);
  * *********************************************************************** */
 int player_spell_levels(void);
 
+/* ***********************************************************************
+ * called from: libgui - item_use
+ * *********************************************************************** */
+bool player_knows_spell(int spell);
 
 // last updated 18may2000 {dlb}
 /* ***********************************************************************
@@ -242,8 +259,8 @@ int player_teleport(bool calc_unid = true);
 /* ***********************************************************************
  * called from: ability - acr - items - misc - spells1 - spells3
  * *********************************************************************** */
-bool items_give_ability(const int slot, char abil);
-int scan_randarts(char which_property, bool calc_unid = true);
+bool items_give_ability(const int slot, randart_prop_type abil);
+int scan_randarts(randart_prop_type which_property, bool calc_unid = true);
 
 
 /* ***********************************************************************
@@ -260,6 +277,9 @@ int slaying_bonus(char which_affected);
 int player_see_invis(bool calc_unid = true);
 bool player_monster_visible( const monsters *mon );
 
+bool player_beheld_by( const monsters *mon );
+void update_beholders( const monsters *mon, bool force = false);
+void check_beholders();
 
 /* ***********************************************************************
  * called from: acr - decks - it_use2 - ouch
@@ -276,7 +296,7 @@ void display_char_status(void);
 /* ***********************************************************************
  * called from: item_use - items - misc - spells - spells3
  * *********************************************************************** */
-void forget_map(unsigned char chance_forgotten);
+void forget_map(unsigned char chance_forgotten = 100, bool force = false);
 
 
 // last updated 19may2000 {dlb}
@@ -291,20 +311,21 @@ void gain_exp(unsigned int exp_gained, unsigned int* actual_gain = NULL,
  * called from: acr - it_use2 - item_use - mutation - transfor - player -
  *              misc - stuff
  * *********************************************************************** */
-void modify_stat(stat_type which_stat, char amount, bool suppress_msg);
+void modify_stat(stat_type which_stat, char amount, bool suppress_msg,
+                 const std::string& cause, bool see_source = true);
+void modify_stat(stat_type which_stat, char amount, bool suppress_msg,
+                 const char* cause, bool see_source = true);
+void modify_stat(stat_type which_stat, char amount, bool suppress_msg,
+                 const monsters* cause);
+void modify_stat(stat_type which_stat, char amount, bool suppress_msg,
+                 const item_def &cause, bool removed = false);
 
 
 // last updated 19may2000 {dlb}
 /* ***********************************************************************
  * called from: decks - it_use2 - player
  * *********************************************************************** */
-void level_change(bool skip_ability_increase = false);
-
-
-/* ***********************************************************************
- * called from: skills
- * *********************************************************************** */
-void redraw_skill(const std::string &your_name, const std::string &class_name);
+void level_change(bool skip_attribute_increase = false);
 
 
 /* ***********************************************************************
@@ -313,8 +334,10 @@ void redraw_skill(const std::string &your_name, const std::string &class_name);
  * *********************************************************************** */
 bool player_genus( genus_type which_genus,
                    species_type species = SP_UNKNOWN );
+bool is_player_same_species( const int mon, bool = false );
 
 bool you_can_wear( int eq, bool special_armour = false );
+bool player_has_feet(void);
 bool you_tran_can_wear( int eq, bool check_mutation = false );
 
 /* ***********************************************************************
@@ -375,30 +398,19 @@ void deflate_hp(int new_level, bool floor);
  * *********************************************************************** */
 void set_hp(int new_amount, bool max_too);
 
+int get_real_hp(bool trans, bool rotted = false);
 
 /* ***********************************************************************
  * called from: it_use3 - newgame
  * *********************************************************************** */
 void set_mp(int new_amount, bool max_too);
 
-
-/* ***********************************************************************
- * called from: newgame
- * *********************************************************************** */
-int get_species_index_by_abbrev( const char *abbrev );
-int get_species_index_by_name( const char *name );
-const char *get_species_abbrev( int which_species );
-
-int get_class_index_by_abbrev( const char *abbrev );
-int get_class_index_by_name( const char *name );
-const char *get_class_abbrev( int which_job );
-const char *get_class_name( int which_job );
-
 // last updated 19apr2001 {gdl}
 /* ***********************************************************************
  * called from:
  * *********************************************************************** */
-void contaminate_player(int change, bool statusOnly = false);
+void contaminate_player(int change, bool controlled = false,
+                        bool status_only = false);
 
 /* @return true iff they were poisoned (false if they are immune) */
 bool poison_player( int amount, bool force = false );
@@ -419,7 +431,7 @@ void dec_disease_player();
 
 bool rot_player( int amount );
 
-bool player_has_spell( int spell );
+bool player_has_spell( spell_type spell );
 size_type player_size( int psize = PSIZE_TORSO, bool base = false );
 
 bool player_weapon_wielded();
