@@ -46,29 +46,34 @@
      KFEAT = 262,
      KITEM = 263,
      KMONS = 264,
-     NAME = 265,
-     DEPTH = 266,
-     ORIENT = 267,
-     PLACE = 268,
-     CHANCE = 269,
-     MONS = 270,
-     ITEM = 271,
-     MARKER = 272,
-     COLOUR = 273,
-     PRELUDE = 274,
-     MAIN = 275,
-     VALIDATE = 276,
-     VETO = 277,
-     NSUBST = 278,
-     WELCOME = 279,
-     COMMA = 280,
-     INTEGER = 281,
-     CHARACTER = 282,
-     STRING = 283,
-     MAP_LINE = 284,
-     MONSTER_NAME = 285,
-     ITEM_INFO = 286,
-     LUA_LINE = 287
+     KMASK = 265,
+     NAME = 266,
+     DEPTH = 267,
+     ORIENT = 268,
+     PLACE = 269,
+     CHANCE = 270,
+     MONS = 271,
+     ITEM = 272,
+     MARKER = 273,
+     COLOUR = 274,
+     PRELUDE = 275,
+     MAIN = 276,
+     VALIDATE = 277,
+     VETO = 278,
+     NSUBST = 279,
+     WELCOME = 280,
+     LFLAGS = 281,
+     BFLAGS = 282,
+     FLOORCOL = 283,
+     ROCKCOL = 284,
+     COMMA = 285,
+     INTEGER = 286,
+     CHARACTER = 287,
+     STRING = 288,
+     MAP_LINE = 289,
+     MONSTER_NAME = 290,
+     ITEM_INFO = 291,
+     LUA_LINE = 292
    };
 #endif
 /* Tokens.  */
@@ -79,43 +84,48 @@
 #define KFEAT 262
 #define KITEM 263
 #define KMONS 264
-#define NAME 265
-#define DEPTH 266
-#define ORIENT 267
-#define PLACE 268
-#define CHANCE 269
-#define MONS 270
-#define ITEM 271
-#define MARKER 272
-#define COLOUR 273
-#define PRELUDE 274
-#define MAIN 275
-#define VALIDATE 276
-#define VETO 277
-#define NSUBST 278
-#define WELCOME 279
-#define COMMA 280
-#define INTEGER 281
-#define CHARACTER 282
-#define STRING 283
-#define MAP_LINE 284
-#define MONSTER_NAME 285
-#define ITEM_INFO 286
-#define LUA_LINE 287
+#define KMASK 265
+#define NAME 266
+#define DEPTH 267
+#define ORIENT 268
+#define PLACE 269
+#define CHANCE 270
+#define MONS 271
+#define ITEM 272
+#define MARKER 273
+#define COLOUR 274
+#define PRELUDE 275
+#define MAIN 276
+#define VALIDATE 277
+#define VETO 278
+#define NSUBST 279
+#define WELCOME 280
+#define LFLAGS 281
+#define BFLAGS 282
+#define FLOORCOL 283
+#define ROCKCOL 284
+#define COMMA 285
+#define INTEGER 286
+#define CHARACTER 287
+#define STRING 288
+#define MAP_LINE 289
+#define MONSTER_NAME 290
+#define ITEM_INFO 291
+#define LUA_LINE 292
 
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 46 "levcomp.ypp"
+#line 48 "levcomp.ypp"
 {
     int i;
     const char *text;
     raw_range range;
 }
-/* Line 1489 of yacc.c.  */
-#line 119 "levcomp.tab.h"
+/* Line 1529 of yacc.c.  */
+#line 129 "levcomp.tab.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
