@@ -3,7 +3,7 @@
  *  Summary:    Shop keeper functions.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: haranp $ on $Date: 2007-11-30T14:03:02.050008Z $
+ *  Modified for Crawl Reference by $Author: zelgadis $ on $Date: 2008-06-19T05:05:08.087134Z $
  *
  *  Change History (most recent first):
  *
@@ -15,13 +15,14 @@
 #include "AppHdr.h"
 #include "chardump.h"
 #include "shopping.h"
+#include "message.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #ifdef DOS
-#include <conio.h>
+ #include <conio.h>
 #endif
 
 #include "externs.h"
@@ -42,12 +43,25 @@
 #include "stuff.h"
 #include "view.h"
 
-static void in_a_shop(int shopidx);
-static void more3();
-static void purchase( int shop, int item_got, int cost, bool id);
-static void shop_print(const char *shoppy, int sh_line);
+static void _in_a_shop(int shopidx);
+static void _purchase( int shop, int item_got, int cost, bool id);
 
-static std::string hyphenated_suffix(char prev, char last)
+static void _shop_print( const char *shoppy, int line )
+{
+    cgotoxy(1, line + 19, GOTO_CRT);
+    cprintf("%s", shoppy);
+    clear_to_end_of_line();
+}
+
+static void _shop_more()
+{
+    cgotoxy(65, 20, GOTO_CRT);
+    cprintf("-more-");
+    get_ch();
+    return;
+}
+
+static std::string _hyphenated_suffix(char prev, char last)
 {
     std::string s;
     if (prev > last + 2)
@@ -60,7 +74,7 @@ static std::string hyphenated_suffix(char prev, char last)
     return (s);
 }
 
-static std::string purchase_keys(const std::string &s)
+static std::string _purchase_keys(const std::string &s)
 {
     if (s.empty())
         return "";
@@ -73,22 +87,22 @@ static std::string purchase_keys(const std::string &s)
             continue;
 
         char prev = s[i - 1];
-        list += hyphenated_suffix(prev, last);
+        list += _hyphenated_suffix(prev, last);
         list += (last = s[i]);
     }
 
-    list += hyphenated_suffix( s[s.length() - 1], last );
+    list += _hyphenated_suffix( s[s.length() - 1], last );
     list += "</w>";
     return (list);
 }
 
-static void list_shop_keys(const std::string &purchasable)
+static void _list_shop_keys(const std::string &purchasable)
 {
     char buf[200];
     const int numlines = get_number_of_lines();
-    gotoxy(1, numlines - 1);
+    cgotoxy(1, numlines - 1, GOTO_CRT);
 
-    std::string pkeys = purchase_keys(purchasable);
+    std::string pkeys = _purchase_keys(purchasable);
     if (!pkeys.empty())
         pkeys = "[" + pkeys + "] Buy Item";
 
@@ -99,7 +113,7 @@ static void list_shop_keys(const std::string &purchasable)
     formatted_string fs = formatted_string::parse_string(buf);
     fs.cprintf("%*s", get_number_of_cols() - fs.length() - 1, "");
     fs.display();
-    gotoxy(1, numlines);
+    cgotoxy(1, numlines, GOTO_CRT);
 
     fs = formatted_string::parse_string(
             "[<w>?</w>/<w>*</w>]   Inventory  "
@@ -108,10 +122,10 @@ static void list_shop_keys(const std::string &purchasable)
     fs.display();
 }
 
-static std::vector<int> shop_stock(int shopidx)
+static std::vector<int> _shop_get_stock(int shopidx)
 {
     std::vector<int> result;
-    
+
     int itty = igrd[0][5 + shopidx];
 
     while ( itty != NON_ITEM )
@@ -122,7 +136,7 @@ static std::vector<int> shop_stock(int shopidx)
     return result;
 }
 
-static int shop_item_value(const item_def& item, int greed, bool id)
+static int _shop_get_item_value(const item_def& item, int greed, bool id)
 {
     int result = (greed * item_value(item, id) / 10);
     if ( you.duration[DUR_BARGAIN] ) // 20% discount
@@ -130,25 +144,26 @@ static int shop_item_value(const item_def& item, int greed, bool id)
         result *= 8;
         result /= 10;
     }
-    
+
     if (result < 1)
         result = 1;
 
     return result;
 }
 
-static std::string shop_print_stock( const std::vector<int>& stock,
+static std::string _shop_print_stock( const std::vector<int>& stock,
                                      const shop_struct& shop )
 {
-    ShopInfo &si = stashes.get_shop(shop.x, shop.y);
+    ShopInfo &si  = StashTrack.get_shop(shop.x, shop.y);
     const bool id = shoptype_identifies_stock(shop.type);
     std::string purchasable;
     for (unsigned int i = 0; i < stock.size(); ++i)
     {
-        const int gp_value = shop_item_value(mitm[stock[i]], shop.greed, id);
+        const int gp_value = _shop_get_item_value(mitm[stock[i]], shop.greed,
+                                                  id);
         const bool can_afford = (you.gold >= gp_value);
 
-        gotoxy(1, i+1);
+        cgotoxy(1, i+1, GOTO_CRT);
         const char c = i + 'a';
         if (can_afford)
             purchasable += c;
@@ -166,50 +181,61 @@ static std::string shop_print_stock( const std::vector<int>& stock,
     return purchasable;
 }
 
-static void in_a_shop( int shopidx )
+static void _in_a_shop( int shopidx )
 {
     const shop_struct& shop = env.shop[shopidx];
 
     cursor_control coff(false);
 
+    clrscr();
+
     const std::string hello = "Welcome to " + shop_name(shop.x, shop.y) + "!";
-    shop_print(hello.c_str(), 20);
+    bool first = true;
 
-    more3();    
-    
     const bool id_stock = shoptype_identifies_stock(shop.type);
- 
+
     while ( true )
     {
-        stashes.get_shop(shop.x, shop.y).reset();
-        
-        std::vector<int> stock = shop_stock(shopidx);
+        StashTrack.get_shop(shop.x, shop.y).reset();
+
+        std::vector<int> stock = _shop_get_stock(shopidx);
 
         clrscr();
-        if ( stock.empty() )
+        if (stock.empty())
         {
-            shop_print("I'm sorry, my shop is empty now.", 20);
-            more3();
+            _shop_print("I'm sorry, my shop is empty now.", 1);
+            _shop_more();
             return;
         }
-            
-        const std::string purchasable = shop_print_stock(stock, shop);
-        list_shop_keys(purchasable);
-            
+
+        const std::string purchasable = _shop_print_stock(stock, shop);
+        _list_shop_keys(purchasable);
+
         snprintf( info, INFO_SIZE, "You have %d gold piece%s.", you.gold,
                   (you.gold == 1) ? "" : "s" );
         textcolor(YELLOW);
-        shop_print(info, 19);
+        _shop_print(info, 0);
+
+        if (first)
+        {
+            first = false;
+            snprintf( info, INFO_SIZE, "%s What would you like to %s? ",
+                      hello.c_str(),
+                      purchasable.length() ? "purchase" : "do");
+        }
+        else
+        {
+            snprintf( info, INFO_SIZE, "What would you like to %s? ",
+                      purchasable.length() ? "purchase" : "do");
+        }
 
-        snprintf( info, INFO_SIZE, "What would you like to %s?",
-                  purchasable.length()? "purchase" : "do");
         textcolor(CYAN);
-        shop_print(info, 20);
-            
+        _shop_print(info, 1);
+
         textcolor(LIGHTGREY);
 
         int ft = get_ch();
-        
+
         if ( ft == '\\' )
             check_item_knowledge();
         else if (ft == 'x' || ft == ESCAPE)
@@ -217,13 +243,13 @@ static void in_a_shop( int shopidx )
         else if (ft == 'v')
         {
             textcolor(CYAN);
-            shop_print("Examine which item?", 20);
+            _shop_print("Examine which item?", 1);
             textcolor(LIGHTGREY);
 
             bool is_ok = true;
 
             ft = get_ch();
-            if ( !isalpha(ft) )
+            if (!isalpha(ft))
             {
                 is_ok = false;
             }
@@ -234,10 +260,10 @@ static void in_a_shop( int shopidx )
                     is_ok = false;
             }
 
-            if ( !is_ok )
+            if (!is_ok)
             {
-                shop_print("Huh?", 20);
-                more3();
+                _shop_print("Huh?", 1);
+                _shop_more();
                 continue;
             }
 
@@ -250,47 +276,49 @@ static void in_a_shop( int shopidx )
             // noted when you buy it.
             item_def& item = mitm[stock[ft]];
             const unsigned long old_flags = item.flags;
-            if ( id_stock )
+            if (id_stock)
+            {
                 item.flags |= (ISFLAG_IDENT_MASK | ISFLAG_NOTED_ID |
                                ISFLAG_NOTED_GET);
+            }
             describe_item(item);
-            if ( id_stock )
+            if (id_stock)
                 item.flags = old_flags;
         }
         else if (ft == '?' || ft == '*')
             invent(-1, false);
-        else if ( !isalpha(ft) )
+        else if (!isalpha(ft))
         {
-            shop_print("Huh?", 20);
-            more3();
+            _shop_print("Huh?", 1);
+            _shop_more();
         }
         else
         {
             ft = tolower(ft) - 'a';
             if (ft >= static_cast<int>(stock.size()) )
             {
-                shop_print("No such item.", 20);
-                more3();
+                _shop_print("No such item.", 1);
+                _shop_more();
                 continue;
             }
 
             item_def& item = mitm[stock[ft]];
-            
-            const int gp_value = shop_item_value(item, shop.greed, id_stock);
+
+            const int gp_value = _shop_get_item_value(item, shop.greed, id_stock);
             if (gp_value > you.gold)
             {
-                shop_print("I'm sorry, you don't seem to have enough money.",
-                           20);
-                more3();
+                _shop_print("I'm sorry, you don't seem to have enough money.", 1);
+                _shop_more();
             }
             else
             {
-                snprintf(info, INFO_SIZE, "Purchase %s (%d gold)? [y/n]",
+                textcolor(channel_to_colour(MSGCH_PROMPT));
+                snprintf(info, INFO_SIZE, "Purchase %s (%d gold)? (y/n) ",
                          item.name(DESC_NOCAP_A).c_str(), gp_value);
-                shop_print(info, 20);
+                _shop_print(info, 1);
 
                 if ( yesno(NULL, true, 'n', false, false, true) )
-                    purchase( shopidx, stock[ft], gp_value, id_stock );
+                    _purchase( shopidx, stock[ft], gp_value, id_stock );
             }
         }
     }
@@ -298,28 +326,12 @@ static void in_a_shop( int shopidx )
 
 bool shoptype_identifies_stock(shop_type type)
 {
-    return
-        type != SHOP_WEAPON_ANTIQUE &&
-        type != SHOP_ARMOUR_ANTIQUE &&
-        type != SHOP_GENERAL_ANTIQUE;
+    return (type != SHOP_WEAPON_ANTIQUE
+            && type != SHOP_ARMOUR_ANTIQUE
+            && type != SHOP_GENERAL_ANTIQUE);
 }
 
-static void shop_print( const char *shoppy, int sh_lines )
-{
-    gotoxy(1, sh_lines);
-    cprintf("%s", shoppy);
-    clear_to_end_of_line();
-}
-
-static void more3()
-{
-    gotoxy(70, 20);
-    cprintf("-more-");
-    get_ch();
-    return;
-}
-
-static void purchase( int shop, int item_got, int cost, bool id )
+static void _purchase( int shop, int item_got, int cost, bool id )
 {
     you.gold -= cost;
 
@@ -327,11 +339,17 @@ static void purchase( int shop, int item_got, int cost, bool id )
 
     origin_purchased(item);
 
+    // This is so that note_messages can be used to note items being
+    // bought, and also so you can use message history to review
+    // what you bought.
+    mprf("You bought %s.", item.name(DESC_NOCAP_A, false, id).c_str());
+    mesclr();
+
     if ( id )
     {
         // Identify the item and its type.
         // This also takes the ID note if necessary.
-        set_ident_type(item.base_type, item.sub_type, ID_KNOWN_TYPE);
+        set_ident_type(item, ID_KNOWN_TYPE);
         set_ident_flags(item, ISFLAG_IDENT_MASK);
     }
 
@@ -343,17 +361,18 @@ static void purchase( int shop, int item_got, int cost, bool id )
     if (num < quant)
     {
         snprintf( info, INFO_SIZE, "I'll put %s outside for you.",
-                 (quant == 1) ? "it" :
-                 (num > 0) ? "the rest" : "these" );
+                                   (quant == 1) ? "it" :
+                                   (num > 0)    ? "the rest"
+                                                : "these" );
 
-        shop_print( info, 20 );
-        more3();
+        _shop_print(info, 1);
+        _shop_more();
 
         move_item_to_grid( &item_got, env.shop[shop].x, env.shop[shop].y );
     }
-}                               // end purchase()
+}
 
-// This probably still needs some work.  Rings used to be the only 
+// This probably still needs some work.  Rings used to be the only
 // artefacts which had a change in price, and that value corresponds
 // to returning 50 from this function.  Good artefacts will probably
 // be returning just over 30 right now.  Note that this isn't used
@@ -370,9 +389,9 @@ int randart_value( const item_def &item )
     // Brands are already accounted for via existing ego checks
 
     // This should probably be more complex... but this isn't so bad:
-    ret += 3 * prop[ RAP_AC ] + 3 * prop[ RAP_EVASION ] 
-            + 3 * prop[ RAP_ACCURACY ] + 3 * prop[ RAP_DAMAGE ] 
-            + 6 * prop[ RAP_STRENGTH ] + 6 * prop[ RAP_INTELLIGENCE ] 
+    ret += 3 * prop[ RAP_AC ] + 3 * prop[ RAP_EVASION ]
+            + 3 * prop[ RAP_ACCURACY ] + 3 * prop[ RAP_DAMAGE ]
+            + 6 * prop[ RAP_STRENGTH ] + 6 * prop[ RAP_INTELLIGENCE ]
             + 6 * prop[ RAP_DEXTERITY ];
 
     // These resistances have meaningful levels
@@ -457,8 +476,7 @@ int randart_value( const item_def &item )
 unsigned int item_value( item_def item, bool ident )
 {
     // Note that we pass item in by value, since we want a local
-    // copy to mangle as necessary... maybe that should be fixed,
-    // but this function isn't called too often.
+    // copy to mangle as necessary.
     item.flags = (ident) ? (item.flags | ISFLAG_IDENT_MASK) : (item.flags);
 
     int valued = 0;
@@ -565,7 +583,7 @@ unsigned int item_value( item_def item, bool ident )
             valued += 35;
             break;
 
-        case WPN_ANCUS:
+        case WPN_ANKUS:
         case WPN_WAR_AXE:
         case WPN_MORNINGSTAR:
         case WPN_SABRE:
@@ -583,10 +601,13 @@ unsigned int item_value( item_def item, bool ident )
         case WPN_LONG_SWORD:
         case WPN_LONGBOW:
         case WPN_SCIMITAR:
+        case WPN_BLESSED_FALCHION:
             valued += 45;
             break;
 
         case WPN_SPIKED_FLAIL:
+        case WPN_BLESSED_LONG_SWORD:
+        case WPN_BLESSED_SCIMITAR:
             valued += 50;
 
         case WPN_HAND_CROSSBOW:
@@ -608,18 +629,15 @@ unsigned int item_value( item_def item, bool ident )
 
         case WPN_BATTLEAXE:
         case WPN_GREAT_MACE:
+        case WPN_EVENINGSTAR:
             valued += 65;
             break;
 
         case WPN_DIRE_FLAIL:
-        case WPN_LOCHABER_AXE:
+        case WPN_BARDICHE:
             valued += 90;
             break;
 
-        case WPN_EVENINGSTAR:
-            valued += 65;
-            break;
-
         case WPN_EXECUTIONERS_AXE:
             valued += 100;
             break;
@@ -638,9 +656,13 @@ unsigned int item_value( item_def item, bool ident )
             break;
 
         case WPN_KATANA:
-        case WPN_TRIPLE_SWORD:
         case WPN_DEMON_BLADE:
-        case WPN_BLESSED_BLADE:
+        case WPN_TRIPLE_SWORD:
+        case WPN_BLESSED_KATANA:
+        case WPN_BLESSED_EUDEMON_BLADE:
+        case WPN_BLESSED_DOUBLE_SWORD:
+        case WPN_BLESSED_GREAT_SWORD:
+        case WPN_BLESSED_TRIPLE_SWORD:
         case WPN_LAJATANG:
             valued += 200;
             break;
@@ -663,7 +685,6 @@ unsigned int item_value( item_def item, bool ident )
                 valued *= 60;
                 break;
 
-            case SPWPN_DISRUPTION:
             case SPWPN_FLAME:
             case SPWPN_FROST:
             case SPWPN_HOLY_WRATH:
@@ -684,6 +705,7 @@ unsigned int item_value( item_def item, bool ident )
 
             case SPWPN_FLAMING:
             case SPWPN_FREEZING:
+            case SPWPN_DRAGON_SLAYING:
                 valued *= 25;
                 break;
 
@@ -758,16 +780,16 @@ unsigned int item_value( item_def item, bool ident )
         {
             if (item_type_known(item))
                 valued += (7 * randart_value( item ));
-            else 
+            else
                 valued += 50;
         }
-        else if (item_type_known(item) 
-                && get_equip_desc(item) != 0)
+        else if (item_type_known(item)
+                 && get_equip_desc(item) != 0)
         {
             valued += 20;
         }
 
-        if (item_cursed( item ))
+        if (item_known_cursed(item))
         {
             valued *= 6;
             valued /= 10;
@@ -798,10 +820,15 @@ unsigned int item_value( item_def item, bool ident )
         case MI_NEEDLE:
             valued += 2;
             break;
+        case MI_JAVELIN:
+            valued += 8;
+            break;
+        case MI_THROWING_NET:
+            valued += 30;
+            break;
         default:
-            // was: cases 6 through 16 with empty strcat()'s 15jan2000 {dlb}
             valued += 5;
-            break;              //strcat(glog , ""); break;
+            break;
         }
         break;
 
@@ -893,6 +920,8 @@ unsigned int item_value( item_def item, bool ident )
             break;
 
         case ARM_HELMET:
+        case ARM_CAP:
+        case ARM_WIZARD_HAT:
         case ARM_BUCKLER:
             valued += 25;
             break;
@@ -976,7 +1005,7 @@ unsigned int item_value( item_def item, bool ident )
         }
 
         if (get_equip_race(item) == ISFLAG_ELVEN
-                || get_equip_race(item) == ISFLAG_DWARVEN)
+            || get_equip_race(item) == ISFLAG_DWARVEN)
         {
             valued *= 12;
             valued /= 10;
@@ -1019,7 +1048,7 @@ unsigned int item_value( item_def item, bool ident )
             valued += 20;
         }
 
-        if (item_cursed( item ))
+        if (item_known_cursed(item))
         {
             valued *= 6;
             valued /= 10;
@@ -1072,7 +1101,7 @@ unsigned int item_value( item_def item, bool ident )
             case WAND_POLYMORPH_OTHER:
                 valued += 90;
                 break;
-                
+
             case WAND_CONFUSION:
             case WAND_SLOWING:
                 valued += 70;
@@ -1126,6 +1155,7 @@ unsigned int item_value( item_def item, bool ident )
             case POT_RESTORE_ABILITIES:
                 valued += 50;
                 break;
+            case POT_BLOOD:
             case POT_BERSERK_RAGE:
             case POT_HEAL_WOUNDS:
                 valued += 30;
@@ -1134,11 +1164,11 @@ unsigned int item_value( item_def item, bool ident )
             case POT_SPEED:
                 valued += 25;
                 break;
-            case POT_BLOOD:
             case POT_HEALING:
             case POT_LEVITATION:
                 valued += 20;
                 break;
+            case POT_BLOOD_COAGULATED:
             case POT_PORRIDGE:
                 valued += 10;
                 break;
@@ -1228,6 +1258,7 @@ unsigned int item_value( item_def item, bool ident )
                 valued += 95;
                 break;
             case SCR_TORMENT:
+            case SCR_HOLY_WORD:
                 valued += 75;
                 break;
             case SCR_ENCHANT_WEAPON_II:
@@ -1255,13 +1286,15 @@ unsigned int item_value( item_def item, bool ident )
             case SCR_IDENTIFY:
                 valued += 20;
                 break;
+            case SCR_FOG:
+                valued += 10;
+                break;
             case SCR_NOISE:
             case SCR_RANDOM_USELESSNESS:
                 valued += 2;
                 break;
             case SCR_CURSE_ARMOUR:
             case SCR_CURSE_WEAPON:
-            case SCR_FORGETFULNESS:
             case SCR_PAPER:
             case SCR_IMMOLATION:
                 valued++;
@@ -1271,16 +1304,16 @@ unsigned int item_value( item_def item, bool ident )
         break;
 
     case OBJ_JEWELLERY:
-        if (item_cursed( item ))
+        if (item_known_cursed(item))
             valued -= 10;
 
         if ( !item_type_known(item) )
             valued += 50;
         else
         {
-            if (item_ident( item, ISFLAG_KNOW_PLUSES ) 
+            if (item_ident( item, ISFLAG_KNOW_PLUSES )
                 && (item.sub_type == RING_PROTECTION
-                    || item.sub_type == RING_STRENGTH 
+                    || item.sub_type == RING_STRENGTH
                     || item.sub_type == RING_EVASION
                     || item.sub_type == RING_DEXTERITY
                     || item.sub_type == RING_INTELLIGENCE
@@ -1439,7 +1472,7 @@ unsigned int item_value( item_def item, bool ident )
     case OBJ_STAVES:
         if (!item_type_known(item))
             valued = 120;
-        else if (item.sub_type == STAFF_SMITING 
+        else if (item.sub_type == STAFF_SMITING
                 || item.sub_type == STAFF_STRIKING
                 || item.sub_type == STAFF_WARDING
                 || item.sub_type == STAFF_DISCOVERY)
@@ -1450,7 +1483,7 @@ unsigned int item_value( item_def item, bool ident )
             valued = 250;
 
         if (item_is_rod( item ) && item_ident( item, ISFLAG_KNOW_PLUSES ))
-            valued += 50 * (item.plus2 / ROD_CHARGE_MULT); 
+            valued += 50 * (item.plus2 / ROD_CHARGE_MULT);
 
         break;
 
@@ -1479,11 +1512,19 @@ void shop()
 
     if (i == MAX_SHOPS)
     {
-        mpr("Help! Non-existent shop.", MSGCH_DANGER);
+        mpr("Help! Non-existent shop.", MSGCH_ERROR);
         return;
     }
 
-    in_a_shop(i);
+    // Quick out, if no inventory
+    if ( _shop_get_stock(i).empty() )
+    {
+        const shop_struct& shop = env.shop[i];
+        mprf("%s appears to be closed.", shop_name(shop.x, shop.y).c_str());
+        return;
+    }
+
+    _in_a_shop(i);
     burden_change();
     redraw_screen();
 }                               // end shop()
@@ -1545,7 +1586,7 @@ std::string shop_name(int sx, int sy)
          || shop_type == SHOP_WEAPON_ANTIQUE) ? "Weapon" :
         (shop_type == SHOP_ARMOUR
          || shop_type == SHOP_ARMOUR_ANTIQUE) ? "Armour" :
-        
+
         (shop_type == SHOP_JEWELLERY)         ? "Jewellery" :
         (shop_type == SHOP_WAND)              ? "Magical Wand" :
         (shop_type == SHOP_BOOK)              ? "Book" :
