@@ -3,7 +3,7 @@
  *  Summary:    Skill exercising functions.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2007-09-24T14:24:48.772888Z $
+ *  Modified for Crawl Reference by $Author: dolorous $ on $Date: 2008-07-02T04:33:02.329283Z $
  *
  *  Change History (most recent first):
  *
@@ -22,10 +22,13 @@
 
 #include "externs.h"
 
+#include "itemprop.h"
 #include "macro.h"
 #include "notes.h"
+#include "output.h"
 #include "player.h"
 #include "skills2.h"
+#include "spl-cast.h"
 #include "stuff.h"
 #include "tutorial.h"
 
@@ -33,28 +36,28 @@
 // MAX_COST_LIMIT is the maximum XP amount it will cost to raise a skill
 //                by 10 skill points (ie one standard practice).
 //
-// MAX_SPENDING_LIMIT is the maximum XP amount we allow the player to 
+// MAX_SPENDING_LIMIT is the maximum XP amount we allow the player to
 //                    spend on a skill in a single raise.
 //
-// Note that they don't have to be equal, but it is important to make 
+// Note that they don't have to be equal, but it is important to make
 // sure that they're set so that the spending limit will always allow
 // for 1 skill point to be earned.
 #define MAX_COST_LIMIT           250
 #define MAX_SPENDING_LIMIT       250
 
-static int exercise2( int exsk );
+static int _exercise2( int exsk );
 
 // These values were calculated by running a simulation of gaining skills.
 // The goal is to try and match the old cost system which used the player's
-// experience level (which has a number of problems) so things shouldn't 
-// seem too different to the player... but we still try to err on the 
+// experience level (which has a number of problems) so things shouldn't
+// seem too different to the player... but we still try to err on the
 // high side for the lower levels. -- bwr
 int skill_cost_needed( int level )
 {
-    // The average starting skill total is actually lower, but 
-    // some classes get about 2200, and they would probably be 
+    // The average starting skill total is actually lower, but
+    // some classes get about 2200, and they would probably be
     // start around skill cost level 3 if we used the average.  -- bwr
-    int ret = 2200; 
+    int ret = 2200;
 
     switch (level)
     {
@@ -86,28 +89,24 @@ void calc_total_skill_points( void )
     you.total_skill_points = 0;
 
     for (i = 0; i < NUM_SKILLS; i++)
-    {   
         you.total_skill_points += you.skill_points[i];
-    }
 
-    for (i = 1; i <= 27; i++) 
-    {
-        if (you.total_skill_points < skill_cost_needed(i)) 
+    for (i = 1; i <= 27; i++)
+        if (you.total_skill_points < skill_cost_needed(i))
             break;
-    } 
 
     you.skill_cost_level = i - 1;
 
 #if DEBUG_DIAGNOSTICS
-    you.redraw_experience = 1;
+    you.redraw_experience = true;
 #endif
 }
 
 // skill_cost_level makes skills more expensive for more experienced characters
-// skill_level      makes higher skills more expensive 
-static int calc_skill_cost( int skill_cost_level, int skill_level )
+// skill_level      makes higher skills more expensive
+static int _calc_skill_cost( int skill_cost_level, int skill_level )
 {
-    int ret = 1 + skill_level;    
+    int ret = 1 + skill_level;
 
     // does not yet allow for loss of skill levels.
     if (skill_level > 9)
@@ -161,7 +160,7 @@ int exercise(int exsk, int deg)
             break;
 
         if (you.practise_skill[exsk] || one_chance_in(4))
-            ret += exercise2( exsk );
+            ret += _exercise2( exsk );
 
         deg--;
     }
@@ -169,89 +168,86 @@ int exercise(int exsk, int deg)
 #ifdef DEBUG_DIAGNOSTICS
     if (ret)
     {
-        mprf(MSGCH_DIAGNOSTICS,
-                "Exercised %s (deg: %d) by %d",
-                skill_name(exsk),
-                deg,
-                ret);
+        mprf(MSGCH_DIAGNOSTICS, "Exercised %s (deg: %d) by %d",
+                                skill_name(exsk), deg, ret);
     }
 #endif
 
     return (ret);
 }                               // end exercise()
 
-static int exercise2( int exsk )
+static int _exercise2(int exsk)
 {
-    int deg = 1;
+    int deg = 10;
     int bonus = 0;
     char old_best_skill = best_skill(SK_FIGHTING, (NUM_SKILLS - 1), 99);
 
-    int skill_change = calc_skill_cost(you.skill_cost_level, you.skills[exsk]);
-    int i;
-
-    // being good at some weapons makes others easier to learn:
+    // Being good at some weapons makes others easier to learn.
     if (exsk < SK_ARMOUR)
     {
-        /* blades to blades */
-        if ((exsk == SK_SHORT_BLADES || exsk == SK_LONG_SWORDS)
+        // Short Blades and Long Blades.
+        if ((exsk == SK_SHORT_BLADES || exsk == SK_LONG_BLADES)
             && (you.skills[SK_SHORT_BLADES] > you.skills[exsk]
-                || you.skills[SK_LONG_SWORDS] > you.skills[exsk]))
+                || you.skills[SK_LONG_BLADES] > you.skills[exsk]))
         {
-            bonus += random2(3);
+            bonus += random2(30);
         }
 
-        /* Axes and Polearms */
+        // Axes and Polearms.
         if ((exsk == SK_AXES || exsk == SK_POLEARMS)
             && (you.skills[SK_AXES] > you.skills[exsk]
                 || you.skills[SK_POLEARMS] > you.skills[exsk]))
         {
-            bonus += random2(3);
+            bonus += random2(30);
         }
 
-        /* Polearms and Staves */
+        // Polearms and Staves.
         if ((exsk == SK_POLEARMS || exsk == SK_STAVES)
             && (you.skills[SK_POLEARMS] > you.skills[exsk]
                 || you.skills[SK_STAVES] > you.skills[exsk]))
         {
-            bonus += random2(3);
+            bonus += random2(30);
         }
 
-        /* Axes and Maces */
+        // Axes and Maces.
         if ((exsk == SK_AXES || exsk == SK_MACES_FLAILS)
             && (you.skills[SK_AXES] > you.skills[exsk]
                 || you.skills[SK_MACES_FLAILS] > you.skills[exsk]))
         {
-            bonus += random2(3);
+            bonus += random2(30);
         }
-        
-        /* Slings and Throwing */
+
+        // Slings and Throwing.
         if ((exsk == SK_SLINGS || exsk == SK_THROWING)
             && (you.skills[SK_SLINGS] > you.skills[exsk]
                 || you.skills[SK_THROWING] > you.skills[exsk]))
         {
-            bonus += random2(3);
+            bonus += random2(30);
         }
     }
 
-    // Quick fix for the fact that stealth can't be gained fast enough to
-    // keep up with the monster levels, this should speed its advancement
+    // Quick fix for the fact that stealth can't be gained fast enough
+    // to keep up with the monster levels.  This should speed its
+    // advancement.
     if (exsk == SK_STEALTH)
-        bonus += random2(3);
+        bonus += random2(30);
 
-    // spell casting is cheaper early on, and elementals hinder each other
+    int skill_change = _calc_skill_cost(you.skill_cost_level, you.skills[exsk]);
+
+    // Spellcasting is cheaper early on, and elementals hinder each
+    // other.
     if (exsk >= SK_SPELLCASTING)
     {
         if (you.skill_cost_level < 5)
-        {
             skill_change /= 2;
-        }
         else if (you.skill_cost_level < 15)
         {
             skill_change *= (10 + (you.skill_cost_level - 5));
             skill_change /= 20;
         }
 
-        // being good at elemental magic makes other elements harder to learn:
+        // Being good at elemental magic makes other elements harder to
+        // learn.
         if (exsk >= SK_FIRE_MAGIC && exsk <= SK_EARTH_MAGIC
             && (you.skills[SK_FIRE_MAGIC] > you.skills[exsk]
                 || you.skills[SK_ICE_MAGIC] > you.skills[exsk]
@@ -262,12 +258,12 @@ static int exercise2( int exsk )
                 return (0);
         }
 
-        // some are direct opposites
+        // Some are direct opposites.
         if ((exsk == SK_FIRE_MAGIC || exsk == SK_ICE_MAGIC)
             && (you.skills[SK_FIRE_MAGIC] > you.skills[exsk]
                 || you.skills[SK_ICE_MAGIC] > you.skills[exsk]))
         {
-            // of course, this is cumulative with the one above.
+            // Of course, this is cumulative with the one above.
             if (!one_chance_in(3))
                 return (0);
         }
@@ -280,10 +276,10 @@ static int exercise2( int exsk )
                 return (0);
         }
 
-        // experimental restriction (too many spell schools) -- bwr
+        // Experimental restriction (too many spell schools). -- bwr
         int skill_rank = 1;
 
-        for (i  = SK_CONJURATIONS; i <= SK_DIVINATIONS; i++)
+        for (int i = SK_CONJURATIONS; i <= SK_DIVINATIONS; ++i)
         {
             if (you.skills[exsk] < you.skills[i])
                 skill_rank++;
@@ -295,80 +291,65 @@ static int exercise2( int exsk )
             return (0);
     }
 
-    int fraction = 0;
-    int spending_limit = (you.exp_available < MAX_SPENDING_LIMIT) 
-                                    ? you.exp_available : MAX_SPENDING_LIMIT;
+    int spending_limit = MIN(MAX_SPENDING_LIMIT, you.exp_available);
 
-    // handle fractional learning
+    // Handle fractional learning.
     if (skill_change > spending_limit)
     {
-        // This system is a bit hard on missile weapons in the late game
-        // since they require expendable ammo in order to practise.
-        // Increasing the "deg"ree of exercise would make missile
-        // weapons too easy earlier on, so instead we're giving them
-        // a special case here.
-        if ((exsk != SK_DARTS && exsk != SK_BOWS && exsk != SK_CROSSBOWS)
-                                        || skill_change > you.exp_available)
+        // This system is a bit hard on missile weapons in the late
+        // game, since they require expendable ammo in order to
+        // practise.  Increasing the "deg"ree of exercise would make
+        // missile weapons too easy earlier on, so, instead, we're
+        // giving them a special case here.
+        if (exsk != SK_DARTS && exsk != SK_BOWS && exsk != SK_CROSSBOWS
+            || skill_change > you.exp_available)
         {
-            fraction = (spending_limit * 10) / skill_change;
-            skill_change = (skill_change * fraction) / 10;
+            int fraction = (spending_limit * 10) / skill_change;
 
-            deg = (deg * fraction) / 10;
+            deg *= fraction;
 
             if (deg == 0)
-                bonus = (bonus * fraction) / 10;
+                bonus *= fraction;
         }
         else
-        {
-            if ((skill_change / 2) > MAX_SPENDING_LIMIT)
-            {
-                deg = 0;
-                fraction = 5;
-            }
-            else
-            {
-                deg = 1;
-            }
+            deg = ((skill_change / 2) > MAX_SPENDING_LIMIT) ? 5 : 10;
 
-            skill_change = spending_limit;
-        }
+        skill_change = spending_limit;
     }
 
     skill_change -= random2(5);
 
-    if (skill_change < 1)
+    if (skill_change <= 0)
     {
-        // No free lunch, this is a problem now that we don't
-        // have overspending.
-        if (deg > 0 || fraction > 0 || bonus > 0)
-            skill_change = 1;
-        else
-            skill_change = 0;
+        // No free lunch.  This is a problem now that we don't have
+        // overspending.
+        skill_change = (deg > 0 || bonus > 0) ? 1 : 0;
     }
 
-    // Can safely return at any stage before this
-    int skill_inc = (deg + bonus) * 10 + fraction;
+    // We can safely return at any stage before this.
+    int skill_inc = deg + bonus;
 
-    // Starting to learn skills is easier if the appropriate stat is high
+    // Starting to learn skills is easier if the appropriate stat is
+    // high.
     if (you.skills[exsk] == 0)
     {
         if ((exsk >= SK_FIGHTING && exsk <= SK_STAVES) || exsk == SK_ARMOUR)
         {
-            // These skills are easier for the strong
-            skill_inc *= ((you.strength < 5) ? 5 : you.strength);
+            // These skills are easier for the strong.
+            skill_inc *= MAX(5, you.strength);
             skill_inc /= 10;
         }
         else if (exsk >= SK_SLINGS && exsk <= SK_UNARMED_COMBAT)
         {
-            // These skills are easier for the dexterous
+            // These skills are easier for the dexterous.
             // Note: Armour is handled above.
-            skill_inc *= ((you.dex < 5) ? 5 : you.dex);
+            skill_inc *= MAX(5, you.dex);
             skill_inc /= 10;
         }
         else if (exsk >= SK_SPELLCASTING && exsk <= SK_POISON_MAGIC)
         {
-            // These skills are easier for the smart
-            skill_inc *= ((you.intel < 5) ? 5 : you.intel);
+            // These skills are easier for the smart.
+            skill_inc *= MAX(5, you.intel);
             skill_inc /= 10;
         }
     }
@@ -381,33 +362,33 @@ static int exercise2( int exsk )
 
     you.total_skill_points += skill_inc;
     if (you.skill_cost_level < 27
-       && you.total_skill_points >= skill_cost_needed(you.skill_cost_level + 1))
+        && you.total_skill_points
+               >= skill_cost_needed(you.skill_cost_level + 1))
     {
         you.skill_cost_level++;
-    }     
+    }
 
-    if (you.exp_available < 0)
-        you.exp_available = 0;
+    you.exp_available = std::max(0, you.exp_available);
 
-    you.redraw_experience = 1;
+    you.redraw_experience = true;
 
 /*
     New (LH): debugging bit: when you exercise a skill, displays the skill
-    exercised and how much you spent on it. Too irritating to be a regular 
+    exercised and how much you spent on it. Too irritating to be a regular
     WIZARD feature.
 
 
 #if DEBUG_DIAGNOSTICS
-    mprf( MSGCH_DIAGNOSTICS, "Exercised %s * %d for %d xp.", 
+    mprf( MSGCH_DIAGNOSTICS, "Exercised %s * %d for %d xp.",
           skill_name(exsk), skill_inc, skill_change );
 #endif
 */
 
     if (you.skill_points[exsk] >
-        (skill_exp_needed(you.skills[exsk] + 2) 
-         * species_skills(exsk, you.species) / 100))
+           (skill_exp_needed(you.skills[exsk] + 1)
+            * species_skills(exsk, you.species) / 100))
     {
-        
+
         you.skills[exsk]++;
         take_note(Note(NOTE_GAIN_SKILL, exsk, you.skills[exsk]));
 
@@ -420,6 +401,8 @@ static int exercise2( int exsk )
         {
             mprf(MSGCH_INTRINSIC_GAIN,
                  "You have gained %s skill!", skill_name( exsk ) );
+
+            tut_gained_new_skill(exsk);
         }
         else
         {
@@ -429,11 +412,12 @@ static int exercise2( int exsk )
 
         learned_something_new(TUT_SKILL_RAISE);
 
-        // Recalculate this skill's order for tie breaking skills 
+        // Recalculate this skill's order for tie breaking skills
         // at its new level.   See skills2.cc::init_skill_order()
         // for more details.  -- bwr
         you.skill_order[exsk] = 0;
-        for (i = SK_FIGHTING; i < NUM_SKILLS; i++)
+
+        for (int i = SK_FIGHTING; i < NUM_SKILLS; ++i)
         {
             if (i == exsk)
                 continue;
@@ -446,36 +430,39 @@ static int exercise2( int exsk )
             calc_hp();
 
         if (exsk == SK_INVOCATIONS || exsk == SK_SPELLCASTING)
-        {
             calc_mp();
-        }
 
         if (exsk == SK_DODGING || exsk == SK_ARMOUR)
-            you.redraw_evasion = 1;
+            you.redraw_evasion = true;
 
-        if (exsk == SK_ARMOUR || exsk == SK_SHIELDS 
-                || exsk == SK_ICE_MAGIC || exsk == SK_EARTH_MAGIC
-                || you.duration[ DUR_TRANSFORMATION ] > 0)
+        if (exsk == SK_ARMOUR || exsk == SK_SHIELDS
+            || exsk == SK_ICE_MAGIC || exsk == SK_EARTH_MAGIC
+            || you.duration[ DUR_TRANSFORMATION ] > 0)
         {
-            you.redraw_armour_class = 1;
+            you.redraw_armour_class = true;
         }
 
-        const unsigned char best = best_skill( SK_FIGHTING, 
+        const unsigned char best = best_skill( SK_FIGHTING,
                                                (NUM_SKILLS - 1), 99 );
 
-        const unsigned char best_spell = best_skill( SK_SPELLCASTING, 
+        const unsigned char best_spell = best_skill( SK_SPELLCASTING,
                                                      SK_POISON_MAGIC, 99 );
 
-        if ((exsk == SK_SPELLCASTING) 
-                && (you.skills[exsk] == 1 && best_spell == SK_SPELLCASTING))
+        if (exsk == SK_SPELLCASTING
+            && you.skills[exsk] == 1 && best_spell == SK_SPELLCASTING)
         {
             mpr("You're starting to get the hang of this magic thing.");
         }
 
         if (best != old_best_skill || old_best_skill == exsk)
-        {
             redraw_skill( you.your_name, player_title() );
+
+        if (you.equip[EQ_WEAPON] != -1
+            && item_is_staff( you.inv[you.equip[EQ_WEAPON]] ))
+        {
+            maybe_identify_staff(you.inv[you.equip[EQ_WEAPON]]);
         }
     }
+
     return (skill_inc);
-}                               // end exercise2()
+}
