@@ -1,8 +1,10 @@
 /*
- *  File:       spells1.cc
+ *  File:       spells1.h
  *  Summary:    Implementations of some additional spells.
  *  Written by: Linley Henzell
  *
+ *  Modified for Crawl Reference by $Author: dolorous $ on $Date: 2008-06-10T23:51:06.609558Z $
+ *
  *  Change History (most recent first):
  *
  *               <1>     -/--/--        LRH             Created
@@ -14,7 +16,7 @@
 
 
 #include "externs.h"
-#include "direct.h"
+#include "directn.h"
 
 struct bolt;
 
@@ -45,27 +47,34 @@ char cast_greatest_healing(void);
  * called from: ability - spell
  * *********************************************************************** */
 char cast_lesser_healing(void);
-#endif 
+#endif
 
 // last updated 24may2000 {dlb}
 /* ***********************************************************************
  * called from: ability - spell
  * *********************************************************************** */
-int cast_healing(int power);
+int cast_healing(int pow, int target_x = -1, int target_y = -1);
+
+void vitalisation_chain(int amount);
+void reduce_vitalisation_chain(int amount);
+void remove_divine_robustness();
+void remove_divine_stamina();
+int cast_vitalisation(int pow);
 
 // last updated 24may2000 {dlb}
 /* ***********************************************************************
  * called from: beam - it_use3 - spells - spells1
  * *********************************************************************** */
 void big_cloud(cloud_type cl_type, kill_category whose, int cl_x, int cl_y,
-               int pow, int size);
+               int pow, int size, int spread_rate = -1);
 
 
 // last updated 24may2000 {dlb}
 /* ***********************************************************************
  * called from: acr (WIZARD only) - item_use - spell
  * *********************************************************************** */
-int blink(int pow, bool high_level_controlled_blink);
+int blink(int pow, bool high_level_controlled_blink,
+          bool wizard_blink = false);
 
 
 /* ***********************************************************************
@@ -74,16 +83,17 @@ int blink(int pow, bool high_level_controlled_blink);
 int cast_big_c(int pow, cloud_type cty, kill_category whose, bolt &beam);
 void cast_confusing_touch(int power);
 void cast_cure_poison(int mabil);
-int  allowed_deaths_door_hp(void);
+int allowed_deaths_door_hp(void);
 void cast_deaths_door(int pow);
-int cast_fire_storm(int powc, bolt &beam);
-bool cast_revivification(int power);
+void cast_fire_storm(int pow, bolt &beam);
+void cast_chain_lightning(int pow);
+bool cast_revivification(int pow);
 void cast_berserk(void);
 void cast_ring_of_flames(int power);
 bool conjure_flame(int pow);
 void extension(int pow);
-int fireball(int power, bolt &beam);
-int stinking_cloud(int pow, bolt &beam);
+bool fireball(int pow, bolt &beam);
+bool stinking_cloud(int pow, bolt &beam);
 void abjuration(int pow);
 
 // last updated 24may2000 {dlb}
@@ -117,7 +127,7 @@ void antimagic();
 /* ***********************************************************************
  * called from: acr (WIZARD only) - item_use - spell
  * *********************************************************************** */
-void identify(int power);
+void identify(int power, int item_slot = -1);
 
 
 // last updated 24may2000 {dlb}
