@@ -1,9 +1,9 @@
 /*
- *  File:       spells2.cc
+ *  File:       spells2.h
  *  Summary:    Implementations of some additional spells.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-02T13:23:51.271663Z $
+ *  Modified for Crawl Reference by $Author: dolorous $ on $Date: 2008-06-24T03:26:09.123836Z $
  *
  *  Change History (most recent first):
  *
@@ -14,7 +14,7 @@
 #define SPELLS2_H
 
 #include "enum.h"
-#include "externs.h"
+#include "itemprop.h"  // from brand_type()
 
 struct dist;
 
@@ -22,30 +22,14 @@ struct dist;
 /* ***********************************************************************
  * called from: spell
  * *********************************************************************** */
-bool brand_weapon(int which_brand, int power);
-
-
-// last updated 24may2000 {dlb}
-/* ***********************************************************************
- * called from: ability - spell
- * *********************************************************************** */
-int animate_a_corpse(int axps, int ayps, beh_type corps_beh,
-                     int corps_hit, int class_allowed);
-
-
-// last updated 24may2000 {dlb}
-/* ***********************************************************************
- * called from: ability - it_use3 - monstuff - mstuff2 - spell
- * *********************************************************************** */
-int animate_dead(actor *caster, int power, beh_type corps_beh,
-                 int corps_hit, int actual);
+bool brand_weapon(brand_type which_brand, int power);
 
 
 // last updated 24may2000 {dlb}
 /* ***********************************************************************
  * called from: spell
  * *********************************************************************** */
-char burn_freeze(int pow, char b_f);
+char burn_freeze(int pow, beam_type b_f);
 
 
 // last updated 24may2000 {dlb}
@@ -55,26 +39,19 @@ char burn_freeze(int pow, char b_f);
 int corpse_rot(int power);
 
 
-// last updated 24may2000 {dlb}
-/* ***********************************************************************
- * called from: it_use3 - spell
- * *********************************************************************** */
-int summon_elemental(int pow, int restricted_type, unsigned char unfriendly);
-
-
-class dist;
+struct dist;
 // last updated 24may2000 {dlb}
 /* ***********************************************************************
  * called from: spell
  * *********************************************************************** */
-int vampiric_drain(int pow, const dist &);
+bool vampiric_drain(int pow, const dist &vmove);
 
 
 // last updated 24may2000 {dlb}
 /* ***********************************************************************
  * called from: spell
  * *********************************************************************** */
-unsigned char detect_creatures( int pow );
+int detect_creatures( int pow, bool telepathic = false );
 
 
 // last updated 24may2000 {dlb}
@@ -88,7 +65,7 @@ unsigned char detect_items( int pow );
 /* ***********************************************************************
  * called from: spell
  * *********************************************************************** */
-unsigned char detect_traps( int pow ); 
+unsigned char detect_traps( int pow );
 
 
 // last updated 24may2000 {dlb}
@@ -105,13 +82,6 @@ void cast_refrigeration(int pow);
 void cast_toxic_radiance(void);
 
 
-// last updated 24may2000 {dlb}
-/* ***********************************************************************
- * called from: spell
- * *********************************************************************** */
-void cast_twisted(int power, beh_type corps_beh, int corps_hit);
-
-
 // last updated 24may2000 {dlb}
 /* ***********************************************************************
  * called from: ability
@@ -119,65 +89,80 @@ void cast_twisted(int power, beh_type corps_beh, int corps_hit);
 void drain_life(int pow);
 
 
-// last updated 24may2000 {dlb}
-/* ***********************************************************************
- * called from: ability - spell
- * *********************************************************************** */
-void holy_word(int pow, bool silent = false);
-
-
 // last updated 24may2000 {dlb}
 /* ***********************************************************************
  * called from: ability - food - it_use2 - spell
  * returns TRUE if a stat was restored.
  * *********************************************************************** */
-bool restore_stat(unsigned char which_stat, bool suppress_msg);
+bool restore_stat(unsigned char which_stat, unsigned char stat_gain,
+                  bool suppress_msg, bool recovery = false);
 
 
+// last updated 24may2000 {dlb}
+/* ***********************************************************************
+ * called from: spell
+ * *********************************************************************** */
+bool summon_animals(int pow);
+
 // last updated 24may2000 {dlb}
 /* ***********************************************************************
  * called from: ability - spell
  * *********************************************************************** */
-void summon_ice_beast_etc(int pow, int ibc, bool divine_gift = false);
-
-
-// last updated 24may2000 {dlb}
-/* ***********************************************************************
- * called from: spell
- * *********************************************************************** */
-void summon_scorpions(int pow);
-
-void summon_animals(int pow);
-
-
-// last updated 24may2000 {dlb}
-/* ***********************************************************************
- * called from: spell
- * *********************************************************************** */
-void summon_small_mammals(int pow);
-
-void summon_berserker();
+bool cast_summon_butterflies(int pow, god_type god = GOD_NO_GOD);
+bool cast_summon_small_mammals(int pow, god_type god = GOD_NO_GOD);
+bool cast_sticks_to_snakes(int pow, god_type god = GOD_NO_GOD);
+bool cast_summon_scorpions(int pow, god_type god = GOD_NO_GOD);
 
 // last updated 24may2000 {dlb}
 /* ***********************************************************************
  * called from: ability - religion - spell
  * *********************************************************************** */
-bool summon_swarm( int pow, bool unfriendly, bool god_gift );
-
+bool cast_summon_swarm(int pow, god_type god = GOD_NO_GOD,
+                       bool force_hostile = false);
 
 // last updated 24may2000 {dlb}
 /* ***********************************************************************
- * called from: spell
+ * called from: ability - spell
  * *********************************************************************** */
-void summon_things(int pow);
-
+bool cast_call_canine_familiar(int pow, god_type god = GOD_NO_GOD);
 
 // last updated 24may2000 {dlb}
 /* ***********************************************************************
- * called from: spell
+ * called from: it_use3 - spell
  * *********************************************************************** */
-void summon_undead(int pow);
+bool cast_summon_elemental(int pow, god_type god = GOD_NO_GOD,
+                           monster_type restricted_type = MONS_PROGRAM_BUG,
+                           int unfriendly = 2);
 
+// last updated 24may2000 {dlb}
+/* ***********************************************************************
+ * called from: ability - spell
+ * *********************************************************************** */
+bool cast_summon_ice_beast(int pow, god_type god = GOD_NO_GOD);
+bool cast_summon_ugly_thing(int pow, god_type god = GOD_NO_GOD);
+bool cast_summon_dragon(int pow, god_type god = GOD_NO_GOD);
+
+// last updated 24may2000 {dlb}
+/* ***********************************************************************
+ * called from: ability - spell
+ * *********************************************************************** */
+bool summon_berserker(int pow, god_type god = GOD_NO_GOD,
+                      bool force_hostile = false);
+bool summon_guardian(int pow, god_type god = GOD_NO_GOD, bool quiet = false);
+bool summon_daeva(int pow, god_type god = GOD_NO_GOD, bool quiet = false);
+
+// last updated 24may2000 {dlb}
+/* ***********************************************************************
+ * called from: ability - religion - spell
+ * *********************************************************************** */
+bool cast_tukimas_dance(int pow, god_type god = GOD_NO_GOD,
+                        bool force_hostile = false);
+
+// last updated 24may2000 {dlb}
+/* ***********************************************************************
+ * called from: ability - spell
+ * *********************************************************************** */
+bool cast_conjure_ball_lightning(int pow, god_type god = GOD_NO_GOD);
 
 // last updated 24may2000 {dlb}
 /* ***********************************************************************
