@@ -1,8 +1,10 @@
 /*
- *  File:       spells3.cc
+ *  File:       spells3.h
  *  Summary:    Implementations of some additional spells.
  *  Written by: Linley Henzell
  *
+ *  Modified for Crawl Reference by $Author: dolorous $ on $Date: 2008-06-16T16:31:01.959669Z $
+ *
  *  Change History (most recent first):
  *
  *               <1>     -/--/--        LRH             Created
@@ -12,6 +14,7 @@
 #ifndef SPELLS3_H
 #define SPELLS3_H
 
+#include "itemprop.h"  // from _raise_corpse()
 
 struct dist;
 struct bolt;
@@ -20,7 +23,7 @@ struct bolt;
 /* ***********************************************************************
  * called from: spells1 - spells3
  * *********************************************************************** */
-bool allow_control_teleport( bool silent = false );
+bool allow_control_teleport(bool quiet = false);
 
 
 // updated 24may2000 {dlb}
@@ -34,14 +37,7 @@ int airstrike(int power, dist &beam);
 /* ***********************************************************************
  * called from: spell
  * *********************************************************************** */
-int cast_bone_shards(int power, bolt &);
-
-
-// updated 24may2000 {dlb}
-/* ***********************************************************************
- * called from: spell - spells1
- * *********************************************************************** */
-bool cast_death_channel(int power);
+bool cast_bone_shards(int power, bolt &);
 
 
 // updated 24may2000 {dlb}
@@ -63,7 +59,11 @@ bool cast_selective_amnesia(bool force);
  * called from: ability - spell
  * *********************************************************************** */
 int cast_smiting(int power, dist &);
-
+bool remove_sanctuary(bool did_attack = false);
+void decrease_sanctuary_radius(void);
+bool cast_sanctuary(const int power);
+int halo_radius(void);
+bool inside_halo(int posx, int posy);
 
 // updated 24may2000 {dlb}
 /* ***********************************************************************
@@ -72,13 +72,6 @@ int cast_smiting(int power, dist &);
 bool project_noise(void);
 
 
-// updated 24may2000 {dlb}
-/* ***********************************************************************
- * called from: religion - spell
- * *********************************************************************** */
-void dancing_weapon(int pow, bool force_hostile);
-
-
 // updated 24may2000 {dlb}
 /* ***********************************************************************
  * called from: item_use - spell
@@ -118,14 +111,46 @@ bool remove_curse(bool suppress_msg);
 /* ***********************************************************************
  * called from: spell
  * *********************************************************************** */
-void sublimation(int power);
+bool cast_sublimation_of_blood(int pow);
 
+bool cast_call_imp(int pow, god_type god = GOD_NO_GOD);
+bool summon_lesser_demon(int pow, god_type god = GOD_NO_GOD,
+                         bool quiet = false);
+bool summon_common_demon(int pow, god_type god = GOD_NO_GOD,
+                         bool quiet = false);
+bool summon_greater_demon(int pow, god_type god = GOD_NO_GOD,
+                          bool quiet = false);
+bool summon_demon_type(monster_type mon, int pow, god_type god = GOD_NO_GOD);
+bool cast_summon_demon(int pow, god_type god = GOD_NO_GOD);
+bool cast_demonic_horde(int pow, god_type god = GOD_NO_GOD);
+bool cast_summon_greater_demon(int pow, god_type god = GOD_NO_GOD);
+bool cast_shadow_creatures(god_type god = GOD_NO_GOD);
+bool cast_summon_horrible_things(int pow, god_type god = GOD_NO_GOD);
 
-// updated Oct 2 -- bwr
+// last updated 24may2000 {dlb}
+/* ***********************************************************************
+ * called from: ability - spell
+ * *********************************************************************** */
+bool animate_a_corpse(const coord_def &a, corpse_type class_allowed,
+                      beh_type beha, unsigned short hitting,
+                      god_type god = GOD_NO_GOD, bool actual = true,
+                      bool quiet = false);
+
+// last updated 24may2000 {dlb}
+/* ***********************************************************************
+ * called from: ability - it_use3 - monstuff - mstuff2 - spell
+ * *********************************************************************** */
+int animate_dead(actor *caster, int pow, beh_type beha, unsigned short hitting,
+                 god_type god = GOD_NO_GOD, bool actual = true);
+
+// updated 24may2000 {dlb}
 /* ***********************************************************************
  * called from: spell
  * *********************************************************************** */
-void simulacrum(int power);
+bool cast_simulacrum(int pow, god_type god = GOD_NO_GOD);
+bool cast_twisted_resurrection(int pow, god_type god = GOD_NO_GOD);
+bool cast_summon_wraiths(int pow, god_type god = GOD_NO_GOD);
+bool cast_death_channel(int pow, god_type god = GOD_NO_GOD);
 
 
 // updated 24may2000 {dlb}
