@@ -3,7 +3,7 @@
  *  Summary:    Spellbook/Staff contents array and management functions
  *  Written by: Josh Fishman
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T13:43:41.265643Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-07T11:58:54.306684Z $
  *
  *  Change History (most recent first):
  *
@@ -21,7 +21,7 @@
 #include <string.h>
 
 #ifdef DOS
-#include <conio.h>
+ #include <conio.h>
 #endif
 
 #include "externs.h"
@@ -40,6 +40,7 @@
 #include "religion.h"
 #include "spl-cast.h"
 #include "spl-util.h"
+#include "state.h"
 #include "stuff.h"
 
 #define SPELLBOOK_SIZE 8
@@ -49,7 +50,7 @@ static spell_type spellbook_template_array[NUMBER_SPELLBOOKS][SPELLBOOK_SIZE] =
 {
     // 0 - Minor Magic I
     {SPELL_MAGIC_DART,
-     SPELL_SUMMON_SMALL_MAMMAL,
+     SPELL_SUMMON_SMALL_MAMMALS,
      SPELL_THROW_FLAME,
      SPELL_BLINK,
      SPELL_SLOW,
@@ -69,11 +70,11 @@ static spell_type spellbook_template_array[NUMBER_SPELLBOOKS][SPELLBOOK_SIZE] =
      },
     // 2 - Minor Magic III
     {SPELL_MAGIC_DART,
-     SPELL_SUMMON_SMALL_MAMMAL,
+     SPELL_SUMMON_SMALL_MAMMALS,
      SPELL_BLINK,
      SPELL_REPEL_MISSILES,
      SPELL_SLOW,
-     SPELL_SUMMON_LARGE_MAMMAL,
+     SPELL_CALL_CANINE_FAMILIAR,
      SPELL_MEPHITIC_CLOUD,
      SPELL_NO_SPELL,
      },
@@ -120,20 +121,20 @@ static spell_type spellbook_template_array[NUMBER_SPELLBOOKS][SPELLBOOK_SIZE] =
     // 7 - Book of Summonings
     {SPELL_ABJURATION_I,
      SPELL_RECALL,
-     SPELL_SUMMON_LARGE_MAMMAL,
+     SPELL_CALL_CANINE_FAMILIAR,
+     SPELL_SUMMON_UGLY_THING,
      SPELL_SHADOW_CREATURES,
      SPELL_SUMMON_WRAITHS,
      SPELL_SUMMON_HORRIBLE_THINGS,
      SPELL_NO_SPELL,
-     SPELL_NO_SPELL,
      },
     // 8 - Book of Fire
     {SPELL_EVAPORATE,
      SPELL_FIRE_BRAND,
      SPELL_SUMMON_ELEMENTAL,
      SPELL_BOLT_OF_MAGMA,
-     SPELL_DELAYED_FIREBALL,
      SPELL_IGNITE_POISON,
+     SPELL_DELAYED_FIREBALL,
      SPELL_RING_OF_FLAMES,
      SPELL_NO_SPELL,
      },
@@ -141,8 +142,8 @@ static spell_type spellbook_template_array[NUMBER_SPELLBOOKS][SPELLBOOK_SIZE] =
     {SPELL_FREEZING_AURA,
      SPELL_SLEEP,
      SPELL_CONDENSATION_SHIELD,
-     SPELL_BOLT_OF_COLD,
      SPELL_OZOCUBUS_REFRIGERATION,
+     SPELL_BOLT_OF_COLD,
      SPELL_SIMULACRUM,
      SPELL_MASS_SLEEP,
      SPELL_NO_SPELL,
@@ -160,7 +161,7 @@ static spell_type spellbook_template_array[NUMBER_SPELLBOOKS][SPELLBOOK_SIZE] =
      },
     // 11 - Book of Spatial Translocations
     {SPELL_APPORTATION,
-     SPELL_PORTALED_PROJECTILE,
+     SPELL_PORTAL_PROJECTILE,
      SPELL_BLINK,
      SPELL_RECALL,
      SPELL_TELEPORT_OTHER,
@@ -212,7 +213,7 @@ static spell_type spellbook_template_array[NUMBER_SPELLBOOKS][SPELLBOOK_SIZE] =
     {SPELL_CONFUSING_TOUCH,
      SPELL_SLOW,
      SPELL_CONFUSE,
-     SPELL_PARALYSE,
+     SPELL_PETRIFY,
      SPELL_NO_SPELL,
      SPELL_NO_SPELL,
      SPELL_NO_SPELL,
@@ -279,7 +280,7 @@ static spell_type spellbook_template_array[NUMBER_SPELLBOOKS][SPELLBOOK_SIZE] =
      SPELL_NO_SPELL,
      SPELL_NO_SPELL,
      },
-    // 23 - Book of Necromancy 
+    // 23 - Book of Necromancy
     {SPELL_PAIN,
      SPELL_ANIMATE_SKELETON,
      SPELL_VAMPIRIC_DRAINING,
@@ -300,7 +301,7 @@ static spell_type spellbook_template_array[NUMBER_SPELLBOOKS][SPELLBOOK_SIZE] =
      SPELL_NO_SPELL,
      },
     // 25 - Book of Callings
-    {SPELL_SUMMON_SMALL_MAMMAL,
+    {SPELL_SUMMON_SMALL_MAMMALS,
      SPELL_STICKS_TO_SNAKES,
      SPELL_CALL_IMP,
      SPELL_SUMMON_ELEMENTAL,
@@ -352,7 +353,7 @@ static spell_type spellbook_template_array[NUMBER_SPELLBOOKS][SPELLBOOK_SIZE] =
     //   what this spell is trying to be), one is more than enough...
     //   let people use necromancy.
     // - there is no need for another air spell... air spells are
-    //   already very common (ie. this level nine spell occured in
+    //   already very common (i.e,. this level nine spell occurred in
     //   two books!)
 
     // 29 - Book of the Sky
@@ -361,9 +362,9 @@ static spell_type spellbook_template_array[NUMBER_SPELLBOOKS][SPELLBOOK_SIZE] =
      SPELL_AIRSTRIKE,
      SPELL_FLY,
      SPELL_SILENCE,
-     SPELL_DEFLECT_MISSILES,
      SPELL_LIGHTNING_BOLT,
-     SPELL_CONJURE_BALL_LIGHTNING, 
+     SPELL_DEFLECT_MISSILES,
+     SPELL_CONJURE_BALL_LIGHTNING,
      },
 
     // 30 - Book of Divinations
@@ -513,7 +514,7 @@ static spell_type spellbook_template_array[NUMBER_SPELLBOOKS][SPELLBOOK_SIZE] =
     // 44 - Book of Cantrips      //jmf: added 04jan2000
     {SPELL_CONFUSING_TOUCH,
      SPELL_ANIMATE_SKELETON,
-     SPELL_SUMMON_SMALL_MAMMAL,
+     SPELL_SUMMON_SMALL_MAMMALS,
      SPELL_DETECT_SECRET_DOORS,
      SPELL_APPORTATION,
      SPELL_NO_SPELL,
@@ -533,10 +534,10 @@ static spell_type spellbook_template_array[NUMBER_SPELLBOOKS][SPELLBOOK_SIZE] =
      },
 
     // 46 - Book of Beasts //jmf: added 19mar2000
-    {SPELL_SUMMON_SMALL_MAMMAL,
+    {SPELL_SUMMON_SMALL_MAMMALS,
      SPELL_STICKS_TO_SNAKES,
      SPELL_DETECT_CREATURES,
-     SPELL_SUMMON_LARGE_MAMMAL,
+     SPELL_CALL_CANINE_FAMILIAR,
      SPELL_TAME_BEASTS,
      SPELL_DRAGON_FORM,
      SPELL_NO_SPELL,
@@ -549,7 +550,7 @@ static spell_type spellbook_template_array[NUMBER_SPELLBOOKS][SPELLBOOK_SIZE] =
      SPELL_PROJECTED_NOISE,
      SPELL_MEPHITIC_CLOUD,
      SPELL_POISON_WEAPON,
-     SPELL_PARALYSE,
+     SPELL_PETRIFY,
      SPELL_INVISIBILITY,
      SPELL_NO_SPELL,
      },
@@ -593,7 +594,7 @@ static spell_type spellbook_template_array[NUMBER_SPELLBOOKS][SPELLBOOK_SIZE] =
     {SPELL_ABJURATION_I,
      SPELL_RECALL,
      SPELL_SUMMON_ELEMENTAL,
-     SPELL_SWARM,
+     SPELL_SUMMON_SWARM,
      SPELL_NO_SPELL,
      SPELL_NO_SPELL,
      SPELL_NO_SPELL,
@@ -712,7 +713,7 @@ int spellbook_contents( item_def &book, read_book_action_type action,
 
     bool spell_skills = false;
 
-    for (i = SK_SPELLCASTING; i <= SK_POISON_MAGIC; i++) 
+    for (i = SK_SPELLCASTING; i <= SK_POISON_MAGIC; i++)
     {
         if (you.skills[i])
         {
@@ -738,11 +739,9 @@ int spellbook_contents( item_def &book, read_book_action_type action,
 
         out.cprintf(" ");
 
-        bool knowsSpell = false;
-        for (i = 0; i < 25 && !knowsSpell; i++)
-        {
-            knowsSpell = (you.spells[i] == stype);
-        }
+        bool knows_spell = false;
+        for (i = 0; i < 25 && !knows_spell; i++)
+            knows_spell = (you.spells[i] == stype);
 
         const int level_diff = spell_difficulty( stype );
         const int levels_req = spell_levels_required( stype );
@@ -750,9 +749,9 @@ int spellbook_contents( item_def &book, read_book_action_type action,
         int colour = DARKGREY;
         if (action == RBOOK_USE_STAFF)
         {
-            if (book.base_type == OBJ_BOOKS?
+            if (book.base_type == OBJ_BOOKS ?
                 you.experience_level >= level_diff
-                && you.magic_points >= level_diff
+                    && you.magic_points >= level_diff
                 : book.plus >= level_diff * ROD_CHARGE_MULT)
             {
                 colour = LIGHTGREY;
@@ -760,11 +759,11 @@ int spellbook_contents( item_def &book, read_book_action_type action,
         }
         else
         {
-            if (knowsSpell)
+            if (knows_spell)
                 colour = LIGHTGREY;
-            else if (you.experience_level >= level_diff 
-                        && spell_levels >= levels_req
-                        && spell_skills)
+            else if (you.experience_level >= level_diff
+                     && spell_levels >= levels_req
+                     && spell_skills)
             {
                 colour = LIGHTBLUE;
             }
@@ -772,12 +771,7 @@ int spellbook_contents( item_def &book, read_book_action_type action,
 
         out.textcolor( colour );
 
-        // Old:
-        // textcolor(knowsSpell ? DARKGREY : LIGHTGREY);
-        //              was: ? LIGHTGREY : LIGHTBLUE
-
         char strng[2];
-
         strng[0] = index_to_letter(spelcount);
         strng[1] = 0;
 
@@ -785,9 +779,9 @@ int spellbook_contents( item_def &book, read_book_action_type action,
         out.cprintf(" - ");
 
         out.cprintf( "%s", spell_title(stype) );
-        out.gotoxy( 35, -1 );
+        out.cgotoxy( 35, -1 );
+
 
-        
         if (action == RBOOK_USE_STAFF)
             out.cprintf( "Evocations" );
         else
@@ -807,7 +801,7 @@ int spellbook_contents( item_def &book, read_book_action_type action,
             }
         }
 
-        out.gotoxy( 65, -1 );
+        out.cgotoxy( 65, -1 );
 
         char sval[3];
 
@@ -953,55 +947,50 @@ bool is_valid_spell_in_book( int splbook, int spell )
     return which_spell_in_book(splbook, spell) != SPELL_NO_SPELL;
 }
 
-static bool which_spellbook( int &book, int &spell )
+static int _which_spellbook( void )
 {
+    int book = -1;
     const int avail_levels = player_spell_levels();
 
     // Knowing delayed fireball will allow Fireball to be learned for free -bwr
     if (avail_levels < 1 && !player_has_spell(SPELL_DELAYED_FIREBALL))
     {
         mpr("You can't memorise any more spells yet.");
-        return (false);
+        return (-1);
     }
     else if (inv_count() < 1)
     {
         canned_msg(MSG_NOTHING_CARRIED);
-        return (false);
+        return (-1);
     }
 
     mprf("You can memorise %d more level%s of spells.",
          avail_levels, (avail_levels > 1) ? "s" : "" );
 
     book = prompt_invent_item("Memorise from which spellbook?", MT_INVLIST,
-                              OBJ_BOOKS );
-    if (book == PROMPT_ABORT)
-    {
-        canned_msg( MSG_OK );
-        return (false);
-    }
+                              OSEL_MEMORISE );
+    if (prompt_failed(book))
+        return (-1);
 
     if (you.inv[book].base_type != OBJ_BOOKS
         || you.inv[book].sub_type == BOOK_MANUAL)
     {
         mpr("That isn't a spellbook!");
-        return (false);
+        return (-1);
     }
 
     if (you.inv[book].sub_type == BOOK_DESTRUCTION)
     {
         tome_of_power( book );
-        return (false);
+        return (-1);
     }
 
-    spell = read_book( you.inv[book], RBOOK_MEMORISE );
-    clrscr();
+    return (book);
+}
 
-    return (true);
-}                               // end which_spellbook()
-
-// Returns false if the player cannot read/memorize from the book,
+// Returns false if the player cannot read/memorise from the book,
 // and true otherwise. -- bwr
-static bool player_can_read_spellbook( const item_def &book )
+bool player_can_read_spellbook( const item_def &book )
 {
     if (book.base_type != OBJ_BOOKS)
         return (true);
@@ -1055,7 +1044,7 @@ int read_book( item_def &book, read_book_action_type action )
         return (0);
     }
 
-    // remember that this function is called from staff spells as well:
+    // Remember that this function is called from staff spells as well.
     const int keyin = spellbook_contents( book, action );
 
     if (book.base_type == OBJ_BOOKS)
@@ -1063,31 +1052,31 @@ int read_book( item_def &book, read_book_action_type action )
 
     redraw_screen();
 
-    /* Put special book effects in another function which can be called from
-       memorise as well */
+    // Put special book effects in another function which can be called
+    // from memorise as well.
 
     set_ident_flags( book, ISFLAG_KNOW_TYPE );
 
     return (keyin);
-}                               // end read_book()
+}
 
-// recoded to answer whether an UNDEAD_STATE is
+// Recoded to answer whether an UNDEAD_STATE is
 // barred from a particular spell passed to the
-// function - note that the function can be expanded
+// function. Note that the function can be expanded
 // to prevent memorisation of certain spells by
 // the living by setting up an US_ALIVE case returning
 // a value of false for a set of spells ... might be
-// an idea worth further consideration - 12mar2000 {dlb}
+// an idea worth further consideration. - 12mar2000 {dlb}
 bool undead_cannot_memorise(spell_type spell, char being)
 {
     switch (being)
     {
-    case US_HUNGRY_DEAD:
+    case US_HUNGRY_DEAD: // Ghouls
         switch (spell)
         {
-        //case SPELL_REGENERATION:
         case SPELL_AIR_WALK:
         case SPELL_ALTER_SELF:
+        case SPELL_BERSERKER_RAGE:
         case SPELL_BLADE_HANDS:
         case SPELL_BORGNJORS_REVIVIFICATION:
         case SPELL_CURE_POISON_II:
@@ -1100,24 +1089,37 @@ bool undead_cannot_memorise(spell_type spell, char being)
         case SPELL_STATUE_FORM:
         case SPELL_SYMBOL_OF_TORMENT:
         case SPELL_TAME_BEASTS:
-        case SPELL_BERSERKER_RAGE:
-            return true;
+            return (true);
         default:
-            return false;
+            return (false);
         }
         break;
 
-    case US_UNDEAD:
+    case US_SEMI_UNDEAD: // Vampires
+        switch (spell)
+        {
+        case SPELL_BORGNJORS_REVIVIFICATION:
+        case SPELL_DEATHS_DOOR:
+        case SPELL_NECROMUTATION:
+            return (true);
+        default:
+            // In addition, the above US_HUNGRY_DEAD spells are not castable
+            // when satiated or worse.
+            return (false);
+        }
+        break;
+
+    case US_UNDEAD: // Mummies
         switch (spell)
         {
         case SPELL_AIR_WALK:
         case SPELL_ALTER_SELF:
+        case SPELL_BERSERKER_RAGE:
         case SPELL_BLADE_HANDS:
         case SPELL_BORGNJORS_REVIVIFICATION:
         case SPELL_CURE_POISON_II:
         case SPELL_DEATHS_DOOR:
         case SPELL_DRAGON_FORM:
-        case SPELL_GLAMOUR:
         case SPELL_ICE_FORM:
         case SPELL_INTOXICATE:
         case SPELL_NECROMUTATION:
@@ -1129,69 +1131,64 @@ bool undead_cannot_memorise(spell_type spell, char being)
         case SPELL_SUMMON_HORRIBLE_THINGS:
         case SPELL_SYMBOL_OF_TORMENT:
         case SPELL_TAME_BEASTS:
-        case SPELL_BERSERKER_RAGE:
-            return true;
+            return (true);
         default:
-            return false;
+            return (false);
         }
         break;
     }
 
-    return false;
-}                               // end undead_cannot_memorise()
+    return (false);
+}
 
 bool player_can_memorise(const item_def &book)
 {
-    if (book.base_type != OBJ_BOOKS || book.sub_type == BOOK_MANUAL)
-        return false;
-    
+    if (book.base_type != OBJ_BOOKS || book.sub_type == BOOK_MANUAL
+        || book.sub_type == BOOK_DESTRUCTION)
+    {
+        return (false);
+    }
+
     if (!player_spell_levels())
-        return false;
-        
+        return (false);
+
     for (int j = 0; j < SPELLBOOK_SIZE; j++)
     {
-         const spell_type stype = which_spell_in_book(book.book_number(), j);
+        const spell_type stype = which_spell_in_book(book.book_number(), j);
 
-         if (stype == SPELL_NO_SPELL)
-             continue;
+        if (stype == SPELL_NO_SPELL)
+            continue;
 
-         // easiest spell already too difficult
-         if (spell_difficulty(stype) > you.experience_level
-             || player_spell_levels() < spell_levels_required(stype))
-         {
-             return false;
-         }
+        // Easiest spell already too difficult?
+        if (spell_difficulty(stype) > you.experience_level
+            || player_spell_levels() < spell_levels_required(stype))
+        {
+            return (false);
+        }
 
-         bool knowsSpell = false;
-         for (int i = 0; i < 25 && !knowsSpell; i++)
-         {
-              knowsSpell = (you.spells[i] == stype);
-         }
+        bool knows_spell = false;
+        for (int i = 0; i < 25 && !knows_spell; i++)
+            knows_spell = (you.spells[i] == stype);
 
-         // don't already know this spell
-         if (!knowsSpell)
-         {
-             return true;
-         }
+        // You don't already know this spell.
+        if (!knows_spell)
+            return (true);
     }
-    return false;
+    return (false);
 }
-                         
-bool learn_spell(void)
+
+bool learn_spell(int book)
 {
     int chance = 0;
     int levels_needed = 0;
-    int book, spell;
     int index;
 
     int i;
     int j = 0;
 
     for (i = SK_SPELLCASTING; i <= SK_POISON_MAGIC; i++)
-    {
         if (you.skills[i])
             j++;
-    }
 
     if (j == 0)
     {
@@ -1211,9 +1208,15 @@ bool learn_spell(void)
         return (false);
     }
 
-    if (!which_spellbook( book, spell ))
+    if (book < 0)
+        book = _which_spellbook();
+
+    if (book < 0) // still -1?
         return (false);
 
+    int spell = read_book( you.inv[book], RBOOK_MEMORISE );
+    clrscr();
+
     mesclr(true);
     redraw_screen();
 
@@ -1225,8 +1228,8 @@ bool learn_spell(void)
 
     index = letter_to_index( spell );
 
-    if (index >= SPELLBOOK_SIZE ||
-        !is_valid_spell_in_book( you.inv[book].sub_type, index ))
+    if (index >= SPELLBOOK_SIZE
+        || !is_valid_spell_in_book( you.inv[book].sub_type, index ))
     {
         canned_msg( MSG_HUH );
         return (false);
@@ -1307,8 +1310,8 @@ bool learn_spell(void)
         return (false);
     }
 
-    if (you.mutation[MUT_BLURRY_VISION] > 0
-        && random2(4) < you.mutation[MUT_BLURRY_VISION])
+    if (player_mutation_level(MUT_BLURRY_VISION) > 0
+        && x_chance_in_y(player_mutation_level(MUT_BLURRY_VISION), 4))
     {
         mpr("The writing blurs into unreadable gibberish.");
         you.turn_is_over = true;
@@ -1323,7 +1326,7 @@ bool learn_spell(void)
         if (you.inv[ book ].sub_type == BOOK_NECRONOMICON)
         {
             mpr("The pages of the Necronomicon glow with a dark malevolence...");
-            miscast_effect( SPTYP_NECROMANCY, 8, random2avg(88, 3), 100, 
+            miscast_effect( SPTYP_NECROMANCY, 8, random2avg(88, 3), 100,
                             "reading the Necronomicon" );
         }
         else if (you.inv[ book ].sub_type == BOOK_DEMONOLOGY)
@@ -1344,7 +1347,7 @@ bool learn_spell(void)
             return (false);
         else if (!yesno("Memorise anyway?"))
             return (false);
-#else 
+#else
         return (false);
 #endif
     }
@@ -1361,7 +1364,7 @@ int count_staff_spells(const item_def &item, bool need_id)
 {
     if (item.base_type != OBJ_STAVES)
         return (-1);
-    
+
     if (need_id && !item_type_known(item))
         return (0);
 
@@ -1397,12 +1400,12 @@ int rod_shield_leakage()
 int staff_spell( int staff )
 {
     item_def& istaff(you.inv[staff]);
-    // Spell staves are mostly for the benefit of non-spellcasters, so we're 
+    // Spell staves are mostly for the benefit of non-spellcasters, so we're
     // not going to involve INT or Spellcasting skills for power. -- bwr
-    int powc = (5 + you.skills[SK_EVOCATIONS] 
+    int powc = (5 + you.skills[SK_EVOCATIONS]
                  + roll_dice( 2, you.skills[SK_EVOCATIONS] ))
                 * 100
-                / rod_shield_leakage(); 
+                / rod_shield_leakage();
 
     if (!item_is_rod(istaff))
     {
@@ -1410,9 +1413,20 @@ int staff_spell( int staff )
         return (-1);
     }
 
+    bool need_id = false;
     if (!item_type_known(istaff))
     {
+        set_ident_type( OBJ_STAVES, istaff.sub_type, ID_KNOWN_TYPE );
         set_ident_flags( istaff, ISFLAG_KNOW_TYPE );
+        need_id = true;
+    }
+    if (!item_ident( istaff, ISFLAG_KNOW_PLUSES))
+    {
+        set_ident_flags( istaff, ISFLAG_KNOW_PLUSES );
+        need_id = true;
+    }
+    if (need_id)
+    {
         mprf(MSGCH_EQUIPMENT, "%s", istaff.name(DESC_INVENTORY_EQUIP).c_str());
         you.wield_change = true;
     }
@@ -1437,7 +1451,7 @@ int staff_spell( int staff )
 
         // Note that auto_list is ignored here.
         keyin = get_ch();
-        
+
         if (keyin == '?' || keyin == '*')
         {
             keyin = read_book( you.inv[staff], RBOOK_USE_STAFF );
@@ -1480,34 +1494,42 @@ int staff_spell( int staff )
         food -= 10 * you.skills[SK_EVOCATIONS];
         if (food < diff * 5)
             food = diff * 5;
+
+        food = calc_hunger(food);
     }
-        
-    if (food && (you.hunger_state <= HS_STARVING || you.hunger <= food))
+
+    if (food && (you.hunger_state == HS_STARVING || you.hunger <= food))
     {
         mpr("You don't have the energy to cast that spell.");
+        crawl_state.zero_turns_taken();
         return (-1);
     }
-    
+
     if (istaff.plus < mana)
     {
         mpr("The rod doesn't have enough magic points.");
+        crawl_state.zero_turns_taken();
         // Don't lose a turn for trying to evoke without enough MP - that's
         // needlessly cruel for an honest error.
         return (-1);
     }
 
     const int flags = get_spell_flags(spell);
+
     // Labyrinths block divinations.
     if (you.level_type == LEVEL_LABYRINTH
         && testbits(flags, SPFLAG_MAPPING))
     {
         mpr("Something interferes with your magic!");
     }
-    // All checks passed, we can cast the spell    
+    // All checks passed, we can cast the spell.
     else if (your_spells(spell, powc, false) == SPRET_ABORT)
+    {
+        crawl_state.zero_turns_taken();
         return (-1);
+    }
 
-    make_hungry( food, true );
+    make_hungry(food, true, true);
     istaff.plus -= mana;
     you.wield_change = true;
     you.turn_is_over = true;
