@@ -3,7 +3,7 @@
  *  Summary:    Some spellbook related functions.
  *  Written by: Josh Fishman
  *
- *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2007-11-19T14:28:34.408079Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-02-18T15:51:08.326662Z $
  *
  *  Change History (most recent first):
  *
@@ -49,7 +49,9 @@ int read_book( item_def &item, read_book_action_type action );
  * called from: acr
  * *********************************************************************** */
 bool player_can_memorise(const item_def &book);
-bool learn_spell(void);
+bool learn_spell(int book = -1);
+
+bool player_can_read_spellbook( const item_def &book );
 
 spell_type which_spell_in_book(int sbook_type, int spl);
 
