@@ -1,9 +1,9 @@
 /*
- *  File:       spl-cast.cc
+ *  File:       spl-cast.h
  *  Summary:    Spell casting functions.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-10-25T18:23:34.828022Z $
+ *  Modified for Crawl Reference by $Author: zelgadis $ on $Date: 2008-06-29T05:16:39.795855Z $
  *
  *  Change History (most recent first):
  *
@@ -25,9 +25,13 @@ enum spflag_type
     SPFLAG_DIR                  = 0x0008,       // use DIR_DIR targeting
     SPFLAG_TARGETING_MASK       = 0x000f,       // used to test for targeting
     SPFLAG_HELPFUL              = 0x0010,       // TARG_FRIENDS used
-    SPFLAG_NOT_SELF             = 0x0020,       // aborts on isMe
-    SPFLAG_UNHOLY               = 0x0040,       // counts at "unholy"
-    SPFLAG_MAPPING              = 0x0080        // a mapping spell of some kind
+    SPFLAG_NEUTRAL              = 0x0020,       // TARG_ANY used
+    SPFLAG_NOT_SELF             = 0x0040,       // aborts on isMe
+    SPFLAG_UNHOLY               = 0x0080,       // counts at "unholy"
+    SPFLAG_MAPPING              = 0x0100,       // a mapping spell of some kind
+    SPFLAG_ESCAPE               = 0x0200,       // useful for running away
+    SPFLAG_RECOVERY             = 0x0400,       // healing or recovery spell
+    SPFLAG_AREA                 = 0x0800        // area affect
 };
 
 enum spret_type
@@ -56,6 +60,8 @@ void exercise_spell( spell_type spell_ex, bool spc, bool divide );
  * *********************************************************************** */
 bool cast_a_spell( void );
 
+bool maybe_identify_staff( item_def &item, spell_type spell = SPELL_NO_SPELL );
+
 void inspect_spells();
 
 
@@ -63,16 +69,15 @@ void inspect_spells();
 /* ***********************************************************************
  * called from: ability - debug - it_use3 - spell
  * *********************************************************************** */
-spret_type your_spells( spell_type spell, int powc = 0,
-                        bool allow_fail = true );
+spret_type your_spells(spell_type spell, int powc = 0, bool allow_fail = true);
 
 // last updated 12may2000 {dlb}
 /* ***********************************************************************
  * called from: acr - decks - fight - it_use2 - it_use3 - item_use - items -
  *              misc - mstuff2 - religion - spell - spl-book - spells4
  * *********************************************************************** */
-void miscast_effect( unsigned int sp_type, int mag_pow, int mag_fail, 
-                     int force_effect, const char *cause = NULL );
+void miscast_effect(unsigned int sp_type, int mag_pow, int mag_fail,
+                    int force_effect, const char *cause = NULL);
 
 const char* failure_rate_to_string( int fail );
 
