@@ -1,8 +1,10 @@
 /*
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T13:43:41.265643Z $
+ *  Modified for Crawl Reference by $Author: dolorous $ on $Date: 2008-07-06T14:05:39.071393Z $
  */
 
 /*
+   XXX: Some of these notes are obsolete now!
+
    In case anyone ever wants to add new spells, or just understand my reasons
    for putting a particular spell into a particular type, read on:
 
@@ -12,7 +14,7 @@
    This type has a near monopoly on effective and relatively risk-free combat
    spells. All other types of combat spells are either indirect (enchantments),
    risky/detrimental/not versatile (necromancy) or just plain crappy (burn and
-   freeze), although smiting (holy) is not too bad.
+   freeze), although smiting is not too bad.
    Conjuration spells all involve the magical creation of matter and/or energy
    (which are the same thing anyway, right?). They are distinguished from
    summoning spells in that they do not involve the summoning of an entire
@@ -106,16 +108,7 @@
    I don't plan to implement a 'Poisoner' class, as it would become unplayable
    deep in the dungeon where most monsters are poison resistant.
 
-   Many spells use magic from two types. These spells are equally
-   available to either type; a conjurer is no worse at a fire/conjuration than
-   at a pure conjuration. I guess a spell could be of three types, but they
-   would have to be types with short names (limited space in the spell
-   windows).
-   - Note : this is no longer true, with the implementation of magic skills.
-   Your skill for a spell is effectively the average of all types used in it.
-   Poison has no skills, but still has a staff
-
-
+   Your skill for a spell is effectively the average of all schools used in it.
 */
 
 /*
@@ -138,30 +131,30 @@
 
 {
     SPELL_IDENTIFY, "Identify",
-    SPTYP_DIVINATION,
-    SPFLAG_NONE,
-    6,
-    0,
-    NULL,
-    false,
-    true
+     SPTYP_DIVINATION,
+     SPFLAG_NONE,
+     6,
+     0,
+     NULL,
+     false,
+     true
 },
 
 {
     SPELL_TELEPORT_SELF, "Teleport Self",
      SPTYP_TRANSLOCATION,
-     SPFLAG_NONE,
+     SPFLAG_ESCAPE,
      5,
      0,
      NULL,
      false,
-     false
+     true
 },
 
 {
     SPELL_CAUSE_FEAR, "Cause Fear",
      SPTYP_ENCHANTMENT,
-     SPFLAG_NONE,
+     SPFLAG_AREA,
      5,
      200,
      NULL,
@@ -214,35 +207,69 @@
 },
 
 {
-  SPELL_SWAP, "Swap",
-    SPTYP_TRANSLOCATION,
-    SPFLAG_NONE,
-    4,
-    0,
-    NULL,
-    false,
-    false
+    SPELL_SWAP, "Swap",
+     SPTYP_TRANSLOCATION,
+     SPFLAG_NONE,
+     4,
+     0,
+     NULL,
+     false,
+     false
 },
 
 {
-  SPELL_APPORTATION, "Apportation",
-    SPTYP_TRANSLOCATION,
-    SPFLAG_NONE,
-    1,
-    1000,
-    NULL,
-    false,
-    false
+    SPELL_APPORTATION, "Apportation",
+     SPTYP_TRANSLOCATION,
+     SPFLAG_NONE,
+     1,
+     1000,
+     NULL,
+     false,
+     false
 },
 
 {
-  SPELL_TWIST, "Twist",
-    SPTYP_TRANSLOCATION,
-    SPFLAG_DIR_OR_TARGET,
-    1,
-    25,
-    NULL,
-    true
+    SPELL_TWIST, "Twist",
+     SPTYP_TRANSLOCATION,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
+     1,
+     25,
+     NULL,
+     true,
+     false
+},
+
+{
+    SPELL_FAR_STRIKE, "Far Strike",
+     SPTYP_TRANSLOCATION,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
+     3,
+     100,
+     NULL,
+     true,
+     false
+},
+
+{
+    SPELL_DELAYED_FIREBALL, "Delayed Fireball",
+     SPTYP_FIRE | SPTYP_CONJURATION,
+     SPFLAG_NONE,
+     7,
+     0,
+     NULL,
+     false,
+     true
+},
+
+{
+    SPELL_STRIKING, "Striking",
+     0,
+     SPFLAG_DIR_OR_TARGET,
+     1,
+     25,
+     NULL,
+     true,
+     false
 },
 
 {
@@ -253,13 +280,13 @@
      100,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_DIG, "Dig",
      SPTYP_TRANSMIGRATION | SPTYP_EARTH,
-     SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF | SPFLAG_NEUTRAL,
      4,
      200,
      NULL,
@@ -271,30 +298,33 @@
     SPELL_BOLT_OF_FIRE, "Bolt of Fire",
      SPTYP_CONJURATION | SPTYP_FIRE,
      SPFLAG_DIR_OR_TARGET,
-     5,
+     6,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
     SPELL_BOLT_OF_COLD, "Bolt of Cold",
      SPTYP_CONJURATION | SPTYP_ICE,
      SPFLAG_DIR_OR_TARGET,
-     5,
+     6,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
     SPELL_LIGHTNING_BOLT, "Lightning Bolt",
      SPTYP_CONJURATION | SPTYP_AIR,
      SPFLAG_DIR_OR_TARGET,
-     6,
+     5,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -304,7 +334,8 @@
      5,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -314,7 +345,8 @@
      5,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -324,7 +356,8 @@
      3,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -332,8 +365,8 @@
      SPTYP_ENCHANTMENT,
      SPFLAG_DIR_OR_TARGET | SPFLAG_HELPFUL,
      6,  // lowered to 6 from 8, since it's easily available from various items
-        // and Swiftness is level 2 (and gives a similar effect).  It's also
-        // not that much better than Invisibility.  -- bwr
+         // and Swiftness is level 2 (and gives a similar effect).  It's also
+         // not that much better than Invisibility.  -- bwr
      200,
      NULL,
      false,
@@ -347,7 +380,19 @@
      4,
      200,
      NULL,
-     true
+     true,
+     false
+},
+
+{
+    SPELL_PETRIFY, "Petrify",
+     SPTYP_ENCHANTMENT | SPTYP_TRANSMIGRATION,
+     SPFLAG_DIR_OR_TARGET,
+     4,
+     200,
+     NULL,
+     true,
+     false
 },
 
 {
@@ -357,7 +402,8 @@
      3,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -378,7 +424,8 @@
      2,
      50,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -388,82 +435,85 @@
      2,
      50,
      NULL,
-     true
+     true,
+     false
 },
 
 {
     SPELL_CONTROLLED_BLINK, "Controlled Blink",
      SPTYP_TRANSLOCATION,
-     SPFLAG_NONE,
+     SPFLAG_ESCAPE,
      7,
      0,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_FREEZING_CLOUD, "Freezing Cloud",
      SPTYP_CONJURATION | SPTYP_ICE | SPTYP_AIR,
-     SPFLAG_GRID,
+     SPFLAG_GRID | SPFLAG_AREA,
      7,
      200,
      "Where do you want to put it?",
-     true
+     true,
+     false
 },
 
 {
     SPELL_MEPHITIC_CLOUD, "Mephitic Cloud",
      SPTYP_CONJURATION | SPTYP_POISON | SPTYP_AIR,
-     SPFLAG_DIR_OR_TARGET,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_AREA,
      3,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
     SPELL_RING_OF_FLAMES, "Ring of Flames",
      SPTYP_ENCHANTMENT | SPTYP_FIRE,
-     SPFLAG_NONE,
+     SPFLAG_AREA,
      8,
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_RESTORE_STRENGTH, "Restore Strength",
      SPTYP_HOLY,
-     SPFLAG_NONE,
+     SPFLAG_RECOVERY | SPFLAG_HELPFUL,
      2,
      0,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_RESTORE_INTELLIGENCE, "Restore Intelligence",
      SPTYP_HOLY,
-     SPFLAG_NONE,
+     SPFLAG_RECOVERY | SPFLAG_HELPFUL,
      2,
      0,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_RESTORE_DEXTERITY, "Restore Dexterity",
      SPTYP_HOLY,
-     SPFLAG_NONE,
+     SPFLAG_RECOVERY | SPFLAG_HELPFUL,
      2,
      0,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -473,33 +523,36 @@
      5,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
     SPELL_OLGREBS_TOXIC_RADIANCE, "Olgreb's Toxic Radiance",
      SPTYP_POISON,
-     SPFLAG_NONE,
+     SPFLAG_AREA,
      4,
      0,
      NULL,
-     true
+     true,
+     false
 },
 
 {
     SPELL_TELEPORT_OTHER, "Teleport Other",
      SPTYP_TRANSLOCATION,
-     SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_NOT_SELF | SPFLAG_ESCAPE,
      4,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
     SPELL_LESSER_HEALING, "Lesser Healing",
      SPTYP_HOLY,
-     SPFLAG_NONE,
+     SPFLAG_RECOVERY | SPFLAG_HELPFUL,
      2,
      0,
      NULL,
@@ -510,34 +563,34 @@
 {
     SPELL_GREATER_HEALING, "Greater Healing",
      SPTYP_HOLY,
-     SPFLAG_NONE,
+     SPFLAG_RECOVERY | SPFLAG_HELPFUL,
      6,
      0,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_CURE_POISON_I, "Cure Poison",
      SPTYP_HOLY,
-     SPFLAG_NONE,
+     SPFLAG_RECOVERY | SPFLAG_HELPFUL,
      3,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_PURIFICATION, "Purification",
      SPTYP_HOLY,
-     SPFLAG_NONE,
+     SPFLAG_RECOVERY | SPFLAG_HELPFUL,
      5,
      0,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -548,7 +601,7 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -559,18 +612,18 @@
      0,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_MASS_CONFUSION, "Mass Confusion",
      SPTYP_ENCHANTMENT,
-     SPFLAG_NONE,
+     SPFLAG_AREA,
      6,
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -581,29 +634,29 @@
      200,
      "Smite whom?",
      false,
-    false
+     false
 },
 
 {
     SPELL_REPEL_UNDEAD, "Repel Undead",
      SPTYP_HOLY,
-     SPFLAG_NONE,
+     SPFLAG_AREA,
      3,
      0,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_HOLY_WORD, "Holy Word",
      SPTYP_HOLY,
-     SPFLAG_NONE,
+     SPFLAG_AREA,
      7,
      0,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -614,29 +667,29 @@
      0,
      NULL,
      false,
-    false
+     true
 },
 
 {
-    SPELL_SUMMON_SMALL_MAMMAL, "Summon Small Mammals",
+    SPELL_SUMMON_SMALL_MAMMALS, "Summon Small Mammals",
      SPTYP_SUMMONING,
      SPFLAG_NONE,
      1,
      80,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_ABJURATION_I, "Abjuration",
      SPTYP_SUMMONING,
-     SPFLAG_NONE,
+     SPFLAG_AREA | SPFLAG_NEUTRAL,
      3,
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -647,7 +700,7 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -658,7 +711,7 @@
      150,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -668,7 +721,8 @@
      6,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -678,7 +732,8 @@
      8,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -688,28 +743,30 @@
      3,
      1000,
      NULL,
-     true
+     true,
+     false
 },
 
 {
     SPELL_POISONOUS_CLOUD, "Poisonous Cloud",
      SPTYP_CONJURATION | SPTYP_POISON | SPTYP_AIR,
-     SPFLAG_GRID,
+     SPFLAG_GRID | SPFLAG_AREA,
      6,
      200,
      "Where do you want to put it?",
-     true
-}
-,
+     true,
+     false
+},
 
 {
     SPELL_FIRE_STORM, "Fire Storm",
      SPTYP_CONJURATION | SPTYP_FIRE,
-     SPFLAG_GRID,
+     SPFLAG_GRID | SPFLAG_AREA,
      9,
      200,
      "Where?",
-     true
+     true,
+     false
 },
 
 {
@@ -720,13 +777,13 @@
      50,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_BLINK, "Blink",
      SPTYP_TRANSLOCATION,
-     SPFLAG_NONE,
+     SPFLAG_ESCAPE,
      2,
      0,
      NULL,
@@ -734,7 +791,6 @@
      true
 },
 
-
 // The following name was found in the hack.exe file of an early version
 // of PCHACK - credit goes to its creator (whoever that may be):
 {
@@ -744,18 +800,19 @@
      4,
      100,
      NULL,
-     true
+     true,
+     false
 },
 
 {
-    SPELL_SWARM, "Summon Swarm",
+    SPELL_SUMMON_SWARM, "Summon Swarm",
      SPTYP_SUMMONING,
      SPFLAG_NONE,
      6,
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -766,7 +823,7 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -776,7 +833,8 @@
      4,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -787,7 +845,7 @@
      45,
      NULL,
      false,
-     false
+     true
 },
 
 {
@@ -798,13 +856,13 @@
      100,
      NULL,
      true,
-     false
+     true
 },
 
 {
-    SPELL_ANIMATE_DEAD, "Animate Dead",
+   SPELL_ANIMATE_DEAD, "Animate Dead",
      SPTYP_NECROMANCY,
-     SPFLAG_NONE,
+     SPFLAG_AREA | SPFLAG_NEUTRAL,
      4,
      0,
      NULL,
@@ -819,7 +877,8 @@
      1,
      25,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -830,7 +889,7 @@
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -851,9 +910,9 @@
      0,
      NULL,
      false,
-    false
+     true
 },
-  
+
 {
     SPELL_VAMPIRIC_DRAINING, "Vampiric Draining",
      SPTYP_NECROMANCY,
@@ -862,7 +921,7 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -873,7 +932,7 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -884,7 +943,7 @@
      50,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -895,29 +954,29 @@
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_BURN, "Burn", // used by wanderers
      SPTYP_FIRE,
-     SPFLAG_NONE,
+     SPFLAG_DIR | SPFLAG_NOT_SELF,
      1,
      25,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_FREEZE, "Freeze",
      SPTYP_ICE,
-     SPFLAG_NONE,
+     SPFLAG_DIR | SPFLAG_NOT_SELF,
      1,
      25,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -928,17 +987,18 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_OZOCUBUS_REFRIGERATION, "Ozocubu's Refrigeration",
      SPTYP_ICE,
-     SPFLAG_NONE,
+     SPFLAG_AREA,
      5,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -948,7 +1008,8 @@
      4,
      100,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -959,7 +1020,7 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -970,7 +1031,7 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -981,7 +1042,7 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -992,7 +1053,7 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1003,7 +1064,7 @@
      0,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1013,18 +1074,19 @@
      4,
      100,
      NULL,
-     true
+     true,
+     false
 },
 
 {
-    SPELL_GUARDIAN, "Guardian",
+    SPELL_SUMMON_GUARDIAN, "Summon Guardian",
      SPTYP_HOLY,
      SPFLAG_NONE,
      7,
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1035,7 +1097,7 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1045,7 +1107,8 @@
      6,         // why is this the only holy spell with a secondary? {dlb}
      200,
      NULL,
-     true
+     true,
+     false
 }
 ,
 
@@ -1056,7 +1119,8 @@
      8,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -1067,7 +1131,7 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1078,18 +1142,40 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_ABJURATION_II, "Abjuration",
      SPTYP_HOLY,
-     SPFLAG_NONE,
+     SPFLAG_AREA | SPFLAG_NEUTRAL,
      4,
      200,
      NULL,
      false,
-    false
+     false
+},
+
+{
+    SPELL_FULSOME_DISTILLATION, "Fulsome Distillation",
+     SPTYP_TRANSMIGRATION | SPTYP_NECROMANCY,
+     SPFLAG_NONE,
+     1,
+     50,
+     NULL,
+     false,
+     true
+},
+
+{
+    SPELL_POISON_ARROW, "Poison Arrow",
+     SPTYP_CONJURATION | SPTYP_POISON,
+     SPFLAG_DIR_OR_TARGET,
+     6,
+     200,
+     NULL,
+     true,
+     false
 },
 
 {
@@ -1100,7 +1186,7 @@
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1111,7 +1197,7 @@
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1121,17 +1207,19 @@
      3,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
     SPELL_BANISHMENT, "Banishment",
      SPTYP_TRANSLOCATION,
      SPFLAG_DIR_OR_TARGET | SPFLAG_UNHOLY,
-     5,
+     4,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -1142,7 +1230,7 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1152,7 +1240,8 @@
      1,
      25,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -1163,7 +1252,7 @@
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1174,7 +1263,7 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1184,7 +1273,8 @@
      9,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -1195,7 +1285,7 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1206,7 +1296,7 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1217,18 +1307,18 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_CORPSE_ROT, "Corpse Rot",
      SPTYP_NECROMANCY,
-     SPFLAG_NONE,
+     SPFLAG_AREA | SPFLAG_NEUTRAL,
      2,
      0,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1239,7 +1329,7 @@
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1250,7 +1340,7 @@
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1261,7 +1351,7 @@
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1272,7 +1362,7 @@
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1283,7 +1373,7 @@
      25,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1293,7 +1383,8 @@
      6,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -1303,7 +1394,8 @@
      3,
      50,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -1314,7 +1406,7 @@
      0,
      NULL,
      false,
-    false
+     false
 }
 ,
 
@@ -1326,7 +1418,7 @@
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1336,7 +1428,8 @@
      1,
      25,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -1347,18 +1440,18 @@
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
-    SPELL_FLY, "Fly",
+    SPELL_FLY, "Flight",
      SPTYP_ENCHANTMENT | SPTYP_AIR,
      SPFLAG_NONE,
      4,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1369,7 +1462,7 @@
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1379,7 +1472,8 @@
      7,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -1390,64 +1484,62 @@
      60,                        // not 50, note the fuzz
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_CURE_POISON_II, "Cure Poison",
      SPTYP_POISON,
-     SPFLAG_NONE,
+     SPFLAG_RECOVERY | SPFLAG_HELPFUL,
      2,
      200,
      NULL,
      false,
-    false
-}
-,
+     true
+},
 
 {
     SPELL_CONTROL_TELEPORT, "Control Teleport",
      SPTYP_ENCHANTMENT | SPTYP_TRANSLOCATION,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      4,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_POISON_AMMUNITION, "Poison Ammunition",
      SPTYP_ENCHANTMENT | SPTYP_POISON,
-     SPFLAG_NONE,
-     4, // jmf: SPTYP_TRANSMIGRATION vs SPTYP_ENCHANTMENT?
-     0,
-     NULL,
-     false,
-    false
-}
-,
-
-{
-    SPELL_POISON_WEAPON, "Poison Weapon",
-     SPTYP_ENCHANTMENT | SPTYP_POISON,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      4,
      0,
      NULL,
      false,
-    false
+     true
+},
+
+{
+    SPELL_POISON_WEAPON, "Poison Weapon",
+     SPTYP_ENCHANTMENT | SPTYP_POISON,
+     SPFLAG_HELPFUL,
+     4,
+     0,
+     NULL,
+     false,
+     true
 },
 
 {
     SPELL_RESIST_POISON, "Resist Poison",
      SPTYP_ENCHANTMENT | SPTYP_POISON,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      4,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1458,7 +1550,7 @@
      0,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1469,7 +1561,7 @@
      0,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1480,7 +1572,7 @@
      100,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1491,18 +1583,18 @@
      0,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_PORTAL, "Portal",
      SPTYP_TRANSLOCATION,
-     SPFLAG_NONE,
+     SPFLAG_ESCAPE,
      7,
      0,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1513,18 +1605,18 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_SPIDER_FORM, "Spider Form",
      SPTYP_TRANSMIGRATION | SPTYP_POISON,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      3,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1535,7 +1627,7 @@
      25,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1545,95 +1637,96 @@
      6,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
     SPELL_BLADE_HANDS, "Blade Hands",
      SPTYP_TRANSMIGRATION,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      5,  // only removes weapon, so I raised this from 4 -- bwr
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_STATUE_FORM, "Statue Form",
      SPTYP_TRANSMIGRATION | SPTYP_EARTH,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      6,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_ICE_FORM, "Ice Form",
      SPTYP_ICE | SPTYP_TRANSMIGRATION,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      4, // doesn't allow for equipment, so I lowered this from 5 -- bwr
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_DRAGON_FORM, "Dragon Form",
      SPTYP_FIRE | SPTYP_TRANSMIGRATION,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      8,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_NECROMUTATION, "Necromutation",
      SPTYP_TRANSMIGRATION | SPTYP_NECROMANCY,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      8,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_DEATH_CHANNEL, "Death Channel",
      SPTYP_NECROMANCY,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      9,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_SYMBOL_OF_TORMENT, "Symbol of Torment",
      SPTYP_NECROMANCY,
-     SPFLAG_NONE,
+     SPFLAG_AREA,
      6,
      0,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_DEFLECT_MISSILES, "Deflect Missiles",
      SPTYP_ENCHANTMENT | SPTYP_AIR,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      6,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1659,22 +1752,23 @@
 {
     SPELL_ICE_STORM, "Ice Storm",
      SPTYP_CONJURATION | SPTYP_ICE,
-     SPFLAG_DIR_OR_TARGET,
+     SPFLAG_DIR_OR_TARGET | SPFLAG_AREA,
      9,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
     SPELL_ARC, "Arc",
      SPTYP_AIR,
-     SPFLAG_NONE,
+     SPFLAG_DIR | SPFLAG_NOT_SELF,
      1,
      25,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1685,19 +1779,19 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_SHADOW_CREATURES, "Shadow Creatures",
-     SPTYP_SUMMONING,  // jmf: or SPTYP_SUMMONING | SPTYP_CONJURATION
-     SPFLAG_NONE,
+     SPTYP_SUMMONING, // jmf: or SPTYP_SUMMONING | SPTYP_CONJURATION
+     SPFLAG_UNHOLY,
      5,
      0,
      NULL,
+     false,
      false
-}
-,
+},
 
 {
     SPELL_CONFUSING_TOUCH, "Confusing Touch",
@@ -1707,25 +1801,20 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_SURE_BLADE, "Sure Blade",
      SPTYP_ENCHANTMENT,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      2,
      200,
      NULL,
      false,
-    false
+     true
 },
 
-
-
-  //jmf: new spells
-
-
 {
     SPELL_FLAME_TONGUE, "Flame Tongue",
      SPTYP_CONJURATION | SPTYP_FIRE,
@@ -1734,29 +1823,29 @@
      25,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_PASSWALL, "Passwall",
      SPTYP_TRANSMIGRATION | SPTYP_EARTH,
-     SPFLAG_NONE,
+     SPFLAG_ESCAPE,
      3,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_IGNITE_POISON, "Ignite Poison",
      SPTYP_FIRE | SPTYP_TRANSMIGRATION,
-     SPFLAG_NONE,
-     7,
+     SPFLAG_AREA,
+     6,
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1767,18 +1856,18 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
-    SPELL_SUMMON_LARGE_MAMMAL, "Call Canine Familiar",
+    SPELL_CALL_CANINE_FAMILIAR, "Call Canine Familiar",
      SPTYP_SUMMONING,
      SPFLAG_NONE,
      3,
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1789,18 +1878,18 @@
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_TAME_BEASTS, "Tame Beasts",
      SPTYP_ENCHANTMENT,
-     SPFLAG_NONE,
+     SPFLAG_AREA,
      5,
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1810,18 +1899,19 @@
      2,
      56,
      NULL,
-     true
+     true,
+     false
 },
 
 {
     SPELL_MASS_SLEEP, "Metabolic Englaciation",
      SPTYP_ENCHANTMENT | SPTYP_ICE,
-     SPFLAG_NONE,
+     SPFLAG_AREA,
      7,
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1832,7 +1922,7 @@
      0,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1843,29 +1933,29 @@
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_SEE_INVISIBLE, "See Invisible",
      SPTYP_ENCHANTMENT | SPTYP_DIVINATION,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      4,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_FORESCRY, "Forescry",
      SPTYP_DIVINATION,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      5,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -1876,84 +1966,73 @@
      200,
      NULL,
      false,
-    false
-},
-
-{
-    SPELL_EXCRUCIATING_WOUNDS, "Excruciating Wounds",
-     SPTYP_ENCHANTMENT | SPTYP_NECROMANCY,
-     SPFLAG_NONE,
-     5,     // fairly high level - potentially one of the best brands
-     200,
-     NULL,
-     false,
-    false
+     false
 },
 
 {
     SPELL_WARP_BRAND, "Warp Weapon",
      SPTYP_ENCHANTMENT | SPTYP_TRANSLOCATION,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      7,     // this is high for a reason - Warp brands are very powerful.
      0,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_SILENCE, "Silence",
      SPTYP_ENCHANTMENT | SPTYP_AIR,
-     SPFLAG_NONE,
+     SPFLAG_AREA,
      5,
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_SHATTER, "Shatter",
      SPTYP_TRANSMIGRATION | SPTYP_EARTH,
-     SPFLAG_NONE,
+     SPFLAG_AREA,
      9,
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_DISPERSAL, "Dispersal",
      SPTYP_TRANSLOCATION,
-     SPFLAG_NONE,
+     SPFLAG_AREA | SPFLAG_ESCAPE,
      7,
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_DISCHARGE, "Static Discharge",
      SPTYP_CONJURATION | SPTYP_AIR,
-     SPFLAG_NONE,
+     SPFLAG_AREA,
      4,
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_BEND, "Bend",
      SPTYP_TRANSLOCATION,
-     SPFLAG_NONE,
+     SPFLAG_DIR,
      1,
      100,
      NULL,
      false,
-    false
+     false
 },
 
 {
@@ -1963,7 +2042,8 @@
      1,
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -1974,26 +2054,18 @@
      0,
      NULL,
      false,
-    false
-},
-
-{
-    SPELL_GLAMOUR, "Glamour",
-     SPTYP_ENCHANTMENT,
-     SPFLAG_NONE,
-     5,
-     200,
-     NULL
+     false
 },
 
 {
     SPELL_EVAPORATE, "Evaporate",
      SPTYP_FIRE | SPTYP_TRANSMIGRATION,
-     SPFLAG_NONE,
+     SPFLAG_DIR_OR_TARGET,
      2,   // XXX: level 2 or 3, what should it be now? -- bwr
      200,
      NULL,
-     true
+     true,
+     false
 },
 
 {
@@ -2004,29 +2076,29 @@
      0,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_FRAGMENTATION, "Lee's Rapid Deconstruction",
      SPTYP_TRANSMIGRATION | SPTYP_EARTH,
-     SPFLAG_NONE,
+     SPFLAG_GRID,
      5,
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_AIR_WALK, "Air Walk",
      SPTYP_TRANSMIGRATION | SPTYP_AIR,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      9,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
@@ -2036,148 +2108,129 @@
      1,
      50,
      NULL,
-     true
+     true,
+     false
 },
 
 {
     SPELL_ROTTING, "Rotting",
      SPTYP_TRANSMIGRATION | SPTYP_NECROMANCY,
-     SPFLAG_NONE,
+     SPFLAG_AREA,
      5,
      200,
      NULL,
      false,
-    false
+     false
 },
 
 {
     SPELL_MAXWELLS_SILVER_HAMMER, "Maxwell's Silver Hammer",
      SPTYP_TRANSMIGRATION | SPTYP_EARTH,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      2,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_CONDENSATION_SHIELD, "Condensation Shield",
      SPTYP_ICE | SPTYP_TRANSMIGRATION,
-     SPFLAG_NONE,
+     SPFLAG_HELPFUL,
      4,
      200,
      NULL,
      false,
-    false
+     true
 },
 
 {
     SPELL_SEMI_CONTROLLED_BLINK, "Semi-Controlled Blink",
      SPTYP_TRANSLOCATION,
-     SPFLAG_NONE,
+     SPFLAG_ESCAPE,
      3,
      100,
      NULL,
      false,
-    false
+     true
 },
 
 {
-  SPELL_STONESKIN, "Stoneskin",
-    SPTYP_EARTH | SPTYP_TRANSMIGRATION, // was ench -- bwr
-    SPFLAG_NONE,
-    2,
-    200,
+    SPELL_STONESKIN, "Stoneskin",
+     SPTYP_EARTH | SPTYP_TRANSMIGRATION, // was ench -- bwr
+     SPFLAG_HELPFUL,
+     2,
+     200,
      NULL,
      false,
-    false
+     true
 },
 
 {
-  SPELL_SIMULACRUM, "Simulacrum",
-    SPTYP_ICE | SPTYP_NECROMANCY,
-    SPFLAG_NONE,
-    6,
-    200,
+    SPELL_SIMULACRUM, "Simulacrum",
+     SPTYP_ICE | SPTYP_NECROMANCY,
+     SPFLAG_NONE,
+     6,
+     200,
      NULL,
      false,
-    false
+     false
 },
 
 {
-  SPELL_CONJURE_BALL_LIGHTNING, "Conjure Ball Lightning",
-    SPTYP_AIR | SPTYP_CONJURATION,
-    SPFLAG_NONE,
-    8,
-    200,
+    SPELL_CONJURE_BALL_LIGHTNING, "Conjure Ball Lightning",
+     SPTYP_AIR | SPTYP_CONJURATION,
+     SPFLAG_NONE,
+     7,
+     200,
      NULL,
      false,
-    false
+     false
 },
 
 {
-  SPELL_CHAIN_LIGHTNING, "Chain Lightning",
-    SPTYP_AIR | SPTYP_CONJURATION,
-    SPFLAG_NONE,
-    8,
-    200,
+    SPELL_CHAIN_LIGHTNING, "Chain Lightning",
+     SPTYP_AIR | SPTYP_CONJURATION,
+     SPFLAG_AREA,
+     8,
+     200,
      NULL,
      false,
-    false
+     false
 },
 
 {
-  SPELL_DELAYED_FIREBALL, "Delayed Fireball",
-    SPTYP_FIRE | SPTYP_CONJURATION,
-    SPFLAG_NONE,
-    7,
-    0,
+    SPELL_EXCRUCIATING_WOUNDS, "Excruciating Wounds",
+     SPTYP_ENCHANTMENT | SPTYP_NECROMANCY,
+     SPFLAG_HELPFUL,
+     5,     // fairly high level - potentially one of the best brands
+     200,
      NULL,
      false,
-    false
+     false
 },
 
 {
-  SPELL_FULSOME_DISTILLATION, "Fulsome Distillation",
-    SPTYP_TRANSMIGRATION | SPTYP_NECROMANCY,
-    SPFLAG_NONE,
-    1,
-    50,
+    SPELL_PORTAL_PROJECTILE, "Portal Projectile",
+     SPTYP_TRANSLOCATION,
+     SPFLAG_TARGET,
+     2,
+     50,
      NULL,
      false,
-    false
+     false
 },
 
 {
-  SPELL_POISON_ARROW, "Poison Arrow",
-    SPTYP_CONJURATION | SPTYP_POISON,
-    SPFLAG_DIR_OR_TARGET,
-    6,
-    200,
-    NULL,
-    true
-},
-
-{
-    SPELL_PORTALED_PROJECTILE, "Portaled Projectile",
-        SPTYP_TRANSLOCATION,
-        SPFLAG_TARGET,
-        2,
-        50,
-        NULL,
-        false,
-        false
-},
-
-{
-  SPELL_STRIKING, "Striking",
-    0,
-    SPFLAG_DIR_OR_TARGET,
-    1,
-    25,
-    NULL,
-    true
+    SPELL_SUMMON_UGLY_THING, "Summon Ugly Thing",
+     SPTYP_SUMMONING,
+     SPFLAG_NONE,
+     5,
+     200,
+     NULL,
+     false,
+     false
 },
 
 {
@@ -2188,174 +2241,183 @@
      200,
      NULL,
      false,
-    false 
+     false
 },
 
 {
     SPELL_VAMPIRE_SUMMON, "Vampire Summon",
-    SPTYP_SUMMONING,
-    SPFLAG_UNHOLY,
-    3,
-    0,
-    NULL,
-    false,
-    false
+     SPTYP_SUMMONING,
+     SPFLAG_UNHOLY,
+     3,
+     0,
+     NULL,
+     false,
+     false
 },
 
 {
     SPELL_BRAIN_FEED, "Brain Feed",
-    SPTYP_NECROMANCY,
-    SPFLAG_UNHOLY,
-    3,
-    0,
-    NULL,
-    false,
-    false
+     SPTYP_NECROMANCY,
+     SPFLAG_UNHOLY | SPFLAG_TARGET,
+     3,
+     0,
+     NULL,
+     false,
+     false
 },
 
 {
     SPELL_FAKE_RAKSHASA_SUMMON, "Rakshasa Summon",
-    SPTYP_SUMMONING | SPTYP_NECROMANCY,
-    SPFLAG_UNHOLY,
-    3,
-    0,
-    NULL,
-    false
+     SPTYP_SUMMONING | SPTYP_NECROMANCY,
+     SPFLAG_UNHOLY,
+     3,
+     0,
+     NULL,
+     false,
+     false
 },
 
 {
     SPELL_STEAM_BALL, "Steam Ball",
-    SPTYP_CONJURATION | SPTYP_FIRE,
-    SPFLAG_DIR_OR_TARGET,
-    4,
-    0,
-    NULL,
-    true
+     SPTYP_CONJURATION | SPTYP_FIRE,
+     SPFLAG_DIR_OR_TARGET,
+     4,
+     0,
+     NULL,
+     true,
+     false
 },
 
 {
     SPELL_SUMMON_UFETUBUS, "Summon Ufetubus",
-    SPTYP_SUMMONING,
-    SPFLAG_UNHOLY,
-    4,
-    0,
-    NULL,
-    false,
-    false
+     SPTYP_SUMMONING,
+     SPFLAG_UNHOLY,
+     4,
+     0,
+     NULL,
+     false,
+     false
 },
 
 {
     SPELL_SUMMON_BEAST, "Summon Beast",
-    SPTYP_SUMMONING,
-    SPFLAG_NONE,
-    4,
-    0,
-    NULL,
-    false
+     SPTYP_SUMMONING,
+     SPFLAG_UNHOLY,
+     4,
+     0,
+     NULL,
+     false,
+     false
 },
 
 {
     SPELL_ENERGY_BOLT, "Energy Bolt",
-    SPTYP_CONJURATION,
-    SPFLAG_DIR_OR_TARGET,
-    4,
-    0,
-    NULL,
-    true
+     SPTYP_CONJURATION,
+     SPFLAG_DIR_OR_TARGET,
+     4,
+     0,
+     NULL,
+     true,
+     false
 },
 
 {
     SPELL_POISON_SPLASH, "Poison Splash",
-    SPTYP_POISON,
-    SPFLAG_DIR_OR_TARGET,
-    2,
-    0,
-    NULL,
-    true
+     SPTYP_POISON,
+     SPFLAG_DIR_OR_TARGET,
+     2,
+     0,
+     NULL,
+     true,
+     false
 },
 
 {
     SPELL_SUMMON_UNDEAD, "Summon Undead",
-    SPTYP_SUMMONING | SPTYP_NECROMANCY,
-    SPFLAG_NONE,
-    7,
-    0,
-    NULL,
-    false,
-    false,
+     SPTYP_SUMMONING | SPTYP_NECROMANCY,
+     SPFLAG_NONE,
+     7,
+     0,
+     NULL,
+     false,
+     false,
 },
 
 {
     SPELL_CANTRIP, "Cantrip",
-    SPTYP_NONE,
-    SPFLAG_NONE,
-    1,
-    0,
-    NULL,
-    false,
-    false
+     SPTYP_NONE,
+     SPFLAG_NONE,
+     1,
+     0,
+     NULL,
+     false,
+     false
 },
 
 {
     SPELL_QUICKSILVER_BOLT, "Quicksilver Bolt",
-    SPTYP_CONJURATION,
-    SPFLAG_DIR_OR_TARGET,
-    5,
-    0,
-    NULL,
-    true
+     SPTYP_CONJURATION,
+     SPFLAG_DIR_OR_TARGET,
+     5,
+     0,
+     NULL,
+     true,
+     false
 },
 
 {
     SPELL_METAL_SPLINTERS, "Metal Splinters",
-    SPTYP_CONJURATION,
-    SPFLAG_DIR_OR_TARGET,
-    5,
-    0,
-    NULL,
-    true
+     SPTYP_CONJURATION,
+     SPFLAG_DIR_OR_TARGET,
+     5,
+     0,
+     NULL,
+     true,
+     false
 },
 
 {
     SPELL_MIASMA, "Miasma",
-    SPTYP_CONJURATION | SPTYP_NECROMANCY,
-    SPFLAG_DIR_OR_TARGET,
-    6,
-    0,
-    NULL,
-    true
+     SPTYP_CONJURATION | SPTYP_NECROMANCY,
+     SPFLAG_DIR_OR_TARGET,
+     6,
+     0,
+     NULL,
+     true,
+     false
 },
 
 {
     SPELL_SUMMON_DRAKES, "Summon Drakes",
-    SPTYP_SUMMONING,
-    SPFLAG_NONE,
-    6,
-    0,
-    NULL,
-    false,
-    false
+     SPTYP_SUMMONING,
+     SPFLAG_NONE,
+     6,
+     0,
+     NULL,
+     false,
+     false
 },
 
 {
     SPELL_BLINK_OTHER, "Blink Other",
-    SPTYP_TRANSLOCATION,
-    SPFLAG_NONE,
-    2,
-    0,
-    NULL,
-    true
+     SPTYP_TRANSLOCATION,
+     SPFLAG_ESCAPE | SPFLAG_DIR_OR_TARGET,
+     2,
+     0,
+     NULL,
+     true,
+     false
 },
 
 {
     SPELL_SUMMON_MUSHROOMS, "Summon Mushrooms",
-    SPTYP_SUMMONING,
-    SPFLAG_NONE,
-    4,
-    0,
-    NULL,
-    false,
-    false
+     SPTYP_SUMMONING,
+     SPFLAG_NONE,
+     4,
+     0,
+     NULL,
+     false,
+     false
 },
 
 {
