@@ -3,13 +3,13 @@
  *  Summary:    data handlers for player-avilable spell list         *
  *  Written by: don brodale <dbrodale@bigfootinteractive.com>        *
  *                                                                   *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-11T13:21:31.315931Z $
  *
  *  Changelog(most recent first):                                    *
  *
  *  <3>     04oct2001     bwr     absorbed spells0.cc
  *  <2>     24jun2000     jmf     changed to use new data structure
- *  <1>     12jun2000     dlb     created after much thought         
+ *  <1>     12jun2000     dlb     created after much thought
  */
 
 #include "AppHdr.h"
@@ -24,7 +24,7 @@
 #include "externs.h"
 
 #include "beam.h"
-#include "direct.h"
+#include "directn.h"
 #include "debug.h"
 #include "stuff.h"
 #include "itemname.h"
@@ -44,7 +44,6 @@
 #include <conio.h>
 #endif
 
-
 static struct spell_desc spelldata[] = {
 #include "spl-data.h"
 };
@@ -53,28 +52,48 @@ static int spell_list[NUM_SPELLS];
 
 #define SPELLDATASIZE (sizeof(spelldata)/sizeof(struct spell_desc))
 
-static struct spell_desc *seekspell(spell_type spellid);
-static bool cloud_helper(int (*func)(int, int, int, cloud_type, kill_category),
-                         int x, int y, 
-                         int pow, cloud_type ctype, kill_category );
+static struct spell_desc *_seekspell(spell_type spellid);
+static bool _cloud_helper(int (*func)(int, int, int, int, cloud_type,
+                                      kill_category),
+                          int x, int y, int pow, int spread_rate,
+                          cloud_type ctype, kill_category );
 
-/*
- *             BEGIN PUBLIC FUNCTIONS
- */
+//
+//             BEGIN PUBLIC FUNCTIONS
+//
 
-// all this does is merely refresh the internal spell list {dlb}:
+// All this does is merely refresh the internal spell list {dlb}:
 void init_spell_descs(void)
 {
     for (int i = 0; i < NUM_SPELLS; i++)
         spell_list[i] = -1;
 
     // can only use up to SPELLDATASIZE _MINUS ONE_, or the
-    // last entry tries to set spell_list[SPELL_NO_SPELL] 
+    // last entry tries to set spell_list[SPELL_NO_SPELL]
     // which corrupts the heap.
     for (unsigned int i = 0; i < SPELLDATASIZE - 1; i++)
         spell_list[spelldata[i].id] = i;
 }                               // end init_spell_descs()
 
+typedef std::map<std::string, spell_type> spell_name_map;
+static spell_name_map spell_name_cache;
+
+void init_spell_name_cache()
+{
+    for (int i = 0; i < NUM_SPELLS; i++)
+    {
+        spell_type type = static_cast<spell_type>(i);
+
+        if (!is_valid_spell(type))
+            continue;
+
+        const char *sptitle = spell_title(type);
+        ASSERT(sptitle);
+        const std::string spell_name = lowercase_string(sptitle);
+        spell_name_cache[spell_name] = type;
+    }
+}
+
 spell_type spell_by_name(std::string name, bool partial_match)
 {
     if (name.empty())
@@ -82,24 +101,37 @@ spell_type spell_by_name(std::string name, bool partial_match)
 
     lowercase(name);
 
+    if (!partial_match)
+    {
+        spell_name_map::iterator i = spell_name_cache.find(name);
+
+        if (i != spell_name_cache.end())
+            return (i->second);
+
+        return (SPELL_NO_SPELL);
+    }
+
     int spellmatch = -1;
     for (int i = 0; i < NUM_SPELLS; i++)
     {
         spell_type type = static_cast<spell_type>(i);
-        const char *sptitle = spell_title(type);
-        if (!sptitle)
+        if (!is_valid_spell(type))
             continue;
 
+        const char *sptitle = spell_title(type);
         const std::string spell_name = lowercase_string(sptitle);
-        if (name == spell_name)
-            return (type);
 
-        if (partial_match && spell_name.find(name) != std::string::npos)
+        if (spell_name.find(name) != std::string::npos)
+        {
+            if (spell_name == name)
+                return static_cast<spell_type>(i);
+
             spellmatch = i;
+        }
     }
 
-    return (spellmatch != -1? static_cast<spell_type>(spellmatch)
-            : SPELL_NO_SPELL);
+    return (spellmatch != -1 ? static_cast<spell_type>(spellmatch)
+                             : SPELL_NO_SPELL);
 }
 
 int get_spell_slot_by_letter( char letter )
@@ -194,28 +226,63 @@ int spell_hunger(spell_type which_spell)
     return hunger;
 }
 
+// Used to determine whether or not a monster should always fire this spell
+// if selected.  If not, we should use a tracer.
+
+// Note - this function assumes that the monster is "nearby" its target!
 bool spell_needs_tracer(spell_type spell)
 {
-    return (seekspell(spell)->ms_needs_tracer);
+    return (_seekspell(spell)->ms_needs_tracer);
 }
 
 bool spell_needs_foe(spell_type spell)
 {
-    return (!seekspell(spell)->ms_utility);
+    return (!_seekspell(spell)->ms_utility);
+}
+
+bool spell_harms_target(spell_type spell)
+{
+    const unsigned int flags = _seekspell(spell)->flags;
+
+    if (flags & (SPFLAG_HELPFUL | SPFLAG_NEUTRAL))
+        return false;
+
+    if (flags & SPFLAG_TARGETING_MASK)
+        return true;
+
+    return false;
+}
+
+bool spell_harms_area(spell_type spell)
+{
+    const unsigned int flags = _seekspell(spell)->flags;
+
+    if (flags & (SPFLAG_HELPFUL | SPFLAG_NEUTRAL))
+        return false;
+
+    if (flags & SPFLAG_AREA)
+        return true;
+
+    return false;
+}
+
+bool spell_sanctuary_castable(spell_type spell)
+{
+    return false;
 }
 
 // applied to spell misfires (more power = worse) and triggers
 // for Xom acting (more power = more likely to grab his attention) {dlb}
 int spell_mana(spell_type which_spell)
 {
-    return (seekspell(which_spell)->level);
+    return (_seekspell(which_spell)->level);
 }
 
 // applied in naughties (more difficult = higher level knowledge = worse)
 // and triggers for Sif acting (same reasoning as above, just good) {dlb}
 int spell_difficulty(spell_type which_spell)
 {
-    return (seekspell(which_spell)->level);
+    return (_seekspell(which_spell)->level);
 }
 
 int spell_levels_required( spell_type which_spell )
@@ -238,23 +305,23 @@ int spell_levels_required( spell_type which_spell )
 
 unsigned int get_spell_flags( spell_type which_spell )
 {
-    return (seekspell(which_spell)->flags);
+    return (_seekspell(which_spell)->flags);
 }
 
 const char *get_spell_target_prompt( spell_type which_spell )
 {
-    return (seekspell(which_spell)->target_prompt);
+    return (_seekspell(which_spell)->target_prompt);
 }
 
 bool spell_typematch(spell_type which_spell, unsigned int which_discipline)
 {
-    return (seekspell(which_spell)->disciplines & which_discipline);
+    return (_seekspell(which_spell)->disciplines & which_discipline);
 }
 
 //jmf: next two for simple bit handling
 unsigned int get_spell_disciplines(spell_type spell)
 {
-    return (seekspell(spell)->disciplines);
+    return (_seekspell(spell)->disciplines);
 }
 
 int count_bits(unsigned int bits)
@@ -263,18 +330,15 @@ int count_bits(unsigned int bits)
     int c = 0;
 
     for (n = 1; n < INT_MAX; n <<= 1)
-    {
         if (n & bits)
             c++;
-    }
 
     return (c);
 }
 
 const char *spell_title(spell_type spell)
 {
-    const spell_desc *spd = seekspell(spell);
-    return (spd? spd->title : NULL);
+    return (_seekspell(spell)->title);
 }
 
 
@@ -286,7 +350,8 @@ const char *spell_title(spell_type spell)
 
 // Apply a function-pointer to all visible squares
 // Returns summation of return values from passed in function.
-int apply_area_visible( int (*func) (int, int, int, int), int power )
+int apply_area_visible( int (*func) (int, int, int, int), int power,
+                        bool pass_through_trans)
 {
     int x, y;
     int rv = 0;
@@ -296,44 +361,40 @@ int apply_area_visible( int (*func) (int, int, int, int), int power )
     {
         for (y = you.y_pos - 8; y <= you.y_pos + 8; y++)
         {
-            if (see_grid(x, y))
+            if ((pass_through_trans && see_grid(x, y))
+                 || (!pass_through_trans && see_grid_no_trans(x, y)))
                 rv += func(x, y, power, 0);
         }
     }
 
     return (rv);
-}                               // end apply_area_visible()
+}
 
 // Applies the effect to all nine squares around/including the target.
 // Returns summation of return values from passed in function.
-int apply_area_square( int (*func) (int, int, int, int), int cx, int cy, 
+int apply_area_square( int (*func) (int, int, int, int), int cx, int cy,
                         int power )
 {
     int x, y;
     int rv = 0;
 
     for (x = cx - 1; x <= cx + 1; x++)
-    {
         for (y = cy - 1; y <= cy + 1; y++)
-        {
             rv += func(x, y, power, 0);
-        }
-    }
 
     return (rv);
-}                               // end apply_area_square()
+}
 
 
 // Applies the effect to the eight squares beside the target.
 // Returns summation of return values from passed in function.
-int apply_area_around_square( int (*func) (int, int, int, int), 
+int apply_area_around_square( int (*func) (int, int, int, int),
                               int targ_x, int targ_y, int power)
 {
     int x, y;
     int rv = 0;
 
     for (x = targ_x - 1; x <= targ_x + 1; x++)
-    {
         for (y = targ_y - 1; y <= targ_y + 1; y++)
         {
             if (x == targ_x && y == targ_y)
@@ -341,14 +402,14 @@ int apply_area_around_square( int (*func) (int, int, int, int),
             else
                 rv += func(x, y, power, 0);
         }
-    }
+
     return (rv);
-}                               // end apply_area_around_square()
+}
 
 // Effect up to max_targs monsters around a point, chosen randomly
 // Return varies with the function called;  return values will be added up.
 int apply_random_around_square( int (*func) (int, int, int, int),
-                                int targ_x, int targ_y, 
+                                int targ_x, int targ_y,
                                 bool hole_in_middle, int power, int max_targs )
 {
     int rv = 0;
@@ -357,14 +418,10 @@ int apply_random_around_square( int (*func) (int, int, int, int),
         return 0;
 
     if (max_targs >= 9 && !hole_in_middle)
-    {
         return (apply_area_square( func, targ_x, targ_y, power ));
-    }
 
     if (max_targs >= 8 && hole_in_middle)
-    {
         return (apply_area_around_square( func, targ_x, targ_y, power ));
-    }
 
     FixedVector< coord_def, 8 > targs;
     int count = 0;
@@ -373,15 +430,15 @@ int apply_random_around_square( int (*func) (int, int, int, int),
     {
         for (int y = targ_y - 1; y <= targ_y + 1; y++)
         {
-            if (hole_in_middle && (x == targ_x && y == targ_y)) 
+            if (hole_in_middle && (x == targ_x && y == targ_y))
                 continue;
 
-            if (mgrd[x][y] == NON_MONSTER 
+            if (mgrd[x][y] == NON_MONSTER
                 && !(x == you.x_pos && y == you.y_pos))
             {
                 continue;
             }
-                
+
             // Found target
             count++;
 
@@ -457,9 +514,9 @@ int apply_random_around_square( int (*func) (int, int, int, int),
                 targs[ count - 1 ].x = x;
                 targs[ count - 1 ].y = y;
             }
-            else if (random2( count ) < max_targs)
+            else if (x_chance_in_y(max_targs, count))
             {
-                const int pick = random2( max_targs );  
+                const int pick = random2( max_targs );
                 targs[ pick ].x = x;
                 targs[ pick ].y = y;
             }
@@ -483,7 +540,7 @@ int apply_random_around_square( int (*func) (int, int, int, int),
     return (rv);
 }                               // end apply_random_around_square()
 
-// apply func to one square of player's choice beside the player
+// Apply func to one square of player's choice beside the player.
 int apply_one_neighbouring_square(int (*func) (int, int, int, int), int power)
 {
     struct dist bmove;
@@ -503,7 +560,7 @@ int apply_one_neighbouring_square(int (*func) (int, int, int, int), int power)
         canned_msg(MSG_NOTHING_HAPPENS);
 
     return (rv);
-}                               // end apply_one_neighbouring_square()
+}
 
 int apply_area_within_radius( int (*func) (int, int, int, int),
                               int x, int y, int pow, int radius, int ctype )
@@ -543,17 +600,19 @@ int apply_area_within_radius( int (*func) (int, int, int, int),
 // We really need some sort of a queue structure, since ideally I'd like
 // to do a (shallow) breadth-first-search of the dungeon floor.
 // This ought to work okay for small clouds.
-void apply_area_cloud( int (*func) (int, int, int, cloud_type, kill_category),
+void apply_area_cloud( int (*func) (int, int, int, int, cloud_type,
+                                    kill_category),
                        int x, int y,
                        int pow, int number, cloud_type ctype,
-                       kill_category whose )
+                       kill_category whose, int spread_rate )
 {
     int spread, clouds_left = number;
     int good_squares = 0, neighbours[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
     int dx = 1, dy = 1;
     bool x_first;
 
-    if (clouds_left && cloud_helper(func, x, y, pow, ctype, whose))
+    if (clouds_left && _cloud_helper(func, x, y, pow, spread_rate,
+                                     ctype, whose))
         clouds_left--;
 
     if (!clouds_left)
@@ -568,28 +627,32 @@ void apply_area_cloud( int (*func) (int, int, int, cloud_type, kill_category),
 
     if (x_first)
     {
-        if (clouds_left && cloud_helper(func, x + dx, y, pow, ctype, whose))
+        if (clouds_left && _cloud_helper(func, x + dx, y, pow, spread_rate,
+                                         ctype, whose))
         {
             clouds_left--;
             good_squares++;
             neighbours[0]++;
         }
 
-        if (clouds_left && cloud_helper(func, x - dx, y, pow, ctype, whose))
+        if (clouds_left && _cloud_helper(func, x - dx, y, pow, spread_rate,
+                                         ctype, whose))
         {
             clouds_left--;
             good_squares++;
             neighbours[1]++;
         }
 
-        if (clouds_left && cloud_helper(func, x, y + dy, pow, ctype, whose))
+        if (clouds_left && _cloud_helper(func, x, y + dy, pow, spread_rate,
+                                         ctype, whose))
         {
             clouds_left--;
             good_squares++;
             neighbours[2]++;
         }
 
-        if (clouds_left && cloud_helper(func, x, y - dy, pow, ctype, whose))
+        if (clouds_left && _cloud_helper(func, x, y - dy, pow, spread_rate,
+                                         ctype, whose))
         {
             clouds_left--;
             good_squares++;
@@ -598,28 +661,32 @@ void apply_area_cloud( int (*func) (int, int, int, cloud_type, kill_category),
     }
     else
     {
-        if (clouds_left && cloud_helper(func, x, y + dy, pow, ctype, whose))
+        if (clouds_left && _cloud_helper(func, x, y + dy, pow, spread_rate,
+                                         ctype, whose))
         {
             clouds_left--;
             good_squares++;
             neighbours[2]++;
         }
 
-        if (clouds_left && cloud_helper(func, x, y - dy, pow, ctype, whose))
+        if (clouds_left && _cloud_helper(func, x, y - dy, pow, spread_rate,
+                                         ctype, whose))
         {
             clouds_left--;
             good_squares++;
             neighbours[3]++;
         }
 
-        if (clouds_left && cloud_helper(func, x + dx, y, pow, ctype, whose))
+        if (clouds_left && _cloud_helper(func, x + dx, y, pow, spread_rate,
+                                         ctype, whose))
         {
             clouds_left--;
             good_squares++;
             neighbours[0]++;
         }
 
-        if (clouds_left && cloud_helper(func, x - dx, y, pow, ctype, whose))
+        if (clouds_left && _cloud_helper(func, x - dx, y, pow, spread_rate,
+                                         ctype, whose))
         {
             clouds_left--;
             good_squares++;
@@ -627,36 +694,40 @@ void apply_area_cloud( int (*func) (int, int, int, cloud_type, kill_category),
         }
     }
 
-    // now diagonals; we could randomize dx & dy again here
-    if (clouds_left && cloud_helper(func, x + dx, y + dy, pow, ctype, whose))
+    // Mow diagonals; we could randomize dx & dy again here.
+    if (clouds_left && _cloud_helper(func, x + dx, y + dy, pow, spread_rate,
+                                     ctype, whose))
     {
         clouds_left--;
         good_squares++;
         neighbours[4]++;
     }
 
-    if (clouds_left && cloud_helper(func, x - dx, y + dy, pow, ctype, whose))
+    if (clouds_left && _cloud_helper(func, x - dx, y + dy, pow, spread_rate,
+                                     ctype, whose))
     {
         clouds_left--;
         good_squares++;
         neighbours[5]++;
     }
 
-    if (clouds_left && cloud_helper(func, x + dx, y - dy, pow, ctype, whose))
+    if (clouds_left && _cloud_helper(func, x + dx, y - dy, pow, spread_rate,
+                                     ctype, whose))
     {
         clouds_left--;
         good_squares++;
         neighbours[6]++;
     }
 
-    if (clouds_left && cloud_helper(func, x - dx, y - dy, pow, ctype, whose))
+    if (clouds_left && _cloud_helper(func, x - dx, y - dy, pow, spread_rate,
+                                     ctype, whose))
     {
         clouds_left--;
         good_squares++;
         neighbours[7]++;
     }
 
-    if (!(clouds_left && good_squares))
+    if (!clouds_left || !good_squares)
         return;
 
     for (int i = 0; i < 8 && clouds_left; i++)
@@ -671,28 +742,36 @@ void apply_area_cloud( int (*func) (int, int, int, cloud_type, kill_category),
         switch (i)
         {
         case 0:
-            apply_area_cloud(func, x + dx, y, pow, spread, ctype, whose);
+            apply_area_cloud(func, x + dx, y, pow, spread, ctype, whose,
+                             spread_rate);
             break;
         case 1:
-            apply_area_cloud(func, x - dx, y, pow, spread, ctype, whose);
+            apply_area_cloud(func, x - dx, y, pow, spread, ctype, whose,
+                             spread_rate);
             break;
         case 2:
-            apply_area_cloud(func, x, y + dy, pow, spread, ctype, whose);
+            apply_area_cloud(func, x, y + dy, pow, spread, ctype, whose,
+                             spread_rate);
             break;
         case 3:
-            apply_area_cloud(func, x, y - dy, pow, spread, ctype, whose);
+            apply_area_cloud(func, x, y - dy, pow, spread, ctype, whose,
+                             spread_rate);
             break;
         case 4:
-            apply_area_cloud(func, x + dx, y + dy, pow, spread, ctype, whose);
+            apply_area_cloud(func, x + dx, y + dy, pow, spread, ctype, whose,
+                             spread_rate);
             break;
         case 5:
-            apply_area_cloud(func, x - dx, y + dy, pow, spread, ctype, whose);
+            apply_area_cloud(func, x - dx, y + dy, pow, spread, ctype, whose,
+                             spread_rate);
             break;
         case 6:
-            apply_area_cloud(func, x + dx, y - dy, pow, spread, ctype, whose);
+            apply_area_cloud(func, x + dx, y - dy, pow, spread, ctype, whose,
+                             spread_rate);
             break;
         case 7:
-            apply_area_cloud(func, x - dx, y - dy, pow, spread, ctype, whose);
+            apply_area_cloud(func, x - dx, y - dy, pow, spread, ctype, whose,
+                             spread_rate);
             break;
         }
     }
@@ -700,28 +779,30 @@ void apply_area_cloud( int (*func) (int, int, int, cloud_type, kill_category),
 
 // Select a spell direction and fill dist and pbolt appropriately.
 // Return false if the user canceled, true otherwise.
-bool spell_direction( dist &spelld, bolt &pbolt, 
+bool spell_direction( dist &spelld, bolt &pbolt,
                       targeting_type restrict, targ_mode_type mode,
-                      bool needs_path, const char *prompt )
+                      bool needs_path, bool may_target_monster,
+                      const char *prompt, bool cancel_at_self )
 {
     if (restrict != DIR_DIR)
         message_current_target();
 
-    direction( spelld, restrict, mode, false, needs_path, prompt );
+    direction( spelld, restrict, mode, -1, false, needs_path,
+               may_target_monster, prompt, NULL, cancel_at_self );
 
     if (!spelld.isValid)
     {
-        // check for user cancel
+        // Check for user cancel.
         canned_msg(MSG_OK);
-        return false;
+        return (false);
     }
 
     pbolt.set_target(spelld);
     pbolt.source_x = you.x_pos;
     pbolt.source_y = you.y_pos;
 
-    return true;
-}                               // end spell_direction()
+    return (true);
+}
 
 const char* spelltype_short_name( int which_spelltype )
 {
@@ -828,27 +909,34 @@ int spell_type2skill(unsigned int spelltype)
  **************************************************
  */
 
-//jmf: simplified; moved init code to top function, init_spell_descs()
-static spell_desc *seekspell(spell_type spell)
+//jmf: Simplified; moved init code to top function, init_spell_descs().
+static spell_desc *_seekspell(spell_type spell)
 {
     const int index = spell_list[spell];
-    return (index != -1? &spelldata[index] : NULL);
+    ASSERT(index != -1);
+    return (&spelldata[index]);
 }
 
-static bool cloud_helper(int (*func)(int, int, int, cloud_type, kill_category),
-                         int x, int y, 
-                         int pow, cloud_type ctype, kill_category whose )
+bool is_valid_spell(spell_type spell)
+{
+    return (spell < NUM_SPELLS && spell_list[spell] != -1);
+}
+
+static bool _cloud_helper(int (*func)(int, int, int, int, cloud_type,
+                                      kill_category),
+                          int x, int y, int pow, int spread_rate,
+                          cloud_type ctype, kill_category whose )
 {
     if (!grid_is_solid(grd[x][y]) && env.cgrid[x][y] == EMPTY_CLOUD)
     {
-        func(x, y, pow, ctype, whose);
-        return true;
+        func(x, y, pow, spread_rate, ctype, whose);
+        return (true);
     }
 
-    return false;
+    return (false);
 }
 
 int spell_power_cap(spell_type spell)
 {
-    return seekspell(spell)->power_cap;
+    return (_seekspell(spell)->power_cap);
 }
