@@ -3,7 +3,7 @@
  *  Summary:    data handlers for player spell list
  *  Written by: don brodale <dbrodale@bigfootinteractive.com>
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-11T13:21:31.315931Z $
  *
  *  Changelog(most recent first):
  *
@@ -16,7 +16,7 @@
 #define SPL_UTIL_H
 
 #include "enum.h"    // just for NUM_SPELL_TYPES and TARG_ENEMY
-#include "direct.h"  // just for DIR_NONE 
+#include "directn.h"  // just for DIR_NONE
 
 enum spschool_flag_type
 {
@@ -61,9 +61,12 @@ struct spell_desc
 };
 
 
+bool is_valid_spell(spell_type spell);
+
 //* * called from: acr
 void init_spell_descs(void);
 
+void       init_spell_name_cache();
 spell_type spell_by_name(std::string name, bool partial_match = false);
 
 int get_spell_slot_by_letter( char letter );
@@ -87,6 +90,9 @@ const char *get_spell_target_prompt( spell_type which_spell );
 
 bool spell_needs_tracer(spell_type spell);
 bool spell_needs_foe(spell_type spell);
+bool spell_harms_target(spell_type spell);
+bool spell_harms_area(spell_type spell);
+bool spell_sanctuary_castable(spell_type spell);
 int spell_levels_required(spell_type which_spell);
 
 unsigned int get_spell_flags( spell_type which_spell );
@@ -103,7 +109,8 @@ const char* spelltype_short_name( int which_spelltype );
 
 //int spell_restriction(int which_spell, int which_restriction);
 
-int apply_area_visible(int (*func) (int, int, int, int), int power);
+int apply_area_visible(int (*func) (int, int, int, int), int power,
+                       bool pass_through_trans = false);
 
 int apply_area_square(int (*func) (int, int, int, int),
                               int cx, int cy, int power);
@@ -112,7 +119,7 @@ int apply_area_around_square( int (*func) (int, int, int, int),
                               int targ_x, int targ_y, int power );
 
 int apply_random_around_square( int (*func) (int, int, int, int),
-                                int targ_x, int targ_y, bool hole_in_middle, 
+                                int targ_x, int targ_y, bool hole_in_middle,
                                 int power, int max_targs );
 
 int apply_one_neighbouring_square(int (*func) (int, int, int, int),
@@ -124,12 +131,13 @@ int apply_area_within_radius(int (*func) (int, int, int, int),
 bool spell_direction( dist &spelld, bolt &pbolt,
                       targeting_type restrict = DIR_NONE,
                       targ_mode_type mode = TARG_ENEMY,
-                      bool needs_path = true,
-                      const char *prompt = NULL );
+                      bool needs_path = true, bool may_target_monster = true,
+                      const char *prompt = NULL, bool cancel_at_self = false );
 
-void apply_area_cloud(int (*func) (int, int, int, cloud_type, kill_category),
+void apply_area_cloud(int (*func) (int, int, int, int, cloud_type,
+                                   kill_category),
                       int x, int y, int pow, int number, cloud_type ctype,
-                      kill_category);
+                      kill_category kc, int spread_rate = -1);
 
 const char *spelltype_name(unsigned int which_spelltype);
 
