@@ -2,7 +2,10 @@
  *  File:       stash.h
  *  Summary:    Classes tracking player stashes
  *  Written by: Darshan Shaligram
+ *
+ *  Modified for Crawl Reference by $Author: zelgadis $ on $Date: 2008-06-05T08:43:08.573090Z $
  */
+
 #ifndef STASH_H
 #define STASH_H
 
@@ -18,6 +21,8 @@
 #include "travel.h"
 
 class input_history;
+class reader;
+class writer;
 
 // Stash definitions
 void stash_init_new_level();
@@ -43,8 +48,8 @@ public:
 
     static std::string stash_item_name(const item_def &item);
     void update();
-    void save(FILE*) const;
-    void load(FILE*);
+    void save(writer&) const;
+    void load(reader&);
 
     std::string description() const;
     std::string feature_description() const;
@@ -84,6 +89,10 @@ public:
                         // items nor dump itself. Disabled stashes are
                         // also never removed from the level's map of
                         // stashes.
+private:
+    void _update_corpses(long rot_time);
+    void add_item(const item_def &item, bool add_to_front = false);
+
 private:
     bool verified;      // Is this correct to the best of our knowledge?
     unsigned char x, y;
@@ -104,6 +113,8 @@ private:
 
     static bool are_items_same(const item_def &, const item_def &);
     static bool is_filtered(const item_def &item);
+
+    friend class LevelStashes;
 };
 
 class ShopInfo
@@ -118,9 +129,8 @@ public:
 
     std::string description() const;
 
-    // Note that we aren't bothering to use virtual functions here.
-    void save(FILE*) const;
-    void load(FILE*);
+    void save(writer&) const;
+    void load(reader&);
 
     bool show_menu(const std::string &place, bool can_travel) const;
     bool is_visited() const { return items.size() || visited; }
@@ -229,28 +239,32 @@ public:
 
     void  kill_stash(const Stash &s);
 
-    void  save(FILE *) const;
-    void  load(FILE *);
+    void  save(writer&) const;
+    void  load(reader&);
 
     void  write(std::ostream &os, bool identify = false) const;
     std::string level_name() const;
     std::string short_level_name() const;
 
-    int   stash_count() const { return stashes.size() + shops.size(); }
-    int   visible_stash_count() const { return count_stashes() + shops.size(); }
+    int   stash_count() const { return m_stashes.size() + m_shops.size(); }
+    int   visible_stash_count() const { return _num_enabled_stashes() + m_shops.size(); }
 
     bool  is_current() const;
-private:
-    // which level
-    level_id place;
 
+ private:
+    int _num_enabled_stashes() const;
+    void _update_corpses(long rot_time);
+
+ private:
     typedef std::map<int, Stash>  stashes_t;
     typedef std::vector<ShopInfo> shops_t;
     
-    stashes_t stashes;
-    shops_t   shops;
+    // which level
+    level_id m_place;
+    stashes_t m_stashes;
+    shops_t m_shops;
 
-    int count_stashes() const;
+    friend class StashTracker;
 };
 
 extern std::ostream &operator << (std::ostream &, const LevelStashes &);
@@ -264,7 +278,7 @@ public:
             || you.level_type == LEVEL_ABYSS;
     }
 
-    StashTracker() : levels()
+    StashTracker() : levels(), last_corpse_update(0.0)
     {
     }
 
@@ -289,6 +303,8 @@ public:
                                 // previously marked as stashes.
     };
 
+    void update_corpses();
+
     void update_visible_stashes(StashTracker::stash_update_mode = ST_PASSIVE);
 
     // Update stash at (x,y) on current level, defaulting to player's current
@@ -304,8 +320,8 @@ public:
     // used if no parameters are supplied.
     void no_stash(int x = -1, int y = -1);
 
-    void save(FILE*) const;
-    void load(FILE*);
+    void save(writer&) const;
+    void load(reader&);
 
     void write(std::ostream &os, bool identify = false) const;
 
@@ -314,15 +330,18 @@ public:
 private:
     void get_matching_stashes(const base_pattern &search, 
                               std::vector<stash_search_result> &results) const;
-    void display_search_results(std::vector<stash_search_result> &results);
-    void show_stash_search_prompt();    
+    bool display_search_results(std::vector<stash_search_result> &results,
+                                const char* sort_style);
+    void show_stash_search_prompt();
 
 private:
     typedef std::map<level_id, LevelStashes> stash_levels_t;
     stash_levels_t levels;
+
+    double last_corpse_update;
 };
 
-extern StashTracker stashes;
+extern StashTracker StashTrack;
 
 bool is_stash(int x, int y);
 void describe_stash(int x, int y);
