@@ -3,7 +3,7 @@
  *  Summary:    Misc stuff.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T16:15:56.136594Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-09T21:43:14.392740Z $
  *
  *  Change History (most recent first):
  *
@@ -16,7 +16,7 @@
 #include "AppHdr.h"
 #include "cio.h"
 #include "database.h"
-#include "direct.h"
+#include "directn.h"
 #include "message.h"
 #include "misc.h"
 #include "monplace.h"
@@ -27,6 +27,7 @@
 #include <cstdarg>
 #include <sstream>
 #include <iomanip>
+#include <algorithm>
 
 #include <errno.h>
 #include <stdlib.h>
@@ -49,11 +50,13 @@
 #endif
 
 #ifdef DOS
-#include <conio.h>
+ #include <conio.h>
 #endif
 
 #ifdef UNIX
-#include "libunix.h"
+ #ifndef USE_TILE
+ #include "libunix.h"
+ #endif
 #endif
 
 #include "branch.h"
@@ -69,10 +72,185 @@
 #include "notes.h"
 #include "output.h"
 #include "player.h"
+#include "religion.h"
 #include "skills2.h"
 #include "tutorial.h"
 #include "view.h"
 
+stack_iterator::stack_iterator(const coord_def& pos)
+{
+    cur_link = igrd(pos);
+    if ( cur_link != NON_ITEM )
+        next_link = mitm[cur_link].link;
+    else
+        next_link = NON_ITEM;
+}
+
+stack_iterator::stack_iterator(int start_link)
+{
+    cur_link = start_link;
+    if ( cur_link != NON_ITEM )
+        next_link = mitm[cur_link].link;
+    else
+        next_link = NON_ITEM;
+}
+
+stack_iterator::operator bool() const
+{
+    return ( cur_link != NON_ITEM );
+}
+
+item_def& stack_iterator::operator*() const
+{
+    ASSERT( cur_link != NON_ITEM );
+    return mitm[cur_link];
+}
+
+item_def* stack_iterator::operator->() const
+{
+    ASSERT( cur_link != NON_ITEM );
+    return &mitm[cur_link];
+}
+
+int stack_iterator::link() const
+{
+    return cur_link;
+}
+
+const stack_iterator& stack_iterator::operator ++ ()
+{
+    cur_link = next_link;
+    if ( cur_link != NON_ITEM )
+        next_link = mitm[cur_link].link;
+    return *this;
+}
+
+stack_iterator stack_iterator::operator++(int dummy)
+{
+    const stack_iterator copy = *this;
+    ++(*this);
+    return copy;
+}
+
+radius_iterator::radius_iterator( const coord_def& _center, int _radius,
+                                  bool _roguelike_metric, bool _require_los,
+                                  bool _exclude_center )
+    : center(_center), radius(_radius), roguelike_metric(_roguelike_metric),
+      require_los(_require_los), exclude_center(_exclude_center),
+      iter_done(false)
+{
+    reset();
+}
+
+void radius_iterator::reset()
+{
+    iter_done = false;
+
+    location.x = center.x - radius;
+    location.y = center.y - radius;
+
+    if ( !this->on_valid_square() )
+        ++(*this);
+}
+
+bool radius_iterator::done() const
+{
+    return iter_done;
+}
+
+coord_def radius_iterator::operator *() const
+{
+    return location;
+}
+
+const coord_def* radius_iterator::operator->() const
+{
+    return &location;
+}
+
+void radius_iterator::step()
+{
+    const int minx = std::max(X_BOUND_1+1, center.x - radius);
+    const int maxx = std::min(X_BOUND_2-1, center.x + radius);
+    const int maxy = std::min(Y_BOUND_2-1, center.y + radius);
+
+    // Sweep L-R, U-D
+    location.x++;
+    if (location.x > maxx)
+    {
+        location.x = minx;
+        location.y++;
+        if (location.y > maxy)
+            iter_done = true;
+    }
+}
+
+void radius_iterator::step_back()
+{
+    const int minx = std::max(X_BOUND_1+1, center.x - radius);
+    const int maxx = std::min(X_BOUND_2-1, center.x + radius);
+    const int miny = std::max(Y_BOUND_1+1, center.y - radius);
+
+    location.x--;
+    if ( location.x < minx )
+    {
+        location.x = maxx;
+        location.y--;
+        if ( location.y < miny )
+            iter_done = true;   // hmm
+    }
+}
+
+bool radius_iterator::on_valid_square() const
+{
+    if (!in_bounds(location))
+        return (false);
+    if (!roguelike_metric && (location - center).abs() > radius*radius)
+        return (false);
+    if (require_los && !see_grid(location))
+        return (false);
+    if (exclude_center && location == center)
+        return (false);
+
+    return (true);
+}
+
+const radius_iterator& radius_iterator::operator++()
+{
+    do
+    {
+        this->step();
+    }
+    while (!this->done() && !this->on_valid_square());
+
+    return (*this);
+}
+
+const radius_iterator& radius_iterator::operator--()
+{
+    do
+    {
+        this->step_back();
+    }
+    while (!this->done() && !this->on_valid_square());
+
+    return (*this);
+}
+
+radius_iterator radius_iterator::operator++(int dummy)
+{
+    const radius_iterator copy = *this;
+    ++(*this);
+    return (copy);
+}
+
+radius_iterator radius_iterator::operator--(int dummy)
+{
+    const radius_iterator copy = *this;
+    --(*this);
+    return (copy);
+}
+
 // Crude, but functional.
 std::string make_time_string( time_t abs_time, bool terse )
 {
@@ -127,10 +305,10 @@ static bool tag_follower_at(const coord_def &pos)
     if (fmenv->speed_increment < 50)
         return (false);
 
-    // only friendly monsters, or those actively seeking the
+    // Only friendly monsters, or those actively seeking the
     // player, will follow up/down stairs.
-    if (!(mons_friendly(fmenv) ||
-          (fmenv->behaviour == BEH_SEEK && fmenv->foe == MHITYOU)))
+    if (!mons_friendly(fmenv)
+        && (!mons_is_seeking(fmenv) || fmenv->foe != MHITYOU))
     {
         return (false);
     }
@@ -143,14 +321,18 @@ static bool tag_follower_at(const coord_def &pos)
             return (false);
 
         // Orcs will follow Beogh worshippers.
-        if (!(mons_species(fmenv->type) == MONS_ORC
-              && you.religion == GOD_BEOGH))
+        if (you.religion != GOD_BEOGH || !is_follower(fmenv))
             return (false);
     }
 
-    // monster is chasing player through stairs:
+    // Monster is chasing player through stairs.
     fmenv->flags |= MF_TAKING_STAIRS;
 
+    // Clear patrolling/travel markers.
+    fmenv->patrol_point = coord_def(0, 0);
+    fmenv->travel_path.clear();
+    fmenv->travel_target = MTRAV_NONE;
+
 #if DEBUG_DIAGNOSTICS
     mprf(MSGCH_DIAGNOSTICS, "%s is marked for following.",
          fmenv->name(DESC_CAP_THE, true).c_str() );
@@ -271,6 +453,8 @@ int random_range(int low, int high, int nrolls)
     return (sum / nrolls);
 }
 
+// Chooses one of the numbers passed in at random. The list of numbers
+// must be terminated with -1.
 int random_choose(int first, ...)
 {
     va_list args;
@@ -288,7 +472,32 @@ int random_choose(int first, ...)
     }
 
     ASSERT(nargs > 0);
-    
+
+    va_end(args);
+    return (chosen);
+}
+
+// Chooses one of the strings passed in at random. The list of strings
+// must be terminated with NULL.
+const char* random_choose_string(const char* first, ...)
+{
+    va_list args;
+    va_start(args, first);
+
+    const char* chosen = first;
+    int count = 1, nargs = 100;
+
+    while (nargs-- > 0)
+    {
+        char* pick = va_arg(args, char*);
+        if (pick == NULL)
+            break;
+        if (one_chance_in(++count))
+            chosen = pick;
+    }
+
+    ASSERT(nargs > 0);
+
     va_end(args);
     return (chosen);
 }
@@ -312,7 +521,7 @@ int random_choose_weighted(int weight, int first, ...)
     }
 
     ASSERT(nargs > 0);
-    
+
     va_end(args);
     return (chosen);
 }
@@ -386,8 +595,8 @@ int roll_dice( int num, int size )
     int ret = 0;
     int i;
 
-    // If num <= 0 or size <= 0, then we'll just return the default 
-    // value of zero.  This is good behaviour in that it will be 
+    // If num <= 0 or size <= 0, then we'll just return the default
+    // value of zero.  This is good behaviour in that it will be
     // appropriate for calculated values that might be passed in.
     if (num > 0 && size > 0)
     {
@@ -429,7 +638,9 @@ void cio_init()
 {
     crawl_state.io_inited = true;
 
-#ifdef UNIX
+#if defined(USE_TILE)
+    libgui_init();
+#elif defined(UNIX)
     unixcurses_startup();
 #endif
 
@@ -440,7 +651,7 @@ void cio_init()
 #ifdef DOS
     init_libdos();
 #endif
-    
+
     crawl_view.init_geometry();
 
     if (Options.char_set == CSET_UNICODE && !crawl_state.unicode_ok)
@@ -455,8 +666,10 @@ void cio_cleanup()
 {
     if (!crawl_state.io_inited)
         return;
-    
-#ifdef UNIX
+
+#if defined(USE_TILE)
+    libgui_shutdown();
+#elif defined(UNIX)
     unixcurses_shutdown();
 #endif
 
@@ -464,7 +677,7 @@ void cio_cleanup()
     deinit_libw32c();
 #endif
 
-    msg::deinitalise_mpr_streams();
+    msg::deinitialise_mpr_streams();
 
     crawl_state.io_inited = false;
 }
@@ -472,7 +685,7 @@ void cio_cleanup()
 void end(int exit_code, bool print_error, const char *format, ...)
 {
     std::string error = print_error? strerror(errno) : "";
-    
+
     cio_cleanup();
     databaseSystemShutdown();
     if (format)
@@ -482,7 +695,7 @@ void end(int exit_code, bool print_error, const char *format, ...)
         char buffer[100];
         vsnprintf(buffer, sizeof buffer, format, arg);
         va_end(arg);
-        
+
         if (error.empty())
             error = std::string(buffer);
         else
@@ -504,24 +717,32 @@ void end(int exit_code, bool print_error, const char *format, ...)
         getchar();
     }
 #endif
-    
+
     exit(exit_code);
 }
 
 void redraw_screen(void)
 {
+    if (!crawl_state.need_save)
+    {
+        // If the game hasn't started, don't do much.
+        clrscr();
+        return;
+    }
+
     draw_border();
 
-    you.redraw_hit_points = true;
+    you.redraw_hit_points   = true;
     you.redraw_magic_points = true;
-    you.redraw_strength = true;
+    you.redraw_strength     = true;
     you.redraw_intelligence = true;
-    you.redraw_dexterity = true;
+    you.redraw_dexterity    = true;
     you.redraw_armour_class = true;
-    you.redraw_evasion = true;
-    you.redraw_gold = true;
-    you.redraw_experience = true;
-    you.wield_change = true;
+    you.redraw_evasion      = true;
+    you.redraw_gold         = true;
+    you.redraw_experience   = true;
+    you.wield_change        = true;
+    you.redraw_quiver       = true;
 
     set_redraw_status(
         REDRAW_LINE_1_MASK | REDRAW_LINE_2_MASK | REDRAW_LINE_3_MASK );
@@ -540,12 +761,13 @@ void redraw_screen(void)
     update_turn_count();
     activate_notes(note_status);
 
-    viewwindow(1, false);
-}                               // end redraw_screen()
+    viewwindow(true, false);
+}
 
 // STEPDOWN FUNCTION to replace conditional chains in spells2.cc 12jan2000 {dlb}
 // it is a bit more extensible and optimizes the logical structure, as well
-// usage: summon_swarm() summon_undead() summon_scorpions() summon_things()
+// usage: cast_summon_swarm() cast_summon_wraiths() cast_summon_scorpions()
+//        cast_summon_horrible_things()
 // ex(1): stepdown_value (foo, 2, 2, 6, 8) replaces the following block:
 //
 
@@ -573,9 +795,9 @@ void redraw_screen(void)
      bar = (bar - 6) / 2 + 6;
  */
 
-// I hope this permits easier/more experimentation with value stepdowns in
-// the code it really needs to be rewritten to accept arbitrary (unevenly
-// spaced) steppings
+// I hope this permits easier/more experimentation with value stepdowns
+// in the code.  It really needs to be rewritten to accept arbitrary
+// (unevenly spaced) steppings.
 int stepdown_value(int base_value, int stepping, int first_step,
                    int last_step, int ceiling_value)
 {
@@ -600,10 +822,10 @@ int stepdown_value(int base_value, int stepping, int first_step,
     else
         return return_value;    // otherwise, value returned "as is"
 
-}                               // end stepdown_value()
+}
 
 int skill_bump( int skill )
-{   
+{
     return ((you.skills[skill] < 3) ? you.skills[skill] * 2
                                     : you.skills[skill] + 3);
 }
@@ -644,36 +866,45 @@ bool one_chance_in(int a_million)
     return (random2(a_million) == 0);
 }
 
-// simple little function to quickly modify all three stats
+bool x_chance_in_y(int x, int y)
+{
+    if (x <= 0)
+        return (false);
+
+    if (x >= y)
+        return (true);
+
+    return (random2(y) < x);
+}
+
+// Simple little function to quickly modify all three stats
 // at once - does check for '0' modifiers to prevent needless
 // adding .. could use checking for sums less than zero, I guess.
-// used in conjunction with newgame::species_stat_init() and
-// newgame::job_stat_init() routines 24jan2000 {dlb}
+// Used in conjunction with newgame::species_stat_init() and
+// newgame::job_stat_init() routines 24jan2000. {dlb}
 void modify_all_stats(int STmod, int IQmod, int DXmod)
 {
     if (STmod)
     {
-        you.strength += STmod;
+        you.strength     += STmod;
         you.max_strength += STmod;
-        you.redraw_strength = 1;
+        you.redraw_strength = true;
     }
 
     if (IQmod)
     {
-        you.intel += IQmod;
+        you.intel     += IQmod;
         you.max_intel += IQmod;
-        you.redraw_intelligence = 1;
+        you.redraw_intelligence = true;
     }
 
     if (DXmod)
     {
-        you.dex += DXmod;
+        you.dex     += DXmod;
         you.max_dex += DXmod;
-        you.redraw_dexterity = 1;
+        you.redraw_dexterity = true;
     }
-
-    return;
-}                               // end modify_stat()
+}
 
 void canned_msg(canned_message_type which_message)
 {
@@ -681,39 +912,49 @@ void canned_msg(canned_message_type which_message)
     {
     case MSG_SOMETHING_APPEARS:
         mprf("Something appears %s!",
-             (you.species == SP_NAGA || you.mutation[MUT_HOOVES])
-             ? "before you" : "at your feet");
+             (you.species == SP_NAGA || player_mutation_level(MUT_HOOVES))
+                 ? "before you" : "at your feet");
         break;
-
     case MSG_NOTHING_HAPPENS:
         mpr("Nothing appears to happen.");
         break;
     case MSG_YOU_RESIST:
         mpr("You resist.");
+        learned_something_new(TUT_YOU_RESIST);
+        break;
+    case MSG_YOU_PARTIALLY_RESIST:
+        mpr("You partially resist.");
         break;
     case MSG_TOO_BERSERK:
         mpr("You are too berserk!");
+        crawl_state.cancel_cmd_repeat();
         break;
     case MSG_PRESENT_FORM:
         mpr("You can't do that in your present form.");
+        crawl_state.cancel_cmd_repeat();
         break;
     case MSG_NOTHING_CARRIED:
         mpr("You aren't carrying anything.");
+        crawl_state.cancel_cmd_repeat();
         break;
     case MSG_CANNOT_DO_YET:
         mpr("You can't do that yet.");
+        crawl_state.cancel_cmd_repeat();
         break;
     case MSG_OK:
-        mpr("Okay, then.");
+        mpr("Okay, then.", MSGCH_PROMPT);
+        crawl_state.cancel_cmd_repeat();
         break;
     case MSG_UNTHINKING_ACT:
         mpr("Why would you want to do that?");
+        crawl_state.cancel_cmd_repeat();
         break;
     case MSG_SPELL_FIZZLES:
         mpr("The spell fizzles.");
         break;
     case MSG_HUH:
-        mpr("Huh?");
+        mpr("Huh?", MSGCH_EXAMINE_FILTER);
+        crawl_state.cancel_cmd_repeat();
         break;
     case MSG_EMPTY_HANDED:
         mpr("You are now empty-handed.");
@@ -723,27 +964,58 @@ void canned_msg(canned_message_type which_message)
     return;
 }                               // end canned_msg()
 
+// Like yesno, but requires a full typed answer.
+// Unlike yesno, prompt should have no trailing space.
+// Returns true if the user typed "yes", false if something else or cancel.
+bool yes_or_no( const char* fmt, ... )
+{
+    char buf[200];
+    va_list args;
+    va_start(args, fmt);
+    vsnprintf(buf, sizeof buf, fmt, args);
+    va_end(args);
+    buf[sizeof(buf)-1] = 0;
+
+    mprf(MSGCH_PROMPT, "%s? (Confirm with \"yes\".) ", buf);
+
+    if (cancelable_get_line(buf, sizeof buf))
+        return (false);
+    if (strcasecmp(buf, "yes") != 0)
+        return (false);
+
+    return (true);
+}
+
 // jmf: general helper (should be used all over in code)
 //      -- idea borrowed from Nethack
 bool yesno( const char *str, bool safe, int safeanswer, bool clear_after,
-            bool interrupt_delays, bool noprompt )
+            bool interrupt_delays, bool noprompt,
+            const explicit_keymap *map )
 {
-    if (interrupt_delays)
+    if (interrupt_delays && !crawl_state.is_repeating_cmd())
         interrupt_activity( AI_FORCE_INTERRUPT );
-    for (;;)
+
+    std::string prompt = make_stringf("%s ", str ? str : "Buggy prompt?");
+
+    while (true)
     {
-        if ( !noprompt )
-            mpr(str, MSGCH_PROMPT);
+        if (!noprompt)
+            mpr(prompt.c_str(), MSGCH_PROMPT);
 
         int tmp = getchm(KC_CONFIRM);
 
-        if ((tmp == ' ' || tmp == 27 || tmp == '\r' || tmp == '\n') 
-                && safeanswer)
+        if (map && map->find(tmp) != map->end())
+            tmp = map->find(tmp)->second;
+
+        if (safeanswer
+            && (tmp == ' ' || tmp == 27 || tmp == '\r' || tmp == '\n'))
+        {
             tmp = safeanswer;
+        }
 
         if (Options.easy_confirm == CONFIRM_ALL_EASY
             || tmp == safeanswer
-            || (Options.easy_confirm == CONFIRM_SAFE_EASY && safe))
+            || Options.easy_confirm == CONFIRM_SAFE_EASY && safe)
         {
             tmp = toupper( tmp );
         }
@@ -752,33 +1024,94 @@ bool yesno( const char *str, bool safe, int safeanswer, bool clear_after,
             mesclr();
 
         if (tmp == 'N')
-            return false;
+            return (false);
         else if (tmp == 'Y')
-            return true;
+            return (true);
         else if (!noprompt)
             mpr("[Y]es or [N]o only, please.");
     }
 }                               // end yesno()
 
-// like yesno(), but returns 0 for no, 1 for yes, and -1 for quit
-int yesnoquit( const char* str, bool safe, int safeanswer, bool clear_after )
+static std::string _list_alternative_yes(char yes1, char yes2,
+                                         bool lowered = false,
+                                         bool brackets = false)
 {
-    interrupt_activity( AI_FORCE_INTERRUPT );
-    while (1)
+    std::string help = "";
+    bool print_yes = false;
+    if (yes1 != 'Y')
     {
-        mpr(str, MSGCH_PROMPT);
+        if (lowered)
+            help += tolower(yes1);
+        else
+            help += yes1;
+        print_yes = true;
+    }
+
+    if (yes2 != 'Y' && yes2 != yes1)
+    {
+        if (print_yes)
+            help += "/";
+
+        if (lowered)
+            help += tolower(yes2);
+        else
+            help += yes2;
+        print_yes = true;
+    }
+
+    if (print_yes)
+    {
+        if (brackets)
+            help = " (" + help + ")";
+        else
+            help = "/" + help;
+    }
+
+    return help;
+}
+
+static std::string _list_allowed_keys(char yes1, char yes2,
+                                      bool lowered = false,
+                                      bool allow_all = false)
+{
+    std::string result = " [";
+                result += (lowered ? "y" : "Y");
+                result += _list_alternative_yes(yes1, yes2, lowered);
+                if (allow_all)
+                    result += (lowered? "/a" : "/A");
+                result += (lowered ? "/n/q" : "/N/Q");
+                result += "]";
+
+    return (result);
+}
+
+// Like yesno(), but returns 0 for no, 1 for yes, and -1 for quit.
+// alt_yes and alt_yes2 allow up to two synonyms for 'Y'.
+// FIXME: This function is shaping up to be a monster. Help!
+int yesnoquit( const char* str, bool safe, int safeanswer, bool allow_all,
+               bool clear_after, char alt_yes, char alt_yes2 )
+{
+    if (!crawl_state.is_repeating_cmd())
+        interrupt_activity( AI_FORCE_INTERRUPT );
+
+    std::string prompt = make_stringf("%s%s ", str ? str : "Buggy prompt?",
+                                      _list_allowed_keys(alt_yes, alt_yes2,
+                                                         safe, allow_all).c_str());
+    while (true)
+    {
+        mpr(prompt.c_str(), MSGCH_PROMPT);
 
         int tmp = getchm(KC_CONFIRM);
 
-        if ( tmp == CK_ESCAPE || tmp == 'q' || tmp == 'Q' )
+        if (tmp == CK_ESCAPE || tmp == 'q' || tmp == 'Q')
             return -1;
-        
+
         if ((tmp == ' ' || tmp == '\r' || tmp == '\n') && safeanswer)
             tmp = safeanswer;
 
         if (Options.easy_confirm == CONFIRM_ALL_EASY
             || tmp == safeanswer
-            || (Options.easy_confirm == CONFIRM_SAFE_EASY && safe))
+            || safe && Options.easy_confirm == CONFIRM_SAFE_EASY)
         {
             tmp = toupper( tmp );
         }
@@ -788,12 +1121,28 @@ int yesnoquit( const char* str, bool safe, int safeanswer, bool clear_after )
 
         if (tmp == 'N')
             return 0;
-        else if (tmp == 'Y')
+        else if (tmp == 'Y' || tmp == alt_yes || tmp == alt_yes2)
             return 1;
+        else if (allow_all)
+        {
+            if (tmp == 'A')
+                return 2;
+            else
+                mprf("Choose [Y]es%s, [N]o, [Q]uit, or [A]ll!",
+                     _list_alternative_yes(alt_yes, alt_yes2, false, true).c_str());
+        }
         else
-            mpr("[Y]es, [N]o or [Q]uit only, please.");
+        {
+            mprf("[Y]es%s, [N]o or [Q]uit only, please.",
+                 _list_alternative_yes(alt_yes, alt_yes2, false, true).c_str());
+        }
     }
-}    
+}
+
+int grid_distance( const coord_def& p1, const coord_def& p2 )
+{
+    return grid_distance(p1.x, p1.y, p2.x, p2.y);
+}
 
 // More accurate than distance() given the actual movement geometry -- bwr
 int grid_distance( int x, int y, int x2, int y2 )
@@ -825,7 +1174,7 @@ bool silenced(int x, int y)
     if (you.duration[DUR_SILENCE] > 0
         && distance(x, y, you.x_pos, you.y_pos) <= 36)  // (6 * 6)
     {
-        return true;
+        return (true);
     }
     else
     {
@@ -833,21 +1182,21 @@ bool silenced(int x, int y)
         //  for (int i = 0; i < MAX_SILENCES; i++)
         //  {
         //      if (distance(x, y, silencer[i].x, silencer[i].y) <= 36)
-        //         return true;
+        //         return (true);
         //  }
-        return false;
+        return (false);
     }
-}                               // end silenced()
+}
 
 bool player_can_hear(int x, int y)
 {
     return (!silenced(x, y) && !silenced(you.x_pos, you.y_pos));
-}                               // end player_can_hear()
+}
 
-// Returns true if inside the area the player can move and dig (ie exclusive)
+// Returns true if inside the area the player can move and dig (ie exclusive).
 bool in_bounds( int x, int y )
 {
-    return (x > X_BOUND_1 && x < X_BOUND_2 
+    return (x > X_BOUND_1 && x < X_BOUND_2
             && y > Y_BOUND_1 && y < Y_BOUND_2);
 }
 
@@ -860,6 +1209,12 @@ bool map_bounds( int x, int y )
             && y >= Y_BOUND_1 && y <= Y_BOUND_2);
 }
 
+coord_def random_in_bounds()
+{
+    return coord_def( random_range(MAPGEN_BORDER, GXM - MAPGEN_BORDER - 1),
+                      random_range(MAPGEN_BORDER, GYM - MAPGEN_BORDER - 1) );
+}
+
 // Returns a random location in (x_pos, y_pos)... the grid will be
 // DNGN_FLOOR if clear, and NON_MONSTER if empty.  Exclusive tells
 // if we're using in_bounds() or map_bounds() restriction.
@@ -880,8 +1235,8 @@ void random_in_bounds( int &x_pos, int &y_pos, int terr,
         else if (terr == DNGN_DEEP_WATER
                  && grd[x_pos][y_pos] == DNGN_SHALLOW_WATER)
             done = true;
-        else if (empty 
-                && mgrd[x_pos][y_pos] != NON_MONSTER 
+        else if (empty
+                && mgrd[x_pos][y_pos] != NON_MONSTER
                 && (x_pos != you.x_pos || y_pos != you.y_pos))
         {
             done = true;
@@ -914,7 +1269,7 @@ bool is_element_colour( int col )
 
 int element_colour( int element, bool no_random )
 {
-    // Doing this so that we don't have to do recursion here at all 
+    // Doing this so that we don't have to do recursion here at all
     // (these were the only cases which had possible double evaluation):
     if (element == EC_FLOOR)
         element = env.floor_colour;
@@ -928,7 +1283,7 @@ int element_colour( int element, bool no_random )
     int ret = BLACK;
 
     // Setting no_random to true will get the first colour in the cases
-    // below.  This is potentially useful for calls to this function 
+    // below.  This is potentially useful for calls to this function
     // which might want a consistent result.
     int tmp_rand = (no_random ? 0 : random2(120));
 
@@ -936,13 +1291,13 @@ int element_colour( int element, bool no_random )
     {
     case EC_FIRE:
         ret = (tmp_rand < 40) ? RED :
-              (tmp_rand < 80) ? YELLOW 
+              (tmp_rand < 80) ? YELLOW
                               : LIGHTRED;
         break;
 
     case EC_ICE:
         ret = (tmp_rand < 40) ? LIGHTBLUE :
-              (tmp_rand < 80) ? BLUE 
+              (tmp_rand < 80) ? BLUE
                               : WHITE;
         break;
 
@@ -956,7 +1311,7 @@ int element_colour( int element, bool no_random )
 
     case EC_ELECTRICITY:
         ret = (tmp_rand < 40) ? LIGHTCYAN :
-              (tmp_rand < 80) ? LIGHTBLUE 
+              (tmp_rand < 80) ? LIGHTBLUE
                               : CYAN;
         break;
 
@@ -971,7 +1326,7 @@ int element_colour( int element, bool no_random )
     case EC_MAGIC:
         ret = (tmp_rand < 30) ? LIGHTMAGENTA :
               (tmp_rand < 60) ? LIGHTBLUE :
-              (tmp_rand < 90) ? MAGENTA 
+              (tmp_rand < 90) ? MAGENTA
                               : BLUE;
         break;
 
@@ -1002,7 +1357,7 @@ int element_colour( int element, bool no_random )
         break;
 
     case EC_DARK:
-        ret = DARKGREY;
+        ret = (tmp_rand < 80) ? DARKGREY : LIGHTGREY;
         break;
 
     case EC_HOLY:
@@ -1011,30 +1366,31 @@ int element_colour( int element, bool no_random )
 
     case EC_VEHUMET:
         ret = (tmp_rand < 40) ? LIGHTRED :
-              (tmp_rand < 80) ? LIGHTMAGENTA 
+              (tmp_rand < 80) ? LIGHTMAGENTA
                               : LIGHTBLUE;
         break;
 
     case EC_BEOGH:
-        ret = (tmp_rand < 60) ? LIGHTRED : BROWN;
+        ret = (tmp_rand < 60) ? LIGHTRED // plain Orc (+ orcish idol) colour
+                              : BROWN;   // Orcish mines wall colour
         break;
 
     case EC_CRYSTAL:
         ret = (tmp_rand < 40) ? LIGHTGREY :
-              (tmp_rand < 80) ? GREEN 
+              (tmp_rand < 80) ? GREEN
                               : LIGHTRED;
         break;
 
     case EC_SLIME:
         ret = (tmp_rand < 40) ? GREEN :
-              (tmp_rand < 80) ? BROWN 
+              (tmp_rand < 80) ? BROWN
                               : LIGHTGREEN;
         break;
 
     case EC_SMOKE:
         ret = (tmp_rand < 30) ? LIGHTGREY :
               (tmp_rand < 60) ? DARKGREY :
-              (tmp_rand < 90) ? LIGHTBLUE 
+              (tmp_rand < 90) ? LIGHTBLUE
                               : MAGENTA;
         break;
 
@@ -1047,36 +1403,36 @@ int element_colour( int element, bool no_random )
               (tmp_rand <  72) ? LIGHTBLUE :
               (tmp_rand <  84) ? MAGENTA :
               (tmp_rand <  96) ? RED :
-              (tmp_rand < 108) ? GREEN 
+              (tmp_rand < 108) ? GREEN
                                : BLUE;
         break;
 
     case EC_ELVEN:
         ret = (tmp_rand <  40) ? LIGHTGREEN :
               (tmp_rand <  80) ? GREEN :
-              (tmp_rand < 100) ? LIGHTBLUE 
+              (tmp_rand < 100) ? LIGHTBLUE
                                : BLUE;
         break;
 
     case EC_DWARVEN:
         ret = (tmp_rand <  40) ? BROWN :
               (tmp_rand <  80) ? LIGHTRED :
-              (tmp_rand < 100) ? LIGHTGREY 
+              (tmp_rand < 100) ? LIGHTGREY
                                : CYAN;
         break;
 
     case EC_ORCISH:
         ret = (tmp_rand <  40) ? DARKGREY :
               (tmp_rand <  80) ? RED :
-              (tmp_rand < 100) ? BROWN 
+              (tmp_rand < 100) ? BROWN
                                : MAGENTA;
         break;
 
     case EC_GILA:
         ret = (tmp_rand <  30) ? LIGHTMAGENTA :
               (tmp_rand <  60) ? MAGENTA :
-              (tmp_rand <  90) ? YELLOW : 
-              (tmp_rand < 105) ? LIGHTRED 
+              (tmp_rand <  90) ? YELLOW :
+              (tmp_rand < 105) ? LIGHTRED
                                : RED;
         break;
 
@@ -1095,12 +1451,34 @@ int element_colour( int element, bool no_random )
         ret = random_choose_weighted(80, BLUE, 20, LIGHTBLUE, 5, CYAN, 0);
         break;
 
+    case EC_DECAY:
+        ret = (tmp_rand < 60) ? BROWN : GREEN;
+        break;
+
+    case EC_SILVER:
+        ret = (tmp_rand < 90) ? LIGHTGREY : WHITE;
+        break;
+
+    case EC_GOLD:
+        ret = (tmp_rand < 60) ? YELLOW : BROWN;
+        break;
+
+    case EC_IRON:
+        ret = (tmp_rand < 40) ? CYAN :
+              (tmp_rand < 80) ? LIGHTGREY :
+                                DARKGREY;
+        break;
+
+    case EC_BONE:
+        ret = (tmp_rand < 90) ? WHITE : LIGHTGREY;
+        break;
+
     case EC_RANDOM:
         ret = 1 + random2(15);              // always random
         break;
 
-    case EC_FLOOR: // should alredy be handled 
-    case EC_ROCK:  // should alredy be handled 
+    case EC_FLOOR: // should already be handled
+    case EC_ROCK:  // should already be handled
     default:
         break;
     }
@@ -1134,18 +1512,17 @@ int fuzz_value(int val, int lowfuzz, int highfuzz, int naverage)
     return val + random2avg(lfuzz + hfuzz + 1, naverage) - lfuzz;
 }
 
-// returns 0 if the point is not near stairs
-// returns 1 if the point is near unoccupied stairs
-// returns 2 if the point is near player-occupied stairs
-
+// Returns 0 if the point is not near stairs.
+// Returns 1 if the point is near unoccupied stairs.
+// Returns 2 if the point is near player-occupied stairs.
 int near_stairs(const coord_def &p, int max_dist,
-                dungeon_char_type &stair_type,
-                branch_type &branch)
+                dungeon_char_type &stair_type, branch_type &branch)
 {
     coord_def inc;
-    for (inc.x = -max_dist; inc.x <= max_dist; inc.x++)
+
+    for (inc.x = -max_dist; inc.x <= max_dist; ++inc.x)
     {
-        for(inc.y = -max_dist; inc.y <= max_dist; inc.y++)
+        for (inc.y = -max_dist; inc.y <= max_dist; ++inc.y)
         {
             const coord_def np(p + inc);
 
@@ -1155,13 +1532,13 @@ int near_stairs(const coord_def &p, int max_dist,
             const dungeon_feature_type feat = grd(np);
             if (is_stair(feat))
             {
-                // shouldn't happen for escape hatches
-                if (grid_is_rock_stair(feat))
+                // Shouldn't happen for escape hatches.
+                if (grid_is_escape_hatch(feat))
                     continue;
 
                 stair_type = get_feature_dchar(feat);
-                
-                // is it a branch stair?
+
+                // Is it a branch stair?
                 for (int i = 0; i < NUM_BRANCHES; ++i)
                 {
                      if (branches[i].entry_stairs == feat)
@@ -1175,18 +1552,18 @@ int near_stairs(const coord_def &p, int max_dist,
                          break;
                      }
                 }
-                return (np == you.pos()? 2 : 1);
+                return (np == you.pos()) ? 2 : 1;
             }
         }
     }
 
-    return false;
+    return (false);
 }
 
-bool is_trap_square(int x, int y)
+bool is_trap_square(dungeon_feature_type grid)
 {
-    return (grd[x][y] >= DNGN_TRAP_MECHANICAL
-        && grd[x][y] <= DNGN_UNDISCOVERED_TRAP);
+    return (grid >= DNGN_TRAP_MECHANICAL
+            && grid <= DNGN_UNDISCOVERED_TRAP);
 }
 
 // Does the equivalent of KILL_RESET on all monsters in LOS. Should only be
@@ -1196,7 +1573,6 @@ void zap_los_monsters()
     losight(env.show, grd, you.x_pos, you.y_pos);
 
     for (int y = crawl_view.vlos1.y; y <= crawl_view.vlos2.y; ++y)
-    {
         for (int x = crawl_view.vlos1.x; x <= crawl_view.vlos2.x; ++x)
         {
             if (!in_vlos(x, y))
@@ -1212,12 +1588,12 @@ void zap_los_monsters()
                 continue;
 
             int imon = mgrd[gx][gy];
-            
-            // at tutorial beginning disallow items in line of sight
+
+            // At tutorial beginning disallow items in line of sight.
             if (Options.tutorial_events[TUT_SEEN_FIRST_OBJECT])
             {
                 int item = igrd[gx][gy];
-            
+
                 if (item != NON_ITEM && is_valid_item(mitm[item]) )
                     destroy_item(item);
             }
@@ -1231,14 +1607,15 @@ void zap_los_monsters()
 
             if (mons_class_flag( mon->type, M_NO_EXP_GAIN ))
                 continue;
-            
+
 #ifdef DEBUG_DIAGNOSTICS
             mprf(MSGCH_DIAGNOSTICS, "Dismissing %s",
                  mon->name(DESC_PLAIN, true).c_str() );
 #endif
+            // Mark as summoned so its items will also be discarded.
+            mon->mark_summoned(1, true);
             monster_die(mon, KILL_DISMISSED, 0);
         }
-    }
 }
 
 //////////////////////////////////////////////////////////////////////////
