@@ -3,7 +3,7 @@
  *  Summary:    Terrain related functions.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-07-09T21:43:14.392740Z $
  *
  *  Change History (most recent first):
  *
@@ -11,11 +11,13 @@
  */
 
 #include "AppHdr.h"
-
 #include "externs.h"
 #include "terrain.h"
 
+#include <algorithm>
+
 #include "dgnevent.h"
+#include "directn.h"
 #include "itemprop.h"
 #include "items.h"
 #include "message.h"
@@ -30,16 +32,12 @@
 #include "spells3.h"
 #include "stuff.h"
 #include "transfor.h"
+#include "traps.h"
 #include "view.h"
 
-bool grid_is_wall( dungeon_feature_type grid )
+bool grid_is_wall(dungeon_feature_type grid)
 {
-    return (grid == DNGN_ROCK_WALL
-            || grid == DNGN_STONE_WALL
-            || grid == DNGN_METAL_WALL
-            || grid == DNGN_GREEN_CRYSTAL_WALL
-            || grid == DNGN_WAX_WALL
-            || grid == DNGN_PERMAROCK_WALL);
+    return (grid >= DNGN_MINWALL && grid <= DNGN_MAXWALL);
 }
 
 bool grid_is_stone_stair(dungeon_feature_type grid)
@@ -58,16 +56,9 @@ bool grid_is_stone_stair(dungeon_feature_type grid)
     }
 }
 
-bool grid_is_rock_stair(dungeon_feature_type grid)
+bool grid_is_escape_hatch(dungeon_feature_type grid)
 {
-    switch (grid)
-    {
-    case DNGN_ROCK_STAIRS_UP:
-    case DNGN_ROCK_STAIRS_DOWN:
-        return (true);
-    default:
-        return (false);
-    }
+    return (grid == DNGN_ESCAPE_HATCH_UP || grid == DNGN_ESCAPE_HATCH_DOWN);
 }
 
 bool grid_sealable_portal(dungeon_feature_type grid)
@@ -97,7 +88,7 @@ command_type grid_stair_direction(dungeon_feature_type grid)
     case DNGN_STONE_STAIRS_UP_I:
     case DNGN_STONE_STAIRS_UP_II:
     case DNGN_STONE_STAIRS_UP_III:
-    case DNGN_ROCK_STAIRS_UP:
+    case DNGN_ESCAPE_HATCH_UP:
     case DNGN_RETURN_FROM_ORCISH_MINES:
     case DNGN_RETURN_FROM_HIVE:
     case DNGN_RETURN_FROM_LAIR:
@@ -119,14 +110,14 @@ command_type grid_stair_direction(dungeon_feature_type grid)
     case DNGN_EXIT_HELL:
     case DNGN_EXIT_PORTAL_VAULT:
         return (CMD_GO_UPSTAIRS);
-        
+
     case DNGN_ENTER_PORTAL_VAULT:
     case DNGN_ENTER_HELL:
     case DNGN_ENTER_LABYRINTH:
     case DNGN_STONE_STAIRS_DOWN_I:
     case DNGN_STONE_STAIRS_DOWN_II:
     case DNGN_STONE_STAIRS_DOWN_III:
-    case DNGN_ROCK_STAIRS_DOWN:
+    case DNGN_ESCAPE_HATCH_DOWN:
     case DNGN_ENTER_DIS:
     case DNGN_ENTER_GEHENNA:
     case DNGN_ENTER_COCYTUS:
@@ -154,7 +145,7 @@ command_type grid_stair_direction(dungeon_feature_type grid)
     case DNGN_ENTER_RESERVED_3:
     case DNGN_ENTER_RESERVED_4:
         return (CMD_GO_DOWNSTAIRS);
-        
+
     default:
         return (CMD_NO_CMD);
     }
@@ -162,7 +153,7 @@ command_type grid_stair_direction(dungeon_feature_type grid)
 
 bool grid_is_opaque( dungeon_feature_type grid )
 {
-    return (grid < DNGN_MINSEE && grid != DNGN_ORCISH_IDOL);
+    return (grid < DNGN_MINSEE);
 }
 
 bool grid_is_solid( dungeon_feature_type grid )
@@ -180,20 +171,34 @@ bool grid_is_solid(const coord_def &c)
     return (grid_is_solid(grd(c)));
 }
 
+bool grid_is_rock(dungeon_feature_type grid)
+{
+    return (grid == DNGN_ORCISH_IDOL
+            || grid == DNGN_GRANITE_STATUE
+            || grid == DNGN_SECRET_DOOR
+            || grid >= DNGN_ROCK_WALL
+               && grid <= DNGN_CLEAR_PERMAROCK_WALL);
+}
+
+bool grid_is_permarock(dungeon_feature_type grid)
+{
+    return (grid == DNGN_PERMAROCK_WALL || grid == DNGN_CLEAR_PERMAROCK_WALL);
+}
+
 bool grid_is_trap(dungeon_feature_type grid)
 {
     return (grid == DNGN_TRAP_MECHANICAL || grid == DNGN_TRAP_MAGICAL
-              || grid == DNGN_TRAP_III);
+            || grid == DNGN_TRAP_NATURAL);
 }
 
-bool grid_is_water( dungeon_feature_type grid )
+bool grid_is_water(dungeon_feature_type grid)
 {
     return (grid == DNGN_SHALLOW_WATER || grid == DNGN_DEEP_WATER);
 }
 
 bool grid_is_watery( dungeon_feature_type grid )
 {
-    return (grid_is_water(grid) || grid == DNGN_BLUE_FOUNTAIN);
+    return (grid_is_water(grid) || grid == DNGN_FOUNTAIN_BLUE);
 }
 
 bool grid_destroys_items( dungeon_feature_type grid )
@@ -201,7 +206,7 @@ bool grid_destroys_items( dungeon_feature_type grid )
     return (grid == DNGN_LAVA || grid == DNGN_DEEP_WATER);
 }
 
-// returns 0 if grid is not an altar, else it returns the GOD_* type
+// Returns 0 if grid is not an altar, else it returns the GOD_* type.
 god_type grid_altar_god( dungeon_feature_type grid )
 {
     if (grid >= DNGN_ALTAR_FIRST_GOD && grid <= DNGN_ALTAR_LAST_GOD)
@@ -210,8 +215,7 @@ god_type grid_altar_god( dungeon_feature_type grid )
     return (GOD_NO_GOD);
 }
 
-// returns DNGN_FLOOR for non-gods, otherwise returns the altar for
-// the god.
+// Returns DNGN_FLOOR for non-gods, otherwise returns the altar for the god.
 dungeon_feature_type altar_for_god( god_type god )
 {
     if (god == GOD_NO_GOD || god >= NUM_GODS)
@@ -220,18 +224,88 @@ dungeon_feature_type altar_for_god( god_type god )
     return static_cast<dungeon_feature_type>(DNGN_ALTAR_FIRST_GOD + god - 1);
 }
 
-bool grid_is_branch_stairs( dungeon_feature_type grid )
+bool grid_is_branch_stairs(dungeon_feature_type grid)
 {
     return ((grid >= DNGN_ENTER_FIRST_BRANCH && grid <= DNGN_ENTER_LAST_BRANCH)
             || (grid >= DNGN_ENTER_DIS && grid <= DNGN_ENTER_TARTARUS));
 }
 
+// Find all connected cells containing ft, starting at d.
+void find_connected_identical(coord_def d, dungeon_feature_type ft,
+                              std::set<coord_def>& out)
+{
+    if (grd[d.x][d.y] != ft) return;
+
+    std::string prop = env.markers.property_at(d, MAT_ANY,
+                                               "connected_exclude");
+
+    if (!prop.empty())
+    {
+        // Even if this square is excluded from being a part of connected
+        // cells, add it if it's the starting square.
+        if (out.size() == 0)
+            out.insert(d);
+        return;
+    }
+
+    if (out.insert(d).second)
+    {
+        find_connected_identical(coord_def(d.x+1, d.y), ft, out);
+        find_connected_identical(coord_def(d.x-1, d.y), ft, out);
+        find_connected_identical(coord_def(d.x, d.y+1), ft, out);
+        find_connected_identical(coord_def(d.x, d.y-1), ft, out);
+    }
+}
+
+// Find all connected cells containing ft_min to ft_max, starting at d.
+void find_connected_range(coord_def d, dungeon_feature_type ft_min,
+                          dungeon_feature_type ft_max,
+                          std::set<coord_def>& out)
+{
+    if (grd[d.x][d.y] < ft_min || grd[d.x][d.y] > ft_max) return;
+
+    std::string prop = env.markers.property_at(d, MAT_ANY,
+                                               "connected_exclude");
+
+    if (!prop.empty())
+    {
+        // Even if this square is excluded from being a part of connected
+        // cells, add it if it's the starting square.
+        if (out.size() == 0)
+            out.insert(d);
+        return;
+    }
+
+    if (out.insert(d).second)
+    {
+        find_connected_range(coord_def(d.x+1, d.y), ft_min, ft_max, out);
+        find_connected_range(coord_def(d.x-1, d.y), ft_min, ft_max, out);
+        find_connected_range(coord_def(d.x, d.y+1), ft_min, ft_max, out);
+        find_connected_range(coord_def(d.x, d.y-1), ft_min, ft_max, out);
+    }
+}
+
+void get_door_description(int door_size, const char** adjective, const char** noun)
+{
+    const char* descriptions[] = {
+        "miniscule " , "buggy door",
+        ""           , "door",
+        "large "     , "door",
+        ""           , "gate",
+        "huge "      , "gate",
+    };
+
+    const unsigned int idx = MIN( (unsigned int) door_size*2,
+                                  ARRAYSZ(descriptions) - 2 );
+    *adjective = descriptions[idx];
+    *noun = descriptions[idx+1];
+}
+
 dungeon_feature_type grid_secret_door_appearance( int gx, int gy )
 {
     dungeon_feature_type ret = DNGN_FLOOR;
 
     for (int dx = -1; dx <= 1; dx++)
-    {
         for (int dy = -1; dy <= 1; dy++)
         {
             // only considering orthogonal grids
@@ -248,26 +322,25 @@ dungeon_feature_type grid_secret_door_appearance( int gx, int gy )
             else if (ret != targ)
                 ret = ((ret < targ) ? ret : targ);
         }
-    }
 
-    return ((ret == DNGN_FLOOR) ? DNGN_ROCK_WALL 
+    return ((ret == DNGN_FLOOR) ? DNGN_ROCK_WALL
                                 : ret);
 }
 
-const char *grid_item_destruction_message( dungeon_feature_type grid )
+const char *grid_item_destruction_message(dungeon_feature_type grid)
 {
-    return grid == DNGN_DEEP_WATER? "You hear a splash."
-         : grid == DNGN_LAVA      ? "You hear a sizzling splash."
-         :                          "You hear a crunching noise.";
+    return (grid == DNGN_DEEP_WATER ? "You hear a splash." :
+            grid == DNGN_LAVA       ? "You hear a sizzling splash."
+                                    : "You hear a crunching noise.");
 }
 
-static coord_def dgn_find_nearest_square(
+static coord_def _dgn_find_nearest_square(
     const coord_def &pos,
     bool (*acceptable)(const coord_def &),
     bool (*traversable)(const coord_def &) = NULL)
 {
     memset(travel_point_distance, 0, sizeof(travel_distance_grid_t));
-    
+
     std::list<coord_def> points[2];
     int iter = 0;
     points[iter].push_back(pos);
@@ -281,10 +354,9 @@ static coord_def dgn_find_nearest_square(
 
             if (p != pos && acceptable(p))
                 return (p);
-            
+
             travel_point_distance[p.x][p.y] = 1;
             for (int yi = -1; yi <= 1; ++yi)
-            {
                 for (int xi = -1; xi <= 1; ++xi)
                 {
                     if (!xi && !yi)
@@ -296,10 +368,9 @@ static coord_def dgn_find_nearest_square(
 
                     if (traversable && !traversable(np))
                         continue;
-                    
+
                     points[!iter].push_back(np);
                 }
-            }
         }
 
         points[iter].clear();
@@ -310,16 +381,16 @@ static coord_def dgn_find_nearest_square(
     return (unfound);
 }
 
-static bool item_safe_square(const coord_def &pos)
+static bool _item_safe_square(const coord_def &pos)
 {
     const dungeon_feature_type feat = grd(pos);
     return (is_traversable(feat) && !grid_destroys_items(feat));
 }
 
 // Moves an item on the floor to the nearest adjacent floor-space.
-static bool dgn_shift_item(const coord_def &pos, item_def &item)
+static bool _dgn_shift_item(const coord_def &pos, item_def &item)
 {
-    const coord_def np = dgn_find_nearest_square(pos, item_safe_square);
+    const coord_def np = _dgn_find_nearest_square(pos, _item_safe_square);
     if (in_bounds(np) && np != pos)
     {
         int index = item.index();
@@ -329,25 +400,26 @@ static bool dgn_shift_item(const coord_def &pos, item_def &item)
     return (false);
 }
 
-static bool is_critical_feature(dungeon_feature_type feat)
+bool is_critical_feature(dungeon_feature_type feat)
 {
     return (grid_stair_direction(feat) != CMD_NO_CMD
             || grid_altar_god(feat) != GOD_NO_GOD);
 }
 
-static bool is_feature_shift_target(const coord_def &pos)
+static bool _is_feature_shift_target(const coord_def &pos)
 {
     return (grd(pos) == DNGN_FLOOR && !dungeon_events.has_listeners_at(pos));
 }
 
-static bool dgn_shift_feature(const coord_def &pos)
+static bool _dgn_shift_feature(const coord_def &pos)
 {
     const dungeon_feature_type dfeat = grd(pos);
     if (!is_critical_feature(dfeat) && !env.markers.find(pos, MAT_ANY))
         return (false);
-    
+
     const coord_def dest =
-        dgn_find_nearest_square(pos, is_feature_shift_target);
+        _dgn_find_nearest_square(pos, _is_feature_shift_target);
+
     if (in_bounds(dest) && dest != pos)
     {
         grd(dest) = dfeat;
@@ -366,7 +438,7 @@ static bool dgn_shift_feature(const coord_def &pos)
     return (true);
 }
 
-static void dgn_check_terrain_items(const coord_def &pos, bool preserve_items)
+static void _dgn_check_terrain_items(const coord_def &pos, bool preserve_items)
 {
     const dungeon_feature_type grid = grd(pos);
     if (grid_is_solid(grid) || grid_destroys_items(grid))
@@ -380,9 +452,10 @@ static void dgn_check_terrain_items(const coord_def &pos, bool preserve_items)
 
             // Game-critical item.
             if (preserve_items || item_is_critical(mitm[curr]))
-                dgn_shift_item(pos, mitm[curr]);
+                _dgn_shift_item(pos, mitm[curr]);
             else
             {
+                item_was_destroyed(mitm[curr]);
                 destroy_item(curr);
                 did_destroy = true;
             }
@@ -392,17 +465,50 @@ static void dgn_check_terrain_items(const coord_def &pos, bool preserve_items)
     }
 }
 
-static void dgn_check_terrain_monsters(const coord_def &pos)
+static void _dgn_check_terrain_monsters(const coord_def &pos)
 {
     const int mindex = mgrd(pos);
     if (mindex != NON_MONSTER)
     {
         monsters *mons = &menv[mindex];
+
+        if (mons->has_ench(ENCH_SUBMERGED)
+            && !monster_can_submerge(mons, grd(pos)))
+        {
+            mons->del_ench(ENCH_SUBMERGED);
+        }
+
         if (grid_is_solid(grd(pos)))
             monster_teleport(mons, true, false);
         else
             mons_check_pool(mons, KILL_MISC, -1);
     }
+
+}
+
+static void _dgn_check_terrain_blood(const coord_def &pos,
+                                     dungeon_feature_type old_feat,
+                                     dungeon_feature_type new_feat)
+{
+    if (env.map(pos).property != FPROP_BLOODY)
+        return;
+
+    if (new_feat == DNGN_UNSEEN)
+    {
+        // Caller has already changed the grid, and old_feat is actually
+        // the new feat.
+        if (old_feat != DNGN_FLOOR && !grid_is_solid(old_feat))
+            env.map(pos).property = FPROP_NONE;
+    }
+    else
+    {
+        if (grid_is_solid(old_feat) != grid_is_solid(new_feat)
+            || grid_is_water(new_feat) || grid_destroys_items(new_feat)
+            || is_critical_feature(new_feat))
+        {
+            env.map(pos).property = FPROP_NONE;
+        }
+    }
 }
 
 void dungeon_terrain_changed(const coord_def &pos,
@@ -411,33 +517,54 @@ void dungeon_terrain_changed(const coord_def &pos,
                              bool preserve_features,
                              bool preserve_items)
 {
+    if (grd(pos) == nfeat)
+        return;
+
+    _dgn_check_terrain_blood(pos, grd(pos), nfeat);
+
     if (nfeat != DNGN_UNSEEN)
     {
         if (preserve_features)
-            dgn_shift_feature(pos);
+            _dgn_shift_feature(pos);
+
         unnotice_feature(level_pos(level_id::current(), pos));
         grd(pos) = nfeat;
         env.grid_colours(pos) = BLACK;
         if (is_notable_terrain(nfeat) && see_grid(pos))
             seen_notable_thing(nfeat, pos.x, pos.y);
+
+        destroy_trap(pos);
     }
-    
-    dgn_check_terrain_items(pos, preserve_items);
+
+    _dgn_check_terrain_items(pos, preserve_items);
+    _dgn_check_terrain_monsters(pos);
+
     if (affect_player && pos == you.pos())
     {
         if (!grid_is_solid(grd(pos)))
         {
             if (!you.airborne())
+            {
+                // If the monster can't stay submerged in the new terrain
+                // and there aren't any adjacent squares where it can
+                // stay submerged then move it.
+                if (mgrd(you.pos()) != NON_MONSTER
+                    && !mons_is_submerged( &menv[ mgrd(you.pos()) ] ))
+                {
+                    monster_teleport( &menv[ mgrd(you.pos()) ], true, false);
+                }
                 move_player_to_grid(pos.x, pos.y, false, true, false);
+            }
         }
         else
             you_teleport_now(true, false);
     }
-    dgn_check_terrain_monsters(pos);
+
+    set_terrain_changed(pos.x, pos.y);
 }
 
-// returns true if we manage to scramble free.
-bool fall_into_a_pool( int entry_x, int entry_y, bool allow_shift, 
+// Returns true if we manage to scramble free.
+bool fall_into_a_pool( int entry_x, int entry_y, bool allow_shift,
                        unsigned char terrain )
 {
     bool escape = false;
@@ -449,7 +576,7 @@ bool fall_into_a_pool( int entry_x, int entry_y, bool allow_shift,
         merfolk_start_swimming();
         return (false);
     }
-    
+
     // sanity check
     if (terrain != DNGN_LAVA && beogh_water_walk())
         return (false);
@@ -487,7 +614,7 @@ bool fall_into_a_pool( int entry_x, int entry_y, bool allow_shift,
     {
         if (allow_shift)
         {
-            if (empty_surrounds( you.x_pos, you.y_pos, DNGN_FLOOR, 1, 
+            if (empty_surrounds( you.x_pos, you.y_pos, DNGN_FLOOR, 1,
                                  false, empty ))
             {
                 escape = true;
@@ -545,5 +672,35 @@ bool fall_into_a_pool( int entry_x, int entry_y, bool allow_shift,
         ouch( INSTANT_DEATH, 0, KILLED_BY_WATER );
 
     return (false);
-}                               // end fall_into_a_pool()
+}
 
+typedef std::map<std::string, dungeon_feature_type> feat_desc_map;
+static feat_desc_map feat_desc_cache;
+
+void init_feat_desc_cache()
+{
+    for (int i = 0; i < NUM_FEATURES; i++)
+    {
+        dungeon_feature_type grid = static_cast<dungeon_feature_type>(i);
+        std::string          desc = feature_description(grid);
+
+        lowercase(desc);
+        if (feat_desc_cache.find(desc) == feat_desc_cache.end())
+            feat_desc_cache[desc] = grid;
+    }
+}
+
+dungeon_feature_type feat_by_desc(std::string desc)
+{
+    lowercase(desc);
+
+    if (desc[desc.size() - 1] != '.')
+        desc += ".";
+
+    feat_desc_map::iterator i = feat_desc_cache.find(desc);
+
+    if (i != feat_desc_cache.end())
+        return (i->second);
+
+    return (DNGN_UNSEEN);
+}
