@@ -1,9 +1,9 @@
 /*
- *  File:       transfor.cc
+ *  File:       transfor.h
  *  Summary:    Misc function related to player transformations.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: zelgadis $ on $Date: 2007-09-15T23:33:50.620328Z $
+ *  Modified for Crawl Reference by $Author: j-p-e-g $ on $Date: 2008-06-13T22:29:02.002324Z $
  *
  *  Change History (most recent first):
  *
@@ -48,13 +48,16 @@ void untransform(void);
  * called from: item_use
  * *********************************************************************** */
 bool can_equip(equipment_type use_which, bool ignore_temporary);
+bool check_transformation_stat_loss(const std::set<equipment_type> &remove,
+                                    bool quiet = false, int str_loss = 0,
+                                    int dex_loss = 0, int int_loss = 0);
 size_type transform_size(int psize = PSIZE_BODY);
 
 // last updated 12may2000 {dlb}
 /* ***********************************************************************
  * called from: ability - spell
  * *********************************************************************** */
-bool transform(int pow, transformation_type which_trans);
+bool transform(int pow, transformation_type which_trans, bool quiet = false);
 
 
 // last updated 12may2000 {dlb}
