@@ -1,20 +1,25 @@
 /*
- *  File:       travel.cc
+ *  File:       travel.h
  *  Summary:    Travel stuff
  *  Written by: Darshan Shaligram
  *
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2008-07-11T16:08:22.705815Z $
+ *
  *  Change History (most recent first):
  *
  *     <1>     -/--/--     SD     Created
  */
 #ifndef TRAVEL_H
-#   define TRAVEL_H
+#define TRAVEL_H
 
-#   include "externs.h"
-#   include <stdio.h>
-#   include <string>
-#   include <vector>
-#   include <map>
+#include "externs.h"
+#include <stdio.h>
+#include <string>
+#include <vector>
+#include <map>
+
+class reader;
+class writer;
 
 enum run_check_type
 {
@@ -67,13 +72,17 @@ void update_excludes();
 bool is_exclude_root(const coord_def &p);
 bool is_stair(dungeon_feature_type gridc);
 bool is_travelable_stair(dungeon_feature_type gridc);
+bool is_gate(dungeon_feature_type gridc);
 command_type direction_to_command( char x, char y );
 bool is_resting( void );
-bool can_travel_interlevel();
+#ifdef CLUA_BINDINGS
+const char *trap_name(int x, int y);
+#endif
 bool is_traversable(dungeon_feature_type grid);
 void explore_pickup_event(int did_pickup, int tried_pickup);
+bool is_excluded(const coord_def &p);
 
-void find_travel_pos(int you_x, int you_y, char *move_x, char *move_y, 
+void find_travel_pos(int you_x, int you_y, char *move_x, char *move_y,
                      std::vector<coord_def>* coords = NULL);
 
 bool is_travelsafe_square(int x, int y, bool ignore_hostile = false,
@@ -93,7 +102,7 @@ bool is_travelsafe_square(int x, int y, bool ignore_hostile = false,
 void start_explore(bool grab_items = false);
 
 struct level_pos;
-struct level_id;
+class level_id;
 struct travel_target;
 
 void start_translevel_travel(const travel_target &pos);
@@ -119,7 +128,7 @@ bool prompt_stop_explore(int es_why);
 enum translevel_prompt_flags
 {
     TPF_NO_FLAGS          = 0,
-    
+
     TPF_ALLOW_WAYPOINTS   = 0x1,
     TPF_ALLOW_UPDOWN      = 0x2,
     TPF_REMEMBER_TARGET   = 0x4,
@@ -157,21 +166,25 @@ extern travel_distance_grid_t travel_point_distance;
 
 enum explore_stop_type
 {
-    ES_NONE               = 0x00,
-    ES_ITEM               = 0x01,
-    ES_PICKUP             = 0x02,
-    ES_GREEDY_PICKUP      = 0x04,
-    ES_STAIR              = 0x08,
-    ES_SHOP               = 0x10,
-    ES_ALTAR              = 0x20,
-    ES_PORTAL             = 0x40
+    ES_NONE               = 0x000,
+    ES_ITEM               = 0x001,
+    ES_PICKUP             = 0x002,
+    ES_GREEDY_PICKUP      = 0x004,
+    ES_GREEDY_ITEM        = 0x008,
+    ES_STAIR              = 0x010,
+    ES_SHOP               = 0x020,
+    ES_ALTAR              = 0x040,
+    ES_PORTAL             = 0x080,
+    ES_GLOWING_ITEM       = 0x100,
+    ES_ARTEFACT           = 0x200,
+    ES_RUNE               = 0x400
 };
 
 ////////////////////////////////////////////////////////////////////////////
 // Structs for interlevel travel.
 
 // Identifies a level.
-struct level_id
+class level_id
 {
 public:
     branch_type branch;     // The branch in which the level is.
@@ -194,11 +207,6 @@ public:
     level_id(branch_type br, int dep, level_area_type ltype = LEVEL_DUNGEON)
         : branch(br), depth(dep), level_type(ltype)
     {
-        if (level_type != LEVEL_DUNGEON)
-        {
-            depth = -1;
-            branch = NUM_BRANCHES;
-        }
     }
     level_id(const level_id &ot)
         : branch(ot.branch), depth(ot.depth), level_type(ot.level_type)
@@ -220,7 +228,7 @@ public:
         depth  = -1;
         level_type = LEVEL_DUNGEON;
     }
-    
+
     bool is_valid() const
     {
         return (branch != NUM_BRANCHES && depth != -1)
@@ -246,12 +254,25 @@ public:
 
         if (level_type != LEVEL_DUNGEON)
             return (false);
-        
+
         return (branch < id.branch) || (branch==id.branch && depth < id.depth);
     }
 
-    void save(FILE *) const;
-    void load(FILE *);
+    bool operator == ( const branch_type _branch ) const
+    {
+        return (branch == _branch && level_type == LEVEL_DUNGEON);
+    }
+
+    bool operator != ( const branch_type _branch  ) const
+    {
+        return !operator == (_branch);
+    }
+
+
+    int absdepth() const;
+
+    void save(writer&) const;
+    void load(reader&);
 };
 
 // A position on a particular level.
@@ -265,7 +286,7 @@ struct level_pos
         pos.x = pos.y = -1;
     }
 
-    level_pos(const level_id &lid, const coord_def &coord) 
+    level_pos(const level_id &lid, const coord_def &coord)
         : id(lid), pos(coord)
     {
     }
@@ -290,7 +311,7 @@ struct level_pos
     {
         return (id < lp.id) || (id == lp.id && pos < lp.pos);
     }
-    
+
     bool is_valid() const
     {
         return id.depth > -1 && pos.x != -1 && pos.y != -1;
@@ -302,8 +323,8 @@ struct level_pos
         pos = coord_def(-1, -1);
     }
 
-    void save(FILE *) const;
-    void load(FILE *);
+    void save(writer&) const;
+    void load(reader&);
 };
 
 struct travel_target
@@ -332,7 +353,7 @@ class explore_discoveries
 {
 public:
     explore_discoveries();
-    
+
     void found_feature(const coord_def &pos, dungeon_feature_type grid);
     void found_item(const coord_def &pos, const item_def &item);
 
@@ -364,7 +385,7 @@ private:
 private:
     template <class C> void say_any(const C &coll, const char *stub) const;
     template <class citer> bool has_duplicates(citer, citer) const;
-    
+
     std::string cleaned_feature_description(dungeon_feature_type feature) const;
     void add_item(const item_def &item);
     void add_stair(const named_thing<int> &stair);
@@ -372,7 +393,7 @@ private:
         const std::vector< named_thing<int> > &v) const;
     bool merge_feature(
         std::vector< named_thing<int> > &v,
-        const named_thing<int> &feat) const;    
+        const named_thing<int> &feat) const;
 };
 
 struct stair_info
@@ -383,7 +404,7 @@ public:
         PHYSICAL,
         PLACEHOLDER
     };
-    
+
 public:
     coord_def position;     // Position of stair
     dungeon_feature_type grid; // Grid feature of the stair.
@@ -402,8 +423,8 @@ public:
 
     void clear_distance() { distance = -1; }
 
-    void save(FILE *) const;
-    void load(FILE *);
+    void save(writer&) const;
+    void load(reader&);
 
     std::string describe() const;
 
@@ -433,8 +454,8 @@ struct LevelInfo
     {
     }
 
-    void save(FILE *) const;
-    void load(FILE *);
+    void save(writer&) const;
+    void load(reader&);
 
     std::vector<stair_info> &get_stairs()
     {
@@ -465,7 +486,10 @@ struct LevelInfo
     // Updates/creates a StairInfo for the stair at (x, y) in grid coordinates
     void update_stair(int x, int y, const level_pos &p, bool guess = false);
 
-    // Returns true if the given branch is known to be accessible from the 
+    // Clears all stair info for stairs matching this grid type.
+    void clear_stairs(dungeon_feature_type grid);
+
+    // Returns true if the given branch is known to be accessible from the
     // current level.
     bool is_known_branch(unsigned char branch) const;
 
@@ -524,7 +548,7 @@ public:
 
     const level_pos &get_waypoint(int number) const
     {
-        return waypoints[number]; 
+        return waypoints[number];
     }
 
     int get_waypoint_count() const;
@@ -541,8 +565,8 @@ public:
 
     void update();
 
-    void save(FILE *) const;
-    void load(FILE *);
+    void save(writer&) const;
+    void load(reader&);
 
     bool is_known_branch(unsigned char branch) const;
 
@@ -566,7 +590,7 @@ public:
     virtual ~travel_pathfind();
 
     // Finds travel direction or explore target.
-    const coord_def pathfind(run_mode_type rt);
+    coord_def pathfind(run_mode_type rt);
 
     // For flood-fills (explore), sets starting (seed) square.
     void set_floodseed(const coord_def &seed, bool double_flood = false);
@@ -588,7 +612,7 @@ public:
     // The next square to go to to move towards the travel destination. Return
     // value is undefined if pathfind was not called with RMODE_TRAVEL.
     const coord_def travel_move() const;
-    
+
     // Square to go to for (greedy) explore. Return value is undefined if
     // pathfind was not called with RMODE_EXPLORE or RMODE_EXPLORE_GREEDY.
     const coord_def explore_target() const;
@@ -614,10 +638,10 @@ protected:
 protected:
     static const int UNFOUND_DIST  = -30000;
     static const int INFINITE_DIST =  INFINITE_DISTANCE;
-    
+
 protected:
     run_mode_type runmode;
-    
+
     // Where pathfinding starts, and the destination. Note that dest is not
     // relevant for explore!
     coord_def start, dest;
@@ -644,7 +668,7 @@ protected:
 
     // Are we greedy exploring?
     bool need_for_greed;
-    
+
     // Targets for explore and greedy explore.
     coord_def unexplored_place, greedy_place;
 
