@@ -1,15 +1,15 @@
 /*
- *  File:       unrand.cc
+ *  File:       unrand.h
  *  Summary:    Definitions for unrandom artefacts.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dolorous $ on $Date: 2008-07-04T01:27:02.490149Z $
  *
- *  Change History (most recent first): 
+ *  Change History (most recent first):
  *               <3>     7 Aug 2001      MV             Added many new items
  *               <2>     5/09/99        JDJ             Cekugob no longer has fire and cold
- *                                                                      resistances. 
- *               <1>     -/--/--        LRH             Created 
+ *                                                                      resistances.
+ *               <1>     -/--/--        LRH             Created
  */
 #ifndef UNRAND_H
 #define UNRAND_H
@@ -37,9 +37,7 @@
    But add 100 to make the item stickycursed. Note that the values for
    wpns and armr are +50.
    plus2: For wpns, plus to-dam. Curses are irrelevant here. Mostly unused
-   for armr and totally for rings. Armour: if boots, plus2 == 1 means
-   naga barding; 2 means centaur barding. If headgear, 1 means helmet,
-   2 means helm, 3 means cap, 4 means wizard's hat.
+   for armr and totally for rings.
    colour: Obvious. Don't use BLACK, use DARKGREY instead.
 
    * Note * any exact combination of class, type, plus & plus2 must be unique,
@@ -69,7 +67,7 @@
    and no susceptibility here either.
    Range: 0 or 1.
 
-   res magic: This is cumulative, but no susceptibility. To be meaningful, 
+   res magic: This is cumulative, but no susceptibility. To be meaningful,
    should be set to about 20 - 60.
    Range: 0 to MAXINT probably, but about 100 is a realistic ceiling.
 
@@ -100,9 +98,10 @@
    +/- to-hit/to-dam: Obvious. Affects both melee and missile. Should be left
    at 0 for weapons, which get +s normally.
 
-   cursed: 0 or 1.  Sets the item's initial curse status.  Cursed items 
-   will tend to recurse themselves when rewielded.  Maybe this should be
-   made to be a value that determines how often it will recurse? -- bwr
+   cursed: -1, 0, any positive value.
+   Setting this to any value other than 0 will set the item's initial curse
+   status as cursed. If the value is greater than zero, the item will also have
+   one_chance_in(value) of recursing itself when re-equipped.
 
    stealth: -100 to 80.  Adds to stealth value.
 
@@ -134,7 +133,7 @@
 {
     "Dum", "",
 /* class, type, plus (to-hit), plus2 (depends on class), colour */
-        OBJ_UNASSIGNED, 250, 250, 250, 0,
+        OBJ_UNASSIGNED, 250, 250, 250, BLACK,
 /* Properties, all approx thirty of them: */
     {
 /* brand, +/- to AC, +/- to ev, +/- to str, +/- to int, +/- to dex */
@@ -162,7 +161,7 @@
 
 
 /* 2 */
-{                               
+{
     "long sword \"Bloodbane\"", "blackened long sword",
         OBJ_WEAPONS, WPN_LONG_SWORD, +7, +8, DARKGREY,
     {
@@ -181,7 +180,7 @@
 ,
 
 /* 3 */
-{ 
+{
     "ring of Shadows", "black ring",
         OBJ_JEWELLERY, RING_INVISIBILITY, 0, 0, DARKGREY,
     {
@@ -190,7 +189,7 @@
         1, 0, 0, 0, 0, 0,       // see invis
         0, 0, 0, 0, 0,
         0, 0, 0, -3, 0,         // to hit
-        0, 10                   // stealth
+        0, 40                   // stealth
     }
     ,
         "",
@@ -201,8 +200,8 @@
 
 /* 4 */
 {
-    "long sword of Flaming Death", "smoking long sword",
-        OBJ_WEAPONS, WPN_LONG_SWORD, +6, +2, RED,
+    "scimitar of Flaming Death", "smoking scimitar",
+        OBJ_WEAPONS, WPN_SCIMITAR, +7, +5, RED,
     {
         SPWPN_FLAMING, 0, 0, 0, 0, 0,
         2, -1, 0, 1, 0, 20,     // res fire, cold, poison, magic
@@ -228,7 +227,7 @@
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
-        1, 0                    // cursed
+        3, 0                    // cursed
     }
     ,
         "",
@@ -240,19 +239,19 @@
 
 /* 6 */
 {
-    "Holy Armour of Zin", "glowing golden plate mail",
-        OBJ_ARMOUR, ARM_PLATE_MAIL, +6, 0, YELLOW,
+    "amulet of the Air", "sky-blue amulet",
+        OBJ_JEWELLERY, AMU_CONTROLLED_FLIGHT, 0, 0, LIGHTCYAN,
     {
-        0, 0, 0, 3, 0, 0,       // str
-        0, 0, 0, 0, 2, 50,      // life prot, magic
-        0, 0, 0, 0, 0, 0,
+        0, 0, 3, 0, 0, 0,       // EV
+        0, 0, 1, 0, 0, 0,      // resElec
+        0, 0, 1, 0, 0, 0,       // levitate
         0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
-        0, 0
+        0, 50                   // stealth
     }
     ,
         "",
-        "A suit of mail and large plates of golden metal.",
+        "A sky-blue amulet.",
         ""
 }
 ,
@@ -281,7 +280,7 @@
     "mace of Brilliance", "brightly glowing mace",
         OBJ_WEAPONS, WPN_MACE, +5, +5, WHITE,
     {
-        SPWPN_HOLY_WRATH, 3, 0, 0, 0, 0,        // AC
+        SPWPN_HOLY_WRATH, 5, 0, 0, 5, 0,        // AC, int
         0, 0, 0, 0, 1, 0,                       // life prot
         1, 0, 0, 0, 0, 0,                       // see invis
         0, 0, 0, 0, 0,
@@ -314,15 +313,12 @@
 }
 ,
 
-
-
-
 /* 10 */
 {
-    "buckler \"Bullseye\"", "round buckler",
-        OBJ_ARMOUR, ARM_BUCKLER, +10, 0, RED,
+    "shield \"Bullseye\"", "round shield",
+        OBJ_ARMOUR, ARM_SHIELD, +10, 0, RED,
     {
-        0, 0, -3, 0, 0, 0,      // EV
+        0, 0, -5, 0, 0, 0,      // EV
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
@@ -339,7 +335,7 @@
 /* 11 */
 {
     "crown of Dyrovepreva", "jewelled bronze crown",
-        OBJ_ARMOUR, ARM_HELMET, +3, THELM_SPECIAL, BROWN,
+        OBJ_ARMOUR, ARM_CAP, +3, 0, BROWN,
     {
         0, 0, 0, 0, 2, 0,       // int
         0, 0, 1, 0, 0, 0,       // res elec
@@ -361,12 +357,12 @@
     "demon blade \"Leech\"", "runed demon blade",
         OBJ_WEAPONS, WPN_DEMON_BLADE, +13, +4, MAGENTA,
     {
-        SPWPN_VAMPIRICISM, -1, -1, -1, -1, -1, // AC, EV, str, int, dex
-        0, 0, 0, 0, 1, 0,                      // life prot
+        SPWPN_VAMPIRICISM, 0, -1, -1, -1, -1, // AC, EV, str, int, dex
+        0, 0, 0, 0, 1, 0,                     // life prot
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
-        1, 0                                   // cursed
+        4, 0                                  // cursed
     }
     ,
         "",
@@ -417,7 +413,7 @@
 /* 15 */
 {
     "dagger of Chilly Death", "sapphire dagger",
-        OBJ_WEAPONS, WPN_DAGGER, +2, +6, LIGHTBLUE,
+        OBJ_WEAPONS, WPN_DAGGER, +5, +7, LIGHTBLUE,
     {
         SPWPN_FREEZING, 0, 0, 0, 0, 0,
         -1, 2, 0, 1, 0, 20,     // res fire, cold, poison, magic
@@ -474,14 +470,14 @@
 /* 18 */
 {
     "scythe \"Finisher\"", "blackened scythe",
-        OBJ_WEAPONS, WPN_SCYTHE, +3, +5, DARKGRAY,
+        OBJ_WEAPONS, WPN_SCYTHE, +3, +5, DARKGREY,
     {
         SPWPN_SPEED, 0, 0, 3, 0, 0,       // str
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
-        1, 0                    // cursed
+        -1, 0                    // cursed
     }
     ,
         "",
@@ -493,7 +489,7 @@
 /* 19 */
 {
     "sling \"Punk\"", "blue sling",
-        OBJ_WEAPONS, WPN_SLING, +3, +4, LIGHTBLUE,
+        OBJ_WEAPONS, WPN_SLING, +9, +12, LIGHTBLUE,
     {
         SPWPN_FROST, 0, 0, 0, 0, 0,
         0, 1, 0, 0, 0, 0,               // res cold
@@ -530,7 +526,7 @@
 /* 21 */
 {
     "cloak of Flash", "vibrating cloak",
-        OBJ_ARMOUR, ARM_CLOAK, +2, 0, RED,
+        OBJ_ARMOUR, ARM_CLOAK, +3, 0, RED,
     {
         0, 0, 4, 0, 0, 0,       // EV
         0, 0, 0, 0, 0, 0,
@@ -550,7 +546,7 @@
     "giant club \"Skullcrusher\"", "brutal giant club",
         OBJ_WEAPONS, WPN_GIANT_CLUB, +0, +5, BROWN,
     {
-        SPWPN_VORPAL, 0, 0, 5, 0, 0,       // str
+        SPWPN_SPEED, 0, 0, 5, 0, 0,       // str
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
@@ -586,7 +582,7 @@
     "glaive of the Guard", "polished glaive",
         OBJ_WEAPONS, WPN_GLAIVE, +5, +8, LIGHTCYAN,
     {
-        SPWPN_PROTECTION, 0, 0, 0, 0, 0,
+        SPWPN_ELECTROCUTION, 5, 0, 0, 0, 0,       // AC
         0, 0, 0, 0, 0, 0,
         1, 0, 0, 0, 0, 1,       // see invis, go berserk
         0, 0, 0, 0, 0,
@@ -603,7 +599,7 @@
 /* 25 */
 {
     "sword of Jihad", "crystal sword",
-        OBJ_WEAPONS, WPN_LONG_SWORD, +4, +4, WHITE,
+        OBJ_WEAPONS, WPN_LONG_SWORD, +12, +10, WHITE,
     {
         SPWPN_HOLY_WRATH, 0, 3, 0, 0, 0,        // EV
         0, 0, 0, 0, 1, 20,                      // life prot, res magic
@@ -628,7 +624,7 @@
         0, 0, 0, 0, 0, 0,
         0, 0, 1, 0, 0,          // prevent spellcasting
         0, 0, 0, 0, 0,
-        1, 0                    // cursed
+        -1, 0                    // cursed
     }
     ,
         "",
@@ -660,7 +656,7 @@
         OBJ_WEAPONS, WPN_CROSSBOW, +6, +9, LIGHTRED,
     {
         SPWPN_FLAME, 0, 0, 0, 0, 0,
-        2, -1, 0, 0, 0, 0,              // +2 fire, -1 cold
+        2, -1, 0, 0, 0, 40,              // +2 fire, -1 cold, res magic
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
@@ -679,7 +675,7 @@
     {
         0, 0, 0, 0, 0, 0,
         2, 0, 0, 0, 0, 0,       // res fire
-        0, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0, 1,       // go berserk
         0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
         0, 0
@@ -710,8 +706,8 @@
 ,
 /* 31 */
 {
-    "sword of Doom Knight", "adamantine great sword",
-        OBJ_WEAPONS, WPN_GREAT_SWORD, +4, +4, BLUE,
+    "sword of the Doom Knight", "adamantine great sword",
+        OBJ_WEAPONS, WPN_GREAT_SWORD, +13, +13, BLUE,
     {
         SPWPN_PAIN, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 50,      // res magic
@@ -747,18 +743,18 @@
 /* 33 */
 {
     "robe of Folly", "dull robe",
-        OBJ_ARMOUR, ARM_ROBE, -1, 0, LIGHTGRAY,
+        OBJ_ARMOUR, ARM_ROBE, -1, 0, LIGHTGREY,
     {
         0, 0, 0, 0, -5, 0,      // int
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0,
         0, 0, 1, 0, 0,          // prevent spellcasting
         0, 0, 0, 0, 0,
-        1, 0                    // cursed
+        2, 0                    // cursed
     }
     ,
         "",
-        "A dull gray robe.",
+        "A dull grey robe.",
         ""
 }
 ,
@@ -772,7 +768,7 @@
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
         1, 0, 0, 0, 3,          // force berserk, to dam
-        1, -20                  // cursed, stealth
+        3, -20                  // cursed, stealth
     }
     ,
         "",
@@ -819,7 +815,7 @@
 /* 37 */
 {
     "ring of Robustness", "steel ring",
-        OBJ_JEWELLERY, RING_PROTECTION, +8, 0, LIGHTGRAY,
+        OBJ_JEWELLERY, RING_PROTECTION, +8, 0, LIGHTGREY,
     {
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0,
@@ -836,7 +832,7 @@
 ,
 /* 38 */
 {
-    "Edison's patent armour", "weird-looking armour",
+    "Maxwell's patent armour", "weird-looking armour",
         OBJ_ARMOUR, ARM_PLATE_MAIL, +10, 0, LIGHTGREEN,
     {
         0, 0, 0, 0, 0, 0,
@@ -844,7 +840,7 @@
         0, 0, 0, 0, 0, 0,
         0, 0, 1, 0, 1,          // prevent spellcasting, prevent teleport
         0, 0, 0, 0, 0,
-        1, 0                    // cursed
+        -1, 0                    // cursed
     }
     ,
         "",
@@ -855,7 +851,7 @@
 /* 39 */
 {
     "spear of Voo-Doo", "ebony spear",
-        OBJ_WEAPONS, WPN_SPEAR, +2, +10, DARKGRAY,
+        OBJ_WEAPONS, WPN_SPEAR, +2, +10, DARKGREY,
     {
         SPWPN_VAMPIRICISM, 0, 0, 0, 0, 0,
         0, 0, 0, 1, 1, 0,       // res poison, prot life
@@ -872,11 +868,11 @@
 ,
 /* 40 */
 {
-    "trident of the Octopus king", "mangy trident",
+    "trident of the Octopus King", "mangy trident",
         OBJ_WEAPONS, WPN_TRIDENT, +10, +4, CYAN,
     {
         SPWPN_VENOM, 0, 0, 0, 0, 0,
-        0, 0, 0, 1, 0, 50,              // res poison, res magic
+        0, 0, 1, 1, 0, 50,              // res elec, res poison, res magic
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
@@ -893,7 +889,7 @@
 /* 41 */
 {
     "mask of the Dragon", "blue mask",
-        OBJ_ARMOUR, ARM_HELMET, +0, THELM_SPECIAL, BLUE,
+        OBJ_ARMOUR, ARM_CAP, +0, 0, BLUE,
     {
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 40,      // res magic
@@ -929,13 +925,13 @@
 /* 43 */
 {
     "Elemental Staff", "black staff",
-        OBJ_WEAPONS, WPN_QUARTERSTAFF, +3, +1, DARKGRAY,
+        OBJ_WEAPONS, WPN_QUARTERSTAFF, +3, +1, DARKGREY,
     {
         SPWPN_PROTECTION, 0, 0, 0, 0, 0,
-        1, 1, 0, 0, 0, 60,      // res fire, cold, magic
+        2, 2, 0, 0, 0, 60,      // res fire, cold, magic
         0, 0, 0, 0, 0, 0,
         0, 1, 0, 0, 0,          // noise
-        0, 0, 0, 0, 0,
+        0, 2, 0, 0, 0,          // speed metabolism
         0, 0
     }
     ,
@@ -948,7 +944,7 @@
 /* 44 */
 {
     "hand crossbow \"Sniper\"", "black crossbow",
-        OBJ_WEAPONS, WPN_HAND_CROSSBOW, +10, +0, DARKGRAY,
+        OBJ_WEAPONS, WPN_HAND_CROSSBOW, +10, +0, DARKGREY,
     {
         SPWPN_VENOM, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0,
@@ -984,7 +980,7 @@
 /* 46 */
 {
     "robe of Night", "black robe",
-        OBJ_ARMOUR, ARM_ROBE, +4, 0, DARKGRAY,
+        OBJ_ARMOUR, ARM_ROBE, +4, 0, DARKGREY,
     {
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 30,      // res magic
@@ -1003,7 +999,7 @@
 /* 47 */
 {
     "plutonium sword", "glowing long sword",
-        OBJ_WEAPONS, WPN_LONG_SWORD, +5, +10, LIGHTGREEN,
+        OBJ_WEAPONS, WPN_LONG_SWORD, +12, +16, LIGHTGREEN,
     {
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0,
@@ -1020,10 +1016,10 @@
 ,
 /* 48 */
 {
-    "mace \"Undeadhunter\"", "steel mace",
-        OBJ_WEAPONS, WPN_MACE, +4, +6, LIGHTGRAY,
+    "great mace \"Undeadhunter\"", "great steel mace",
+        OBJ_WEAPONS, WPN_GREAT_MACE, +7, +7, LIGHTGREY,
     {
-        SPWPN_DISRUPTION, 0, 0, 0, 0, 0,
+        SPWPN_HOLY_WRATH, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 1, 0,       // life prot
         0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0,
@@ -1057,7 +1053,7 @@
 /* 50 */
 {
     "hat of the Alchemist", "dirty hat",
-        OBJ_ARMOUR, ARM_HELMET, +2, THELM_WIZARD_HAT, MAGENTA,
+        OBJ_ARMOUR, ARM_WIZARD_HAT, +2, 0, MAGENTA,
     {
         0, 0, 0, 0, 0, 0,
         1, 1, 1, 0, 0, 30,      // res fire, cold, elec, magic
@@ -1114,7 +1110,7 @@
     "blowgun of the Assassin", "tiny blowgun",
         OBJ_WEAPONS, WPN_BLOWGUN, +6, +6, WHITE,
     {
-        SPWPN_VENOM, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0,
         0, 1, 0, 0, 0, 0,       // turn invisible
         0, 0, 0, 0, 0,
@@ -1128,27 +1124,119 @@
 }
 ,
 
-  
+/* 54 */
+{
+    "Wyrmbane", "scale-covered lance",
+        OBJ_WEAPONS, WPN_SPEAR, +9, +6, GREEN,
+    {
+        SPWPN_DRAGON_SLAYING, 5, 0, 0, 0, 0, // AC
+        1, 0, 0, 1, 0, 0,     // res fire, poison
+        0, 0, 0, 0, 0, 1,     // go berserk
+        0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0,
+        0, 0
+    }
+    ,
+        "",
+        "",
+        ""
+},
+
+/* 55 */
+{
+    "Spriggan's Knife", "dainty little knife",
+        OBJ_WEAPONS, WPN_KNIFE, +4, +10, LIGHTCYAN,
+    {
+        0, 0, 4, 0, 0, 4,  // +EV, +Dex
+        0, 0, 0, 0, 0, 20, // +MR
+        0, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0,
+        0, 50 // +stealth
+    }
+    ,
+        "This knife was made by Spriggans, or for Spriggans, or possibly from "
+        "Spriggans. Anyway, it's in some way associated with those fey folk.",
+        "A dainty little knife.",
+        ""
+},
+
+/* 56 */
+{
+    "cloak of Starlight", "phosphorescent cloak",
+        OBJ_ARMOUR, ARM_CLOAK, 0, 0, WHITE,
+    {
+        0, 0, 4, 0, 0, 0,  // EV
+        0, 1, 1, 0, 0, 0,  // Cold, resElec
+        0, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0,
+        0, -30             // Stealth
+    }
+    ,
+        "A cloak woven of pure light beams.",
+        "A phosphorescent cloak.",
+        ""
+},
+
+/* 57 */
+{
+    "brooch of Shielding", "shield-shaped amulet",
+        OBJ_JEWELLERY, AMU_WARDING, 0, 0, LIGHTBLUE,
+    {
+        0, 4, 4, 0, 0, 0, // +AC, EV
+        0, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0,
+        0, 0
+    }
+    ,
+        "Back in the good old days, every adventurer had one of these handy "
+        "devices. That, and a pony.",
+        "A shield-shaped amulet.",
+        ""
+},
+
+/* 58 */
+{
+    "whip \"Serpent-Scourge\"", "forked whip",
+        OBJ_WEAPONS, WPN_WHIP, +5, +10, DARKGREY,
+    {
+        SPWPN_VENOM, 0, 0, 0, 0, 0,
+        0, 0, 0, 1, 0, 0, // rPois
+        0, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0,
+        0, 0
+    }
+    ,
+        "",
+        "A double-ended whip made from the cured hides of the "
+        "Lair of the Beasts' deadly grey snakes.",
+        ""
+},
+
 /* This is a dummy */
 /* 1 */
 {
     "Dum", "",
 /* class, type, plus (to-hit), plus2 (depends on class), colour */
-        OBJ_UNASSIGNED, 250, 250, 250, 0,
+        OBJ_UNASSIGNED, 250, 250, 250, BLACK,
 /* Properties, all approx thirty of them: */
     {
 /* brand, +/- to AC, +/- to ev, +/- to str, +/- to int, +/- to dex */
         0, 0, 0, 0, 0, 0,
 /* res fire, res cold, res elec, res poison, life protection, res magic */
-            0, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0, 0,
 /* see invis, turn invis, levitate, blink, teleport at will, go berserk */
-            0, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0, 0,
 /* sense surroundings, make noise, no spells, teleport, no teleprt */
-            0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0,
 /* force berserk, speed metabolism, mutate, +/- to hit, +/- to dam (not weapons) */
-            0, 0, 0, 0, 0,
-/* some as yet unused properties */
-            0, 0
+        0, 0, 0, 0, 0,
+/* cursed, stealth */
+        0, 0
     }
     ,
 /* 3 strings for describing the item in the 'V' display. */
